--[[

	Atlas, a World of Warcraft instance map browser
	Copyright 2005 - 2008 Dan Gilbert
	Email me at loglow@gmail.com

	This file is part of Atlas.

	Atlas is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Atlas is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Atlas; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

--]]

-- Atlas Data
-- Compiled by Dan Gilbert
-- loglow@gmail.com
-- Many thanks to all contributors!
-- Now with GUIDs!
-- Now externally localized!

local BLUE = "|cff6666ff";
local GREY = "|cff999999";
local GREN = "|cff66cc33";
local _RED = "|cffcc6666";
local ORNG = "|cffcc9933";
local PURP = "|cff9900ff";
local WHIT = "|cffffffff";
local INDENT = "      ";

local ZONE = 1;
local NPC = 2;
local ITEM = 3;
local OBJECT = 4;
local FACTION = 5;
local QUEST = 6;

AtlasMaps = {

--************************************************
-- Kalimdor Instances
--************************************************

	RagefireChasm = {
		ZoneName = { AtlasLocale["Ragefire Chasm"], 2437 };
		Acronym = AtlasLocale["RFC"];
		Location = { AtlasLocale["Orgrimmar"], 1637 };
		LevelRange = "12-18";
		MinLevel = "8";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Maur Grimtotem"], NPC, 11834 };
		{ GREY..INDENT..AtlasLocale["Oggleflint <Ragefire Chieftain>"], NPC, 11517 };
		{ GREY.."2) "..AtlasLocale["Taragaman the Hungerer"], NPC, 11520 };
		{ GREY.."3) "..AtlasLocale["Jergosh the Invoker"], NPC, 11518 };
		{ GREY.."4) "..AtlasLocale["Bazzalan"], NPC, 11519 };
	};
	WailingCaverns = {
		ZoneName = { AtlasLocale["Wailing Caverns"], 718 };
		Acronym = AtlasLocale["WC"];
		Location = { AtlasLocale["The Barrens"], 17 };
		LevelRange = "16-25";
		MinLevel = "10";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Disciple of Naralex"], NPC, 3678 };
		{ GREY..INDENT..AtlasLocale["Mysterious Wailing Caverns Chest"], OBJECT, 180055 };
		{ GREY.."2) "..AtlasLocale["Lord Cobrahn <Fanglord>"], NPC, 3669 };
		{ GREY.."3) "..AtlasLocale["Lady Anacondra <Fanglord>"], NPC, 3671 };
		{ GREY.."4) "..AtlasLocale["Kresh"], NPC, 3653 };
		{ GREY.."5) "..AtlasLocale["Lord Pythas <Fanglord>"], NPC, 3670 };
		{ GREY.."6) "..AtlasLocale["Skum"], NPC, 3674 };
		{ GREY.."7) "..AtlasLocale["Lord Serpentis <Fanglord>"].." ("..AtlasLocale["Upper"]..")", NPC, 3673 };
		{ GREY.."8) "..AtlasLocale["Verdan the Everliving"].." ("..AtlasLocale["Upper"]..")", NPC, 5775 };
		{ GREY.."9) "..AtlasLocale["Mutanus the Devourer"], NPC, 3654 };
		{ GREY..INDENT..AtlasLocale["Naralex"], NPC, 3679 };
		{ GREY.."10) "..AtlasLocale["Deviate Faerie Dragon"].." ("..AtlasLocale["Rare"]..")", NPC, 5912 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Embrace of the Viper"] };
	};
	BlackfathomDeeps = {
		ZoneName = { AtlasLocale["Blackfathom Deeps"], 719 };
		Acronym = AtlasLocale["BFD"];
		Location = { AtlasLocale["Ashenvale"], 331 };
		LevelRange = "20-28";
		MinLevel = "19";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Ghamoo-ra"], NPC, 4887 };
		{ GREY.."2) "..AtlasLocale["Lorgalis Manuscript"], ITEM, 5359 };
		{ GREY.."3) "..AtlasLocale["Lady Sarevess"], NPC, 4831 };
		{ GREY.."4) "..AtlasLocale["Argent Guard Thaelrid <The Argent Dawn>"], NPC, 4787 };
		{ GREY.."5) "..AtlasLocale["Gelihast"], NPC, 6243 };
		{ GREY..INDENT..AtlasLocale["Shrine of Gelihast"] };
		{ GREY.."6) "..AtlasLocale["Lorgus Jett"].." ("..AtlasLocale["Varies"]..")", NPC, 12902 };
		{ GREY.."7) "..AtlasLocale["Fathom Stone"], OBJECT, 177964 };
		{ GREY..INDENT..AtlasLocale["Baron Aquanis"], NPC, 12876 };
		{ GREY.."8) "..AtlasLocale["Twilight Lord Kelris"], NPC, 4832 };
		{ GREY.."9) "..AtlasLocale["Old Serra'kis"], NPC, 4830 };
		{ GREY.."10) "..AtlasLocale["Aku'mai"], NPC, 4829 };
		{ GREY..INDENT..AtlasLocale["Morridune"], NPC, 6729 };
		{ GREY..INDENT..AtlasLocale["Altar of the Deeps"] };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	RazorfenKraul = {
		ZoneName = { AtlasLocale["Razorfen Kraul"], 491 };
		Acronym = AtlasLocale["RFK"];
		Location = { AtlasLocale["The Barrens"], 17 };
		LevelRange = "23-33";
		MinLevel = "19";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Roogug"], NPC, 6168 };
		{ GREY.."2) "..AtlasLocale["Aggem Thorncurse <Death's Head Prophet>"], NPC, 4424 };
		{ GREY.."3) "..AtlasLocale["Death Speaker Jargba <Death's Head Captain>"], NPC, 4428 };
		{ GREY.."4) "..AtlasLocale["Overlord Ramtusk"], NPC, 4420 };
		{ GREY..INDENT..AtlasLocale["Razorfen Spearhide"].." ("..AtlasLocale["Rare"]..")", NPC, 4438 };
		{ GREY.."5) "..AtlasLocale["Agathelos the Raging"], NPC, 4422 };
		{ GREY.."6) "..AtlasLocale["Blind Hunter"].." ("..AtlasLocale["Rare"]..")", NPC, 4425 };
		{ GREY.."7) "..AtlasLocale["Charlga Razorflank <The Crone>"], NPC, 4421 };
		{ GREY.."8) "..AtlasLocale["Willix the Importer"], NPC, 4508 };
		{ GREY..INDENT..AtlasLocale["Heralath Fallowbrook"], NPC, 4510 };
		{ GREY.."9) "..AtlasLocale["Earthcaller Halmgar"].." ("..AtlasLocale["Rare"]..")", NPC, 4842 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	RazorfenDowns = {
		ZoneName = { AtlasLocale["Razorfen Downs"], 722 };
		Acronym = AtlasLocale["RFD"];
		Location = { AtlasLocale["The Barrens"], 17 };
		LevelRange = "33-42";
		MinLevel = "30";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Tuten'kash"], NPC, 7355 };
		{ GREY.."2) "..AtlasLocale["Henry Stern"], NPC, 8696 };
		{ GREY..INDENT..AtlasLocale["Belnistrasz"], NPC, 8516 };
		{ GREY..INDENT..AtlasLocale["Sah'rhee"], NPC, 8767 };
		{ GREY..INDENT..AtlasLocale["Lady Falther'ess"].." ("..AtlasLocale["Scourge Invasion"]..")", NPC, 14686 };
		{ GREY.."3) "..AtlasLocale["Mordresh Fire Eye"], NPC, 7357 };
		{ GREY.."4) "..AtlasLocale["Glutton"], NPC, 8567 };
		{ GREY.."5) "..AtlasLocale["Ragglesnout"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Varies"]..")", NPC, 7354 };
		{ GREY.."6) "..AtlasLocale["Amnennar the Coldbringer"], NPC, 7358 };
		{ GREY.."7) "..AtlasLocale["Plaguemaw the Rotting"], NPC, 7356 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	ZulFarrak = {
		ZoneName = { AtlasLocale["Zul'Farrak"], 978 };
		Acronym = AtlasLocale["ZF"];
		Location = { AtlasLocale["Tanaris"], 440 };
		LevelRange = "42-48";
		MinLevel = "35";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Mallet of Zul'Farrak"].." ("..AtlasLocale["Gahz'rilla"]..")", ITEM, 9240 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Antu'sul <Overseer of Sul>"], NPC, 8127 };
		{ GREY.."2) "..AtlasLocale["Theka the Martyr"], NPC, 7272 };
		{ GREY.."3) "..AtlasLocale["Witch Doctor Zum'rah"], NPC, 7271 };
		{ GREY..INDENT..AtlasLocale["Zul'Farrak Dead Hero"], NPC, 7276 };
		{ GREY.."4) "..AtlasLocale["Nekrum Gutchewer"], NPC, 7796 };
		{ GREY..INDENT..AtlasLocale["Shadowpriest Sezz'ziz"], NPC, 7275 };
		{ GREY..INDENT..AtlasLocale["Dustwraith"].." ("..AtlasLocale["Rare"]..")", NPC, 10081 };
		{ GREY.."5) "..AtlasLocale["Sergeant Bly"], NPC, 7604 };
		{ GREY..INDENT..AtlasLocale["Weegli Blastfuse"], NPC, 7607 };
		{ GREY..INDENT..AtlasLocale["Murta Grimgut"], NPC, 7608 };
		{ GREY..INDENT..AtlasLocale["Raven"], NPC, 7605 };
		{ GREY..INDENT..AtlasLocale["Oro Eyegouge"], NPC, 7606 };
		{ GREY..INDENT..AtlasLocale["Sandfury Executioner"], NPC, 7274 };
		{ GREY.."6) "..AtlasLocale["Hydromancer Velratha"], NPC, 7795 };
		{ GREY..INDENT..AtlasLocale["Gahz'rilla"].." ("..AtlasLocale["Summon"]..")", NPC, 7273 };
		{ GREY..INDENT..AtlasLocale["Elder Wildmane"].." ("..AtlasLocale["Lunar Festival"]..")", NPC, 15578 };
		{ GREY.."7) "..AtlasLocale["Chief Ukorz Sandscalp"], NPC, 7267 };
		{ GREY..INDENT..AtlasLocale["Ruuzlu"], NPC, 7797 };
		{ GREY.."8) "..AtlasLocale["Zerillis"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Wanders"]..")", NPC, 10082 };
		{ GREY.."9) "..AtlasLocale["Sandarr Dunereaver"].." ("..AtlasLocale["Rare"]..")", NPC, 10080 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	Maraudon = {
		ZoneName = { AtlasLocale["Maraudon"], 2100 };
		Acronym = AtlasLocale["Mara"];
		Location = { AtlasLocale["Desolace"], 405 };
		LevelRange = "42-51";
		MinLevel = "30";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Scepter of Celebras"].." ("..AtlasLocale["Portal"]..")", ITEM, 17191 };
		{ BLUE.."A) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Orange"]..")" };
		{ BLUE.."B) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Purple"]..")" };
		{ BLUE.."C) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Portal"]..")" };
		{ GREY.."1) "..AtlasLocale["Veng <The Fifth Khan>"], NPC, 13738 };
		{ GREY.."2) "..AtlasLocale["Noxxion"], NPC, 13282 };
		{ GREY.."3) "..AtlasLocale["Razorlash"], NPC, 12258 };
		{ GREY.."4) "..AtlasLocale["Maraudos <The Fourth Khan>"], NPC, 13739 };
		{ GREY.."5) "..AtlasLocale["Lord Vyletongue"], NPC, 12236 };
		{ GREY.."6) "..AtlasLocale["Meshlok the Harvester"].." ("..AtlasLocale["Rare"]..")", NPC, 12237 };
		{ GREY.."7) "..AtlasLocale["Celebras the Cursed"], NPC, 12225 };
		{ GREY.."8) "..AtlasLocale["Landslide"], NPC, 12203 };
		{ GREY.."9) "..AtlasLocale["Tinkerer Gizlock"], NPC, 13601 };
		{ GREY.."10) "..AtlasLocale["Rotgrip"], NPC, 13596 };
		{ GREY.."11) "..AtlasLocale["Princess Theradras"], NPC, 12201 };
		{ GREY.."12) "..AtlasLocale["Elder Splitrock"].." ("..AtlasLocale["Lunar Festival"]..")", NPC, 15556 };
	};
	DireMaulEast = {
		ZoneName = { AtlasLocale["Dire Maul"].." ("..AtlasLocale["East"]..")", 2557 };
		Acronym = AtlasLocale["DME"];
		Location = { AtlasLocale["Feralas"], 357 };
		LevelRange = "55-58";
		MinLevel = "50";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Brazier of Invocation"].." ("..AtlasLocale["D2 Summon"]..")", ITEM, 22057 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Entrance"] };
		{ BLUE.."C) "..AtlasLocale["Entrance"] };
		{ BLUE.."D) "..AtlasLocale["Exit"] };
		{ GREY.."1) "..AtlasLocale["Pusillin"].." ("..AtlasLocale["Chase Begins"]..")", NPC, 14354 };
		{ GREY.."2) "..AtlasLocale["Pusillin"].." ("..AtlasLocale["Chase Ends"]..")", NPC, 14354 };
		{ GREY.."3) "..AtlasLocale["Zevrim Thornhoof"], NPC, 11490 };
		{ GREY..INDENT..AtlasLocale["Hydrospawn"], NPC, 13280 };
		{ GREY..INDENT..AtlasLocale["Lethtendris"], NPC, 14327 };
		{ GREY..INDENT..AtlasLocale["Pimgib"], NPC, 14349 };
		{ GREY.."4) "..AtlasLocale["Old Ironbark"], NPC, 11491 };
		{ GREY.."5) "..AtlasLocale["Alzzin the Wildshaper"], NPC, 11492 };
		{ GREY..INDENT..AtlasLocale["Isalien"].." ("..AtlasLocale["Summon"]..")", NPC, 16097 };
		{ GREY..INDENT..AtlasLocale["Felvine Shard"], OBJECT, 179559 };
		{ GREN.."1') "..AtlasLocale["A Dusty Tome"].." ("..AtlasLocale["Varies"]..")", OBJECT, 179547 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Dire Maul Books"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Ironweave Battlesuit"] };
	};
	DireMaulNorth = {
		ZoneName = { AtlasLocale["Dire Maul"].." ("..AtlasLocale["North"]..")", 2557 };
		Acronym = AtlasLocale["DMN"];
		Location = { AtlasLocale["Feralas"], 357 };
		LevelRange = "57-60";
		MinLevel = "50";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Crescent Key"], ITEM, 18249 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Gordok Courtyard Key"], ITEM, 18266 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Gordok Inner Door Key"], ITEM, 18268 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Library"] };
		{ BLUE.."C) "..AtlasLocale["Dire Maul"].." ("..AtlasLocale["West"]..")", ZONE, 2557 };
		{ GREY.."1) "..AtlasLocale["Guard Mol'dar"], NPC, 14326 };
		{ GREY.."2) "..AtlasLocale["Stomper Kreeg <The Drunk>"], NPC, 14322 };
		{ GREY.."3) "..AtlasLocale["Guard Fengus"], NPC, 14321 };
		{ GREY.."4) "..AtlasLocale["Knot Thimblejack"], NPC, 14338 };
		{ GREY..INDENT..AtlasLocale["Guard Slip'kik"], NPC, 14323 };
		{ GREY.."5) "..AtlasLocale["Captain Kromcrush"], NPC, 14325 };
		{ GREY.."6) "..AtlasLocale["King Gordok"], NPC, 11501 };
		{ GREY..INDENT..AtlasLocale["Cho'Rush the Observer"], NPC, 14324 };
		{ GREN.."1') "..AtlasLocale["Library"] };
		{ GREN..INDENT..AtlasLocale["Falrin Treeshaper"], NPC, 16032 };
		{ GREN..INDENT..AtlasLocale["Lorekeeper Lydros"], NPC, 14368 };
		{ GREN..INDENT..AtlasLocale["Lorekeeper Javon"], NPC, 14381 };
		{ GREN..INDENT..AtlasLocale["Lorekeeper Kildrath"], NPC, 14383 };
		{ GREN..INDENT..AtlasLocale["Lorekeeper Mykos"], NPC, 14382 };
		{ GREN..INDENT..AtlasLocale["Shen'dralar Provisioner"], NPC, 14371 };
		{ GREN..INDENT..AtlasLocale["Skeletal Remains of Kariel Winthalus"], OBJECT, 179544 };
		{ GREN.."2') "..AtlasLocale["A Dusty Tome"].." ("..AtlasLocale["Varies"]..")", OBJECT, 179547 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Dire Maul Books"] };
		{ GREY..INDENT..AtlasLocale["Tribute Run"], OBJECT, 190005 };
	};
	DireMaulWest = {
		ZoneName = { AtlasLocale["Dire Maul"].." ("..AtlasLocale["West"]..")", 2557 };
		Acronym = AtlasLocale["DMW"];
		Location = { AtlasLocale["Feralas"], 357 };
		LevelRange = "57-60";
		MinLevel = "50";
		PlayerLimit = "5";
		Continent = AtlasLocale["Kalimdor"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Crescent Key"], ITEM, 18249 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["J'eevee's Jar"].." ("..AtlasLocale["Lord Hel'nurath"]..")", ITEM, 18663 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Pylons"] };
		{ BLUE.."C) "..AtlasLocale["Dire Maul"].." ("..AtlasLocale["North"]..")", ZONE, 2557 };
		{ GREY.."1) "..AtlasLocale["Shen'dralar Ancient"], NPC, 14358 };
		{ GREY.."2) "..AtlasLocale["Tendris Warpwood"], NPC, 11489 };
		{ GREY..INDENT..AtlasLocale["Ancient Equine Spirit"], NPC, 14566 };
		{ GREY.."3) "..AtlasLocale["Illyanna Ravenoak"], NPC, 11488 };
		{ GREY..INDENT..AtlasLocale["Ferra"], NPC, 14308 };
		{ GREY.."4) "..AtlasLocale["Magister Kalendris"], NPC, 11487 };
		{ GREY.."5) "..AtlasLocale["Tsu'zee"].." ("..AtlasLocale["Rare"]..")", NPC, 11467 };
		{ GREY.."6) "..AtlasLocale["Immol'thar"], NPC, 11496 };
		{ GREY..INDENT..AtlasLocale["Lord Hel'nurath"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Summon"]..")", NPC, 14506 };
		{ GREY.."7) "..AtlasLocale["Prince Tortheldrin"], NPC, 11486 };
		{ GREY..INDENT..AtlasLocale["The Prince's Chest"], OBJECT, 179545 };
		{ GREY.."8) "..AtlasLocale["Revanchion"].." ("..AtlasLocale["Scourge Invasion"]..")", NPC, 11467 };
		{ GREN.."1') "..AtlasLocale["Library"] };
		{ GREN..INDENT..AtlasLocale["Falrin Treeshaper"], NPC, 16032 };
		{ GREN..INDENT..AtlasLocale["Lorekeeper Lydros"], NPC, 14368 };
		{ GREN..INDENT..AtlasLocale["Lorekeeper Javon"], NPC, 14381 };
		{ GREN..INDENT..AtlasLocale["Lorekeeper Kildrath"], NPC, 14383 };
		{ GREN..INDENT..AtlasLocale["Lorekeeper Mykos"], NPC, 14382 };
		{ GREN..INDENT..AtlasLocale["Shen'dralar Provisioner"], NPC, 14371 };
		{ GREN..INDENT..AtlasLocale["Skeletal Remains of Kariel Winthalus"], OBJECT, 179544 };
		{ GREN.."2') "..AtlasLocale["A Dusty Tome"].." ("..AtlasLocale["Varies"]..")", OBJECT, 179547 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Dire Maul Books"] };
	};
	OnyxiasLair = {
		ZoneName = { AtlasLocale["Onyxia's Lair"], 2159 };
		Acronym = AtlasLocale["Ony"];
		Location = { AtlasLocale["Dustwallow Marsh"], 15 };
		LevelRange = "60+";
		MinLevel = "60";
		PlayerLimit = "40";
		Continent = AtlasLocale["Kalimdor"];
		{ ORNG..AtlasLocale["Attunement Required"] };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Drakefire Amulet"], ITEM, 16309 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Onyxian Warders"], NPC, 12129 };
		{ GREY.."2) "..AtlasLocale["Whelp Eggs"] };
		{ GREY.."3) "..AtlasLocale["Onyxia"], NPC, 10184 };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Fire"] };
	};
	TheTempleofAhnQiraj = {
		ZoneName = { AtlasLocale["Temple of Ahn'Qiraj"], 3428 };
		Acronym = AtlasLocale["AQ40"];
		Location = { AtlasLocale["Silithus"], 1377 };
		LevelRange = "60+";
		MinLevel = "60";
		PlayerLimit = "40";
		Continent = AtlasLocale["Kalimdor"];
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Brood of Nozdormu"], FACTION, 910 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["The Prophet Skeram"].." ("..AtlasLocale["Outside"]..")", NPC, 15263 };
		{ GREY.."2) "..AtlasLocale["Bug Trio"].." ("..AtlasLocale["Optional"]..")" };
		{ GREY..INDENT..AtlasLocale["Vem"], NPC, 15544 };
		{ GREY..INDENT..AtlasLocale["Lord Kri"], NPC, 15511 };
		{ GREY..INDENT..AtlasLocale["Princess Yauj"], NPC, 15543 };
		{ GREY.."3) "..AtlasLocale["Battleguard Sartura"], NPC, 15516 };
		{ GREY.."4) "..AtlasLocale["Fankriss the Unyielding"], NPC, 15510 };
		{ GREY.."5) "..AtlasLocale["Viscidus"].." ("..AtlasLocale["Optional"]..")", NPC, 15299 };
		{ GREY.."6) "..AtlasLocale["Princess Huhuran"], NPC, 15509 };
		{ GREY.."7) "..AtlasLocale["Twin Emperors"] };
		{ GREY..INDENT..AtlasLocale["Emperor Vek'lor"], NPC, 15276 };
		{ GREY..INDENT..AtlasLocale["Emperor Vek'nilash"], NPC, 15275 };
		{ GREY.."8) "..AtlasLocale["Ouro"].." ("..AtlasLocale["Optional"]..")", NPC, 15517 };
		{ GREY.."9) "..AtlasLocale["C'Thun"], NPC, 15589 };
		{ GREN.."1') "..AtlasLocale["Andorgos <Brood of Malygos>"], NPC, 15502 };
		{ GREN..INDENT..AtlasLocale["Vethsera <Brood of Ysera>"], NPC, 15504 };
		{ GREN..INDENT..AtlasLocale["Kandrostrasz <Brood of Alexstrasza>"], NPC, 15503 };
		{ GREN.."2') "..AtlasLocale["Arygos"], NPC, 15380 };
		{ GREN..INDENT..AtlasLocale["Caelestrasz"], NPC, 15379 };
		{ GREN..INDENT..AtlasLocale["Merithra of the Dream"], NPC, 15378 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["AQ Enchants"] };
		{ GREY..INDENT..AtlasLocale["AQ40 Sets"] };
		{ GREY..INDENT..AtlasLocale["AQ Opening Quest Chain"] };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Nature"] };
	};
	TheRuinsofAhnQiraj = {
		ZoneName = { AtlasLocale["Ruins of Ahn'Qiraj"], 3429 };
		Acronym = AtlasLocale["AQ20"];
		Location = { AtlasLocale["Silithus"], 1377 };
		LevelRange = "60+";
		MinLevel = "60";
		PlayerLimit = "20";
		Continent = AtlasLocale["Kalimdor"];
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Cenarion Circle"], FACTION, 609 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Kurinnaxx"], NPC, 15348 };
		{ GREY..INDENT..AtlasLocale["Lieutenant General Andorov"], NPC, 15471 };
		{ GREY..INDENT..AtlasLocale["Four Kaldorei Elites"], NPC, 15473 };
		{ GREY.."2) "..AtlasLocale["General Rajaxx"], NPC, 15341 };
		{ GREY..INDENT..AtlasLocale["Captain Qeez"], NPC, 15391 };
		{ GREY..INDENT..AtlasLocale["Captain Tuubid"], NPC, 15392 };
		{ GREY..INDENT..AtlasLocale["Captain Drenn"], NPC, 15389 };
		{ GREY..INDENT..AtlasLocale["Captain Xurrem"], NPC, 15390 };
		{ GREY..INDENT..AtlasLocale["Major Yeggeth"], NPC, 15386 };
		{ GREY..INDENT..AtlasLocale["Major Pakkon"], NPC, 15388 };
		{ GREY..INDENT..AtlasLocale["Colonel Zerran"], NPC, 15385 };
		{ GREY.."3) "..AtlasLocale["Moam"].." ("..AtlasLocale["Optional"]..")", NPC, 15340 };
		{ GREY.."4) "..AtlasLocale["Buru the Gorger"].." ("..AtlasLocale["Optional"]..")", NPC, 15370 };
		{ GREY.."5) "..AtlasLocale["Ayamiss the Hunter"].." ("..AtlasLocale["Optional"]..")", NPC, 15369 };
		{ GREY.."6) "..AtlasLocale["Ossirian the Unscarred"], NPC, 15339 };
		{ GREN.."1') "..AtlasLocale["Safe Room"] };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Class Books"] };
		{ GREY..INDENT..AtlasLocale["AQ Enchants"] };
		{ GREY..INDENT..AtlasLocale["AQ20 Sets"] };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Nature"] };
	};

--************************************************
-- Eastern Kingdoms Instances
--************************************************
	
	BlackrockDepths = {
		ZoneName = { AtlasLocale["Blackrock Depths"], 1584 };
		Acronym = AtlasLocale["BRD"];
		Location = { AtlasLocale["Blackrock Mountain"], 25 };
		LevelRange = "52-58";
		MinLevel = "40";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Shadowforge Key"], ITEM, 11000 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Prison Cell Key"], ITEM, 11140 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Banner of Provocation"].." ("..AtlasLocale["D2 Summon"]..")", ITEM, 21986 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Lord Roccor"], NPC, 9025 };
		{ GREY.."2) "..AtlasLocale["Kharan Mighthammer"], NPC, 9021 };
		{ GREY.."3) "..AtlasLocale["Commander Gor'shak <Kargath Expeditionary Force>"], NPC, 9020 };
		{ GREY.."4) "..AtlasLocale["Marshal Windsor"], NPC, 9023 };
		{ GREY.."5) "..AtlasLocale["High Interrogator Gerstahn <Twilight's Hammer Interrogator>"], NPC, 9018 };
		{ GREY.."6) "..AtlasLocale["Ring of Law"] };
		{ GREY..INDENT..AtlasLocale["Anub'shiah"].." ("..AtlasLocale["Random"]..")", NPC, 9031 };
		{ GREY..INDENT..AtlasLocale["Eviscerator"].." ("..AtlasLocale["Random"]..")", NPC, 9029 };
		{ GREY..INDENT..AtlasLocale["Gorosh the Dervish"].." ("..AtlasLocale["Random"]..")", NPC, 9027 };
		{ GREY..INDENT..AtlasLocale["Grizzle"].." ("..AtlasLocale["Random"]..")", NPC, 9028 };
		{ GREY..INDENT..AtlasLocale["Hedrum the Creeper"].." ("..AtlasLocale["Random"]..")", NPC, 9032 };
		{ GREY..INDENT..AtlasLocale["Ok'thor the Breaker"].." ("..AtlasLocale["Random"]..")", NPC, 9030 };
		{ GREY..INDENT..AtlasLocale["Theldren"].." ("..AtlasLocale["Summon"]..")", NPC, 16059 };
		{ GREY..INDENT..AtlasLocale["Lefty"].." ("..AtlasLocale["BRDRogue"]..GREY..")", NPC, 16049 };
		{ GREY..INDENT..AtlasLocale["Malgen Longspear"].." ("..AtlasLocale["BRDHunter"]..GREY..")", NPC, 16052 };
		{ GREY..INDENT..AtlasLocale["Gnashjaw <Malgen Longspear's Pet>"].." ("..AtlasLocale["BRDPet"]..GREY..")", NPC, 16095 };
		{ GREY..INDENT..AtlasLocale["Korv"].." ("..AtlasLocale["BRDShaman"]..GREY..")", NPC, 16050 };
		{ GREY..INDENT..AtlasLocale["Rezznik"].." ("..AtlasLocale["BRDEngineer"]..GREY..")", NPC, 16050 };
		{ GREY..INDENT..AtlasLocale["Rotfang"].." ("..AtlasLocale["BRDRogue"]..GREY..")", NPC, 16050 };
		{ GREY..INDENT..AtlasLocale["Snokh Blackspine"].." ("..AtlasLocale["BRDMage"]..GREY..")", NPC, 16050 };
		{ GREY..INDENT..AtlasLocale["Va'jashni"].." ("..AtlasLocale["BRDPriest"]..GREY..")", NPC, 16055 };
		{ GREY..INDENT..AtlasLocale["Volida"].." ("..AtlasLocale["BRDMage"]..GREY..")", NPC, 16055 };
		{ GREY..INDENT..AtlasLocale["Houndmaster Grebmar"].." ("..AtlasLocale["Lower"]..")", NPC, 9319 };
		{ GREY..INDENT..AtlasLocale["Elder Morndeep"].." ("..AtlasLocale["Lunar Festival"]..")", NPC, 15549 };
		{ GREY..INDENT..AtlasLocale["High Justice Grimstone"], NPC, 10096 };
		{ GREY.."7) "..AtlasLocale["Monument of Franclorn Forgewright"], OBJECT, 164689 };
		{ GREY..INDENT..AtlasLocale["Pyromancer Loregrain"].." ("..AtlasLocale["Rare"]..")", NPC, 9024 };
		{ GREY.."8) "..AtlasLocale["The Vault"] };
		{ GREY..INDENT..AtlasLocale["Warder Stilgiss"].." ("..AtlasLocale["Rare"]..")", NPC, 9041 };
		{ GREY..INDENT..AtlasLocale["Verek"].." ("..AtlasLocale["Rare"]..")", NPC, 9042 };
		{ GREY..INDENT..AtlasLocale["Watchman Doomgrip"], NPC, 9476 };
		{ GREY.."9) "..AtlasLocale["Fineous Darkvire <Chief Architect>"], NPC, 9056 };
		{ GREY.."10) "..AtlasLocale["Lord Incendius"], NPC, 9017 };
		{ GREY..INDENT..AtlasLocale["The Black Anvil"] };
		{ GREY.."11) "..AtlasLocale["Bael'Gar"], NPC, 9016 };
		{ GREY.."12) "..AtlasLocale["Shadowforge Lock"] };
		{ GREY.."13) "..AtlasLocale["General Angerforge"], NPC, 9033 };
		{ GREY.."14) "..AtlasLocale["Golem Lord Argelmach"], NPC, 8983 };
		{ GREY..INDENT..AtlasLocale["Field Repair Bot 74A"], NPC, 14337 };
		{ GREY..INDENT..AtlasLocale["Blacksmithing Plans"], OBJECT, 173232 };
		{ GREY.."15) "..AtlasLocale["The Grim Guzzler"] };
		{ GREY..INDENT..AtlasLocale["Hurley Blackbreath"], NPC, 9537 };
		{ GREY..INDENT..AtlasLocale["Lokhtos Darkbargainer <The Thorium Brotherhood>"], NPC, 12944 };
		{ GREY..INDENT..AtlasLocale["Mistress Nagmara"], NPC, 9500 };
		{ GREY..INDENT..AtlasLocale["Phalanx"], NPC, 9502 };
		{ GREY..INDENT..AtlasLocale["Plugger Spazzring"], NPC, 9499 };
		{ GREY..INDENT..AtlasLocale["Private Rocknot"], NPC, 9503 };
		{ GREY..INDENT..AtlasLocale["Ribbly Screwspigot"], NPC, 9543 };
		{ GREY.."16) "..AtlasLocale["Ambassador Flamelash"], NPC, 9156 };
		{ GREY.."17) "..AtlasLocale["Panzor the Invincible"].." ("..AtlasLocale["Rare"]..")", NPC, 8923 };
		{ GREY..INDENT..AtlasLocale["Blacksmithing Plans"], OBJECT, 173232 };
		{ GREY.."18) "..AtlasLocale["Summoner's Tomb"] };
		{ GREY..INDENT..AtlasLocale["Chest of The Seven"], OBJECT, 169243 };
		{ GREY.."19) "..AtlasLocale["The Lyceum"] };
		{ GREY.."20) "..AtlasLocale["Magmus"], NPC, 9938 };
		{ GREY.."21) "..AtlasLocale["Emperor Dagran Thaurissan"], NPC, 9019 };
		{ GREY..INDENT..AtlasLocale["Princess Moira Bronzebeard <Princess of Ironforge>"], NPC, 8929 };
		{ GREY..INDENT..AtlasLocale["High Priestess of Thaurissan"], NPC, 10076 };
		{ GREY.."22) "..AtlasLocale["The Black Forge"] };
		{ GREY.."23) "..AtlasLocale["Molten Core"], ZONE, 2717 };
		{ GREY..INDENT..AtlasLocale["Core Fragment"], OBJECT, 179553 };
		{ GREY.."24) "..AtlasLocale["Overmaster Pyron"], NPC, 9026 };
		{ GREY.."25) "..AtlasLocale["Blacksmithing Plans"], OBJECT, 173232 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["The Gladiator"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Ironweave Battlesuit"] };
	};
	BlackrockSpireLower = {
		ZoneName = { AtlasLocale["Lower"].." "..AtlasLocale["Blackrock Spire"], 1583 };
		Acronym = AtlasLocale["LBRS"];
		Location = { AtlasLocale["Blackrock Mountain"], 25 };
		LevelRange = "55-60";
		MinLevel = "55";
		PlayerLimit = "10";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Brazier of Invocation"].." ("..AtlasLocale["D2 Summon"]..")", ITEM, 22057 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Upper Blackrock Spire"].." ("..AtlasLocale["UBRS"]..")", ZONE, 1583 };
		{ BLUE.."C-F) "..AtlasLocale["Connections"] };
		{ GREY.."1) "..AtlasLocale["Vaelan"].." ("..AtlasLocale["Upper"]..")", NPC, 10296 };
		{ GREY.."2) "..AtlasLocale["Warosh <The Cursed>"].." ("..AtlasLocale["Wanders"]..")", NPC, 10799 };
		{ GREY..INDENT..AtlasLocale["Elder Stonefort"].." ("..AtlasLocale["Lunar Festival"]..")", NPC, 15560 };
		{ GREY.."3) "..AtlasLocale["Roughshod Pike"], OBJECT, 175886 };
		{ GREY.."4) "..AtlasLocale["Spirestone Butcher"].." ("..AtlasLocale["Rare"]..")", NPC, 9219 };
		{ GREY.."5) "..AtlasLocale["Highlord Omokk"], NPC, 9196 };
		{ GREY.."6) "..AtlasLocale["Spirestone Battle Lord"].." ("..AtlasLocale["Rare"]..")", NPC, 9218 };
		{ GREY..INDENT..AtlasLocale["Spirestone Lord Magus"].." ("..AtlasLocale["Rare"]..")", NPC, 9217 };
		{ GREY.."7) "..AtlasLocale["Shadow Hunter Vosh'gajin"], NPC, 9236 };
		{ GREY..INDENT..AtlasLocale["Fifth Mosh'aru Tablet"], OBJECT, 175949 };
		{ GREY.."8) "..AtlasLocale["Bijou"], NPC, 10257 };
		{ GREY.."9) "..AtlasLocale["War Master Voone"], NPC, 9237 };
		{ GREY..INDENT..AtlasLocale["Mor Grayhoof"].." ("..AtlasLocale["Summon"]..")", NPC, 16080 };
		{ GREY..INDENT..AtlasLocale["Sixth Mosh'aru Tablet"], OBJECT, 175950 };
		{ GREY.."10) "..AtlasLocale["Bijou's Belongings"], OBJECT, 175334 };
		{ GREY.."11) "..AtlasLocale["Human Remains"].." ("..AtlasLocale["Lower"]..")", OBJECT, 176090 };
		{ GREY..INDENT..AtlasLocale["Unfired Plate Gauntlets"].." ("..AtlasLocale["Lower"]..")", OBJECT, 176089 };
		{ GREY.."12) "..AtlasLocale["Bannok Grimaxe <Firebrand Legion Champion>"].." ("..AtlasLocale["Rare"]..")", NPC, 9596 };
		{ GREY.."13) "..AtlasLocale["Mother Smolderweb"], NPC, 10596 };
		{ GREY.."14) "..AtlasLocale["Crystal Fang"].." ("..AtlasLocale["Rare"]..")", NPC, 10376 };
		{ GREY.."15) "..AtlasLocale["Urok's Tribute Pile"], OBJECT, 175621 };
		{ GREY..INDENT..AtlasLocale["Urok Doomhowl"].." ("..AtlasLocale["Summon"]..")", NPC, 10584 };
		{ GREY.."16) "..AtlasLocale["Quartermaster Zigris <Bloodaxe Legion>"], NPC, 9736 };
		{ GREY.."17) "..AtlasLocale["Halycon"], NPC, 10220 };
		{ GREY..INDENT..AtlasLocale["Gizrul the Slavener"], NPC, 10268 };
		{ GREY.."18) "..AtlasLocale["Ghok Bashguud <Bloodaxe Champion>"].." ("..AtlasLocale["Rare"]..")", NPC, 9718 };
		{ GREY.."19) "..AtlasLocale["Overlord Wyrmthalak"], NPC, 9568 };
		{ GREN.."1') "..AtlasLocale["Burning Felguard"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Summon"]..")", NPC, 10263 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Ironweave Battlesuit"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Spider's Kiss"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["D1/D2 Sets"] };
	};
	BlackrockSpireUpper = {
		ZoneName = { AtlasLocale["Upper"].." "..AtlasLocale["Blackrock Spire"], 1583 };
		Acronym = AtlasLocale["UBRS"];
		Location = { AtlasLocale["Blackrock Mountain"], 25 };
		LevelRange = "56-60";
		MinLevel = "55";
		PlayerLimit = "10";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Seal of Ascension"], ITEM, 12344 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Brazier of Invocation"].." ("..AtlasLocale["D2 Summon"]..")", ITEM, 22057 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Lower Blackrock Spire"].." ("..AtlasLocale["LBRS"]..")", ZONE, 1583 };
		{ BLUE.."C-E) "..AtlasLocale["Connections"] };
		{ GREY.."1) "..AtlasLocale["Pyroguard Emberseer"], NPC, 9816 };
		{ GREY.."2) "..AtlasLocale["Solakar Flamewreath"], NPC, 10264 };
		{ GREY..INDENT..AtlasLocale["Father Flame"], OBJECT, 175245 };
		{ GREY.."3) "..AtlasLocale["Darkstone Tablet"], OBJECT, 175385 };
		{ GREY..INDENT..AtlasLocale["Doomrigger's Coffer"], OBJECT, 175382 };
		{ GREY.."4) "..AtlasLocale["Jed Runewatcher <Blackhand Legion>"].." ("..AtlasLocale["Rare"]..")", NPC, 10509 };
		{ GREY.."5) "..AtlasLocale["Goraluk Anvilcrack <Blackhand Legion Armorsmith>"].." ("..AtlasLocale["Rare"]..")", NPC, 10899 };
		{ GREY.."6) "..AtlasLocale["Warchief Rend Blackhand"], NPC, 10429 };
		{ GREY..INDENT..AtlasLocale["Gyth <Rend Blackhand's Mount>"], NPC, 10339 };
		{ GREY.."7) "..AtlasLocale["Awbee"], NPC, 10740 };
		{ GREY.."8) "..AtlasLocale["The Beast"], NPC, 10430 };
		{ GREY..INDENT..AtlasLocale["Lord Valthalak"].." ("..AtlasLocale["Summon"]..")", NPC, 16042 };
		{ GREY..INDENT..AtlasLocale["Finkle Einhorn"], NPC, 10776 };
		{ GREY.."9) "..AtlasLocale["General Drakkisath"], NPC, 10363 };
		{ GREY..INDENT..AtlasLocale["Drakkisath's Brand"], OBJECT, 179880 };
		{ GREY.."10) "..AtlasLocale["Blackwing Lair"].." ("..AtlasLocale["BWL"]..")", ZONE, 2677 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Ironweave Battlesuit"] };
		{ GREY..INDENT..AtlasLocale["D1/D2 Sets"] };
	};
	BlackwingLair = {
		ZoneName = { AtlasLocale["Blackwing Lair"], 2677 };
		Acronym = AtlasLocale["BWL"];
		Location = { AtlasLocale["Blackrock Spire"], 1583 };
		LevelRange = "60+";
		MinLevel = "60";
		PlayerLimit = "40";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Attunement Required"] };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Connection"] };
		{ BLUE.."C) "..AtlasLocale["Connection"] };
		{ GREY.."1) "..AtlasLocale["Razorgore the Untamed"], NPC, 12435 };
		{ GREY.."2) "..AtlasLocale["Vaelastrasz the Corrupt"], NPC, 13020 };
		{ GREY.."3) "..AtlasLocale["Broodlord Lashlayer"], NPC, 12017 };
		{ GREY.."4) "..AtlasLocale["Firemaw"], NPC, 11983 };
		{ GREY.."5) "..AtlasLocale["Master Elemental Shaper Krixix"], NPC, 14401 };
		{ GREY.."6) "..AtlasLocale["Ebonroc"], NPC, 14601 };
		{ GREY.."7) "..AtlasLocale["Flamegor"], NPC, 11981 };
		{ GREY.."8) "..AtlasLocale["Chromaggus"], NPC, 14020 };
		{ GREY.."9) "..AtlasLocale["Nefarian"], NPC, 11583 };
		{ GREN.."1') "..AtlasLocale["Alchemy Lab"] };
		{ GREN.."2') "..AtlasLocale["Draconic for Dummies"], OBJECT, 180666 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["T2 Sets"] };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Fire"] };
	};
	Gnomeregan = {
		ZoneName = { AtlasLocale["Gnomeregan"], 133 };
		Acronym = AtlasLocale["Gnome"];
		Location = { AtlasLocale["Dun Morogh"], 1 };
		LevelRange = "24-32";
		MinLevel = "23";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Workshop Key"].." ("..AtlasLocale["Back"]..")", ITEM, 6893 };
		{ BLUE.."A) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Front"]..")" };
		{ BLUE.."B) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Back"]..")" };
		{ GREY.."1) "..AtlasLocale["Blastmaster Emi Shortfuse"], NPC, 7998 };
		{ GREY..INDENT..AtlasLocale["Grubbis"], NPC, 7361 };
		{ GREY..INDENT..AtlasLocale["Chomper"], NPC, 6215 };
		{ GREY.."2) "..AtlasLocale["Clean Room"] };
		{ GREY..INDENT..AtlasLocale["Tink Sprocketwhistle <Engineering Supplies>"], NPC, 9676 };
		{ GREY..INDENT..AtlasLocale["The Sparklematic 5200"], OBJECT, 142487 };
		{ GREY..INDENT..AtlasLocale["Mail Box"] };
		{ GREY.."3) "..AtlasLocale["Kernobee"], NPC, 7850 };
		{ GREY..INDENT..AtlasLocale["Alarm-a-bomb 2600"], NPC, 7897 };
		{ GREY..INDENT..AtlasLocale["Matrix Punchograph 3005-B"], OBJECT, 142475 };
		{ GREY.."4) "..AtlasLocale["Viscous Fallout"], NPC, 7079 };
		{ GREY.."5) "..AtlasLocale["Electrocutioner 6000"], NPC, 6235 };
		{ GREY..INDENT..AtlasLocale["Matrix Punchograph 3005-C"], OBJECT, 142476 };
		{ GREY.."6) "..AtlasLocale["Crowd Pummeler 9-60"].." ("..AtlasLocale["Upper"]..")", NPC, 6229 };
		{ GREY..INDENT..AtlasLocale["Matrix Punchograph 3005-D"], OBJECT, 142696 };
		{ GREY.."7) "..AtlasLocale["Dark Iron Ambassador"].." ("..AtlasLocale["Rare"]..")", NPC, 6228 };
		{ GREY.."8) "..AtlasLocale["Mekgineer Thermaplugg"], NPC, 7800 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	MoltenCore = {
		ZoneName = { AtlasLocale["Molten Core"], 2717 };
		Acronym = AtlasLocale["MC"];
		Location = { AtlasLocale["Blackrock Depths"], 1584 };
		LevelRange = "60+";
		MinLevel = "60";
		PlayerLimit = "40";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Attunement Required"] };
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Hydraxian Waterlords"], FACTION, 749 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Aqual Quintessence"].." ("..AtlasLocale["Boss"]..")", ITEM, 17333 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Eternal Quintessence"].." ("..AtlasLocale["Boss"]..")", ITEM, 22754 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Lucifron"], NPC, 12118 };
		{ GREY.."2) "..AtlasLocale["Magmadar"], NPC, 11982 };
		{ GREY.."3) "..AtlasLocale["Gehennas"], NPC, 12259 };
		{ GREY.."4) "..AtlasLocale["Garr"], NPC, 12057 };
		{ GREY.."5) "..AtlasLocale["Shazzrah"], NPC, 12264 };
		{ GREY.."6) "..AtlasLocale["Baron Geddon"], NPC, 12056 };
		{ GREY.."7) "..AtlasLocale["Golemagg the Incinerator"], NPC, 11988 };
		{ GREY.."8) "..AtlasLocale["Sulfuron Harbinger"], NPC, 12098 };
		{ GREY.."9) "..AtlasLocale["Majordomo Executus"], NPC, 12018 };
		{ GREY.."10) "..AtlasLocale["Ragnaros"], NPC, 11502 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Random Boss Loot"] };
		{ GREY..INDENT..AtlasLocale["T1 Sets"] };
		{ GREY..INDENT..AtlasLocale["T2 Sets"] };
		{ "" };
		{ "" };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Fire"] };
	};
	SMLibrary = {
		ZoneName = { AtlasLocale["Scarlet Monastery"]..": "..AtlasLocale["Library"], 796 };
		Acronym = AtlasLocale["SM Lib"];
		Location = { AtlasLocale["Tirisfal Glades"], 85 };
		LevelRange = "31-37";
		MinLevel = "25";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Houndmaster Loksey"], NPC, 3974 };
		{ GREY.."2) "..AtlasLocale["Arcanist Doan"], NPC, 6487 };
		{ GREN.."1') "..AtlasLocale["Doan's Strongbox"], OBJECT, 103821 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Chain of the Scarlet Crusade"] };
	};
	SMArmory = {
		ZoneName = { AtlasLocale["Scarlet Monastery"]..": "..AtlasLocale["Armory"], 796 };
		Acronym = AtlasLocale["SM Arm"];
		Location = { AtlasLocale["Tirisfal Glades"], 85 };
		LevelRange = "33-40";
		MinLevel = "25";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["The Scarlet Key"], ITEM, 7146 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Herod <The Scarlet Champion>"], NPC, 3975 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Chain of the Scarlet Crusade"] };
	};
	SMCathedral = {
		ZoneName = { AtlasLocale["Scarlet Monastery"]..": "..AtlasLocale["Cathedral"], 796 };
		Acronym = AtlasLocale["SM Cath"];
		Location = { AtlasLocale["Tirisfal Glades"], 85 };
		LevelRange = "36-42";
		MinLevel = "25";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["The Scarlet Key"], ITEM, 7146 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["High Inquisitor Fairbanks"], NPC, 4542 };
		{ GREY.."2) "..AtlasLocale["Scarlet Commander Mograine"], NPC, 3976 };
		{ GREY.."3) "..AtlasLocale["High Inquisitor Whitemane"], NPC, 3977 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Chain of the Scarlet Crusade"] };
	};
	SMGraveyard = {
		ZoneName = { AtlasLocale["Scarlet Monastery"]..": "..AtlasLocale["Graveyard"], 796 };
		Acronym = AtlasLocale["SM GY"];
		Location = { AtlasLocale["Tirisfal Glades"], 85 };
		LevelRange = "28-35";
		MinLevel = "25";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Interrogator Vishas"], NPC, 3983 };
		{ GREY..INDENT..AtlasLocale["Vorrel Sengutz"], NPC, 3981 };
		{ GREY.."2) "..AtlasLocale["Scorn"].." ("..AtlasLocale["Scourge Invasion"]..")", NPC, 14693 };
		{ GREY.."3) "..AtlasLocale["Bloodmage Thalnos"], NPC, 4543 };
		{ GREN.."1') "..AtlasLocale["Ironspine"].." ("..AtlasLocale["Rare"]..")", NPC, 6489 };
		{ GREN..INDENT..AtlasLocale["Azshir the Sleepless"].." ("..AtlasLocale["Rare"]..")", NPC, 6490 };
		{ GREN..INDENT..AtlasLocale["Fallen Champion"].." ("..AtlasLocale["Rare"]..")", NPC, 6488 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Chain of the Scarlet Crusade"] };
	};
	Scholomance = {
		ZoneName = { AtlasLocale["Scholomance"], 2057 };
		Acronym = AtlasLocale["Scholo"];
		Location = { AtlasLocale["Western Plaguelands"], 28 };
		LevelRange = "58-60";
		MinLevel = "45";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Argent Dawn"], FACTION, 529 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Skeleton Key"], ITEM, 13704 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Viewing Room Key"].." ("..AtlasLocale["Viewing Room"]..")", ITEM, 13873 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Blood of Innocents"].." ("..AtlasLocale["Kirtonos the Herald"]..")", ITEM, 13523 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Brazier of Invocation"].." ("..AtlasLocale["D2 Summon"]..")", ITEM, 22057 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Divination Scryer"].." ("..AtlasLocale["Death Knight Darkreaver"]..")", ITEM, 18746 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Connection"] };
		{ BLUE.."C) "..AtlasLocale["Connection"] };
		{ GREY.."1) "..AtlasLocale["Blood Steward of Kirtonos"], NPC, 14861 };
		{ GREY..INDENT..AtlasLocale["The Deed to Southshore"], OBJECT, 176486 };
		{ GREY.."2) "..AtlasLocale["Kirtonos the Herald"].." ("..AtlasLocale["Summon"]..")", NPC, 10506 };
		{ GREY.."3) "..AtlasLocale["Jandice Barov"], NPC, 10503 };
		{ GREY..INDENT..AtlasLocale["Journal of Jandice Barov"], OBJECT, 180794 };
		{ GREY.."4) "..AtlasLocale["The Deed to Tarren Mill"], OBJECT, 176487 };
		{ GREY..INDENT..AtlasLocale["Lord Blackwood"].." ("..AtlasLocale["Scourge Invasion"]..")", NPC, 14695 };
		{ GREY.."5) "..AtlasLocale["Rattlegore"].." ("..AtlasLocale["Lower"]..")", NPC, 11622 };
		{ GREY..INDENT..AtlasLocale["Death Knight Darkreaver"].." ("..AtlasLocale["Summon"]..")", NPC, 14516 };
		{ GREY.."6) "..AtlasLocale["Marduk Blackpool"], NPC, 10433 };
		{ GREY..INDENT..AtlasLocale["Vectus"], NPC, 10432 };
		{ GREY.."7) "..AtlasLocale["Ras Frostwhisper"], NPC, 10508 };
		{ GREY..INDENT..AtlasLocale["The Deed to Brill"], OBJECT, 176484 };
		{ GREY..INDENT..AtlasLocale["Kormok"].." ("..AtlasLocale["Summon"]..")", NPC, 16118 };
		{ GREY.."8) "..AtlasLocale["Instructor Malicia"], NPC, 10505 };
		{ GREY.."9) "..AtlasLocale["Doctor Theolen Krastinov <The Butcher>"], NPC, 11261 };
		{ GREY.."10) "..AtlasLocale["Lorekeeper Polkelt"], NPC, 10901 };
		{ GREY.."11) "..AtlasLocale["The Ravenian"], NPC, 10507 };
		{ GREY.."12) "..AtlasLocale["Lord Alexei Barov"], NPC, 10504 };
		{ GREY..INDENT..AtlasLocale["The Deed to Caer Darrow"], OBJECT, 176485 };
		{ GREY.."13) "..AtlasLocale["Lady Illucia Barov"], NPC, 10502 };
		{ GREY.."14) "..AtlasLocale["Darkmaster Gandling"], NPC, 1853 };
		{ GREN.."1') "..AtlasLocale["Torch Lever"] };
		{ GREN.."2') "..AtlasLocale["Old Treasure Chest"], OBJECT, 176944 };
		{ GREN.."3') "..AtlasLocale["Alchemy Lab"] };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Necropile Raiment"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Cadaverous Garb"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Bloodmail Regalia"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Deathbone Guardian"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Ironweave Battlesuit"] };
		{ GREY..INDENT..AtlasLocale["D1/D2 Sets"] };
	};
	ShadowfangKeep = {
		ZoneName = { AtlasLocale["Shadowfang Keep"], 209 };
		Acronym = AtlasLocale["SFK"];
		Location = { AtlasLocale["Silverpine Forest"], 130 };
		LevelRange = "17-25";
		MinLevel = "10";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Rethilgore <The Cell Keeper>"], NPC, 3914 };
		{ GREY..INDENT..AtlasLocale["Sorcerer Ashcrombe"], NPC, 3850 };
		{ GREY..INDENT..AtlasLocale["Deathstalker Adamant"], NPC, 3849 };
		{ GREY.."2) "..AtlasLocale["Deathstalker Vincent"], NPC, 4444 };
		{ GREY.."3) "..AtlasLocale["Fel Steed"], NPC, 3864 };
		{ GREY..INDENT..AtlasLocale["Jordan's Hammer"], OBJECT, 91138 };
		{ GREY.."4) "..AtlasLocale["Razorclaw the Butcher"], NPC, 3886 };
		{ GREY.."5) "..AtlasLocale["Baron Silverlaine"], NPC, 3887 };
		{ GREY.."6) "..AtlasLocale["Commander Springvale"], NPC, 4278 };
		{ GREY.."7) "..AtlasLocale["Sever"].." ("..AtlasLocale["Scourge Invasion"]..")", NPC, 4279 };
		{ GREY.."8) "..AtlasLocale["Odo the Blindwatcher"], NPC, 4279 };
		{ GREY.."9) "..AtlasLocale["Deathsworn Captain"].." ("..AtlasLocale["Rare"]..")", NPC, 3872 };
		{ GREY.."10) "..AtlasLocale["Fenrus the Devourer"], NPC, 4274 };
		{ GREY..INDENT..AtlasLocale["Arugal's Voidwalker"], NPC, 4627 };
		{ GREY..INDENT..AtlasLocale["The Book of Ur"], OBJECT, 36738 };
		{ GREY.."11) "..AtlasLocale["Wolf Master Nandos"], NPC, 3927 };
		{ GREY.."12) "..AtlasLocale["Archmage Arugal"], NPC, 4275 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	Stratholme = {
		ZoneName = { AtlasLocale["Stratholme"], 2017 };
		Acronym = AtlasLocale["Strat"];
		Location = { AtlasLocale["Eastern Plaguelands"], 139 };
		LevelRange = "58-60";
		MinLevel = "53";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Argent Dawn"], FACTION, 529 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["The Scarlet Key"].." ("..AtlasLocale["Living Side"]..")", ITEM, 7146 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Key to the City"].." ("..AtlasLocale["Undead Side"]..")", ITEM, 12382 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Various Postbox Keys"].." ("..AtlasLocale["Postmaster Malown"]..")" };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Brazier of Invocation"].." ("..AtlasLocale["D2 Summon"]..")", ITEM, 22057 };
		{ BLUE.."A) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Front"]..")" };
		{ BLUE.."B) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Side"]..")" };
		{ GREY.."1) "..AtlasLocale["Skul"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Varies"]..")", NPC, 10393 };
		{ GREY..INDENT..AtlasLocale["Stratholme Courier"], NPC, 11082 };
		{ GREY..INDENT..AtlasLocale["Fras Siabi"], NPC, 11058 };
		{ GREY.."2) "..AtlasLocale["Atiesh <Hand of Sargeras>"].." ("..AtlasLocale["Summon"]..")", NPC, 16387 };
		{ GREY..INDENT..AtlasLocale["Balzaphon"].." ("..AtlasLocale["Scourge Invasion"]..")", NPC, 14684 };
		{ GREY.."3) "..AtlasLocale["Hearthsinger Forresten"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Varies"]..")", NPC, 10558 };
		{ GREY.."4) "..AtlasLocale["The Unforgiven"], NPC, 10516 };
		{ GREY.."5) "..AtlasLocale["Elder Farwhisper"].." ("..AtlasLocale["Lunar Festival"]..")", NPC, 15607 };
		{ GREY.."6) "..AtlasLocale["Timmy the Cruel"], NPC, 10808 };
		{ GREY.."7) "..AtlasLocale["Malor the Zealous"], NPC, 11032 };
		{ GREY..INDENT..AtlasLocale["Malor's Strongbox"], OBJECT, 176112 };
		{ GREY.."8) "..AtlasLocale["Crimson Hammersmith"].." ("..AtlasLocale["Summon"]..")", NPC, 11120 };
		{ GREY..INDENT..AtlasLocale["Blacksmithing Plans"], OBJECT, 173232 };
		{ GREY.."9) "..AtlasLocale["Cannon Master Willey"], NPC, 10997 };
		{ GREY.."10) "..AtlasLocale["Archivist Galford"], NPC, 10811 };
		{ GREY.."11) "..AtlasLocale["Grand Crusader Dathrohan"], NPC, 10812 };
		{ GREY..INDENT..AtlasLocale["Balnazzar"], NPC, 10813 };
		{ GREY..INDENT..AtlasLocale["Sothos"].." & "..AtlasLocale["Jarien"].." ("..AtlasLocale["Summon"]..")", NPC, 16102 };
		{ GREY.."12) "..AtlasLocale["Magistrate Barthilas"].." ("..AtlasLocale["Varies"]..")", NPC, 10435 };
		{ GREY.."13) "..AtlasLocale["Aurius"], NPC, 10917 };
		{ GREY.."14) "..AtlasLocale["Stonespine"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Wanders"]..")", NPC, 10809 };
		{ GREY.."15) "..AtlasLocale["Baroness Anastari"], NPC, 10436 };
		{ GREY..INDENT..AtlasLocale["Black Guard Swordsmith"].." ("..AtlasLocale["Summon"]..")", NPC, 11121 };
		{ GREY..INDENT..AtlasLocale["Blacksmithing Plans"], OBJECT, 173232 };
		{ GREY.."16) "..AtlasLocale["Nerub'enkan"], NPC, 10437 };
		{ GREY.."17) "..AtlasLocale["Maleki the Pallid"], NPC, 10438 };
		{ GREY.."18) "..AtlasLocale["Ramstein the Gorger"], NPC, 10439 };
		{ GREY.."19) "..AtlasLocale["Baron Rivendare"], NPC, 10440 };
		{ GREY..INDENT..AtlasLocale["Ysida Harmon"], NPC, 16031 };
		{ GREN.."1') "..AtlasLocale["Crusaders' Square Postbox"], OBJECT, 176349 };
		{ GREN.."2') "..AtlasLocale["Market Row Postbox"], OBJECT, 176346 };
		{ GREN.."3') "..AtlasLocale["Festival Lane Postbox"], OBJECT, 176350 };
		{ GREN.."4') "..AtlasLocale["Elders' Square Postbox"], OBJECT, 176351 };
		{ GREN.."5') "..AtlasLocale["King's Square Postbox"], OBJECT, 176352 };
		{ GREN.."6') "..AtlasLocale["Fras Siabi's Postbox"], OBJECT, 176353 };
		{ GREN..INDENT..AtlasLocale["Third Postbox Opened"]..": "..AtlasLocale["Postmaster Malown"], NPC, 11143 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["The Postmaster"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Ironweave Battlesuit"] };
		{ GREY..INDENT..AtlasLocale["D1/D2 Sets"] };
	};
	TheDeadmines = {
		ZoneName = { AtlasLocale["The Deadmines"], 1581 };
		Acronym = AtlasLocale["DM"];
		Location = { AtlasLocale["Westfall"], 40 };
		LevelRange = "16-22";
		MinLevel = "13";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Exit"] };
		{ GREY.."1) "..AtlasLocale["Rhahk'Zor <The Foreman>"], NPC, 644 };
		{ GREY.."2) "..AtlasLocale["Miner Johnson"].." ("..AtlasLocale["Rare"]..")", NPC, 3586 };
		{ GREY.."3) "..AtlasLocale["Sneed <Lumbermaster>"], NPC, 643 };
		{ GREY..INDENT..AtlasLocale["Sneed's Shredder <Lumbermaster>"], NPC, 642 };
		{ GREY.."4) "..AtlasLocale["Gilnid <The Smelter>"], NPC, 1763 };
		{ GREY.."5) "..AtlasLocale["Defias Gunpowder"], OBJECT, 17155 };
		{ GREY.."6) "..AtlasLocale["Mr. Smite <The Ship's First Mate>"], NPC, 646 };
		{ GREY..INDENT..AtlasLocale["Cookie <The Ship's Cook>"], NPC, 645 };
		{ GREY..INDENT..AtlasLocale["Captain Greenskin"], NPC, 647 };
		{ GREY..INDENT..AtlasLocale["Edwin VanCleef <Defias Kingpin>"], NPC, 639 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Defias Leather"] };
	};
	TheStockade = {
		ZoneName = { AtlasLocale["The Stockade"], 717 };
		Acronym = AtlasLocale["Stocks"];
		Location = { AtlasLocale["Stormwind City"], 1519 };
		LevelRange = "21-27";
		MinLevel = "15";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["Targorr the Dread"].." ("..AtlasLocale["Varies"]..")", NPC, 1696 };
		{ GREY.."2) "..AtlasLocale["Kam Deepfury"], NPC, 1666 };
		{ GREY.."3) "..AtlasLocale["Hamhock"], NPC, 1717 };
		{ GREY.."4) "..AtlasLocale["Bazil Thredd"], NPC, 1716 };
		{ GREY.."5) "..AtlasLocale["Dextren Ward"], NPC, 1663 };
		{ GREY.."6) "..AtlasLocale["Bruegal Ironknuckle"].." ("..AtlasLocale["Rare"]..")", NPC, 1720 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	TheSunkenTemple = {
		ZoneName = { AtlasLocale["Sunken Temple"], 1417 };
		Acronym = AtlasLocale["ST"];
		Location = { AtlasLocale["Swamp of Sorrows"], 8 };
		LevelRange = "46-53";
		MinLevel = "45";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["AKA"]..": "..AtlasLocale["The Temple of Atal'Hakkar"] };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Yeh'kinya's Scroll"].." ("..AtlasLocale["Avatar of Hakkar"]..")", ITEM, 10818 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Connection"] };
		{ BLUE.."C) "..AtlasLocale["Balcony Minibosses"].." ("..AtlasLocale["Upper"]..")" };
		{ BLUE..INDENT..AtlasLocale["Gasher"], NPC, 5713 };
		{ BLUE..INDENT..AtlasLocale["Loro"], NPC, 5714 };
		{ BLUE..INDENT..AtlasLocale["Hukku"], NPC, 5715 };
		{ BLUE..INDENT..AtlasLocale["Zolo"], NPC, 5712 };
		{ BLUE..INDENT..AtlasLocale["Mijan"], NPC, 5717 };
		{ BLUE..INDENT..AtlasLocale["Zul'Lor"], NPC, 5716 };
		{ GREY.."1) "..AtlasLocale["Altar of Hakkar"], OBJECT, 148836 };
		{ GREY..INDENT..AtlasLocale["Atal'alarion <Guardian of the Idol>"], NPC, 8580 };
		{ GREY.."2) "..AtlasLocale["Spawn of Hakkar"].." ("..AtlasLocale["Wanders"]..")", NPC, 5708 };
		{ GREY.."3) "..AtlasLocale["Avatar of Hakkar"], NPC, 8443 };
		{ GREY.."4) "..AtlasLocale["Jammal'an the Prophet"], NPC, 5710 };
		{ GREY..INDENT..AtlasLocale["Ogom the Wretched"], NPC, 5711 };
		{ GREY.."5) "..AtlasLocale["Elder Starsong"].." ("..AtlasLocale["Lunar Festival"]..")", NPC, 15593 };
		{ GREY.."6) "..AtlasLocale["Dreamscythe"], NPC, 5721 };
		{ GREY..INDENT..AtlasLocale["Weaver"], NPC, 5720 };
		{ GREY.."7) "..AtlasLocale["Morphaz"], NPC, 5719 };
		{ GREY..INDENT..AtlasLocale["Hazzas"], NPC, 5722 };
		{ GREY.."8) "..AtlasLocale["Shade of Eranikus"], NPC, 5709 };
		{ GREY..INDENT..AtlasLocale["Essence Font"], OBJECT, 148512 };
		{ GREY..INDENT..AtlasLocale["Malfurion Stormrage"].." ("..AtlasLocale["Summon"]..")", NPC, 15362 };
		{ GREN.."1'-6') "..AtlasLocale["Statue Activation Order"] };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	Uldaman = {
		ZoneName = { AtlasLocale["Uldaman"], 1337 };
		Acronym = AtlasLocale["Ulda"];
		Location = { AtlasLocale["Badlands"], 3 };
		LevelRange = "36-42";
		MinLevel = "29";
		PlayerLimit = "5";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Staff of Prehistoria"].." ("..AtlasLocale["Ironaya"]..")", ITEM, 7733 };
		{ BLUE.."A) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Front"]..")" };
		{ BLUE.."B) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Back"]..")" };
		{ GREY.."1) "..AtlasLocale["Baelog"], NPC, 6906 };
		{ GREY..INDENT..AtlasLocale["Eric \"The Swift\""], NPC, 6907 };
		{ GREY..INDENT..AtlasLocale["Olaf"], NPC, 6908 };
		{ GREY..INDENT..AtlasLocale["Baelog's Chest"], OBJECT, 123329 };
		{ GREY..INDENT..AtlasLocale["Conspicuous Urn"], OBJECT, 125477 };
		{ GREY.."2) "..AtlasLocale["Remains of a Paladin"], NPC, 6912 };
		{ GREY.."3) "..AtlasLocale["Revelosh"], NPC, 6910 };
		{ GREY.."4) "..AtlasLocale["Ironaya"], NPC, 7228 };
		{ GREY.."5) "..AtlasLocale["Obsidian Sentinel"], NPC, 7023 };
		{ GREY.."6) "..AtlasLocale["Annora <Enchanting Trainer>"], NPC, 11073 };
		{ GREY.."7) "..AtlasLocale["Ancient Stone Keeper"], NPC, 7206 };
		{ GREY.."8) "..AtlasLocale["Galgann Firehammer"], NPC, 7291 };
		{ GREY..INDENT..AtlasLocale["Tablet of Will"], OBJECT, 142088 };
		{ GREY..INDENT..AtlasLocale["Shadowforge Cache"], OBJECT, 113757 };
		{ GREY.."9) "..AtlasLocale["Grimlok <Stonevault Chieftain>"], NPC, 4854 };
		{ GREY.."10) "..AtlasLocale["Archaedas <Ancient Stone Watcher>"].." ("..AtlasLocale["Lower"]..")", NPC, 2748 };
		{ GREY.."11) "..AtlasLocale["The Discs of Norgannon"].." ("..AtlasLocale["Lower"]..")", OBJECT, 131474 };
		{ GREY..INDENT..AtlasLocale["Ancient Treasure"].." ("..AtlasLocale["Lower"]..")", OBJECT, 141979 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	ZulGurub = {
		ZoneName = { AtlasLocale["Zul'Gurub"], 19 };
		Acronym = AtlasLocale["ZG"];
		Location = { AtlasLocale["Stranglethorn Vale"], 33 };
		LevelRange = "60+";
		MinLevel = "51";
		PlayerLimit = "20";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Zandalar Tribe"], FACTION, 270 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Gurubashi Mojo Madness"].." ("..AtlasLocale["Edge of Madness"]..")", ITEM, 19931 };
		{ ORNG..AtlasLocale["Key"]..": "..AtlasLocale["Mudskunk Lure"].." ("..AtlasLocale["Gahz'ranka"]..")", ITEM, 19974 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ GREY.."1) "..AtlasLocale["High Priestess Jeklik"].." ("..AtlasLocale["Bat"]..")", NPC, 14517 };
		{ GREY.."2) "..AtlasLocale["High Priest Venoxis"].." ("..AtlasLocale["Snake"]..")", NPC, 14507 };
		{ GREY.."3) "..AtlasLocale["Zanza the Restless"], NPC, 15042 };
		{ GREY.."4) "..AtlasLocale["High Priestess Mar'li"].." ("..AtlasLocale["Spider"]..")", NPC, 14510 };
		{ GREY.."5) "..AtlasLocale["Bloodlord Mandokir"].." ("..AtlasLocale["Raptor"]..", "..AtlasLocale["Optional"]..")", NPC, 11382 };
		{ GREY..INDENT..AtlasLocale["Ohgan"], NPC, 14988 };
		{ GREY.."6) "..AtlasLocale["Edge of Madness"].." ("..AtlasLocale["Optional"]..")" };
		{ GREY..INDENT..AtlasLocale["Gri'lek"].." ("..AtlasLocale["Random"]..")", NPC, 15082 };
		{ GREY..INDENT..AtlasLocale["Hazza'rah"].." ("..AtlasLocale["Random"]..")", NPC, 15083 };
		{ GREY..INDENT..AtlasLocale["Renataki"].." ("..AtlasLocale["Random"]..")", NPC, 15084 };
		{ GREY..INDENT..AtlasLocale["Wushoolay"].." ("..AtlasLocale["Random"]..")", NPC, 15085 };
		{ GREY.."7) "..AtlasLocale["Gahz'ranka"].." ("..AtlasLocale["Optional"]..", "..AtlasLocale["Summon"]..")", NPC, 15114 };
		{ GREY.."8) "..AtlasLocale["High Priest Thekal"].." ("..AtlasLocale["Tiger"]..")", NPC, 14509 };
		{ GREY..INDENT..AtlasLocale["Zealot Zath"].." ("..AtlasLocale["BRDRogue"]..GREY..")", NPC, 11348 };
		{ GREY..INDENT..AtlasLocale["Zealot Lor'Khan"].." ("..AtlasLocale["BRDShaman"]..GREY..")", NPC, 11347 };
		{ GREY.."9) "..AtlasLocale["High Priestess Arlokk"].." ("..AtlasLocale["Panther"]..")", NPC, 14515 };
		{ GREY.."10) "..AtlasLocale["Jin'do the Hexxer"].." ("..AtlasLocale["Optional"]..")", NPC, 11380 };
		{ GREY.."11) "..AtlasLocale["Hakkar"], NPC, 14834 };
		{ GREN.."1') "..AtlasLocale["Muddy Churning Waters"], OBJECT, 180369 };
		{ GREN.."2') "..AtlasLocale["Jinxed Hoodoo Pile"], OBJECT, 180228 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["Random Boss Loot"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["Primal Blessing"] };
		{ GREY..INDENT..AtlasLocale["Set: "]..AtlasLocale["The Twin Blades of Hakkari"] };
		{ GREY..INDENT..AtlasLocale["ZG Ring Sets"] };
		{ GREY..INDENT..AtlasLocale["ZG Sets"] };
		{ GREY..INDENT..AtlasLocale["ZG Enchants"] };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Nature"] };
	};
	Naxxramas = {
		ZoneName = { AtlasLocale["Naxxramas"], 3456 };
		Acronym = AtlasLocale["Naxx"];
		Location = { AtlasLocale["Eastern Plaguelands"], 139 };
		LevelRange = "60+";
		MinLevel = "55";
		PlayerLimit = "40";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Attunement Required"] };
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Argent Dawn"], FACTION, 529 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE..INDENT..AtlasLocale["Archmage Tarsis Kir-Moldir"], NPC, 16381 };
		{ BLUE..INDENT..AtlasLocale["Mr. Bigglesworth"].." ("..AtlasLocale["Wanders"]..")", NPC, 16998 };
		{ GREY..AtlasLocale["Abomination Wing"] };
		{ GREY..INDENT.."1) "..AtlasLocale["Patchwerk"], NPC, 16028 };
		{ GREY..INDENT.."2) "..AtlasLocale["Grobbulus"], NPC, 15931 };
		{ GREY..INDENT.."3) "..AtlasLocale["Gluth"], NPC, 15932 };
		{ GREY..INDENT.."4) "..AtlasLocale["Thaddius"], NPC, 15928 };
		{ ORNG..AtlasLocale["Spider Wing"] };
		{ ORNG..INDENT.."1) "..AtlasLocale["Anub'Rekhan"], NPC, 15956 };
		{ ORNG..INDENT.."2) "..AtlasLocale["Grand Widow Faerlina"], NPC, 15953 };
		{ ORNG..INDENT.."3) "..AtlasLocale["Maexxna"], NPC, 15952 };
		{ PURP..AtlasLocale["Plague Wing"] };
		{ PURP..INDENT.."1) "..AtlasLocale["Noth the Plaguebringer"], NPC, 15954 };
		{ PURP..INDENT.."2) "..AtlasLocale["Heigan the Unclean"], NPC, 15936 };
		{ PURP..INDENT.."3) "..AtlasLocale["Loatheb"], NPC, 16011 };
		{ _RED..AtlasLocale["Deathknight Wing"] };
		{ _RED..INDENT.."1) "..AtlasLocale["Instructor Razuvious"], NPC, 16061 };
		{ _RED..INDENT.."2) "..AtlasLocale["Gothik the Harvester"], NPC, 16060 };
		{ _RED..INDENT.."3) "..AtlasLocale["The Four Horsemen"] };
		{ _RED..INDENT..INDENT..AtlasLocale["Thane Korth'azz"], NPC, 16064 };
		{ _RED..INDENT..INDENT..AtlasLocale["Lady Blaumeux"], NPC, 16065 };
		{ _RED..INDENT..INDENT..AtlasLocale["Highlord Mograine <The Ashbringer>"], NPC, 16062 };
		{ _RED..INDENT..INDENT..AtlasLocale["Sir Zeliek"], NPC, 16063 };
		{ GREN..AtlasLocale["Frostwyrm Lair"] };
		{ GREN..INDENT.."1) "..AtlasLocale["Sapphiron"], NPC, 15989 };
		{ GREN..INDENT.."2) "..AtlasLocale["Kel'Thuzad"], NPC, 15990 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
		{ GREY..INDENT..AtlasLocale["T3 Sets"] };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Nature"]..", "..AtlasLocale["Fire"]..", "..AtlasLocale["Arcane"]..", "..AtlasLocale["Shadow"]..", "..AtlasLocale["Frost"]};
	};
	
--************************************************
-- Instance Entrance Maps
--************************************************

	BlackfathomDeepsEnt = {
		ZoneName = { AtlasLocale["Blackfathom Deeps"].." ("..AtlasLocale["Entrance"]..")", 719 };
		Location = { AtlasLocale["Ashenvale"], 331 };
		Acronym = AtlasLocale["BFD"];
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Blackfathom Deeps"], ZONE, 719 };
	};
	BlackrockMountainEnt = {
		ZoneName = { AtlasLocale["Blackrock Mountain"].." ("..AtlasLocale["Entrance"]..")", 25 };
		Location = { AtlasLocale["Blackrock Mountain"], 51, 46 };
		Acronym = AtlasLocale["BRM"];
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Searing Gorge"], ZONE, 51 };
		{ BLUE.."B) "..AtlasLocale["Burning Steppes"], ZONE, 46 };
		{ BLUE.."C) "..AtlasLocale["Blackrock Depths"].." ("..AtlasLocale["BRD"]..")", ZONE, 1584 };
		{ BLUE..INDENT..AtlasLocale["Molten Core"].." ("..AtlasLocale["MC"]..") ("..AtlasLocale["through "]..AtlasLocale["BRD"]..")", ZONE, 2717 };
		{ BLUE.."D) "..AtlasLocale["Lower Blackrock Spire"].." ("..AtlasLocale["LBRS"]..")", ZONE, 1583 };
		{ BLUE..INDENT..AtlasLocale["Upper Blackrock Spire"].." ("..AtlasLocale["UBRS"]..")", ZONE, 1583 };
		{ BLUE..INDENT..AtlasLocale["Blackwing Lair"].." ("..AtlasLocale["BWL"]..") ("..AtlasLocale["through "]..AtlasLocale["UBRS"]..")", ZONE, 2677 };
		{ BLUE..INDENT..AtlasLocale["Bodley"].." ("..AtlasLocale["Ghost"]..")", NPC, 16033 };
		{ GREY.."1) "..AtlasLocale["Overmaster Pyron"].." ("..AtlasLocale["Wanders"]..")", NPC, 9026 };
		{ GREY.."2) "..AtlasLocale["Lothos Riftwaker"].." ("..AtlasLocale["MC"].. " "..AtlasLocale["Teleport"]..")", NPC, 14387 };
		{ GREY.."3) "..AtlasLocale["Franclorn Forgewright"].." ("..AtlasLocale["Ghost"]..")", NPC, 8888 };
		{ GREY.."4) "..AtlasLocale["Meeting Stone"].." ("..AtlasLocale["BRD"]..")" };
		{ GREY.."5) "..AtlasLocale["Orb of Command"].." ("..AtlasLocale["BWL"].. " "..AtlasLocale["Teleport"]..")", OBJECT, 179879 };
		{ GREY.."6) "..AtlasLocale["Meeting Stone"].." ("..AtlasLocale["LBRS"]..", "..AtlasLocale["UBRS"]..")" };
		{ GREY.."7) "..AtlasLocale["Scarshield Quartermaster <Scarshield Legion>"].." ("..AtlasLocale["Rare"]..")", NPC, 9046 };
	};
	GnomereganEnt = {
		ZoneName = { AtlasLocale["Gnomeregan"].." ("..AtlasLocale["Entrance"]..")", 133 };
		Location = { AtlasLocale["Dun Morogh"], 1 };
		Acronym = AtlasLocale["Gnome"];
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE..INDENT..AtlasLocale["Meeting Stone"] };
		{ BLUE.."B) "..AtlasLocale["Gnomeregan"].." ("..AtlasLocale["Front"]..")", ZONE, 133 };
		{ BLUE.."C) "..AtlasLocale["Gnomeregan"].." ("..AtlasLocale["Back"]..")", ZONE, 133 };
		{ GREY.."1) "..AtlasLocale["Elevator"] };
		{ GREY.."2) "..AtlasLocale["Transpolyporter"] };
		{ GREY..INDENT..AtlasLocale["Sprok <Away Team>"], NPC, 8320 };
		{ GREY.."3) "..AtlasLocale["Matrix Punchograph 3005-A"], OBJECT, 142345 };
		{ GREY..INDENT..AtlasLocale["Namdo Bizzfizzle <Engineering Supplies>"], NPC, 2683 };
		{ GREY.."4) "..AtlasLocale["Techbot"], NPC, 6231 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Trash Mobs"] };
	};
	MaraudonEnt = {
		ZoneName = { AtlasLocale["Maraudon"].." ("..AtlasLocale["Entrance"]..")", 2100 };
		Location = { AtlasLocale["Desolace"], 405 };
		Acronym = AtlasLocale["Mara"];
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE..INDENT..AtlasLocale["The Nameless Prophet"], NPC, 13718 };
		{ BLUE.."B) "..AtlasLocale["Maraudon"].." ("..AtlasLocale["Purple"]..")", ZONE, 2100 };
		{ BLUE.."C) "..AtlasLocale["Maraudon"].." ("..AtlasLocale["Orange"]..")", ZONE, 2100 };
		{ BLUE.."D) "..AtlasLocale["Maraudon"].." ("..AtlasLocale["Portal"]..")", ZONE, 2100 };
		{ GREY.."1) "..AtlasLocale["Kolk <The First Kahn>"], NPC, 13742 };
		{ GREY.."2) "..AtlasLocale["Gelk <The Second Kahn>"], NPC, 13741 };
		{ GREY.."3) "..AtlasLocale["Magra <The Third Kahn>"], NPC, 13740 };
		{ GREY.."4) "..AtlasLocale["Cavindra"], NPC, 13697 };
		{ GREY.."5) "..AtlasLocale["Cursed Centaur"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Varies"]..")", NPC, 11688 };
	};
	TheDeadminesEnt = {
		ZoneName = { AtlasLocale["The Deadmines"].." ("..AtlasLocale["Entrance"]..")", 1581 };
		Location = { AtlasLocale["Westfall"], 40 };
		Acronym = AtlasLocale["DM"];
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["The Deadmines"], ZONE, 1581 };
		{ GREY.."1) "..AtlasLocale["Marisa du'Paige"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Varies"]..")", NPC, 599 };
		{ GREY.."2) "..AtlasLocale["Brainwashed Noble"].." ("..AtlasLocale["Rare"]..")", NPC, 596 };
		{ GREY.."3) "..AtlasLocale["Foreman Thistlenettle"], NPC, 626 };
	};
	TheSunkenTempleEnt = {
		ZoneName = { AtlasLocale["Sunken Temple"].." ("..AtlasLocale["Entrance"]..")", 1417 };
		Location = { AtlasLocale["Swamp of Sorrows"], 8 };
		Acronym = AtlasLocale["ST"];
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE..INDENT..AtlasLocale["Meeting Stone"] };
		{ BLUE..INDENT..AtlasLocale["Jade"].." ("..AtlasLocale["Rare"]..")", NPC, 1063 };
		{ BLUE.."B) "..AtlasLocale["Sunken Temple"], ZONE, 1417 };
		{ GREY.."1) "..AtlasLocale["Kazkaz the Unholy"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Upper"]..")", NPC, 5401 };
		{ GREY.."2) "..AtlasLocale["Zekkis"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Lower"]..")", NPC, 5400 };
		{ GREY.."3) "..AtlasLocale["Veyzhak the Cannibal"].." ("..AtlasLocale["Rare"]..")", NPC, 5399 };
	};
	UldamanEnt = {
		ZoneName = { AtlasLocale["Uldaman"].." ("..AtlasLocale["Entrance"]..")", 1337 };
		Location = { AtlasLocale["Badlands"], 3 };
		Acronym = AtlasLocale["Ulda"];
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Uldaman"], ZONE, 1337 };
		{ GREY.."1) "..AtlasLocale["Hammertoe Grez"], NPC, 2909 };
		{ GREY.."2) "..AtlasLocale["Magregan Deepshadow"].." ("..AtlasLocale["Wanders"]..")", NPC, 2932 };
		{ GREY.."3) "..AtlasLocale["Tablet of Ryun'Eh"], ITEM, 4631 };
		{ GREY.."4) "..AtlasLocale["Krom Stoutarm's Chest"], OBJECT, 124389 };
		{ GREY.."5) "..AtlasLocale["Garrett Family Chest"], OBJECT, 124388 };
		{ GREN.."1') "..AtlasLocale["Digmaster Shovelphlange"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Varies"]..")", NPC, 7057 };
	};
	WailingCavernsEnt = {
		ZoneName = { AtlasLocale["Wailing Caverns"].." ("..AtlasLocale["Entrance"]..")", 718 };
		Location = { AtlasLocale["The Barrens"], 17 };
		Acronym = AtlasLocale["WC"];
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Wailing Caverns"], ZONE, 718 };
		{ GREY.."1) "..AtlasLocale["Mad Magglish"].." ("..AtlasLocale["Varies"]..")", NPC, 3655 };
		{ GREY.."2) "..AtlasLocale["Trigore the Lasher"].." ("..AtlasLocale["Rare"]..")", NPC, 3652 };
		{ GREY.."3) "..AtlasLocale["Boahn <Druid of the Fang>"].." ("..AtlasLocale["Rare"]..")", NPC, 3672 };
		{ "" };
		{ ORNG..AtlasLocale["Above the Entrance:"] };
		{ GREY..INDENT..AtlasLocale["Ebru <Disciple of Naralex>"], NPC, 5768 };
		{ GREY..INDENT..AtlasLocale["Nalpak <Disciple of Naralex>"], NPC, 5767 };
		{ GREY..INDENT..AtlasLocale["Kalldan Felmoon <Specialist Leatherworking Supplies>"], NPC, 5783 };
		{ GREY..INDENT..AtlasLocale["Waldor <Leatherworking Trainer>"], NPC, 5784 };
	};
	DireMaulEnt = {
		ZoneName = { AtlasLocale["Dire Maul"].." ("..AtlasLocale["Entrance"]..")", 2557 };
		Location = { AtlasLocale["Feralas"], 357 };
		Acronym = AtlasLocale["DM"];
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Dire Maul"].." ("..AtlasLocale["East"]..")", ZONE, 2557 };
		{ BLUE.."C) "..AtlasLocale["Dire Maul"].." ("..AtlasLocale["North"]..")", ZONE, 2557 };
		{ BLUE.."D) "..AtlasLocale["Dire Maul"].." ("..AtlasLocale["West"]..")", ZONE, 2557 };
		{ GREY.."1) "..AtlasLocale["Dire Pool"] };
		{ GREY.."2) "..AtlasLocale["Dire Maul Arena"] };
		{ GREY..INDENT..AtlasLocale["Mushgog"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Random"]..")", NPC, 11447 };
		{ GREY..INDENT..AtlasLocale["Skarr the Unbreakable"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Random"]..")", NPC, 11498 };
		{ GREY..INDENT..AtlasLocale["The Razza"].." ("..AtlasLocale["Rare"]..", "..AtlasLocale["Random"]..")", NPC, 11497 };
		{ GREY..INDENT..AtlasLocale["Elder Mistwalker"].." ("..AtlasLocale["Lunar Festival"]..")", NPC, 15587 };
		{ GREY.."3) "..AtlasLocale["Griniblix the Spectator"], NPC, 14395 };
	};
	SMEnt = {
		ZoneName = { AtlasLocale["Scarlet Monastery"].." ("..AtlasLocale["Entrance"]..")", 796 };
		Location = { AtlasLocale["Tirisfal Glades"], 85 };
		Acronym = AtlasLocale["SM"];
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Graveyard"], ZONE, 796 };
		{ BLUE.."C) "..AtlasLocale["Cathedral"], ZONE, 796 };
		{ BLUE.."D) "..AtlasLocale["Armory"], ZONE, 796 };
		{ BLUE.."E) "..AtlasLocale["Library"], ZONE, 796 };
	};
	Azuregos = {
		ZoneName = { AtlasLocale["Azuregos"], NPC, 6109 };
		Location = { AtlasLocale["Azshara"], ZONE, 16 };
		LevelRange = "60+";
		MinLevel = "--";
		PlayerLimit = "40";
		{ GREY.."1) "..AtlasLocale["Azuregos"], NPC, 6109 };
		{ GREY..INDENT..AtlasLocale["Spirit of Azuregos"], NPC, 15481 };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Frost"] };
	};
	FourDragons = {
		ZoneName = { AtlasLocale["Emerald Dragons"] };
		Location = { AtlasLocale["Various"] };
		LevelRange = "60+";
		MinLevel = "--";
		PlayerLimit = "40";
		{ GREY.."1) "..AtlasLocale["Duskwood"], ZONE, 10 };
		{ GREY.."2) "..AtlasLocale["The Hinterlands"], ZONE, 47 };
		{ GREY.."3) "..AtlasLocale["Feralas"], ZONE, 357 };
		{ GREY.."4) "..AtlasLocale["Ashenvale"], ZONE, 331 };
		{ "" };
		{ GREN..INDENT..AtlasLocale["Lethon"], NPC, 14888 };
		{ GREN..INDENT..AtlasLocale["Emeriss"], NPC, 14889 };
		{ GREN..INDENT..AtlasLocale["Taerar"], NPC, 14890 };
		{ GREN..INDENT..AtlasLocale["Ysondre"], NPC, 14887 };
		{ "" };
		{ GREY..INDENT..AtlasLocale["Emerald Dragons Trash"] };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Nature"]..", "..AtlasLocale["Shadow"] };
	};
	LordKazzak = {
		ZoneName = { AtlasLocale["Lord Kazzak"], NPC, 18728 };
		Location = { AtlasLocale["Blasted Lands"], ZONE, 4 };
		LevelRange = "60+";
		MinLevel = "--";
		PlayerLimit = "40";
		{ GREY.."1) "..AtlasLocale["Lord Kazzak"], NPC, 18728 };
		{ GREY.."2) "..AtlasLocale["Nethergarde Keep"] };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ ORNG..AtlasLocale["Damage: "]..AtlasLocale["Shadow"] };
	};
	AlteracValleyNorth = {
		ZoneName = { AtlasLocale["Alterac Valley"].." ("..AtlasLocale["North"]..")", 2597 };
		Acronym = AtlasLocale["AV"];
		Location = { AtlasLocale["Alterac Mountains"], 36 };
		LevelRange = "51-60";
		MinLevel = "51";
		PlayerLimit = "40";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Stormpike Guard"], FACTION, 730 };
		{ BLUE.."A) "..AtlasLocale["Entrance"] };
		{ BLUE.."B) "..AtlasLocale["Dun Baldar"] };
		{ BLUE..INDENT..AtlasLocale["Vanndar Stormpike <Stormpike General>"], NPC, 11948 };
		{ BLUE..INDENT..AtlasLocale["Dun Baldar North Marshal"], NPC, 14762 };
		{ BLUE..INDENT..AtlasLocale["Dun Baldar South Marshal"], NPC, 14763 };
		{ BLUE..INDENT..AtlasLocale["Icewing Marshal"], NPC, 14764 };
		{ BLUE..INDENT..AtlasLocale["Stonehearth Marshal"], NPC, 14765 };
		{ BLUE..INDENT..AtlasLocale["Iceblood Marshal"], NPC, 14766 };
		{ BLUE..INDENT..AtlasLocale["Tower Point Marshal"], NPC, 14767 };
		{ BLUE..INDENT..AtlasLocale["East Frostwolf Marshal"], NPC, 14768 };
		{ BLUE..INDENT..AtlasLocale["West Frostwolf Marshal"], NPC, 14769 };
		{ BLUE..INDENT..AtlasLocale["Prospector Stonehewer"], NPC, 13816 };
		{ _RED.."1) "..AtlasLocale["Irondeep Mine"] };
		{ GREY..INDENT..AtlasLocale["Morloch"].." ("..AtlasLocale["Neutral"]..")", NPC, 11657 };
		{ GREY..INDENT..AtlasLocale["Umi Thorson"].." ("..AtlasLocale["Alliance"]..")", NPC, 13078 };
		{ GREY..INDENT..AtlasLocale["Keetar"].." ("..AtlasLocale["Horde"]..")", NPC, 13079 };
		{ GREY.."2) "..AtlasLocale["Arch Druid Renferal"], NPC, 13442 };
		{ ORNG.."3) "..AtlasLocale["Dun Baldar North Bunker"] };
		{ GREY..INDENT..AtlasLocale["Wing Commander Mulverick"].." ("..AtlasLocale["Horde"]..")", NPC, 13181 };
		{ GREY.."4) "..AtlasLocale["Murgot Deepforge"], NPC, 13257 };
		{ GREY..INDENT..AtlasLocale["Dirk Swindle <Bounty Hunter>"], NPC, 14188 };
		{ GREY..INDENT..AtlasLocale["Athramanis <Bounty Hunter>"], NPC, 14187 };
		{ GREY..INDENT..AtlasLocale["Lana Thunderbrew <Blacksmithing Supplies>"], NPC, 4257 };
		{ _RED.."5) "..AtlasLocale["Stormpike Aid Station"] };
		{ GREY.."6) "..AtlasLocale["Stormpike Stable Master <Stable Master>"], NPC, 13617 };
		{ GREY..INDENT..AtlasLocale["Stormpike Ram Rider Commander"], NPC, 13577 };
		{ GREY..INDENT..AtlasLocale["Svalbrad Farmountain <Trade Goods>"], NPC, 5135 };
		{ GREY..INDENT..AtlasLocale["Kurdrum Barleybeard <Reagents & Poison Supplies>"], NPC, 5139 };
		{ GREY.."7) "..AtlasLocale["Stormpike Quartermaster"], NPC, 12096 };
		{ GREY..INDENT..AtlasLocale["Jonivera Farmountain <General Goods>"], NPC, 5134 };
		{ GREY..INDENT..AtlasLocale["Brogus Thunderbrew <Food & Drink>"], NPC, 4255 };
		{ GREY.."8) "..AtlasLocale["Wing Commander Ichman"].." ("..AtlasLocale["Rescued"]..")", NPC, 13437 };
		{ GREY..INDENT..AtlasLocale["Wing Commander Slidore"].." ("..AtlasLocale["Rescued"]..")", NPC, 13438 };
		{ GREY..INDENT..AtlasLocale["Wing Commander Vipore"].." ("..AtlasLocale["Rescued"]..")", NPC, 13439 };
		{ ORNG.."9) "..AtlasLocale["Dun Baldar South Bunker"] };
		{ GREY..INDENT..AtlasLocale["Corporal Noreg Stormpike"], NPC, 13447 };
		{ GREY..INDENT..AtlasLocale["Gaelden Hammersmith <Stormpike Supply Officer>"], NPC, 13216 };
		{ _RED.."10) "..AtlasLocale["Stormpike Graveyard"] };
		{ GREY.."11) "..AtlasLocale["Icewing Cavern"] };
		{ GREY..INDENT..AtlasLocale["Stormpike Banner"], OBJECT, 179024 };
		{ GREY.."12) "..AtlasLocale["Stormpike Lumber Yard"] };
		{ GREY..INDENT..AtlasLocale["Wing Commander Jeztor"].." ("..AtlasLocale["Horde"]..")", NPC, 13180 };
		{ ORNG.."13) "..AtlasLocale["Icewing Bunker"] };
		{ GREY..INDENT..AtlasLocale["Wing Commander Guse"].." ("..AtlasLocale["Horde"]..")", NPC, 13179 };
		{ _RED.."14) "..AtlasLocale["Stonehearth Graveyard"] };
		{ GREY.."15) "..AtlasLocale["Stormpike Ram Rider Commander"], NPC, 13577 };
		{ ORNG.."16) "..AtlasLocale["Stonehearth Outpost"] };
		{ GREY..INDENT..AtlasLocale["Captain Balinda Stonehearth <Stormpike Captain>"], NPC, 11949 };
		{ _RED.."17) "..AtlasLocale["Snowfall Graveyard"] };
		{ GREY..INDENT..AtlasLocale["Ichman's Beacon"], ITEM, 17505 };
		{ GREY..INDENT..AtlasLocale["Mulverick's Beacon"].." ("..AtlasLocale["Horde"]..")", ITEM, 17323 };
		{ GREY..INDENT..AtlasLocale["Korrak the Bloodrager"], NPC, 12159 };
		{ ORNG.."18) "..AtlasLocale["Stonehearth Bunker"] };
		{ GREY.."19) "..AtlasLocale["Ivus the Forest Lord"].." ("..AtlasLocale["Summon"]..")", NPC, 13419 };
		{ GREY.."20) "..AtlasLocale["Western Crater"] };
		{ GREY..INDENT..AtlasLocale["Vipore's Beacon"], ITEM, 17506 };
		{ GREY..INDENT..AtlasLocale["Jeztor's Beacon"].." ("..AtlasLocale["Horde"]..")", ITEM, 17325 };
		{ GREY.."21) "..AtlasLocale["Eastern Crater"] };
		{ GREY..INDENT..AtlasLocale["Slidore's Beacon"], ITEM, 17507 };
		{ GREY..INDENT..AtlasLocale["Guse's Beacon"].." ("..AtlasLocale["Horde"]..")", ITEM, 17324 };
		{ GREY.."22) "..AtlasLocale["Steamsaw"].." ("..AtlasLocale["Horde"]..")", OBJECT, 178664 };
		{ "" };
		{ GREN..INDENT..AtlasLocale["Friendly Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Honored Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Revered Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Exalted Reputation Rewards"] };
		{ "" };
		{ _RED..AtlasLocale["Red"]..": ".._RED..AtlasLocale["Graveyards, Capturable Areas"] };
		{ ORNG..AtlasLocale["Orange"]..": "..ORNG..AtlasLocale["Bunkers, Towers, Destroyable Areas"] };
		{ GREY..AtlasLocale["White"]..": "..GREY..AtlasLocale["Assault NPCs, Quest Areas"] };
	};
	AlteracValleySouth = {
		ZoneName = { AtlasLocale["Alterac Valley"].." ("..AtlasLocale["South"]..")", 2597 };
		Acronym = AtlasLocale["AV"];
		Location = { AtlasLocale["Hillsbrad Foothills"], 36 };
		LevelRange = "51-60";
		MinLevel = "51";
		PlayerLimit = "40";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Frostwolf Clan"], FACTION, 729 };
		{ BLUE.."A) "..AtlasLocale["Entrance"].." ("..AtlasLocale["Horde"]..")" };
		{ BLUE.."B) "..AtlasLocale["Frostwolf Keep"] };
		{ BLUE..INDENT..AtlasLocale["Drek'Thar <Frostwolf General>"], NPC, 11946 };
		{ BLUE..INDENT..AtlasLocale["Duros"], NPC, 12122 };
		{ BLUE..INDENT..AtlasLocale["Drakan"], NPC, 12121 };
		{ BLUE..INDENT..AtlasLocale["West Frostwolf Warmaster"], NPC, 14777 };
		{ BLUE..INDENT..AtlasLocale["East Frostwolf Warmaster"], NPC, 14772 };
		{ BLUE..INDENT..AtlasLocale["Tower Point Warmaster"], NPC, 14776 };
		{ BLUE..INDENT..AtlasLocale["Iceblood Warmaster"], NPC, 14773 };
		{ BLUE..INDENT..AtlasLocale["Stonehearth Warmaster"], NPC, 14775 };
		{ BLUE..INDENT..AtlasLocale["Icewing Warmaster"], NPC, 14774 };
		{ BLUE..INDENT..AtlasLocale["Dun Baldar North Warmaster"], NPC, 14770 };
		{ BLUE..INDENT..AtlasLocale["Dun Baldar South Warmaster"], NPC, 14771 };
		{ GREY.."1) "..AtlasLocale["Lokholar the Ice Lord"].." ("..AtlasLocale["Summon"]..")", NPC, 13256 };
		{ ORNG.."2) "..AtlasLocale["Iceblood Garrison"] };
		{ GREY..INDENT..AtlasLocale["Captain Galvangar <Frostwolf Captain>"], NPC, 11947 };
		{ ORNG.."3) "..AtlasLocale["Iceblood Tower"] };
		{ _RED.."4) "..AtlasLocale["Iceblood Graveyard"] };
		{ GREY..INDENT..AtlasLocale["Wing Commander Ichman"].." ("..AtlasLocale["Alliance"]..")", NPC, 13437 };
		{ ORNG.."5) "..AtlasLocale["Tower Point"] };
		{ GREY..INDENT..AtlasLocale["Wing Commander Slidore"].." ("..AtlasLocale["Alliance"]..")", NPC, 13438 };
		{ GREY.."6) "..AtlasLocale["Coldtooth Mine"] };
		{ GREY..INDENT..AtlasLocale["Taskmaster Snivvle"].." ("..AtlasLocale["Neutral"]..")", NPC, 11677 };
		{ GREY..INDENT..AtlasLocale["Masha Swiftcut"].." ("..AtlasLocale["Horde"]..")", NPC, 13088 };
		{ GREY..INDENT..AtlasLocale["Aggi Rumblestomp"].." ("..AtlasLocale["Alliance"]..")", NPC, 13086 };
		{ _RED.."7) "..AtlasLocale["Frostwolf Graveyard"] };
		{ GREY.."8) "..AtlasLocale["Wing Commander Vipore"].." ("..AtlasLocale["Alliance"]..")", NPC, 13439 };
		{ GREY..INDENT..AtlasLocale["Jotek"], NPC, 13798 };
		{ GREY..INDENT..AtlasLocale["Smith Regzar"], NPC, 13176 };
		{ GREY..INDENT..AtlasLocale["Primalist Thurloga"], NPC, 13236 };
		{ GREY..INDENT..AtlasLocale["Sergeant Yazra Bloodsnarl"], NPC, 13448 };
		{ GREY.."9) "..AtlasLocale["Frostwolf Stable Master <Stable Master>"], NPC, 13616 };
		{ GREY..INDENT..AtlasLocale["Frostwolf Wolf Rider Commander"], NPC, 13441 };
		{ GREY.."10) "..AtlasLocale["Frostwolf Quartermaster"], NPC, 12097 };
		{ ORNG.."11) "..AtlasLocale["West Frostwolf Tower"] };
		{ ORNG.."12) "..AtlasLocale["East Frostwolf Tower"] };
		{ GREY.."13) "..AtlasLocale["Wing Commander Guse"].." ("..AtlasLocale["Rescued"]..")", NPC, 13179 };
		{ GREY..INDENT..AtlasLocale["Wing Commander Jeztor"].." ("..AtlasLocale["Rescued"]..")", NPC, 13180 };
		{ GREY..INDENT..AtlasLocale["Wing Commander Mulverick"].." ("..AtlasLocale["Rescued"]..")", NPC, 13181 };
		{ _RED.."14) "..AtlasLocale["Frostwolf Relief Hut"] };
		{ GREY.."15) "..AtlasLocale["Wildpaw Cavern"] };
		{ GREY..INDENT..AtlasLocale["Frostwolf Banner"], OBJECT, 179025 };
		{ GREY.."16) "..AtlasLocale["Steamsaw"].." ("..AtlasLocale["Alliance"]..")", OBJECT, 178665 };
		{ "" };
		{ GREN..INDENT..AtlasLocale["Friendly Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Honored Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Revered Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Exalted Reputation Rewards"] };
		{ "" };
		{ _RED..AtlasLocale["Red"]..": ".._RED..AtlasLocale["Graveyards, Capturable Areas"] };
		{ ORNG..AtlasLocale["Orange"]..": "..ORNG..AtlasLocale["Bunkers, Towers, Destroyable Areas"] };
		{ GREY..AtlasLocale["White"]..": "..GREY..AtlasLocale["Assault NPCs, Quest Areas"] };
	};
	ArathiBasin = {
		ZoneName = { AtlasLocale["Arathi Basin"], 3358 };
		Acronym = AtlasLocale["AB"];
		Location = { AtlasLocale["Arathi Highlands"], 45 };
		LevelRange = "20-60";
		MinLevel = "20";
		PlayerLimit = "15";
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["The Defilers"].." ("..AtlasLocale["Horde"]..")", FACTION, 510 };
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["The League of Arathor"].." ("..AtlasLocale["Alliance"]..")", FACTION, 509 };
		{ BLUE.."A) "..AtlasLocale["Trollbane Hall"].." ("..AtlasLocale["Alliance"]..")" };
		{ BLUE.."B) "..AtlasLocale["Defiler's Den"].." ("..AtlasLocale["Horde"]..")" };
		{ GREY.."1) "..AtlasLocale["Stables"] };
		{ GREY.."2) "..AtlasLocale["Gold Mine"] };
		{ GREY.."3) "..AtlasLocale["Blacksmith"] };
		{ GREY.."4) "..AtlasLocale["Lumber Mill"] };
		{ GREY.."5) "..AtlasLocale["Farm"] };
		{ "" };
		{ GREN..INDENT..AtlasLocale["Friendly Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Honored Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Revered Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Exalted Reputation Rewards"] };
	};
	WarsongGulch = {
		ZoneName = { AtlasLocale["Warsong Gulch"], 3277 };
		Acronym = AtlasLocale["WSG"];
		Location = { AtlasLocale["Ashenvale"].." / "..AtlasLocale["The Barrens"], 331, 17 };
		LevelRange = "10-60";
		MinLevel = "10";
		PlayerLimit = "10";
		Continent = AtlasLocale["Kalimdor"];
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Warsong Outriders"].." ("..AtlasLocale["Horde"]..")", FACTION, 889 };
		{ ORNG..AtlasLocale["Rep"]..": "..AtlasLocale["Silverwing Sentinels"].." ("..AtlasLocale["Alliance"]..")", FACTION, 890 };
		{ BLUE.."A) "..AtlasLocale["Silverwing Hold"].." ("..AtlasLocale["Alliance"]..")" };
		{ BLUE.."B) "..AtlasLocale["Warsong Lumber Mill"].." ("..AtlasLocale["Horde"]..")" };
		{ "" };
		{ GREN..INDENT..AtlasLocale["Friendly Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Honored Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Revered Reputation Rewards"] };
		{ GREN..INDENT..AtlasLocale["Exalted Reputation Rewards"] };
	};
	DLEast = {
		ZoneName = { AtlasLocale["Dungeon Locations"].." ("..AtlasLocale["Eastern Kingdoms"]..")" };
		Location = { AtlasLocale["Eastern Kingdoms"] };
		Continent = AtlasLocale["Eastern Kingdoms"];
		{ BLUE.."A) "..AtlasLocale["Alterac Valley"]..", ".._RED..AtlasLocale["Alterac Mountains"].." / "..AtlasLocale["Hillsbrad Foothills"], ZONE, 2597, 36, 267 };
		{ BLUE.."B) "..AtlasLocale["Arathi Basin"]..", ".._RED..AtlasLocale["Arathi Highlands"], ZONE, 3358, 45 };
		{ GREY.."1) "..AtlasLocale["Scarlet Monastery"]..", ".._RED..AtlasLocale["Tirisfal Glades"], ZONE, 796, 85 };
		{ GREY.."2) "..AtlasLocale["Stratholme"]..", ".._RED..AtlasLocale["Eastern Plaguelands"], ZONE, 2017, 139 };
		{ GREY.."3) "..AtlasLocale["Naxxramas"]..", ".._RED..AtlasLocale["Eastern Plaguelands"], ZONE, 3456, 139 };
		{ GREY.."4) "..AtlasLocale["Scholomance"]..", ".._RED..AtlasLocale["Western Plaguelands"], ZONE, 2057, 28 };
		{ GREY.."5) "..AtlasLocale["Shadowfang Keep"]..", ".._RED..AtlasLocale["Silverpine Forest"], ZONE, 209, 130 };
		{ GREY.."6) "..AtlasLocale["Gnomeregan"]..", ".._RED..AtlasLocale["Dun Morogh"], ZONE, 133, 1 };
		{ GREY.."7) "..AtlasLocale["Uldaman"]..", ".._RED..AtlasLocale["Badlands"], ZONE, 1337, 3 };
		{ GREY.."8) "..AtlasLocale["Blackwing Lair"]..", ".._RED..AtlasLocale["Blackrock Spire"], ZONE, 2677, 1583 };
		{ GREY..INDENT..AtlasLocale["Blackrock Depths"]..", ".._RED..AtlasLocale["Blackrock Mountain"], ZONE, 1584, 25 };
		{ GREY..INDENT..AtlasLocale["Blackrock Spire"]..", ".._RED..AtlasLocale["Blackrock Mountain"], ZONE, 1583, 25 };
		{ GREY..INDENT..AtlasLocale["Molten Core"]..", ".._RED..AtlasLocale["Blackrock Depths"], ZONE, 2717, 1584 };
		{ GREY.."9) "..AtlasLocale["The Stockade"]..", ".._RED..AtlasLocale["Stormwind City"], ZONE, 717, 1519 };
		{ GREY.."10) "..AtlasLocale["The Deadmines"]..", ".._RED..AtlasLocale["Westfall"], ZONE, 1581, 40 };
		{ GREY.."11) "..AtlasLocale["Sunken Temple"]..", ".._RED..AtlasLocale["Swamp of Sorrows"], ZONE, 1417, 8 };
		{ GREY.."12) "..AtlasLocale["Zul'Gurub"]..", ".._RED..AtlasLocale["Stranglethorn Vale"], ZONE, 19, 33 };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ BLUE..AtlasLocale["Blue"]..": "..ORNG..AtlasLocale["Battlegrounds"] };
		{ GREY..AtlasLocale["White"]..": "..ORNG..AtlasLocale["Instances"] };
	};
	DLWest = {
		ZoneName = { AtlasLocale["Dungeon Locations"].." ("..AtlasLocale["Kalimdor"]..")" };
		Location = { AtlasLocale["Kalimdor"] };
		Continent = AtlasLocale["Kalimdor"];
		{ BLUE.."A) "..AtlasLocale["Warsong Gulch"]..", ".._RED..AtlasLocale["The Barrens"].." / "..AtlasLocale["Ashenvale"], ZONE, 3277, 17, 331 };
		{ GREY.."1) "..AtlasLocale["Blackfathom Deeps"]..", ".._RED..AtlasLocale["Ashenvale"], ZONE, 719, 331 };
		{ GREY.."2) "..AtlasLocale["Ragefire Chasm"]..", ".._RED..AtlasLocale["Orgrimmar"], ZONE, 2437, 1637 };
		{ GREY.."3) "..AtlasLocale["Wailing Caverns"]..", ".._RED..AtlasLocale["The Barrens"], ZONE, 718, 17 };
		{ GREY.."4) "..AtlasLocale["Maraudon"]..", ".._RED..AtlasLocale["Desolace"], ZONE, 2100, 405 };
		{ GREY.."5) "..AtlasLocale["Dire Maul"]..", ".._RED..AtlasLocale["Feralas"], ZONE, 2557, 357 };
		{ GREY.."6) "..AtlasLocale["Razorfen Kraul"]..", ".._RED..AtlasLocale["The Barrens"], ZONE, 491, 17 };
		{ GREY.."7) "..AtlasLocale["Razorfen Downs"]..", ".._RED..AtlasLocale["The Barrens"], ZONE, 722, 17 };
		{ GREY.."8) "..AtlasLocale["Onyxia's Lair"]..", ".._RED..AtlasLocale["Dustwallow Marsh"], ZONE, 2159, 15 };
		{ GREY.."9) "..AtlasLocale["Zul'Farrak"]..", ".._RED..AtlasLocale["Tanaris"], ZONE, 978, 440 };
		{ GREY.."10) "..AtlasLocale["Ruins of Ahn'Qiraj"]..", ".._RED..AtlasLocale["Silithus"], ZONE, 3429, 1377 };
		{ GREY..INDENT..AtlasLocale["Temple of Ahn'Qiraj"]..", ".._RED..AtlasLocale["Silithus"], ZONE, 3428, 1377 };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ "" };
		{ BLUE..AtlasLocale["Blue"]..": "..ORNG..AtlasLocale["Battlegrounds"] };
		{ GREY..AtlasLocale["White"]..": "..ORNG..AtlasLocale["Instances"] };
	};
	FPAllianceEast = {
		ZoneName = { AtlasLocale["Alliance"].." ("..AtlasLocale["Eastern Kingdoms"]..")" };
		Location = { AtlasLocale["Eastern Kingdoms"] };
		{ GREY.."1) "..AtlasLocale["Light's Hope Chapel"]..", "..BLUE..AtlasLocale["Eastern Plaguelands"], ZONE, 139 };
		{ GREY.."2) "..AtlasLocale["Chillwind Point"]..", "..BLUE..AtlasLocale["Western Plaguelands"], ZONE, 28 };
		{ GREY.."3) "..AtlasLocale["Aerie Peak"]..", "..BLUE..AtlasLocale["The Hinterlands"], ZONE, 47 };
		{ GREY.."4) "..AtlasLocale["Southshore"]..", "..BLUE..AtlasLocale["Hillsbrad Foothills"], ZONE, 267 };
		{ GREY.."5) "..AtlasLocale["Refuge Pointe"]..", "..BLUE..AtlasLocale["Arathi Highlands"], ZONE, 45 };
		{ GREY.."6) "..AtlasLocale["Menethil Harbor"]..", "..BLUE..AtlasLocale["Wetlands"], ZONE, 11 };
		{ GREY.."7) "..AtlasLocale["Ironforge"]..", "..BLUE..AtlasLocale["Dun Morogh"], ZONE, 1 };
		{ GREY.."8) "..AtlasLocale["Thelsamar"]..", "..BLUE..AtlasLocale["Loch Modan"], ZONE, 38 };
		{ GREY.."9) "..AtlasLocale["Thorium Point"]..", "..BLUE..AtlasLocale["Searing Gorge"], ZONE, 51 };
		{ GREY.."10) "..AtlasLocale["Morgan's Vigil"]..", "..BLUE..AtlasLocale["Burning Steppes"], ZONE, 46 };
		{ GREY.."11) "..AtlasLocale["Stormwind City"]..", "..BLUE..AtlasLocale["Elwynn Forest"], ZONE, 12 };
		{ GREY.."12) "..AtlasLocale["Lakeshire"]..", "..BLUE..AtlasLocale["Redridge Mountains"], ZONE, 44 };
		{ GREY.."13) "..AtlasLocale["Sentinel Hill"]..", "..BLUE..AtlasLocale["Westfall"], ZONE, 40 };
		{ GREY.."14) "..AtlasLocale["Darkshire"]..", "..BLUE..AtlasLocale["Duskwood"], ZONE, 10 };
		{ GREY.."15) "..AtlasLocale["Nethergarde Keep"]..", "..BLUE..AtlasLocale["Blasted Lands"], ZONE, 4 };
		{ GREY.."16) "..AtlasLocale["Booty Bay"]..", "..BLUE..AtlasLocale["Stranglethorn Vale"], ZONE, 33 };
	};
	FPAllianceWest = {
		ZoneName = { AtlasLocale["Alliance"].." ("..AtlasLocale["Kalimdor"]..")" };
		Location = { AtlasLocale["Kalimdor"] };
		{ GREY.."1) "..AtlasLocale["Rut'Theran Village"]..", "..BLUE..AtlasLocale["Teldrassil"], ZONE, 141 };
		{ GREY.."2) "..GREN..AtlasLocale["Nighthaven"]..GREY..", "..BLUE..AtlasLocale["Moonglade"]..GREN.." ("..AtlasLocale["Druid-only"]..")", ZONE, 493 };
		{ GREY..INDENT..AtlasLocale["South of the path along Lake Elune'ara"]..", "..BLUE..AtlasLocale["Moonglade"], ZONE, 493 };
		{ GREY.."3) "..AtlasLocale["Everlook"]..", "..BLUE..AtlasLocale["Winterspring"], ZONE, 618 };
		{ GREY.."4) "..AtlasLocale["Auberdine"]..", "..BLUE..AtlasLocale["Darkshore"], ZONE, 148 };
		{ GREY.."5) "..AtlasLocale["Talonbranch Glade"]..", "..BLUE..AtlasLocale["Felwood"], ZONE, 361 };
		{ GREY.."6) "..AtlasLocale["Stonetalon Peak"]..", "..BLUE..AtlasLocale["Stonetalon Mountains"], ZONE, 406 };
		{ GREY.."7) "..AtlasLocale["Astranaar"]..", "..BLUE..AtlasLocale["Ashenvale"], ZONE, 331 };
		{ GREY.."8) "..AtlasLocale["Talrendis Point"]..", "..BLUE..AtlasLocale["Azshara"], ZONE, 16 };
		{ GREY.."9) "..AtlasLocale["Nijel's Point"]..", "..BLUE..AtlasLocale["Desolace"], ZONE, 405 };
		{ GREY.."10) "..AtlasLocale["Ratchet"]..", "..BLUE..AtlasLocale["The Barrens"], ZONE, 17 };
		{ GREY.."11) "..AtlasLocale["Theramore Isle"]..", "..BLUE..AtlasLocale["Dustwallow Marsh"], ZONE, 15 };
		{ GREY.."12) "..AtlasLocale["Feathermoon Stronghold"]..", "..BLUE..AtlasLocale["Feralas"], ZONE, 357 };
		{ GREY.."13) "..AtlasLocale["Thalanaar"]..", "..BLUE..AtlasLocale["Feralas"], ZONE, 357 };
		{ GREY.."14) "..AtlasLocale["Marshal's Refuge"]..", "..BLUE..AtlasLocale["Un'Goro Crater"], ZONE, 490 };
		{ GREY.."15) "..AtlasLocale["Cenarion Hold"]..", "..BLUE..AtlasLocale["Silithus"], ZONE, 1377 };
		{ GREY.."16) "..AtlasLocale["Gadgetzan"]..", "..BLUE..AtlasLocale["Tanaris"], ZONE, 440 };
	};
	FPHordeEast = {
		ZoneName = { AtlasLocale["Horde"].." ("..AtlasLocale["Eastern Kingdoms"]..")" };
		Location = { AtlasLocale["Eastern Kingdoms"] };
		{ GREY.."1) "..AtlasLocale["Light's Hope Chapel"]..", ".._RED..AtlasLocale["Eastern Plaguelands"], ZONE, 139 };
		{ GREY.."2) "..AtlasLocale["Undercity"]..", ".._RED..AtlasLocale["Tirisfal Glades"], ZONE, 85 };
		{ GREY.."3) "..AtlasLocale["The Sepulcher"]..", ".._RED..AtlasLocale["Silverpine Forest"], ZONE, 130 };
		{ GREY.."4) "..AtlasLocale["Tarren Mill"]..", ".._RED..AtlasLocale["Hillsbrad Foothills"], ZONE, 267 };
		{ GREY.."5) "..AtlasLocale["Revantusk Village"]..", ".._RED..AtlasLocale["The Hinterlands"], ZONE, 47 };
		{ GREY.."6) "..AtlasLocale["Hammerfall"]..", ".._RED..AtlasLocale["Arathi Highlands"], ZONE, 45 };
		{ GREY.."7) "..AtlasLocale["Thorium Point"]..", ".._RED..AtlasLocale["Searing Gorge"], ZONE, 51 };
		{ GREY.."8) "..AtlasLocale["Kargath"]..", ".._RED..AtlasLocale["Badlands"], ZONE, 3 };
		{ GREY.."9) "..AtlasLocale["Flame Crest"]..", ".._RED..AtlasLocale["Burning Steppes"], ZONE, 46 };
		{ GREY.."10) "..AtlasLocale["Stonard"]..", ".._RED..AtlasLocale["Swamp of Sorrows"], ZONE, 8 };
		{ GREY.."11) "..AtlasLocale["Grom'Gol Base Camp"]..", ".._RED..AtlasLocale["Stranglethorn Vale"], ZONE, 33 };
		{ GREY.."12) "..AtlasLocale["Booty Bay"]..", ".._RED..AtlasLocale["Stranglethorn Vale"], ZONE, 33 };
	};
	FPHordeWest = {
		ZoneName = { AtlasLocale["Horde"].." ("..AtlasLocale["Kalimdor"]..")" };
		Location = { AtlasLocale["Kalimdor"] };
		{ GREY.."1) "..GREN..AtlasLocale["Nighthaven"]..GREY..", ".._RED..AtlasLocale["Moonglade"]..GREN.." ("..AtlasLocale["Druid-only"]..")", ZONE, 493 };
		{ GREY..INDENT..AtlasLocale["West of the path to Timbermaw Hold"]..", ".._RED..AtlasLocale["Moonglade"], ZONE, 493 };
		{ GREY.."2) "..AtlasLocale["Everlook"]..", ".._RED..AtlasLocale["Winterspring"], ZONE, 618 };
		{ GREY.."3) "..AtlasLocale["Bloodvenom Post"]..", ".._RED..AtlasLocale["Felwood"], ZONE, 361 };
		{ GREY.."4) "..AtlasLocale["Zoram'gar Outpost"]..", ".._RED..AtlasLocale["Ashenvale"], ZONE, 331 };
		{ GREY.."5) "..AtlasLocale["Valormok"]..", ".._RED..AtlasLocale["Azshara"], ZONE, 16 };
		{ GREY.."6) "..AtlasLocale["Splintertree Post"]..", ".._RED..AtlasLocale["Ashenvale"], ZONE, 331 };
		{ GREY.."7) "..AtlasLocale["Orgrimmar"]..", ".._RED..AtlasLocale["Durotar"], ZONE, 14 };
		{ GREY.."8) "..AtlasLocale["Sun Rock Retreat"]..", ".._RED..AtlasLocale["Stonetalon Mountains"], ZONE, 406 };
		{ GREY.."9) "..AtlasLocale["Crossroads"]..", ".._RED..AtlasLocale["The Barrens"], ZONE, 17 };
		{ GREY.."10) "..AtlasLocale["Ratchet"]..", ".._RED..AtlasLocale["The Barrens"], ZONE, 17 };
		{ GREY.."11) "..AtlasLocale["Shadowprey Village"]..", ".._RED..AtlasLocale["Desolace"], ZONE, 405 };
		{ GREY.."12) "..AtlasLocale["Thunder Bluff"]..", ".._RED..AtlasLocale["Mulgore"], ZONE, 215 };
		{ GREY.."13) "..AtlasLocale["Camp Taurajo"]..", ".._RED..AtlasLocale["The Barrens"], ZONE, 17 };
		{ GREY.."14) "..AtlasLocale["Brackenwall Village"]..", ".._RED..AtlasLocale["Dustwallow Marsh"], ZONE, 15 };
		{ GREY.."15) "..AtlasLocale["Camp Mojache"]..", ".._RED..AtlasLocale["Feralas"], ZONE, 357 };
		{ GREY.."16) "..AtlasLocale["Freewind Post"]..", ".._RED..AtlasLocale["Thousand Needles"], ZONE, 400 };
		{ GREY.."17) "..AtlasLocale["Marshal's Refuge"]..", ".._RED..AtlasLocale["Un'Goro Crater"], ZONE, 490 };
		{ GREY.."18) "..AtlasLocale["Cenarion Hold"]..", ".._RED..AtlasLocale["Silithus"], ZONE, 1377 };
		{ GREY.."19) "..AtlasLocale["Gadgetzan"]..", ".._RED..AtlasLocale["Tanaris"], ZONE, 440 };
	};

--************************************************
-- HORDE LEVELING GUIDE
--************************************************	
	
	TaurenLeveling1 = {
		ZoneName = { AtlasLocale["Tauren Leveling Guide"].." (1-6)" };
		Location = { AtlasLocale["Mulgore"] };
		{ WHIT.."01) talk to grull hawkwing (44,77) and accept the" };
		{ WHIT.."hunt begins then go outside the area kill" };
		{ WHIT.."plainstriders and loot 7 feathers and 7 meat. run" };
		{ WHIT.."back turn it in and accept the hunt continues and" };
		{ WHIT.."etched note." };
		{ "" };
		{ WHIT.."02) talk to your class trainer and turn in etched" };
		{ WHIT.."note then talk to chief hawkwind and accept" };
		{ WHIT.."a humble task." };
		{ "" };		
		{ WHIT.."03) go to (50,81) and turn in a humble task at" };
		{ WHIT.."greatmother hawkwind, accept the next part then" };
		{ WHIT.."click the water pouch on the fountain, then turn" }; 
		{ WHIT.."in a humble task and accept" };
		{ WHIT.."rites of the earthmother." };
		{ "" };
		{ WHIT.."04) grind to level 3 then run to camp narache and" };
		{ WHIT.."accept break sharptusk!." };
		{ "" };
		{ WHIT.."05) south of the camp to (44,88) kill 10 cougars " };
		{ WHIT.."and loot their pelts , run to (42,92) and talk to" };
		{ WHIT.."seer graytongue and turn in " };
		{ WHIT.."rite of the earthmotherand accept" };
		{ WHIT.."rite of strength." };
		{ "" };
		{ WHIT.."06) run back to camp narache to" };
		{ WHIT.."chief hawkwind turn in the hunt continues"; };
		{ WHIT.."and accept the battleboars grinding mobs along" };
		{ WHIT.."the way until you hit level 4." };
		{ "" };		
		{ WHIT.."07) run to (53,81) and kill battleboar until" };
		{ WHIT.."you have 8 flank and 8 snout." };
		{ "" };		
		{ WHIT.."08) run to the cave at (58,85) kill the" };
		{ WHIT.."quilboar for 12 belts as you make your way to " };
		{ WHIT.."the tent at (64,77) and kill sharptusk making" };
		{ WHIT.."sure to loot him for break sharptusk! then run" };
		{ WHIT.."to the littlecave at (63,82) and inside is a" };
		{ WHIT.."dirt-stained map. use it to start" };
		{ WHIT.." attack on camp narache" };
		{ "" };		
		{ WHIT.."09) hearth back to camp narache go to " };
		{ WHIT.."chief hawkwind (44,77) and turn in" };
		{ WHIT.."the battleboars, attack on camp narache and" };
		{ WHIT.."rite of strength then accept" };
		{ WHIT.."rites of the earthmother part 2." };
		{ "" };		
		{ WHIT.."10) talk to brave windfeather and turn in " };
		{ WHIT.."break sharptusk!" };
		{ "" };		
		{ WHIT.."11) go to (38,81) and speak to antur fallow and " };
		{WHIT.."accept a task unfinished then continue following" };
		{ WHIT.."the road to bloodhoof village." };
		{ "" };		
		{ WHIT.."12) talk to innkeeper kauth and turn in " };
		{ WHIT.."a task unfinished and make bloodhoof village " };
		{ WHIT.."your home. you should be level 6 now, if not" };
		{ WHIT.."grind the exp you should need until you are" };
	};
	TaurenLeveling2 = {
		ZoneName = { AtlasLocale["Tauren Leveling Guide"].." (6-10)" };
		Location = { AtlasLocale["Mulgore"] };
		{ WHIT.."01) talk to baine bloodhoof and turn in " };
		{ WHIT.."rites of the earthmother accept sharing the land," };
		{ WHIT.."rite of vision and dwarven digging." };
		{ "" };			
		{ WHIT.."02) accept poison water from " };
		{ WHIT.."mull thunderhorn(48,60). talk to zarlman47,57" };
		{ WHIT.."turn in rite of vision and accept part 2." };
		{ "" };			
		{ WHIT.."03) talk to ruul eagletalon (47,62) accept " };
		{ WHIT.."dangers of the windfury then talk to " };
		{ WHIT.."harken windtotem (48,59)and accept swoop" };
		{ WHIT.."hunting. talk to maur raincaller (47,57)" };
		{ WHIT.."accept mazzranache" };
		{ "" };			
		{ WHIT.."04) go southeast to (51,66) and kill" };
		{ WHIT.."trophy swops for 8 quills, prairie wolfs for" };
		{ WHIT.."6 paws, plainstriders for 4 talons and" };
		{ WHIT.."underneath trees loot 2 ambercorns and 2 well" };
		{ WHIT.." stones near the well(53,64)" };
		{ "" };			
		{ WHIT.."05) go to (52,70) and kill the palemane for" };
		{ WHIT.."sharing the land." };
		{ "" };			
		{ WHIT.."06) at(33,41) kill harpies for 8 windfury talons. " };
		{ WHIT.."then to (31,50) and acquire broken tools for" };
		{ WHIT.."dwarven digging." };
		{ "" };			
		{ WHIT.."07) then go to (59,62) and accept the ravaged" };
		{ WHIT.."caravan from morin cloudstalker. go to (54,48)" };
		{ WHIT.."and click on the crates to turn it in then" };
		{ WHIT.."accept the second part. run back to morin" };
		{ WHIT.."cloudstalker (59,62) and turn it in then" };
		{ WHIT.."accept the venture co. and supervisor fizsprocket." };
		{ "" };			
		{ WHIT.."08) grind to level 8 if you aren't there yet." };
		{ "" };	
		{ WHIT.."09) go back to bloodhoof village and turn in" };
		{ WHIT.."poison water, dangers of the windfury, swoop" };
		{ WHIT.."hunting then turn in rite of vision and accept" };
		{ WHIT.."the next part. accept winterhoof cleansing." };
		{ "" };	
		{ WHIT.."10) go to (53,67) and cleanse it using"};
		{ WHIT.."thunderhorn's item. then run back to mull" };
		{ WHIT.."thunderhorn (49,60) and turn in winterhoof" };
		{ WHIT.."cleansing and accept thunderhorn totem." };
		{ "" };	
		{ WHIT.."11) go to (47,57) and drink water of the seers in" };
		{WHIT.." your backpack, then follow the wolf spirit." };
		{ "" };	
		{ WHIT.."12) turn in rite of vision(33,36) when the spirit" };
		{ WHIT.." stops and accept rite of wisdom." };
		{ "" };	
		{ WHIT.."13) continue to (59,25) to lorekeeper raintotem" };
		{ WHIT.."and accept a sacred burial." };
		{ "" };	
		{ WHIT.."14) go to the ancestral spirit (60,52) turn in" };
		{ WHIT.."rite of wisdom accept journey into thunder bluff" };
		{ WHIT.."and kill 8 bristleback interlopers nearby and" };
		{ WHIT.."once done speak to lorekeeper raintotem (59,25)" };
		{ WHIT.."and turn in a sacred burial." };
		{ "" };	
		{ WHIT.."15) go to thunder bluff to the forge at (39,55)" };
		{ WHIT.."and destroy the prospector's picks here by" };
		{ WHIT.."clicking on them to destroy them." };
		{ "" };
		{ WHIT.."16) go to (69,51) and talk to cairne bloodhoof to" };
		{ WHIT.."turn in journey into thunder bluff accept ride of" };
		{ WHIT.."the earthmother. you should now be level 10, if" };
		{ WHIT.."not grind to it. its time to do your class quest." };		
	};	
	TaurenLeveling3 = {
		ZoneName = { AtlasLocale["Tauren Leveling Guide"].." (10-12)" };
		Location = { AtlasLocale["Mulgore"] };
		{ WHIT.."01) hearth to bloodhoof village. talk to skorm" };
		{ WHIT.."whitecloud (46,60) accept the hunter's way." };
		{ "" };		
		{ WHIT.."02) turn in dwarven digging and thunderhorn" };
		{ WHIT.."totem then accept thunderhorn cleansing." };
		{ "" };		
		{ WHIT.."03) go to 48,60 and speak to baine bloodhoof" };
		{ WHIT.."and turn in sharing the land." };
		{ "" };		
		{ WHIT.."04) go to the water well at (49,60) and use it to" };
		{ WHIT.."cleanse your totem in your inventory." };
		{ "" };		
		{ WHIT.."05) go to(45,16) and kill flatland prowlers until" };
		{ WHIT.."you have 4 claws." };
		{ "" };		
		{ WHIT.."06) go to (61,47) and you should see the venture" };
		{ WHIT.."co. mine. kill 14 workers and 6 supervisors." };
		{ WHIT.."take a right at the first intersection and you" };
		{ WHIT.."should see fizsprocket, then kill him." };
		{ "" };
		{ WHIT.."07) go to (59,62) and turn in the venture co. and" };
		{ WHIT.."supervisor fizsprocket." };
		{ "" };
		{ WHIT.."08) go to (49,60) and turn in thunderhorn" };
		{ WHIT.."cleansing then accept wildmane totem. go to" };
		{ WHIT.."(69,51) and turn in rites of the earthmother." };
		{ "" };
		{ WHIT.."09) go to (46,60) and turn in the hunters way." };
		{ "" };
		{ WHIT.."10) go to (38,60) in thunder bluff and you should" };
		{ WHIT.."see eyahn eagletalon. accept preparation for" };
		{ WHIT.."ceremony." };
		{ "" };
		{ WHIT.."11) go behind thunder buff and kill windfury" };
		{ WHIT.."matriarchs and windfury sorceresses until you" };
		{ WHIT.."have 6 azure and 6 bronze feathers then run back" };
		{ WHIT.."to (38,60) and turn it in." };
		{ "" };
		{ WHIT.."12) run back to bloodhoof village make sure you" };
		{ WHIT.."kill prairie alphas on the way until you have 8" };
		{ WHIT.."teeth." };
		{ "" };
		{ WHIT.."13) go to (49,60) and turn in windmane totem." };
		{ "" };
		{ WHIT.."14) you should be level 12 now, if not grind to" };
		{ WHIT.."it then run into the barrens following the path" };
		{ WHIT.."until you come to camp taurajo. talk to kirge" };
		{ WHIT.."sternhorn (44,58) and accept journey to the" };
		{ WHIT.."crossroadsand get the flight path." };
		{ "" };
		{ WHIT.."15) run north up the road until you get to the" };
		{ WHIT.."crossroads and turn in journey to the crossroads" };
		{ WHIT.."get the flight path." };
	};
	UndeadLeveling1 = {
		ZoneName = { AtlasLocale["Undead Leveling Guide"].." (1-6)" };
		Location = { AtlasLocale["Tirisfal Glades"] };
		{ WHIT.."01) run up the steps and exit the crypt." };
		{ "" };
		{ WHIT.."02) in front of you from undertaker mordo accept" };
		{ WHIT.."the quest rude awakening then run down the hill" };
		{ WHIT.."and turn it in at shadow priest sarvis." };
		{ "" };
		{ WHIT.."03) accept the mindless ones and if you are a" };
		{ WHIT.."warlock accept piercing the veil." };
		{ "" };
		{ WHIT.."04) exit the building and to your left down the" };
		{ WHIT.."hill complete the mindless ones and piercing the" };
		{ WHIT.."veil then run back and turn them in." };
		{ "" };
		{ WHIT.."05) accept rattling the rattlecages, tainted" };
		{ WHIT.."scroll, and the damned." };
		{ "" };
		{ WHIT.."06) turn in tainted scroll at your class trainer" };
		{ WHIT.."then get new spells/skills." };
		{ "" };
		{ WHIT.."07) complete the damned along with rattling the" };
		{ WHIT.."rattlecages these are found out around the" };
		{ WHIT.."village, once down turn them in and accept" };
		{ WHIT.."marla's last wish." };
		{ "" };
		{ WHIT.."08) go outside the church, accept night web's" };
		{ WHIT.."hollow and scavenging deathknell." };
		{ "" };
		{ WHIT.."09) complete scavenging deathknell which are" };
		{ WHIT.."found around and inside buildings in the village" };
		{ WHIT.."then run down to (36, 62) and kill samuel fipps" };
		{ WHIT.."for marla's last wish." };
		{ "" };
		{ WHIT.."10) run to the cave at (27,59) and do night web's" };
		{ WHIT.."hollow then run to the cemetery in the village to" };
		{ WHIT.."(31,64) and click the grave to bury the skull." };
		{ "" };
		{ WHIT.."11) run back to town and turn in all the quests" };
		{ WHIT.."which are night web's hollow, scavenging " };
		{ WHIT.."deathknell and  marla's last wish. if you are a" };
		{ WHIT.."priest accept in favor of darkness which requires" };
		{ WHIT.."level 5" };
		{ "" };
		{ WHIT.."12) accept the scarlet crusade outside the church" };
		{ WHIT.."and go do it at(35,68) then run back and turn in." };
		{ "" };
		{ WHIT.."13) accept the red messenger and do it at (36,68)" };
		{ WHIT.."killing meven korgal then run back and turn in." };
		{ "" };
		{ WHIT.."14) accept vital intelligence then get new skills" };
		{ WHIT.."if you are level 6, if not grind until level 6" }; 
		{ "" };
		{ WHIT.."15) run to the north of death knell and accept a" };
		{ WHIT.."rogue's deal then leave the starting area." };
	};
	UndeadLeveling2 = {
		ZoneName = { AtlasLocale["Undead Leveling Guide"].." (6-10)" };
		Location = { AtlasLocale["Tirisfal Glades"] };
		{ WHIT.."01) continue down the hill and talk to deathguard" };
		{ WHIT.."simmer (41,54) accept fields of grief then go to" };
		{ WHIT.."the pumpkin farm at (37,52) and kill any farmers" };
		{ WHIT.."in the way while you collect 10 pumpkins." };
		{ "" };
		{ WHIT.."02) go back to the road along (46,57) to gordo" };
		{ WHIT.."and accept gordo's task then continue along the" };
		{ WHIT.."road towards brill getting 3 gloom weed along" };
		{ WHIT.."the way then turn it in at junior apothecary" };
		{ WHIT.."holland and accept doom weed." };
		{ "" };
		{ WHIT.."03) stop in brill (59,52) and turn in fields of" };
		{ WHIT.."grief at apothecary johaan then accept the next" };
		{ WHIT.."part." };
		{ "" };
		{ WHIT.."04) turn in vital intelligence at executor zygand" };
		{ WHIT.."and accept at war with the scarlet crusade." };
		{ "" };
		{ WHIT.."05) go into the inn and turn in a rogue's deal at" };
		{ WHIT.."the innkeeper and make it your home. priests turn" };
		{ WHIT.."in in favor of darkness and accept garments of" };
		{ WHIT.."darkness then run behind the cemetery at (59,46)" };
		{ WHIT.."and use power word: fortitude on the guy then" };
		{ WHIT.."run back and turn it in for your robe." };
		{ "" };
		{ WHIT.."06) talk to deathguard dillinger (58,51) and" };
		{ WHIT.."accept a putrid task then accept wanted: maggot" };
		{ WHIT.."eye from the wanted sign at (61,52)." };
		{ "" };
		{ WHIT.."07) talk to magistrate sevren and accept" };
		{ WHIT.."graverobbers then talk to coleman farthing and" };
		{ WHIT.."accept deaths in the family and the haunted mills" };
		{ WHIT.."then talk to gretchen deadmar and accept the" };
		{ WHIT.."chill of death." };
		{ "" };
		{ WHIT.."08) go into the inn and in the basement turn in" };
		{ WHIT.."the pumpkin to finish the quest at captured" };
		{ WHIT.."zealot." };
		{ "" };
		{ WHIT.."09) go back to the bridge at (53,53) and kill the" };
		{ WHIT.."scourge for 7 putrid claws and keep an eye out" };
		{ WHIT.."for any gloom weeds." };
		{ "" };
		{ WHIT.."10) run back to the pumpkin farm at (62,52) and" };
		{ WHIT.."just behind kill 10 scarlet warriors and any" };
		{ WHIT.."duskbats for 5 pelts which you can do on your" };
		{ WHIT.."way questing as well as darkhounds for 5 vials of" };
		{ WHIT.."darkhound blood." };
		{ "" };
		{ WHIT.."11) go back to brill and speak to the trade" };
		{ WHIT.."supplies merchant and buy a coarse thread." };
		{ "" };
		{ WHIT.."12) turn in a putrid task accept the mills" };
		{ WHIT.."overrun and turn in the chill of death." };
		{ "" };
		{ WHIT.."13) you should have all the gloom weed by now" };
		{ WHIT.."turn the quest in at junior apothecary holland" };
		{ WHIT.."(58,49) and accept doom weed." };
		{ "" };
		{ WHIT.."14) turn in at war with the scarlet crusade" };
		{ WHIT.."accept the next part. turn in a new plague and" };
		{ WHIT.."accept part 3.accept proof of demise." };
		{ "" };
		{ WHIT.."15) go to (59,30) looting any doom weed along" };
		{ WHIT.."the way and whilst here kill 5 rot hide mongrels" };
		{ WHIT.."and maggot kill for his paw whilst collecting 8" };
		{ WHIT.."embalming ichors from the rot hide mongrels you" };
		{ WHIT.."kill." };
		{ "" };
		{ WHIT.."16) go to (55,42) and kill 8 graverobbers," };
		{ WHIT.."collecting any remaining doom weed and" };
		{ WHIT.."embalming ichors." };
		{ "" };
		{ WHIT.."17) go just behind where you killed maggot eye" };
		{ WHIT.."to the beach and kill murlocs until you have 5" };
		{ WHIT.."scales." };
		{ "" };
		{ WHIT.."18) go back to (57,49) to junior apothecary" };
		{ WHIT.."holland and turn in doom weed." };
		{ "" };
		{ WHIT.."19) go back into brill and turn in all quests" };
		{ WHIT.."which include a new plague, wanted: maggot eye" }; 
		{ WHIT.."and graverobbers then get all new quests which " };
		{ WHIT.."include a new plague, forsaken duties and the" };
		{ WHIT.."prodical lich." };
		{ "" };
		{ WHIT.."20) travel to the tower at (53,66) and kill 3" };
		{ WHIT.."scarlet missionaries, 3 scarlet zealots and " };
		{ WHIT.."captain parrine (51,68) make sure you loot all 10" };
		{ WHIT.."scarlet insignia rings." };
		{ "" };
		{ WHIT.."21) run back to brill and turn in at war with the" };
		{ WHIT.."scarlet crusade and accept the next part.turn in" };
		{ WHIT.."proof of demise. you should now be level 10, its" };
		{ WHIT.."time to do your class quest." };		
	};
	UndeadLeveling3 = {
		ZoneName = { AtlasLocale["Undead Leveling Guide"].." (10-12)" };
		Location = { AtlasLocale["Tirisfal Glades"] };
		{ WHIT.."01) go to agamand hills (48,42)." };
		{ "" };
		{ WHIT.."02) kill rattlecage soldiers and darkeye" };
		{ WHIT.."bonecasters looting them for notched ribs and" };
		{ WHIT.."blackened skulls." };
		{ "" };
		{ WHIT.."03) kill devlin agamand (47,42), kill nissa" };
		{ WHIT.."agamand (49,36) then kill gregor and thurman" };
		{ WHIT.."(46,32)making sure to loot all their remains." };
		{ "" };
		{ WHIT.."04) kill cracked skull soldiers until they drop a" };
		{ WHIT.."letter, then use it to start a letter undelivered." };
		{ "" };
		{ WHIT.."05) go back to brill and turn in a letter" };
		{ WHIT.."undelivered and accept speak with sevren." };
		{ "" };
		{ WHIT.."06) go to around (62,52) and turn in all these" };
		{ WHIT.."the haunted mills, deaths in the family, the" };
		{ WHIT.."mills overrunand speak with sevren." };
		{ "" };
		{ WHIT.."07) run to the undercity." };
		{ "" };
		{ WHIT.."08) go to the mage quarter and speak with bethor" };
		{ WHIT.."iceshard and turn in the prodical lich (84,17)" };
		{ WHIT.."then accept the lich's identity." };
		{ "" };
		{ WHIT.."09) you can grab any professions you want while" };
		{ WHIT.."here also such as mining or herbalism to make" };
		{ WHIT.."you some gold at early levels." };
		{ "" };
		{ WHIT.."10) exit the undercity the way you came in and" };
		{ WHIT.."go to(65,60) and turn in forsaken duties then" };
		{ WHIT.."accept return to the magistrate and rear guard" };
		{ WHIT.."patrol." };
		{ "" };
		{ WHIT.."11) go to (75,61) and kill 8 bleeding horrors and" };
		{ WHIT.."8 wandering spirits for rear guard patrol then" };
		{ WHIT.."run to (76,54) and kill 5 scarlet friars and" };
		{ WHIT.."captain vachon for at war with the scarlet" };
		{ WHIT.."crusade." };
		{ "" };
		{ WHIT.."12) travel east to (82,54) and kill spiders until" };
		{ WHIT.."you get 4 venoms." };
		{ "" };
		{ WHIT.."13) go to brightwater lake and swim to gunther's" };
		{ WHIT.."retreat, and take the book here (84,17) then go" };
		{ WHIT.."back to (65,60) and turn in rear guard patrol." };
		{ "" };
		{ WHIT.."14) enter the undercity. go to (84,17) and turn" };
		{ WHIT.."in the lich's identity accept return the book." };
		{ "" };
		{ WHIT.."15) leave the undercity and run back to brill." };
		{ "" };
		{ WHIT.."16) turn in at war with the scarlet crusade and" };
		{ WHIT.."accept the next part. turn in a new plague accept" };
		{ WHIT.."the next part and turn in return to the" };
		{ WHIT.."magistrate. go to the inn in the basement and" };
		{ WHIT.."turn in a new plague." };
		{ "" };
		{ WHIT.."17) go to the tower up the pathway at (79,26) and" };
		{ WHIT.."kill captain melrache for at war with the scarlet" };
		{ WHIT.."crusade." };
		{ "" };
		{ WHIT.."18) go back to gunther's retreat at (68,41) and" };
		{ WHIT.."turn in return the book and accept proving" };
		{ WHIT.."allegiance then take a candle next to you." };
		{ "" };
		{ WHIT.."19) at the little island just behind at (66,44)" };
		{ WHIT.."use the candle to summon lillith nefara, kill her" };
		{ WHIT.."then go back and turn the quest in then accept" };
		{ WHIT.."the prodigal lich returns." };
		{ "" };
		{ WHIT.."20) go back to brill and turn in at war with the" };
		{ WHIT.."scarlet crusade then run to the undercity and at" };
		{ WHIT.."the mage quarter (84,17) turn in the prodigal" };
		{ WHIT.."lich returns." };
		{ "" };
		{ WHIT.."21) get your training for level 12, and train" };
		{ WHIT.."weapons if you need to." };
		{ "" };
		{ WHIT.."22) get on the zeppelin to go to orgrimmar." };
		{ "" };
		{ WHIT.."23) get the orgrimmar flight path." };
		{ "" };
		{ WHIT.."24) go south to razor hill and accept conscript" };
		{ WHIT.."of the horde." };
		{ "" };
		{ WHIT.."25) head west to the barrens and turn in" };
		{ WHIT.."crossroads conscription accept it again." };
		{ "" };
		{ WHIT.."26) run to the crossroads and turn in crossroads" };
		{ WHIT.."conscription." };		
	};	
	TrollOrcLeveling1 = {
		ZoneName = { AtlasLocale["Troll/Orc Leveling Guide"].." (1-6)" };
		Location = { AtlasLocale["Durotar"] };
		{ WHIT.."01) i do every single quest in durotar! here's" };
		{ WHIT.."the fastest way to do them:" };
		{ "" };
		{ WHIT.."02) start off accepting your place in the world." };
		{ WHIT.."(right in front of you)" };
		{ "" };
		{ WHIT.."03) then turn it in (at 42.68) accept cutting" };
		{ WHIT.."teeth." };
		{ "" };
		{ WHIT.."04) go start doing: cutting teeth" };
		{ WHIT.."(keep step #5 in mind)" };
		{ "" };
		{ WHIT.."05) once you hit level 2, accept sarkoth (40.62)." };
		{ "" };
		{ WHIT.."06) go do sarkoth (at 40.66)." };
		{ "" };
		{ WHIT.."07) then turn sarkoth in (at 40.66) accept" };
		{ WHIT.."sarkoth pt.2." };
		{ "" };
		{ WHIT.."08) make sure cutting teeth is complete." };
		{ "" };
		{ WHIT.."09) turn in sarkoth pt.2 and cutting teeth(42.68)" };
		{ WHIT.."accept etched tablet and sting of the scorpid." };
		{ "" };
		{ WHIT.."10) turn in etched tablet (around 43.69) and get" };
		{ WHIT.."new spells/abilities for your class." };
		{ "" };
		{ WHIT.."11) then accept and do the following: sting of" };
		{ WHIT.."the scorpid, vile familiars," };
		{ WHIT.."galgar's cactus apple surprise and lazy peons" };
		{ WHIT.."these are done north and ne of valley of trials." };
		{ "" };
		{ WHIT.."12) turn all those quests in, then accept burning" };
		{ WHIT.."blade medallion and thazz'ril's pick." };
		{ "" };
		{ WHIT.."13) go do: burning blade medallion and" };
		{ WHIT.."thazz'rils pick  these are done in the cave 44.56." };
		{ "" };
		{ WHIT.."14) once those two quests are done use your" };
		{ WHIT.."hearthstone." };
		{ "" };
		{ WHIT.."15) turn those 2 quests in and accept report to" };
		{ WHIT.."sen'jin village." };
		{ "" };
		{ WHIT.."16) the starting area is now completed. leave" };
		{ WHIT.."starting noob zone (by heading east)." };
	};	
	TrollOrcLeveling2 = {
		ZoneName = { AtlasLocale["Troll/Orc Leveling Guide"].." (6-10)" };
		Location = { AtlasLocale["Durotar"] };
		{ WHIT.."01) accept a peon's burden (at 52.68)." };
		{ "" };
		{ WHIT.."02) accept thwarting kolkar aggression (at 54.75)." };
		{ "" };
		{ WHIT.."03) go turn in report to sen'jin village " };
		{ WHIT.."(at sen'jen village, 55.74)." };
		{ "" };
		{ WHIT.."04) then collect all the quests in sen'jin " };
		{ WHIT.."village. which include:  a solvent spirit, " };
		{ WHIT.."practical prey,minshina's skull, report to orgnil"  };
		{ WHIT.."and zalazane." };
		{ "" };
		{ WHIT.."05) do not do a solvent spirit at the water " };
		{ WHIT.."around sen'jin village. make sure the quest is " };
		{ WHIT.."accepted." };
		{ "" };
		{ WHIT.."06) then run up to razor hill(52.44), grinding " };
		{ WHIT.."mobs along the way, and make sure you get to " };
		{ WHIT.."level 6 before you get up there for new spells." };
		{ "" };
		{ WHIT.."07) turn in report to orgnil and collect all the " };
		{ WHIT.."quests at razor hill dark storms,vanquish the" };
		{ WHIT.."betrayers, encroachment break a few eggs." };
		{ "" };
		{ WHIT.."08) go up to the watch tower north-west of razor " };
		{ WHIT.."hill, at 49.40 and accept carry your weight." };
		{ "" };
		{ WHIT.."09) go in the inn (51.41) and turn in a peon's " };
		{ WHIT.."burden.  and also make razor hill your home." };
		{ "" };
		{ WHIT.."10) get first aid. (at 54.41)." };
		{ "" };
		{ WHIT.."11) then go do this: vanquish the betrayers along" };
		{ WHIT.."with carry your weight these are done at " };
		{ WHIT.."tiragarde keep, (57.55)." };
		{ "" };
		{ WHIT.."12) after killing benedict he is in the building " };
		{ WHIT.."at 59.58 and get his key, go up the steps open" };
		{ WHIT.." the chest and grab the note that starts: the " };
		{ WHIT.."admiral's orders accept the quest." };
		{ "" };
		{ WHIT.."13) go turn in vanquish the betrayers, the " };
		{ WHIT.."admiral's orders and carry your weight at razor " };
		{ WHIT.." hill (52,44) and accept from the wreckage, the " };
		{ WHIT.."admiral's orders (part2)." };
		{ "" };
		{ WHIT.."14) go do a solvent spirit and from the wreckage " };
		{ WHIT.."do the two quests at the water east of tiragarde" }; 
		{ WHIT.."keep (around 62.50)." };
		{ "" };
		{ WHIT.."15) then do the first half of encroachment(49.49)" };
		{ "" };
		{ WHIT.."16) then if you're not at least 3 bars away from " };
		{ WHIT.."lvl 8, grind till you are, go turn in" }; 
		{ WHIT.."from the wreckage at razor hill(52.44) and get " };
		{ WHIT.."new spells/abilities." };
		{ "" };
		{ WHIT.."17) run way down south and do thwarting kolkar " };
		{ WHIT.."aggression (at 48.79)." };
		{ "" };
		{ WHIT.."18) once that is done turn it in (at 54.75)." };
		{ "" };
		{ WHIT.."19) turn in a solvent spirit at sen'jin(55.74)" };
		{ "" };
		{ WHIT.."20) then go to echo isles the islands south-east " };
		{ WHIT.."of durotar and do the following 4 quests " };
		{ WHIT.."together." };
		{ "" };
		{ WHIT.."21) break a few eggs" };
		{ "" };
		{ WHIT.."22) practical prey" };
		{ "" };
		{ WHIT.."23) minshina's skull the skull is at 67.87" };
		{ "" };
		{ WHIT.."24) zalazane zalazane is at 67.86" };
		{ "" };
		{ WHIT.."25) after completing all these quests, die on " };
		{ WHIT.."purpose, so you end up right at sen'jin village" };
		{ WHIT.."(55.74)" };
		{ "" };
		{ WHIT.."26) turn in practical prey, minshina's skull and " };
		{ WHIT.."zalazane. save the quest reward item faintly" };
		{ WHIT.."glowing skull for a later quest called burning " };
		{ WHIT.."shadows." };
		{ "" };
		{ WHIT.."27) hearth to razor hill.  turn in break a few " };
		{ WHIT.."eggs" };
		{ "" };
		{ WHIT.."28) then do the second half of encroachment west " };
		{ WHIT.."of razor hill at 42.38." };
	};	
	TrollOrcLeveling3 = {
		ZoneName = { AtlasLocale["Troll/Orc Leveling Guide"].." (10-12)" };
		Location = { AtlasLocale["Durotar"] };	
		{ WHIT.."01) then go up and accept lost but not forgotten " };
		{ WHIT.."at the little hut at 43.30" };
		{ "" };		
		{ "" };
		{ WHIT.."02) then go up (grind mobs along the way) and " };
		{ WHIT.."accept winds in the desert at the goblin, 46.22." };
		{ "" };
		{ WHIT.."03) then do winds in the desert." };
		{ "" };
		{ WHIT.."04) then turn it in, accept securing the lines " };
		{ WHIT.."(at 46.22)." };
		{ "" };
		{ WHIT.."05) grind mobs to lvl 10." };
		{ "" };
		{ WHIT.."06) go down to razor hill(52.44), turn in " };
		{ WHIT.."encroachment" };
		{ "" };
		{ WHIT.."07) get new spells/abilities." };
		{ "" };
		{ WHIT.."08) do your level 10 class quests. i do this as a" };
		{ WHIT.."hunter: do all three pet quests taming the beast" };
		{ "" };
		{ WHIT.."09) dire molted bore (south of razor hill)" };
		{ "" };
		{ WHIT.."10) surf crawler (north east of razor hill at " };
		{ WHIT.."(58.30), don't go down to senjen village, it's " };
		{ WHIT.."too far away. then hearth if you can, if not run" };
		{ WHIT.."back." };
		{ "" };
		{ WHIT.."11) armored scorpid (north west of razor hill)" };
		{ "" };
		{ WHIT.."12) after completing all those, accept training " };
		{ WHIT.."the beast." };
		{ "" };
		{ WHIT.."13) go way up north-east, (at 57.16) and tame an " };
		{ WHIT.."encrusted surf crawler for your first real pet." };
		{ "" };
		{ WHIT.."14) then go way west (grinding along the way) and" };
		{ WHIT.."accept need for a cure (at 41.18)" };
		{ "" };
		{ WHIT.."15) go to orgrimmar (at 45.11)" };
		{ "" };
		{ WHIT.."16) turn in training the beast at hunter trainer" };
		{ WHIT.."(66.18)" };
		{ "" };
		{ WHIT.."17) then turn in the admiral's orders in thrall's" };
		{ WHIT.."chamber(34.36)" };
		{ "" };
		{ WHIT.."18) accept hidden enemies thrall gives it" };
		{ "" };
		{ WHIT.."19) go into cleft of shadow, accept finding the " };
		{ WHIT.."antidote (46.53)" };
		{ "" };
		{ WHIT.."20) then go do: securing the lines " };
		{ WHIT.."(at 53.23 in durotar)" };
		{ "" };
		{ WHIT.."21) turn in securing the lines (at 46.22)" };
		{ "" };
		{ WHIT.."22) then go do: finding the antidote " };
		{ WHIT.."(the scorpids are all over the place) " };
		{ WHIT.."and lost but not forgotten" };
		{ WHIT.."(all along the river west of durotar)" };
		{ "" };
		{ WHIT.."23) then go do: dark storms (at 41.26)" };
		{ "" };
		{ WHIT.."24) turn in lost but not forgotten (43.30)" };
		{ "" };
		{ WHIT.."25) turn in dark storms (at razor hill, 52.44)" }; 
		{ "" };
		{ WHIT.."26) accept margoz." };
		{ "" };
		{ WHIT.."27) turn in margoz (at 56.20) accept skull rock" };
		{ "" };
		{ WHIT.."28) go do skull rock along with hidden enemies " };
		{ WHIT.."(in the cave at 54.11)" };
		{ "" };
		{ WHIT.."29) if you can, try to kill gazz'uz, he drops " };
		{ WHIT.."eye of burning shadow item which starts burning" };
		{ WHIT.."Shadows. hint: use the quest reward item faintly " };
		{ WHIT.."glowing skull to help you kill him, he's tough "  };
		{ WHIT.."without it, also try to party up with somebody if" };
		{ WHIT.."you can."  };
		{ "" };
		{ WHIT.."30) turn in skull rock (56.20) accept neeru " };
		{ WHIT.."fireblade." };
		{ "" };
		{ WHIT.."31) go to into orgrimmar." };
		{ "" };
		{ WHIT.."32) turn in hidden enemies (33.37 in orgrimmar) " };
		{ WHIT.."accept the next part to hidden enemies." };
		{ "" };
		{ WHIT.."33) go to cleft of shadows(47.53) and turn in " };
		{ WHIT.."finding the antidote." };
		{ "" };
		{ WHIT.."34) turn in neeru fireblade and burning shadows " };
		{ WHIT.."at neeru fireblade (49.50) accept ak'zeloth." };
		{ "" };
		{ WHIT.."35) also keep talking to him to complete hidden " };
		{ WHIT.."enemies." };
		{ "" };
		{ WHIT.."36) then go back to thrall and turn in hidden " };
		{ WHIT.."enemies again. the 3rd part refers to rfc which " };
		{ WHIT.."i skip." };
		{ "" };
		{ WHIT.."37) leave orgrimmar, and turn in need for a cure" };
		{ WHIT.."(41.18) you will most likely have to accept it " };
		{ WHIT.."again to turn it in." };
		{ "" };
		{ WHIT.."38) hearth to razor hill." };
		{ "" };
		{ WHIT.."39) build up first aid." };
		{ "" };
		{ WHIT.."40) get new spells/abilities." };
		{ "" };
		{ WHIT.."41) accept conscript of the horde " };
		{ WHIT.."(at 50.43 in razor hill)" };
		{ "" };
		{ WHIT.."42) run west into the barrens."  };
	};
	HordeLeveling1 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (12-15)" };
		Location = { AtlasLocale["Barrens"] };	
		{ WHIT.."01) turn in conscript of the horde (62.19 in the " };
		{ WHIT.."barrens) accept crossroads conscription." };
		{ "" };
		{ WHIT.."02) then turn in ak'zeloth (62.20) i skip the " };
		{ WHIT.."demon seed." };
		{ "" };
		{ WHIT.."03) run to crossroads (at 52.30) (if you're a " };
		{ WHIT.."hunter do step 4)." };
		{ "" };
		{ WHIT.."04) while on my way to xrs, i  abandon my crab " };
		{ WHIT.."and tame a savannah huntress cat, for my new "};
		{ WHIT.."pet." };
		{ "" };
		{ WHIT.."05) once at xrs (at 52.30) turn in crossroads " };
		{ WHIT.."conscription then accept all quests in xrs which " };
		{ WHIT.."includes:"  };
		{ WHIT.."meats to orgrimmar, plainstrider menace, the " };
		{ WHIT.."forgotten pools, raptor thieves, wharfmaster" };
		{ WHIT.." dizzywig, fungal spores, disrupt the attacks, " };
		{ WHIT.."supplies for the crossroads and harpy raiders."  };
		{ WHIT.."turn in meats to orgrimmar accept, ride to " };
		{ WHIT.."orgrimmar." };
		{ "" };
		{ WHIT.."06) make xrs your home." };
		{ "" };
		{ WHIT.."07) get fp (flight path)." };
		{ "" };
		{ WHIT.."08) then go do:  disrupt the attacks along with " };
		{ WHIT.."plainstrider menace and raptor thieves (all " };
		{ WHIT.."around 54.26). note: if you find chen's empty keg" };
		{ WHIT.."accept it." };
		{ "" };
		{ WHIT.."09) once disrupt the attacks is completed go turn" };
		{ WHIT.."it in (at 52.30) and accept the disruption ends." };
		{ "" };
		{ WHIT.."10) go do the disruption ends and supplies for " };
		{ WHIT.."the crossroads (at 56.26)." };
		{ "" };
		{ WHIT.."11) if you find chen's empty keg accept it." };
		{ "" };
		{ WHIT.."12) once plainstrider menace, the disruption ends" };
		{ WHIT.."and supplies for the crossroads are complete" };
		{ "" };
		{ WHIT.."13) go to ratchet, get the flight path there " };
		{ WHIT.."(63.37) and accept raptor horns, samophlange,"  };
		{ WHIT.."southsea freebooters, the guns of northwatch.  " };
		{ WHIT.."turn in chen's empty keg if you have it accept " };
		{ WHIT.."the next part to it, and focus on doing this " };
		{ WHIT.."quest while throughout the barrens." };
		{ "" };
		{ WHIT.."14) grab the quest wanted: baron longshore " };
		{ WHIT.."(the wanted sign by the bank)." };
		{ "" };
		{ WHIT.."15) turn in wharfmaster dizzywig (goblin on the " };
		{ WHIT.."docks) accept miner's fortune." };
		{ "" };
		{ WHIT.."16) then go do southsea freebooters along with " };
		{ WHIT.."wanted: baron longshore along the shore just " };
		{ WHIT.."south of ratchet." };
		{ "" };
		{ WHIT.."17) once those are done, turn them in (at 62.38)," };
		{ WHIT.."accept new ones, then go do:" };
		{ "" };
		{ WHIT.."18) the missing shipment (just turn this in at " };
		{ WHIT.."the goblin at the docks) accept the missing " };
		{ WHIT.."shipment part2." };
		{ "" };
		{ WHIT.."19) run back, to gazlowe (62.38), turn in the " };
		{ WHIT.."missing shipment part2 accept stolen booty." };
		{ "" };
		{ WHIT.."20) then go do stolen booty (south of ratchet " };
		{ WHIT.."again) (telescopic lens=63.49, and shipment of " };
		{ WHIT.."boots=62.49)." };
		{ "" };
		{ WHIT.."21) once stolen booty is completed hearth to xrs." };
		{ WHIT.."(don't turn it in yet)." };
		{ "" };
		{ WHIT.."22) turn in the disruption ends and supplies for " };
		{ WHIT.."the crossroads." };
		{ "" };
		{ WHIT.."23) turn in plainstrider menace it should be done" };
		{ WHIT.."by now accept the zhevra. also accept " };
		{ WHIT.."consumed by hatred and lost in battle." };
		{ "" };
		{ WHIT.."24) at this point, i make sure i have three/four " };
		{ WHIT.."6 slot bags, if i don't, i buy them at the bag " };
		{ WHIT.."vendor." };
		{ "" };
		{ WHIT.."25) run west from xrs, go to the guy in the hut " };
		{ WHIT.."(at 45.28)." };
		{ "" };
		{ WHIT.."26) accept: kolkar leaders and centaur bracers." };
		{ "" };
		{ WHIT.."27) then go start doing:" };
		{ "" };
		{ WHIT.."28) kolkar leaders, centaur bracers, raptor " };
		{ WHIT.."thieves, the zhevra, fungal spores and the "  };
		{ WHIT.."forgotten pools all of these are done just " };
		{ WHIT.."slightly north of where you are" };
		{ "" };
		{ WHIT.."29) once kolkar leaders is done. (barak at 42.23)" };
		{ "" };
		{ WHIT.."30) go do: harpy raiders (at 38.17)." };
		{ "" };
		{ WHIT.."31) once harpy raiders is done, grind your way " };
		{ WHIT.."down into stonetalon mountains (at 35.27 in the " };
		{ WHIT.."barrens)." };		
	};		
	HordeLeveling2 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (15-16)" };
		Location = { AtlasLocale["Stonetalon Mountains"] };
		{ WHIT.."01) accept goblin invaders and avenge my village " };
		{ WHIT.."(at 35.27 in the barrens)." };
		{ "" };
		{ WHIT.."02) go do: avenge my village then turn it in." };
		{ "" };
		{ WHIT.."03) then do: kill grundig darkcloud (at 73.86)." };
		{ "" };
		{ WHIT.."04) then accept and do the escort quest there " };
		{ WHIT.."(protect kaya 73.85 in the hut), this quest"  };
		{ WHIT.."can be a little tough, kill the sorcerer first " };
		{ WHIT.."that pops out during the one battle you have to " };
		{ WHIT.."fight. if it's too hard for you, you can skip it" };
		{ "" };
		{ WHIT.."05) turn in kill grundig darkcloud and protect " };
		{ WHIT.."kaya (at 35.27 in the barrens) accept kaya's " };
		{ WHIT.."alive." };
		{ "" };
		{ WHIT.."06) then go back to the barrens." };		
	};
	HordeLeveling3 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (16-18)" };
		Location = { AtlasLocale["Barrens"] };
		{ WHIT.."01) grind your way back to the guy in the hut " };
		{ WHIT.."(at 45.28)" };
		{ "" };
		{ WHIT.."02) turn in kolkar leaders accept verog the " };
		{ WHIT.."dervish.  also turn in centaur bracers if it is " };
		{ WHIT.."done" };
		{ "" };
		{ WHIT.."03) make sure you finish up raptor thieves, the " };
		{ WHIT.."zhevra, fungal spores, and the forgotten pools " };
		{ WHIT.."before returning to the xrs." };
		{ "" };
		{ WHIT.."04) run to xrs." };
		{ "" };
		{ WHIT.."05) turn in all quests, grab all new ones." };
		{ "" };
		{ WHIT.."06) grind your way down south (to 49.50) and do " };
		{ WHIT.."lost in battle (manrik's wife)  she's laying dead"  };
		{ WHIT.."by the hut all raped and shit, west of the " };
		{ WHIT.."bridge. note: she was a slut" };
		{ "" };
		{ WHIT.."07) go down to camp taurajo and accept tribes at " };
		{ WHIT.."war (gnoll in the cage at 44.59), and get fp " };
		{ WHIT.."there." };
		{ "" };
		{ WHIT.."08) then do apothecary zamah which has you run " };
		{ WHIT.."all the way to thunder bluff (tb) (in mulgore at" };
		{ WHIT.."39.27)." };
		{ "" };
		{ WHIT.."09) once at tb, go to weapon master (40.62), get" };
		{ WHIT.."guns and staff skills. (i do this as a hunter)," };
		{ WHIT.."get what weapon skills you need for your class)" };
		{ "" };
		{ WHIT.."10) get new spells/abilities." };
		{ "" };
		{ WHIT.."11) turn in apothecary zamah the cave below the " };
		{ WHIT.."spirit rise (at 29.29)." };
		{ "" };
		{ WHIT.."12) go to first aid guy at spirit rise to build " };
		{ WHIT.."up first aid." };
		{ "" };
		{ WHIT.."13) go up to the tower in middle of town to get " };
		{ WHIT.."fp, but don't fly back to xrs." };
		{ "" };
		{ WHIT.."14) hearth back to xrs." };
		{ "" };
		{ WHIT.."15) turn in lost in battle." };
		{ "" };
		{ WHIT.."16) then go north west of xrs and do prowlers of " };
		{ WHIT.."the barrens (37.20)." };
		{ "" };
		{ WHIT.."17) then harpy lieutenants (38.14)." };
		{ "" };
		{ WHIT.."18) then grind your way east and do samophlange " };
		{ WHIT.."(52.11) complete the whole chain, which in the " };
		{ WHIT.."end has you kill the goblin for the key."  };
		{ "" };
		{ WHIT.."19) go east to sludge fen and do ignition(56.8)." };
		{ "" };
		{ WHIT.."20) then do the escape."  };
		{ "" };
		{ WHIT.."21) go north-east and do miner's fortune (61.5)." };
		{ "" };
		{ WHIT.."22) then grind your way south to ratchet (63.37)." };
		{ "" };
		{ WHIT.."23) turn in stolen booty, samophlange, the escape" };
		{ WHIT.."accept ziz fizziks  and skip wenikee boltbucket" };
		{ "" };
		{ WHIT.."24) turn in miner's fortune" };
		{ "" };
		{ WHIT.."25) then go west of ratchet and do the stagnant " };
		{ WHIT.."oasis and verog the dervish (at 54.43)" };
		{ "" };
		{ WHIT.."26) then run to xrs (52.30)." };
		{ "" };
		{ WHIT.."27) turn in prowlers of the barrens, harpy " };
		{ WHIT.."lieutenants, and the stagnant oasis accept " };
		{ WHIT.."altered beings, echeyakee, serena bloodfeather, " };
		{ WHIT.."report to kadrak, and egg hunt." };
		{ "" };
		{ WHIT.."28) go west of xrs (45.28), to turn in: centaur " };
		{ WHIT.."bracers and verog the dervish i skip hezrul " };
		{ WHIT.."bloodmark." };
		{ "" };
		{ WHIT.."29) then grind your way north and do: serena " };
		{ WHIT.."bloodfeather (at 38.11)." };
		{ "" };
		{ WHIT.."30) then go east (grind mobs along the way) and " };
		{ WHIT.."do echeyakee (coords are at 55.17)." };
		{ "" };
		{ WHIT.."31) hearth to xrs." };
		{ "" };
		{ WHIT.."32) turn in echeyakee . accept the angry " };
		{ WHIT.."scytheclaws." };
		{ "" };
		{ WHIT.."33) turn in serena bloodfeather accept letter to" };
		{ WHIT.."jin'zil. make sure that consumed by hatred is " };
		{ WHIT.."accepted." };	
	};		
	HordeLeveling4 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (18-20)" };
		Location = { AtlasLocale["Barrens"] };
		{ WHIT.."01) go down south and do:" };
		{ "" };
		{ WHIT.."02) altered beings (55.42)." };
		{ "" };
		{ WHIT.."03) raptor horns along with stolen silver(57.54)." };
		{ "" };
		{ WHIT.."04) the angry scytheclaws (51.46)." };
		{ "" };
		{ WHIT.."05) tribes at war along with consumed by hatred " };
		{ WHIT.."(at 51.54)." };
		{ "" };
		{ WHIT.."06) then grind your way to camp taurajo (45.58)." };
		{ "" };
		{ WHIT.."07) accept weapons of choice." };
		{ "" };
		{ WHIT.."08) go to gnoll in the cage." };
		{ "" };
		{ WHIT.."09) turn in tribes at war accept blood shards of " };
		{ WHIT.."agamaggan and betrayal from within." };
		{ "" };
		{ WHIT.."10) turn in blood shards of agamaggan." };
		{ "" };
		{ WHIT.."11) turn in 10 bloodshards for spirit of the " };
		{ WHIT.."wind (repeatable quest) for fast run speed." };
		{ "" };
		{ WHIT.."12) at this point, you should be about half way " };
		{ WHIT.."to level 20." };
		{ "" };
		{ WHIT.."13) now comes the fun part:  wailing caverns " };
		{ WHIT.."instance! (at 46.36).  you will need a full " };
		{ WHIT.."group to do this." };
		{ WHIT.."optional: grind all the way to level 20! (which " };
		{ WHIT.."is what i do, but it's not recommended).  i grind" };
		{ WHIT.."away at beasts and bristlebacks just north of ct."  };
		{ "" };
		{ WHIT.."14) keep an eye out for the mob named "};
		{ WHIT.."lakota'mani" };
		{ WHIT.."a big grey kodo (45.53), he drops hoof of " };
		{ WHIT.."lakota'mani which starts lakota'mani.  turn this " };
		{ WHIT.."in at ct when you get the chance." };
		{ "" };
		{ WHIT.."15) once you are at least 5 bars away from  " };
		{ WHIT.."level 20, run to xrs." };
		{ "" };
		{ WHIT.."16) turn in stolen silver, consumed by hatred, " };
		{ WHIT.."altered beings and the angry scytheclaws accept " };
		{ WHIT.."all new quests.  skip mura runetotem, hamuul " };
		{ WHIT.."runetotem. you should be lvl 20 now." };
		{ "" };
		{ WHIT.."17) fly to orgrimmar, get new spells/abilities. " };
		{ WHIT.."turn in ride to orgrimmar do the rest of this " };
		{ WHIT.."chain." };
		{ "" };
		{ WHIT.."18) grab the ashenvale hunt quest." };
		{ "" };
		{ WHIT.."19) hearth back to xrs." };
		{ "" };
		{ WHIT.."20) fly to ratchet." };
		{ "" };
		{ WHIT.."21) turn in chen's empty keg (if you did it, if " };
		{ WHIT.."not don't worry). accept the next part to it." };
		{ "" };
		{ WHIT.."22) turn in raptor horns accept deepmoss spider " };
		{ WHIT.."eggs." };
		{ "" };
		{ WHIT.."23) then go do: the guns of northwatch (60.55)." };
		{ "" };
		{ WHIT.."24) then once the guns of northwatch is done, do:" };
		{ "" };
		{ WHIT.."25) free from the hold (escort quest). not worth " };
		{ WHIT.."the time or exp(make sure you grind during)" };
		{ "" };
		{ WHIT.."26) turn in both quests at ratchet, then." };
		{ "" };
		{ WHIT.."27) hearth to xrs. (if hearth stone is still on a" };
		{ WHIT.."cooldown, fly to xrs)." };
		{ "" };
		{ WHIT.."28) run west into stonetalon mountains." };	
	};	
	HordeLeveling5 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (20-21)" };
		Location = { AtlasLocale["Stonetalon Mountains"] };
		{ WHIT.."01) run to malaka'jin (at 71.94)" };
		{ "" };
		{ WHIT.."02) accept blood feeders." };
		{ "" };
		{ WHIT.."03) turn in letter to jin'zil (74.97 in the cave)" };
		{ WHIT.."accept jin'zil's forest magic." };
		{ "" };
		{ WHIT.."04) then go accept arachnophobia (wanted poster " };
		{ WHIT.."is at 59.75)." };
		{ "" };
		{ WHIT.."05) go do: blood feeders along with deepmoss " };
		{ WHIT.."spider eggs and arachnophobia (around 54.76)." };
		{ "" };
		{ WHIT.."06) then turn in ziz fizziks  to the goblin in " };
		{ WHIT.."the hut at windshear crag, 58.62 accept super " };
		{ WHIT.."reaper 6000." };
		{ "" };
		{ WHIT.."07) then do: goblin invaders and super reaper " };
		{ WHIT.."6000 (the mobs are just north in windshear crag)" };
		{ "" };
		{ WHIT.."08) then turn in super reaper 6000 (58.62). " };
		{ WHIT.."accept further instructions." };
		{ "" };
		{ WHIT.."09) run to sun rock retreat at 46.59." };
		{ "" };
		{ WHIT.."10) turn in arachnophobia and kaya's alive." };
		{ "" };
		{ WHIT.."11) get fp there." };
		{ "" };
		{ WHIT.."12) run up the little pathway and accept " };
		{ WHIT.."boulderslide ravine and trouble in the deeps " };
		{ WHIT.."(47.64)" };
		{ "" };
		{ WHIT.."13) then go do boulderslide ravine (at 61.92)" };
		{ "" };
		{ WHIT.."14) then turn in blood feeders (71.95)" };
		{ "" };
		{ WHIT.."15) turn in goblin invaders at 35.27 in the " };
		{ WHIT.."barrens accept shredding machines (i skip the " };
		{ WHIT.."elder crone)" };
		{ "" };
		{ WHIT.."16) hearth to xrs." };
		{ "" };
		{ WHIT.."17) run north to ashenvale stopping along the " };
		{ WHIT.."way to turn in report to kadrak at 48.5 but skip" };
		{ WHIT.."the warsong reports." };
	};	
	HordeLeveling6 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (21-21)" };
		Location = { AtlasLocale["Ashenvale"] };
		{ WHIT.."01) stable pet" };
		{ "" };
		{ WHIT.."02) run to splintertree post (at 73.65)" };
		{ "" };
		{ WHIT.."03) turn in the ashenvale hunt.  then accept and " };
		{ WHIT.."turn in the ashenvale hunt again." };
		{ "" };
		{ WHIT.."04) get fp in splintertree post." };
		{ "" };
		{ WHIT.."05) run all the way to zoram strand (at 13.31), " };
		{ WHIT.."grind mobs along the way." };
		{ "" };
		{ WHIT.."06) tame a clattering crawler to get " };
		{ WHIT.."*claw rank 3*, use that pet for the following " };
		{ WHIT.."quests" };
		{ "" };
		{ WHIT.."07) get fp there. (12.33). turn in trouble in the" };
		{ WHIT.."deeps .. i skip the essence of aku'mai." };
		{ "" };
		{ WHIT.."08) do the following quests:" };
		{ "" };
		{ WHIT.."09) accept and do naga at the zoram strand then " };
		{ WHIT.."turn it in." };
		{ "" };
		{ WHIT.."10) hearth back to xrs, if hearth stone is still " };
		{ WHIT.."on a cooldown, fly there." };
		{ "" };
		{ WHIT.."11) remember to buy ammo/arrows. get cat " };
		{ WHIT.."back out" };
	};	
	HordeLeveling7 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (21-23)" };
		Location = { AtlasLocale["Southern Barrens"] };
		{ WHIT.."01) fly to ct." };
		{ "" };
		{ WHIT.."02) turn in jorn skyseer ... accept ishamuhale " };
		{ WHIT.."and melor sends word" };
		{ "" };
		{ WHIT.."03) make ct your home." };
		{ "" };
		{ WHIT.."04) i give bloodshards to the gnoll in the cage " };
		{ WHIT.."to get increased agility/spirit." };
		{ "" };
		{ WHIT.."05) run south and keep an eye out for the mob " };
		{ WHIT.."named owatanka (by 45.62), he drops owatanka's " };
		{ WHIT.."tailspike which starts owatanka." };
		{ "" };
		{ WHIT.."06) egg hunt (44.71)" };
		{ "" };
		{ WHIT.."07) chen's empty keg (if you have it)" };
		{ "" };
		{ WHIT.."08) betrayal from within and weapons of choice " };
		{ WHIT.."43.79)" };
		{ "" };
		{ WHIT.."09) accept gann's reclamation from gann " };
		{ WHIT.."stonespire.  then go do it (at 46.86). then turn" };
		{ WHIT.."it in, accept revenge of gann." };
		{ "" };
		{ WHIT.."10) hearth back to camp taurajo, turn in weapons" };
		{ WHIT.."of choice and betrayal from within. accept" };
		{ WHIT.."betrayal from within part2." };
		{ "" };
		{ WHIT.."11) fly to xr." };
		{ "" };
		{ WHIT.."12) turn in betrayal from within part2 and egg " };
		{ WHIT.."hunt." };
		{ "" };
		{ WHIT.."13) do ishamuhale (at 60.32)" };
		{ "" };
		{ WHIT.."14) turn in further instructions (at ratchet " };
		{ WHIT.."63.37). accept further instructions part2." };
		{ "" };
		{ WHIT.."15) turn in deepmoss spider eggs and chen's " };
		{ WHIT.."empty keg." };
		{ "" };
		{ WHIT.."16) fly to stonetalon mountains." };
	};	
	HordeLeveling8 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (23-25)" };
		Location = { AtlasLocale["Stonetalon Mountains"] };
		{ WHIT.."01) accept all quests at sun rock retreat which " };
		{ WHIT.."include: cenarius' legacy, cycle of rebirth and " };
		{ WHIT.."harpies threaten." };
		{ "" };
		{ WHIT.."02) make sun rock retreat your home." };
		{ "" };
		{ WHIT.."03) turn in boulderslide ravine. i skip earthen " };
		{ WHIT.."arise (accept elemental war there though)" };
		{ "" };
		{ WHIT.."04) do:" };
		{ "" };
		{ WHIT.."05) cycle of rebirth (pick up the seeds around " };
		{ WHIT.."48.41)" };
		{ "" };
		{ WHIT.."06) cenarius' legacy (done at around 35.14)" };
		{ "" };
		{ WHIT.."07) jin'zil's forest magic (the mobs are around " };
		{ WHIT.."the path at 45.27, and the area at 34.14)" };
		{ "" };
		{ WHIT.."08) go back to sun rock retreat and turn in " };
		{ WHIT.."cycle of rebirth. accept new life." };
		{ "" };
		{ WHIT.."09) turn in cenarius' legacy. accept ordanus." };
		{ "" };
		{ WHIT.."10) note: i skip getting my level 24 " };
		{ WHIT.."spells/abilities cause there's little that are " };
		{ WHIT.."useful this level" };
		{ "" };
		{ WHIT.."11) go turn in further instructions part2 (58.62)" };
		{ WHIT.."accept gerenzo wrenchwhistle." };
		{ "" };
		{ WHIT.."12) go do gerenzo wrenchwhistle (at 64.41) and " };
		{ WHIT.."shredding machines (kill xt:4 and xt:9)" };
		{ "" };
		{ WHIT.."13) then turn in gerenzo wrenchwhistle. (58.62)" };
		{ "" };
		{ WHIT.."14) run down south and turn in jin'zil's forest" };
		{ WHIT.."magic (74.97)" };
		{ "" };
		{ WHIT.."15) turn in shredding machines (in the barrens " };
		{ WHIT.."at 35.27)" };
		{ "" };
		{ WHIT.."16) hearth back to sun rock retreat." };
		{ "" };
		{ WHIT.."17) stock back up on food/water." };
		{ "" };
		{ WHIT.."18) go do harpies threaten along with elemental " };
		{ WHIT.."war and new life (all at the charred vale, 32.67)" };
		{ "" };
		{ WHIT.."19) once they are all done, make sure you are at " };
		{ WHIT.."least 5 bars away from level 25, so you may have" };
		{ WHIT.."to grind a little."  };
		{ "" };
		{ WHIT.."20) then go turn them all in, and accept calling " };
		{ WHIT.."in the reserves." };
		{ "" };
		{ WHIT.."21) then fly to ct." };
	};	
	HordeLeveling9 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (25-25)" };
		Location = { AtlasLocale["Southern Barrens"] };
		{ WHIT.."01) turn in ishamuhale. accept enraged thunder " };
		{ WHIT.."lizards." };
		{ "" };
		{ WHIT.."02) make ct your home." };
		{ "" };
		{ WHIT.."03) accept a new ore sample (need to be lvl 25 " };
		{ WHIT.."to get this)" };
		{ "" };
		{ WHIT.."04) go down and do:" };
		{ "" };
		{ WHIT.."05) enraged thunder lizards." };
		{ "" };
		{ WHIT.."06) you should find the mob washte pawne, he " };
		{ WHIT.."drops washte pawne feather which starts washte " };
		{ WHIT.."pawne." };
		{ "" };
		{ WHIT.."07) revenge of gann (at 47.85) once done, turn " };
		{ WHIT.."it in, accept the next part." };
		{ "" };
		{ WHIT.."08) do revenge of gann part 3 (at 46.85), then " };
		{ WHIT.."turn it in." };
		{ "" };
		{ WHIT.."09) head south down the path to the great lift " };
		{ WHIT.."at 44.91 in the barrens and turn in" };
		{ WHIT.."calling in the reserves." };
		{ "" };
		{ WHIT.."10) accept message to freewind post." };
		{ "" };
		{ WHIT.."11) run to freewind post. (45.50 in thousand " };
		{ WHIT.."needles)" };
	};	
	HordeLeveling10 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (25-26)" };
		Location = { AtlasLocale["Thousand Needles"] };
		{ WHIT.."01) turn in message to freewind post accept " };
		{ WHIT.."pacify the centaur." };
		{ "" };
		{ WHIT.."02) accept wanted - arnak grimtotem, alien egg " };
		{ WHIT.."and wind rider." };
		{ "" };
		{ WHIT.."03) get fp there." };
		{ "" };
		{ WHIT.."04) go do (in the following order):" };
		{ "" };
		{ WHIT.."05) pacify the centaur (mobs are just north)" };
		{ "" };
		{ WHIT.."06) test of faith (the cave at 52.43)" };
		{ "" };
		{ WHIT.."07) a new ore sample (if you can't find the drop," };
		{ WHIT.."just skip it for now)" };
		{ "" };
		{ WHIT.."08) alien egg this egg has 4 possible spawn " };
		{ WHIT.."points, either at around: 52.56 / 45.63 / " };
		{ WHIT.."41.60 / 50.56, and there might be more, just " };
		{ WHIT.."check around those areas." };
		{ "" };
		{ WHIT.."09) if you're not level 26 (or two bars away " };
		{ WHIT.."from it), grind till you are." };
		{ "" };
		{ WHIT.."10) go up to freewind post (45.50)." };
		{ "" };
		{ WHIT.."11) turn in pacify the centaur. accept grimtotem" };
		{ WHIT.."spying." };
		{ "" };
		{ WHIT.."12) turn in alien egg. accept serpent wild." };
		{ "" };
		{ WHIT.."13) hearth to camp taurajo." };
		{ "" };
		{ WHIT.."14) turn in enraged thunder lizards and washte " };
		{ WHIT.."pawne. accept cry of the thunderhawk." };
		{ "" };
		{ WHIT.."15) turn in a new ore sample." };
		{ "" };
		{ WHIT.."16) go do cry of the thunderhawk then turn it in." };
		{ WHIT.."i skip mahren skyseer." };
		{ "" };
		{ WHIT.."17) fly to thunder bluff to get new " };
		{ WHIT.."spells/abilities." };
		{ "" };
		{ WHIT.."18) turn in melor sends word(61.80 on the hunter" };
		{ WHIT.."rise). accept steelsnap." };
		{ "" };
		{ WHIT.."19) accept the sacred flame (55.51)" };
		{ "" };
		{ WHIT.."20) fly to splintertree post, ashenvale." };
	};	
	HordeLeveling11 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (26-28)" };
		Location = { AtlasLocale["Ashenvale"] };
		{ WHIT.."01) make it your home." };
		{ "" };
		{ WHIT.."02) do the following order:" };
		{ "" };
		{ WHIT.."03) accept all the quests, which include: " };
		{ WHIT.."stonetalon standstill, satyr horns, ashenvale " };
		{ WHIT.."outrunners, accept this at 71.68.  make sure the" };
		{ WHIT.."ashenvale hunt is turned in at senani " };
		{ WHIT.."thunderheart you can accept this quest right at " };
		{ WHIT.."the npc itself. note: i skip warsong supplies " };
		{ WHIT.."and the lost pages." };
		{ "" };
		{ WHIT.."04) kill the first of the three mobs related to " };
		{ WHIT.."the ashenvale hunt:" };
		{ "" };
		{ WHIT.."05) sharptalon's claw (kill sharptalon (patrols " };
		{ WHIT.."around 74.70), then he drops the item that starts" };
		{ WHIT.."this quest (hint: easy way to kill it (he's lvl " };
		{ WHIT.."31 mob), is to bring his health to a 3rd left," };
		{ WHIT.."then drag him into the guards at splintertree " };
		{ WHIT.."post, they will finish em)." };
		{ "" };
		{ WHIT.."06) do: ashenvale outrunners (around 71.72) (use" };
		{ WHIT.."track hidden, to find them better). once the " };
		{ WHIT.."quest is completed, turn it in when you get the " };
		{ WHIT.."chance." };
		{ "" };
		{ WHIT.."07) then go slightly west and do torek's assault" };
		{ WHIT.."(starts at 68.75)" };
		{ "" };
		{ WHIT.."08) then go do stonetalon standstill (the lake " };
		{ WHIT.."at 53.70) find and kill tideress he drops an item" };
		{ WHIT.."which starts: the befouled element." };
		{ "" };
		{ WHIT.."09) kill ursangous, (around 42.67) ursangous's" };
		{ WHIT.."paw (ashenvale hunt)" };
		{ "" };
		{ WHIT.."10) kill shadumbra (around 56.54) shadumbra's " };
		{ WHIT.."head (ashenvale hunt)" };
		{ "" };
		{ WHIT.."11) the sacred flame first find a phial killing " };
		{ WHIT.."the dryads at 61.52, then fill at the moonwell " };
		{ WHIT.."60.72" };
		{ "" };
		{ WHIT.."12) hearth to splintertree post to turn in " };
		{ WHIT.."stonetalon standstill, the befouled element " };
		{ WHIT.."accept je'neu of the earthen ring, torek's " };
		{ WHIT.."assault and all three of the ashenvale hunt " };
		{ WHIT.."quests, then accept and complete the hunt " };
		{ WHIT.."completed." };
		{ "" };
		{ WHIT.."13) fly to zoram strand, and turn in je'neu of " };
		{ WHIT.."the earthen ring." };
		{ "" };
		{ WHIT.."14) accept and do vorsha the lasher. once " };
		{ WHIT.."done go back and turn it in." };
		{ "" };
		{ WHIT.."15) accept and do: between a rock and a " };
		{ WHIT.."thistlefur (34.37) and troll charm (the cave is " };
		{ WHIT.."at 38.30)."   };
		{ WHIT.."note:  while you are in the cave, there is an " };
		{ WHIT.."escort quest (WHITe bear-formed druid in a cage)"  };
		{ WHIT.."called freedom to ruul, try to do this quest, you" };
		{ WHIT.."may have to fight 3-5 mobs at once several times,"  };
		{ WHIT.."if you fail then it can be skipped." };
		{ "" };
		{ WHIT.."16) once they are both completed, go turn in " };
		{ WHIT.."between a rock and a thistlefur and troll " };
		{ WHIT.."charm " };
		{ WHIT.."back at the zoram strand, then hearth back to " };
		{ WHIT.."splintertree post. turn in freedom to ruul."  };
		{ "" };
		{ WHIT.."17) if not level 27 grind till it." };
		{ "" };
		{ WHIT.."18) stable pet" };
		{ "" };
		{ WHIT.."19) tame a ghostpaw alpha (lvl 27) to get " };
		{ WHIT.."*bite rank 4*, then abandon him" };
		{ "" };
		{ WHIT.."20) tame an elder ashenvale bear (lvl 25-26) to " };
		{ WHIT.."get *claw rank 4*, then abandon him" };
		{ "" };
		{ WHIT.."21) get cat back out." };
		{ "" };
		{ WHIT.."23) go do ordanus (61.52) fight your way to " };
		{ WHIT.."him, just kill the guy, grab his head." };
		{ "" };
		{ WHIT.."24) go do: satyr horns (done at the night run," };
		{ WHIT.."67.53)(entrance to the area is around 64.42)." };
		{ "" };
		{ WHIT.."26) run back to splintertree post. turn in " };
		{ WHIT.."satyr horns." };
		{ "" };
		{ WHIT.."27) fly to stonetalon mountains." };
		{ "" };
		{ WHIT.." you may want to grind a bit here or run an " };
		{ WHIT.."instance" };
	};	
	HordeLeveling12 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (28-28)" };
		Location = { AtlasLocale["Stonetalon Mountains"] };
		{ WHIT.."fly to stonetalon mountains" };
		{ WHIT.."01) turn in ordanus. i skip the den." };
		{ "" };
		{ WHIT.."02) make sun rock retreat your home." };
		{ "" };
		{ WHIT.."03) accept and do bloodfury bloodline, go kill " };
		{ WHIT.."bloodfury ripper (at 30.63) grind mobs along the" };
		{ WHIT.."way, then hearth back." };
		{ "" };
		{ WHIT.."04) turn bloodfury bloodline in." };
		{ "" };
		{ WHIT.."05) fly to thunder bluff." };
		{ "" };
		{ WHIT.."06) make thunder bluff your home." };
		{ "" };
		{ WHIT.."07) turn in the sacred flame (54.51 in tb). " };
		{ WHIT.."accept the sacred flame part2." };
		{ "" };
		{ WHIT.."08) fly to thousand needles." };
	};	
	HordeLeveling13 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (28-30)" };
		Location = { AtlasLocale["Thousand Needles"] };
		{ WHIT.."01) turn in the sacred flame. accept the sacred" };
		{ WHIT.."flame part3." };
		{ "" };
		{ WHIT.."02) i do the following order:" };
		{ "" };
		{ WHIT.."03) the sacred flame (charge the brazier in the" };
		{ WHIT.."cave at 44.37)" };
		{ "" };
		{ WHIT.."04) go to WHITereach post 21.32, accept " };
		{ WHIT.."hypercapacitor gizmo, turn in serpent wild. " };
		{ WHIT.."accept sacred fire." };
		{ "" };
		{ WHIT.."05) go do: sacred fire (35.36)" };
		{ "" };
		{ WHIT.."06) wind rider (11.36)" };
		{ "" };
		{ WHIT.."07) homeward bound (escort starts at 17.40)(if " };
		{ WHIT.."the elite gets in the way, skip it)" };
		{ "" };
		{ WHIT.."08) steelsnap (steelsnap patrols around 16.25)" };
		{ "" };
		{ WHIT.."09) hearth to thunder bluff." };
		{ "" };
		{ WHIT.."10) get new spells/abilities." };
		{ "" };
		{ WHIT.."11) turn in steelsnap (on hunter rise). accept " };
		{ WHIT.."frostmaw." };
		{ "" };
		{ WHIT.."12) turn in sacred fire (on elder rise). accept" };
		{ WHIT.."arikara." };
		{ "" };
		{ WHIT.."13) fly back to 1k needles." };
		{ "" };
		{ WHIT.."14) make freewind post your home." };
		{ "" };
		{ WHIT.."15) turn in the sacred flame and wind rider." };
		{ "" };
		{ WHIT.."16) go to the darkcloud pinnacle the entrance" };
		{ WHIT.."ramp is at 31.36 and do the following quests:"  };
		{ WHIT.."grimtotem spying (the notes are up in the rises" };
		{ WHIT.."at 31.32, 33.39, and 39.41)" };
		{ "" };
		{ WHIT.."17) arikara (done up at 37.35)" };
		{ "" };
		{ WHIT.."18) wanted - arnak grimtotem (he is up at 38.27)" };
		{ "" };
		{ WHIT.."19) free at last (escort quest starts up at 38.27)" };
		{ "" };
		{ WHIT.."20) go to WHITereach post (21.32), turn in " };
		{ WHIT.."arikara, homeward bound ." };
		{ "" };
		{ WHIT.."22) you should be level 29, if not grind to " };
		{ WHIT.."level 29, i grind away at centuars around " };
		{ WHIT.."camp e'thok"  };
		{ "" };
		{ WHIT.."23) then do hypercapacitor gizmo (elite) kill " };
		{ WHIT.."mobs around the cage, at 22.24, free him" };
		{ "" };
		{ WHIT.."24) kill the galak messenger, he drops " };
		{ WHIT.."assassination note which starts assassination " };
		{ WHIT.."plot, turn in." };
		{ WHIT.."also turn in hypercapacitor gizmo ." };
		{ "" };
		{ WHIT.."25) then do: protect kanati greycloud." };
		{ "" };
		{ WHIT.."26) if you are not at least 4 bars from level " };
		{ "" };
		{ WHIT.."30, grind until you are. hearth back to freewind" };
		{ WHIT.."post." };
		{ "" };
		{ WHIT.."27) turn in free at last, wanted - arnak " };
		{ WHIT.."grimtotem, grimtotem spying ." };
		{ "" };
		{ WHIT.."28) note: i skip test of endurance." };
		{ "" };
		{ WHIT.."29) fly to orgrimmar." };
		{ "" };
		{ WHIT.."30) make orgrimmar your home." };
		{ "" };
		{ WHIT.."31) get new spells/abilities." };
		{ "" };
		{ WHIT.."32) you need to get on the zeppelin just right " };
		{ WHIT.."outside of orgrimmar." };
		{ "" };
		{ WHIT.."33) run to tarren mill in hillsbrad foothills."  };
	};	
	HordeLeveling14 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (30-30)" };
		Location = { AtlasLocale["Arathi Highlands"] };
		{ WHIT.."01) once at tarren mill (at 62.20) accept" };
		{ WHIT.."regthar deathgate and the hammer may fall" };
		{ "" };
		{ WHIT.."02) get flight path there." };
		{ "" };
		{ WHIT.."03) then run east into arathi highlands." };
		{ WHIT.."run to hammerfall (at 73.36)" };
		{ "" };
		{ WHIT.."04) accept hammerfall." };
		{ "" };
		{ WHIT.."05) turn in hammerfall. accept raising spirits" };
		{ "" };
		{ WHIT.."06) get fp there." };
		{ "" };
		{ WHIT.."07) then do raising spirits (just to the left of" };
		{ WHIT.."hammerfall all around 64.37. then turn it in."  };
		{ WHIT.."accept raising spirits part2." };
		{ "" };
		{ WHIT.."08) turn in raising spirits part2. accept " };
		{ WHIT.."raising spirits part3." };
		{ "" };
		{ WHIT.."09) turn in raising spirits part3. skip guile of" };
		{ WHIT.."the raptor for now." };
		{ "" };
		{ WHIT.."10) hearth to orgrimmar." };
		{ "" };
		{ WHIT.."11) get on the zeppelin to go to grom'gol." };
	};	
	HordeLeveling15 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (30-31)" };
		Location = { AtlasLocale["Stranglethorn Vale"] };
		{ WHIT.."01) get the fp at grom'gol." };
		{ "" };
		{ WHIT.."02) go up north (35.10) and start doing the stv" };
		{ WHIT.."hunt quests:" };
		{ "" };
		{ WHIT.."03) welcome to the jungle (just accept this and" };
		{ WHIT.."turn in in right there)" };
		{ "" };
		{ WHIT.."04) tiger mastery (young stranglethorn tigers) " };
		{ WHIT.."(35.10)" };
		{ "" };
		{ WHIT.."05) panther mastery (young panthers) (39.9)" };
		{ "" };
		{ WHIT.."06) panther mastery (panthers) (35.10)" };
		{ "" };
		{ WHIT.."07) tiger mastery (stranglethorn tigers) (35.10)" };
		{ "" };
		{ WHIT.."08) raptor mastery(stranglethorn raptors)(25.15)" };
		{ "" };
		{ WHIT.."09) accept tiger mastery " };
		{ WHIT.."(elder stranglethorn tigers). don't do it now." };
		{ "" };
		{ WHIT.."10) accept raptor mastery (lashtail raptors). " };
		{ WHIT.."but don't do it now." };
		{ "" };
		{ WHIT.."11) skip the rest for now." };
		{ "" };
		{ WHIT.."12) hearth to orgrimmar." };
		{ "" };
		{ WHIT.."13) fly to xrs." };
		{ "" };
		{ WHIT.."14) run west in the xrs and accept the swarm " };
		{ WHIT.."grows." };
		{ "" };
		{ WHIT.."15) then run west from the xrs to the guy in the" };
		{ WHIT.."hut (45.28) and turn in regthar deathgate." };
		{ WHIT.."accept the kolkar of desolace." };
		{ "" };
		{ WHIT.."16) run back to xrs." };
		{ "" };
		{ WHIT.."17) fly to 1k needles." };
		{ "" };
		{ WHIT.."18) go east into shimmering flats (at 77.77)."	 };
	};	
	HordeLeveling16 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (31-32)" };
		Location = { AtlasLocale["Shimmering Flats"] };
		{ WHIT.."01) accept:" };
		{ "" };
		{ WHIT.."02) hemet nesingwary" };
		{ "" };
		{ WHIT.."03) wharfmaster dizzywig" };
		{ "" };
		{ WHIT.."04) accept and do the following 5 quests " };
		{ WHIT.."together:" };
		{ "" };
		{ WHIT.."05) a bump in the road" };
		{ "" };
		{ WHIT.."06) hardened shells" };
		{ "" };
		{ WHIT.."07) load lightening" };
		{ "" };
		{ WHIT.."08) rocket car parts" };
		{ "" };
		{ WHIT.."09) salt flat venom" };
		{ "" };
		{ WHIT.."10) turn them all in." };
		{ "" };
		{ WHIT.."11) accept:" };
		{ "" };
		{ WHIT.."12) goblin sponsorship" };
		{ "" };
		{ WHIT.."13) martek the exiled" };
		{ "" };
		{ WHIT.."14) encrusted tail fins(only if you " };
		{ WHIT.."have a good group)" };
		{ "" };
		{ WHIT.."15) you may or may not be level 32 right now, " };
		{ WHIT.."if not that's okay." };
		{ "" };
		{ WHIT.."16) go south into tanaris to get fp in gadgetzan" };
		{ WHIT.."at 51.25" };
		{ "" };
		{ WHIT.."17) hearth to orgrimmar." };
		{ "" };
		{ WHIT.."18) turn in the swarm grows (at 75.34). accept " };
		{ WHIT.."the swarm grows part2." };
		{ "" };
		{ WHIT.."19) accept alliance relations (get it from " };
		{ WHIT.."craven drok in the cleft of shadow, 50.47)" };
		{ "" };
		{ WHIT.."20) then go to keldran in orgrimmar (22.53) to " };
		{ WHIT.."accept alliance relations part2." };
		{ "" };
		{ WHIT.."21) stop at first aid guy to buy silk bandage" };
		{ WHIT.."training.  note:  make sure you save all your" };
		{ WHIT.."extra cloths in either your bank or mailbox, you" };
		{ WHIT.."will need them later for cloth donation quests." };
		{ "" };
		{ WHIT.."22) then fly to stonetalon mountains." };
		{ "" };
		{ WHIT.."23) run into desolace."		 };
	};	
	HordeLeveling17 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (32-33)" };
		Location = { AtlasLocale["Desolace"] };
		{ WHIT.."01) first thing to do here is start killing mobs" };
		{ WHIT.."at the thunder axe fortress (55.24)." };
		{ "" };
		{ WHIT.."02) until this item drops: flayed demon skin " };
		{ WHIT.."which starts: the corrupter" };
		{ "" };
		{ WHIT.."03) then go down the path and do:" };
		{ "" };
		{ WHIT.."04) kodo roundup (starts at 60.61) (don't have " };
		{ WHIT.."to finish all of it now)" };
		{ "" };
		{ WHIT.."05) then go to ghost walker post (56.59)" };
		{ "" };
		{ WHIT.."06) turn in the kolkar of desolace. accept khan " };
		{ WHIT.."dez'hepah." };
		{ "" };
		{ WHIT.."07) accept gelkis alliance (this is the one you " };
		{ WHIT.."should choose to do).  skip magram alliance." };
		{ "" };
		{ WHIT.."08) turn in alliance relations. accept alliance " };
		{ WHIT.."relations part2. accept befouled by satyr." };
		{ "" };
		{ WHIT.."09) turn in the corrupter. accept the corrupter " };
		{ WHIT.."part2." };
		{ "" };
		{ WHIT.."10) turn in alliance relations part2. accept " };
		{ WHIT.."the burning of spirits." };
		{ "" };
		{ WHIT.."11) accept catch of the day." };
		{ WHIT.."do in the following order:" };
		{ "" };
		{ WHIT.."12) befouled by satyr (75.22)  (keep step #13 " };
		{ WHIT.."in mind)" };
		{ "" };
		{ WHIT.."13) the corrupter part2 (collect a shadowstalker" };
		{ WHIT.."scalp from a hatefury shadowstalker)" };
		{ "" };
		{ WHIT.."14) khan dez'hepah (73.48)" };
		{ "" };
		{ WHIT.."15) gelkis alliance (68.71)" };
		{ "" };
		{ WHIT.."16) go back to ghost walker post (56.59), and " };
		{ WHIT.."turn in khan dez'hepah. accept centaur bounty." };
		{ WHIT.."turn in befouled by satyr. turn in the corrupter " };
		{ WHIT.."part2. accept the corrupter part3." };
		{ "" };
		{ WHIT.."17) then run to shadowprey village (stopping " };
		{ WHIT.."along the way to turn this in: gelkis alliance " };
		{ WHIT.."at 36.79) accept stealing supplies." };
		{ "" };
		{ WHIT.."18) accept all quests at shadowprey village " };
		{ WHIT.."(at 24.71), which include:  hunting in " };
		{ WHIT.."stranglethorn. hand of iruxos, clam bait, and " };
		{ WHIT.."other fish to fry." };
		{ "" };
		{ WHIT.."19) make shadowprey village your home." };
		{ WHIT.."then do:" };
		{ "" };
		{ WHIT.."21) go in the water and collect 10 shellfish " };
		{ WHIT.."(from the shellfish traps)" };
		{ "" };
		{ WHIT.."22) turn those in for 2 bloodbelly fish." };
		{ "" };
		{ WHIT.."23) travel up the water (collecting soft-shelled" };
		{ WHIT.."clam meat for the quest clam bait along the way)" };
		{ "" };
		{ WHIT.."24) then accept claim rackmore's treasure! " };
		{ WHIT.."(the chest/wrecked boat along the shore, 36.30)"   };
		{ WHIT.."(the silver key is dropped by a drysnap, and the" };
		{ WHIT.."golden key is dropped by a slitherblade)" };
		{ "" };
		{ WHIT.."25) go accept sceptre of light (the argent dawn " };
		{ WHIT.."dude, at 38.27)(you might have a full questlog)" };
		{ "" };
		{ WHIT.."26) then go do following at thunder axe fortress" };
		{ WHIT.."(54.29):" };
		{ "" };
		{ WHIT.."27) the burning of spirits" };
		{ "" };
		{ WHIT.."28) sceptre of light" };
		{ "" };
		{ WHIT.."29) hand of iruxos"  };
		{ "" };
		{ WHIT.."30) then grind your way back to the argent dawn" };
		{ WHIT.."dude (38.27)." };
		{ "" };
		{ WHIT.."31) then turn sceptre of light in and get book " };
		{ WHIT.."of the ancients quest." };
		{ "" };
		{ WHIT.."32) then go do all this stuff in the water to " };
		{ WHIT.."the west:" };
		{ "" };
		{ WHIT.."33) other fish to fry" };
		{ "" };
		{ WHIT.."34) clam bait" };
		{ "" };
		{ WHIT.."35) book of the ancients (27.6)" };
		{ "" };
		{ WHIT.."36) the corrupter part3 (collect a oracle " };
		{ WHIT.."crystal from a slitherblade oracle)" };
		{ "" };
		{ WHIT.."37) claim rackmore's treasure!" };
		{ "" };
		{ WHIT.."38) then turn in claim rackmore's treasure! at " };
		{ WHIT.."the little chest (29.8)" };
		{ "" };
		{ WHIT.."39) turn in book of the ancients (38.27)" };
		{ "" };
		{ WHIT.."40) then go accept bone collector (62.38) " };
		{ WHIT.."(grinding mobs along the way)" };
		{ "" };
		{ WHIT.."41) go to ghost walker post (56.59), accept and " };
		{ WHIT.."turn in the burning of spirits, the corrupter " };
		{ WHIT.."part3. accept and then turn in the corrupter " };
		{ WHIT.."part4. accept alliance relations. skip " };
		{ WHIT.."the corrupter part5." };
		{ "" };
		{ WHIT.."43) then go do:" };
		{ "" };
		{ WHIT.."44) bone collector (done at the kodo graveyard," };
		{ WHIT.."(51.58)" };
		{ "" };
		{ WHIT.."45) then centaur bounty and stealing supplies " };
		{ WHIT.."(70.74)" };
		{ "" };
		{ WHIT.."46) then go turn in centaur bounty (56.59)" };
		{ "" };
		{ WHIT.."47) then turn in bone collector (62.38)" };
		{ "" };
		{ WHIT.."48) hearth to shadowprey village. " };
		{ WHIT.."hunters - stable pet" };
		{ "" };
		{ WHIT.."50) turn in all quests there, which are: " };
		{ WHIT.."hand of iruxos, other fish to fry and clam bait" };
		{ "" };
		{ WHIT.."51) you should be level 34 now for sure." };
		{ "" };
		{ WHIT.."52) turn in stealing supplies (36.79) . " };
		{ WHIT.."i skip ongeku" };
		{ "" };
		{ WHIT.."53) go tame a scorpashi lasher (lvl 33) to get " };
		{ WHIT.."*claw rank 5*"		 };
	};	
	HordeLeveling18 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (33-34)" };
		Location = { AtlasLocale["Dust Wallowmarsh"] };
		{ WHIT.."54) fly to ct (in the barrens). (the flight " };
		{ WHIT.."master is on the docks in shadowprey village)" };
		{ "" };
		{ WHIT.."55) once at ct, run south-east into dust " };
		{ WHIT.."wallowmarsh (51.79 in the barrens)." };
		{ "" };
		{ WHIT.."56) collect the 3 quest-objects at the shady " };
		{ WHIT.."rest inn" };
		{ "" };
		{ WHIT.."57) suspicious hoofprints (just right outside " };
		{ WHIT.."in front of the inn)" };
		{ "" };
		{ WHIT.."58) lieutenant paval reethe (laying on one of " };
		{ WHIT.."the planks on the ground)" };
		{ "" };
		{ WHIT.."59) the black shield (on top of the fireplace)" };
		{ "" };
		{ WHIT.."60) now run to brackenwall village (35.29)" };
		{ "" };
		{ WHIT.."61) turn those 3 quests in (suspicious " };
		{ WHIT.."hoofprints, lieutenant paval reethe and the " };
		{ WHIT.."black shield). accept and then turn in the black" };
		{ WHIT.."shield part2. accept the black shield part3." };
		{ "" };
		{ WHIT.."62) stop at the troll vendor, buy the 3 first " };
		{ WHIT.."aid books." };
		{ "" };
		{ WHIT.."63) fly to ratchet to turn in goblin sponsorship" };
		{ WHIT.."& wharfmaster dizzywig. accept " };
		{ WHIT.."goblin sponsorship part2 and parts for kravel." };
		{ "" };
		{ WHIT.."64) get on the boat to go to bb (booty bay). " };
		{ WHIT.."(while waiting for the boat, build up first aid)"	 };
		{ "" };
		{ WHIT.."65) at this point you might want to run scarlet " };
		{ WHIT.."monastery to catch up if you've fallen behind." };
	};	
	HordeLeveling19 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (34-36)" };
		Location = { AtlasLocale["Stranglethorn Vale"] };
		{ WHIT.."01) turn in goblin sponsorship part2. accept " };
		{ WHIT.."goblin sponsorship part3.  make bb your home!  " };
		{ WHIT.."accept singing blue shards, bloodscalp ears, " };
		{ WHIT.."hostile takeover and investigate the camp.  " };
		{ WHIT.."turn in goblin sponsorship part3 at baron " };
		{ WHIT.."revilgaz. accept goblin sponsorship part4." };
		{ "" };
		{ WHIT.."02) cat should still be stabled, you should " };
		{ WHIT.."still have the scorpashi lasher as a pet." };
		{ "" };
		{ WHIT.."03) then do the following:" };
		{ "" };
		{ WHIT.."04) fly to grom'gol. your questlog might get " };
		{ WHIT.."overfilled here, run some then pick up the rest." };
		{ "" };
		{ WHIT.."05) grab all quests in grom'gol: the defense of " };
		{ WHIT.."grom'gol, mok'thardin's enchantment," };
		{ WHIT.."hunt for yenniku, trollbane(get this later), " };
		{ WHIT.."bloody bone necklaces, the vile reef." };
		{ "" };
		{ WHIT.."06) get new hunter spells/abilities." };
		{ "" };
		{ WHIT.."07) go tame a stranglethorn tiger to get " };
		{ WHIT.."*dash rank 1*" };
		{ "" };
		{ WHIT.."08) then, this is the main pet i use for the " };
		{ WHIT.."rest of the game." };
		{ "" };
		{ WHIT.."09) important: make sure you save any green " };
		{ WHIT.."hills of stranglethorn pages you find and mail " };
		{ WHIT.."them to your alt for storage, you will need them " };
		{ WHIT.."for a later quest. you will need one of each of " };
		{ WHIT.."the following pages: 1, 4, 6, 8, 10, 11, 14, 16," };
		{ WHIT.."18, 20, 21, 24, 25, 26, and 27.  once your alt " };
		{ WHIT.."collects all the pages, mail them back to your " };
		{ WHIT.."main. now go complete these quests:" };
		{ "" };
		{ WHIT.."10) singing blue shards (25.19)" };
		{ "" };
		{ WHIT.."11) tiger mastery (elder stranglethorn tigers) " };
		{ WHIT.."(31.19)" };
		{ "" };
		{ WHIT.."12) bloodscalp ears" };
		{ "" };
		{ WHIT.."13) hunt for yenniku" };
		{ "" };
		{ WHIT.."14) bloody bone necklaces (you don't have to " };
		{ WHIT.."finish all of this now)" };
		{ "" };
		{ WHIT.."15) raptor mastery (lashtail raptors)" };
		{ "" };
		{ WHIT.."16) the defense of grom'gol" };
		{ "" };
		{ WHIT.."17) once all those are done (besides bloody bone" };
		{ WHIT.."necklaces), go to grom'gol." };
		{ "" };
		{ WHIT.."18) turn in hunt for yenniku. accept headhunting" };
		{ "" };
		{ WHIT.."19) turn in the defense of grom'gol. accept the " };
		{ WHIT.."defense of grom'gol part2" };
		{ "" };
		{ WHIT.."20) should be lvl 35 now, either way buy lvl 35 " };
		{ WHIT.."food/water/repair (so that you are prepared for " };
		{ WHIT.."when you do turn level 35), then go do in the " };
		{ WHIT.."following order:" };
		{ "" };
		{ WHIT.."21) headhunting along with finishing up bloody " };
		{ WHIT.."bone necklaces (21.14)" };
		{ "" };
		{ WHIT.."23) the vile reef along with encrusted tail fins" };
		{ WHIT.."(24.24) use a breath potion if you can." };
		{ WHIT.."note: you will need help with both of those." };
		{ "" };
		{ WHIT.."24) then go to nesingwary's expedition (35.10). " };
		{ WHIT.."turn in all quests, accept all new ones (except " };
		{ WHIT.."the green hills of stranglethorn).  then go do:" };
		{ "" };
		{ WHIT.."25) tiger mastery (sin'dall)  (he is usually on " };
		{ WHIT.."top of the hill at 31.17).  once that is done, " };
		{ WHIT.."go turn it in, then go do:" };
		{ "" };
		{ WHIT.."26) hostile takeover along with goblin " };
		{ WHIT.."sponsorship (44.19)" };
		{ "" };
		{ WHIT.."27) panther mastery along with mok'thardin's " };
		{ WHIT.."enchantment (kill shadowmaw panthers) (48.21)" };
		{ "" };
		{ WHIT.."28) the defense of grom'gol part2 (36.30), once " };
		{ WHIT.."that's done." };
		{ "" };
		{ WHIT.."29) head up north and turn in panther mastery. " };
		{ WHIT.."accept panther mastery (bhag'thera) but don't do" };
		{ WHIT.."it now." };
		{ "" };
		{ WHIT.."30) hearth to bb, turn in singing blue shards, " };
		{ WHIT.."hostile takeover, bloodscalp ears, investigate " };
		{ WHIT.."the camp" };
		{ "" };
		{ WHIT.."31) turn in goblin sponsorship part4. accept " };
		{ WHIT.."goblin sponsorship part5." };
		{ "" };
		{ WHIT.."32) fly to grom'gol, turn in all quests: the " };
		{ WHIT.."defense of grom'gol," };
		{ WHIT.."mok'thardin's enchantment, headhunting, bloody " };
		{ WHIT.."bone necklaces and the vile reef." };
		{ "" };
		{ WHIT.."33) should be level 36 now, if not, grind to it." };
		{ WHIT.."then get new hunter spells/abilities." };
		{ "" };
		{ WHIT.."34) get on the zeppelin to go to the undercity." };
		{ "" };
		{ WHIT.."35) once in the uc, turn in 60 silk cloth for a " };
		{ WHIT.."donation of silk quest (at 71.28)." };
		{ "" };
		{ WHIT.."36) accept to steal from thieves (63.49), crown " };
		{ WHIT.."of will. (throne room)" };
		{ "" };
		{ WHIT.."37) then fly to hammerfall."		 };
	};	
	HordeLeveling20 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (36-37)" };
		Location = { AtlasLocale["Arathi Highlands"] };
		{ WHIT.."01) make hammerfall your home." };
		{ "" };
		{ WHIT.."02) turn in trollbane. i skip sigil of strom." };
		{ "" };
		{ WHIT.."03) accept foul magics and guile of the raptor." };
		{ "" };
		{ WHIT.."04) then go west of hammerfall and accept the " };
		{ WHIT.."princess trapped (62.33)." };
		{ "" };
		{ WHIT.."05) then go do the princess trapped (mobs are " };
		{ WHIT.."east of hammerfall)." };
		{ "" };
		{ WHIT.."06) go in the cave (look for the tree, shows " };
		{ WHIT.."you where the hidden path is)." };
		{ "" };
		{ WHIT.."07) then turn in the princess trapped (in the " };
		{ WHIT.."cave). accept stones of binding." };
		{ "" };
		{ WHIT.."08) build up first aid, go do triage (doctor " };
		{ WHIT.."gregory victor, first aid training in hammerfall)." };
		{ "" };
		{ WHIT.."09) then do:" };
		{ "" };
		{ WHIT.."10) stones of binding (first key, just west of " };
		{ WHIT.."hammerfall 66.29)" };
		{ "" };
		{ WHIT.."11) then do: to steal from thieves (at 54.40)." };
		{ "" };
		{ WHIT.."12) go down south a little and get the next key " };
		{ WHIT.."for stones of binding (52.50)" };
		{ "" };
		{ WHIT.."13) then go down and do:" };
		{ "" };
		{ WHIT.."14) guile of the raptor (killing highland " };
		{ WHIT.."fleshstalkers, around 50.75)." };
		{ "" };
		{ WHIT.."15) then go up and do the " };
		{ WHIT.."hammer may fall(34.45)." };
		{ "" };
		{ WHIT.."16) then go up and do foul magics (at 31.28)." };
		{ "" };
		{ WHIT.."17) then go west and get the last key for stones " };
		{ WHIT.."of binding (25.31)." };
		{ "" };
		{ WHIT.."18) go discover stromguard, and turn in stones " };
		{ WHIT.."of binding (the circle of inner binding(36.57)" };
		{ "" };
		{ WHIT.."19) note: i skip breaking the keystone (elite)" };
		{ "" };
		{ WHIT.."20) hearth to hammerfall." };
		{ "" };
		{ WHIT.."21) turn in foul magics, guile of the raptor and " };
		{ WHIT.."the hammer may fall. skip call to arms" };
		{ "" };
		{ WHIT.."22) complete the guile of the raptor quest chain " };
		{ WHIT.."by running back and forth." };
		{ "" };
		{ WHIT.."23) note: i skip all stromguard quests (i still " };
		{ WHIT.."recommend doing them if you can find group)" };
		{ "" };
		{ WHIT.."24) fly to tarren mill."		 };
	};	
	HordeLeveling21 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (37-37)" };
		Location = { AtlasLocale["Alterac Mountains"] };
		{ WHIT.."01) once at tm, accept stone tokens, prison " };
		{ WHIT.."break in. turn in crown of will , accept crown " };
		{ WHIT.."of will (part 2)" };
		{ "" };
		{ WHIT.."02) frostmaw quest (37.69 in alterac mountains)" };
		{ "" };
		{ WHIT.."03) then go do: crown of will (part 2) (at ruins " };
		{ WHIT.."of alterac, around 40.50)" };
		{ "" };
		{ WHIT.."04) then go do stone tokens along with prison " };
		{ WHIT.."break in in alterac mountains (20.85)" };
		{ "" };
		{ WHIT.."05) then run back to tarren mill and turn in " };
		{ WHIT.."stone tokens and prison break in. skip dalaran " };
		{ WHIT.."patrols and bracers of binding." };
		{ "" };
		{ WHIT.."06) turn in crown of will (part 2). skip crown " };
		{ WHIT.."of will part3." };
		{ "" };
		{ WHIT.."07) then fly to the uc." };
		{ "" };
		{ WHIT.."08) once at uc, turn in to steal from thieves." };
		{ "" };
		{ WHIT.."09) make sure you buy some fungus food " };
		{ WHIT.."(food for a turtle you’re about to tame later)" };
		{ "" };
		{ WHIT.."10) get on zeppelin to go to orgrimmar." };
		{ "" };
		{ WHIT.."11) once in orgrimmar, turn in alliance " };
		{ WHIT.."relations at keldran. (at 21.53)" };
		{ "" };
		{ WHIT.."12) buy 3 soothing spices(47,80) then fly to xrs" };
		{ "" };
		{ WHIT.."13) make xrs your home." };
		{ "" };
		{ WHIT.."14) fly to freewind post (1k needles)."		 };
	};	
	HordeLeveling22 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (37-38)" };
		Location = { AtlasLocale["Thousand Needles"] };
		{ WHIT.."01) stable pet" };
		{ "" };
		{ WHIT.."02) run towards the shimmering flats." };
		{ "" };
		{ WHIT.."03) turn in the swarm grows. accept the swarm " };
		{ WHIT.."grows part2 (67.63)" };
		{ "" };
		{ WHIT.."04) go tame a level 34-35 turtle(for bite rank5) " };
		{ WHIT.."feed the fungus/fruit/fish to him." };
		{ "" };
		{ WHIT.."05) stop at the goblins and turn in parts for " };
		{ WHIT.."kravel. accept delivery to the gnomes. turn in" };
		{ WHIT.."delivery to the gnomes. turn in goblin " };
		{ WHIT.."sponsorship part3. accept the eighteenth pilot. " };
		{ WHIT.."turn in the eighteenth pilot. accept razzeric's " };
		{ WHIT.."tweaking. turn in encrusted tail fins*" };
		{ "" };
		{ WHIT.."06) accept the rumormonger." };
		{ "" };
		{ WHIT.."07) do the following quests:" };
		{ "" };
		{ WHIT.."08) the swarm grows and parts of the swarm " };
		{ WHIT.."(quest starts from an item drop) (71.85)" };
		{ "" };
		{ WHIT.."09) then go turn in the swarm grows (67.63)" };
		{ "" };
		{ WHIT.."10) hearth to xrs." };
		{ "" };
		{ WHIT.."11) abandon turtle, get cat back out." };
		{ "" };
		{ WHIT.."12) turn in parts of the swarm. accept parts of " };
		{ WHIT.."the swarm part2." };
		{ "" };
		{ WHIT.."13) fly to tb and turn in frostmaw (61.80). " };
		{ WHIT.."accept deadmire." };
		{ "" };
		{ WHIT.."14) get new spells/abilities." };
		{ "" };
		{ WHIT.."15) fly to dustwallow marsh."		 };
	};	
	HordeLeveling23 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (38-39)" };
		Location = { AtlasLocale["Dustwallow Marsh"] };
		{ WHIT.."01) make brackenwall village your home." };
		{ "" };
		{ WHIT.."02) accept theramore spies, the black shield"  };
		{ "" };
		{ WHIT.."03) while doing the following steps, focus on " };
		{ WHIT.."also doing theramore spies." };
		{ "" };
		{ WHIT.."04) go slightly south of brackenwall village and " };
		{ WHIT.."accept hungry! (the orge at 35.38)" };
		{ "" };
		{ WHIT.."05) stop at jarl's cabin (55.25) and get the " };
		{ WHIT.."lost report quest (the dirt mound next to his " };
		{ WHIT.."cabin)" };
		{ "" };
		{ WHIT.."06) accept soothing spices  then turn in and " };
		{ WHIT.."accept jarl needs eyes ." };
		{ "" };
		{ WHIT.."07) go do hungry! (around 57.15)." };
		{ "" };
		{ WHIT.."08) i do the escort quest: stinky's escape " };
		{ WHIT.."(starts at 46.17) (grind mobs during this quest)" };
		{ "" };
		{ WHIT.."09) then go do jarl needs eyes along with the " };
		{ WHIT.."black shield (at 32.22)" };
		{ "" };
		{ WHIT.."10) after that finish up theramore spies." };
		{ "" };
		{ WHIT.."11) then stop at brackenwall village and turn in " };
		{ WHIT.."the lost report and" };
		{ WHIT.."theramore spies. accept the theramore docks. " };
		{ WHIT.."turn in the black shield. accept the black " };
		{ WHIT.."shield again and turn it in." };
		{ "" };
		{ WHIT.."12) turn in hungry! (35.38)" };
		{ "" };
		{ WHIT.."13) pick up the seaforium booster(53,63). for " };
		{ WHIT.."the quest razzeric's tweaking" };
		{ "" };
		{ WHIT.."14) go finish up the following quest:" };
		{ "" };
		{ WHIT.."15) deadmire (this croc roams around 47.57)" };
		{ "" };
		{ WHIT.."16) stop back at jarl's cabin (55.25), go to the " };
		{ WHIT.."dirt mound once again to get the severed head" };
		{ "" };
		{ WHIT.."17) turn in jarl needs eyes." };
		{ "" };
		{ WHIT.."18) then go do the theramore docks the captain's " };
		{ WHIT.."documents are under the water at (71.51).  " };
		{ WHIT.."do not go through the town, just go around the " };
		{ WHIT.."side of the town and into the water." };
		{ "" };
		{ WHIT.."19) then hearth back to brackenwall village." };
		{ "" };
		{ WHIT.."20) turn in the theramore docks and the severed " };
		{ WHIT.."head. accept the troll witchdoctor." };
		{ "" };
		{ WHIT.."21) fly to thunder bluff." };
		{ "" };
		{ WHIT.."22) turn in deadmire and the black shield part3." };
		{ "" };
		{ WHIT.."23) fly to ratchet, turn in stinky's escape " };
		{ WHIT.."while you're there." };
		{ "" };
		{ WHIT.."24) get on the boat to go to bb."	 };
		{ "" };
		{ WHIT.." be sure to grind a lot here or run an instance " };
		{ WHIT.."to make up for the lack of xp" };
		{ "" };
		{ WHIT.." because of the removed burning crusade quests"		 };
	};	
	HordeLeveling24 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (39-40)" };
		Location = { AtlasLocale["Stranglethorn Vale"] };
		{ WHIT.."01) accept the captain's chest, the bloodsail " };
		{ WHIT.."buccaneers and scaring shaky." };
		{ "" };
		{ WHIT.."02) make bb your home, then go up the steps and " };
		{ WHIT.."turn in the rumormonger." };
		{ WHIT.."accept dream dust in the swamp." };
		{ "" };
		{ WHIT.."03) fly to grom'gol." };
		{ "" };
		{ WHIT.."04) accept mok'thardin's enchantment." };
		{ "" };
		{ WHIT.."05) turn in the troll witchdoctor. right click " };
		{ WHIT.."the cauldron. accept marg speaks." };
		{ "" };
		{ WHIT.."06) go do:" };
		{ "" };
		{ WHIT.."07) raptor mastery along with mok'thardin's " };
		{ WHIT.."enchantment (31.41)  (kill jungle stalkers)" };
		{ "" };
		{ WHIT.."08) once that is done, go back to grom'gol and " };
		{ WHIT.."turn in mok'thardin's enchantment."  };
		{ WHIT.."accept mok'thardin's enchantment part3." };
		{ "" };
		{ WHIT.."09) then do panther mastery (bhag'thera)" };
		{ WHIT.."(he has 3 different spawn points, either at " };
		{ WHIT.."48.20, 49.23, or 47.26)" };
		{ "" };
		{ WHIT.."10) then turn in panther mastery (bhag'thera) " };
		{ WHIT.."and raptor mastery (jungle stalkers)" };
		{ WHIT.."at nesingwary's expedition (35.10)" };
		{ "" };
		{ WHIT.."11) accept raptor mastery (tethis) but don't do " };
		{ WHIT.."it now." };
		{ "" };
		{ WHIT.."12) hearth to bb." };
		{ "" };
		{ WHIT.."13) then go do:" };
		{ "" };
		{ WHIT.."14) the bloodsail buccaneers (just slightly " };
		{ WHIT.."north-west of bb at 27.69, there's a little note " };
		{ WHIT.."on a barrel, click on it, accept new quest)." };
		{ "" };
		{ WHIT.."15) scaring shaky along with mok'thardin's " };
		{ WHIT.."enchantment part3 (32.66)" };
		{ "" };
		{ WHIT.."16) once that's done run back into bb." };
		{ "" };
		{ WHIT.."17) turn in scaring shaky. accept return to " };
		{ WHIT.."mackinley." };
		{ "" };
		{ WHIT.."18) turn in the bloodsail buccaneers. accept " };
		{ WHIT.."the bloodsail buccaneers." };
		{ "" };
		{ WHIT.."19) turn in return to mackinley." };
		{ "" };
		{ WHIT.."20) then turn in the bloodsail buccaneers at " };
		{ WHIT.."fleet master seahorn." };
		{ "" };
		{ WHIT.."21) fly to grom'gol."		 };
		{ "" };
		{ WHIT.."22) turn in mok'thardin's enchantment part3. " };
		{ WHIT.."accept mok'thardin's enchantment part4." };
		{ "" };
		{ WHIT.."23) make grom'gol your home." };
		{ "" };
		{ WHIT.."24) you should be level 40 for sure now." };
		{ "" };
		{ WHIT.."25) i get new hunter spells/abilities at " };
		{ WHIT.."grom'gol, also make sure you get the lvl 40 " };
		{ WHIT.."arrows/bullets." };
		{ "" };
		{ WHIT.."26) then get on the zeppelin to go to the uc." };
		{ "" };
		{ WHIT.."27) fly to hammerfall." };
		{ "" };
		{ WHIT.."28) run all the way to the badlands." };
		{ "" };
		{ WHIT.."29) i suggest world chat /join world " };
		{ WHIT.."or get bennylava's triviabot to keep yourself " };
		{ WHIT.."entertained on the way"	 };
	};	
	HordeLeveling25 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (40-41)" };
		Location = { AtlasLocale["Badlands"] };
		{ WHIT.."01) as you enter badlands, run southwest(42.52)" };
		{ "" };
		{ WHIT.."02) turn in martek the exiled (at 42.52)  accept " };
		{ WHIT.."indurium and barbecued buzzard wings" };
		{ "" };
		{ WHIT.."03) go down south a little and do indurium " };
		{ WHIT.."(51.67)" };
		{ "" };
		{ WHIT.."04) go back up and turn indurium in  accept " };
		{ WHIT.."news for fizzle" };
		{ "" };
		{ WHIT.."05) then go north-west and accept study of the " };
		{ WHIT.."elements: rock (at 25.44)."  };
		{ WHIT.."note: if you have the frost oil item, accept and " };
		{ WHIT.."complete the quest coolant heads prevail." };
		{ "" };
		{ WHIT.."06) grind your way west to kargath (4.46)" };
		{ "" };
		{ WHIT.."07) get fp at kargath." };
		{ "" };
		{ WHIT.."08) do not make kargath your home. (it should " };
		{ WHIT.."still be grom'gol)" };
		{ "" };
		{ WHIT.."09) accept: unclaimed baggage, coyote thieves, " };
		{ WHIT.."report to helgrum, broken alliances and" };
		{ WHIT.."badlands reagent run." };
		{ "" };
		{ WHIT.."10) then go do:" };
		{ "" };
		{ WHIT.."11) barbecued buzzard wings" };
		{ "" };
		{ WHIT.."12) coyote thieves" };
		{ "" };
		{ WHIT.."13) broken alliances (i skip the next part to " };
		{ WHIT.."this quest)" };
		{ "" };
		{ WHIT.."14) badlands reagent run" };
		{ "" };
		{ WHIT.."15) unclaimed baggage (this is done at angor " };
		{ WHIT.."fortress, 42.31) (the pack is in an orange " };
		{ WHIT.."barrel on the left entrance of the fortress, and " };
		{ WHIT.."the rapier is on the weapon rack going through " };
		{ WHIT.."the right entrance of the fortress)" };
		{ "" };
		{ WHIT.."16) study of the elements: rock (lesser rock " };
		{ WHIT.."elementals), turn in, then do" };
		{ "" };
		{ WHIT.."17) study of the elements: rock (rock " };
		{ WHIT.."elementals), turn in, then do" };
		{ "" };
		{ WHIT.."18) study of the elements: rock (greater rock " };
		{ WHIT.."elementals)" };
		{ "" };
		{ WHIT.."19) then make sure all badlands quests (besides " };
		{ WHIT.."report to helgrum) are done and turned in." };
		{ "" };
		{ WHIT.."20) you should be about level 41 right now, if " };
		{ WHIT.."not, that's okay." };
		{ "" };
		{ WHIT.."21) hearth to grom'gol."		 };
		{ "" };
		{ WHIT.."22) go turn in all the the green hills of " };
		{ WHIT.."stranglethorn chapters/quests at nesingwary's " };
		{ WHIT.."expedition (35.10) if you have all the pages for " };
		{ WHIT.."it, if not you have another chance to turn this " };
		{ WHIT.."in again later. you can also look in the ah for " };
		{ WHIT.."your missing pages." };
		{ "" };
		{ WHIT.."23) then run all the way to swamp of sorrows, " };
		{ WHIT.."stopping along the way to accept this quest:" };
		{ WHIT.."nothing but the truth (in duskwood at 87.35)" };
		{ "" };
		{ WHIT.."24) then turn in nothing but the truth (the guy " };
		{ WHIT.."right next to him)" };
		{ "" };
		{ WHIT.."25) i skip nothing but the truth part2" };
		{ "" };
		{ WHIT.."26) then run into swamp of sorrows."		 };
	};	
	HordeLeveling26 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (41-42)" };
		Location = { AtlasLocale["Swamp of Sorrows"] };
		{ WHIT.."01) start off doing: dream dust in the swamp " };
		{ WHIT.."(14.59) just clear the whelps in this area once, " };
		{ WHIT.."you will finish this quest up later at the " };
		{ WHIT.."scalebane dragons far east." };
		{ "" };
		{ WHIT.."02) find and kill cudgel, he drops noboru's " };
		{ WHIT.."cudgel, which starts noboru the cudgel." };
		{ "" };
		{ WHIT.."03) go turn in noboru the cudgel (25.31) accept " };
		{ WHIT.."draenethyst crystals." };
		{ "" };
		{ WHIT.."04) go to stonard (45.54)" };
		{ "" };
		{ WHIT.."05) make stonard your home." };
		{ "" };
		{ WHIT.."06) accept: lack of surplus and little morsels. " };
		{ WHIT.."i skip fresh meat" };
		{ "" };
		{ WHIT.."07) stable pet." };
		{ "" };
		{ WHIT.."08) get fp." };
		{ "" };
		{ WHIT.."09) turn in report to helgrum  accept pool of " };
		{ WHIT.."tears." };
		{ "" };
		{ WHIT.."10) go tame a deathstrike tarantula (for bite " };
		{ WHIT.."rank6), then abandon." };
		{ "" };
		{ WHIT.."11) go tame a silt crawler (for claw rank6), " };
		{ WHIT.."then abandon." };
		{ "" };
		{ WHIT.."12) then hearth to stonard." };
		{ "" };
		{ WHIT.."13) get cat back out." };
		{ "" };
		{ WHIT.."14) go do the following quests:" };
		{ "" };
		{ WHIT.."15) pool of tears (the artifacts are in the " };
		{ WHIT.."water around temple of atal'hakkar)" };
		{ "" };
		{ WHIT.."16) lack of surplus then turn it in (at 81.80)  " };
		{ WHIT.."skip lack of surplus part2." };
		{ "" };
		{ WHIT.."17) dream dust in the swamp (killing the " };
		{ WHIT.."scalebane dragons)" };
		{ "" };
		{ WHIT.."18) draenethyst crystals along with little " };
		{ WHIT.."morsels (62.22)" };
		{ "" };
		{ WHIT.."19) go turn in draenethyst crystals (25.31)" };
		{ "" };
		{ WHIT.."20) hearth to stonard." };
		{ "" };
		{ WHIT.."21) turn in little morsels and pool of tears  " };
		{ WHIT.."accept the atal'ai exile." };
		{ "" };
		{ WHIT.."22) should be about lvl 42 now, if not that's " };
		{ WHIT.."okay.  get new hunter spells/abilities." };
		{ "" };
		{ WHIT.."23) fly to booty bay."		 };
	};	
	HordeLeveling27 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (42-43)" };
		Location = { AtlasLocale["Stranglethorn Vale"] };
		{ WHIT.."01) turn in dream dust in the swamp.  accept " };
		{ WHIT.."tran'rek and rumors for kravel" };
		{ "" };
		{ WHIT.."02) accept all of the following quests in bb: " };
		{ WHIT.."the bloodsail buccaneers, up to snuff," };
		{ WHIT.."venture company mining, zanzil's secret, akiris " };
		{ WHIT.."by the bundle, voodoo dues, stoley's debt," };
		{ WHIT.."stranglethorn fever and keep an eye out." };
		{ "" };
		{ WHIT.."03) make bb your home." };
		{ "" };
		{ WHIT.."04) then go do in the following order:" };
		{ "" };
		{ WHIT.."05) the bloodsail buccaneers along with up to " };
		{ WHIT.."snuff and keep an eye out (these are done at " };
		{ WHIT.."the shore around 31.80, and some more mobs can " };
		{ WHIT.."be found at 27.70).  also do stranglethorn fever"  };
		{ WHIT.."(for stranglethorn fever you will need one " };
		{ WHIT.."gorilla fang, the witch doctor is in the cave " };
		{ WHIT.."at 34.60)" };
		{ "" };
		{ WHIT.."06) mok'thardin's enchantment part4 along with " };
		{ WHIT.."akiris by the bundle (25.63)" };
		{ "" };
		{ WHIT.."07) go do raptor mastery (tethis) (28.44)" };
		{ "" };
		{ WHIT.."08) zanzil's secret along with voodoo dues " };
		{ WHIT.."(34.52 and 39.58)" };
		{ "" };
		{ WHIT.."09) venture company mining (41.44)" };
		{ "" };
		{ WHIT.."10) then go up (at 35.10) and turn in raptor " };
		{ WHIT.."mastery (tethis)  accept big game hunter (elite)" };
		{ "" };
		{ WHIT.."11) then go to grom'gol." };
		{ "" };
		{ WHIT.."12) turn in mok'thardin's enchantment" };
		{ "" };
		{ WHIT.."13) go do big game hunter (elite) (he is at " };
		{ WHIT.."38.35)  i am always able to solo him, but you " };
		{ WHIT.."could try partying up with someone, if you can't " };
		{ WHIT.."do it, skip it." };
		{ "" };
		{ WHIT.."14) then run all the way up and turn in big game " };
		{ WHIT.."hunter.  also if you haven't yet, turn in all " };
		{ WHIT.."the the green hills of stranglethorn chapters. " };
		{ WHIT.."you can also look in the ah for your missing " };
		{ WHIT.."pages, or you can simply skip the quest." };
		{ "" };
		{ WHIT.."15) then hearth to bb." };
		{ "" };
		{ WHIT.."16) turn in all quests: venture company mining, " };
		{ WHIT.."zanzil's secret, stranglethorn fever, up to " };
		{ WHIT.."snuff, the bloodsail buccaneers, akiris by the " };
		{ WHIT.."bundle, voodoo dues, and keep an eye out." };
		{ "" };
		{ WHIT.."17) get on the boat to go to ratchet" };
		{ "" };
		{ WHIT.."18) fly to dustwallow marsh."		 };
	};	
	HordeLeveling28 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (43-44)" };
		Location = { AtlasLocale["Dustwallow Marsh"] };
		{ WHIT.."01) accept identifying the brood, army of the " };
		{ WHIT.."black dragon. and " };
		{ WHIT.."overlord mok'morokk's concern" };
		{ "" };
		{ WHIT.."02) go down to (40.36) and accept questioning " };
		{ WHIT.."reethe then do it." };
		{ "" };
		{ WHIT.."03) then go back to brakenwall village and turn " };
		{ WHIT.."in questioning reethe" };
		{ "" };
		{ WHIT.."04) go do the following steps (9 - 16) in any " };
		{ WHIT.."order:" };
		{ "" };
		{ WHIT.."5) identifying the brood" };
		{ "" };
		{ WHIT.."6) if you didn't do earlier razzeric's tweaking " };
		{ WHIT.."(there is a box at 54.56)" };
		{ "" };
		{ WHIT.."7) marg speaks kill the mobs at around (58.63)" };
		{ "" };
		{ WHIT.."8) overlord mok'morokk's concern (snuff=44.66, " };
		{ WHIT.."grog=38.65, strongbox=in the cave at 36.69" };
		{ WHIT.."[the entrance to the cave is at 38.66])" };
		{ "" };
		{ WHIT.."9) army of the black dragon" };
		{ "" };
		{ WHIT.."10) go to brackenwall village(dustwallow marsh)." };
		{ "" };
		{ WHIT.."11) turn in questioning reethe, army of the " };
		{ WHIT.."black dragon, overlord mok'morokk's concern " };
		{ WHIT.."and identifying the brood  accept the brood  " };
		{ WHIT.."of onyxia" };
		{ "" };
		{ WHIT.."12) run back and forth until the brood of onyxia " };
		{ WHIT.."is done, but don't actually do the final quest " };
		{ WHIT.."yet (this will be done later at level 48, so you " };
		{ WHIT.."can abandon it for now)." };
		{ "" };
		{ WHIT.."13) turn in marg speaks.  accept report to zor" };
		{ "" };
		{ WHIT.."14) fly to tanaris"		 };
		{ "" };
		{ WHIT.." be sure to grind a lot here or run an instance " };
		{ WHIT.."to make up for the lack of xp" };
		{ "" };
		{ WHIT.." because of the removed burning crusade quests"		 };		
	};
	HordeLeveling29 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (44-45)" };
		Location = { AtlasLocale["Tanaris"] };
		{ WHIT.."01) go into gadgetzan." };
		{ "" };
		{ WHIT.."02) accept: wanted: caliph scorpidsting and " };
		{ WHIT.."wanted: andre firebeard  (wanted poster)" };
		{ "" };
		{ WHIT.."03) turn in tran'rek" };
		{ "" };
		{ WHIT.."04) accept: gadgetzan water survey. wastewander " };
		{ WHIT.."justice and water pouch bounty" };
		{ "" };
		{ WHIT.."05) make gadgetzan your home." };
		{ "" };
		{ WHIT.."06) go to the shimmering flats (in thousand " };
		{ WHIT.."needles at 78.77) and turn in rumors for kravel," };
		{ WHIT.."news for fizzle and razzeric's tweaking.  " };
		{ WHIT.."accept safety first" };
		{ "" };
		{ WHIT.."07) accept keeping pace" };
		{ "" };
		{ WHIT.."08) turn keeping pace in at zamek, then go pick " };
		{ WHIT.."up rizzle's plans at 77.77 accept rizzle's " };
		{ WHIT.."schematics" };
		{ "" };
		{ WHIT.."09) accept back to booty bay (kravel koalbeard)" };
		{ "" };
		{ WHIT.."10) go turn in rizzle's schematics at pozzik." };
		{ "" };
		{ WHIT.."11) hearth back to gadgetzan." };
		{ "" };
		{ WHIT.."12) turn in safety first.  i skip the next part " };
		{ WHIT.."to this quest." };
		{ "" };
		{ WHIT.."13) then grind your way east to steamwheedle " };
		{ WHIT.."port. (67.23)" };
		{ "" };
		{ WHIT.."14) accept: pirate hats ahoy!, screecher spirits " };
		{ WHIT.."and southsea shakedown" };
		{ "" };
		{ WHIT.."15) turn in stoley's debt.  accept stoley's " };
		{ WHIT.."shipment" };
		{ "" };
		{ WHIT.."16) then go complete these quests:" };
		{ "" };
		{ WHIT.."17) wastewander justice along with water pouch " };
		{ WHIT.."bounty (at 64.29" };
		{ "" };
		{ WHIT.."18) then go do all of the following " };
		{ WHIT.."(steps 19 - 23) at lost rigger cove (72.47):" };
		{ "" };
		{ WHIT.."19) southsea shakedown" };
		{ "" };
		{ WHIT.."20) pirate hats ahoy!" };
		{ "" };
		{ WHIT.."21) stoley's shipment" };
		{ "" };
		{ WHIT.."22) ship schedules (find this in one of the " };
		{ WHIT.."pirate's footlocker at lost rigger cove. if " };
		{ WHIT.."the item cannot be found, just skip it)" };
		{ "" };
		{ WHIT.."23) wanted: andre firebeard" };
		{ "" };
		{ WHIT.."24) once they're all done, hearth to gadgetzan." };
		{ "" };
		{ WHIT.."25) turn in water pouch bounty, wastewander " };
		{ WHIT.."justice.  accept more wastewander justice"		 };
		{ "" };
		{ WHIT.."26) go do gadgetzan water survey (grinding mobs " };
		{ WHIT.."along the way, coords are at 38.29), then go " };
		{ WHIT.."back and turn it in." };
		{ "" };
		{ WHIT.."27) go to steamwheedle port." };
		{ "" };
		{ WHIT.."28) turn in all quests there  accept deliver to " };
		{ WHIT.."mackinley." };
		{ "" };
		{ WHIT.."29) then go do: more wastewander justice along " };
		{ WHIT.."with wanted: caliph scorpidsting (59.37)" };
		{ "" };
		{ WHIT.."30) then hearth to gadgetzan.  make sure you " };
		{ WHIT.."save (put in your bank) all the wastewander " };
		{ WHIT.."water pouches you have, because you will need " };
		{ WHIT.."them for a later quest." };
		{ "" };
		{ WHIT.."31) turn in more wastewander justice and wanted:" };
		{ WHIT.."caliph scorpidsting" };
		{ "" };
		{ WHIT.."32) fly to freewind post." };
		{ "" };
		{ WHIT.."33) run west into feralas."		 };
	};
	HordeLeveling30 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (45-46)" };
		Location = { AtlasLocale["Feralas"] };
		{ WHIT.."01) run into camp mojache. (75.44)" };
		{ "" };
		{ WHIT.."02) accept all quests there: a new cloak's " };
		{ WHIT.."sheen, the ogres of feralas, gordunni cobalt," };
		{ WHIT.."war on the woodpaw, the mark of quality, a " };
		{ WHIT.."strange request." };
		{ "" };
		{ WHIT.."03) get fp." };
		{ "" };
		{ WHIT.."04) make camp mojache your home." };
		{ "" };
		{ WHIT.."05) then go do in the following order:" };
		{ "" };
		{ WHIT.."06) war on the woodpaw go just north of camp " };
		{ WHIT.."mojache to do this. (71.37)" };
		{ "" };
		{ WHIT.."07) the ogres of feralas along with gordunni " };
		{ WHIT.."cobalt (75.31)" };
		{ "" };
		{ WHIT.."08) make sure you click on one of the scrolls " };
		{ WHIT.."laying on the ground which starts: the gordunni " };
		{ WHIT.."scroll" };
		{ "" };
		{ WHIT.."09) then go do: a new cloak's sheen (67.48)" };
		{ "" };
		{ WHIT.."10) go back to camp mojache, turn in all those " };
		{ WHIT.."quests, accept all new quests." };
		{ "" };
		{ WHIT.."11) go do alpha strike (at 72.56) then turn it " };
		{ WHIT.."in.  accept woodpaw investigation" };
		{ "" };
		{ WHIT.."12) go do woodpaw investigation (at 71.55) " };
		{ WHIT.."complete it  accept the battle plans" };
		{ "" };
		{ WHIT.."13) then go do a grim discovery (66.46)" };
		{ "" };
		{ WHIT.."14) hearth back to camp mojache, turn in the " };
		{ WHIT.."battle plans and a grim discovery, accept all " };
		{ WHIT.."new quests." };
		{ "" };
		{ WHIT.."15) please note:  if the oox-22/fe distress " };
		{ WHIT.."beacon item drops, accept the quest " };
		{ WHIT.."find oox-22/fe!. turn in find oox-22/fe! (53.55)" };
		{ WHIT.."then go do:" };
		{ "" };
		{ WHIT.."16) stinglasher along with zukk'ash infestation " };
		{ WHIT.."(74.62)" };
		{ "" };
		{ WHIT.."17) screecher spirits (55.56)" };
		{ "" };
		{ WHIT.."18) the ogres of feralas part2 (59.68)" };
		{ WHIT.."note: while in this area, make sure you pick up " };
		{ WHIT.."a hippogryph egg down south-west " };
		{ WHIT.."(around 55.76),"  };
		{ WHIT.."this will be turned in at tanaris for a later " };
		{ WHIT.."quest." };
		{ "" };
		{ WHIT.."19) dark ceremony (59.68)" };
		{ "" };
		{ WHIT.."20) the mark of quality (53.55)" };
		{ "" };
		{ WHIT.."21) run back to camp mojache, then go turn those " };
		{ WHIT.."quests in, accept all new quests just from those"  };
		{ WHIT.."quest givers.  also accept the sunken temple."		 };
		{ "" };
		{ WHIT.."22) then do the following:" };
		{ "" };
		{ WHIT.."23) i then fly to orgrimmar" };
		{ "" };
		{ WHIT.."24) then i make it my home. then make sure i get " };
		{ WHIT.."120 wool cloth, 120 silk cloth, 120 mageweave " };
		{ WHIT.."out from either my bank or mailbox. for those of " };
		{ WHIT.."you who can't add, that's 6 stacks of 20 for " };
		{ WHIT.."each." };
		{ "" };
		{ WHIT.."25) then go turn in: zukk'ash report (56.46)" };
		{ "" };
		{ WHIT.."26) go complete a donation of silk and a " };
		{ WHIT.."donation of mageweave (at 63.51)" };
		{ "" };
		{ WHIT.."27) accept ripple recovery (from dran droffers) " };
		{ WHIT.."(59.36)" };
		{ "" };
		{ WHIT.."28) then turn in ripple recovery (the guy right " };
		{ WHIT.."next to dran)  accept ripple recovery again." };
		{ "" };
		{ WHIT.."29) turn in parts of the swarm and a grim " };
		{ WHIT.."discovery at belgrom rockmaul  accept betrayed " };
		{ WHIT.."(75.34) also get new spells/abilities while you " };
		{ WHIT.."are in orgrimmar." };
		{ "" };
		{ WHIT.."30) go turn in a strange request  skip: retrun " };
		{ WHIT.."to witch doctor uzer'i (cleft of shadow, 49.50)" };
		{ "" };
		{ WHIT.."31) go turn in report to zor  accept/complete " };
		{ WHIT.."service to the horde (valley of wisdom, 38.38)" };
		{ "" };
		{ WHIT.."32) go turn in the gordunni orb (valley of " };
		{ WHIT.."spirits, 39.86)" };
		{ "" };
		{ WHIT.."33) turn in a donation of silk and a donation of " };
		{ WHIT.."mageweave (for the troll faction, at 37.87)" };
		{ "" };
		{ WHIT.."34) fly to splintertree post (ashenvale)" };
		{ "" };
		{ WHIT.."35) then go east into azshara."		 };
	};
	HordeLeveling31 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (46-46)" };
		Location = { AtlasLocale["Azshara"] };
		{ WHIT.."01) go accept spiritual unrest and a land filled " };
		{ WHIT.."with hatred (at 10.78)" };
		{ "" };
		{ WHIT.."02) then go do them (at 17.66 and 20.62), and " };
		{ WHIT.."turn them in." };
		{ "" };
		{ WHIT.."03) then go to valormok (at 21.52)" };
		{ "" };
		{ WHIT.."04) turn in betrayed (skip the next part to this " };
		{ WHIT.."quest for now)" };
		{ "" };
		{ WHIT.."05) then get fp there and hearth to orgrimmar, " };
		{ WHIT.."if hearthstone is still on a cooldown, just fly" };
		{ WHIT.." to orgrimmar." };
		{ "" };
		{ WHIT.."06) then head to the undercity." };
		{ "" };
		{ WHIT.."07) then go to apothecary quarter and accept: " };
		{ WHIT.."seeping corruption and errand for apothecary " };
		{ WHIT.."zinge" };
		{ "" };
		{ WHIT.."08) then go turn in errand for apothecary zinge " };
		{ WHIT.."(out in the other room)" };
		{ "" };
		{ WHIT.."09) go return back, and turn in errand for " };
		{ WHIT.."apothecary zinge again  accept into the field" };
		{ "" };
		{ WHIT.."10) you can put the field testing kit and the " };
		{ WHIT.."box of empty vials in the bank for now." };
		{ WHIT.."accept: lines of communication in the uc " };
		{ WHIT.."(magic quarter)" };
		{ "" };
		{ WHIT.."11) get on the zeppelin to go to grom'gol." };
	};
	HordeLeveling32 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (46-47)" };
		Location = { AtlasLocale["Hinterlands"] };
		{ WHIT.."here are the prerequisites to this section:" };
		{ WHIT.."accept: grim message at grom'gol (in stv)" };
		{ WHIT.."01) go turn in ripple recovery (at 26.48)  " };
		{ WHIT.."accept a sticky situation" };
		{ "" };
		{ WHIT.."02) fly all the way to revantusk village(77.80)" };
		{ "" };
		{ WHIT.."03) accept: vilebranch hooligans, cannibalistic " };
		{ WHIT.."cousins, message to the wildhammer," };
		{ WHIT.."stalking the stalkers, hunt the savages, and " };
		{ WHIT.."avenging the fallen." };
		{ "" };
		{ WHIT.."04) before you go out and start questing make " };
		{ WHIT.."sure you stock up heavily on food/water/supplies " };
		{ WHIT.."cause you won't be back to town for while." };
		{ "" };
		{ WHIT.."05) then go do the following:" };
		{ "" };
		{ WHIT.."06) vilebranch hooligans" };
		{ "" };
		{ WHIT.."07) cannibalistic cousins" };
		{ "" };
		{ WHIT.."08) a sticky situation (57.41) turn this in when " };
		{ WHIT.."you get the chance  accept ripple delivery" };
		{ "" };
		{ WHIT.."09) stalking the stalkers" };
		{ "" };
		{ WHIT.."10) hunt the savages" };
		{ "" };
		{ WHIT.."11) testing the vessel" };
		{ "" };
		{ WHIT.."12) avenging the fallen" };
		{ "" };
		{ WHIT.."13) lines of communication" };
		{ "" };
		{ WHIT.."14) message to the wildhammer" };
		{ "" };
		{ WHIT.."15) rin'ji is trapped! (the escort quest, starts " };
		{ WHIT.."at 31.48)" };
		{ "" };
		{ WHIT.."16) grim message while doing this quest accept " };
		{ WHIT.."venom bottles (those little bottles on the table" };
		{ WHIT..",23.58)" };
		{ "" };
		{ WHIT.."17) yeah the good 'ol hinterlands grind have " };
		{ WHIT.."fun lol." };
		{ "" };
		{ WHIT.."18) if the oox-09/hl distress beacon item drops, " };
		{ WHIT.."accept the quest find oox-09/hl!" };
		{ "" };
		{ WHIT.."19) turn in find oox-09/hl! (at 49.38) (don't do " };
		{ WHIT.."the escort yet)" };
		{ "" };
		{ WHIT.."20) turn in rin'ji is trapped! (at 86.59)  " };
		{ WHIT.."accept rin'ji's secret" };
		{ "" };
		{ WHIT.."21) go to revantusk village." };
		{ "" };
		{ WHIT.."22) turn in all quests."		 };
		{ "" };
		{ WHIT.."23) then get fp and fly to hammerfall (arathi " };
		{ WHIT.."highlands)." };
		{ "" };
		{ WHIT.."24) i then get all my magewaeve out from my " };
		{ WHIT.."mailbox." };
		{ "" };
		{ WHIT.."25) go to doctor gregory victor to build up " };
		{ WHIT.."first aid" };
		{ "" };
		{ WHIT.."26) i'm usually able to get at least 260 first " };
		{ WHIT.."aid for the runecloth bandage. however, try to " };
		{ WHIT.."save the rest of the mageweave cloth for the " };
		{ WHIT.."cloth donation quest turn ins (in all the major " };
		{ WHIT.."cities)." };
		{ "" };
		{ WHIT.."27) then fly back to tm." };
		{ "" };
		{ WHIT.."28) then set your hearth here." };
		{ "" };
		{ WHIT.."28) turn in venom bottles  accept undamaged " };
		{ WHIT.."venom sac" };
		{ "" };
		{ WHIT.."29) then ride back into hinterlands (don't fly)" };
		{ "" };
		{ WHIT.."30) go do undamaged venom sac and the atal'ai " };
		{ WHIT.."exile (33.72) accept return to fel'zerul" };
		{ "" };
		{ WHIT.."31) then hearth to tm." };
		{ "" };
		{ WHIT.."32) turn in undamaged venom sac  i skip consult " };
		{ WHIT.."master gadrin" };
		{ "" };
		{ WHIT.."33) fly to the uc." };
		{ "" };
		{ WHIT.."34) go to magic quarter and turn in a donation " };
		{ WHIT.."of mageweave (you need 60 mageweave cloth)"  };
		{ "" };
		{ WHIT.."35) then turn in: lines of communication and " };
		{ WHIT.."rin'ji's secret then complete oran's gratitude" };
		{ "" };
		{ WHIT.."36) get on the zeppelin to go to grom'gol."		 };
	};
	HordeLeveling33 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (47-47)" };
		Location = { AtlasLocale["Stranglethorn Vale"] };
		{ WHIT.."01) once in grom'gol, turn in grim message (if " };
		{ WHIT.."you did the hinterlands section)." };
		{ "" };
		{ WHIT.."02) fly to bb." };
		{ "" };
		{ WHIT.."03) accept the bloodsail buccaneers part5 and " };
		{ WHIT.."whiskey slim's lost grog" };
		{ "" };
		{ WHIT.."04) make bb your home." };
		{ "" };
		{ WHIT.."05) turn in back to booty bay and deliver to " };
		{ WHIT.."mackinley" };
		{ "" };
		{ WHIT.."06) make sure that the captain's chest is " };
		{ WHIT.."accepted, then go do it (at 36.65).  if your " };
		{ WHIT.."class has troubles, either find a group to help," };
		{ WHIT.."or you can simply skip it." };
		{ "" };
		{ WHIT.."07) find a small bottle laying around the shore " };
		{ WHIT.."east of bb, until this item: carefully folded " };
		{ WHIT.."note shows up which starts message in a bottle" };
		{ "" };
		{ WHIT.."08) go turn message in a bottle in (at 38.80)  " };
		{ WHIT.."accept and do message in a bottle part2."   };
		{ WHIT.."once done turn it in." };
		{ "" };
		{ WHIT.."09) then go do the bloodsail buccaneers part5 " };
		{ WHIT.."(kill the three pirates in the three ships) " };
		{ WHIT.."while doing it, find cortello's riddle (it's " };
		{ WHIT.."usually a little scroll downstairs in the middle " };
		{ WHIT.."ship)" };
		{ "" };
		{ WHIT.."10) then hearth back to bb, if hearth stone " };
		{ WHIT.."still is on a cooldown, die on purpose." };
		{ "" };
		{ WHIT.."11) turn in the captain's chest and the " };
		{ WHIT.."bloodsail buccaneers part5" };
		{ "" };
		{ WHIT.."12) fly to kargath (badlands)" };
		{ "" };
		{ WHIT.."13) i get a stack of silk cloth out of the " };
		{ WHIT.."mailbox for the quest caught, then" };
		{ "" };
		{ WHIT.."14) go into searing gorge."		 };
	};
	HordeLeveling34 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (47-48)" };
		Location = { AtlasLocale["Searing Gorge"] };
		{ WHIT.."01) first thing i do here is go south-east and " };
		{ WHIT.."do caught! (guy in the outhouse, 65.62), then " };
		{ WHIT.."turn it in. accept ledger from tanaris  click on " };
		{ WHIT.."outhouse to get the goodsteel ledger" };
		{ "" };
		{ WHIT.."02) i then go start killing glassweb spiders for " };
		{ WHIT.."the ledger from tanaris quest." };
		{ "" };
		{ WHIT.."03) then go up north-west, talk to kalaran " };
		{ WHIT.."windblade (at 39.38) on way to thorium point " };
		{ WHIT.."(35.25). do first his listen to me quest divine " };
		{ WHIT.."retribution. in order to get the flawless flame " };
		{ WHIT.."once at thorium point collect all quests there " };
		{ WHIT.."and get the fp there too." };
		{ "" };
		{ WHIT.."04) then do the following (steps 5 - 13 in any " };
		{ WHIT.."order):" };
		{ "" };
		{ WHIT.."05) fiery menace!" };
		{ "" };
		{ WHIT.."06) curse these fat fingers" };
		{ "" };
		{ WHIT.."07) stolen: smithing tuyere and lookout's " };
		{ WHIT.."spyglass" };
		{ "" };
		{ WHIT.."08) the flawless flame turn in, then get and do " };
		{ WHIT.."forging the shaft" };
		{ "" };
		{ WHIT.."09) job opportunity: culling the competition" };
		{ "" };
		{ WHIT.."10) wanted: overseer maltorius (the plans for " };
		{ WHIT.."what the flux? quest is right behind overseer " };
		{ WHIT.."maltorius)" };
		{ "" };
		{ WHIT.."11) what the flux?" };
		{ "" };
		{ WHIT.."12) incendosaurs? whateverosaur is more like it" };
		{ "" };
		{ WHIT.."13) the key to freedom starts from an item drop " };
		{ WHIT.."grimsite outhouse key, turn in at the outhouse"  };
		{ WHIT.."(south-east, 65.62)" };
		{ "" };
		{ WHIT.."14) turn in forging the shaft  accept the " };
		{ WHIT.."flame's casing" };
		{ "" };
		{ WHIT.."15) do the flame's casing (around 24.36).  just " };
		{ WHIT.."keep killing the mobs there until the item drops." };
		{ "" };
		{ WHIT.."16) then turn in the flame's casing  accept and " };
		{ WHIT.."do the torch of retribution, then turn it in" };
		{ WHIT.."accept and do the torch of retribution part2 " };
		{ WHIT.."(just pick up the torch on the ground." };
		{ "" };
		{ WHIT.."17) accept squire maltrake, then turn it in  " };
		{ WHIT.."accept set them ablaze!" };
		{ "" };
		{ WHIT.."18) do set them ablaze! (north=33.54, " };
		{ WHIT.."south=44.61, east=50.54, west=35.60). once that " };
		{ WHIT.."is done, go back and turn it in." };
		{ "" };
		{ WHIT.."19) click on the little chest on the ground " };
		{ WHIT.."(at 38.38) and accept trinkets, then click on " };
		{ WHIT.."the chest again to turn it in."		 };
		{ "" };
		{ WHIT.."20) make sure you keep the black dragonflight " };
		{ WHIT.."molt in your inventory as you will need it later." };
		{ "" };
		{ WHIT.."21) once all that's done, turn all searing gorge" };
		{ WHIT.."quests in. make sure you have all 20 solid " };
		{ WHIT.."crystal leg shafts." };
		{ "" };
		{ WHIT.."22) head south into burning steppes (you have to" };
		{ WHIT.."go through black rock mountains)" };
		{ "" };
		{ WHIT.."23) i discover some areas, then get fp there " };
		{ WHIT.."(at 65.25), and fly directly to stonard" };
		{ WHIT.."(swamp of sorrows)"		 };
	};
	HordeLeveling35 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (48-49)" };
		Location = { AtlasLocale["Swamp of Sorrows"] };
		{ WHIT.."01) do not make stonard your home. (it should " };
		{ WHIT.."still be booty bay)" };
		{ "" };
		{ WHIT.."02) go to the fallen hero of the horde (34.66) " };
		{ WHIT.."keep talking to him till you get the disgraced " };
		{ WHIT.."one" };
		{ "" };
		{ WHIT.."03) then go do cortello's riddle (22.48 under " };
		{ WHIT.."the bridge)" };
		{ "" };
		{ WHIT.."04) accept cortello's riddle part2" };
		{ "" };
		{ WHIT.."05) go back to stonard." };
		{ "" };
		{ WHIT.."06) go turn in the disgraced one (and return to " };
		{ WHIT.."fel'zerul if you did hinterlands)"  };
		{ WHIT.."accept the missing orders" };
		{ "" };
		{ WHIT.."07) get new hunter spells/abilities" };
		{ "" };
		{ WHIT.."08) go turn in the missing orders (in the inn)  " };
		{ WHIT.."accept the swamp talker" };
		{ "" };
		{ WHIT.."09) go do the swamp talker (in cave at 65.78)" };
		{ "" };
		{ WHIT.."10) then head north east and" };
		{ "" };
		{ WHIT.."11) go kill the mob jarquia (at around 94.50 or " };
		{ WHIT.."92.65), he drops goodsteel's balanced flameberge" };
		{ WHIT.."for the quest ledger from tanaris" };
		{ "" };
		{ WHIT.."12) then die on purpose, so you end up at " };
		{ WHIT.."stonard." };
		{ "" };
		{ WHIT.."13) go to the fallen hero of the horde (34.66) " };
		{ WHIT.."and turn in the swamp talker" };
		{ "" };
		{ WHIT.."14) accept and do a tale of sorrow " };
		{ WHIT.."(just talk to the guy)" };
		{ "" };
		{ WHIT.."15) hearth to booty bay." };
		{ "" };
		{ WHIT.."16) get on the boat to go to ratchet." };
		{ "" };
		{ WHIT.."17) fly to brackenwall village(dustwallow marsh)" };
		{ WHIT.."once there accept the brood of onyxia" };
		{ "" };
		{ WHIT.."18) then head south and get the overdue package " };
		{ WHIT.."for the quest ledger from tanaris (at 54.55)" };
		{ "" };
		{ WHIT.."19) go down a bit and do the brood of onyxia  " };
		{ WHIT.."(the eggs can be found at 53.72 and 48.75)" };
		{ "" };
		{ WHIT.."20) then head west to bloodfen burrow cave " };
		{ WHIT.."(32.67), and do cortello's riddle"  };
		{ WHIT.."accept the next part to it." };
		{ "" };
		{ WHIT.."21) then go to brackenwall village." };
		{ "" };
		{ WHIT.."22) turn in the brood of onyxia  accept " };
		{ WHIT.."challenge overlord mok'morokk" };
		{ "" };
		{ WHIT.."23) do challenge overlord mok'morokk then turn " };
		{ WHIT.."it in." };
		{ "" };
		{ WHIT.."24) then fly to tanaris."		 };
	};
	HordeLeveling36 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (49-49)" };
		Location = { AtlasLocale["Feralas"] };
		{ WHIT.."01) accept: improved quality vengeance on the " };
		{ WHIT.."northspring and dark heart skip: the strength of" };
		{ WHIT.."corruption" };
		{ "" };
		{ WHIT.."02) turn in return to witch doctor uzer'i  skip " };
		{ WHIT.."the follow up quests." };
		{ "" };
		{ WHIT.."03) make camp mojache your home." };
		{ "" };
		{ WHIT.."04) stable pet" };
		{ "" };
		{ WHIT.."05) turn in testing the vessel. accept " };
		{ WHIT.."hippogryph muisek." };
		{ "" };
		{ WHIT.."06) hearth back and turn it in and do the rest " };
		{ WHIT.."of the muisek quests." };
		{ "" };
		{ WHIT.."07) go do the chicken escort quest (starts at " };
		{ WHIT.."53.55)." };
		{ "" };
		{ WHIT.."08) go accept zapped giants (at 44.43)" };
		{ "" };
		{ WHIT.."09) go tame an ironfur patriarch(for claw rank7)" };
		{ "" };
		{ WHIT.."10) do the following quests:" };
		{ "" };
		{ WHIT.."11) improved quality (52.32)" };
		{ WHIT.."(while doing this quest you should find an item " };
		{ WHIT.."which starts perfect yeti hide)" };
		{ "" };
		{ WHIT.."12) vengeance on the northspring (40.13)" };
		{ "" };
		{ WHIT.."13) dark heart (41.09)" };
		{ "" };
		{ WHIT.."14) zapped giants (38.18)" };
		{ "" };
		{ WHIT.."15) once those quests are completed and zapped " };
		{ WHIT.."giants is turned in" };
		{ "" };
		{ WHIT.."16) hearth to camp mojache." };
		{ "" };
		{ WHIT.."17) abandon bear, get cat back out." };
		{ "" };
		{ WHIT.."18) turn in improved quality , perfect yeti " };
		{ WHIT.."hide, vengeance on the northspring and dark " };
		{ WHIT.."heart" };
		{ "" };
		{ WHIT.."19) fly to tanaris."		 };
	};
	HordeLeveling37 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (49-50)" };
		Location = { AtlasLocale["Tanaris"] };		
		{ WHIT.."01) go to the egg-o-matic machine in gadgetzan, " };
		{ WHIT.."and turn in the hippogryph egg for"  };
		{ WHIT.."the super egg-o-matic quest." };
		{ "" };
		{ WHIT.."02) make gadgetzan your home." };
		{ "" };
		{ WHIT.."03) turn in ledger from tanaris and into the " };
		{ WHIT.."field  accept and complete shake that thirst"   };
		{ WHIT.."skip  tanaris field sampling  accept: the " };
		{ WHIT.."thirsty goblin, noxious lair, investigation," };
		{ WHIT.."super sticky, thistleshrub valley and the " };
		{ WHIT.."dunemaul compound" };
		{ "" };
		{ WHIT.."04) go turn in the sunken temple (at 52.45)  " };
		{ WHIT.."accept the stone circle and gahz'ridian" };
		{ "" };
		{ WHIT.."05) go do in the following order:" };
		{ "" };
		{ WHIT.."06) the dunemaul compound along with " };
		{ WHIT.."gahz'ridian (done at 46.66 and 40.73, gor'marok " };
		{ WHIT.."is in the cave at 40.58)" };
		{ "" };
		{ WHIT.."07) noxious lair investigation (34.47)" };
		{ "" };
		{ WHIT.."08) thistleshrub valley along with the thirsty " };
		{ WHIT.."goblin (28.65)" };
		{ "" };
		{ WHIT.."09) tooga's quest (escort quest starts in the " };
		{ WHIT.."thistleshrub valley, lead tooga to torta 66.25)." };
		{ "" };
		{ WHIT.."10) then turn in tooga's quest along with " };
		{ WHIT.."screecher spirits" };
		{ "" };
		{ WHIT.."11) run to gadgetzan." };
		{ "" };
		{ WHIT.."12) turn in the thirsty goblin  accept in good " };
		{ WHIT.."taste turn it in  accept sprinkle's secret " };
		{ WHIT.."ingredient" };
		{ "" };
		{ WHIT.."13) turn in thistleshrub valley, the dunemaul " };
		{ WHIT.."compound and noxious lair investigation" };
		{ "" };
		{ WHIT.."14) accept the scrimshank redemption" };
		{ "" };
		{ WHIT.."15) go do:" };
		{ "" };
		{ WHIT.."16) turn in gahz'ridian (at 52.45)." };
		{ "" };
		{ WHIT.."17) the scrimshank redemption (in the cave at " };
		{ WHIT.."53.70) the secret for finding the item for this " };
		{ WHIT.."quest is keep making right turns in the cave and" };
		{ WHIT.."it will lead you to it."  };
		{ "" };
		{ WHIT.."18) if the oox-17/tn distress beacon item drops," };
		{ WHIT.."accept the quest find oox-17/tn!" };
		{ "" };
		{ WHIT.."19) turn in find oox-17/tn! (at 60.64)" };
		{ "" };
		{ WHIT.."20) then do the chicken escort rescue oox-17/tn!" };
		{ "" };
		{ WHIT.."21) once all those quests are finished" };
		{ "" };
		{ WHIT.."22) hearth to gadgetzan."		 };
		{ "" };
		{ WHIT.."23) turn in the scrimshank redemption  accept " };
		{ WHIT.."insect part analysis" };
		{ "" };
		{ WHIT.."24) turn in insect part analysis  accept insect " };
		{ WHIT.."part analysis again." };
		{ "" };
		{ WHIT.."25) turn in insect part analysis  accept rise of " };
		{ WHIT.."the silithid" };
		{ "" };
		{ WHIT.."26) fly to orgrimmar." };
		{ "" };
		{ WHIT.."27) make orgrimmar your home." };
		{ "" };
		{ WHIT.."28) get the box of empty vials out of your bank." };
		{ "" };
		{ WHIT.."29) fly to azshara."		 };
	};
	HordeLeveling38 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (50-50)" };
		Location = { AtlasLocale["Azshara"] };
		{ WHIT.."01) accept: stealing knowledge." };
		{ "" };
		{ WHIT.."02) go do:" };
		{ "" };
		{ WHIT.."03) stealing knowledge (done at the ruins of " };
		{ WHIT.."eldarath, 36.54)" };
		{ "" };
		{ WHIT.."04) seeping corruption (tide pools are at: " };
		{ WHIT.."1=47.61, 2=47.51, 3=48.48, 4=47.46)" };
		{ "" };
		{ WHIT.."05) once their done, turn in stealing knowledge." };
		{ WHIT.."accept all 4 delivery quests." };
		{ "" };
		{ WHIT.."06) turn in delivery to archmage xylem (there is" };
		{ WHIT.."a teleporter at 28.50,). accept" };
		{ WHIT.."xylem's payment to jediga" };
		{ "" };
		{ WHIT.."07) then fly to thunder bluff." };
		{ "" };
		{ WHIT.."08) turn in delivery to magatha (on elder rise)." };
		{ WHIT.."accept magatha's payment to jediga" };
		{ "" };
		{ WHIT.."09) hearth to orgrimmar." };
		{ "" };
		{ WHIT.."10) go turn in rise of the silithid. accept " };
		{ WHIT.."march of the silithid (at 56.46)" };
		{ "" };
		{ WHIT.."11) turn in delivery to jes'rimon (at 55.34). " };
		{ WHIT.."accept jes'rimon's payment to jediga and" };
		{ WHIT.."bone-bladed weapons." };
		{ "" };
		{ WHIT.."12) then turn in ripple delivery (at 59.36)  " };
		{ WHIT.."(if you did the hinterlands section)" };
		{ "" };
		{ WHIT.."13) then go get new hunter spells/abilities. " };
		{ WHIT.."accept the hunter's charm (if you're a hunter)." };
		{ "" };
		{ WHIT.."14) then go to the undercity, head to the " };
		{ WHIT.."apothecarium quarter." };
		{ "" };
		{ WHIT.."15) turn in delivery to andron gant. accept " };
		{ WHIT.."andron's payment to jediga." };
		{ "" };
		{ WHIT.."16) then turn in seeping corruption and accept " };
		{ WHIT.."vivian lagrave while you are there." };
		{ "" };
		{ WHIT.."17) then accept seeping corruption. then turn it " };
		{ WHIT.."in (at the tauren right next to him)." };
		{ "" };
		{ WHIT.."18) then complete seeping corruption for the " };
		{ WHIT.."final time." };
		{ "" };
		{ WHIT.."19) then accept a sample of slime and a batch of" };
		{ WHIT.."ooze." };
		{ "" };
		{ WHIT.."20) then fly to tarren mill (hillsbrad)." };
		{ "" };
		{ WHIT.."21) then go into hinterlands (the entrance is at" };
		{ WHIT.."86.30 in hillsbrad)."		 };
	};
	HordeLeveling39 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (50-51)" };
		Location = { AtlasLocale["Hinterlands"] };
		{ WHIT.."01) as you enter hinterlands go to 26.48 and " };
		{ WHIT.."turn in ripple recovery. skip a sticky situation" };
		{ "" };
		{ WHIT.."02) next go down south to 33.73 and turn in the " };
		{ WHIT.."atal'ai exile (33.73). skip return to fel'zerul" };
		{ "" };
		{ WHIT.."03) then go do sprinkle's secret ingredient " };
		{ WHIT.."(the lake at 41.60)" };
		{ "" };
		{ WHIT.."04) go to raventusk village (at 77.80)" };
		{ "" };
		{ WHIT.."05) accept: snapjaws, mon!, gammerita, mon! and " };
		{ WHIT.."lard lost his lunch." };
		{ "" };
		{ WHIT.."06) stable cat." };
		{ "" };
		{ WHIT.."07) go tame a saltwater snapjaw (for bite rank7)" };
		{ WHIT.."then abandon, get cat back out." };
		{ "" };
		{ WHIT.."08) go do in the following order:" };
		{ "" };
		{ WHIT.."09) snapjaws, mon! along with gammerita, mon! " };
		{ WHIT.."and whiskey slim's lost grog" };
		{ "" };
		{ WHIT.."10) cortello's riddle (at 80.46, little chest " };
		{ WHIT.."in the water, 14 slot bag along with 8800 xp)" };
		{ "" };
		{ WHIT.."11) lard lost his lunch (at 84.42)" };
		{ "" };
		{ WHIT.."12) then go turn in snapjaws, mon!, gammerita, " };
		{ WHIT.."mon! and lard lost his lunch" };
		{ "" };
		{ WHIT.."13) get fp there and fly to hammerfall " };
		{ WHIT.."(arathi highlands)" };
		{ "" };
		{ WHIT.."14) build up first aid at doctor gregory victor." };
		{ "" };
		{ WHIT.."15) then hearth to orgrimmar." };
		{ "" };
		{ WHIT.."16) fly to azshara." };
		{ "" };
		{ WHIT.."17) turn in all 4 delivery quests." };
		{ "" };
		{ WHIT.."18) if you're not level 51, grind till you are." };
		{ "" };
		{ WHIT.."19) fly to ratchet." };
		{ "" };
		{ WHIT.."20) pick up the stone circle at liv rizzlefix " };
		{ WHIT.."shop. also accept volcanic activity while " };
		{ WHIT.."you're there." };
		{ "" };
		{ WHIT.."21) get on the boat to go to bb" };
		{ "" };
		{ WHIT.."22) fly to stonard (swamp of sorrows)" };
		{ "" };
		{ WHIT.."23) go into blasted lands."		 };
	};
	HordeLeveling40 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (51-51)" };
		Location = { AtlasLocale["Blasted Lands"] };
		{ WHIT.."01) i then find all the items to complete these " };
		{ WHIT.."5 quests:" };
		{ "" };
		{ WHIT.."02) a boar's vitality" };
		{ "" };
		{ WHIT.."03) snickerfang jowls" };
		{ "" };
		{ WHIT.."04) the basilisk's bite" };
		{ "" };
		{ WHIT.."05) the decisive striker" };
		{ "" };
		{ WHIT.."06) vulture's vigor" };
		{ "" };
		{ WHIT.."07) you need to find these many items to " };
		{ WHIT.."complete those quests:" };
		{ "" };
		{ WHIT.."08) vulture gizzards = 14" };
		{ "" };
		{ WHIT.."09) basilisk brain = 11" };
		{ "" };
		{ WHIT.."10) blasted boar lungs = 6" };
		{ "" };
		{ WHIT.."11) scorpok pincers = 6" };
		{ "" };
		{ WHIT.."12) snickerfang jowls = 5" };
		{ "" };
		{ WHIT.."13) while i'm in blasted lands, i discover most " };
		{ WHIT.."of all the areas for xp." };
		{ "" };
		{ WHIT.."14) if an imperfect draenethyst fragment drops " };
		{ WHIT.."turn in for this quest:" };
		{ WHIT.."everything counts in large amounts." };
		{ "" };
		{ WHIT.."15) note: i have never found a flawless " };
		{ WHIT.."draenethyst sphere for the quest to serve " };
		{ WHIT.."kum'isha" };
		{ "" };
		{ WHIT.."16) turn in all quests." };
		{ "" };
		{ WHIT.."17) i turn in the decisive striker in asap for " };
		{ WHIT.."ground scorpok assay (+25 to agility), so i can " };
		{ WHIT.."use the extra +25 agility boost while in the " };
		{ WHIT.."blasted lands. class may differ" };
		{ "" };
		{ WHIT.."18) go back to stonard (swamp of sorrows)" };
		{ "" };
		{ WHIT.."19) fly to bb, take the boat to rachet"		 };
		{ "" };
		{ WHIT.."21) fly to thunder bluff." };
		{ "" };
		{ WHIT.."22) make thunder bluff your home." };
		{ "" };
		{ WHIT.."23) note: only do the following if you are using" };
		{ WHIT.."a class that buys mana to drink and that you " };
		{ WHIT.."have at least 15 gold: go to the av battlemaster" };
		{ WHIT.."(at 56.76 in tb) and enter av (alterac valley)"  };
		{ WHIT.."(choose first available). once in av go to " };
		{ WHIT.."grunnda wolfheart <frostwolf supply officer> " };
		{ WHIT.."(at 49.82) and buy as many alterac manna " };
		{ WHIT.."biscuits as you can (buy as many so you only " };
		{ WHIT.."have like 5-10 gold left)." };
		{ WHIT.."then to leave av, simply type: /afk. these " };
		{ WHIT.."biscuits will help replenish mana and health " };
		{ WHIT.."much faster, thus making leveling a lot faster." };
		{ "" };
		{ WHIT.."24) fly to tanaris." };
		{ "" };
		{ WHIT.."25) turn in sprinkle's secret ingredient. accept" };
		{ WHIT.."delivery for marin" };
		{ "" };
		{ WHIT.."26) turn in march of the silithid. accept bungle" };
		{ WHIT.."in the jungle" };
		{ "" };
		{ WHIT.."27) turn in delivery for marin. accept " };
		{ WHIT.."noggenfogger elixir then turn it in." };
		{ "" };
		{ WHIT.."28) if you have a mithril casing get that out of" };
		{ WHIT.."your mailbox." };
		{ "" };
		{ WHIT.."29) go turn in the stone circle (at 52.45 in " };
		{ WHIT.."tanaris)" };
		{ "" };
		{ WHIT.."30) then go into un'goro crater."		 };
	};
	HordeLeveling41 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (51-53)" };
		Location = { AtlasLocale["Un'Goro Crater"] };
		{ WHIT.."01) go accept: the apes of un'goro and the fare" };
		{ WHIT.."of lar'korwi (at 71.75)" };
		{ "" };
		{ WHIT.."02) go do the fare of lar'korwi (at 68.56)" };
		{ "" };
		{ WHIT.."03) then go start doing: super sticky (up north" };
		{ WHIT.."in the middle) until this drops a mangled " };
		{ WHIT.."journal" };
		{ "" };
		{ WHIT.."04) then go into marshal's refuge (43.10), " };
		{ WHIT.."accept all quests there:  chasing a-me 01," };
		{ WHIT.."shizzle's flyer lost!, beware of pterrordax, " };
		{ WHIT.."roll the bones, alien ecology, expedition " };
		{ WHIT.."salvation, larion and muigin" };
		{ "" };
		{ WHIT.."05) turn in williden's journal." };
		{ "" };
		{ WHIT.."06) get fp there, then go do the ungoro grind :)." };
		{ "" };
		{ WHIT.."07) note: don't worry about the ungoro dirt " };
		{ WHIT.."mounds, you'll find enough soil from mob drops. " };
		{ WHIT.."also i don't worry about the bloodpetal sprouts," };
		{ WHIT.."as they are not turned in for any xp." };
		{ "" };
		{ WHIT.."08) while questing, find 7 crystals of each " };
		{ WHIT.."color (red, blue, green, and yellow for the"  };
		{ WHIT.."crystals of power quest)" };
		{ "" };
		{ WHIT.."09) super sticky" };
		{ "" };
		{ WHIT.."10) the apes of un'goro along with " };
		{ WHIT.."chasing a-me 01 (if you have a mithril casing do" };
		{ WHIT.."the escort)(63.17)" };
		{ "" };
		{ WHIT.."11) larion and muigin" };
		{ "" };
		{ WHIT.."12) beware of pterrordax along with shizzle's " };
		{ WHIT.."flyer and" };
		{ "" };
		{ WHIT.."13) roll the bones" };
		{ "" };
		{ WHIT.."14) expedition salvation (68.36 and 38.65)" };
		{ "" };
		{ WHIT.."15) a batch of ooze (i make sure i get at least" };
		{ WHIT.."25 ooze samples)" };
		{ "" };
		{ WHIT.."16) go turn in the apes of un'goro. accept the" };
		{ WHIT.."mighty u'cha" };
		{ "" };
		{ WHIT.."17) turn in the fare of lar'korwi. accept the " };
		{ WHIT.."scent of lar'korwi" };
		{ "" };
		{ WHIT.."18) then go do:" };
		{ "" };
		{ WHIT.."19) bone-bladed weapons along with the scent of " };
		{ WHIT.."lar'korwi" };
		{ "" };
		{ WHIT.."20) it's a secret to everybody. click on boat " };
		{ WHIT.."and then bag under water (at 63.68)" };
		{ "" };
		{ WHIT.."21) then turn in the scent of lar'korwi. accept" };
		{ WHIT.."the bait for lar'korwi (71.75)" };
		{ "" };
		{ WHIT.."22) then go do alien ecology along with bungle " };
		{ WHIT.."in the jungle (50.80)"		 };
		{ "" };
		{ WHIT.."23) go accept finding the source (at 30.50), " };
		{ WHIT.."then do it along with volcanic activity" };
		{ "" };
		{ WHIT.."24) the hotspot path for the quest finding the " };
		{ WHIT.."source starts at 51,42 and the hot spot is at " };
		{ WHIT.."50,46"  };
		{ "" };
		{ WHIT.."25) lost! make sure all quests are done before " };
		{ WHIT.."doing this (besides the mighty u'cha and" };
		{ WHIT.."the bait for lar'korwi)" };
		{ "" };
		{ WHIT.."26) this leads back to marshal's refuge, turn in" };
		{ WHIT.."all quests there, accept new ones (including " };
		{ WHIT.."all the pylon quests in the cave)." };
		{ "" };
		{ WHIT.."27) then go do (in the following order):" };
		{ "" };
		{ WHIT.."28) the northern pylon (at 56.13)" };
		{ "" };
		{ WHIT.."29) the mighty u'cha (cave at 63.13)" };
		{ "" };
		{ WHIT.."30) the eastern pylon (at 77.50)" };
		{ "" };
		{ WHIT.."31) the bait for lar'korwi (80.50).  then turn" };
		{ WHIT.."it in along with the mighty u'cha (71.75)" };
		{ "" };
		{ WHIT.."32) go west killing oozes along the way." };
		{ "" };
		{ WHIT.."33) turn in finding the source. accept the new" };
		{ WHIT.."springs (30.50)" };
		{ "" };
		{ WHIT.."34) the western pylon (at 23.58)" };
		{ "" };
		{ WHIT.."35) go back to marshal's refuge killing oozes" };
		{ WHIT.."along the way." };
		{ "" };
		{ WHIT.."36) turn in quests. make sure you complete" };
		{ WHIT.."making sense of it (just talk to the gnome in" };
		{ WHIT.."the cave)" };
		{ "" };
		{ WHIT.."37) fly to tanaris." };
		{ "" };
		{ WHIT.."38) turn in super sticky and bungle in the " };
		{ WHIT.."jungle. i skip pawn captures queen" };
		{ "" };
		{ WHIT.."39) then hearth to thunder bluff." };
		{ "" };
		{ WHIT.."40) go to elder rise." };
		{ "" };
		{ WHIT.."41) accept un'goro soil. then turn it in. accept" };
		{ WHIT.."morrowgrain research. then turn that in." };
		{ "" };
		{ WHIT.."42) skip the next part (the part that says bring" };
		{ WHIT.."10 morrowgrain)" };
		{ "" };
		{ WHIT.."43) go get new spells/abilities." };
		{ "" };
		{ WHIT.."44) fly to xrs." };
		{ "" };
		{ WHIT.."45) make xrs your home." };
		{ "" };
		{ WHIT.."46) then fly to ratchet." };
		{ "" };
		{ WHIT.."47) turn in volcanic activity and marvon's " };
		{ WHIT.."workshop - i skip zapper fuel"		 };
		{ "" };
		{ WHIT.."48) get on the boat to go to bb." };
		{ "" };
		{ WHIT.."49) turn in whiskey slim's lost grog" };
		{ "" };
		{ WHIT.."50) also turn in rescue oox-17/tn!(if did it)" };
		{ "" };
		{ WHIT.."51) fly to kargath (badlands)." };
		{ "" };
		{ WHIT.."52) turn in vivian lagrave" };
		{ "" };
		{ WHIT.."53) accept: dreadmaul rock and the rise of the " };
		{ WHIT.."machines" };
		{ "" };
		{ WHIT.."54) then fly to burning steppes."		 };
	};
	HordeLeveling42 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (53-53)" };
		Location = { AtlasLocale["Burning Steppes"] };
		{ WHIT.."01) accept: broodling essence and tablet of the " };
		{ WHIT.."seven" };
		{ "" };
		{ WHIT.."02) then go do:" };
		{ "" };
		{ WHIT.."03) broodling essence (to the east)(also kill " };
		{ WHIT.."as many dragon whelps you see, their squishy " };
		{ WHIT.."mobs that die real fast)" };
		{ "" };
		{ WHIT.."04) work your way to the cave (at 94.31) and " };
		{ WHIT.."accept a taste of flame. then turn it in." };
		{ "" };
		{ WHIT.."05) dreadmaul rock (79.45). then krom'grul " };
		{ WHIT.."(krom'grul has 2 different spawn points, in " };
		{ WHIT.."either cave)" };
		{ "" };
		{ WHIT.."06) the rise of the machines" };
		{ "" };
		{ WHIT.."07) tablet of the seven (it's at 54.40)" };
		{ "" };
		{ WHIT.."08) go turn in tablet of the seven and broodling" };
		{ WHIT.."essence. accept felnok steelspring" };
		{ "" };
		{ WHIT.."09) then fly to kargath (badlands)" };
		{ "" };
		{ WHIT.."10) turn in krom'grul" };
		{ "" };
		{ WHIT.."11) turn in the rise of the machines. accept the" };
		{ WHIT.."rise of the machines part2" };
		{ "" };
		{ WHIT.."12) go turn in the rise of the machines part2" };
		{ WHIT.."(at 25.46 in badlands)" };
		{ "" };
		{ WHIT.."13) then hearth to xrs." };
		{ "" };
		{ WHIT.."14) then fly to orgrimmar." };
		{ "" };
		{ WHIT.."15) go turn in bone-bladed weapons (55.34)" };
		{ "" };
		{ WHIT.."16) fly to splintertree post, (ashenvale)." };
		{ "" };
		{ WHIT.."17) go north into felwood."		 };
	};
	HordeLeveling43 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (53-54)" };
		Location = { AtlasLocale["Azshara"] };
		{ WHIT.."01) accept betrayed" };
		{ "" };
		{ WHIT.."02) go turn in the hunter's charm (this is a " };
		{ WHIT.."hunter only quest, at 42.42). accept courser " };
		{ WHIT.."antlers" };
		{ "" };
		{ WHIT.."03) go accept kim'jael indeed! (at 53.21 on top " };
		{ WHIT.."of the hill)" };
		{ "" };
		{ WHIT.."04) then go do:" };
		{ "" };
		{ WHIT.."05) courser antlers" };
		{ "" };
		{ WHIT.."06) i grind away at blood elves all the way to " };
		{ WHIT.."level 54 while doing kim'jael indeed! and " };
		{ WHIT.."betrayed (turn this in at 59.31. accept and do " };
		{ WHIT.."the next part to this quest)(this is all done at" };
		{ WHIT.."the thalassian base camp, 56.28)" };
		{ "" };
		{ WHIT.."07) i grind blood elves, cause their squishy " };
		{ WHIT.."mobs (mobs that have 30% fewer health/armor, " };
		{ WHIT.."than the average mob that level). i used to " };
		{ WHIT.."grind satyrs on previous speed runs, i was able " };
		{ WHIT.."to cut off a whole hour with the blood elves." };
		{ "" };
		{ WHIT.."08) optional: instead of the grinding you could " };
		{ WHIT.."go do brd instance instead." };
		{ "" };
		{ WHIT.."09) once you turn level 54 and those 3 quests " };
		{ WHIT.."are done, go turn in kim'jael indeed!. accept" };
		{ WHIT.."kim'jael's missing equipment." };
		{ "" };
		{ WHIT.."10) turn in courser antlers. i skip " };
		{ WHIT.."wavethrashing (42.42)" };
		{ "" };
		{ WHIT.."11) then go do kim'jael's missing equipment" };
		{ "" };
		{ WHIT.."12) go turn in kim'jael's missing equipment" };
		{ "" };
		{ WHIT.."13) turn in betrayed. accept betrayed (22.51)" };
		{ "" };
		{ WHIT.."14) hearth to orgrimmar." };
		{ "" };
		{ WHIT.."15) i then go turn in bone-bladed weapons and " };
		{ WHIT.."betrayed" };
		{ "" };
		{ WHIT.."16) then fly to splintertree post (ashenvale)." };
		{ "" };
		{ WHIT.."17) then go into felwood."		 };
	};
	HordeLeveling44 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (54-54)" };
		Location = { AtlasLocale["Felwood"] };
		{ WHIT.."01) accept timbermaw ally (at 50.85)  just talk " };
		{ WHIT.."to the npc there he will give it to you." };
		{ "" };
		{ WHIT.."02) go accept forces of jaedenar and verifying " };
		{ WHIT.."the corruption (at the emerald sanctuary 51.82). " };
		{ WHIT.."get the fp there too." };
		{ "" };
		{ WHIT.."03) go accept cleansing felwood (at 46.84)" };
		{ "" };
		{ WHIT.."04) start killing oozes, make sure to kill about" };
		{ WHIT.."30 oozes here total for the quest a sample of " };
		{ WHIT.."slime." };
		{ "" };
		{ WHIT.."05) then go complete forces of jaedenar (38.60)" };
		{ "" };
		{ WHIT.."06) then go to bloodvenom post. (34.52)" };
		{ "" };
		{ WHIT.."07) accept: well of corruption, a husband's last" };
		{ WHIT.."battle and wild guardians" };
		{ "" };
		{ WHIT.."08) get fp there." };
		{ "" };
		{ WHIT.."09) then go do in the following order:" };
		{ "" };
		{ WHIT.."10) verifying the corruption (41.41)" };
		{ "" };
		{ WHIT.."11) cleansing felwood (54.16)" };
		{ "" };
		{ WHIT.."12) go accept deadwood of the north (at 64.8) " };
		{ WHIT.."but don't do it now." };
		{ "" };
		{ WHIT.."13) grind your way through the cave to go to " };
		{ WHIT.."winterspring."		 };
	};
	HordeLeveling45 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (54-55)" };
		Location = { AtlasLocale["Winterspring"] };
		{ WHIT.."01) exit the cave and accept winterfall activity" };
		{ WHIT.."right outside of it." };
		{ "" };
		{ WHIT.."02) go to donova snowden (at 31.45)" };
		{ "" };
		{ WHIT.."03) turn in the new springs and it's a secret to" };
		{ WHIT.."everybody. accept strange sources and" };
		{ WHIT.."threat of the winterfall." };
		{ "" };
		{ WHIT.."04) note: i skip the videre elixir and the rest" };
		{ WHIT.."of that chain." };
		{ "" };
		{ WHIT.."05) go to everlook. (61.38)" };
		{ "" };
		{ WHIT.."06) accept: are we there, yeti?, the everlook" };
		{ WHIT.."report, duke nicholas zverenhoff, sister pamela," };
		{ WHIT.."and ursius of the shardtooth." };
		{ "" };
		{ WHIT.."07) turn in felnok steelspring. accept chillwind" };
		{ WHIT.."horns." };
		{ "" };
		{ WHIT.."08) make everlook your home." };
		{ "" };
		{ WHIT.."09) n/a" };
		{ "" };
		{ WHIT.."10) go do ursius of the shardtooth (just keep " };
		{ WHIT.."grinding mobs on the hill north of everlook till" };
		{ WHIT.."ursius shows up)" };
		{ "" };
		{ WHIT.."11) then turn in ursius of the shardtooth. " };
		{ WHIT.."accept brumeran of the chillwind" };
		{ "" };
		{ WHIT.."12) go do brumeran of the chillwind (he patrols " };
		{ WHIT.."around 60.60, see map for complete patrol path)" };
		{ "" };
		{ WHIT.."13) then go discover darkwhisper gorge " };
		{ WHIT.."(at 59.73) for the quest strange sources" };
		{ "" };
		{ WHIT.."14) then hearth back to everlook." };
		{ "" };
		{ WHIT.."15) turn in brumeran of the chillwind. " };
		{ WHIT.."skip shy-rotam" };
		{ "" };
		{ WHIT.."16) then go do the following quests (steps 17-20" };
		{ WHIT.."in any order) to the west and some to the north:" };
		{ "" };
		{ WHIT.."17) are we there, yeti?" };
		{ "" };
		{ WHIT.."18) threat of the winterfall you should find " };
		{ WHIT.."empty firewater flask which starts winterfall " };
		{ WHIT.."firewater while doing this quest. when the " };
		{ WHIT.."winterfall ritual totem item drops, save it and " };
		{ WHIT.."put it in your bank you will need it later." };
		{ "" };
		{ WHIT.."19) wild guardians (the ragged owls are west of " };
		{ WHIT.."everlook, and the raging owls are just north)" };
		{ "" };
		{ WHIT.."20) chillwind horns (you can find a larger " };
		{ WHIT.."concentration of chimaera around 66.29)" };
		{ "" };
		{ WHIT.."21) once threat of the winterfall and winterfall" };
		{ WHIT.."firewater is completed go turn them in along " };
		{ WHIT.."with strange sources (at 31.45). accept falling" };
		{ WHIT.."to corruption"		 };
		{ "" };
		{ WHIT.."22) once the rest of those quests are complete, " };
		{ WHIT.."go back to everlook and turn in are we there, " };
		{ WHIT.."yeti?. accept are we there, yeti? part2. turn in" };
		{ WHIT.."chillwind horns. i skip return to tinkee" };
		{ "" };
		{ WHIT.."23) then go do:" };
		{ "" };
		{ WHIT.."24) winterfall activity (67.37)" };
		{ "" };
		{ WHIT.."25) are we there, yeti? (find 2 pristine yeti " };
		{ WHIT.."horns) (66.43)" };
		{ "" };
		{ WHIT.."26) then hearth to everlook." };
		{ "" };
		{ WHIT.."27) go turn in are we there, yeti?. accept are " };
		{ WHIT.."we there, yeti? part3" };
		{ "" };
		{ WHIT.."28) then fly to felwood (bloodvenom post)."		 };
	};
	HordeLeveling46 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (55-55)" };
		Location = { AtlasLocale["Felwood"] };
		{ WHIT.."01) turn in wild guardians. " };
		{ WHIT.."skip wild guardians part2" };
		{ "" };
		{ WHIT.."02) fly to emerald sanctuary (in felwood)." };
		{ "" };
		{ WHIT.."03) turn in verifying the corruption and forces " };
		{ WHIT.."of jaedenar. accept collection of the corrupt " };
		{ WHIT.."water"  };
		{ "" };
		{ WHIT.."04) go turn in cleansing felwood. then get a " };
		{ WHIT.."cenarion beacon for the quest salve via hunting"  };
		{ WHIT.."(just talk to the npc to get it) (46.83)" };
		{ "" };
		{ WHIT.."05) go do a husband's last battle along with " };
		{ WHIT.."timbermaw ally (48.94)" };
		{ "" };
		{ WHIT.."06) once they are done go turn in timbermaw ally" };
		{ WHIT.."(at 50.85). accept speak to nafien" };
		{ "" };
		{ WHIT.."07) go do well of corruption (at 32.66).  " };
		{ WHIT.."also make sure you collect 6 corrupted soul " };
		{ WHIT.."shards for the quest salve via hunting" };
		{ "" };
		{ WHIT.."08) collection of the corrupt water (35.59)" };
		{ "" };
		{ WHIT.."09) then go to bloodvenom post." };
		{ "" };
		{ WHIT.."10) turn in a husband's last battle and well of " };
		{ WHIT.."corruption. accept corrupted sabers" };
		{ "" };
		{ WHIT.."11) fly to emerald sanctuary." };
		{ "" };
		{ WHIT.."12) turn in collection of the corrupt water. i " };
		{ WHIT.."skip seeking spiritual aid (51.82)" };
		{ "" };
		{ WHIT.."13) go turn in salve via hunting " };
		{ WHIT.."(the 6 corrupted soul shards) (46.83)" };
		{ "" };
		{ WHIT.."14) go do corrupted sabers (32.66) then go turn " };
		{ WHIT.."in.(note: when you turn it in, you have to talk"  };
		{ WHIT.."to the cat first once you get to the npc)" };
		{ "" };
		{ WHIT.."15) then go up north and do:" };
		{ "" };
		{ WHIT.."16) deadwood of the north (61.9) when the " };
		{ WHIT.."deadwood ritual totem item drops, save it and " };
		{ WHIT.."put it in your bank when you get a chance, you " };
		{ WHIT.."will need it later." };
		{ "" };
		{ WHIT.."17) falling to corruption (the cauldron at 60.5)" };
		{ WHIT.."(hint: i have my pet distract the mobs, while " };
		{ WHIT.."doing this quest). then accept mystery goo" };
		{ "" };
		{ WHIT.."18) go turn in deadwood of the north (64.8). " };
		{ WHIT.."accept speak to salfa.  and turn in speak to " };
		{ WHIT.."nafien" };
		{ "" };
		{ WHIT.."19) then run through the cave (you should be " };
		{ WHIT.."unfriendly with timbermaw, so they shouldn't " };
		{ WHIT.."attack you)." };
		{ "" };
		{ WHIT.."20) turn in winterfall activity and speak to " };
		{ WHIT.."salfa" };
		{ "" };
		{ WHIT.."21) then go turn in mystery goo (at 31.45 in " };
		{ WHIT.."winterspring). skip toxic horrors" };
		{ "" };
		{ WHIT.."22) then hearth to everlook."		 };
		{ "" };
		{ WHIT.."23) fly to orgrimmar." };
		{ "" };
		{ WHIT.."24) if you happen to see warcaller gorlach, " };
		{ WHIT.."accept a call to arms: the plaguelands!" };
		{ "" };
		{ WHIT.."25) go to the undercity." };
		{ "" };
		{ WHIT.."26) go complete a sample of slime and a batch of" };
		{ WHIT.."ooze" };
		{ "" };
		{ WHIT.."27) go accept a call to arms: the plaguelands! " };
		{ WHIT.."(this same quest can also get accepted in " };
		{ WHIT.."orgrimmar)" };
		{ "" };
		{ WHIT.."28) make undercity your home." };
		{ "" };
		{ WHIT.."29) go to the bulwark. (which is a small village" };
		{ WHIT.."east of tirisfal glades)" };
	};
	HordeLeveling47 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (55-56)" };
		Location = { AtlasLocale["Silithus"] };
		{ WHIT.."here are the prerequisites to this section:" };
		{ WHIT.."accept: are we there, yeti? part3 (in everlook, " };
		{ WHIT.."winterspring) accept: taking back silithus in " };
		{ WHIT.."orgrimmar." };
		{ "" };
		{ WHIT.."01) accept: report to general kirika, the " };
		{ WHIT.."twilight mystery, deadly desert venom and " };
		{ WHIT.."wanted - deathclasp, terror of the sands " };
		{ WHIT.."(wanted poster).  turn in taking back silithus. " };
		{ WHIT.."accept securing the supply lines" };
		{ "" };
		{ WHIT.."02) go do: securing the supply lines and deadly " };
		{ WHIT.."desert venom (do these 2 quests ne of cenarion " };
		{ WHIT.."hold)" };
		{ "" };
		{ WHIT.."03) then turn them in and accept new quests." };
		{ "" };
		{ WHIT.."04) then go do:" };
		{ "" };
		{ WHIT.."05) stepping up security (mobs are all over to " };
		{ WHIT.."the west)" };
		{ "" };
		{ WHIT.."06) the twilight mystery (around 21.17)" };
		{ "" };
		{ WHIT.."07) go turn in report to general kirika at " };
		{ WHIT.."50.69. accept scouring the desert" };
		{ "" };
		{ WHIT.."08) go do noggle's last hope along with scouring" };
		{ WHIT.."the desert once you find the silithyst item " };
		{ WHIT.."which looks like a glowing red thing, bring it " };
		{ WHIT.."back to the pvp horde base and stand in the " };
		{ WHIT.."teleporter looking thing, then turn the quest in" };
		{ WHIT.."for 6,600 xp!" };
		{ "" };
		{ WHIT.."09) wanted - deathclasp, terror of the sands " };
		{ WHIT.."(elite) (40.90) this quest is very easy to solo " };
		{ WHIT.."at lvl 55. if your class has troubles, either " };
		{ WHIT.."find a group to help, or you can simply skip it." };
		{ "" };
		{ WHIT.."10) once all of those are done, go turn them in " };
		{ WHIT.."at cenarion hold, accept new ones." };
		{ "" };
		{ WHIT.."11) go do: noggle's lost satchel (it's at around" };
		{ WHIT.."44.91)" };
		{ "" };
		{ WHIT.."12) the deserter (he's in the cave at 67.71). " };
		{ WHIT.."i skip the twilight lexicon" };
		{ "" };
		{ WHIT.."13) then i die on purpose, so i end up at " };
		{ WHIT.."cenarion hold." };
		{ "" };
		{ WHIT.."14) turn in noggle's lost satchel" };
		{ "" };
		{ WHIT.."15) then fly to marshal's refuge in un'goro " };
		{ WHIT.."crater." };
		{ "" };
		{ WHIT.."16) go scare quixxil with the mechanical yeti " };
		{ WHIT.."for the quest are we there, yeti? part3" };
		{ "" };
		{ WHIT.."17) then fly to tanaris." };
		{ "" };
		{ WHIT.."18) go scare sprinkle with the mechanical yeti " };
		{ WHIT.."for the quest are we there, yeti? part3" };
		{ "" };
		{ WHIT.."19) then hearth to orgrimmar." };
		{ "" };
		{ WHIT.."20) go to the undercity."		 };
		{ "" };
		{ WHIT.."21) go complete a sample of slime and a batch " };
		{ WHIT.."of ooze." };
		{ "" };
		{ WHIT.."22) go accept a call to arms: the plaguelands! " };
		{ WHIT.."(this same quest can also get accepted in " };
		{ WHIT.."orgrimmar)" };
		{ "" };
		{ WHIT.."23) make undercity your home." };
	};
	HordeLeveling48 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (56-56)" };
		Location = { AtlasLocale["Western Plaguelands"] };
		{ WHIT.."01) turn in a call to arms: the plaguelands! " };
		{ WHIT.."accept scarlet diversions." };
		{ "" };
		{ WHIT.."02) (make sure you get a flame in a bottle first" };
		{ WHIT.."before doing this quest, it's in the box)" };
		{ "" };
		{ WHIT.."03) turn in the everlook report." };
		{ "" };
		{ WHIT.."04) accept and complete argent dawn commission." };
		{ "" };
		{ WHIT.."05) accept the so-called mark of the " };
		{ WHIT.."lightbringer and a plague upon thee." };
		{ "" };
		{ WHIT.."06) go into the western plaguelands (felstone " };
		{ WHIT.."field at 37.56) and" };
		{ "" };
		{ WHIT.."07) accept better late than never (the lady in " };
		{ WHIT.."the house)" };
		{ "" };
		{ WHIT.."08) then go to the barn next door, and" };
		{ "" };
		{ WHIT.."09) complete better late than never. click on " };
		{ WHIT.."the chest again to accept better late than " };
		{ WHIT.."never again." };
		{ "" };
		{ WHIT.."10) go do scarlet diversions (burn the tent " };
		{ WHIT.."down, and plant the banner, 40.51)" };
		{ "" };
		{ WHIT.."11) go back to the bulwark. (note: make sure you" };
		{ WHIT.."save all bone fragments you find, you will need " };
		{ WHIT.."30 of them for a later quest)" };
		{ "" };
		{ WHIT.."12) turn in scarlet diversions. accept all along" };
		{ WHIT.."the watchtowers and the scourge cauldrons" };
		{ "" };
		{ WHIT.."13) turn in the scourge cauldrons. accept " };
		{ WHIT.."target: felstone field" };
		{ "" };
		{ WHIT.."14) go complete the whole cauldron quest chain, " };
		{ WHIT.."you need to kill the cauldron lord at each " };
		{ WHIT.."targeted field (listed below), grab their key " };
		{ WHIT.."and use it at the big cauldron in the middle of " };
		{ WHIT.."the field. (just keep going back and forth from " };
		{ WHIT.."the bulwark to the next field completing each " };
		{ WHIT.."one, turning them in and then accepting and " };
		{ WHIT.."doing the next one)." };
		{ "" };
		{ WHIT.."15) target: felstone field (37.56)" };
		{ "" };
		{ WHIT.."16) target: dalson's tears (46.52). note: while " };
		{ WHIT.."you are here, there is a small quest chain here" };
		{ WHIT.."that starts with a little diary in the barn " };
		{ WHIT.."(47.50) called mrs. dalson's diary, just click " };
		{ WHIT.."that to complete the quest.  now, after you did" };
		{ WHIT.."that, then you need to kill a wandering skeleton" };
		{ WHIT.."to get a key that opens the outhouse behind the " };
		{ WHIT.."barn. after you get the key and open the " };
		{ WHIT.."outhouse, it will release a farmer dalson, a " };
		{ WHIT.."level 56 undead. kill him, grab that key, and " };
		{ WHIT.."then run in the house (that is right next to the" };
		{ WHIT.."barn) and go up the stairs, there you will find" };
		{ WHIT.."a cabinet, open the cabinet with that key (for " };
		{ WHIT.."the quest locked away) and finally it's over." };
		{ WHIT.."easy 10200 xp!  or was it?"		 };
		{ "" };
		{ WHIT.."17) target: writhing haunt (53.65) (while you're" };
		{ WHIT.."there accept the wildlife suffers too (at 53.64," };
		{ WHIT.."the almost dead tauren in the hut)" };
		{ "" };
		{ WHIT.."18) target: gahrron's withering (62.59)(once you" };
		{ WHIT.."turn in the key and accept return to the bulwark"  };
		{ WHIT.."then just go directly into eastern plaguelands " };
		{ WHIT.."(without turning that quest in yet)."		 };
	};
	HordeLeveling49 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (56-57)" };
		Location = { AtlasLocale["Eastern Plaguelands"] };
		{ WHIT.."01) go accept: demon dogs, blood tinged skies " };
		{ WHIT.."and carrion grubbage (at 4.38)" };
		{ "" };
		{ WHIT.."02) grind your way to (23.68) and accept: to " };
		{ WHIT.."kill with purpose and un-life's little annoyances."  };
		{ WHIT.."note: i skip the ranger lord's behest." };
		{ "" };
		{ WHIT.."03) go down in the crypt at the undercroft " };
		{ WHIT.."(24.78) and accept hameya's plea (the scroll on " };
		{ WHIT.."the ground)." };
		{ "" };
		{ WHIT.."04) grind your way to darrowshire (32.83) while " };
		{ WHIT.."doing these quests:" };
		{ "" };
		{ WHIT.."05) demon dogs" };
		{ "" };
		{ WHIT.."06) blood tinged skies" };
		{ "" };
		{ WHIT.."07) carrion grubbage" };
		{ "" };
		{ WHIT.."08) then turn in sister pamela. accept and do " };
		{ WHIT.."pamela's doll (there's 3 parts of the doll)"   };
		{ "" };
		{ WHIT.."09) turn in pamela's doll. accept auntie marlene" };
		{ WHIT.."and uncle carlin" };
		{ "" };
		{ WHIT.."10) then go complete blood tinged skies" };
		{ "" };
		{ WHIT.."11) then make sure you kill 20 plaguehound runts" };
		{ WHIT.."for the quest demon dogs" };
		{ "" };
		{ WHIT.."12) then go to light's hope chapel (at 75.53)" };
		{ "" };
		{ WHIT.."13) accept: zaeldarr the outcast and " };
		{ WHIT.."the restless souls" };
		{ "" };
		{ WHIT.."14) turn in duke nicholas zverenhoff and " };
		{ WHIT.."uncle carlin. accept defenders of darrowshire" };
		{ "" };
		{ WHIT.."15) get fp there.  then go do:" };
		{ "" };
		{ WHIT.."16) to kill with purpose" };
		{ "" };
		{ WHIT.."17) defenders of darrowshire" };
		{ "" };
		{ WHIT.."18) hameya's plea (66.11)" };
		{ "" };
		{ WHIT.."19) demon dogs" };
		{ "" };
		{ WHIT.."20) carrion grubbage" };
		{ "" };
		{ WHIT.."21) un-life's little annoyances" };
		{ "" };
		{ WHIT.."22) a plague upon thee " };
		{ WHIT.."(done at plaguewood up north)" };
		{ "" };
		{ WHIT.."23) augustus' receipt book (get this in the hut " };
		{ WHIT.."at 13.34, the book is upstairs in the inn at " };
		{ WHIT.."12.27)" };
		{ "" };
		{ WHIT.."24) the restless souls (egan is in the hut at " };
		{ WHIT.."11.28)(the next part refers to stratholme," };
		{ WHIT.."which i skip)" };
		{ "" };
		{ WHIT.."25) then go turn in: augustus' receipt book."		 };
		{ "" };
		{ WHIT.."26) grind your way through the cave (12.24) " };
		{ WHIT.."(watch out it is not easy)" };
		{ WHIT.."note: if the cave is too hard for you, another " };
		{ WHIT.."easy alternative is to take the tower gryphon " };
		{ WHIT.."ride from the plaguewood tower (18.27) to the " };
		{ WHIT.."crown guard tower (35.68), as long as the horde " };
		{ WHIT.."has control over the towers that is. then turn " };
		{ WHIT.."in demon dogs, blood tinged skies and carrion " };
		{ WHIT.."grubbage  accept redemption. just talk to the " };
		{ WHIT.."guy to complete it. i skip of forgotten memories" };
		{ WHIT.."(you can try it if you find a group)" };
		{ "" };
		{ WHIT.."27) then go turn in: to kill with purpose and " };
		{ WHIT.."un-life's little annoyances (at 23.68)" };
		{ "" };
		{ WHIT.."28) then go do zaeldarr the outcast (24.78)" };
		{ "" };
		{ WHIT.."29) then go to the dirt mound behind the crypt " };
		{ WHIT.."and turn in hameya's plea" };
		{ "" };
		{ WHIT.."30) hearth to undercity." };
		{ "" };
		{ WHIT.."31) turn in better late than never. accept " };
		{ WHIT.."the jeremiah blues" };
		{ "" };
		{ WHIT.."32) turn in the jeremiah blues (just right " };
		{ WHIT.."underneath the bank). accept good luck charm" };
		{ "" };
		{ WHIT.."33) go back to the bulwark."		 };
	};
	HordeLeveling50 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (57-59)" };
		Location = { AtlasLocale["Western Plaguelands"] };
		{ WHIT.."01) turn in return to the bulwark and a plague " };
		{ WHIT.."upon thee. accept a plague upon thee part2. " };
		{ WHIT.."then go accept and complete mission accomplished!" };
		{ WHIT.."(right there at the bulwark)" };
		{ "" };
		{ WHIT.."02) go turn in good luck charm. accept two " };
		{ WHIT.."halves become one (the lady in the house (37.56)" };
		{ "" };
		{ WHIT.."03) do two halves become one (jabbering ghouls " };
		{ WHIT.."out in felstone field drop the item you need, " };
		{ WHIT.."once you get the item right-click it), then go " };
		{ WHIT.."back to the lady in the house to turn it in." };
		{ "" };
		{ WHIT.."04) then do a plague upon thee part2. accept " };
		{ WHIT.."a plague upon thee part3 (this is all done at " };
		{ WHIT.."48.31)" };
		{ "" };
		{ WHIT.."05) then go accept unfinished business part1 " };
		{ WHIT.."(at 51.28), then do it. (the mobs for this quest" };
		{ WHIT.."are at 50.42 and 53.44)" };
		{ "" };
		{ WHIT.."06) turn in unfinished business part1 (back at " };
		{ WHIT.."51.28). accept unfinished business part2, then " };
		{ WHIT.."go do it. the 2 mobs for this quest are at " };
		{ WHIT.."(57.37 and 54.24). while doing this quest make " };
		{ WHIT.."sure you do the so-called mark of the " };
		{ WHIT.."lightbringer along with it, it's up in the top " };
		{ WHIT.."of the tower at 54.23)" };
		{ "" };
		{ WHIT.."07) turn in unfinished business part2 (back at " };
		{ WHIT.."51.28). accept unfinished business part3, then " };
		{ WHIT.."go do it. need to go up in the tower (at 45.19) " };
		{ WHIT.."in a certain spot, watch out for elites."  };
		{ "" };
		{ WHIT.."08) then turn in unfinished business part3." };
		{ "" };
		{ WHIT.."09) then go complete and turn in the wildlife " };
		{ WHIT.."suffers too. accept the wildlife suffers too " };
		{ WHIT.."part2(53.64)" };
		{ "" };
		{ WHIT.."10) do the wildlife suffers too part2, then turn" };
		{ WHIT.."it in. skip glyphed oaken branch." };
		{ "" };
		{ WHIT.."11) go turn in auntie marlene (in the house at " };
		{ WHIT.."49.78). accept a strange historian" };
		{ "" };
		{ WHIT.."12) go get the ring for a strange historian (the" };
		{ WHIT.."ring is out in the graveyard)"   };
		{ "" };
		{ WHIT.."13) go into andorhal." };
		{ "" };
		{ WHIT.."14) do all along the watchtowers while working " };
		{ WHIT.."your way to the inn (at 39.66)" };
		{ "" };
		{ WHIT.."15) turn in a strange historian (at chromie up " };
		{ WHIT.."stairs in the inn at 39.68). accept"  };
		{ WHIT.."the annals of darrowshire and a matter of time." };
		{ "" };
		{ WHIT.."16) then go do (while grinding away at mobs):" };
		{ "" };
		{ WHIT.."17) all along the watchtowers" };
		{ "" };
		{ WHIT.."18) the annals of darrowshire the book for this " };
		{ WHIT.."is in the building at (42.67), the actual" };
		{ WHIT.."annals of darrowshire book looks different from " };
		{ WHIT.."the others, it has a lighter tint on the top " };
		{ WHIT.."portion of the pages in the book, unlike the " };
		{ WHIT.."rest which has a darker tint on the top portion " };
		{ WHIT.."of the pages." 		 };
		{ "" };
		{ WHIT.."19) a matter of time (47.62)" };
		{ "" };
		{ WHIT.."20) go turn in a matter of time and the annals " };
		{ WHIT.."of darrowshire. accept counting out time and"  };
		{ WHIT.."brother carlin." };
		{ "" };
		{ WHIT.."21) go do counting out time then turn it in. " };
		{ WHIT.."(it's repeatable but you only get xp once from it)" };
		{ "" };
		{ WHIT.."22) then go back to the bulwark." };
		{ "" };
		{ WHIT.."23) turn in a plague upon thee part3 and the " };
		{ WHIT.."so-called mark of the lightbringer. accept" };
		{ WHIT.."defiling uther's tomb." };
		{ "" };
		{ WHIT.."24) turn in all along the watchtowers. " };
		{ WHIT.."accept scholomance." };
		{ "" };
		{ WHIT.."25) turn in scholomance at the undead guy right " };
		{ WHIT.."next to him. accept skeletal fragments" };
		{ "" };
		{ WHIT.."26) stock up on enriched manna biscuits (which " };
		{ WHIT.."replenishes mana faster from drinking)" };
		{ "" };
		{ WHIT.."27) i then go back to andorhal, and do skeletal " };
		{ WHIT.."fragments." };
		{ "" };
		{ WHIT.."28) stop at uther's tomb (51.82) and do defiling" };
		{ WHIT.."uther's tomb (you need to equip the quest item" };
		{ WHIT.."to use it at the tomb)." };
		{ "" };
		{ WHIT.."29) as you hit  lvl 58 and skeletal fragments " };
		{ WHIT.."and defiling uther's tomb are done and you also " };
		{ WHIT.."have 30 bone fragments (please note, if you are " };
		{ WHIT.."far off from the 30 bone fragments, then you can" };
		{ WHIT.."just skip it), then go back to the bulwark and " };
		{ WHIT.."turn in defiling uther's tomb and skeletal " };
		{ WHIT.."fragments i skip mold rhymes with." };
		{ "" };
		{ WHIT.."30) i turn in any scourge stones, i have." };
		{ "" };
		{ WHIT.."31) i then stock up on enriched manna biscuits." };
		{ "" };
		{ WHIT.."32) go to undercity and then fly to light's hope" };
		{ WHIT.."chapel, eastern plaguelands." };
		{ "" };
		{ WHIT.."33) turn in zaeldarr the outcast, defenders of " };
		{ WHIT.."darrowshire and brother carlin." };
		{ WHIT.."i skip villains of darrowshire." };
		{ "" };
		{ WHIT.."34) accept and turn in bonescythe digs." };
		{ "" };
		{ WHIT.."35) then fly to hammerfall (arathi highlands)" };
		{ "" };
		{ WHIT.."36) build up first aid, make sure you get heavy " };
		{ WHIT.."runecloth bandage." };
		{ "" };
		{ WHIT.."37) then hearth to undercity."		 };
	};
	HordeLeveling51 = {
		ZoneName = { AtlasLocale["Horde Leveling Guide"].." (59-60)" };
		Location = { AtlasLocale["Winterspring"] };
		{ WHIT.."01) first thing to do here is stable your pet." };
		{ "" };
		{ WHIT.."02) then make everlook your home." };
		{ "" };
		{ WHIT.."03) turn in are we there, yeti? part3" };
		{ "" };
		{ WHIT.."04) accept: luck be with you (elite) and ursius" };
		{ WHIT.."of the shardtooth (elite)" };
		{ "" };
		{ WHIT.."05) then go tame a winterspring screecher " };
		{ WHIT.."(for claw rank8) (head south to do this)" };
		{ "" };
		{ WHIT.."06) then do luck be with you (use your pet to " };
		{ WHIT.."distract the giants while picking up the " };
		{ WHIT.."frostmaul shards)" };
		{ "" };
		{ WHIT.."07) then hearth to everlook." };
		{ "" };
		{ WHIT.."08) go do ursius of the shardtooth(elite) " };
		{ WHIT.."(just grind mobs on the hill north of everlook " };
		{ WHIT.."till ursius shows)" };
		{ "" };
		{ WHIT.."09) go turn in luck be with you and ursius of " };
		{ WHIT.."the shardtooth. accept brumeran of the " };
		{ WHIT.."chillwind(elite)" };
		{ "" };
		{ WHIT.."10) then go south and do:" };
		{ "" };
		{ WHIT.."11) brumeran of the chillwind (elite)" };
		{ "" };
		{ WHIT.."12) luck be with you (elite) (if you still need " };
		{ WHIT.."to finish it up)" };
		{ "" };
		{ WHIT.."13) wild guardians part2 (grind away at owls at " };
		{ WHIT.."the owl wing thicket for a few hours, before" };
		{ WHIT.."leaving here)" };
		{ "" };
		{ WHIT.."14) then when a few bars away from lvl 59, go " };
		{ WHIT.."back up to everlook." };
		{ "" };
		{ WHIT.."15) turn in luck be with you and brumeran of the" };
		{ WHIT.."chillwind. skip shy-rotam(elite)" };
		{ "" };
		{ WHIT.."16) fly to bloodvenom post (felwood)." };
		{ "" };
		{ WHIT.."17) turn in wild guardians part2. accept wild " };
		{ WHIT.."guardians part3" };
		{ "" };
		{ WHIT.."18) then hearth back to everlook." };
		{ "" };
		{ WHIT.."19) then go down to the owl wing thicket and do " };
		{ WHIT.."wild guardians part3" };
		{ "" };
		{ WHIT.."20) keep grinding away at owls till about " };
		{ WHIT.."lvl 59 1/2, then" };
		{ "" };
		{ WHIT.."21) die on purpose, to end up at everlook. then " };
		{ WHIT.."fly to bloodvenom post (felwood)." };
		{ "" };
		{ WHIT.."22) turn in wild guardians part3 and guarding " };
		{ WHIT.."secrets. accept guarding secrets part2" };
		{ "" };
		{ WHIT.."23) i then fly to xrs, then to thunderbluff, " };
		{ WHIT.."go to elder rise." };
		{ "" };
		{ WHIT.."24) turn in guarding secrets part2 and glyphed " };
		{ WHIT.."oaken branch (both at nara wildmane)" };
		{ "" };
		{ WHIT.."25) hearth back to everlook. at this point " };
		{ WHIT.."should be about 3/4 way to 60." };
		{ "" };
		{ WHIT.."26) i then go back down to the owl wing thicket " };
		{ WHIT.."and grind to 60." };
		{ "" };
		{ WHIT.."(see end game credits)" };
	};
};