--[[
Name: NPCDB-1.0
Revision: $Rev: 17545 $
Author(s): Shino (Geigerkind)
Website: http://legacy-logs/
Description: A library to provide localizations for relevant npcs.
Dependencies: AceLibrary, AceLocale-2.3
]]

local MAJOR_VERSION = "NPCDB-1.0"
local MINOR_VERSION = tonumber(string.sub("$Revision: 17545 $", 12, -3))

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary") end

if not AceLibrary:HasInstance("AceLocale-2.3") then error(MAJOR_VERSION .. " requires AceLocale-2.3") end

local _, x = AceLibrary("AceLocale-2.3"):GetLibraryVersion()
MINOR_VERSION = MINOR_VERSION * 100000 + x

if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

local NPCDB = AceLibrary("AceLocale-2.3"):new(MAJOR_VERSION)

local npcs = {
	["Waypoint"] = true,
	["Spawnpoint"] = true,
	["Stormwind City Guard"] = true,
	["Infernal"] = true,
	["Blue Dragonspawn"] = true,
	["Earth Elemental"] = true,
	["Maginor Dumas"] = true,
	["Master Mathias Shaw"] = true,
	["Kendor Kabonka"] = true,
	["Grizzle Halfmane"] = true,
	["Dungar Longdrink"] = true,
	["High Priestess Laurena"] = true,
	["Felhunter"] = true,
	["Demisette Cloyce"] = true,
	["General Marcus Jonathan"] = true,
	["Lieutenant Doren"] = true,
	["Water Elemental"] = true,
	["Thor"] = true,
	["Grimtooth"] = true,
	["El Pollo Grande"] = true,
	["Mai'Zoth"] = true,
	["Coldridge Mountaineer"] = true,
	["Donal Osgood"] = true,
	["Stonard Hunter"] = true,
	["Stonard Orc"] = true,
	["Stonard Wayfinder"] = true,
	["Stonard Grunt"] = true,
	["Stonard Cartographer"] = true,
	["Stonard Shaman"] = true,
	["Keras Wolfheart"] = true,
	["Ander Germaine"] = true,
	["Osborne the Night Man"] = true,
	["Lord Grayson Shadowbreaker"] = true,
	["Ariena Stormfeather"] = true,
	["Swamp Talker"] = true,
	["Brother Neals"] = true,
	["Malosh"] = true,
	["Haromm"] = true,
	["Ogromm"] = true,
	["Kartosh"] = true,
	["Banalash"] = true,
	["Red Dragonspawn"] = true,
	["Red Wyrmkin"] = true,
	["Red Scalebane"] = true,
	["Scalebane Lieutenant"] = true,
	["Wyrmkin Firebrand"] = true,
	["Scalebane Royal Guard"] = true,
	["Jade"] = true,
	["Grom'gol Grunt"] = true,
	["Watcher Merant"] = true,
	["Angus Stern"] = true,
	["Senator Barin Redstone"] = true,
	["Archbishop Benedictus"] = true,
	["Felder Stover"] = true,
	["Charys Yserian"] = true,
	["Adair Gilroy"] = true,
	["Elly Langston"] = true,
	["Mayda Thane"] = true,
	["Georgio Bolero"] = true,
	["Rogvar"] = true,
	["Thysta"] = true,
	["Ian Strom"] = true,
	["Grimand Elmore"] = true,
	["Zardeth of the Black Claw"] = true,
	["Helgrum the Swift"] = true,
	["Fel'zerul"] = true,
	["Jannos Ironwill"] = true,
	["Morgg Stormshot"] = true,
	["Menethil Guard"] = true,
	["Mok'rash"] = true,
	["Negolash"] = true,
	["Gunther Arcanus"] = true,
	["Bethor Iceshard"] = true,
	["King Mukla"] = true,
	["Shellei Brondir"] = true,
	["Thorgrum Borrelson"] = true,
	["Gryth Thurden"] = true,
	["Northshire Guard"] = true,
	["Deathguard Randolph"] = true,
	["Deathguard Oliver"] = true,
	["Deathguard Phillip"] = true,
	["Deathguard Bartrand"] = true,
	["Highlord Bolvar Fordragon"] = true,
	["Lady Katrana Prestor"] = true,
	["Grand Admiral Jes-Tereth"] = true,
	["Mithras Ironhill"] = true,
	["Caledra Dawnbreeze"] = true,
	["Stormwind Royal Guard"] = true,
	["Zun'dartha"] = true,
	["Skeletal Flayer"] = true,
	["Skeletal Sorcerer"] = true,
	["Skeletal Terror"] = true,
	["Skeletal Executioner"] = true,
	["Skeletal Warlord"] = true,
	["Skeletal Acolyte"] = true,
	["Slavering Ghoul"] = true,
	["Rotting Ghoul"] = true,
	["Soulless Ghoul"] = true,
	["Searing Ghoul"] = true,
	["Freezing Ghoul"] = true,
	["Cold Wraith"] = true,
	["Blood Wraith"] = true,
	["Hungering Wraith"] = true,
	["Wailing Death"] = true,
	["Flesh Golem"] = true,
	["Vile Slime"] = true,
	["Devouring Ooze"] = true,
	["Carrion Vulture"] = true,
	["Rotting Behemoth"] = true,
	["Decaying Horror"] = true,
	["Diseased Black Bear"] = true,
	["Diseased Grizzly"] = true,
	["Diseased Wolf"] = true,
	["Carrion Lurker"] = true,
	["Venom Mist Lurker"] = true,
	["Plague Lurker"] = true,
	["Scarlet Mage"] = true,
	["Scarlet Sentinel"] = true,
	["Scarlet Hunter"] = true,
	["Scarlet Magus"] = true,
	["Scarlet Knight"] = true,
	["Scarlet Paladin"] = true,
	["Scarlet Invoker"] = true,
	["Scarlet Cavalier"] = true,
	["Scarlet Judge"] = true,
	["Scarlet Interrogator"] = true,
	["Scarlet High Clerist"] = true,
	["Grand Inquisitor Isillien"] = true,
	["Scarlet Executioner"] = true,
	["Highlord Taelan Fordring"] = true,
	["Foreman Jerris"] = true,
	["Foreman Marcrid"] = true,
	["High Protector Tarsen"] = true,
	["High Protector Lorik"] = true,
	["Foulmane"] = true,
	["Lord Maldazzar"] = true,
	["Putridius"] = true,
	["The Husk"] = true,
	["Araj the Summoner"] = true,
	["Darkmaster Gandling"] = true,
	["High Priest Thel'danis"] = true,
	["Tirion Fordring"] = true,
	["Scarlet Worker"] = true,
	["Scarlet Lumberjack"] = true,
	["Scarlet Smith"] = true,
	["Treant"] = true,
	["Stormwind City Patroller"] = true,
	["Senator Mehr Stonehallow"] = true,
	["Ancient Protector"] = true,
	["Master Apothecary Faranell"] = true,
	["Natheril Raincaller"] = true,
	["High Executor Darthalia"] = true,
	["Zora Guthrek"] = true,
	["Karos Razok"] = true,
	["Borgus Stoutarm"] = true,
	["Aethalas"] = true,
	["Lyranne Feathersong"] = true,
	["Southshore Guard"] = true,
	["Zarise"] = true,
	["Daryl Stack"] = true,
	["Tarren Mill Deathguard"] = true,
	["Felicia Maline"] = true,
	["Varimathras"] = true,
	["Darla Harris"] = true,
	["Major Samuelson"] = true,
	["Commander Aggro'gosh"] = true,
	["Bro'kin"] = true,
	["Larimaine Purdue"] = true,
	["Fleet Master Seahorn"] = true,
	["Baron Revilgaz"] = true,
	["Saltwater Snapjaw"] = true,
	["Skymane Gorilla"] = true,
	["Jaguero Stalker"] = true,
	["Zanzil the Outcast"] = true,
	["Catelyn the Blade"] = true,
	["Fleet Master Firallon"] = true,
	["Captain Keelhaul"] = true,
	["Hammerfall Guardian"] = true,
	["Sly Garrett"] = true,
	["Viznik Goldgrubber"] = true,
	["Vilebranch Witch Doctor"] = true,
	["Vilebranch Headhunter"] = true,
	["Vilebranch Shadowcaster"] = true,
	["Vilebranch Berserker"] = true,
	["Vilebranch Hideskinner"] = true,
	["Vilebranch Shadow Hunter"] = true,
	["Vilebranch Blood Drinker"] = true,
	["Vilebranch Soul Eater"] = true,
	["Vilebranch Aman'zasi Guard"] = true,
	["Green Sludge"] = true,
	["Jade Ooze"] = true,
	["Razorbeak Skylord"] = true,
	["Mechanical Dragonling"] = true,
	["Vilebranch Wolf Pup"] = true,
	["Vilebranch Raiding Wolf"] = true,
	["Ruppo Zipcoil"] = true,
	["Highvale Ranger"] = true,
	["Hanashi"] = true,
	["Shadra"] = true,
	["Archaedas"] = true,
	["Blacklash"] = true,
	["Hematus"] = true,
	["King Magni Bronzebeard"] = true,
	["Zaruk"] = true,
	["Susan Tillinghast"] = true,
	["Malygen"] = true,
	["Kurden Bloodclaw"] = true,
	["Bale"] = true,
	["Cedrik Prose"] = true,
	["Brikk Keencraft"] = true,
	["Jaxin Chong"] = true,
	["Crazk Sparks"] = true,
	["Hurklor"] = true,
	["Jansen Underwood"] = true,
	["Urda"] = true,
	["Gringer"] = true,
	["Gyll"] = true,
	["Gorrik"] = true,
	["Grunenstur Balindom"] = true,
	["Hurom Juggendolf"] = true,
	["Durdek Karrin"] = true,
	["Garek"] = true,
	["Silvermane Stalker"] = true,
	["Savage Owlbeast"] = true,
	["Zaricotl"] = true,
	["Lanie Reed"] = true,
	["Tal"] = true,
	["Una"] = true,
	["Siln Skychaser"] = true,
	["Beram Skychaser"] = true,
	["Turak Runetotem"] = true,
	["Sheal Runetotem"] = true,
	["Kary Thunderhorn"] = true,
	["Holt Thunderhorn"] = true,
	["Torm Ragetotem"] = true,
	["Sark Ragetotem"] = true,
	["Malakai Cross"] = true,
	["Father Cobb"] = true,
	["Archmage Shymm"] = true,
	["Thurston Xane"] = true,
	["Cairne Bloodhoof"] = true,
	["Honor Guard"] = true,
	["Bluffwatcher"] = true,
	["Unseen"] = true,
	["Brave Proudsnout"] = true,
	["Brave Lightninghorn"] = true,
	["Brave Running Wolf"] = true,
	["Brave Greathoof"] = true,
	["Nazgrel"] = true,
	["Orgrimmar Grunt"] = true,
	["Gabrielle Chase"] = true,
	["Grisha"] = true,
	["Doras"] = true,
	["Olvia"] = true,
	["Grol'dar"] = true,
	["Mirket"] = true,
	["Zevrost"] = true,
	["Ormok"] = true,
	["Sergra Darkthorn"] = true,
	["Grelkor"] = true,
	["Kardris Dreamseeker"] = true,
	["Ormak Grimshot"] = true,
	["Grezz Ragefist"] = true,
	["Sorek"] = true,
	["Gazlowe"] = true,
	["Shenthul"] = true,
	["Xor'juul"] = true,
	["Melor Stonehoof"] = true,
	["Ancient of Lore"] = true,
	["Ancient of War"] = true,
	["Tarhus"] = true,
	["Ratchet Bruiser"] = true,
	["Arch Druid Fandral Staghelm"] = true,
	["Teldrassil Sentinel"] = true,
	["Sewer Beast"] = true,
	["Aman"] = true,
	["Devrak"] = true,
	["Onu"] = true,
	["Kurll"] = true,
	["Rarck"] = true,
	["Vrang Wildgore"] = true,
	["Raene Wolfrunner"] = true,
	["Jadenvis Seawatcher"] = true,
	["Gruna"] = true,
	["Syurana"] = true,
	["Vesprystus"] = true,
	["Caylais Moonfeather"] = true,
	["Brakgul Deathbringer"] = true,
	["Healing Stream Totem IV"] = true,
	["Healing Stream Totem V"] = true,
	["Shandris Feathermoon"] = true,
	["Magatha Grimtotem"] = true,
	["Zor Lonetree"] = true,
	["Rat"] = true,
	["Arias'ta Bladesinger"] = true,
	["Elanaria"] = true,
	["Astarii Starseeker"] = true,
	["Jandria"] = true,
	["Jeen'ra Nightrunner"] = true,
	["Ainethil"] = true,
	["Lysheana"] = true,
	["Ealyshia Dewwhisper"] = true,
	["Geenia Sunshadow"] = true,
	["Dorion"] = true,
	["Telonis"] = true,
	["Erion Shadewhisper"] = true,
	["Anishar"] = true,
	["Mathrengyl Bearwalker"] = true,
	["Denatharion"] = true,
	["Frostsaber Companion"] = true,
	["Nightshade"] = true,
	["Brogus Thunderbrew"] = true,
	["Lana Thunderbrew"] = true,
	["Darnassus Sentinel"] = true,
	["Daelyshia"] = true,
	["Gorm Grimtotem"] = true,
	["Cor Grimtotem"] = true,
	["Tharm"] = true,
	["Gorkas"] = true,
	["Nyse"] = true,
	["Thyssiana"] = true,
	["Baldruc"] = true,
	["Strashaz Warrior"] = true,
	["Strashaz Serpent Guard"] = true,
	["Strashaz Myrmidon"] = true,
	["Strashaz Sorceress"] = true,
	["Strashaz Siren"] = true,
	["Strashaz Hydra"] = true,
	["Teloren"] = true,
	["Darnassian Protector"] = true,
	["Vilebranch Scalper"] = true,
	["Vilebranch Soothsayer"] = true,
	["Jade Sludge"] = true,
	["Emerald Ooze"] = true,
	["Haunting Vision"] = true,
	["Rotting Cadaver"] = true,
	["Blighted Zombie"] = true,
	["Scarlet Avenger"] = true,
	["Scarlet Spellbinder"] = true,
	["Michael Garrett"] = true,
	["Kaal Soulreaper"] = true,
	["Luther Pickman"] = true,
	["Kaelystia Hatebringer"] = true,
	["Pierce Shackleton"] = true,
	["Miles Dexter"] = true,
	["Gregory Charles"] = true,
	["Christoph Walker"] = true,
	["Angela Curthas"] = true,
	["Aelthalyste"] = true,
	["Father Lazarus"] = true,
	["Doctor Herbert Halsey"] = true,
	["Christopher Drakul"] = true,
	["Booty Bay Bruiser"] = true,
	["Kildar"] = true,
	["Truk Wildbeard"] = true,
	["Nazeer Bloodpike"] = true,
	["Stone Keeper"] = true,
	["Guard Edward"] = true,
	["Guard Jarad"] = true,
	["Combat Master Criton"] = true,
	["Captain Garran Vimes"] = true,
	["Theramore Lieutenant"] = true,
	["Thrall"] = true,
	["Theramore Practicing Guard"] = true,
	["Lady Jaina Proudmoore"] = true,
	["Guard Lasiter"] = true,
	["Aldwin Laughlin"] = true,
	["Theramore Guard"] = true,
	["Krumn"] = true,
	["Combat Master Szigeti"] = true,
	["Guard Kahil"] = true,
	["Guard Lana"] = true,
	["Guard Narrisha"] = true,
	["Guard Tark"] = true,
	["Captain Andrews"] = true,
	["Captain Thomas"] = true,
	["Daera Brightspear"] = true,
	["Olmin Burningbeard"] = true,
	["Brogun Stoneshield"] = true,
	["Jondor Steelbrow"] = true,
	["Jonivera Farmountain"] = true,
	["Svalbrad Farmountain"] = true,
	["Kurdrum Barleybeard"] = true,
	["Theodrus Frostbeard"] = true,
	["Braenna Flintcrag"] = true,
	["Juli Stormkettle"] = true,
	["Nittlebur Sparkfizzle"] = true,
	["Valgar Highforge"] = true,
	["Beldruk Doombrow"] = true,
	["Brandur Ironhammer"] = true,
	["Grumnus Steelshaper"] = true,
	["Hulfdan Blackbeard"] = true,
	["Ormyr Flinteye"] = true,
	["Briarthorn"] = true,
	["Alexander Calder"] = true,
	["Medic Tamberlyn"] = true,
	["Medic Helaina"] = true,
	["Murk Spitter"] = true,
	["Murk Worm"] = true,
	["Saturated Ooze"] = true,
	["Gordunni Warlord"] = true,
	["Atal'ai Warrior"] = true,
	["Atal'ai Witch Doctor"] = true,
	["Groddoc Thunderer"] = true,
	["Mummified Atal'ai"] = true,
	["Unliving Atal'ai"] = true,
	["Atal'ai Priest"] = true,
	["Atal'ai Corpse Eater"] = true,
	["Atal'ai Deathwalker"] = true,
	["Atal'ai High Priest"] = true,
	["Ironfur Patriarch"] = true,
	["Sprite Dragon"] = true,
	["Nightmare Scalebane"] = true,
	["Nightmare Wyrmkin"] = true,
	["Nightmare Wanderer"] = true,
	["Rabid Longtooth"] = true,
	["Hakkari Frostwing"] = true,
	["Rage Scar Yeti"] = true,
	["Elder Rage Scar"] = true,
	["Ferocious Rage Scar"] = true,
	["Frayfeather Patriarch"] = true,
	["Lethlas"] = true,
	["Phantim"] = true,
	["Jademir Oracle"] = true,
	["Jademir Tree Warder"] = true,
	["Jademir Boughguard"] = true,
	["Lady Szallah"] = true,
	["Diamond Head"] = true,
	["Bloodroar the Stalker"] = true,
	["Antilus the Soarer"] = true,
	["Arash-ethis"] = true,
	["Qirot"] = true,
	["Land Walker"] = true,
	["Cliff Giant"] = true,
	["Shore Strider"] = true,
	["Deep Strider"] = true,
	["Wave Strider"] = true,
	["Northspring Harpy"] = true,
	["Northspring Roguefeather"] = true,
	["Northspring Slayer"] = true,
	["Northspring Windcaller"] = true,
	["High Explorer Magellas"] = true,
	["Sage Palerunner"] = true,
	["Quartermaster Lungertz"] = true,
	["Veyzhak the Cannibal"] = true,
	["Zekkis"] = true,
	["Kazkaz the Unholy"] = true,
	["Glasshide Petrifier"] = true,
	["Scorpid Dunestalker"] = true,
	["Rabid Blisterpaw"] = true,
	["Searing Roc"] = true,
	["Surf Glider"] = true,
	["Giant Surf Glider"] = true,
	["Coral Shark"] = true,
	["Hazzali Wasp"] = true,
	["Hazzali Stinger"] = true,
	["Hazzali Swarmer"] = true,
	["Hazzali Worker"] = true,
	["Hazzali Tunneler"] = true,
	["Hazzali Sandreaver"] = true,
	["Centipaar Wasp"] = true,
	["Centipaar Stinger"] = true,
	["Centipaar Swarmer"] = true,
	["Centipaar Worker"] = true,
	["Centipaar Tunneler"] = true,
	["Centipaar Sandreaver"] = true,
	["Sea Elemental"] = true,
	["Sea Spray"] = true,
	["Coast Strider"] = true,
	["Deep Dweller"] = true,
	["Dune Smasher"] = true,
	["Raging Dune Smasher"] = true,
	["Dunemaul Enforcer"] = true,
	["Dunemaul Ogre Mage"] = true,
	["Dunemaul Brute"] = true,
	["Dunemaul Warlock"] = true,
	["Wu Shen"] = true,
	["Thistleshrub Dew Collector"] = true,
	["Thistleshrub Rootshaper"] = true,
	["Brother Joshua"] = true,
	["Gnarled Thistleshrub"] = true,
	["Katherine the Pure"] = true,
	["Sandahl"] = true,
	["Elsharin"] = true,
	["Kaerbrus"] = true,
	["Sheldras Moontree"] = true,
	["Theridran"] = true,
	["Einris Brightspear"] = true,
	["Ulfir Ironbeard"] = true,
	["Grunt Zuul"] = true,
	["Grunt Tharlak"] = true,
	["Bruuk Barleybeard"] = true,
	["Ironforge Guard"] = true,
	["Undercity Guardian"] = true,
	["Falstad Wildhammer"] = true,
	["Gunther's Visage"] = true,
	["Carendin Halgar"] = true,
	["High Sorcerer Andromath"] = true,
	["Spawn of Hakkar"] = true,
	["Shade of Eranikus"] = true,
	["Jammal'an the Prophet"] = true,
	["Ogom the Wretched"] = true,
	["Zolo"] = true,
	["Gasher"] = true,
	["Loro"] = true,
	["Hukku"] = true,
	["Zul'Lor"] = true,
	["Mijan"] = true,
	["Rothos"] = true,
	["Morphaz"] = true,
	["Weaver"] = true,
	["Dreamscythe"] = true,
	["Hazzas"] = true,
	["Guardian of Blizzard"] = true,
	["Arch Druid Hamuul Runetotem"] = true,
	["Nara Wildmane"] = true,
	["Crildor"] = true,
	["Margol the Rager"] = true,
	["Dark Iron Steamsmith"] = true,
	["Dark Iron Taskmaster"] = true,
	["Inferno Elemental"] = true,
	["Heavy War Golem"] = true,
	["Magma Elemental"] = true,
	["Greater Lava Spider"] = true,
	["Twilight Dark Shaman"] = true,
	["Twilight Fire Guard"] = true,
	["Twilight Geomancer"] = true,
	["Pephredo"] = true,
	["Enyo"] = true,
	["Tremor Totem"] = true,
	["Den Grunt"] = true,
	["Dreadmaul Ogre Mage"] = true,
	["Dreadmaul Brute"] = true,
	["Dreadmaul Mauler"] = true,
	["Dreadmaul Warlock"] = true,
	["Portal Seeker"] = true,
	["Black Slayer"] = true,
	["Bonepicker"] = true,
	["Snickerfang Hyena"] = true,
	["Scorpok Stinger"] = true,
	["Redstone Basilisk"] = true,
	["Redstone Crystalhide"] = true,
	["Ashmane Boar"] = true,
	["Helboar"] = true,
	["Zayus"] = true,
	["Nethergarde Miner"] = true,
	["Nethergarde Engineer"] = true,
	["Nethergarde Foreman"] = true,
	["Nethergarde Soldier"] = true,
	["Nethergarde Cleric"] = true,
	["Nethergarde Analyst"] = true,
	["Nethergarde Riftwatcher"] = true,
	["Nethergarde Officer"] = true,
	["Shadowsworn Cultist"] = true,
	["Shadowsworn Thug"] = true,
	["Shadowsworn Adept"] = true,
	["Shadowsworn Enforcer"] = true,
	["Shadowsworn Warlock"] = true,
	["Shadowsworn Dreadweaver"] = true,
	["Felhound"] = true,
	["Felguard Sentry"] = true,
	["Flametongue Totem II"] = true,
	["X'yera"] = true,
	["Elemental Protection Totem"] = true,
	["Breyk"] = true,
	["Earthgrab Totem"] = true,
	["Harry Burlguard"] = true,
	["Azuregos"] = true,
	["Muren Stormpike"] = true,
	["Highborne Lichling"] = true,
	["Varo'then's Ghost"] = true,
	["Gakin the Darkbinder"] = true,
	["Haldarr Felsworn"] = true,
	["Draconic Magelord"] = true,
	["Blue Scalebane"] = true,
	["Draconic Mageweaver"] = true,
	["Lord Arkkoroc"] = true,
	["Arkkoran Clacker"] = true,
	["Arkkoran Muckdweller"] = true,
	["Arkkoran Pincer"] = true,
	["Arkkoran Oracle"] = true,
	["Hetaera"] = true,
	["Servant of Arkkoroc"] = true,
	["Son of Arkkoroc"] = true,
	["Cliff Breaker"] = true,
	["Cliff Thunderer"] = true,
	["Cliff Walker"] = true,
	["Duthorian Rall"] = true,
	["Tiza Battleforge"] = true,
	["Timbermaw Pathfinder"] = true,
	["Timbermaw Warrior"] = true,
	["Timbermaw Totemic"] = true,
	["Timbermaw Den Watcher"] = true,
	["Timbermaw Shaman"] = true,
	["Timbermaw Ursa"] = true,
	["Spitelash Warrior"] = true,
	["Spitelash Screamer"] = true,
	["Spitelash Serpent Guard"] = true,
	["Spitelash Siren"] = true,
	["Spitelash Myrmidon"] = true,
	["Blood Elf Surveyor"] = true,
	["Blood Elf Reclaimer"] = true,
	["Legashi Satyr"] = true,
	["Legashi Rogue"] = true,
	["Legashi Hellcaller"] = true,
	["Klannoc Macleod"] = true,
	["Strahad Farsan"] = true,
	["Menara Voidrender"] = true,
	["Young Wavethrasher"] = true,
	["Wavethrasher"] = true,
	["Great Wavethrasher"] = true,
	["Makrinni Razorclaw"] = true,
	["Storm Bay Oracle"] = true,
	["Coralshell Lurker"] = true,
	["Coralshell Tortoise"] = true,
	["Makrinni Scrabbler"] = true,
	["Storm Bay Warrior"] = true,
	["Makrinni Snapclaw"] = true,
	["Thunderhead Hippogryph"] = true,
	["Thunderhead Stagwing"] = true,
	["Thunderhead Skystormer"] = true,
	["Thunderhead Patriarch"] = true,
	["Thunderhead Consort"] = true,
	["Jubahl Corpseseeker"] = true,
	["Ward of Zanzil"] = true,
	["Henen Ragetotem"] = true,
	["Sergeant Rutger"] = true,
	["Spirit Healer"] = true,
	["Devilsaur"] = true,
	["Ironhide Devilsaur"] = true,
	["Tyrant Devilsaur"] = true,
	["Stegodon"] = true,
	["Plated Stegodon"] = true,
	["Spiked Stegodon"] = true,
	["Thunderstomp Stegodon"] = true,
	["Ravasaur"] = true,
	["Ravasaur Runner"] = true,
	["Ravasaur Hunter"] = true,
	["Venomhide Ravasaur"] = true,
	["Bloodpetal Lasher"] = true,
	["Bloodpetal Flayer"] = true,
	["Bloodpetal Thresher"] = true,
	["Bloodpetal Trapper"] = true,
	["Un'Goro Stomper"] = true,
	["Un'Goro Gorilla"] = true,
	["Un'Goro Thunderer"] = true,
	["Tar Beast"] = true,
	["Tar Lurker"] = true,
	["Tar Lord"] = true,
	["Scorching Elemental"] = true,
	["Living Blaze"] = true,
	["Tar Creeper"] = true,
	["Tabetha"] = true,
	["Magus Tirth"] = true,
	["Gorishi Wasp"] = true,
	["Gorishi Worker"] = true,
	["Gorishi Reaver"] = true,
	["Gorishi Stinger"] = true,
	["Gorishi Tunneler"] = true,
	["Muculent Ooze"] = true,
	["Primal Ooze"] = true,
	["Glutinous Ooze"] = true,
	["Stone Guardian"] = true,
	["Vizzklick"] = true,
	["Ravasaur Matriarch"] = true,
	["Clutchmother Zavas"] = true,
	["Gruff"] = true,
	["King Mosh"] = true,
	["Uhk'loc"] = true,
	["Monnos the Elder"] = true,
	["Magister Hawkhelm"] = true,
	["Antilos"] = true,
	["Lady Sesspira"] = true,
	["General Fangferror"] = true,
	["Gatekeeper Rageroar"] = true,
	["Master Feardred"] = true,
	["Baritanas Skyriver"] = true,
	["Fahrad"] = true,
	["Thalon"] = true,
	["Ravenholdt Guard"] = true,
	["Lord Jorach Ravenholdt"] = true,
	["Ravenholdt Assassin"] = true,
	["Zan Shivsproket"] = true,
	["Innkeeper Skindle"] = true,
	["Talvash del Kissel"] = true,
	["Yalda"] = true,
	["Blackrock Soldier"] = true,
	["Blackrock Sorcerer"] = true,
	["Blackrock Slayer"] = true,
	["Blackrock Warlock"] = true,
	["Blackrock Battlemaster"] = true,
	["Obsidian Elemental"] = true,
	["Greater Obsidian Elemental"] = true,
	["Firegut Ogre"] = true,
	["Firegut Ogre Mage"] = true,
	["Firegut Brute"] = true,
	["Thaurissan Spy"] = true,
	["Thaurissan Firewalker"] = true,
	["Thaurissan Agent"] = true,
	["War Reaver"] = true,
	["Black Dragonspawn"] = true,
	["Black Wyrmkin"] = true,
	["Flamescale Dragonspawn"] = true,
	["Flamescale Wyrmkin"] = true,
	["Black Drake"] = true,
	["Scalding Drake"] = true,
	["Searscale Drake"] = true,
	["Black Broodling"] = true,
	["Scalding Broodling"] = true,
	["Flamescale Broodling"] = true,
	["Blackrock Worg"] = true,
	["Condemned Acolyte"] = true,
	["Condemned Monk"] = true,
	["Condemned Cleric"] = true,
	["Cursed Paladin"] = true,
	["Cursed Justicar"] = true,
	["Writhing Mage"] = true,
	["Cursed Ooze"] = true,
	["Tainted Ooze"] = true,
	["Vile Ooze"] = true,
	["Ironbeak Owl"] = true,
	["Ironbeak Screecher"] = true,
	["Ironbeak Hunter"] = true,
	["Warpwood Moss Flayer"] = true,
	["Warpwood Shredder"] = true,
	["Dessecus"] = true,
	["Jadefire Satyr"] = true,
	["Jadefire Rogue"] = true,
	["Jadefire Trickster"] = true,
	["Jadefire Betrayer"] = true,
	["Jadefire Felsworn"] = true,
	["Jadefire Shadowstalker"] = true,
	["Jadefire Hellcaller"] = true,
	["Jaedenar Cultist"] = true,
	["Jaedenar Guardian"] = true,
	["Jaedenar Enforcer"] = true,
	["Jaedenar Adept"] = true,
	["Jaedenar Darkweaver"] = true,
	["Jaedenar Warlock"] = true,
	["Jaedenar Hound"] = true,
	["Jaedenar Hunter"] = true,
	["Toxic Horror"] = true,
	["Infernal Bodyguard"] = true,
	["Infernal Sentry"] = true,
	["Immolatus"] = true,
	["Irontree Wanderer"] = true,
	["Irontree Stomper"] = true,
	["Withered Protector"] = true,
	["Deadwood Warrior"] = true,
	["Deadwood Gardener"] = true,
	["Deadwood Pathfinder"] = true,
	["Deadwood Den Watcher"] = true,
	["Deadwood Avenger"] = true,
	["Deadwood Shaman"] = true,
	["Lore Keeper of Norgannon"] = true,
	["Shayis Steelfury"] = true,
	["Kelgruk Bloodaxe"] = true,
	["Borgus Steelhand"] = true,
	["Ember"] = true,
	["Chief Ukorz Sandscalp"] = true,
	["Witch Doctor Zum'rah"] = true,
	["Gahz'rilla"] = true,
	["Sandfury Executioner"] = true,
	["Shadowpriest Sezz'ziz"] = true,
	["Darnath Bladesinger"] = true,
	["Winstone Wolfe"] = true,
	["Master Kang"] = true,
	["Kum'isha the Collector"] = true,
	["Flawless Draenethyst Sphere"] = true,
	["Stoneskin Totem VI"] = true,
	["Deadwind Brute"] = true,
	["Restless Shade"] = true,
	["Deadwind Mauler"] = true,
	["Deadwind Warlock"] = true,
	["Sky Shadow"] = true,
	["Deadwind Ogre Mage"] = true,
	["Stoneclaw Totem V"] = true,
	["Stoneclaw Totem VI"] = true,
	["Searing Totem V"] = true,
	["Searing Totem VI"] = true,
	["Strength of Earth Totem IV"] = true,
	["Oglethorpe Obnoticus"] = true,
	["Chief Engineer Bilgewhizzle"] = true,
	["Thelman Slatefist"] = true,
	["Frost Resistance Totem II"] = true,
	["Frost Resistance Totem III"] = true,
	["Mana Spring Totem III"] = true,
	["Mana Spring Totem IV"] = true,
	["Flametongue Totem III"] = true,
	["Fire Resistance Totem II"] = true,
	["Fire Resistance Totem III"] = true,
	["Taim Ragetotem"] = true,
	["Frostmaul Giant"] = true,
	["Frostmaul Preserver"] = true,
	["Frostsaber Cub"] = true,
	["Frostsaber"] = true,
	["Frostsaber Stalker"] = true,
	["Frostsaber Huntress"] = true,
	["Frostsaber Pride Watcher"] = true,
	["Cobalt Wyrmkin"] = true,
	["Cobalt Scalebane"] = true,
	["Cobalt Mageweaver"] = true,
	["Winterfall Ursa"] = true,
	["Winterfall Shaman"] = true,
	["Winterfall Den Watcher"] = true,
	["Winterfall Totemic"] = true,
	["Winterfall Pathfinder"] = true,
	["Shardtooth Mauler"] = true,
	["Shardtooth Bear"] = true,
	["Elder Shardtooth"] = true,
	["Rabid Shardtooth"] = true,
	["Fledgling Chillwind"] = true,
	["Chillwind Chimaera"] = true,
	["Chillwind Ravager"] = true,
	["Ragged Owlbeast"] = true,
	["Raging Owlbeast"] = true,
	["Crazed Owlbeast"] = true,
	["Moontouched Owlbeast"] = true,
	["Berserk Owlbeast"] = true,
	["Winterspring Owl"] = true,
	["Winterspring Screecher"] = true,
	["Rogue Ice Thistle"] = true,
	["Ice Thistle Yeti"] = true,
	["Ice Thistle Matriarch"] = true,
	["Ice Thistle Patriarch"] = true,
	["Hederine Initiate"] = true,
	["Hederine Manastalker"] = true,
	["Hederine Slayer"] = true,
	["Magma Totem III"] = true,
	["Magma Totem IV"] = true,
	["Nature Resistance Totem II"] = true,
	["Nature Resistance Totem III"] = true,
	["Windfury Totem II"] = true,
	["Windfury Totem III"] = true,
	["Grace of Air Totem II"] = true,
	["Bloodmage Drazial"] = true,
	["Bloodmage Lynnore"] = true,
	["Suffering Highborne"] = true,
	["Anguished Highborne"] = true,
	["Fallen Hero of the Horde"] = true,
	["Dispatch Commander Ruag"] = true,
	["Razelikh the Defiler"] = true,
	["Grol the Destroyer"] = true,
	["Archmage Allistarj"] = true,
	["Lady Sevine"] = true,
	["Servant of Razelikh"] = true,
	["Servant of Grol"] = true,
	["Servant of Allistarj"] = true,
	["Servant of Sevine"] = true,
	["Kirith the Damned"] = true,
	["Spirit of Kirith"] = true,
	["Ilifar"] = true,
	["Felcular"] = true,
	["Gracina Spiritmight"] = true,
	["Corporal Thund Splithoof"] = true,
	["Curgle Cranklehop"] = true,
	["Troyas Moonbreeze"] = true,
	["Kalin Windflight"] = true,
	["Marli Wishrunner"] = true,
	["Shay Leafrunner"] = true,
	["Talo Thornhoof"] = true,
	["Rin'ji"] = true,
	["Loramus Thalipedes"] = true,
	["Ward of Zum'rah"] = true,
	["Aturk the Anvil"] = true,
	["Ox"] = true,
	["McGavan"] = true,
	["Hydromancer Velratha"] = true,
	["Ruuzlu"] = true,
	["Hank the Hammer"] = true,
	["Galvan the Ancient"] = true,
	["Scorpid Duneburrower"] = true,
	["Trenton Lighthammer"] = true,
	["Marauding Owlbeast"] = true,
	["Vilebranch Ambusher"] = true,
	["Bera Stonehammer"] = true,
	["Bulkrek Ragefist"] = true,
	["Oran Snakewrithe"] = true,
	["Ambassador Ardalan"] = true,
	["Fire Nova Totem IV"] = true,
	["Fire Nova Totem V"] = true,
	["Teremus the Devourer"] = true,
	["Caliph Scorpidsting"] = true,
	["Lurking Feral Scar"] = true,
	["Nethergarde Elite"] = true,
	["Pratt McGrubben"] = true,
	["Jangdor Swiftstrider"] = true,
	["Lingering Highborne"] = true,
	["Wildhammer Sentry"] = true,
	["Peter Galen"] = true,
	["Thorkaf Dragoneye"] = true,
	["Sarah Tanner"] = true,
	["Brumn Winterhoof"] = true,
	["Caryssia Moonhunter"] = true,
	["Se'Jib"] = true,
	["Latronicus Moonspear"] = true,
	["Vestia Moonspear"] = true,
	["Quintis Jonespyre"] = true,
	["Ginro Hearthkindle"] = true,
	["Security Chief Bilgewhizzle"] = true,
	["Fraggar Thundermantle"] = true,
	["Spitelash Battlemaster"] = true,
	["Spitelash Enchantress"] = true,
	["Angelas Moonbreeze"] = true,
	["Brother Sarno"] = true,
	["Stone Watcher of Norgannon"] = true,
	["High Tinker Mekkatorque"] = true,
	["Feathermoon Sentinel"] = true,
	["Darnall"] = true,
	["Mardrack Greenwell"] = true,
	["Faralorn"] = true,
	["Harklane"] = true,
	["Tinkmaster Overspark"] = true,
	["Savanne"] = true,
	["Brannock"] = true,
	["Vivianna"] = true,
	["Kylanna Windwhisper"] = true,
	["Xylinnia Starshine"] = true,
	["Master Mechanic Castpipe"] = true,
	["Xar'Ti"] = true,
	["Binjy Featherwhistle"] = true,
	["Milli Featherwhistle"] = true,
	["Mulgore Protector"] = true,
	["Gammerita"] = true,
	["Deathguard Elite"] = true,
	["Vile Priestess Hexx"] = true,
	["Qiaga the Keeper"] = true,
	["Tyrande Whisperwind"] = true,
	["Sen'jin Guardian"] = true,
	["Guthrum Thunderfist"] = true,
	["Fyldren Moonfeather"] = true,
	["Shyn"] = true,
	["Orwin Gizzmick"] = true,
	["Thadius Grimshade"] = true,
	["Sharpbeak's Father"] = true,
	["Sharpbeak's Mother"] = true,
	["Thyn'tel Bladeweaver"] = true,
	["Edana Hatetalon"] = true,
	["Witch Doctor Uzer'i"] = true,
	["Ziggle Sparks"] = true,
	["Sul'lithuz Abomination"] = true,
	["Rickle Goldgrubber"] = true,
	["Nixx Sprocketspring"] = true,
	["Antu'sul"] = true,
	["Lord Shalzaru"] = true,
	["Jannos Lighthoof"] = true,
	["Loorana"] = true,
	["Kulleg Stonehorn"] = true,
	["Sheendra Tallgrass"] = true,
	["Ruw"] = true,
	["Camp Mojache Brave"] = true,
	["Nijel's Point Guard"] = true,
	["Ghost Walker Brave"] = true,
	["Kargath Grunt"] = true,
	["Servant of Antu'sul"] = true,
	["Logannas"] = true,
	["Bronk"] = true,
	["Worb Strongstitch"] = true,
	["Nioma"] = true,
	["Harggan"] = true,
	["Nina Lightbrew"] = true,
	["Occulus"] = true,
	["Chronalis"] = true,
	["Tick"] = true,
	["Jin'Zallah the Sandbringer"] = true,
	["Omgorn the Lost"] = true,
	["Cyclok the Mad"] = true,
	["Kregg Keelhaul"] = true,
	["Soriid the Devourer"] = true,
	["Haarka the Ravenous"] = true,
	["Greater Firebird"] = true,
	["The Reak"] = true,
	["Ironback"] = true,
	["Jalinde Summerdrake"] = true,
	["Grimungous"] = true,
	["Retherokk the Berserker"] = true,
	["Mith'rethis the Enchanter"] = true,
	["Rekk'tilac"] = true,
	["Smoldar"] = true,
	["Faulty War Golem"] = true,
	["Shleipnarr"] = true,
	["Scald"] = true,
	["Highlord Mastrogonde"] = true,
	["Slave Master Blackheart"] = true,
	["Mojo the Twisted"] = true,
	["Magronos the Unyielding"] = true,
	["Akubar the Seer"] = true,
	["Spiteflayer"] = true,
	["Ravage"] = true,
	["Clack the Reaver"] = true,
	["Deatheye"] = true,
	["Grunter"] = true,
	["Dreadscorn"] = true,
	["Carlo Aurelius"] = true,
	["Atal'ai Deathwalker's Spirit"] = true,
	["Atal'ai Slave"] = true,
	["Nightmare Whelp"] = true,
	["Atal'ai Skeleton"] = true,
	["Hakkari Sapper"] = true,
	["Dark Iron Marksman"] = true,
	["Archmage Xylem"] = true,
	["Captain Vanessa Beltis"] = true,
	["Lindros"] = true,
	["Patrick Mills"] = true,
	["Master Wood"] = true,
	["Deep Lurker"] = true,
	["Lathoric the Black"] = true,
	["Sanath Lim-yo"] = true,
	["Obsidion"] = true,
	["Enslaved Archaeologist"] = true,
	["Warlord Krellian"] = true,
	["Caravan Master Tset"] = true,
	["Dying Archaeologist"] = true,
	["Twilight Idolater"] = true,
	["Kim'jael"] = true,
	["Dorius"] = true,
	["Zamael Lunthistle"] = true,
	["Hakkari Bloodkeeper"] = true,
	["Nilith Lokrav"] = true,
	["Raze"] = true,
	["Shadowsilk Poacher"] = true,
	["Avatar of Hakkar"] = true,
	["Trade Master Kovic"] = true,
	["Clunk"] = true,
	["Skeletal Servant"] = true,
	["Kalaran Windblade"] = true,
	["Kalaran the Deceiver"] = true,
	["Nightmare Suppressor"] = true,
	["Dark Iron Sentry"] = true,
	["Squire Maltrake"] = true,
	["Atal'ai Totem"] = true,
	["Blighted Surge"] = true,
	["Plague Ravager"] = true,
	["Blighted Horror"] = true,
	["Plague Monstrosity"] = true,
	["Scourge Soldier"] = true,
	["Cursed Mage"] = true,
	["Scourge Warder"] = true,
	["Dark Caster"] = true,
	["Scourge Guard"] = true,
	["Dread Weaver"] = true,
	["Scourge Champion"] = true,
	["Cannibal Ghoul"] = true,
	["Gibbering Ghoul"] = true,
	["Diseased Flayer"] = true,
	["Putrid Gargoyle"] = true,
	["Putrid Shrieker"] = true,
	["Unseen Servant"] = true,
	["Eyeless Watcher"] = true,
	["Torn Screamer"] = true,
	["Hate Shrieker"] = true,
	["Death Singer"] = true,
	["Stitched Horror"] = true,
	["Gangled Golem"] = true,
	["Stitched Golem"] = true,
	["Dark Adept"] = true,
	["Death Cultist"] = true,
	["Vile Tutor"] = true,
	["Shadowmage"] = true,
	["Dark Summoner"] = true,
	["Necromancer"] = true,
	["Crypt Stalker"] = true,
	["Crypt Walker"] = true,
	["Crypt Horror"] = true,
	["Crypt Slayer"] = true,
	["Mossflayer Scout"] = true,
	["Mossflayer Shadowhunter"] = true,
	["Mossflayer Cannibal"] = true,
	["Woodsman"] = true,
	["Ranger"] = true,
	["Pathstrider"] = true,
	["Dark Iron Lookout"] = true,
	["Magus Rimtori"] = true,
	["Atal'alarion"] = true,
	["Blood Elf Defender"] = true,
	["Jediga"] = true,
	["Umbranse the Spiritspeaker"] = true,
	["Plaguehound Runt"] = true,
	["Plaguehound"] = true,
	["Frenzied Plaguehound"] = true,
	["Plaguebat"] = true,
	["Noxious Plaguebat"] = true,
	["Monstrous Plaguebat"] = true,
	["Carrion Grub"] = true,
	["Carrion Devourer"] = true,
	["Living Decay"] = true,
	["Rotting Sludge"] = true,
	["Angered Infernal"] = true,
	["Alexandra Constantine"] = true,
	["Kroum"] = true,
	["Mithril Dragonling"] = true,
	["Infernal Servant"] = true,
	["Morta'gya the Keeper"] = true,
	["Hukku's Voidwalker"] = true,
	["Hukku's Succubus"] = true,
	["Hukku's Imp"] = true,
	["Jes'rimon"] = true,
	["The Evalcharr"] = true,
	["Auctioneer Beardo"] = true,
	["Saern Priderunner"] = true,
	["Auctioneer Tolon"] = true,
	["Auctioneer Chilton"] = true,
	["Auctioneer Buckler"] = true,
	["Auctioneer Leeka"] = true,
	["Auctioneer Thathung"] = true,
	["Auctioneer Stampi"] = true,
	["Felbeast"] = true,
	["Jubie Gadgetspring"] = true,
	["Massive Infernal"] = true,
	["Dreadlord"] = true,
	["Felguard Elite"] = true,
	["Manahound"] = true,
	["Auctioneer Fitch"] = true,
	["Auctioneer Redmuse"] = true,
	["Auctioneer Epitwee"] = true,
	["Auctioneer Gullem"] = true,
	["Auctioneer Golothas"] = true,
	["Auctioneer Wabang"] = true,
	["Buzzek Bracketswing"] = true,
	["Raytaf"] = true,
	["Shahiar"] = true,
	["Zaman"] = true,
	["Mosshoof Stag"] = true,
	["Mosshoof Courser"] = true,
	["Timberweb Recluse"] = true,
	["Mistwing Rogue"] = true,
	["Mistwing Ravager"] = true,
	["Forest Ooze"] = true,
	["Deathly Usher"] = true,
	["Muck Splash"] = true,
	["Royal Historian Archesonus"] = true,
	["A tormented voice"] = true,
	["Franclorn Forgewright"] = true,
	["Anvilrage Overseer"] = true,
	["Anvilrage Warden"] = true,
	["Anvilrage Guardsman"] = true,
	["Anvilrage Footman"] = true,
	["Anvilrage Soldier"] = true,
	["Anvilrage Medic"] = true,
	["Anvilrage Officer"] = true,
	["Shadowforge Peasant"] = true,
	["Doomforge Craftsman"] = true,
	["Anvilrage Marshal"] = true,
	["Doomforge Dragoon"] = true,
	["Doomforge Arcanasmith"] = true,
	["Anvilrage Reservist"] = true,
	["Shadowforge Citizen"] = true,
	["Anvilrage Captain"] = true,
	["Shadowforge Senator"] = true,
	["Warbringer Construct"] = true,
	["Ragereaver Golem"] = true,
	["Wrath Hammer Construct"] = true,
	["Molten War Golem"] = true,
	["Fireguard"] = true,
	["Blazing Fireguard"] = true,
	["Fireguard Destroyer"] = true,
	["Twilight's Hammer Torturer"] = true,
	["Twilight Emissary"] = true,
	["Twilight Bodyguard"] = true,
	["Twilight's Hammer Ambassador"] = true,
	["Arena Spectator"] = true,
	["Quarry Slave"] = true,
	["Weapon Technician"] = true,
	["Bloodhound"] = true,
	["Bloodhound Mastiff"] = true,
	["Panzor the Invincible"] = true,
	["The Behemoth"] = true,
	["Dredge Worm"] = true,
	["Deep Stinger"] = true,
	["Dark Screecher"] = true,
	["Burrowing Thundersnout"] = true,
	["Princess Moira Bronzebeard"] = true,
	["Borer Beetle"] = true,
	["Cave Creeper"] = true,
	["Angerclaw Bear"] = true,
	["Angerclaw Grizzly"] = true,
	["Angerclaw Mauler"] = true,
	["Felpaw Wolf"] = true,
	["Felpaw Scavenger"] = true,
	["Felpaw Ravager"] = true,
	["Blackrock Drake"] = true,
	["Hematos"] = true,
	["Krom'Grul"] = true,
	["Thauris Balgarr"] = true,
	["Gruklash"] = true,
	["Firegut Captain"] = true,
	["Malfunctioning Reaver"] = true,
	["Ironhand Guardian"] = true,
	["Golem Lord Argelmach"] = true,
	["Voidwalker Minion"] = true,
	["Bael'Gar"] = true,
	["Lord Incendius"] = true,
	["High Interrogator Gerstahn"] = true,
	["Emperor Dagran Thaurissan"] = true,
	["Commander Gor'shak"] = true,
	["Kharan Mighthammer"] = true,
	["Dughal Stormwing"] = true,
	["Marshal Windsor"] = true,
	["Pyromancer Loregrain"] = true,
	["Lord Roccor"] = true,
	["Overmaster Pyron"] = true,
	["Gorosh the Dervish"] = true,
	["Grizzle"] = true,
	["Eviscerator"] = true,
	["Ok'thor the Breaker"] = true,
	["Anub'shiah"] = true,
	["Hedrum the Creeper"] = true,
	["General Angerforge"] = true,
	["Hate'rel"] = true,
	["Anger'rel"] = true,
	["Vile'rel"] = true,
	["Gloom'rel"] = true,
	["Seeth'rel"] = true,
	["Doom'rel"] = true,
	["Dope'rel"] = true,
	["Warder Stilgiss"] = true,
	["Verek"] = true,
	["Scarshield Grunt"] = true,
	["Scarshield Sentry"] = true,
	["Scarshield Acolyte"] = true,
	["Scarshield Quartermaster"] = true,
	["Fineous Darkvire"] = true,
	["Warlord Goretooth"] = true,
	["Shadowmage Vivian Lagrave"] = true,
	["Hierophant Theodora Mulvadania"] = true,
	["Lexlort"] = true,
	["Galamav the Marksman"] = true,
	["Thal'trak Proudtusk"] = true,
	["Razal'blade"] = true,
	["Thunderheart"] = true,
	["Initiate Amakkar"] = true,
	["Grunt Gargal"] = true,
	["Rage Talon Dragonspawn"] = true,
	["Scarshield Legionnaire"] = true,
	["Scarshield Spellbinder"] = true,
	["Eridan Bluewind"] = true,
	["J.D. Collie"] = true,
	["Larion"] = true,
	["Muigin"] = true,
	["Sha'ni Proudtusk"] = true,
	["Ambassador Flamelash"] = true,
	["Young Diemetradon"] = true,
	["Diemetradon"] = true,
	["Elder Diemetradon"] = true,
	["Fledgling Pterrordax"] = true,
	["Pterrordax"] = true,
	["Frenzied Pterrordax"] = true,
	["Gor'tesh"] = true,
	["Oralius"] = true,
	["Highlord Omokk"] = true,
	["Spirestone Battle Mage"] = true,
	["Spirestone Mystic"] = true,
	["Spirestone Enforcer"] = true,
	["Spirestone Reaver"] = true,
	["Spirestone Ogre Magus"] = true,
	["Spirestone Warlord"] = true,
	["Spirestone Lord Magus"] = true,
	["Spirestone Battle Lord"] = true,
	["Spirestone Butcher"] = true,
	["Shadow Hunter Vosh'gajin"] = true,
	["War Master Voone"] = true,
	["Smolderthorn Mystic"] = true,
	["Smolderthorn Shadow Priest"] = true,
	["Smolderthorn Headhunter"] = true,
	["Scarshield Warlock"] = true,
	["Scarshield Raider"] = true,
	["Firebrand Grunt"] = true,
	["Firebrand Legionnaire"] = true,
	["Firebrand Darkweaver"] = true,
	["Firebrand Invoker"] = true,
	["Firebrand Dreadweaver"] = true,
	["Firebrand Pyromancer"] = true,
	["Smolderthorn Shadow Hunter"] = true,
	["Smolderthorn Witch Doctor"] = true,
	["Smolderthorn Axe Thrower"] = true,
	["Smolderthorn Berserker"] = true,
	["Smolderthorn Seer"] = true,
	["Williden Marshal"] = true,
	["Petra Grossen"] = true,
	["Enraged Wyvern"] = true,
	["Donova Snowden"] = true,
	["Gaeriyan"] = true,
	["Incendosaur"] = true,
	["Houndmaster Grebmar"] = true,
	["Blazerunner"] = true,
	["Living Storm"] = true,
	["Twilight's Hammer Executioner"] = true,
	["Scarshield Worg"] = true,
	["Spawn of Bael'Gar"] = true,
	["Dark Keeper Vorfalk"] = true,
	["Dark Keeper Bethek"] = true,
	["Dark Keeper Uggel"] = true,
	["Dark Keeper Zimrel"] = true,
	["Dark Keeper Ofgut"] = true,
	["Dark Keeper Pelver"] = true,
	["Dark Guard"] = true,
	["Scarlet Warder"] = true,
	["Scarlet Praetorian"] = true,
	["Scarlet Cleric"] = true,
	["Scarlet Curate"] = true,
	["Scarlet Archmage"] = true,
	["Scarlet Enchanter"] = true,
	["Aquementas"] = true,
	["Xavathras"] = true,
	["Cyrus Therepentous"] = true,
	["Gadgetzan Bruiser"] = true,
	["Frenzied Black Drake"] = true,
	["Chieftain Bloodmaw"] = true,
	["Overlord Ror"] = true,
	["Golhine the Hooded"] = true,
	["Miblon Snarltooth"] = true,
	["Watchman Doomgrip"] = true,
	["Cloned Ooze"] = true,
	["Gorishi Egg"] = true,
	["Gorishi Grub"] = true,
	["Plugger Spazzring"] = true,
	["Mistress Nagmara"] = true,
	["Phalanx"] = true,
	["Private Rocknot"] = true,
	["Lord Banehollow"] = true,
	["Shadow Lord Fel'dan"] = true,
	["Rakaiah"] = true,
	["Grark Lorkrub"] = true,
	["Enraged Felbat"] = true,
	["Blackrock Ambusher"] = true,
	["Enraged Gryphon"] = true,
	["Enraged Hippogryph"] = true,
	["Arathandris Silversky"] = true,
	["Maybess Riverbreeze"] = true,
	["Hurley Blackbreath"] = true,
	["High Executioner Nuzrak"] = true,
	["Shadow of Lexlort"] = true,
	["Enohar Thunderbrew"] = true,
	["Blackbreath Crony"] = true,
	["Ribbly Screwspigot"] = true,
	["Yuka Screwspigot"] = true,
	["Grim Patron"] = true,
	["Guzzling Patron"] = true,
	["Cawind Trueaim"] = true,
	["Hammered Patron"] = true,
	["Marshal Maxwell"] = true,
	["Jalinda Sprig"] = true,
	["Helendis Riverhorn"] = true,
	["Ragged John"] = true,
	["Mayara Brightwing"] = true,
	["Overlord Wyrmthalak"] = true,
	["Bloodaxe Veteran"] = true,
	["Bannok Grimaxe"] = true,
	["Arei"] = true,
	["Treant Spirit"] = true,
	["Hahk'Zor"] = true,
	["Gorgon'och"] = true,
	["Blackrock Raider"] = true,
	["Laris Geardawdle"] = true,
	["Karna Remtravel"] = true,
	["Torwa Pathfinder"] = true,
	["Dreka'Sur"] = true,
	["Gargantuan Ooze"] = true,
	["U'cha"] = true,
	["A-Me 01"] = true,
	["Scorching Totem"] = true,
	["Agnar Beastamer"] = true,
	["Ograbisi"] = true,
	["Shill Dinger"] = true,
	["Tobias Seecher"] = true,
	["Crest Killer"] = true,
	["Jaz"] = true,
	["Marshal Reginald Windsor"] = true,
	["Lar'korwi Mate"] = true,
	["Lar'korwi"] = true,
	["Windwall Totem III"] = true,
	["Ember Worg"] = true,
	["Venomtip Scorpid"] = true,
	["Bloodaxe Raider"] = true,
	["Bloodaxe Evoker"] = true,
	["Slavering Ember Worg"] = true,
	["Deathlash Scorpid"] = true,
	["Bloodaxe Worg"] = true,
	["Giant Ember Worg"] = true,
	["Firetail Scorpid"] = true,
	["Spire Scorpid"] = true,
	["Yorba Screwspigot"] = true,
	["Burning Imp"] = true,
	["Bloodaxe Warmonger"] = true,
	["Bloodaxe Summoner"] = true,
	["Ghok Bashguud"] = true,
	["Quartermaster Zigris"] = true,
	["Flamekin Spitter"] = true,
	["Flamekin Sprite"] = true,
	["Flamekin Torcher"] = true,
	["Flamekin Rager"] = true,
	["Pyroguard Emberseer"] = true,
	["Blackhand Dreadweaver"] = true,
	["Blackhand Summoner"] = true,
	["Blackhand Veteran"] = true,
	["Mathredis Firestar"] = true,
	["Auctioneer Grimful"] = true,
	["Auctioneer Grizzlin"] = true,
	["Auctioneer Kresky"] = true,
	["Auctioneer Lympkin"] = true,
	["Salia"] = true,
	["Moora"] = true,
	["Jaedenar Legionnaire"] = true,
	["Locheed"] = true,
	["Prince Xavalis"] = true,
	["Entropic Beast"] = true,
	["Entropic Horror"] = true,
	["Jarquia"] = true,
	["Magmus"] = true,
	["Shadowforge Flame Keeper"] = true,
	["Xon'cha"] = true,
	["Winna Hazzard"] = true,
	["Ringo"] = true,
	["Brackenwall Enforcer"] = true,
	["Lakeshire Guard"] = true,
	["Gorishi Hive Guard"] = true,
	["Gorishi Hive Queen"] = true,
	["Ribbly's Crony"] = true,
	["Gereck"] = true,
	["Grimestack"] = true,
	["High Priestess of Thaurissan"] = true,
	["Deathmaw"] = true,
	["Terrorspark"] = true,
	["Rage Talon Flamescale"] = true,
	["High Justice Grimstone"] = true,
	["Volchan"] = true,
	["Rookery Whelp"] = true,
	["Lord Victor Nefarius"] = true,
	["Spire Scarab"] = true,
	["Lady Sylvanas Windrunner"] = true,
	["Rexxar"] = true,
	["Moonflare Totem"] = true,
	["Onyxia"] = true,
	["General Colbatann"] = true,
	["Mezzir the Howler"] = true,
	["Kashoch the Reaver"] = true,
	["Grizzle Snowpaw"] = true,
	["Rak'shiri"] = true,
	["Lady Hederine"] = true,
	["Azurous"] = true,
	["Misha"] = true,
	["Flame Buffet Totem"] = true,
	["Superior Healing Ward"] = true,
	["Halycon"] = true,
	["Bloodaxe Worg Pup"] = true,
	["Bijou"] = true,
	["Rookery Guardian"] = true,
	["Kibler"] = true,
	["Burning Felhound"] = true,
	["Burning Felguard"] = true,
	["Solakar Flamewreath"] = true,
	["Tinkee Steamboil"] = true,
	["Gizrul the Slavener"] = true,
	["Dulciea Frostmoon"] = true,
	["Vaelan"] = true,
	["Scarshield Infiltrator"] = true,
	["Ranshalla"] = true,
	["Jaron Stoneshaper"] = true,
	["Krakle"] = true,
	["Storm Shadowhoof"] = true,
	["Aurora Skycaller"] = true,
	["Umi Rumplesnicker"] = true,
	["Trull Failbane"] = true,
	["Witch Doctor Mau'ari"] = true,
	["Blackhand Incarcerator"] = true,
	["Blackhand Elite"] = true,
	["Blackhand Assassin"] = true,
	["Blackhand Iron Guard"] = true,
	["Emberstrife"] = true,
	["Gyth"] = true,
	["Vaelastrasz the Red"] = true,
	["Kergul Bloodaxe"] = true,
	["Gruul Darkblade"] = true,
	["General Drakkisath"] = true,
	["Yaelika Farclaw"] = true,
	["Rage Talon Dragon Guard"] = true,
	["Shrye Ragefist"] = true,
	["Rage Talon Captain"] = true,
	["Rage Talon Fire Tongue"] = true,
	["Spire Spider"] = true,
	["Spire Spiderling"] = true,
	["Crystal Fang"] = true,
	["Omusa Thunderhorn"] = true,
	["Ravaged Cadaver"] = true,
	["Mangled Cadaver"] = true,
	["Broken Cadaver"] = true,
	["Spectral Citizen"] = true,
	["Ghostly Citizen"] = true,
	["Vengeful Phantom"] = true,
	["Spiteful Phantom"] = true,
	["Wrath Phantom"] = true,
	["Skeletal Guardian"] = true,
	["Skeletal Berserker"] = true,
	["Skul"] = true,
	["Black Guard Sentry"] = true,
	["Thuzadin Shadowcaster"] = true,
	["Thuzadin Acolyte"] = true,
	["Thuzadin Necromancer"] = true,
	["Plague Ghoul"] = true,
	["Ghoul Ravener"] = true,
	["Fleshflayer Ghoul"] = true,
	["Rockwing Gargoyle"] = true,
	["Rockwing Screecher"] = true,
	["Eye of Naxxramas"] = true,
	["Crypt Crawler"] = true,
	["Crypt Beast"] = true,
	["Patchwork Horror"] = true,
	["Ash'ari Crystal"] = true,
	["Bile Spewer"] = true,
	["Venom Belcher"] = true,
	["Crimson Guardsman"] = true,
	["Crimson Conjuror"] = true,
	["Crimson Initiate"] = true,
	["Crimson Defender"] = true,
	["Crimson Sorcerer"] = true,
	["Crimson Priest"] = true,
	["Crimson Gallant"] = true,
	["Crimson Battle Mage"] = true,
	["Crimson Inquisitor"] = true,
	["Warchief Rend Blackhand"] = true,
	["The Beast"] = true,
	["Gregor Greystone"] = true,
	["Vectus"] = true,
	["Marduk Blackpool"] = true,
	["Magistrate Barthilas"] = true,
	["Baroness Anastari"] = true,
	["Nerub'enkan"] = true,
	["Maleki the Pallid"] = true,
	["Ramstein the Gorger"] = true,
	["Baron Rivendare"] = true,
	["Plagued Rat"] = true,
	["Chromatic Whelp"] = true,
	["Chromatic Dragonspawn"] = true,
	["Prospector Ironboot"] = true,
	["Plagued Insect"] = true,
	["Shrieking Banshee"] = true,
	["Wailing Banshee"] = true,
	["Felnok Steelspring"] = true,
	["Scholomance Adept"] = true,
	["Scholomance Neophyte"] = true,
	["Scholomance Acolyte"] = true,
	["Scholomance Occultist"] = true,
	["Scholomance Student"] = true,
	["Scholomance Necrolyte"] = true,
	["Scholomance Necromancer"] = true,
	["Splintered Skeleton"] = true,
	["Skulking Corpse"] = true,
	["Unstable Corpse"] = true,
	["Reanimated Corpse"] = true,
	["Risen Lackey"] = true,
	["Risen Aberration"] = true,
	["Risen Warrior"] = true,
	["Risen Protector"] = true,
	["Risen Construct"] = true,
	["Risen Guard"] = true,
	["Risen Bonewarder"] = true,
	["Diseased Ghoul"] = true,
	["Ragged Ghoul"] = true,
	["Spectral Tutor"] = true,
	["Spectral Researcher"] = true,
	["Spectral Teacher"] = true,
	["Lady Illucia Barov"] = true,
	["Jandice Barov"] = true,
	["Lord Alexei Barov"] = true,
	["Instructor Malicia"] = true,
	["Kirtonos the Herald"] = true,
	["The Ravenian"] = true,
	["Ras Frostwhisper"] = true,
	["Jed Runewatcher"] = true,
	["The Unforgiven"] = true,
	["Plagued Maggot"] = true,
	["Cliffwatcher Longhorn"] = true,
	["Vaelastrasz"] = true,
	["Vol'jin"] = true,
	["Flametongue Totem IV"] = true,
	["Hearthsinger Forresten"] = true,
	["Fetid Zombie"] = true,
	["Gryfe"] = true,
	["Urok Doomhowl"] = true,
	["Mother Smolderweb"] = true,
	["Urok Enforcer"] = true,
	["Urok Ogre Magus"] = true,
	["Scarlet Medic"] = true,
	["Scarlet Priest"] = true,
	["Rivern Frostwind"] = true,
	["Glacier"] = true,
	["Malyfous Darkhammer"] = true,
	["Xavaric"] = true,
	["Cobalt Whelp"] = true,
	["Cobalt Broodling"] = true,
	["Spell Eater"] = true,
	["Spellmaw"] = true,
	["Manaclaw"] = true,
	["Scryer"] = true,
	["Chromie"] = true,
	["Plagued Hatchling"] = true,
	["Summoned Blackhand Dreadweaver"] = true,
	["Summoned Blackhand Veteran"] = true,
	["Rookery Hatcher"] = true,
	["Remorseful Highborne"] = true,
	["Bile Slime"] = true,
	["Summoned Zombie"] = true,
	["Carrion Scarab"] = true,
	["Temporal Parasite"] = true,
	["Herald of Thrall"] = true,
	["Shy-Rotam"] = true,
	["High Chief Winterfall"] = true,
	["Mulgris Deepriver"] = true,
	["Awbee"] = true,
	["Sian-Rotam"] = true,
	["Blackhand Dragon Handler"] = true,
	["Blackhand Thug"] = true,
	["Finkle Einhorn"] = true,
	["Warosh"] = true,
	["Warosh the Redeemed"] = true,
	["Jabbering Ghoul"] = true,
	["Hitah'ya the Keeper"] = true,
	["Ursius"] = true,
	["Brumeran"] = true,
	["Timmy the Cruel"] = true,
	["Stonespine"] = true,
	["Archivist Galford"] = true,
	["Grand Crusader Dathrohan"] = true,
	["Balnazzar"] = true,
	["Chromatic Elite Guard"] = true,
	["Wandering Skeleton"] = true,
	["Duggan Wildhammer"] = true,
	["Hed'mush the Rotting"] = true,
	["Warlord Thresh'jin"] = true,
	["Zul'Brin Warpbranch"] = true,
	["Ranger Lord Hawkspear"] = true,
	["Gish the Unmoving"] = true,
	["Lord Darkscythe"] = true,
	["Deathspeaker Selendre"] = true,
	["High General Abbendis"] = true,
	["Farmer Dalson"] = true,
	["High Executor Derrington"] = true,
	["Commander Ashlam Valorfist"] = true,
	["Argent Officer Garush"] = true,
	["Argent Officer Pureheart"] = true,
	["Argent Quartermaster Hasana"] = true,
	["Argent Quartermaster Lightspark"] = true,
	["Undead Scarab"] = true,
	["Sindrayl"] = true,
	["Goraluk Anvilcrack"] = true,
	["Lorekeeper Polkelt"] = true,
	["Winterfall Runner"] = true,
	["Aurius"] = true,
	["Lorax"] = true,
	["Shatterspear Troll"] = true,
	["Kelek Skykeeper"] = true,
	["Taronn Redfeather"] = true,
	["Greta Mosshoof"] = true,
	["Tenell Leafrunner"] = true,
	["Ivy Leafrunner"] = true,
	["Rotting Worm"] = true,
	["Haleh"] = true,
	["Joseph Redpath"] = true,
	["Captain Redpath"] = true,
	["Redpath the Corrupted"] = true,
	["Marduk the Black"] = true,
	["Ghost of the Past"] = true,
	["Nessy"] = true,
	["Decrepit Guardian"] = true,
	["Davil Lightfire"] = true,
	["Davil Crokford"] = true,
	["Horgus the Ravager"] = true,
	["Darrowshire Betrayer"] = true,
	["Darrowshire Defender"] = true,
	["Silver Hand Disciple"] = true,
	["Redpath Militia"] = true,
	["Marauding Corpse"] = true,
	["Marauding Skeleton"] = true,
	["Servant of Horgus"] = true,
	["Bloodletter"] = true,
	["Summoned Water Elemental"] = true,
	["Naga Siren"] = true,
	["Jeziba"] = true,
	["Quixxil"] = true,
	["Legacki"] = true,
	["Scarlet Hound"] = true,
	["Frostwolf"] = true,
	["Whitewhisker Vermin"] = true,
	["Winterax Troll"] = true,
	["Winterax Berserker"] = true,
	["Snowblind Harpy"] = true,
	["Irondeep Trogg"] = true,
	["Kodo Spirit"] = true,
	["Alterac Ram"] = true,
	["Wildpaw Gnoll"] = true,
	["Fallen Hero"] = true,
	["Cannon Master Willey"] = true,
	["Captured Arko'narin"] = true,
	["Roxxik"] = true,
	["Arko'narin"] = true,
	["Jessir Moonbow"] = true,
	["Remains of Trey Lightforge"] = true,
	["Alexi Barov"] = true,
	["Weldon Barov"] = true,
	["Della"] = true,
	["Illusory Wraith"] = true,
	["Mindless Undead"] = true,
	["Malor the Zealous"] = true,
	["Smokey LaRue"] = true,
	["Lord Maxwell Tyrosus"] = true,
	["Betina Bigglezink"] = true,
	["Leonid Barthalomew the Revered"] = true,
	["Caretaker Alen"] = true,
	["Duke Nicholas Zverenhoff"] = true,
	["Crimson Monk"] = true,
	["Timothy Worthington"] = true,
	["High Priestess MacDonnell"] = true,
	["Crimson Rifleman"] = true,
	["Shadow Priestess Vandis"] = true,
	["Alchemist Arbington"] = true,
	["Apothecary Dithers"] = true,
	["Fras Siabi"] = true,
	["Carlin Redpath"] = true,
	["Darrowshire Spirit"] = true,
	["Jenova Stoneshield"] = true,
	["Annora"] = true,
	["Hgarth"] = true,
	["Cauldron Lord Bilemaw"] = true,
	["Cauldron Lord Razarch"] = true,
	["Cauldron Lord Malvinious"] = true,
	["Cauldron Lord Soulwrath"] = true,
	["Wynd Nightchaser"] = true,
	["Stratholme Courier"] = true,
	["Drakk Stonehand"] = true,
	["Hahrana Ironhide"] = true,
	["Argent Guard"] = true,
	["Mana Tide Totem II"] = true,
	["Mana Tide Totem III"] = true,
	["Argent Rider"] = true,
	["Azzleby"] = true,
	["Crimson Hammersmith"] = true,
	["Black Guard Swordsmith"] = true,
	["Maethrya"] = true,
	["Yugrek"] = true,
	["Egan"] = true,
	["Spirit of Trey Lightforge"] = true,
	["Undead Postman"] = true,
	["Postmaster Malown"] = true,
	["Myolor Sunderfury"] = true,
	["Ironus Coldsteel"] = true,
	["Krathok Moltenfist"] = true,
	["Okothos Ironrager"] = true,
	["Borgosh Corebender"] = true,
	["Bloodvenom Post Brave"] = true,
	["Shi'alune"] = true,
	["Nixxrak"] = true,
	["Blixxrak"] = true,
	["Wixxrak"] = true,
	["Xizzer Fizzbolt"] = true,
	["Lunnix Sprocketslip"] = true,
	["Himmik"] = true,
	["Evie Whirlbrew"] = true,
	["Qia"] = true,
	["Everlook Bruiser"] = true,
	["Lilith the Lithe"] = true,
	["Kilram"] = true,
	["Seril Scourgebane"] = true,
	["Argent Defender"] = true,
	["Shatterspear Drummer"] = true,
	["Mindless Skeleton"] = true,
	["Crimson Cannon"] = true,
	["Summoned Skeleton"] = true,
	["Eva Sarkhoff"] = true,
	["Lucien Sarkhoff"] = true,
	["Manifestation of Water"] = true,
	["Scholomance Handler"] = true,
	["Doctor Theolen Krastinov"] = true,
	["Onyxian Whelp"] = true,
	["Spectral Projection"] = true,
	["Caer Darrow Citizen"] = true,
	["Magnus Frostwake"] = true,
	["Caer Darrow Guardsman"] = true,
	["Caer Darrow Cannoneer"] = true,
	["Caer Darrow Horseman"] = true,
	["Dark Shade"] = true,
	["Magistrate Marduke"] = true,
	["Spectral Betrayer"] = true,
	["Spectral Defender"] = true,
	["Mossflayer Zombie"] = true,
	["Unliving Mossflayer"] = true,
	["Hakkari Shadowcaster"] = true,
	["Hakkari Shadow Hunter"] = true,
	["Hakkari Blood Priest"] = true,
	["Hakkari Oracle"] = true,
	["Zealot Lor'Khan"] = true,
	["Zealot Zath"] = true,
	["Gurubashi Axe Thrower"] = true,
	["Gurubashi Headhunter"] = true,
	["Gurubashi Berserker"] = true,
	["Gurubashi Blood Drinker"] = true,
	["Gurubashi Warrior"] = true,
	["Gurubashi Champion"] = true,
	["Son of Hakkar"] = true,
	["Soulflayer"] = true,
	["Zulian Cub"] = true,
	["Zulian Tiger"] = true,
	["Zulian Panther"] = true,
	["Bloodseeker Bat"] = true,
	["Razzashi Broodwidow"] = true,
	["Razzashi Serpent"] = true,
	["Razzashi Adder"] = true,
	["Razzashi Cobra"] = true,
	["Hooktooth Frenzy"] = true,
	["Jin'do the Hexxer"] = true,
	["Bloodlord Mandokir"] = true,
	["High Priestess Hai'watna"] = true,
	["Sandfury Speaker"] = true,
	["Witherbark Speaker"] = true,
	["Bloodscalp Speaker"] = true,
	["Skullsplitter Speaker"] = true,
	["Vilebranch Speaker"] = true,
	["Priestess Alathea"] = true,
	["High Priest Rohan"] = true,
	["Illusion of Jandice Barov"] = true,
	["Gordok Enforcer"] = true,
	["Gordok Brute"] = true,
	["Gordok Mauler"] = true,
	["Gordok Ogre-Mage"] = true,
	["Gordok Mage-Lord"] = true,
	["Gordok Captain"] = true,
	["Gordok Spirit"] = true,
	["Mushgog"] = true,
	["Gordok Warlock"] = true,
	["Gordok Reaver"] = true,
	["Wildspawn Satyr"] = true,
	["Wildspawn Rogue"] = true,
	["Wildspawn Trickster"] = true,
	["Wildspawn Betrayer"] = true,
	["Wildspawn Felsworn"] = true,
	["Wildspawn Shadowstalker"] = true,
	["Wildspawn Hellcaller"] = true,
	["Petrified Treant"] = true,
	["Ironbark Protector"] = true,
	["Alzzin's Minion"] = true,
	["Warpwood Guardian"] = true,
	["Warpwood Treant"] = true,
	["Warpwood Tangler"] = true,
	["Warpwood Stomper"] = true,
	["Highborne Summoner"] = true,
	["Tsu'zee"] = true,
	["Eldreth Seether"] = true,
	["Eldreth Sorcerer"] = true,
	["Eldreth Apparition"] = true,
	["Eldreth Spirit"] = true,
	["Eldreth Spectre"] = true,
	["Eldreth Phantasm"] = true,
	["Skeletal Highborne"] = true,
	["Rotting Highborne"] = true,
	["Arcane Aberration"] = true,
	["Mana Remnant"] = true,
	["Residual Monstrosity"] = true,
	["Prince Tortheldrin"] = true,
	["Magister Kalendris"] = true,
	["Illyanna Ravenoak"] = true,
	["Tendris Warpwood"] = true,
	["Zevrim Thornhoof"] = true,
	["Old Ironbark"] = true,
	["Alzzin the Wildshaper"] = true,
	["Immol'thar"] = true,
	["The Razza"] = true,
	["Skarr the Unbreakable"] = true,
	["King Gordok"] = true,
	["Ragnaros"] = true,
	["Timbermaw Warder"] = true,
	["Quartermaster Miranda Breechlock"] = true,
	["Jack Sterling"] = true,
	["Necrofiend"] = true,
	["Timbermaw Mystic"] = true,
	["Timbermaw Woodbender"] = true,
	["Grazle"] = true,
	["Gorn One Eye"] = true,
	["Salfa"] = true,
	["Meilosh"] = true,
	["Kernda"] = true,
	["Scholomance Dark Summoner"] = true,
	["Nefarian"] = true,
	["Risen Guardian"] = true,
	["Irondeep Shaman"] = true,
	["Irondeep Skullthumper"] = true,
	["Whitewhisker Digger"] = true,
	["Whitewhisker Geomancer"] = true,
	["Whitewhisker Overseer"] = true,
	["Bardu Sharpeye"] = true,
	["Alexia Ironknife"] = true,
	["Kirsta Deepshadow"] = true,
	["Cavalier Durgen"] = true,
	["Huntsman Radley"] = true,
	["Bloodshot"] = true,
	["Mickey Levine"] = true,
	["Nathaniel Dumah"] = true,
	["Spectral Marauder"] = true,
	["Spectral Corpse"] = true,
	["Rattlegore"] = true,
	["Scourge Summoning Crystal"] = true,
	["Taiga Wisemane"] = true,
	["Jessica Redpath"] = true,
	["Servant of Weldon Barov"] = true,
	["Servant of Alexi Barov"] = true,
	["Morloch"] = true,
	["Molten Giant"] = true,
	["Molten Destroyer"] = true,
	["Flamewaker"] = true,
	["Flamewaker Priest"] = true,
	["Flamewaker Healer"] = true,
	["Flamewaker Elite"] = true,
	["Lava Annihilator"] = true,
	["Firewalker"] = true,
	["Flameguard"] = true,
	["Firelord"] = true,
	["Flame Imp"] = true,
	["Core Hound"] = true,
	["Core Rager"] = true,
	["Ancient Core Hound"] = true,
	["Snowblind Windcaller"] = true,
	["Taskmaster Snivvle"] = true,
	["Snowblind Ambusher"] = true,
	["Winterax Witch Doctor"] = true,
	["Chal Fairwind"] = true,
	["Hive'Ashi Stinger"] = true,
	["Varian Wrynn"] = true,
	["Sarin Starlight"] = true,
	["Mor'vek"] = true,
	["Arin'sor"] = true,
	["Graw Cornerstone"] = true,
	["Kriss Goldenlight"] = true,
	["Rayan Dawnrisen"] = true,
	["Adon"] = true,
	["Joy Ar'nareth"] = true,
	["Coral Moongale"] = true,
	["Mirador"] = true,
	["Celes Earthborne"] = true,
	["Sar Browneye"] = true,
	["Hive'Ashi Worker"] = true,
	["Hive'Ashi Defender"] = true,
	["Hive'Ashi Sandstalker"] = true,
	["Hive'Ashi Swarmer"] = true,
	["Hive'Zora Waywatcher"] = true,
	["Hive'Zora Tunneler"] = true,
	["Hive'Zora Wasp"] = true,
	["Hive'Zora Reaver"] = true,
	["Hive'Zora Hive Sister"] = true,
	["Hive'Regal Ambusher"] = true,
	["Hive'Regal Burrower"] = true,
	["Hive'Regal Spitfire"] = true,
	["Hive'Regal Slavemaker"] = true,
	["Hive'Regal Hive Lord"] = true,
	["Stonelash Scorpid"] = true,
	["Stonelash Pincer"] = true,
	["Stonelash Flayer"] = true,
	["Sand Skitterer"] = true,
	["Rock Stalker"] = true,
	["Dredge Striker"] = true,
	["Dredge Crusher"] = true,
	["Dust Stormer"] = true,
	["Cyclone Warrior"] = true,
	["Desert Rumbler"] = true,
	["Desert Rager"] = true,
	["Feran Strongwind"] = true,
	["Blaise Montgomery"] = true,
	["Gogo"] = true,
	["Meggi Peppinrocker"] = true,
	["Harlo Wigglesworth"] = true,
	["Andi Lynn"] = true,
	["Theradrim Shardling"] = true,
	["Theradrim Guardian"] = true,
	["Deep Borer"] = true,
	["Mylentha Riverbend"] = true,
	["Bessany Plainswind"] = true,
	["Moren Riverbend"] = true,
	["Bunthen Plainswind"] = true,
	["Tajarri"] = true,
	["Silva Fil'naveth"] = true,
	["Rabine Saturna"] = true,
	["Dendrite Starblaze"] = true,
	["Twilight Keeper Exeter"] = true,
	["Twilight Keeper Havunth"] = true,
	["Jarund Stoutstrider"] = true,
	["Grum Redbeard"] = true,
	["Voriya"] = true,
	["Moonglade Warden"] = true,
	["Paige Felixe"] = true,
	["Hakkari Priest"] = true,
	["Hakkari Witch Doctor"] = true,
	["Keeper Remulos"] = true,
	["Theodore Griffs"] = true,
	["Wildpaw Shaman"] = true,
	["Wildpaw Mystic"] = true,
	["Wildpaw Brute"] = true,
	["Wildpaw Alpha"] = true,
	["Doomguard"] = true,
	["Azore Aldamort"] = true,
	["Buliwyf Stonehand"] = true,
	["Ilyenia Moonfire"] = true,
	["Woo Ping"] = true,
	["Sayoc"] = true,
	["Ansekhwa"] = true,
	["Archibald"] = true,
	["Grinning Dog"] = true,
	["Myranda the Hag"] = true,
	["Spectral Attendant"] = true,
	["Masat T'andr"] = true,
	["Nathanos Blightcaller"] = true,
	["Twilight Avenger"] = true,
	["Twilight Geolord"] = true,
	["Twilight Stonecaller"] = true,
	["Twilight Master"] = true,
	["Blighthound"] = true,
	["Mercutio Filthgorger"] = true,
	["Crypt Robber"] = true,
	["Borelgore"] = true,
	["Duskwing"] = true,
	["Crusader Lord Valdelmar"] = true,
	["Shardi"] = true,
	["Brakkar"] = true,
	["Andruk"] = true,
	["Umber"] = true,
	["Drek'Thar"] = true,
	["Captain Galvangar"] = true,
	["Vanndar Stormpike"] = true,
	["Captain Balinda Stonehearth"] = true,
	["Great Bear Spirit"] = true,
	["Great Cat Spirit"] = true,
	["Flamegor"] = true,
	["Magmadar"] = true,
	["Firemaw"] = true,
	["Golemagg the Incinerator"] = true,
	["Stormpike Herald"] = true,
	["Frostwolf Herald"] = true,
	["Broodlord Lashlayer"] = true,
	["Majordomo Executus"] = true,
	["Dargon"] = true,
	["Daeolyn Summerleaf"] = true,
	["Lorelae Wintersong"] = true,
	["Kharedon"] = true,
	["Meliri"] = true,
	["Malvor"] = true,
	["My'lanna"] = true,
	["Narianna"] = true,
	["Koiter"] = true,
	["Loganaar"] = true,
	["Gor'marok the Ravager"] = true,
	["Stormpike Mountaineer"] = true,
	["Alliance Sentinel"] = true,
	["Stormpike Defender"] = true,
	["Frostwolf Legionnaire"] = true,
	["Frostwolf Warrior"] = true,
	["Frostwolf Guardian"] = true,
	["Baron Geddon"] = true,
	["Garr"] = true,
	["Lava Elemental"] = true,
	["Stormpike Quartermaster"] = true,
	["Frostwolf Quartermaster"] = true,
	["Sulfuron Harbinger"] = true,
	["Firesworn"] = true,
	["Lava Reaver"] = true,
	["Lava Surger"] = true,
	["Lucifron"] = true,
	["Flamewaker Protector"] = true,
	["Drakan"] = true,
	["Duros"] = true,
	["Great Shark"] = true,
	["Mammoth Shark"] = true,
	["Lord Tirion Fordring"] = true,
	["Stormpike Guardsman"] = true,
	["Crimson Elite"] = true,
	["Onyxian Warder"] = true,
	["Guardian of Elune"] = true,
	["Ice Totem"] = true,
	["Son of Flame"] = true,
	["Winterax Axe Thrower"] = true,
	["Winterax Shadow Hunter"] = true,
	["Winterax Hunter"] = true,
	["Korrak the Bloodrager"] = true,
	["Shadowglen Sentinel"] = true,
	["Tortured Druid"] = true,
	["Tortured Sentinel"] = true,
	["Glordrum Steelbeard"] = true,
	["Martin Lindsey"] = true,
	["Shade of Ambermoon"] = true,
	["Princess Theradras"] = true,
	["Landslide"] = true,
	["Spitelash Raider"] = true,
	["Spitelash Witch"] = true,
	["Primordial Behemoth"] = true,
	["Thessala Hydra"] = true,
	["Conquered Soul of the Blightcaller"] = true,
	["Noxious Slime"] = true,
	["Cavern Shambler"] = true,
	["Celebras the Cursed"] = true,
	["Lord Vyletongue"] = true,
	["Meshlok the Harvester"] = true,
	["Zaetar's Spirit"] = true,
	["Spirit of Maraudos"] = true,
	["Spirit of Veng"] = true,
	["Infiltrator Hameya"] = true,
	["Zaeldarr the Outcast"] = true,
	["Mechanical Yeti"] = true,
	["Razorlash"] = true,
	["Gehennas"] = true,
	["Infected Mossflayer"] = true,
	["Ziggurat Protector"] = true,
	["Slaughterhouse Protector"] = true,
	["Shazzrah"] = true,
	["Lava Spawn"] = true,
	["Quel'Lithien Protector"] = true,
	["Crimson Courier"] = true,
	["Shadowprey Guardian"] = true,
	["Demetria"] = true,
	["Scarlet Trooper"] = true,
	["Wailing Spectre"] = true,
	["Damned Soul"] = true,
	["Unliving Caretaker"] = true,
	["Unliving Resident"] = true,
	["Ley Sprite"] = true,
	["Mana Sprite"] = true,
	["Augustus the Touched"] = true,
	["Large Vile Slime"] = true,
	["Doomguard Commander"] = true,
	["Lord Kazzak"] = true,
	["Blackwing Legionnaire"] = true,
	["Gordok Hyena"] = true,
	["Blackwing Mage"] = true,
	["Death Talon Dragonspawn"] = true,
	["Flint Shadowmore"] = true,
	["Razorgore the Untamed"] = true,
	["Blackwing Spellbinder"] = true,
	["Blackwing Taskmaster"] = true,
	["Blackwing Warlock"] = true,
	["Death Talon Wyrmguard"] = true,
	["Death Talon Overseer"] = true,
	["Death Talon Flamescale"] = true,
	["Death Talon Seether"] = true,
	["Death Talon Wyrmkin"] = true,
	["Death Talon Captain"] = true,
	["Death Talon Hatcher"] = true,
	["Arcanite Dragonling"] = true,
	["Emeraldon Boughguard"] = true,
	["Emeraldon Tree Warder"] = true,
	["Emeraldon Oracle"] = true,
	["Verdantine Boughguard"] = true,
	["Verdantine Oracle"] = true,
	["Verdantine Tree Warder"] = true,
	["Melris Malagan"] = true,
	["Justine Demalier"] = true,
	["Dreamtracker"] = true,
	["Dreamroarer"] = true,
	["Dreamstalker"] = true,
	["Grethok the Controller"] = true,
	["Jarrodenus"] = true,
	["Mishellena"] = true,
	["Reginald Windsor"] = true,
	["Bibilfaz Featherwhistle"] = true,
	["Vhulgra"] = true,
	["Khaelyn Steelwing"] = true,
	["Georgia"] = true,
	["Onyxia's Elite Guard"] = true,
	["Faustron"] = true,
	["Lady Onyxia"] = true,
	["Captain Dirgehammer"] = true,
	["Lieutenant Rachel Vaccar"] = true,
	["Archmage Gaiman"] = true,
	["Sergeant Major Skyshadow"] = true,
	["Master Sergeant Biggins"] = true,
	["Captain O'Neal"] = true,
	["Lieutenant Karter"] = true,
	["Lieutenant Jackspring"] = true,
	["Sergeant Major Clate"] = true,
	["Guard Quine"] = true,
	["Guard Hammon"] = true,
	["Legionnaire Teena"] = true,
	["Blood Guard Hini'wana"] = true,
	["Advisor Willington"] = true,
	["Chieftain Earthbind"] = true,
	["Lady Palanseer"] = true,
	["Brave Stonehide"] = true,
	["Stone Guard Zarg"] = true,
	["First Sergeant Hola'mahi"] = true,
	["Raider Bork"] = true,
	["Grunt Korf"] = true,
	["Grunt Bek'rah"] = true,
	["Sergeant Ba'sha"] = true,
	["Chimaerok"] = true,
	["Arcane Chimaerok"] = true,
	["Chimaerok Devourer"] = true,
	["Lord Lakmaeran"] = true,
	["Officer Areyn"] = true,
	["Greshka"] = true,
	["Axtroz"] = true,
	["Somnus"] = true,
	["Spirit Of Redemption"] = true,
	["Doctor Gregory Victor"] = true,
	["Imp Minion"] = true,
	["Critically Injured Soldier"] = true,
	["Critically Injured Alliance Soldier"] = true,
	["Doctor Gustaf VanHowzen"] = true,
	["Jase Farlane"] = true,
	["Leonard Porter"] = true,
	["Werg Thickblade"] = true,
	["Lokhtos Darkbargainer"] = true,
	["Zannok Hidepiercer"] = true,
	["Blimo Gadgetspring"] = true,
	["Gigget Zipcoil"] = true,
	["Nergal"] = true,
	["Vaelastrasz the Corrupt"] = true,
	["Warpwood Crusher"] = true,
	["Whip Lasher"] = true,
	["Gordok Mastiff"] = true,
	["Umi Thorson"] = true,
	["Keetar"] = true,
	["Irondeep Guard"] = true,
	["Irondeep Raider"] = true,
	["Bixi Wobblebonk"] = true,
	["Myrokos Silentform"] = true,
	["Aggi Rumblestomp"] = true,
	["Coldmine Invader"] = true,
	["Masha Swiftcut"] = true,
	["Coldmine Guard"] = true,
	["Coldmine Explorer"] = true,
	["Coldmine Surveyor"] = true,
	["Irondeep Surveyor"] = true,
	["Irondeep Explorer"] = true,
	["Alliance Spirit Guide"] = true,
	["Horde Spirit Guide"] = true,
	["Crimson Bodyguard"] = true,
	["Hive'Ashi Drone"] = true,
	["Lieutenant Rugba"] = true,
	["Lieutenant Spencer"] = true,
	["Commander Randolph"] = true,
	["Commander Dardosh"] = true,
	["Lieutenant Stronghoof"] = true,
	["Lieutenant Vol'talar"] = true,
	["Lieutenant Grummus"] = true,
	["Lieutenant Murp"] = true,
	["Lieutenant Lewis"] = true,
	["Syndicate Brigand"] = true,
	["Syndicate Agent"] = true,
	["Syndicate Master Ryson"] = true,
	["Commander Malgor"] = true,
	["Commander Mulfort"] = true,
	["Commander Louis Philips"] = true,
	["Deathstalker Agent"] = true,
	["Carrion Swarmer"] = true,
	["Aerie Gryphon"] = true,
	["Smith Regzar"] = true,
	["Vahgruk"] = true,
	["War Rider"] = true,
	["Wing Commander Guse"] = true,
	["Wing Commander Jeztor"] = true,
	["Wing Commander Mulverick"] = true,
	["Phase Lasher"] = true,
	["Fel Lash"] = true,
	["Gaelden Hammersmith"] = true,
	["Thanthaldis Snowgleam"] = true,
	["Grunnda Wolfheart"] = true,
	["Jekyll Flandring"] = true,
	["Layo Starstrike"] = true,
	["Ryson's Eye in the Sky"] = true,
	["Primalist Thurloga"] = true,
	["Lokholar the Ice Lord"] = true,
	["Murgot Deepforge"] = true,
	["Wildspawn Imp"] = true,
	["Duke Hydraxis"] = true,
	["Discordant Surge"] = true,
	["Hydrospawn"] = true,
	["Noxxion"] = true,
	["Lord Tony Romano"] = true,
	["Frostwolf Shaman"] = true,
	["Death Lash"] = true,
	["Lieutenant Largent"] = true,
	["Lieutenant Stouthandle"] = true,
	["Lieutenant Greywand"] = true,
	["Lieutenant Lonadin"] = true,
	["Lieutenant Mancuso"] = true,
	["Hive'Ashi Ambusher"] = true,
	["Coldmine Peon"] = true,
	["Coldmine Miner"] = true,
	["Commander Mortimer"] = true,
	["Commander Duffy"] = true,
	["Commander Karl Philips"] = true,
	["Hydraxian Honor Guard"] = true,
	["Subterranean Diemetradon"] = true,
	["Seasoned Guardsman"] = true,
	["Seasoned Mountaineer"] = true,
	["Seasoned Defender"] = true,
	["Seasoned Sentinel"] = true,
	["Seasoned Guardian"] = true,
	["Seasoned Legionnaire"] = true,
	["Seasoned Warrior"] = true,
	["Veteran Defender"] = true,
	["Veteran Guardian"] = true,
	["Veteran Guardsman"] = true,
	["Veteran Legionnaire"] = true,
	["Veteran Mountaineer"] = true,
	["Veteran Sentinel"] = true,
	["Veteran Warrior"] = true,
	["Stormpike Mine Layer"] = true,
	["Frostwolf Mine Layer"] = true,
	["Stormpike Bowman"] = true,
	["Frostwolf Bowman"] = true,
	["Master Engineer Zinfizzlex"] = true,
	["Frostwolf Shredder Unit"] = true,
	["Irondeep Miner"] = true,
	["Irondeep Peon"] = true,
	["Stormpike Shredder Unit"] = true,
	["Sagorne Creststrider"] = true,
	["Ivus the Forest Lord"] = true,
	["Champion Guardian"] = true,
	["Champion Defender"] = true,
	["Champion Guardsman"] = true,
	["Champion Legionnaire"] = true,
	["Champion Mountaineer"] = true,
	["Champion Sentinel"] = true,
	["Champion Warrior"] = true,
	["Wing Commander Ichman"] = true,
	["Wing Commander Slidore"] = true,
	["Wing Commander Vipore"] = true,
	["Frostwolf Wolf Rider"] = true,
	["Frostwolf Wolf Rider Commander"] = true,
	["Arch Druid Renferal"] = true,
	["Druid of the Grove"] = true,
	["Field Marshal Teravaine"] = true,
	["Corporal Noreg Stormpike"] = true,
	["Sergeant Yazra Bloodsnarl"] = true,
	["Warmaster Garrick"] = true,
	["Noxxion's Spawn"] = true,
	["Stormpike Commando"] = true,
	["Seasoned Commando"] = true,
	["Veteran Commando"] = true,
	["Champion Commando"] = true,
	["Frostwolf Reaver"] = true,
	["Seasoned Reaver"] = true,
	["Veteran Reaver"] = true,
	["Champion Reaver"] = true,
	["Seasoned Coldmine Guard"] = true,
	["Veteran Coldmine Guard"] = true,
	["Champion Coldmine Guard"] = true,
	["Seasoned Coldmine Surveyor"] = true,
	["Veteran Coldmine Surveyor"] = true,
	["Champion Coldmine Surveyor"] = true,
	["Seasoned Irondeep Explorer"] = true,
	["Veteran Irondeep Explorer"] = true,
	["Champion Irondeep Explorer"] = true,
	["Seasoned Irondeep Raider"] = true,
	["Veteran Irondeep Raider"] = true,
	["Champion Irondeep Raider"] = true,
	["Seasoned Coldmine Explorer"] = true,
	["Veteran Coldmine Explorer"] = true,
	["Champion Coldmine Explorer"] = true,
	["Seasoned Coldmine Invader"] = true,
	["Veteran Coldmine Invader"] = true,
	["Champion Coldmine Invader"] = true,
	["Seasoned Irondeep Guard"] = true,
	["Veteran Irondeep Guard"] = true,
	["Champion Irondeep Guard"] = true,
	["Seasoned Irondeep Surveyor"] = true,
	["Veteran Irondeep Surveyor"] = true,
	["Champion Irondeep Surveyor"] = true,
	["Stormpike Ram Rider"] = true,
	["Stormpike Ram Rider Commander"] = true,
	["Rotgrip"] = true,
	["Frostwolf Explosives Expert"] = true,
	["Stormpike Explosives Expert"] = true,
	["Stolid Snapjaw"] = true,
	["Tinkerer Gizlock"] = true,
	["Frostwolf Stable Master"] = true,
	["Stormpike Stable Master"] = true,
	["Stabled Frostwolf"] = true,
	["Stabled Alterac Ram"] = true,
	["Selendra"] = true,
	["Celebras the Redeemed"] = true,
	["Veng"] = true,
	["Maraudos"] = true,
	["Corporal Teeka Bloodsnarl"] = true,
	["Sergeant Durgen Stormpike"] = true,
	["Mountaineer Boombellow"] = true,
	["Jotek"] = true,
	["Prospector Stonehewer"] = true,
	["Voggah Deathgrip"] = true,
	["Royal Dreadguard"] = true,
	["Warmaster Laggrond"] = true,
	["Lieutenant Haggerdin"] = true,
	["Frostwolf Ambassador Rokhstrom"] = true,
	["Lieutenant Rotimer"] = true,
	["Scalebeard"] = true,
	["Dire Maul Crystal Totem"] = true,
	["Izzy Coppergrab"] = true,
	["Winterax Mystic"] = true,
	["Winterax Warrior"] = true,
	["Winterax Seer"] = true,
	["Alterac Yeti"] = true,
	["Blackwing Technician"] = true,
	["Chromaggus"] = true,
	["Corrupted Red Whelp"] = true,
	["Corrupted Green Whelp"] = true,
	["Corrupted Blue Whelp"] = true,
	["Corrupted Bronze Whelp"] = true,
	["Enraged Felguard"] = true,
	["Massive Geyser"] = true,
	["Bounty Hunter Kolark"] = true,
	["Artilleryman Sheldonore"] = true,
	["Najak Hexxen"] = true,
	["Ravak Grimtotem"] = true,
	["Athramanis"] = true,
	["Dirk Swindle"] = true,
	["Lord Angler"] = true,
	["Ironbark the Redeemed"] = true,
	["Blue Drakonid"] = true,
	["Green Drakonid"] = true,
	["Bronze Drakonid"] = true,
	["Red Drakonid"] = true,
	["Black Drakonid"] = true,
	["Frostwolf Bloodhound"] = true,
	["Stormpike Owl"] = true,
	["Stormpike Battleguard"] = true,
	["Frostwolf Battleguard"] = true,
	["Chromatic Drakonid"] = true,
	["Petrified Guardian"] = true,
	["Kor'kron Elite"] = true,
	["Ferra"] = true,
	["Guard Fengus"] = true,
	["Stomper Kreeg"] = true,
	["Guard Slip'kik"] = true,
	["Cho'Rush the Observer"] = true,
	["Captain Kromcrush"] = true,
	["Guard Mol'dar"] = true,
	["Lethtendris"] = true,
	["Field Repair Bot 74A"] = true,
	["Knot Thimblejack"] = true,
	["Death Howl"] = true,
	["Alshirr Banebreath"] = true,
	["Ragepaw"] = true,
	["Olm the Wise"] = true,
	["Mongress"] = true,
	["The Ongar"] = true,
	["Highlord Demitrian"] = true,
	["Earthcaller Franzahl"] = true,
	["Pimgib"] = true,
	["Hydroling"] = true,
	["Gordok Bushwacker"] = true,
	["Mizzle the Crafty"] = true,
	["Pusillin"] = true,
	["Azj'Tordin"] = true,
	["Shen'dralar Ancient"] = true,
	["Thornling"] = true,
	["Thief Catcher Shadowdelve"] = true,
	["Shen'dralar Spirit"] = true,
	["Thief Catcher Farmountain"] = true,
	["Thief Catcher Thunderbrew"] = true,
	["Lorekeeper Lydros"] = true,
	["Shen'dralar Zealot"] = true,
	["Cadaverous Worm"] = true,
	["Shen'dralar Provisioner"] = true,
	["Winterfall Ambusher"] = true,
	["Sage Korolusk"] = true,
	["Scholar Runethorn"] = true,
	["Scout Stronghand"] = true,
	["Scout Manslayer"] = true,
	["Scout Tharr"] = true,
	["Huntress Skymane"] = true,
	["Huntress Ravenoak"] = true,
	["Huntress Leafrunner"] = true,
	["Lorekeeper Javon"] = true,
	["Lorekeeper Mykos"] = true,
	["Lorekeeper Kildrath"] = true,
	["Doomguard Minion"] = true,
	["Wandering Eye of Kilrogg"] = true,
	["Lothos Riftwaker"] = true,
	["Rogue Black Drake"] = true,
	["Netherwalker"] = true,
	["Overlord Runthak"] = true,
	["Major Mattingly"] = true,
	["Griniblix the Spectator"] = true,
	["Eye of Immol'thar"] = true,
	["Mana Burst"] = true,
	["Eldreth Darter"] = true,
	["Arcane Torrent"] = true,
	["Arcane Feedback"] = true,
	["Master Elemental Shaper Krixix"] = true,
	["Seeker Cromwell"] = true,
	["Seeker Nahr"] = true,
	["Seeker Thompson"] = true,
	["Officer Jaxon"] = true,
	["Prince Thunderaan"] = true,
	["Mor'zul Bloodbringer"] = true,
	["Gorzeeki Wildeyes"] = true,
	["Officer Pomeroy"] = true,
	["Officer Brady"] = true,
	["Hunter Sagewind"] = true,
	["Hunter Ragetotem"] = true,
	["Hunter Thunderhorn"] = true,
	["Enslaved Doomguard Commander"] = true,
	["The Windreaver"] = true,
	["Whirling Invader"] = true,
	["Blackwing Guardsman"] = true,
	["Princess Tempestria"] = true,
	["Watery Invader"] = true,
	["Blazing Invader"] = true,
	["Baron Charr"] = true,
	["Thundering Invader"] = true,
	["Daio the Decrepit"] = true,
	["Avalanchion"] = true,
	["Kroshius"] = true,
	["Niby the Almighty"] = true,
	["Impsy"] = true,
	["Setis"] = true,
	["Gretheer"] = true,
	["Lapress"] = true,
	["Zora"] = true,
	["Rex Ashil"] = true,
	["Krellack"] = true,
	["Grubthor"] = true,
	["Huricanian"] = true,
	["Twilight Lord Everun"] = true,
	["Xorothian Imp"] = true,
	["Dread Guard"] = true,
	["Injured Peasant"] = true,
	["Plagued Peasant"] = true,
	["Scourge Footsoldier"] = true,
	["Scourge Archer"] = true,
	["Eris Havenfire"] = true,
	["J'eevee"] = true,
	["Xorothian Dreadsteed"] = true,
	["The Cleaner"] = true,
	["Dreadsteed Spirit"] = true,
	["Lord Hel'nurath"] = true,
	["High Priest Venoxis"] = true,
	["Short John Mithril"] = true,
	["High Priest Thekal"] = true,
	["High Priestess Mar'li"] = true,
	["Shadowed Spirit"] = true,
	["Corrupted Spirit"] = true,
	["Malicious Spirit"] = true,
	["Banal Spirit"] = true,
	["High Priestess Arlokk"] = true,
	["Death Knight Darkreaver"] = true,
	["High Priestess Jeklik"] = true,
	["Aspect of Banality"] = true,
	["Aspect of Corruption"] = true,
	["Aspect of Malice"] = true,
	["Aspect of Shadow"] = true,
	["Ur'dan"] = true,
	["Ulathek"] = true,
	["Vartrus the Ancient"] = true,
	["Stoma the Ancient"] = true,
	["Hastat the Ancient"] = true,
	["Simone the Inconspicuous"] = true,
	["Precious"] = true,
	["Franklin the Friendly"] = true,
	["Solenor the Slayer"] = true,
	["Artorius the Amiable"] = true,
	["Razzashi Venombrood"] = true,
	["Simone the Seductress"] = true,
	["Klinfran the Crazed"] = true,
	["Artorius the Doombringer"] = true,
	["Nelson the Nice"] = true,
	["Precious the Devourer"] = true,
	["Terrordale Spirit"] = true,
	["Ancient Equine Spirit"] = true,
	["Derotain Mudsipper"] = true,
	["Darkreaver's Fallen Charger"] = true,
	["Sergeant Thunderhorn"] = true,
	["Ebonroc"] = true,
	["Zapped Shore Strider"] = true,
	["Zapped Land Walker"] = true,
	["Bone Construct"] = true,
	["Overseer Maltorius"] = true,
	["Thorium Brotherhood Lookout"] = true,
	["Master Smith Burninate"] = true,
	["Overseer Oilfist"] = true,
	["Taskmaster Scrange"] = true,
	["Hansel Heavyhands"] = true,
	["Evonice Sootsmoker"] = true,
	["Lookout Captain Lolo Longstriker"] = true,
	["Zapped Wave Strider"] = true,
	["Zapped Deep Strider"] = true,
	["Zapped Cliff Giant"] = true,
	["Stinglasher"] = true,
	["Corrupted Fire Nova Totem V"] = true,
	["Corrupted Stoneskin Totem VI"] = true,
	["Corrupted Healing Stream Totem V"] = true,
	["Corrupted Windfury Totem III"] = true,
	["Corrupted Infernal"] = true,
	["Balzaphon"] = true,
	["Revanchion"] = true,
	["Scorn"] = true,
	["Lord Blackwood"] = true,
	["Silverwing Elite"] = true,
	["Horde Elite"] = true,
	["High Overlord Saurfang"] = true,
	["Field Marshal Afrasiabi"] = true,
	["Revantusk Watcher"] = true,
	["Lard"] = true,
	["PvP CTF Credit Marker"] = true,
	["Sentinel Farsong"] = true,
	["Revantusk Drummer"] = true,
	["Primal Torntusk"] = true,
	["Mystic Yayo'jin"] = true,
	["Zap Farflinger"] = true,
	["Jhordy Lapforge"] = true,
	["Vilebranch Kidnapper"] = true,
	["Gurubashi Bat Rider"] = true,
	["Illiyana Moonblaze"] = true,
	["Kelm Hargunth"] = true,
	["Elder Torntusk"] = true,
	["Creeping Doom"] = true,
	["Dun Baldar North Marshal"] = true,
	["Dun Baldar South Marshal"] = true,
	["Icewing Marshal"] = true,
	["Stonehearth Marshal"] = true,
	["Iceblood Marshal"] = true,
	["Tower Point Marshal"] = true,
	["East Frostwolf Marshal"] = true,
	["West Frostwolf Marshal"] = true,
	["Dun Baldar North Warmaster"] = true,
	["Dun Baldar South Warmaster"] = true,
	["East Frostwolf Warmaster"] = true,
	["Iceblood Warmaster"] = true,
	["Icewing Warmaster"] = true,
	["Stonehearth Warmaster"] = true,
	["Tower Point Warmaster"] = true,
	["West Frostwolf Warmaster"] = true,
	["Captain Shatterskull"] = true,
	["Razzashi Raptor"] = true,
	["Sayge"] = true,
	["Silas Darkmoon"] = true,
	["Withered Mistress"] = true,
	["Sacrificed Troll"] = true,
	["Burth"] = true,
	["Yebb Neblegear"] = true,
	["Kerri Hicks"] = true,
	["Chronos"] = true,
	["Hakkar"] = true,
	["Rinling"] = true,
	["Blood Steward of Kirtonos"] = true,
	["Emissary Roman'khan"] = true,
	["Molthor"] = true,
	["Zandalar Headshrinker"] = true,
	["Razzashi Skitterer"] = true,
	["Atal'ai Mistress"] = true,
	["Voodoo Slave"] = true,
	["Parasitic Serpent"] = true,
	["Ysondre"] = true,
	["Lethon"] = true,
	["Emeriss"] = true,
	["Taerar"] = true,
	["Jin'rokh the Breaker"] = true,
	["Al'tabim the All-Seeing"] = true,
	["Maywiki of Zuldazar"] = true,
	["Falthir the Sightless"] = true,
	["Exzhal"] = true,
	["Zandalar Enforcer"] = true,
	["Captured Hakkari Zealot"] = true,
	["Rin'wosho the Trader"] = true,
	["Kartra Bloodsnarl"] = true,
	["Guse's War Rider"] = true,
	["Jeztor's War Rider"] = true,
	["Mulverick's War Rider"] = true,
	["Slidore's Gryphon"] = true,
	["Ichman's Gryphon"] = true,
	["Vipore's Gryphon"] = true,
	["Frenzied Bloodseeker Bat"] = true,
	["Elfarran"] = true,
	["Lylandris"] = true,
	["Field Marshal Oslight"] = true,
	["Sergeant Maclear"] = true,
	["Shade of Jin'do"] = true,
	["Powerful Healing Ward"] = true,
	["Ohgan"] = true,
	["Deze Snowbane"] = true,
	["Sir Malory Wheeler"] = true,
	["Lady Hoteshem"] = true,
	["Wagner Hammerstrike"] = true,
	["Javnir Nashak"] = true,
	["Deathmaster Dwire"] = true,
	["Deathstalker Mortis"] = true,
	["Spawn of Mar'li"] = true,
	["Zanza the Restless"] = true,
	["Zulian Crocolisk"] = true,
	["Zulian Stalker"] = true,
	["Zulian Guardian"] = true,
	["Vinchaxa"] = true,
	["Zandalarian Emissary"] = true,
	["Riggle Bassbait"] = true,
	["Jang"] = true,
	["Fishbot 5000"] = true,
	["Servant of the Hand"] = true,
	["Gri'lek"] = true,
	["Hazza'rah the Dreamweaver"] = true,
	["Renataki"] = true,
	["Wushoolay"] = true,
	["Booty Bay Elite"] = true,
	["Zulian Prowler"] = true,
	["Stormpike Emissary"] = true,
	["Frostwolf Emissary"] = true,
	["Gurubashi Prisoner"] = true,
	["Mad Servant"] = true,
	["Brain Wash Totem"] = true,
	["Honored Hero"] = true,
	["Gahz'ranka"] = true,
	["Honored Ancestor"] = true,
	["Grinkle"] = true,
	["Chained Spirit"] = true,
	["Barrus"] = true,
	["Rutherford Twing"] = true,
	["Samuel Hawke"] = true,
	["Defiler Elite"] = true,
	["Hammerfall Elite"] = true,
	["Menethil Elite"] = true,
	["Silverpine Elite"] = true,
	["Mad Voidwalker"] = true,
	["Scarlet Inquisitor"] = true,
	["Nightmare Illusion"] = true,
	["Haughty Modiste"] = true,
	["Vile Scarab"] = true,
	["Ralo'shan the Eternal Watcher"] = true,
	["Rutgar Glyphshaper"] = true,
	["Frankal Stonebridge"] = true,
	["Glibb"] = true,
	["Calandrath"] = true,
	["Khur Hornstriker"] = true,
	["Vargus"] = true,
	["Cloud Skydancer"] = true,
	["Runk Windtamer"] = true,
	["Mishta"] = true,
	["Baristolth of the Shifting Sands"] = true,
	["Commander Mar'alith"] = true,
	["Vish Kozus"] = true,
	["Geologist Larksbane"] = true,
	["Cenarion Hold Infantry"] = true,
	["Brood of Nozdormu"] = true,
	["Cenarion Emissary Jademoon"] = true,
	["Cenarion Emissary Blackhoof"] = true,
	["Beetix Ficklespragg"] = true,
	["Noggle Ficklespragg"] = true,
	["Windcaller Proudhorn"] = true,
	["Anachronos"] = true,
	["Hermit Ortell"] = true,
	["Wickerman Guardian"] = true,
	["Deathclasp"] = true,
	["Darkcaller Yanka"] = true,
	["Sergeant Hartman"] = true,
	["Twilight Keeper Mayna"] = true,
	["Twilight Flamereaver"] = true,
	["Vyral the Vile"] = true,
	["Prince Skaldrenox"] = true,
	["High Marshal Whirlaxis"] = true,
	["Baron Kazum"] = true,
	["The Duke of Cynders"] = true,
	["The Duke of Fathoms"] = true,
	["The Duke of Shards"] = true,
	["Crimson Templar"] = true,
	["Azure Templar"] = true,
	["Hoary Templar"] = true,
	["Twilight Overlord"] = true,
	["Mistress Natalia Mar'alith"] = true,
	["The Duke of Zephyrs"] = true,
	["Vekniss Soldier"] = true,
	["Vekniss Warrior"] = true,
	["Vekniss Guardian"] = true,
	["Vekniss Stinger"] = true,
	["Vekniss Wasp"] = true,
	["Vekniss Hive Crawler"] = true,
	["Qiraji Mindslayer"] = true,
	["Qiraji Brainwasher"] = true,
	["Qiraji Lasher"] = true,
	["Qiraji Slayer"] = true,
	["Qiraji Champion"] = true,
	["Demented Druid Spirit"] = true,
	["Spirit Shade"] = true,
	["Obsidian Eradicator"] = true,
	["The Prophet Skeram"] = true,
	["Anubisath Sentinel"] = true,
	["Huum Wildmane"] = true,
	["Emperor Vek'nilash"] = true,
	["Emperor Vek'lor"] = true,
	["Anubisath Defender"] = true,
	["Aurel Goldleaf"] = true,
	["Xil'xix"] = true,
	["Aluntir"] = true,
	["Arakis"] = true,
	["Aendel Windspear"] = true,
	["Viscidus"] = true,
	["Vekniss Drone"] = true,
	["Shade of Taerar"] = true,
	["Lord Skwol"] = true,
	["Bor Wildmane"] = true,
	["Earthen Templar"] = true,
	["Twilight Prophet"] = true,
	["Anubisath Warder"] = true,
	["Obsidian Nullifier"] = true,
	["Mylini Frostmoon"] = true,
	["Qiraji Scarab"] = true,
	["Qiraji Scorpion"] = true,
	["Hive'Zara Drone"] = true,
	["Hive'Zara Collector"] = true,
	["Hive'Zara Soldier"] = true,
	["Hive'Zara Sandstalker"] = true,
	["Qiraji Gladiator"] = true,
	["Hive'Zara Wasp"] = true,
	["Hive'Zara Stinger"] = true,
	["Silicate Feeder"] = true,
	["Giant Eye Tentacle"] = true,
	["Flesh Hunter"] = true,
	["Hive'Zara Tail Lasher"] = true,
	["Obsidian Destroyer"] = true,
	["Ossirian the Unscarred"] = true,
	["Moam"] = true,
	["General Rajaxx"] = true,
	["Qiraji Swarmguard"] = true,
	["Swarmguard Needler"] = true,
	["Kurinnaxx"] = true,
	["Horde Warbringer"] = true,
	["Alliance Brigadier General"] = true,
	["Anubisath Guardian"] = true,
	["Tonk Mine"] = true,
	["Ayamiss the Hunter"] = true,
	["Buru the Gorger"] = true,
	["Merithra of the Dream"] = true,
	["Caelestrasz"] = true,
	["Arygos"] = true,
	["Anachronos the Ancient"] = true,
	["Fandral Staghelm"] = true,
	["Sergeant Stonebrow"] = true,
	["OLDWorld Trigger"] = true,
	["Colonel Zerran"] = true,
	["Major Yeggeth"] = true,
	["Qiraji Warrior"] = true,
	["Major Pakkon"] = true,
	["Captain Drenn"] = true,
	["Captain Xurrem"] = true,
	["Captain Qeez"] = true,
	["Captain Tuubid"] = true,
	["Nafien"] = true,
	["Qiraji Wasp"] = true,
	["Kania"] = true,
	["Qiraji Drone"] = true,
	["Qiraji Tank"] = true,
	["Kaldorei Infantry"] = true,
	["Anubisath Conqueror"] = true,
	["Ahn'Qiraj Trigger"] = true,
	["Sand Vortex"] = true,
	["Corporal Carnes"] = true,
	["Dame Twinbraid"] = true,
	["Private Draxlegauge"] = true,
	["Master Nightsong"] = true,
	["Captain Blackanvil"] = true,
	["Ironforge Brigade Rifleman"] = true,
	["Ironforge Brigade Footman"] = true,
	["Janela Stouthammer"] = true,
	["Arcanist Nozzlespring"] = true,
	["Sergeant Major Germaine"] = true,
	["Bonnie Stoneflayer"] = true,
	["Private Porter"] = true,
	["Hive'Zora Abomination"] = true,
	["Marta Finespindle"] = true,
	["Sentinel Silversky"] = true,
	["Nurse Stonefield"] = true,
	["Keeper Moonshade"] = true,
	["Slicky Gastronome"] = true,
	["Sarah Sadwhistle"] = true,
	["Huntress Swiftriver"] = true,
	["Commander Stronghammer"] = true,
	["Miner Cromwell"] = true,
	["Grunt Maug"] = true,
	["Shrieker Scarab"] = true,
	["Spitting Scarab"] = true,
	["Grace of Air Totem III"] = true,
	["Strength of Earth Totem V"] = true,
	["Minion of Omen"] = true,
	["Omen"] = true,
	["Senior Sergeant T'kelah"] = true,
	["Lieutenant General Andorov"] = true,
	["Kaldorei Elite"] = true,
	["Herbalist Proudfeather"] = true,
	["Spirit of Azuregos"] = true,
	["Eranikus, Tyrant of the Dream"] = true,
	["Nighthaven Defender"] = true,
	["Windcaller Yessendra"] = true,
	["Warden Haro"] = true,
	["Keyl Swiftclaw"] = true,
	["Andorgos"] = true,
	["Kandrostrasz"] = true,
	["Vethsera"] = true,
	["Canal Frenzy"] = true,
	["Batrider Pele'keiki"] = true,
	["Princess Huhuran"] = true,
	["Fankriss the Unyielding"] = true,
	["Lord Kri"] = true,
	["Buru Egg"] = true,
	["Skinner Jamani"] = true,
	["Battleguard Sartura"] = true,
	["Ouro"] = true,
	["Hive'Zara Hatchling"] = true,
	["Meridith the Mermaiden"] = true,
	["Mana Fiend"] = true,
	["Healer Longrunner"] = true,
	["Bloodguard Rawtar"] = true,
	["Fisherman Lin'do"] = true,
	["Chief Sharpclaw"] = true,
	["Anubisath Warrior"] = true,
	["Anubisath Swarmguard"] = true,
	["General Zog"] = true,
	["Windcaller Kaldon"] = true,
	["Twilight Marauder Morna"] = true,
	["Twilight Marauder"] = true,
	["Princess Yauj"] = true,
	["Vem"] = true,
	["Cenarion Outrider"] = true,
	["Hive'Zara Swarmer"] = true,
	["Elder Morndeep"] = true,
	["Doctor Weavil"] = true,
	["Doctor Weavil's Flying Machine"] = true,
	["Number Two"] = true,
	["Hive'Zara Larva"] = true,
	["Elder Splitrock"] = true,
	["Elder Rumblerock"] = true,
	["Elder Silvervein"] = true,
	["Elder Highpeak"] = true,
	["Elder Stonefort"] = true,
	["Elder Obsidian"] = true,
	["Elder Hammershout"] = true,
	["Elder Bellowrage"] = true,
	["Elder Darkcore"] = true,
	["Elder Stormbrow"] = true,
	["Elder Snowcrown"] = true,
	["Elder Ironband"] = true,
	["Elder Graveborn"] = true,
	["Elder Goldwell"] = true,
	["Elder Primestone"] = true,
	["Maws"] = true,
	["Elder Runetotem"] = true,
	["Elder Ragetotem"] = true,
	["Elder Stonespire"] = true,
	["Elder Bloodhoof"] = true,
	["Elder Winterhoof"] = true,
	["Elder Skychaser"] = true,
	["Elder Wildmane"] = true,
	["Elder Darkhorn"] = true,
	["Elder Proudhorn"] = true,
	["Elder Grimtotem"] = true,
	["Elder Windtotem"] = true,
	["Elder Thunderhorn"] = true,
	["Elder Skyseer"] = true,
	["Elder Dawnstrider"] = true,
	["Elder Dreamseer"] = true,
	["Elder Mistwalker"] = true,
	["Elder High Mountain"] = true,
	["Eye of C'Thun"] = true,
	["Ossirian Crystal Trigger"] = true,
	["Minion of Weavil"] = true,
	["Elder Windrun"] = true,
	["Elder Starsong"] = true,
	["Elder Moonstrike"] = true,
	["Elder Moonstrike"] = true,
	["Elder Starglade"] = true,
	["Elder Moonwarden"] = true,
	["Elder Bladeswift"] = true,
	["Elder Bladesing"] = true,
	["Elder Skygleam"] = true,
	["Elder Starweave"] = true,
	["Elder Meadowrun"] = true,
	["Elder Nightwind"] = true,
	["Elder Morningdew"] = true,
	["Elder Riversong"] = true,
	["Elder Brightspear"] = true,
	["Elder Farwhisper"] = true,
	["Cenarion Scout Landion"] = true,
	["Cenarion Scout Azenel"] = true,
	["Cenarion Scout Jalia"] = true,
	["Krug Skullsplit"] = true,
	["Merok Longstride"] = true,
	["J.D. Shadesong"] = true,
	["Shadow Priestess Shai"] = true,
	["Orgrimmar Legion Grunt"] = true,
	["Orgrimmar Legion Axe Thrower"] = true,
	["Hive'Regal Hunter-Killer"] = true,
	["Yauj Brood"] = true,
	["Vekniss Borer"] = true,
	["Xandivious"] = true,
	["Twilight Corrupter"] = true,
	["Eranikus the Redeemed"] = true,
	["Nightmare Phantasm"] = true,
	["Spawn of Fankriss"] = true,
	["Tyrande"] = true,
	["Priestess of the Moon"] = true,
	["Auctioneer Jaxon"] = true,
	["Glob of Viscidus"] = true,
	["Auctioneer Stockton"] = true,
	["Auctioneer Yarly"] = true,
	["Auctioneer Graves"] = true,
	["Auctioneer Silva'las"] = true,
	["Auctioneer Cazarez"] = true,
	["Auctioneer O'reely"] = true,
	["Auctioneer Cain"] = true,
	["Auctioneer Naxxremis"] = true,
	["Auctioneer Tricket"] = true,
	["Auctioneer Rhyker"] = true,
	["Jonathan the Revelator"] = true,
	["Vek Twins Trigger"] = true,
	["Warlord Gorchuk"] = true,
	["Field Marshal Snowfall"] = true,
	["Senior Sergeant Taiga"] = true,
	["Senior Sergeant Grimsford"] = true,
	["Senior Sergeant Kai'jin"] = true,
	["Master Sergeant Fizzlebolt"] = true,
	["Master Sergeant Maclure"] = true,
	["Master Sergeant Moonshadow"] = true,
	["Ouro Scarab"] = true,
	["Timbermaw Ancestor"] = true,
	["Squire Leoren Mal'derath"] = true,
	["Drunken Bruiser"] = true,
	["Claw Tentacle"] = true,
	["Eye Tentacle"] = true,
	["C'Thun"] = true,
	["Giant Claw Tentacle"] = true,
	["Darnassus Commendation Officer"] = true,
	["Gnomeregan Commendation Officer"] = true,
	["Ironforge Commendation Officer"] = true,
	["Stormwind Commendation Officer"] = true,
	["Orgrimmar Commendation Officer"] = true,
	["Darkspear Commendation Officer"] = true,
	["Undercity Commendation Officer"] = true,
	["Thunder Bluff Commendation Officer"] = true,
	["Colossus of Zora"] = true,
	["Colossus of Regal"] = true,
	["Colossus of Ashi"] = true,
	["Colossal Anubisath Warbringer"] = true,
	["Imperial Qiraji Destroyer"] = true,
	["Qiraji Brigadier General"] = true,
	["Greater Anubisath Warbringer"] = true,
	["Greater Silithid Flayer"] = true,
	["Qiraji Lieutenant General"] = true,
	["Supreme Anubisath Warbringer"] = true,
	["Supreme Silithid Flayer"] = true,
	["Officer Vu'Shalay"] = true,
	["Officer Lunalight"] = true,
	["Officer Porterhouse"] = true,
	["Officer Ironbeard"] = true,
	["Officer Redblade"] = true,
	["Officer Maloof"] = true,
	["Officer Thunderstrider"] = true,
	["Officer Gothena"] = true,
	["Christmas Emperor Dagran Thaurissan"] = true,
	["Colossus Researcher Sophia"] = true,
	["Colossus Researcher Nestor"] = true,
	["Colossus Researcher Eazel"] = true,
	["Flesh Tentacle"] = true,
	["Tranquil Air Totem"] = true,
	["Qiraji Major He'al-ie"] = true,
	["Qiraji Brigadier General Pax-lish"] = true,
	["Lieutenant General Nokhor"] = true,
	["Orgrimmar Elite Shieldguard"] = true,
	["Orgrimmar Elite Infantryman"] = true,
	["Orgrimmar Elite Cavalryman"] = true,
	["Tauren Rifleman"] = true,
	["Tauren Primalist"] = true,
	["Stormwind Cavalryman"] = true,
	["Stormwind Infantryman"] = true,
	["Stormwind Archmage"] = true,
	["Kaldorei Marksman"] = true,
	["Ironforge Infantryman"] = true,
	["Ironforge Cavalryman"] = true,
	["Darkspear Shaman"] = true,
	["Valadar Starsong"] = true,
	["High Commander Lynore Windstryke"] = true,
	["Highlord Leoric Von Zeldig"] = true,
	["Malagav the Tactician"] = true,
	["Duke August Foehammer"] = true,
	["Elder Bronzebeard"] = true,
	["Pat's Firework Guy - RED"] = true,
	["Pat's Firework Guy - YELLOW"] = true,
	["Lunar Festival Herald"] = true,
	["Lunar Festival Emissary"] = true,
	["Lunar Firework Credit Marker"] = true,
	["Lunar Cluster Credit Marker"] = true,
	["Lunar Festival Harbinger"] = true,
	["Lunar Festival Vendor"] = true,
	["Giant Spotlight"] = true,
	["Sergeant Carnes"] = true,
	["Fariel Starsong"] = true,
	["Pat's Firework Cluster Guy (ELUNE)"] = true,
	["Thaddius"] = true,
	["Stalagg"] = true,
	["Feugen"] = true,
	["Grobbulus"] = true,
	["Gluth"] = true,
	["Poison Cloud"] = true,
	["Hive'Zara Hornet"] = true,
	["Heigan the Unclean"] = true,
	["Maexxna"] = true,
	["Grand Widow Faerlina"] = true,
	["Noth the Plaguebringer"] = true,
	["Anub'Rekhan"] = true,
	["Ouro Spawner"] = true,
	["Lunar Festival Sentinel"] = true,
	["Vekniss Hatchling"] = true,
	["The Master's Eye"] = true,
	["Buru Egg Trigger"] = true,
	["Dread Creeper"] = true,
	["Carrion Spinner"] = true,
	["Venom Stalker"] = true,
	["Poisonous Skitterer"] = true,
	["Crypt Reaver"] = true,
	["Tomb Horror"] = true,
	["Naxxramas Cultist"] = true,
	["Naxxramas Acolyte"] = true,
	["Sartura's Royal Guard"] = true,
	["Sapphiron"] = true,
	["Kel'Thuzad"] = true,
	["InCombat Trigger"] = true,
	["Loatheb"] = true,
	["Mokvar"] = true,
	["Deliana"] = true,
	["Mux Manascrambler"] = true,
	["Vi'el"] = true,
	["Anthion Harmon"] = true,
	["Patchwork Golem"] = true,
	["Bile Retcher"] = true,
	["Mad Scientist"] = true,
	["Living Monstrosity"] = true,
	["Surgical Assistant"] = true,
	["Embalming Slime"] = true,
	["Stitched Spewer"] = true,
	["Living Poison"] = true,
	["Patchwerk"] = true,
	["Sludge Belcher"] = true,
	["Ysida Harmon"] = true,
	["Falrin Treeshaper"] = true,
	["Bodley"] = true,
	["Plague Beast"] = true,
	["Frenzied Bat"] = true,
	["Plagued Bat"] = true,
	["Lord Valthalak"] = true,
	["Magma Lord Bokk"] = true,
	["Lefty"] = true,
	["Rotfang"] = true,
	["Snokh Blackspine"] = true,
	["Malgen Longspear"] = true,
	["Korv"] = true,
	["Rezznik"] = true,
	["Va'jashni"] = true,
	["Diseased Maggot"] = true,
	["Rotting Maggot"] = true,
	["Volida"] = true,
	["Theldren"] = true,
	["Gothik the Harvester"] = true,
	["Instructor Razuvious"] = true,
	["Highlord Mograine"] = true,
	["Sir Zeliek"] = true,
	["Thane Korth'azz"] = true,
	["Lady Blaumeux"] = true,
	["Spectral Assassin"] = true,
	["Deathcharger Steed"] = true,
	["Tidelord Rrurgaz"] = true,
	["Spirit of Lord Valthalak"] = true,
	["Mor Grayhoof"] = true,
	["Naxxramas Trigger"] = true,
	["Rousch"] = true,
	["Dirk Thunderwood"] = true,
	["Spectral Stalker"] = true,
	["Gnashjaw"] = true,
	["Steamwheedle Bruiser"] = true,
	["Isalien"] = true,
	["Empyrean"] = true,
	["Jarien"] = true,
	["Sothos"] = true,
	["Spirit of Jarien"] = true,
	["Spirit of Sothos"] = true,
	["Korfax, Champion of the Light"] = true,
	["Father Inigo Montoy"] = true,
	["Scarlet Commander Marjhan"] = true,
	["Commander Eligor Dawnbringer"] = true,
	["Archmage Angela Dosantos"] = true,
	["Plagued Swine"] = true,
	["Kormok"] = true,
	["Bone Minion"] = true,
	["Bone Mage"] = true,
	["Unrelenting Trainee"] = true,
	["Unrelenting Deathknight"] = true,
	["Unrelenting Rider"] = true,
	["Spectral Trainee"] = true,
	["Shadow Fissure"] = true,
	["Rohan the Assassin"] = true,
	["Huntsman Leopold"] = true,
	["Mataus the Wrathcaster"] = true,
	["Rimblat Earthshatter"] = true,
	["Rayne"] = true,
	["Naxxramas Military Sub-Boss Trigger"] = true,
	["Cenarion Hold Reservist"] = true,
	["Ghoul Berserker"] = true,
	["Bile Sludge"] = true,
	["Deathknight Captain"] = true,
	["Deathknight"] = true,
	["Spectral Deathknight"] = true,
	["Spectral Horse"] = true,
	["Spectral Rider"] = true,
	["Risen Deathknight"] = true,
	["Dark Touched Warrior"] = true,
	["Doom Touched Warrior"] = true,
	["Death Touched Warrior"] = true,
	["Deathknight Cavalier"] = true,
	["Shade of Naxxramas"] = true,
	["Necro Knight"] = true,
	["Bony Construct"] = true,
	["Stoneskin Gargoyle"] = true,
	["Nerubian Overseer"] = true,
	["Skeletal Smith"] = true,
	["Unholy Axe"] = true,
	["Dispatch Commander Metz"] = true,
	["Unholy Staff"] = true,
	["Unholy Swords"] = true,
	["Tesla Coil"] = true,
	["Guard Didier"] = true,
	["Bragok"] = true,
	["Argent Dawn Infantry"] = true,
	["Injured Argent Dawn Infantry"] = true,
	["Caravan Mule"] = true,
	["Eye Stalk"] = true,
	["Plague Slime"] = true,
	["Infectious Ghoul"] = true,
	["Field Marshal Chambers"] = true,
	["Jessica Chambers"] = true,
	["Packmaster Stonebruiser"] = true,
	["Argent Medic"] = true,
	["Spore"] = true,
	["Fallout Slime"] = true,
	["Mutated Grub"] = true,
	["Skeletal Shocktrooper"] = true,
	["Argent Messenger"] = true,
	["Zombie Chow"] = true,
	["Commander Thomas Helleran"] = true,
	["Master Craftsman Omarion"] = true,
	["Necropolis Acolyte"] = true,
	["Sewage Slime"] = true,
	["Craftsman Wilhelm"] = true,
	["Argent Sentry"] = true,
	["Spirit of the Damned"] = true,
	["Bone Witch"] = true,
	["Archmage Tarsis Kir-Moldir"] = true,
	["Flameshocker"] = true,
	["Lightning Totem"] = true,
	["Atiesh"] = true,
	["Deathchill Servant"] = true,
	["Captain Armando Ossex"] = true,
	["Toxic Tunnel"] = true,
	["Necropolis"] = true,
	["Bronn Fitzwrench"] = true,
	["Ghost of Naxxramas"] = true,
	["Soldier of the Frozen Wastes"] = true,
	["Unstoppable Abomination"] = true,
	["Soul Weaver"] = true,
	["Cracked Necrotic Crystal"] = true,
	["Guardian of Icecrown"] = true,
	["Plagued Gargoyle"] = true,
	["Plagued Ghoul"] = true,
	["Plagued Deathhound"] = true,
	["Spirit of Naxxramas"] = true,
	["Deathknight Vindicator"] = true,
	["Necro Knight Guardian"] = true,
	["Necro Stalker"] = true,
	["Web Wrap"] = true,
	["Naxxramas Follower"] = true,
	["Naxxramas Worshipper"] = true,
	["Argent Warhorse"] = true,
	["Faint Necrotic Crystal"] = true,
	["Garon Hutchins"] = true,
	["Crypt Guard"] = true,
	["Corpse Scarab"] = true,
	["Spirit of Mograine"] = true,
	["Spirit of Blaumeux"] = true,
	["Spirit of Zeliek"] = true,
	["Spirit of Korth'azz"] = true,
	["Festival Flamekeeper"] = true,
	["Deathknight Understudy"] = true,
	["Festival Loremaster"] = true,
	["Festival Talespinner"] = true,
	["Death Lord"] = true,
	["The Lich King"] = true,
	["Plagued Guardian"] = true,
	["Plagued Construct"] = true,
	["Plagued Champion"] = true,
	["Plagued Warrior"] = true,
	["Festival Flamekeeper Costume: Human"] = true,
	["Orgrimmar Fireeater"] = true,
	["Maexxna Spiderling"] = true,
	["Chief Expeditionary Requisitioner Enkles"] = true,
	["Emissary Whitebeard"] = true,
	["Apothecary Quinard"] = true,
	["Emissary Gormok"] = true,
	["General Kirika"] = true,
	["Marshal Bluewall"] = true,
	["Scout Bloodfist"] = true,
	["Rifleman Torrig"] = true,
	["William Kielar"] = true,
	["Landro Longshot"] = true,
	["Felguard"] = true,
	["Renn'az"] = true,
	["Lordaeron Commander"] = true,
	["Lordaeron Soldier"] = true,
	["Alliance Silithyst Sentinel"] = true,
	["Horde Silithyst Sentinel"] = true,
	["Squire Rowe"] = true,
	["Lordaeron Veteran"] = true,
	["Lordaeron Fighter"] = true,
	["The Bug Family"] = true,
	["The Twin Emperors"] = true,
	["The Four Horsemen"] = true,
	["Environment"] = true,
	["Unknown"] = true,
}


NPCDB:RegisterTranslations("deDE", function()
	return {
		["Waypoint"] = "Wegpunkt (Nur für GMs sichtbar) ",
		["Spawnpoint"] = "Spawn Point (Only GM can see it)",
		["Stormwind City Guard"] = "Stadtwache von Sturmwind",
		["Infernal"] = "Höllenbestie",
		["Blue Dragonspawn"] = "Blaudrachenbrut",
		["Earth Elemental"] = "Erdelementar",
		["Maginor Dumas"] = "Maginor Dumas ",
		["Master Mathias Shaw"] = "Meister Mathias Shaw ",
		["Kendor Kabonka"] = "Kendor Kabonka ",
		["Grizzle Halfmane"] = "Grizzle Halbskalp ",
		["Dungar Longdrink"] = "Dungar Trinkfest ",
		["High Priestess Laurena"] = "Hohepriesterin Laurena ",
		["Felhunter"] = "Teufelsjäger",
		["Demisette Cloyce"] = "Demisette Cloyce ",
		["General Marcus Jonathan"] = "General Marcus Jonathan ",
		["Lieutenant Doren"] = "Leutnant Doren",
		["Water Elemental"] = "Wasserelementar",
		["Thor"] = "Thor ",
		["Grimtooth"] = "Grimmzahn",
		["El Pollo Grande"] = "El Pollo Grande ",
		["Mai'Zoth"] = "Mai'Zoth",
		["Coldridge Mountaineer"] = "Gebirgsjäger der Eisklamm",
		["Donal Osgood"] = "Donal Gutmark ",
		["Stonard Hunter"] = "Jäger von Steinard",
		["Stonard Orc"] = "Orc von Steinard",
		["Stonard Wayfinder"] = "Wegfinder von Steinard",
		["Stonard Grunt"] = "Grunzer von Steinard",
		["Stonard Cartographer"] = "Kartograf von Steinard",
		["Stonard Shaman"] = "Schamane von Steinard",
		["Keras Wolfheart"] = "Keras Wolfsherz ",
		["Ander Germaine"] = "Ander Germaine ",
		["Osborne the Night Man"] = "Osborne der Nachtmensch ",
		["Lord Grayson Shadowbreaker"] = "Lord Grayson Schattenbruch ",
		["Ariena Stormfeather"] = "Ariena Sturmfeder ",
		["Swamp Talker"] = "Sumpfredner",
		["Brother Neals"] = "Bruder Neals",
		["Malosh"] = "Malosh ",
		["Haromm"] = "Haromm ",
		["Ogromm"] = "Ogromm ",
		["Kartosh"] = "Kartosh ",
		["Banalash"] = "Banalash ",
		["Red Dragonspawn"] = "Rotdrachenbrut",
		["Red Wyrmkin"] = "Roter Wyrmkin",
		["Red Scalebane"] = "Rote Bannschuppe",
		["Scalebane Lieutenant"] = "Bannschuppenleutnant",
		["Wyrmkin Firebrand"] = "Feuerbrandwyrmkin",
		["Scalebane Royal Guard"] = "Bannschuppenkönigswache",
		["Jade"] = "Jade",
		["Grom'gol Grunt"] = "Grunzer von Grom'gol",
		["Watcher Merant"] = "Behüterin Merant ",
		["Angus Stern"] = "Angus Stern ",
		["Senator Barin Redstone"] = "Senator Barin Rotstein",
		["Archbishop Benedictus"] = "Erzbischof Benedictus",
		["Felder Stover"] = "Felder Ofner ",
		["Charys Yserian"] = "Charys Yserian ",
		["Adair Gilroy"] = "Adair Gilroy ",
		["Elly Langston"] = "Elly Langstein ",
		["Mayda Thane"] = "Mayda Thane ",
		["Georgio Bolero"] = "Georgio Bolero ",
		["Rogvar"] = "Rogvar ",
		["Thysta"] = "Thysta ",
		["Ian Strom"] = "Ian Strom ",
		["Grimand Elmore"] = "Grimand Elmore",
		["Zardeth of the Black Claw"] = "Zardeth der Schwarzen Klaue",
		["Helgrum the Swift"] = "Helgrum der Hurtige",
		["Fel'zerul"] = "Fel'zerul",
		["Jannos Ironwill"] = "Jannos Eisenwill ",
		["Morgg Stormshot"] = "Morgg Sturmschuss",
		["Menethil Guard"] = "Menethil Guard",
		["Mok'rash"] = "Mok'rash",
		["Negolash"] = "Negolash",
		["Gunther Arcanus"] = "Gunther Arkanus",
		["Bethor Iceshard"] = "Bethor Eismacht",
		["King Mukla"] = "König Mukla",
		["Shellei Brondir"] = "Shellei Brondir ",
		["Thorgrum Borrelson"] = "Thorgrum Borrelson ",
		["Gryth Thurden"] = "Gryth Thurden ",
		["Northshire Guard"] = "Wache von Nordhain",
		["Deathguard Randolph"] = "Todeswache Randolph",
		["Deathguard Oliver"] = "Todeswache Oliver",
		["Deathguard Phillip"] = "Todeswache Phillip",
		["Deathguard Bartrand"] = "Todeswache Bartrand",
		["Highlord Bolvar Fordragon"] = "Hochlord Bolvar Drachenwill",
		["Lady Katrana Prestor"] = "Lady Katrana Prestor",
		["Grand Admiral Jes-Tereth"] = "Großadmiralin Jes-Tereth",
		["Mithras Ironhill"] = "Mithras Eisenberg",
		["Caledra Dawnbreeze"] = "Caledra Morgenhauch",
		["Stormwind Royal Guard"] = "Königswache von Sturmwind",
		["Zun'dartha"] = "Zun'dartha",
		["Skeletal Flayer"] = "Skelettschinder",
		["Skeletal Sorcerer"] = "Skelettzauberhexer",
		["Skeletal Terror"] = "Skelettschrecker",
		["Skeletal Executioner"] = "Skeletthenker",
		["Skeletal Warlord"] = "Skelettkriegsherr",
		["Skeletal Acolyte"] = "Skelettakolyt",
		["Slavering Ghoul"] = "Sabbernder Ghul",
		["Rotting Ghoul"] = "Faulender Ghul",
		["Soulless Ghoul"] = "Seelenloser Ghul",
		["Searing Ghoul"] = "Sengender Ghul",
		["Freezing Ghoul"] = "Eiskalter Ghul",
		["Cold Wraith"] = "Kalte Ehre",
		["Blood Wraith"] = "Blutehre",
		["Hungering Wraith"] = "Hungerndes Gespenst",
		["Wailing Death"] = "Klagender Tod",
		["Flesh Golem"] = "Fleischgolem",
		["Vile Slime"] = "Böser Brühschleimer",
		["Devouring Ooze"] = "Verschlingender Brühschlammer",
		["Carrion Vulture"] = "Aasgeier",
		["Rotting Behemoth"] = "Faulendes Ungetüm",
		["Decaying Horror"] = "Verfallender Schrecken",
		["Diseased Black Bear"] = "Kranker Schwarzbär",
		["Diseased Grizzly"] = "Kranker Grizzly",
		["Diseased Wolf"] = "Kranker Wolf",
		["Carrion Lurker"] = "Aaslauerer",
		["Venom Mist Lurker"] = "Giftnebellauerer",
		["Plague Lurker"] = "Seuchenlauerer",
		["Scarlet Mage"] = "Scharlachroter Magier",
		["Scarlet Sentinel"] = "Scharlachrote Schildwache",
		["Scarlet Hunter"] = "Scharlachroter Jäger",
		["Scarlet Magus"] = "Scharlachroter Magus",
		["Scarlet Knight"] = "Scharlachroter Ritter",
		["Scarlet Paladin"] = "Scharlachroter Paladin",
		["Scarlet Invoker"] = "Scharlachroter Herbeirufer",
		["Scarlet Cavalier"] = "Scharlachroter Chevalier",
		["Scarlet Judge"] = "Scharlachroter Richter",
		["Scarlet Interrogator"] = "Scharlachroter Befrager",
		["Scarlet High Clerist"] = "Scharlachroter Hochkleriker",
		["Grand Inquisitor Isillien"] = "Großinquisitor Isillien",
		["Scarlet Executioner"] = "Scharlachroter Henker",
		["Highlord Taelan Fordring"] = "Hochlord Taelan Fordring ",
		["Foreman Jerris"] = "Großknecht Jerris",
		["Foreman Marcrid"] = "Großknecht Marcrid",
		["High Protector Tarsen"] = "Hochbeschützer Tarsen",
		["High Protector Lorik"] = "Hochbeschützer Lorik",
		["Foulmane"] = "Faulmähne",
		["Lord Maldazzar"] = "Lord Maldazzar",
		["Putridius"] = "Putridius",
		["The Husk"] = "Die Hülse",
		["Araj the Summoner"] = "Araj der Beschwörer",
		["Darkmaster Gandling"] = "Dunkelmeister Gandling",
		["High Priest Thel'danis"] = "Hohepriester Thel'danis",
		["Tirion Fordring"] = "Tirion Fordring",
		["Scarlet Worker"] = "Scharlachroter Tagelöhner",
		["Scarlet Lumberjack"] = "Scharlachroter Holzfäller",
		["Scarlet Smith"] = "Scharlachroter Schmied",
		["Treant"] = "Treant",
		["Stormwind City Patroller"] = "Stadtpatrolleur von Sturmwind",
		["Senator Mehr Stonehallow"] = "Senator Mehr Steingunst",
		["Ancient Protector"] = "Urtumbeschützer",
		["Master Apothecary Faranell"] = "Apothekermeister Faranell ",
		["Natheril Raincaller"] = "Natheril Regenrufer ",
		["High Executor Darthalia"] = "Hochexekutorin Darthalia",
		["Zora Guthrek"] = "Zora Guthrek ",
		["Karos Razok"] = "Karos Razok ",
		["Borgus Stoutarm"] = "Borgus Starkarm ",
		["Aethalas"] = "Aethalas ",
		["Lyranne Feathersong"] = "Lyranne Federklang ",
		["Southshore Guard"] = "Wache von Süderstade",
		["Zarise"] = "Zarise ",
		["Daryl Stack"] = "Daryl Ballen ",
		["Tarren Mill Deathguard"] = "Todeswache von Tarrens Mühle",
		["Felicia Maline"] = "Felicia Maline ",
		["Varimathras"] = "Varimathras",
		["Darla Harris"] = "Darla Harris ",
		["Major Samuelson"] = "Major Samuelson ",
		["Commander Aggro'gosh"] = "Kommandant Aggro'gosh",
		["Bro'kin"] = "Bro'kin ",
		["Larimaine Purdue"] = "Larimaine Purdue ",
		["Fleet Master Seahorn"] = "Flottenmeister Seehorn ",
		["Baron Revilgaz"] = "Baron Revilgaz",
		["Saltwater Snapjaw"] = "Salzwasserschnappkiefer",
		["Skymane Gorilla"] = "Blaumähnengorilla",
		["Jaguero Stalker"] = "Jagueropirscher",
		["Zanzil the Outcast"] = "Zanzil der Ausgestoßene",
		["Catelyn the Blade"] = "Catelyn die Klinge ",
		["Fleet Master Firallon"] = "Flottenmeister Firallon",
		["Captain Keelhaul"] = "Käpt'n Kielhol",
		["Hammerfall Guardian"] = "Wächter von Hammerfall",
		["Sly Garrett"] = "Sly Garrett ",
		["Viznik Goldgrubber"] = "Viznik Goldraffer ",
		["Vilebranch Witch Doctor"] = "Hexendoktor der Blutfratzen",
		["Vilebranch Headhunter"] = "Kopfjäger der Blutfratzen",
		["Vilebranch Shadowcaster"] = "Schattenzauberer der Blutfratzen",
		["Vilebranch Berserker"] = "Berserker der Blutfratzen",
		["Vilebranch Hideskinner"] = "Balgabzieher der Blutfratzen",
		["Vilebranch Shadow Hunter"] = "Schattenjäger der Blutfratzen",
		["Vilebranch Blood Drinker"] = "Bluttrinker der Blutfratzen",
		["Vilebranch Soul Eater"] = "Seelenfresser der Blutfratzen",
		["Vilebranch Aman'zasi Guard"] = "Aman'zasiwache der Blutfratzen",
		["Green Sludge"] = "Grüner Schlamm",
		["Jade Ooze"] = "Jadegrüner Brühschlammer",
		["Razorbeak Skylord"] = "Klingenschnabelhimmelsfürst",
		["Mechanical Dragonling"] = "Mechanischer Drachling",
		["Vilebranch Wolf Pup"] = "Wolfswelpe der Blutfratzen",
		["Vilebranch Raiding Wolf"] = "Raubwolf der Blutfratzen",
		["Ruppo Zipcoil"] = "Ruppo Reißspule ",
		["Highvale Ranger"] = "Hochtalwaldläuferin",
		["Hanashi"] = "Hanashi ",
		["Shadra"] = "Shadra ",
		["Archaedas"] = "Archaedas ",
		["Blacklash"] = "Schattenbringer",
		["Hematus"] = "Hematus",
		["King Magni Bronzebeard"] = "König Magni Bronzebart ",
		["Zaruk"] = "Zaruk",
		["Susan Tillinghast"] = "Susan Tillinghast",
		["Malygen"] = "Malygen ",
		["Kurden Bloodclaw"] = "Kurden Blutklaue ",
		["Bale"] = "Bale ",
		["Cedrik Prose"] = "Cedrik Prosa ",
		["Brikk Keencraft"] = "Brikk Kühnwerk ",
		["Jaxin Chong"] = "Jaxin Chong ",
		["Crazk Sparks"] = "Crazk Funks ",
		["Hurklor"] = "Hurklor ",
		["Jansen Underwood"] = "Jansen Unterberg ",
		["Urda"] = "Urda ",
		["Gringer"] = "Gringer ",
		["Gyll"] = "Gyll ",
		["Gorrik"] = "Gorrik ",
		["Grunenstur Balindom"] = "Grunenstur Balindom ",
		["Hurom Juggendolf"] = "Hurom Juggendolf ",
		["Durdek Karrin"] = "Durdek Karrin ",
		["Garek"] = "Garek",
		["Silvermane Stalker"] = "Silbermähnenpirscher",
		["Savage Owlbeast"] = "Wilde Eulenbestie",
		["Zaricotl"] = "Zaricotl",
		["Lanie Reed"] = "Lanie Riet ",
		["Tal"] = "Tal ",
		["Una"] = "Una ",
		["Siln Skychaser"] = "Siln Himmelsjäger ",
		["Beram Skychaser"] = "Beram Himmelsjäger ",
		["Turak Runetotem"] = "Turak Runentotem ",
		["Sheal Runetotem"] = "Sheal Runentotem ",
		["Kary Thunderhorn"] = "Kary Donnerhorn ",
		["Holt Thunderhorn"] = "Holt Donnerhorn ",
		["Torm Ragetotem"] = "Torm Rachtotem ",
		["Sark Ragetotem"] = "Sark Rachtotem ",
		["Malakai Cross"] = "Malakai Kreuz ",
		["Father Cobb"] = "Vater Cobb ",
		["Archmage Shymm"] = "Erzmagier Shymm ",
		["Thurston Xane"] = "Thurston Xane ",
		["Cairne Bloodhoof"] = "Cairne Bluthuf ",
		["Honor Guard"] = "Ehrenwache",
		["Bluffwatcher"] = "Behüter von Donnerfels",
		["Unseen"] = "Unsichtbarer",
		["Brave Proudsnout"] = "Kriegerheld Stolzhaupt",
		["Brave Lightninghorn"] = "Kriegerheld Blitzhorn",
		["Brave Running Wolf"] = "Kriegerheld Rennender Wolf",
		["Brave Greathoof"] = "Kriegerheld Großhuf",
		["Nazgrel"] = "Nazgrel ",
		["Orgrimmar Grunt"] = "Grunzer von Orgrimmar",
		["Gabrielle Chase"] = "Gabrielle Chase ",
		["Grisha"] = "Grisha ",
		["Doras"] = "Doras ",
		["Olvia"] = "Olvia ",
		["Grol'dar"] = "Grol'dar ",
		["Mirket"] = "Mirket ",
		["Zevrost"] = "Zevrost ",
		["Ormok"] = "Ormok ",
		["Sergra Darkthorn"] = "Sergra Dunkeldorn",
		["Grelkor"] = "Grelkor ",
		["Kardris Dreamseeker"] = "Kardris Traumsucher ",
		["Ormak Grimshot"] = "Ormak Grimmschlag ",
		["Grezz Ragefist"] = "Grezz Zornfaust ",
		["Sorek"] = "Sorek ",
		["Gazlowe"] = "Gazlowe",
		["Shenthul"] = "Shenthul ",
		["Xor'juul"] = "Xor'juul ",
		["Melor Stonehoof"] = "Melor Steinhuf",
		["Ancient of Lore"] = "Urtum der Lehren",
		["Ancient of War"] = "Urtum des Krieges",
		["Tarhus"] = "Tarhus ",
		["Ratchet Bruiser"] = "Haudrauf von Ratschet",
		["Arch Druid Fandral Staghelm"] = "Erzdruide Fandral Hirschhaupt",
		["Teldrassil Sentinel"] = "Schildwache von Teldrassil",
		["Sewer Beast"] = "Kanalbestie",
		["Aman"] = "Aman",
		["Devrak"] = "Devrak ",
		["Onu"] = "Onu ",
		["Kurll"] = "Kurll ",
		["Rarck"] = "Rarck ",
		["Vrang Wildgore"] = "Vrang Wildblut ",
		["Raene Wolfrunner"] = "Raene Wolfsläufer",
		["Jadenvis Seawatcher"] = "Jadenvis Meeresblick ",
		["Gruna"] = "Gruna ",
		["Syurana"] = "Syurana ",
		["Vesprystus"] = "Vesprystus ",
		["Caylais Moonfeather"] = "Caylais Mondfeder ",
		["Brakgul Deathbringer"] = "Brakgul Todesbringer ",
		["Healing Stream Totem IV"] = "Totem des heilenden Flusses IV",
		["Healing Stream Totem V"] = "Totem des heilenden Flusses V",
		["Shandris Feathermoon"] = "Shandris Mondfeder ",
		["Magatha Grimtotem"] = "Magatha Grimmtotem ",
		["Zor Lonetree"] = "Zor Einbaum ",
		["Rat"] = "Ratte",
		["Arias'ta Bladesinger"] = "Arias'ta Klingenlied ",
		["Elanaria"] = "Elanaria",
		["Astarii Starseeker"] = "Astarii Sternsucher ",
		["Jandria"] = "Jandria ",
		["Jeen'ra Nightrunner"] = "Jeen'ra Nachtläufer ",
		["Ainethil"] = "Ainethil ",
		["Lysheana"] = "Lysheana ",
		["Ealyshia Dewwhisper"] = "Ealyshia Wispertau ",
		["Geenia Sunshadow"] = "Geenia Sonnenfleck ",
		["Dorion"] = "Dorion ",
		["Telonis"] = "Telonis ",
		["Erion Shadewhisper"] = "Erion Schattenflüsterer ",
		["Anishar"] = "Anishar ",
		["Mathrengyl Bearwalker"] = "Mathrengyl Bärenfährte ",
		["Denatharion"] = "Denatharion ",
		["Frostsaber Companion"] = "Frostsäblergefährtin",
		["Nightshade"] = "Nachtschatten ",
		["Brogus Thunderbrew"] = "Brogus Donnerbräu ",
		["Lana Thunderbrew"] = "Lana Donnerbräu ",
		["Darnassus Sentinel"] = "Schildwache von Darnassus",
		["Daelyshia"] = "Daelyshia ",
		["Gorm Grimtotem"] = "Gorm Grimmtotem",
		["Cor Grimtotem"] = "Cor Grimmtotem",
		["Tharm"] = "Tharm ",
		["Gorkas"] = "Gorkas ",
		["Nyse"] = "Nyse ",
		["Thyssiana"] = "Thyssiana ",
		["Baldruc"] = "Baldruc ",
		["Strashaz Warrior"] = "Strashazkrieger",
		["Strashaz Serpent Guard"] = "Strashazschlangenwache",
		["Strashaz Myrmidon"] = "Strashazmyrmidone",
		["Strashaz Sorceress"] = "Strashazzauberhexerin",
		["Strashaz Siren"] = "Strashazsirene",
		["Strashaz Hydra"] = "Strashazhydra",
		["Teloren"] = "Teloren ",
		["Darnassian Protector"] = "Beschützer von Darnassus",
		["Vilebranch Scalper"] = "Skalpierer der Blutfratzen",
		["Vilebranch Soothsayer"] = "Sterndeuter der Blutfratzen",
		["Jade Sludge"] = "Jadegrüner Schlamm",
		["Emerald Ooze"] = "Smaragdgrüner Brühschlammer",
		["Haunting Vision"] = "Spukvision",
		["Rotting Cadaver"] = "Faulender Kadaver",
		["Blighted Zombie"] = "Verseuchter Zombie",
		["Scarlet Avenger"] = "Scharlachroter Rächer",
		["Scarlet Spellbinder"] = "Scharlachroter Zauberbinder",
		["Michael Garrett"] = "Michael Garrett ",
		["Kaal Soulreaper"] = "Kaal Seelentod ",
		["Luther Pickman"] = "Luther Pickmann ",
		["Kaelystia Hatebringer"] = "Kaelystia Hassbringer ",
		["Pierce Shackleton"] = "Pierce Knechtler ",
		["Miles Dexter"] = "Miles Dexter ",
		["Gregory Charles"] = "Gregory Charles ",
		["Christoph Walker"] = "Christoph Walker ",
		["Angela Curthas"] = "Angela Curthas ",
		["Aelthalyste"] = "Aelthalyste ",
		["Father Lazarus"] = "Vater Lazarus ",
		["Doctor Herbert Halsey"] = "Doktor Herbert Halsey ",
		["Christopher Drakul"] = "Christopher Drakul ",
		["Booty Bay Bruiser"] = "Haudrauf von Beutebucht",
		["Kildar"] = "Kildar ",
		["Truk Wildbeard"] = "Truk Wildbart ",
		["Nazeer Bloodpike"] = "Nazeer Blutlanze",
		["Stone Keeper"] = "Steinbewahrer",
		["Guard Edward"] = "Wache Edward",
		["Guard Jarad"] = "Wache Jarad",
		["Combat Master Criton"] = "Gefechtmeister Criton",
		["Captain Garran Vimes"] = "Hauptmann Garran Mumm",
		["Theramore Lieutenant"] = "Leutnant von Theramore",
		["Thrall"] = "Thrall ",
		["Theramore Practicing Guard"] = "Übungswache von Theramore",
		["Lady Jaina Proudmoore"] = "Lady Jaina Prachtmeer ",
		["Guard Lasiter"] = "Wache Lasiter",
		["Aldwin Laughlin"] = "Aldwin Laughlin ",
		["Theramore Guard"] = "Wache von Theramore",
		["Krumn"] = "Krumn ",
		["Combat Master Szigeti"] = "Gefechtmeister Szigeti",
		["Guard Kahil"] = "Wache Kahil",
		["Guard Lana"] = "Wache Lana",
		["Guard Narrisha"] = "Wache Narrisha",
		["Guard Tark"] = "Wache Tark",
		["Captain Andrews"] = "Hauptmann Andrews ",
		["Captain Thomas"] = "Hauptmann Thomas ",
		["Daera Brightspear"] = "Daera Prunkspeer ",
		["Olmin Burningbeard"] = "Olmin Feuerbart ",
		["Brogun Stoneshield"] = "Brogun Steinschild ",
		["Jondor Steelbrow"] = "Jondor Stahlbraue ",
		["Jonivera Farmountain"] = "Jonivera Bergweh ",
		["Svalbrad Farmountain"] = "Svalbrad Bergweh ",
		["Kurdrum Barleybeard"] = "Kurdrum Gerstenbart ",
		["Theodrus Frostbeard"] = "Theodrus Frostbart ",
		["Braenna Flintcrag"] = "Braenna Kieselbruch ",
		["Juli Stormkettle"] = "Juli Sturmkessel ",
		["Nittlebur Sparkfizzle"] = "Nittlebur Zischelfunks ",
		["Valgar Highforge"] = "Valgar Hochesse ",
		["Beldruk Doombrow"] = "Beldruk Schicksalsbraue ",
		["Brandur Ironhammer"] = "Brandur Eisenhammer ",
		["Grumnus Steelshaper"] = "Grumnus Scharfstahl ",
		["Hulfdan Blackbeard"] = "Hulfdan Schwarzbart ",
		["Ormyr Flinteye"] = "Ormyr Kieselauge ",
		["Briarthorn"] = "Wilddorn ",
		["Alexander Calder"] = "Alexander Calder ",
		["Medic Tamberlyn"] = "Sanitäterin Tamberlyn",
		["Medic Helaina"] = "Sanitäterin Helaina",
		["Murk Spitter"] = "Düsterspeier",
		["Murk Worm"] = "Düsterwurm",
		["Saturated Ooze"] = "Gesättigter Brühschlammer",
		["Gordunni Warlord"] = "Kriegsherr der Gordunni",
		["Atal'ai Warrior"] = "Krieger der Atal'ai",
		["Atal'ai Witch Doctor"] = "Hexendoktor der Atal'ai",
		["Groddoc Thunderer"] = "Groddocdonnerer",
		["Mummified Atal'ai"] = "Mumifizierter Atal'ai",
		["Unliving Atal'ai"] = "Untoter Atal'ai",
		["Atal'ai Priest"] = "Priester der Atal'ai",
		["Atal'ai Corpse Eater"] = "Leichenfresser der Atal'ai",
		["Atal'ai Deathwalker"] = "Todeswandler der Atal'ai",
		["Atal'ai High Priest"] = "Hohepriester der Atal'ai",
		["Ironfur Patriarch"] = "Eisenfellpatriarch",
		["Sprite Dragon"] = "Grimmlingdrache",
		["Nightmare Scalebane"] = "Nachtmahrbannschuppe",
		["Nightmare Wyrmkin"] = "Nachtmahrwyrmkin",
		["Nightmare Wanderer"] = "Nachtmahrwanderer",
		["Rabid Longtooth"] = "Tollwütiger Langzahn",
		["Hakkari Frostwing"] = "Frostflügler der Hakkari",
		["Rage Scar Yeti"] = "Wutschrammyeti",
		["Elder Rage Scar"] = "Alter Wutschrammyeti",
		["Ferocious Rage Scar"] = "Wilder Wutschrammyeti",
		["Frayfeather Patriarch"] = "Fransenfederpatriarch",
		["Lethlas"] = "Lethlas",
		["Phantim"] = "Phantim",
		["Jademir Oracle"] = "Jademirorakel",
		["Jademir Tree Warder"] = "Jademirbaumwärter",
		["Jademir Boughguard"] = "Jademirastwache",
		["Lady Szallah"] = "Lady Szallah",
		["Diamond Head"] = "Diamantenkopf",
		["Bloodroar the Stalker"] = "Blutschrei der Pirscher",
		["Antilus the Soarer"] = "Antilus der Aufsteiger",
		["Arash-ethis"] = "Arash-ethis",
		["Qirot"] = "Qirot",
		["Land Walker"] = "Landgänger",
		["Cliff Giant"] = "Klippenriese",
		["Shore Strider"] = "Uferschreiter",
		["Deep Strider"] = "Tiefenschreiter",
		["Wave Strider"] = "Wellenschreiter",
		["Northspring Harpy"] = "Harpyie der Nordschwingen",
		["Northspring Roguefeather"] = "Wildfeder der Nordschwingen",
		["Northspring Slayer"] = "Töterin der Nordschwingen",
		["Northspring Windcaller"] = "Windruferin der Nordschwingen",
		["High Explorer Magellas"] = "Hochforscher Magellas ",
		["Sage Palerunner"] = "Weiser Bleichlauf",
		["Quartermaster Lungertz"] = "Rüstmeister Lungertz",
		["Veyzhak the Cannibal"] = "Veyzhack der Kannibale",
		["Zekkis"] = "Zekkis",
		["Kazkaz the Unholy"] = "Kazkaz der Unheilige",
		["Glasshide Petrifier"] = "Glashautversteinerer",
		["Scorpid Dunestalker"] = "Skorpiddünenpirscher",
		["Rabid Blisterpaw"] = "Tollwütige Eiterpfote",
		["Searing Roc"] = "Sengender Roc",
		["Surf Glider"] = "Brandungsgleiter",
		["Giant Surf Glider"] = "Riesiger Brandungsgleiter",
		["Coral Shark"] = "Korallenhai",
		["Hazzali Wasp"] = "Wespe der Hazzali",
		["Hazzali Stinger"] = "Stecher der Hazzali",
		["Hazzali Swarmer"] = "Schwärmer der Hazzali",
		["Hazzali Worker"] = "Arbeiter der Hazzali",
		["Hazzali Tunneler"] = "Tunnelgräber der Hazzali",
		["Hazzali Sandreaver"] = "Sandhäscher der Hazzali",
		["Centipaar Wasp"] = "Wespe der Centipaar",
		["Centipaar Stinger"] = "Stecher der Centipaar",
		["Centipaar Swarmer"] = "Schwärmer der Centipaar",
		["Centipaar Worker"] = "Arbeiter der Centipaar",
		["Centipaar Tunneler"] = "Tunnelgräber der Centipaar",
		["Centipaar Sandreaver"] = "Sandhäscher der Centipaar",
		["Sea Elemental"] = "Meereselementar",
		["Sea Spray"] = "Meeresschauer",
		["Coast Strider"] = "Küstenschreiter",
		["Deep Dweller"] = "Tiefenbewohner",
		["Dune Smasher"] = "Dünenzerkracher",
		["Raging Dune Smasher"] = "Tobender Dünenzerkracher",
		["Dunemaul Enforcer"] = "Vollstrecker der Dünenbrecher",
		["Dunemaul Ogre Mage"] = "Ogermagier der Dünenbrecher",
		["Dunemaul Brute"] = "Schläger der Dünenbrecher",
		["Dunemaul Warlock"] = "Hexenmeister der Dünenbrecher",
		["Wu Shen"] = "Wu Shen ",
		["Thistleshrub Dew Collector"] = "Disteltausammler",
		["Thistleshrub Rootshaper"] = "Distelwurzelformer",
		["Brother Joshua"] = "Bruder Joshua ",
		["Gnarled Thistleshrub"] = "Knorriger Distelwandler",
		["Katherine the Pure"] = "Katharina die Reine ",
		["Sandahl"] = "Sandahl ",
		["Elsharin"] = "Elsharin ",
		["Kaerbrus"] = "Kaerbrus ",
		["Sheldras Moontree"] = "Sheldras Mistelmond ",
		["Theridran"] = "Theridran ",
		["Einris Brightspear"] = "Einris Prunkspeer ",
		["Ulfir Ironbeard"] = "Ulfir Eisenbart ",
		["Grunt Zuul"] = "Grunzer Zuul",
		["Grunt Tharlak"] = "Grunzer Tharlak",
		["Bruuk Barleybeard"] = "Bruuk Gerstenbart ",
		["Ironforge Guard"] = "Wache von Eisenschmiede",
		["Undercity Guardian"] = "Wächter von Unterstadt",
		["Falstad Wildhammer"] = "Falstad Wildhammer ",
		["Gunther's Visage"] = "Gunthers Abbild",
		["Carendin Halgar"] = "Carendin Halgar",
		["High Sorcerer Andromath"] = "Zaubermeister Andromath",
		["Spawn of Hakkar"] = "Brut von Hakkar",
		["Shade of Eranikus"] = "Eranikus' Schemen",
		["Jammal'an the Prophet"] = "Jammal'an der Prophet",
		["Ogom the Wretched"] = "Ogom der Elende",
		["Zolo"] = "Zolo",
		["Gasher"] = "Schlitzer",
		["Loro"] = "Loro",
		["Hukku"] = "Hukku",
		["Zul'Lor"] = "Zul'Lor",
		["Mijan"] = "Mijan",
		["Rothos"] = "Rothos",
		["Morphaz"] = "Morphaz",
		["Weaver"] = "Wirker",
		["Dreamscythe"] = "Traumsense",
		["Hazzas"] = "Hazzas",
		["Guardian of Blizzard"] = "Wächter von Blizzard",
		["Arch Druid Hamuul Runetotem"] = "Erzdruide Hamuul Runentotem",
		["Nara Wildmane"] = "Nara Wildmähne",
		["Crildor"] = "Crildor",
		["Margol the Rager"] = "Margol die Wütende",
		["Dark Iron Steamsmith"] = "Dampfschmied der Dunkeleisenzwerge",
		["Dark Iron Taskmaster"] = "Zuchtmeister der Dunkeleisenzwerge",
		["Inferno Elemental"] = "Infernoelementar",
		["Heavy War Golem"] = "Schwerer Kriegsgolem",
		["Magma Elemental"] = "Magmaelementar",
		["Greater Lava Spider"] = "Große Lavaspinne",
		["Twilight Dark Shaman"] = "Dunkelschamane des Schattenhammers",
		["Twilight Fire Guard"] = "Feuerwache des Schattenhammers",
		["Twilight Geomancer"] = "Geomant des Schattenhammers",
		["Pephredo"] = "Pephredo ",
		["Enyo"] = "Enyo ",
		["Tremor Totem"] = "Totem des Erdstoßes",
		["Den Grunt"] = "Grunzer des Höhlenbaus",
		["Dreadmaul Ogre Mage"] = "Ogermagier der Schreckensfelsoger",
		["Dreadmaul Brute"] = "Schläger der Schreckensfelsoger",
		["Dreadmaul Mauler"] = "Raufer der Schreckensfelsoger",
		["Dreadmaul Warlock"] = "Hexenmeister der Schreckensfelsoger",
		["Portal Seeker"] = "Portalsucher",
		["Black Slayer"] = "Schwarzer Töter",
		["Bonepicker"] = "Bonepicker",
		["Snickerfang Hyena"] = "Kicherfanghyäne",
		["Scorpok Stinger"] = "Skorpokstecher",
		["Redstone Basilisk"] = "Rotsteinbasilisk",
		["Redstone Crystalhide"] = "Rotsteinkristallhaut",
		["Ashmane Boar"] = "Aschmähneneber",
		["Helboar"] = "Hölleneber",
		["Zayus"] = "Zayus ",
		["Nethergarde Miner"] = "Minenarbeiter von Nethergarde",
		["Nethergarde Engineer"] = "Ingenieur von Nethergarde",
		["Nethergarde Foreman"] = "Großknecht von Nethergarde",
		["Nethergarde Soldier"] = "Soldat von Nethergarde",
		["Nethergarde Cleric"] = "Kleriker von Nethergarde",
		["Nethergarde Analyst"] = "Analysator von Nethergarde",
		["Nethergarde Riftwatcher"] = "Portalwache von Nethergarde",
		["Nethergarde Officer"] = "Offizier von Nethergarde",
		["Shadowsworn Cultist"] = "Kultist der Schattenanbeter",
		["Shadowsworn Thug"] = "Rohling der Schattenanbeter",
		["Shadowsworn Adept"] = "Adept der Schattenanbeter",
		["Shadowsworn Enforcer"] = "Vollstrecker der Schattenanbeter",
		["Shadowsworn Warlock"] = "Hexenmeister der Schattenanbeter",
		["Shadowsworn Dreadweaver"] = "Schreckenswirker der Schattenanbeter",
		["Felhound"] = "Teufelshund",
		["Felguard Sentry"] = "Wachposten der Teufelswache",
		["Flametongue Totem II"] = "Totem der Flammenzunge II",
		["X'yera"] = "X'yera ",
		["Elemental Protection Totem"] = "Totem des Elementarschutzes",
		["Breyk"] = "Breyk ",
		["Earthgrab Totem"] = "Totem des Erdengriffs",
		["Harry Burlguard"] = "Harry Burlwacht",
		["Azuregos"] = "Azuregos",
		["Muren Stormpike"] = "Muren Sturmlanze",
		["Highborne Lichling"] = "Lichling eines Hochgeborenen",
		["Varo'then's Ghost"] = "Varo'thens Geist",
		["Gakin the Darkbinder"] = "Gakin der Dunkelbinder",
		["Haldarr Felsworn"] = "Teufelsanbeter der Haldarr",
		["Draconic Magelord"] = "Drachischer Magierlord",
		["Blue Scalebane"] = "Blaue Bannschuppe",
		["Draconic Mageweaver"] = "Drachischer Magiewirker",
		["Lord Arkkoroc"] = "Lord Arkkoroc",
		["Arkkoran Clacker"] = "Arkkoranklacker",
		["Arkkoran Muckdweller"] = "Arkkoranmatschkrabbler",
		["Arkkoran Pincer"] = "Arkkoranzange",
		["Arkkoran Oracle"] = "Arkkoranorakel",
		["Hetaera"] = "Hetaera",
		["Servant of Arkkoroc"] = "Diener von Arkkoroc",
		["Son of Arkkoroc"] = "Arkkorocs Sohn",
		["Cliff Breaker"] = "Klippenbrecher",
		["Cliff Thunderer"] = "Klippendonnerer",
		["Cliff Walker"] = "Klippengänger",
		["Duthorian Rall"] = "Duthorian Rall",
		["Tiza Battleforge"] = "Tiza Kriegshammer",
		["Timbermaw Pathfinder"] = "Pfadfinder der Holzschlundfeste",
		["Timbermaw Warrior"] = "Krieger der Holzschlundfeste",
		["Timbermaw Totemic"] = "Totemiker der Holzschlundfeste",
		["Timbermaw Den Watcher"] = "Höhlenbehüter der Holzschlundfeste",
		["Timbermaw Shaman"] = "Schamane der Holzschlundfeste",
		["Timbermaw Ursa"] = "Ursa der Holzschlundfeste",
		["Spitelash Warrior"] = "Krieger der Grollflossen",
		["Spitelash Screamer"] = "Kreischerin der Grollflossen",
		["Spitelash Serpent Guard"] = "Schlangenwache der Grollflossen",
		["Spitelash Siren"] = "Sirene der Grollflossen",
		["Spitelash Myrmidon"] = "Myrmidone der Grollflossen",
		["Blood Elf Surveyor"] = "Blutelfenfeldmesser",
		["Blood Elf Reclaimer"] = "Blutelfeneintreiber",
		["Legashi Satyr"] = "Satyr der Legashi",
		["Legashi Rogue"] = "Schurke der Legashi",
		["Legashi Hellcaller"] = "Höllenrufer der Legashi",
		["Klannoc Macleod"] = "Klannoc Macleod ",
		["Strahad Farsan"] = "Strahad Farsan",
		["Menara Voidrender"] = "Menara Leerbringer",
		["Young Wavethrasher"] = "Junger Wellenhauer",
		["Wavethrasher"] = "Wellenhauer",
		["Great Wavethrasher"] = "Großer Wellenhauer",
		["Makrinni Razorclaw"] = "Makrinniklingenklaue",
		["Storm Bay Oracle"] = "Sturmbuchtorakel",
		["Coralshell Lurker"] = "Korallenpanzerschleicher",
		["Coralshell Tortoise"] = "Korallenpanzerschildkröte",
		["Makrinni Scrabbler"] = "Makrinnischarrer",
		["Storm Bay Warrior"] = "Sturmbuchtkrieger",
		["Makrinni Snapclaw"] = "Makrinnischnappklaue",
		["Thunderhead Hippogryph"] = "Donnerkopfhippogryph",
		["Thunderhead Stagwing"] = "Donnerkopfflügler",
		["Thunderhead Skystormer"] = "Donnerkopfhimmelsstürmer",
		["Thunderhead Patriarch"] = "Donnerkopfpatriarch",
		["Thunderhead Consort"] = "Donnerkopfkonsort",
		["Jubahl Corpseseeker"] = "Jubaal Leichenfund ",
		["Ward of Zanzil"] = "Zauberschutz von Zanzil",
		["Henen Ragetotem"] = "Henen Rachtotem",
		["Sergeant Rutger"] = "Unteroffizier Rutger ",
		["Spirit Healer"] = "Geistheiler",
		["Devilsaur"] = "Teufelssaurier",
		["Ironhide Devilsaur"] = "Eisenhautteufelssaurier",
		["Tyrant Devilsaur"] = "Tyrannoteufelssaurier",
		["Stegodon"] = "Stegodon",
		["Plated Stegodon"] = "Plattenpanzerstegodon",
		["Spiked Stegodon"] = "Stachelstegodon",
		["Thunderstomp Stegodon"] = "Donnerstampferstegodon",
		["Ravasaur"] = "Ravasaurus",
		["Ravasaur Runner"] = "Ravasaurusläufer",
		["Ravasaur Hunter"] = "Ravasaurusjäger",
		["Venomhide Ravasaur"] = "Gifthautravasaurus",
		["Bloodpetal Lasher"] = "Blutblütenpeitscher",
		["Bloodpetal Flayer"] = "Blutblütenschinder",
		["Bloodpetal Thresher"] = "Blutblütendrescher",
		["Bloodpetal Trapper"] = "Blutblütenfalle",
		["Un'Goro Stomper"] = "Stampfer von Un'Goro",
		["Un'Goro Gorilla"] = "Gorilla von Un'Goro",
		["Un'Goro Thunderer"] = "Donnerer von Un'Goro",
		["Tar Beast"] = "Teerbestie",
		["Tar Lurker"] = "Teerlauerer",
		["Tar Lord"] = "Teerfürst",
		["Scorching Elemental"] = "Versengender Elementar",
		["Living Blaze"] = "Lebendiges Feuer",
		["Tar Creeper"] = "Teerkriecher",
		["Tabetha"] = "Tabetha",
		["Magus Tirth"] = "Magus Tirth",
		["Gorishi Wasp"] = "Wespe der Gorishi",
		["Gorishi Worker"] = "Arbeiter der Gorishi",
		["Gorishi Reaver"] = "Häscher der Gorishi",
		["Gorishi Stinger"] = "Stecher der Gorishi",
		["Gorishi Tunneler"] = "Tunnelgräber der Gorishi",
		["Muculent Ooze"] = "Dickflüssiger Brühschlammer",
		["Primal Ooze"] = "Urzeitbrühschlammer",
		["Glutinous Ooze"] = "Klebriger Brühschlammer",
		["Stone Guardian"] = "Steinwächter",
		["Vizzklick"] = "Vizzklick ",
		["Ravasaur Matriarch"] = "Ravasaurusmatriarchin",
		["Clutchmother Zavas"] = "Gelegemutter Zavas",
		["Gruff"] = "Gruff",
		["King Mosh"] = "König Mosh",
		["Uhk'loc"] = "Uhk'loc",
		["Monnos the Elder"] = "Monnos der Älteste",
		["Magister Hawkhelm"] = "Magister Falkhelm",
		["Antilos"] = "Antilos",
		["Lady Sesspira"] = "Lady Sesspira",
		["General Fangferror"] = "General Fangferror",
		["Gatekeeper Rageroar"] = "Torhüter Donnerschrei",
		["Master Feardred"] = "Meister Gräuelbart",
		["Baritanas Skyriver"] = "Baritanas Himmelspfad ",
		["Fahrad"] = "Fahrad ",
		["Thalon"] = "Thalon ",
		["Ravenholdt Guard"] = "Rabenholdtwache ",
		["Lord Jorach Ravenholdt"] = "Lord Jorach Rabenholdt ",
		["Ravenholdt Assassin"] = "Rabenholdtauftragsmörder ",
		["Zan Shivsproket"] = "Zan Messerritzel ",
		["Innkeeper Skindle"] = "Gastwirt Skindel ",
		["Talvash del Kissel"] = "Talvash del Kissel",
		["Yalda"] = "Yalda",
		["Blackrock Soldier"] = "Soldat des Schwarzfels",
		["Blackrock Sorcerer"] = "Zauberhexer des Schwarzfels",
		["Blackrock Slayer"] = "Töter des Schwarzfels",
		["Blackrock Warlock"] = "Hexenmeister des Schwarzfels",
		["Blackrock Battlemaster"] = "Kampfmeister des Schwarzfels",
		["Obsidian Elemental"] = "Obsidianelementar",
		["Greater Obsidian Elemental"] = "Großer Obsidianelementar",
		["Firegut Ogre"] = "Oger der Feuermägen",
		["Firegut Ogre Mage"] = "Ogermagier der Feuermägen",
		["Firegut Brute"] = "Schläger der Feuermägen",
		["Thaurissan Spy"] = "Thaurissanischer Spion",
		["Thaurissan Firewalker"] = "Thaurissanischer Feuerwandler",
		["Thaurissan Agent"] = "Thaurissanischer Agent",
		["War Reaver"] = "Kriegshäscher",
		["Black Dragonspawn"] = "Schwarze Großdrachenbrut",
		["Black Wyrmkin"] = "Schwarzwyrmkin",
		["Flamescale Dragonspawn"] = "Flammenschuppendrachenbrut",
		["Flamescale Wyrmkin"] = "Flammenschuppenwyrmkin",
		["Black Drake"] = "Schwarzdrache",
		["Scalding Drake"] = "Brühender Drache",
		["Searscale Drake"] = "Sengschuppendrache",
		["Black Broodling"] = "Schwarzer Brutling",
		["Scalding Broodling"] = "Brühender Brutling",
		["Flamescale Broodling"] = "Flammenschuppenbrutling",
		["Blackrock Worg"] = "Worg des Schwarzfels",
		["Condemned Acolyte"] = "Verurteilter Akolyt",
		["Condemned Monk"] = "Verurteilter Mönch",
		["Condemned Cleric"] = "Verurteilter Kleriker",
		["Cursed Paladin"] = "Verfluchter Paladin",
		["Cursed Justicar"] = "Verfluchter Justiziar",
		["Writhing Mage"] = "Verdammter Magier",
		["Cursed Ooze"] = "Verfluchter Brühschlammer",
		["Tainted Ooze"] = "Besudelter Brühschlammer",
		["Vile Ooze"] = "Übler Brühschlammer",
		["Ironbeak Owl"] = "Eisenschnabeleule",
		["Ironbeak Screecher"] = "Eisenschnabelkreischer",
		["Ironbeak Hunter"] = "Eisenschnabeljäger",
		["Warpwood Moss Flayer"] = "Wucherborkenschinder",
		["Warpwood Shredder"] = "Wucherborkenschredder",
		["Dessecus"] = "Dessecus",
		["Jadefire Satyr"] = "Jadefeuersatyr",
		["Jadefire Rogue"] = "Jadefeuerschurke",
		["Jadefire Trickster"] = "Jadefeuerschwindler",
		["Jadefire Betrayer"] = "Jadefeuerverräter",
		["Jadefire Felsworn"] = "Jadefeuerteufelsanbeter",
		["Jadefire Shadowstalker"] = "Jadefeuerschattenpirscher",
		["Jadefire Hellcaller"] = "Jadefeuerhöllenrufer",
		["Jaedenar Cultist"] = "Kultist von Jaedenar",
		["Jaedenar Guardian"] = "Wächter von Jaedenar",
		["Jaedenar Enforcer"] = "Vollstrecker von Jaedenar",
		["Jaedenar Adept"] = "Adept von Jaedenar",
		["Jaedenar Darkweaver"] = "Dunkelwirker von Jaedenar",
		["Jaedenar Warlock"] = "Hexenmeister von Jaedenar",
		["Jaedenar Hound"] = "Hund von Jaedenar",
		["Jaedenar Hunter"] = "Jäger von Jaedenar",
		["Toxic Horror"] = "Toxinschrecken",
		["Infernal Bodyguard"] = "Höllenbestienleibwache",
		["Infernal Sentry"] = "Höllenbestienwachposten",
		["Immolatus"] = "Korruptus",
		["Irontree Wanderer"] = "Wanderer der Eisenstämme",
		["Irontree Stomper"] = "Donnerstampfer der Eisenstämme",
		["Withered Protector"] = "Siechender Beschützer",
		["Deadwood Warrior"] = "Krieger der Totenwaldfelle",
		["Deadwood Gardener"] = "Gärtner der Totenwaldfelle",
		["Deadwood Pathfinder"] = "Pfadfinder der Totenwaldfelle",
		["Deadwood Den Watcher"] = "Höhlenbehüter der Totenwaldfelle",
		["Deadwood Avenger"] = "Rächer der Totenwaldfelle",
		["Deadwood Shaman"] = "Schamane der Totenwaldfelle",
		["Lore Keeper of Norgannon"] = "Bewahrer der Lehren von Norgannon",
		["Shayis Steelfury"] = "Shayis Stahlzorn ",
		["Kelgruk Bloodaxe"] = "Kelgruk Blutaxt ",
		["Borgus Steelhand"] = "Borgus Stahlhand ",
		["Ember"] = "Glut",
		["Chief Ukorz Sandscalp"] = "Häuptling Ukorz Sandskalp",
		["Witch Doctor Zum'rah"] = "Hexendoktor Zum'rah",
		["Gahz'rilla"] = "Gahz'rilla",
		["Sandfury Executioner"] = "Henker der Sandwüter",
		["Shadowpriest Sezz'ziz"] = "Schattenpriester Sezz'ziz",
		["Darnath Bladesinger"] = "Darnath Klingenlied ",
		["Winstone Wolfe"] = "Winstone Wolf ",
		["Master Kang"] = "Meister Kang",
		["Kum'isha the Collector"] = "Kum'isha der Sammler",
		["Flawless Draenethyst Sphere"] = "Fehlerlose Draenethystsphäre",
		["Stoneskin Totem VI"] = "Totem der Steinhaut VI",
		["Deadwind Brute"] = "Schläger der Totenwindoger",
		["Restless Shade"] = "Ruheloser Schemen",
		["Deadwind Mauler"] = "Raufer der Totenwindoger",
		["Deadwind Warlock"] = "Hexenmeister der Totenwindoger",
		["Sky Shadow"] = "Himmelsschatten",
		["Deadwind Ogre Mage"] = "Magier der Totenwindoger",
		["Stoneclaw Totem V"] = "Totem der Steinklaue V",
		["Stoneclaw Totem VI"] = "Totem der Steinklaue VI",
		["Searing Totem V"] = "Totem der Verbrennung V",
		["Searing Totem VI"] = "Totem der Verbrennung VI",
		["Strength of Earth Totem IV"] = "Totem der Erdstärke IV",
		["Oglethorpe Obnoticus"] = "Glotz Widrikus ",
		["Chief Engineer Bilgewhizzle"] = "Chefingenieur Bilgenritzel ",
		["Thelman Slatefist"] = "Thelman Fausttadel ",
		["Frost Resistance Totem II"] = "Totem des Frostwiderstands II",
		["Frost Resistance Totem III"] = "Totem des Frostwiderstands III",
		["Mana Spring Totem III"] = "Totem der Manaquelle III",
		["Mana Spring Totem IV"] = "Totem der Manaquelle IV",
		["Flametongue Totem III"] = "Totem der Flammenzunge III",
		["Fire Resistance Totem II"] = "Totem des Feuerwiderstands II",
		["Fire Resistance Totem III"] = "Totem des Feuerwiderstands III",
		["Taim Ragetotem"] = "Taim Rachtotem ",
		["Frostmaul Giant"] = "Frosthagelriese",
		["Frostmaul Preserver"] = "Frosthagelbewahrer",
		["Frostsaber Cub"] = "Frostsäblerjunges",
		["Frostsaber"] = "Frostsäbler",
		["Frostsaber Stalker"] = "Frostsäblerpirscherin",
		["Frostsaber Huntress"] = "Frostsäblerjägerin",
		["Frostsaber Pride Watcher"] = "Rudelbehüterin der Frostsäbler",
		["Cobalt Wyrmkin"] = "Kobaltblauer Wyrmkin",
		["Cobalt Scalebane"] = "Kobaltblaue Bannschuppe",
		["Cobalt Mageweaver"] = "Kobaltblauer Magiewirker",
		["Winterfall Ursa"] = "Ursa der Winterfelle",
		["Winterfall Shaman"] = "Schamane der Winterfelle",
		["Winterfall Den Watcher"] = "Höhlenbehüter der Winterfelle",
		["Winterfall Totemic"] = "Totemiker der Winterfelle",
		["Winterfall Pathfinder"] = "Pfadfinder der Winterfelle",
		["Shardtooth Mauler"] = "Splitterzahnraufer",
		["Shardtooth Bear"] = "Splitterzahnbär",
		["Elder Shardtooth"] = "Alter Splitterzahn",
		["Rabid Shardtooth"] = "Tollwütiger Splitterzahn",
		["Fledgling Chillwind"] = "Eiswindjunges",
		["Chillwind Chimaera"] = "Eiswindschimäre",
		["Chillwind Ravager"] = "Eiswindverheerer",
		["Ragged Owlbeast"] = "Struppige Eulenbestie",
		["Raging Owlbeast"] = "Tobende Eulenbestie",
		["Crazed Owlbeast"] = "Wahnsinnige Eulenbestie",
		["Moontouched Owlbeast"] = "Mondbestrahlte Eulenbestie",
		["Berserk Owlbeast"] = "Berserkereulenbestie",
		["Winterspring Owl"] = "Winterquelleule",
		["Winterspring Screecher"] = "Winterquellkreischer",
		["Rogue Ice Thistle"] = "Wilder Eisdistelyeti",
		["Ice Thistle Yeti"] = "Eisdistelyeti",
		["Ice Thistle Matriarch"] = "Eisdistelmatriarchin",
		["Ice Thistle Patriarch"] = "Eisdistelpatriarch",
		["Hederine Initiate"] = "Initiand von Hederine",
		["Hederine Manastalker"] = "Manapirscher von Hederine",
		["Hederine Slayer"] = "Töter von Hederine",
		["Magma Totem III"] = "Totem des glühenden Magmas III",
		["Magma Totem IV"] = "Totem der glühenden Magma IV",
		["Nature Resistance Totem II"] = "Totem des Naturwiderstands II",
		["Nature Resistance Totem III"] = "Totem des Naturwiderstands III",
		["Windfury Totem II"] = "Totem des Windzorns II",
		["Windfury Totem III"] = "Totem des Windzorns III",
		["Grace of Air Totem II"] = "Totem der luftgleichen Anmut II",
		["Bloodmage Drazial"] = "Blutmagier Drazial",
		["Bloodmage Lynnore"] = "Blutmagierin Lynnore",
		["Suffering Highborne"] = "Leidender Hochgeborener",
		["Anguished Highborne"] = "Gepeinigter Hochgeborener",
		["Fallen Hero of the Horde"] = "Gefallener Held der Horde",
		["Dispatch Commander Ruag"] = "Zentralenkommandant Ruag",
		["Razelikh the Defiler"] = "Razelikh der Entweiher",
		["Grol the Destroyer"] = "Grol der Zerstörer",
		["Archmage Allistarj"] = "Erzmagier Allistarj",
		["Lady Sevine"] = "Lady Sevine",
		["Servant of Razelikh"] = "Diener von Razelikh",
		["Servant of Grol"] = "Diener von Grol",
		["Servant of Allistarj"] = "Diener von Allistarj",
		["Servant of Sevine"] = "Diener von Sevine",
		["Kirith the Damned"] = "Kirith der Verdammte",
		["Spirit of Kirith"] = "Geist von Kirith",
		["Ilifar"] = "Ilifar ",
		["Felcular"] = "Felcular ",
		["Gracina Spiritmight"] = "Gracina Glaubensmacht",
		["Corporal Thund Splithoof"] = "Korporal Thund Spalthuf",
		["Curgle Cranklehop"] = "Curgle Kurbelhüpf",
		["Troyas Moonbreeze"] = "Troyas Mondhauch",
		["Kalin Windflight"] = "Kalin Windsegler",
		["Marli Wishrunner"] = "Marli Traumläufer",
		["Shay Leafrunner"] = "Shay Laubschleicher",
		["Talo Thornhoof"] = "Talo Dornhuf",
		["Rin'ji"] = "Rin'ji ",
		["Loramus Thalipedes"] = "Loramus Thalipedes",
		["Ward of Zum'rah"] = "Zauberschutz von Zum'rah",
		["Aturk the Anvil"] = "Aturk der Amboss",
		["Ox"] = "Ochse ",
		["McGavan"] = "McGavan ",
		["Hydromancer Velratha"] = "Wasserbeschwörerin Velratha",
		["Ruuzlu"] = "Ruuzlu",
		["Hank the Hammer"] = "Hank der Hammer ",
		["Galvan the Ancient"] = "Galvan der Alte ",
		["Scorpid Duneburrower"] = "Dünenbuddlerskorpid",
		["Trenton Lighthammer"] = "Trenton Lichthammer ",
		["Marauding Owlbeast"] = "Plündernde Eulenbestie",
		["Vilebranch Ambusher"] = "Wegelagerer der Blutfratzen",
		["Bera Stonehammer"] = "Bera Steinhammer ",
		["Bulkrek Ragefist"] = "Bulkrek Zornfaust ",
		["Oran Snakewrithe"] = "Oran Schlangenfuß",
		["Ambassador Ardalan"] = "Botschafter Ardalan",
		["Fire Nova Totem IV"] = "Totem der Feuernova IV",
		["Fire Nova Totem V"] = "Totem der Feuernova V",
		["Teremus the Devourer"] = "Teremus der Verschlinger",
		["Caliph Scorpidsting"] = "Kalif Skorpidstich",
		["Lurking Feral Scar"] = "Lauernder Wildschrammyeti",
		["Nethergarde Elite"] = "Elite von Nethergarde",
		["Pratt McGrubben"] = "Pratt McGrubben ",
		["Jangdor Swiftstrider"] = "Jangdor Flinkschreiter ",
		["Lingering Highborne"] = "Verweilender Hochgeborener",
		["Wildhammer Sentry"] = "Wachposten der Wildhämmer",
		["Peter Galen"] = "Peter Galen ",
		["Thorkaf Dragoneye"] = "Thorkaf Drachenaug ",
		["Sarah Tanner"] = "Sarah Gerber ",
		["Brumn Winterhoof"] = "Brumn Winterhuf ",
		["Caryssia Moonhunter"] = "Caryssia Jägermond ",
		["Se'Jib"] = "Se'Jib ",
		["Latronicus Moonspear"] = "Latronicus Mondspeer",
		["Vestia Moonspear"] = "Vestia Mondspeer",
		["Quintis Jonespyre"] = "Quintis Lebenshand",
		["Ginro Hearthkindle"] = "Ginro Herdfeuer",
		["Security Chief Bilgewhizzle"] = "Sicherheitchef Bilgenritzel ",
		["Fraggar Thundermantle"] = "Fraggar Donnermantel",
		["Spitelash Battlemaster"] = "Kampfmeister der Grollflossen",
		["Spitelash Enchantress"] = "Verzauberin der Grollflossen",
		["Angelas Moonbreeze"] = "Angelas Mondhauch",
		["Brother Sarno"] = "Bruder Sarno",
		["Stone Watcher of Norgannon"] = "Steinbehüter von Norgannon",
		["High Tinker Mekkatorque"] = "Hochtüftler Mekkadrill ",
		["Feathermoon Sentinel"] = "Schildwache der Mondfederfeste",
		["Darnall"] = "Darnall ",
		["Mardrack Greenwell"] = "Mardrack Grünquell ",
		["Faralorn"] = "Faralorn ",
		["Harklane"] = "Harklane ",
		["Tinkmaster Overspark"] = "Tüftlermeister Oberfunks ",
		["Savanne"] = "Savanne ",
		["Brannock"] = "Brannock ",
		["Vivianna"] = "Vivianna ",
		["Kylanna Windwhisper"] = "Kylanna Windraunen ",
		["Xylinnia Starshine"] = "Xylinnia Sternenschein ",
		["Master Mechanic Castpipe"] = "Mechanikermeister Gussmuff",
		["Xar'Ti"] = "Xar'Ti ",
		["Binjy Featherwhistle"] = "Binjy Federpfiff ",
		["Milli Featherwhistle"] = "Milli Federpfiff ",
		["Mulgore Protector"] = "Beschützer von Mulgore",
		["Gammerita"] = "Gammerita",
		["Deathguard Elite"] = "Elitesoldat der Todeswache",
		["Vile Priestess Hexx"] = "Üble Priesterin Hexx",
		["Qiaga the Keeper"] = "Qiaga die Bewahrerin",
		["Tyrande Whisperwind"] = "Tyrande Whisperwind ",
		["Sen'jin Guardian"] = "Wächter von Sen'jin",
		["Guthrum Thunderfist"] = "Guthrum Donnerfaust ",
		["Fyldren Moonfeather"] = "Fyldren Mondfeder ",
		["Shyn"] = "Shyn ",
		["Orwin Gizzmick"] = "Orwin Gizzmick",
		["Thadius Grimshade"] = "Thadius Grimmschatten",
		["Sharpbeak's Father"] = "Scharfschnabels Vater",
		["Sharpbeak's Mother"] = "Scharfschnabels Mutter",
		["Thyn'tel Bladeweaver"] = "Thyn'tel Klingenweber",
		["Edana Hatetalon"] = "Edana Hasskralle",
		["Witch Doctor Uzer'i"] = "Hexendoktor Uzer'i",
		["Ziggle Sparks"] = "Ziggle Sparks ",
		["Sul'lithuz Abomination"] = "Monstrosität von Sul'lithuz",
		["Rickle Goldgrubber"] = "Rickle Goldraffer ",
		["Nixx Sprocketspring"] = "Nixx Sprossfeder ",
		["Antu'sul"] = "Antu'sul ",
		["Lord Shalzaru"] = "Lord Shalzaru",
		["Jannos Lighthoof"] = "Jannos Leichthuf ",
		["Loorana"] = "Loorana ",
		["Kulleg Stonehorn"] = "Kulleg Steinhorn ",
		["Sheendra Tallgrass"] = "Sheendra Hochschilf ",
		["Ruw"] = "Ruw ",
		["Camp Mojache Brave"] = "Kriegerheld von Camp Mojache",
		["Nijel's Point Guard"] = "Wache der Nijelspitze",
		["Ghost Walker Brave"] = "Kriegerheld des Geistwandlerpostens",
		["Kargath Grunt"] = "Grunzer von Kargath",
		["Servant of Antu'sul"] = "Diener von Antu'sul",
		["Logannas"] = "Logannas ",
		["Bronk"] = "Bronk ",
		["Worb Strongstitch"] = "Worb Starkahle ",
		["Nioma"] = "Nioma ",
		["Harggan"] = "Harggan ",
		["Nina Lightbrew"] = "Nina Brauleicht ",
		["Occulus"] = "Occulus",
		["Chronalis"] = "Chronalis",
		["Tick"] = "Tick",
		["Jin'Zallah the Sandbringer"] = "Jin'Zallah der Sandbringer",
		["Omgorn the Lost"] = "Omgorn der Verirrte",
		["Cyclok the Mad"] = "Cyclok der Irre",
		["Kregg Keelhaul"] = "Kregg Kielhol",
		["Soriid the Devourer"] = "Soriid der Verschlinger",
		["Haarka the Ravenous"] = "Haarka der Gefräßige",
		["Greater Firebird"] = "Großer Feuervogel",
		["The Reak"] = "Der Reak",
		["Ironback"] = "Eisenpanzer",
		["Jalinde Summerdrake"] = "Jalinde Sommerdrache",
		["Grimungous"] = "Grimungous",
		["Retherokk the Berserker"] = "Retherokk der Berserker",
		["Mith'rethis the Enchanter"] = "Mith'rethis der Verzauberer",
		["Rekk'tilac"] = "Rekk'tilac",
		["Smoldar"] = "Smoldar",
		["Faulty War Golem"] = "Defekter Kriegsgolem",
		["Shleipnarr"] = "Shleipnarr",
		["Scald"] = "Scald",
		["Highlord Mastrogonde"] = "Hochlord Mastrogonde",
		["Slave Master Blackheart"] = "Sklavenmeister Schwarzherz",
		["Mojo the Twisted"] = "Mojo der Verwachsene",
		["Magronos the Unyielding"] = "Magronos der Unerschütterliche",
		["Akubar the Seer"] = "Akubar der Seher",
		["Spiteflayer"] = "Fledderschnabel",
		["Ravage"] = "Verheerer",
		["Clack the Reaver"] = "Clack der Häscher",
		["Deatheye"] = "Todesauge",
		["Grunter"] = "Suhlaman",
		["Dreadscorn"] = "Mutreich",
		["Carlo Aurelius"] = "Carlo Aurelius ",
		["Atal'ai Deathwalker's Spirit"] = "Todeswandlergeist der Atal'ai",
		["Atal'ai Slave"] = "Sklave der Atal'ai",
		["Nightmare Whelp"] = "Nachtmahrwelpe",
		["Atal'ai Skeleton"] = "Skelett der Atal'ai",
		["Hakkari Sapper"] = "Pionier der Hakkari",
		["Dark Iron Marksman"] = "Schütze der Dunkeleisenzwerge",
		["Archmage Xylem"] = "Erzmagier Xylem",
		["Captain Vanessa Beltis"] = "Hauptmann Vanessa Beltis",
		["Lindros"] = "Lindros",
		["Patrick Mills"] = "Patrick Mühlens",
		["Master Wood"] = "Meister Holz",
		["Deep Lurker"] = "Tiefenlauerer",
		["Lathoric the Black"] = "Lathoric der Schwarze",
		["Sanath Lim-yo"] = "Sanath Lim-yo",
		["Obsidion"] = "Obsidion",
		["Enslaved Archaeologist"] = "Versklavter Archäologe",
		["Warlord Krellian"] = "Kriegsherr Krellian",
		["Caravan Master Tset"] = "Karawanenmeister Tset",
		["Dying Archaeologist"] = "Sterbender Archäologe",
		["Twilight Idolater"] = "Götzendiener des Schattenhammers",
		["Kim'jael"] = "Kim'jael",
		["Dorius"] = "Dorius",
		["Zamael Lunthistle"] = "Zamael Distelmond",
		["Hakkari Bloodkeeper"] = "Blutbewahrer der Hakkari",
		["Nilith Lokrav"] = "Nilith Lokrav",
		["Raze"] = "Schleifer ",
		["Shadowsilk Poacher"] = "Schattenseidewilderer ",
		["Avatar of Hakkar"] = "Avatar von Hakkar",
		["Trade Master Kovic"] = "Handwerksmeister Kovic ",
		["Clunk"] = "Clunk ",
		["Skeletal Servant"] = "Skelettdiener",
		["Kalaran Windblade"] = "Kalaran Windklinge",
		["Kalaran the Deceiver"] = "Kalaran der Betrüger",
		["Nightmare Suppressor"] = "Nachtmahrunterdrücker",
		["Dark Iron Sentry"] = "Wachposten der Dunkeleisenzwerge",
		["Squire Maltrake"] = "Knappe Malzdarr",
		["Atal'ai Totem"] = "Totem der Atal'ai",
		["Blighted Surge"] = "Verseuchte Woge",
		["Plague Ravager"] = "Seuchenverheerer",
		["Blighted Horror"] = "Verseuchter Schrecken",
		["Plague Monstrosity"] = "Seuchenmonster",
		["Scourge Soldier"] = "Soldat der Geißel",
		["Cursed Mage"] = "Verfluchter Magier",
		["Scourge Warder"] = "Wärter der Geißel",
		["Dark Caster"] = "Dunkler Zauberer",
		["Scourge Guard"] = "Wache der Geißel",
		["Dread Weaver"] = "Schreckenswirker",
		["Scourge Champion"] = "Champion der Geißel",
		["Cannibal Ghoul"] = "Menschenfresserghul",
		["Gibbering Ghoul"] = "Jammernder Ghul",
		["Diseased Flayer"] = "Kranker Schinder",
		["Putrid Gargoyle"] = "Eitriger Gargoyle",
		["Putrid Shrieker"] = "Eitriger Schriller",
		["Unseen Servant"] = "Ungesehener Diener",
		["Eyeless Watcher"] = "Augenloser Behüter",
		["Torn Screamer"] = "Zerrissener Schreier",
		["Hate Shrieker"] = "Hassschriller",
		["Death Singer"] = "Todessänger",
		["Stitched Horror"] = "Stichelnarbenschrecken",
		["Gangled Golem"] = "Schlottergolem",
		["Stitched Golem"] = "Narbengolem",
		["Dark Adept"] = "Dunkler Adept ",
		["Death Cultist"] = "Todeskultist ",
		["Vile Tutor"] = "Übler Tutor ",
		["Shadowmage"] = "Schattenmagier ",
		["Dark Summoner"] = "Dunkler Beschwörer ",
		["Necromancer"] = "Totenbeschwörer ",
		["Crypt Stalker"] = "Gruftpirscher",
		["Crypt Walker"] = "Gruftwandler",
		["Crypt Horror"] = "Gruftschrecken",
		["Crypt Slayer"] = "Grufttöter",
		["Mossflayer Scout"] = "Späher der Moosschinder",
		["Mossflayer Shadowhunter"] = "Schattenjäger der Moosschinder",
		["Mossflayer Cannibal"] = "Kannibale der Moosschinder",
		["Woodsman"] = "Waldmann",
		["Ranger"] = "Waldläufer",
		["Pathstrider"] = "Pfadschreiter",
		["Dark Iron Lookout"] = "Ausguck der Dunkeleisenzwerge",
		["Magus Rimtori"] = "Magus Rimtori",
		["Atal'alarion"] = "Atal'alarion ",
		["Blood Elf Defender"] = "Blutelfenverteidiger",
		["Jediga"] = "Jediga",
		["Umbranse the Spiritspeaker"] = "Umbranse der Geistsprecher",
		["Plaguehound Runt"] = "Kleiner Seuchenhund",
		["Plaguehound"] = "Seuchenhund",
		["Frenzied Plaguehound"] = "Rasender Seuchenhund",
		["Plaguebat"] = "Seuchenfledermaus",
		["Noxious Plaguebat"] = "Giftige Seuchenfledermaus",
		["Monstrous Plaguebat"] = "Monströse Seuchenfledermaus",
		["Carrion Grub"] = "Aaslarve",
		["Carrion Devourer"] = "Aasverschlinger",
		["Living Decay"] = "Lebender Verfall",
		["Rotting Sludge"] = "Faulender Schlamm",
		["Angered Infernal"] = "Erboste Höllenbestie",
		["Alexandra Constantine"] = "Alexandra Konstantin ",
		["Kroum"] = "Kroum ",
		["Mithril Dragonling"] = "Mithrildrachling",
		["Infernal Servant"] = "Höllenbestiendiener",
		["Morta'gya the Keeper"] = "Morta'gya die Bewahrerin",
		["Hukku's Voidwalker"] = "Hukkus Leerwandler",
		["Hukku's Succubus"] = "Hukkus Sukkubus",
		["Hukku's Imp"] = "Hukkus Wichtel",
		["Jes'rimon"] = "Jes'rimon",
		["The Evalcharr"] = "Evalcharr",
		["Auctioneer Beardo"] = "Auktionator Barto",
		["Saern Priderunner"] = "Saern Stolzlauf",
		["Auctioneer Tolon"] = "Auktionator Tolon",
		["Auctioneer Chilton"] = "Auktionator Chilton",
		["Auctioneer Buckler"] = "Auktionator Schildbuckel",
		["Auctioneer Leeka"] = "Auktionatorin Leeka",
		["Auctioneer Thathung"] = "Auktionator Thathung",
		["Auctioneer Stampi"] = "Auktionator Rikno Noragg",
		["Felbeast"] = "Teufelsbestie",
		["Jubie Gadgetspring"] = "Jubie Federdings ",
		["Massive Infernal"] = "Massive Höllenbestie",
		["Dreadlord"] = "Schreckenslord",
		["Felguard Elite"] = "Eliteteufelswache",
		["Manahound"] = "Manahund",
		["Auctioneer Fitch"] = "Auktionator Fitch",
		["Auctioneer Redmuse"] = "Auktionatorin Rotlaune",
		["Auctioneer Epitwee"] = "Auktionatorin Epitwee",
		["Auctioneer Gullem"] = "Auktionator Gullem",
		["Auctioneer Golothas"] = "Auktionator Golothas",
		["Auctioneer Wabang"] = "Auktionator Wabang",
		["Buzzek Bracketswing"] = "Buzzek Pratzenschwing ",
		["Raytaf"] = "Raytaf ",
		["Shahiar"] = "Shahiar ",
		["Zaman"] = "Zaman ",
		["Mosshoof Stag"] = "Mooshufhirsch",
		["Mosshoof Courser"] = "Mooshufrenner",
		["Timberweb Recluse"] = "Baumspinnereremit",
		["Mistwing Rogue"] = "Nebelschwingenwüter",
		["Mistwing Ravager"] = "Nebelschwingenverheerer",
		["Forest Ooze"] = "Waldbrühschlammer",
		["Deathly Usher"] = "Tödlicher Pförtner",
		["Muck Splash"] = "Dreckplatscher",
		["Royal Historian Archesonus"] = "Königliche Historikerin Archesonus",
		["A tormented voice"] = "Eine gequälte Stimme",
		["Franclorn Forgewright"] = "Franclorn Schmiedevater",
		["Anvilrage Overseer"] = "Vorarbeiter der Zorneshämmer",
		["Anvilrage Warden"] = "Aufseher der Zorneshämmer",
		["Anvilrage Guardsman"] = "Gardist der Zorneshämmer",
		["Anvilrage Footman"] = "Fußsoldat der Zorneshämmer",
		["Anvilrage Soldier"] = "Soldat der Zorneshämmer",
		["Anvilrage Medic"] = "Sanitäter der Zorneshämmer",
		["Anvilrage Officer"] = "Offizier der Zorneshämmer",
		["Shadowforge Peasant"] = "Arbeiter der Schattenschmiede",
		["Doomforge Craftsman"] = "Schicksalsträchtiger Handwerker",
		["Anvilrage Marshal"] = "Marschall der Zorneshämmer",
		["Doomforge Dragoon"] = "Schicksalsträchtiger Dragoner",
		["Doomforge Arcanasmith"] = "Schicksalsträchtiger Arkanaschmied",
		["Anvilrage Reservist"] = "Reservist der Zorneshämmer",
		["Shadowforge Citizen"] = "Bürger der Schattenschmiede",
		["Anvilrage Captain"] = "Hauptmann der Zorneshämmer",
		["Shadowforge Senator"] = "Senator der Schattenschmiede",
		["Warbringer Construct"] = "Kriegshetzerkonstrukt",
		["Ragereaver Golem"] = "Wuthäschergolem",
		["Wrath Hammer Construct"] = "Kriegshammerkonstrukt",
		["Molten War Golem"] = "Geschmolzener Kriegsgolem",
		["Fireguard"] = "Brandwache",
		["Blazing Fireguard"] = "Lodernde Brandwache",
		["Fireguard Destroyer"] = "Brandwachenzerstörer",
		["Twilight's Hammer Torturer"] = "Folterknecht des Schattenhammers",
		["Twilight Emissary"] = "Abgesandter des Schattenhammers",
		["Twilight Bodyguard"] = "Leibwache des Schattenhammers",
		["Twilight's Hammer Ambassador"] = "Botschafter des Schattenhammers",
		["Arena Spectator"] = "Arenazuschauer",
		["Quarry Slave"] = "Steinbruchsklave",
		["Weapon Technician"] = "Waffentechniker",
		["Bloodhound"] = "Bluthund",
		["Bloodhound Mastiff"] = "Aggressiver Bluthund",
		["Panzor the Invincible"] = "Panzor der Unbesiegbare",
		["The Behemoth"] = "Das Ungetüm",
		["Dredge Worm"] = "Baggerwurm",
		["Deep Stinger"] = "Tiefstecher",
		["Dark Screecher"] = "Dunkelkreischer",
		["Burrowing Thundersnout"] = "Grabende Donnerschnauze",
		["Princess Moira Bronzebeard"] = "Prinzessin Moira Bronzebart ",
		["Borer Beetle"] = "Bohrkäfer",
		["Cave Creeper"] = "Höhlenkrabbler",
		["Angerclaw Bear"] = "Wutklauenbär",
		["Angerclaw Grizzly"] = "Wutklauengrizzly",
		["Angerclaw Mauler"] = "Wutklauenraufer",
		["Felpaw Wolf"] = "Teufelspfotenwolf",
		["Felpaw Scavenger"] = "Teufelspfotenaasfresser",
		["Felpaw Ravager"] = "Teufelspfotenverheerer",
		["Blackrock Drake"] = "Drache des Schwarzfels",
		["Hematos"] = "Hematos",
		["Krom'Grul"] = "Krom'Grul",
		["Thauris Balgarr"] = "Thauris Balgarr",
		["Gruklash"] = "Gruklash",
		["Firegut Captain"] = "Firegut Captain",
		["Malfunctioning Reaver"] = "Fehlfunktionierender Häscher",
		["Ironhand Guardian"] = "Eisenhandwächter",
		["Golem Lord Argelmach"] = "Golemlord Argelmach",
		["Voidwalker Minion"] = "Leerwandlerdiener",
		["Bael'Gar"] = "Bael'Gar",
		["Lord Incendius"] = "Lord Incendius",
		["High Interrogator Gerstahn"] = "Verhörmeisterin Gerstahn ",
		["Emperor Dagran Thaurissan"] = "Imperator Dagran Thaurissan",
		["Commander Gor'shak"] = "Kommandant Gor'shak ",
		["Kharan Mighthammer"] = "Kharan Hammermacht",
		["Dughal Stormwing"] = "Dughal Sturmbart",
		["Marshal Windsor"] = "Marschall Windsor",
		["Pyromancer Loregrain"] = "Pyromant Weisenkorn",
		["Lord Roccor"] = "Lord Roccor",
		["Overmaster Pyron"] = "Übermeister Pyron",
		["Gorosh the Dervish"] = "Gorosh der Derwisch",
		["Grizzle"] = "Grizzle",
		["Eviscerator"] = "Ausweider",
		["Ok'thor the Breaker"] = "Ok'thor der Zerstörer",
		["Anub'shiah"] = "Anub'shiah",
		["Hedrum the Creeper"] = "Hedrum der Krabbler",
		["General Angerforge"] = "General Zornesschmied",
		["Hate'rel"] = "Hass'rel",
		["Anger'rel"] = "Zorn'rel",
		["Vile'rel"] = "Bös'rel",
		["Gloom'rel"] = "Dunk'rel",
		["Seeth'rel"] = "Wut'rel",
		["Doom'rel"] = "Un'rel",
		["Dope'rel"] = "Trott'rel",
		["Warder Stilgiss"] = "Wärter Stilgiss",
		["Verek"] = "Verek",
		["Scarshield Grunt"] = "Grunzer der Schmetterschilde ",
		["Scarshield Sentry"] = "Wachposten der Schmetterschilde ",
		["Scarshield Acolyte"] = "Akolyt der Schmetterschilde ",
		["Scarshield Quartermaster"] = "Rüstmeister der Schmetterschilde ",
		["Fineous Darkvire"] = "Fineous Dunkelader ",
		["Warlord Goretooth"] = "Kriegsherr Bluthauer ",
		["Shadowmage Vivian Lagrave"] = "Schattenmagierin Vivian Lagrave ",
		["Hierophant Theodora Mulvadania"] = "Hierophantin Theodora Mulvadania ",
		["Lexlort"] = "Lexlort ",
		["Galamav the Marksman"] = "Galamav der Schütze ",
		["Thal'trak Proudtusk"] = "Thal'trak Ehrenhauer ",
		["Razal'blade"] = "Razal'hieb ",
		["Thunderheart"] = "Donnerherz ",
		["Initiate Amakkar"] = "Initiand Amakkar ",
		["Grunt Gargal"] = "Grunzer Gargal ",
		["Rage Talon Dragonspawn"] = "Drachenbrut der Zornkrallen",
		["Scarshield Legionnaire"] = "Legionär der Schmetterschilde ",
		["Scarshield Spellbinder"] = "Zauberbinder der Schmetterschilde ",
		["Eridan Bluewind"] = "Eridan Blauwind ",
		["J.D. Collie"] = "J.D. Collie",
		["Larion"] = "Larion",
		["Muigin"] = "Muigin",
		["Sha'ni Proudtusk"] = "Sha'ni Ehrenhauer",
		["Ambassador Flamelash"] = "Botschafter Flammenschlag",
		["Young Diemetradon"] = "Junger Dimetrodon",
		["Diemetradon"] = "Dimetrodon",
		["Elder Diemetradon"] = "Alter Dimetrodon",
		["Fledgling Pterrordax"] = "Pterrordaxjunges",
		["Pterrordax"] = "Pterrordax",
		["Frenzied Pterrordax"] = "Rasender Pterrordax",
		["Gor'tesh"] = "Gor'tesh",
		["Oralius"] = "Oralius",
		["Highlord Omokk"] = "Hochlord Omokk",
		["Spirestone Battle Mage"] = "Kampfmagier der Felsspitzoger",
		["Spirestone Mystic"] = "Mystiker der Felsspitzoger",
		["Spirestone Enforcer"] = "Vollstrecker der Felsspitzoger",
		["Spirestone Reaver"] = "Häscher der Felsspitzoger",
		["Spirestone Ogre Magus"] = "Magus der Felsspitzoger",
		["Spirestone Warlord"] = "Kriegsherr der Felsspitzoger",
		["Spirestone Lord Magus"] = "Maguslord der Felsspitzoger",
		["Spirestone Battle Lord"] = "Kampflord der Felsspitzoger",
		["Spirestone Butcher"] = "Metzger der Felsspitzoger",
		["Shadow Hunter Vosh'gajin"] = "Schattenjägerin Vosh'gajin",
		["War Master Voone"] = "Kriegsmeister Voone",
		["Smolderthorn Mystic"] = "Mystiker der Gluthauer",
		["Smolderthorn Shadow Priest"] = "Schattenpriester der Gluthauer",
		["Smolderthorn Headhunter"] = "Kopfjäger der Gluthauer",
		["Scarshield Warlock"] = "Hexenmeister der Schmetterschilde ",
		["Scarshield Raider"] = "Räuber der Schmetterschilde ",
		["Firebrand Grunt"] = "Grunzer der Feuerbrand ",
		["Firebrand Legionnaire"] = "Legionär der Feuerbrand ",
		["Firebrand Darkweaver"] = "Dunkelwirker der Feuerbrand ",
		["Firebrand Invoker"] = "Herbeirufer der Feuerbrand ",
		["Firebrand Dreadweaver"] = "Schreckenswirker der Feuerbrand ",
		["Firebrand Pyromancer"] = "Pyromant der Feuerbrand ",
		["Smolderthorn Shadow Hunter"] = "Schattenjäger der Gluthauer",
		["Smolderthorn Witch Doctor"] = "Hexendoktor der Gluthauer",
		["Smolderthorn Axe Thrower"] = "Axtwerfer der Gluthauer",
		["Smolderthorn Berserker"] = "Berserker der Gluthauer",
		["Smolderthorn Seer"] = "Seher der Gluthauer",
		["Williden Marshal"] = "Williden Marschall",
		["Petra Grossen"] = "Petra Grossen",
		["Enraged Wyvern"] = "Wütender Flügeldrache",
		["Donova Snowden"] = "Donova Schneeweh",
		["Gaeriyan"] = "Gaeriyan",
		["Incendosaur"] = "Incendosaurier",
		["Houndmaster Grebmar"] = "Hundemeister Grebmar",
		["Blazerunner"] = "Sturmflamme",
		["Living Storm"] = "Lebendiger Sturm",
		["Twilight's Hammer Executioner"] = "Henker des Schattenhammers",
		["Scarshield Worg"] = "Worg der Schmetterschilde ",
		["Spawn of Bael'Gar"] = "Brut von Bael'Gar",
		["Dark Keeper Vorfalk"] = "Dunkelbewahrer Vorfalk",
		["Dark Keeper Bethek"] = "Dunkelbewahrer Bethek",
		["Dark Keeper Uggel"] = "Dunkelbewahrer Uggel",
		["Dark Keeper Zimrel"] = "Dunkelbewahrer Zimrel",
		["Dark Keeper Ofgut"] = "Dunkelbewahrer Ofgut",
		["Dark Keeper Pelver"] = "Dunkelbewahrer Pelver",
		["Dark Guard"] = "Dunkelwache",
		["Scarlet Warder"] = "Scharlachroter Wärter",
		["Scarlet Praetorian"] = "Scharlachroter Prätorianer",
		["Scarlet Cleric"] = "Scharlachroter Kleriker",
		["Scarlet Curate"] = "Scharlachroter Kurat",
		["Scarlet Archmage"] = "Scharlachroter Erzmagier",
		["Scarlet Enchanter"] = "Scharlachroter Verzauberer",
		["Aquementas"] = "Aquementas",
		["Xavathras"] = "Xavathras",
		["Cyrus Therepentous"] = "Cyrus Therepentous",
		["Gadgetzan Bruiser"] = "Haudrauf von Gadgetzan",
		["Frenzied Black Drake"] = "Rasender Schwarzdrache ",
		["Chieftain Bloodmaw"] = "Häuptling Blutmagen",
		["Overlord Ror"] = "Oberanführer Ror",
		["Golhine the Hooded"] = "Golhine der Kapuzenträger ",
		["Miblon Snarltooth"] = "Miblon Fletschmaul",
		["Watchman Doomgrip"] = "Wachmann Stahlgriff",
		["Cloned Ooze"] = "Geklonter Brühschlammer",
		["Gorishi Egg"] = "Ei der Gorishi",
		["Gorishi Grub"] = "Larve der Gorishi",
		["Plugger Spazzring"] = "Stöpsel Zapfring",
		["Mistress Nagmara"] = "Herrin Nagmara",
		["Phalanx"] = "Phalanx",
		["Private Rocknot"] = "Gefreiter Rocknot",
		["Lord Banehollow"] = "Lord Schattenfluch",
		["Shadow Lord Fel'dan"] = "Schattenlord Fel'dan",
		["Rakaiah"] = "Rakaiah",
		["Grark Lorkrub"] = "Grark Lorkrub",
		["Enraged Felbat"] = "Wütende Teufelsfledermaus",
		["Blackrock Ambusher"] = "Wegelagerer des Schwarzfels",
		["Enraged Gryphon"] = "Wütender Greif",
		["Enraged Hippogryph"] = "Wütender Hippogryph",
		["Arathandris Silversky"] = "Arathandris Silberwolke ",
		["Maybess Riverbreeze"] = "Maybess Flussbrise ",
		["Hurley Blackbreath"] = "Hurley Pestatem",
		["High Executioner Nuzrak"] = "Oberhenker Nuzrak ",
		["Shadow of Lexlort"] = "Schatten von Lexlort ",
		["Enohar Thunderbrew"] = "Enohar Donnerbräu",
		["Blackbreath Crony"] = "Pestatems Spießgeselle",
		["Ribbly Screwspigot"] = "Ribbly Schraubstutz",
		["Yuka Screwspigot"] = "Yuka Schraubstutz",
		["Grim Patron"] = "Grimmiger Gast",
		["Guzzling Patron"] = "Saufender Gast",
		["Cawind Trueaim"] = "Cawind Blattschuss ",
		["Hammered Patron"] = "Betrunkener Gast",
		["Marshal Maxwell"] = "Marschall Maxwell",
		["Jalinda Sprig"] = "Jalinda Sprig",
		["Helendis Riverhorn"] = "Helendis Flusshorn",
		["Ragged John"] = "Struppiger John",
		["Mayara Brightwing"] = "Mayara Wolkenglanz",
		["Overlord Wyrmthalak"] = "Oberanführer Wyrmthalak",
		["Bloodaxe Veteran"] = "Veteran der Blutäxte ",
		["Bannok Grimaxe"] = "Bannok Grimmaxt ",
		["Arei"] = "Arei",
		["Treant Spirit"] = "Treantgeist",
		["Hahk'Zor"] = "Hahk'Zor",
		["Gorgon'och"] = "Gorgon'och",
		["Blackrock Raider"] = "Räuber des Schwarzfels",
		["Laris Geardawdle"] = "Laris Bummelritzel",
		["Karna Remtravel"] = "Karna Wirrbart",
		["Torwa Pathfinder"] = "Torwa Pfadfinder",
		["Dreka'Sur"] = "Dreka'Sur",
		["Gargantuan Ooze"] = "Riesengroßer Brühschlammer",
		["U'cha"] = "U'cha",
		["A-Me 01"] = "A-Me 01",
		["Scorching Totem"] = "Versengendes Totem",
		["Agnar Beastamer"] = "Agnar Tiermeister",
		["Ograbisi"] = "Ograbisi",
		["Shill Dinger"] = "Shill Dinger",
		["Tobias Seecher"] = "Tobias Sucher",
		["Crest Killer"] = "Wappentöter",
		["Jaz"] = "Jaz",
		["Marshal Reginald Windsor"] = "Marschall Reginald Windsor",
		["Lar'korwi Mate"] = "Lar'korwis Weibchen",
		["Lar'korwi"] = "Lar'korwi",
		["Windwall Totem III"] = "Totem der Windmauer III",
		["Ember Worg"] = "Glutworg",
		["Venomtip Scorpid"] = "Giftstichskorpid",
		["Bloodaxe Raider"] = "Räuber der Blutäxte ",
		["Bloodaxe Evoker"] = "Rufer der Blutäxte ",
		["Slavering Ember Worg"] = "Sabbernder Glutworg",
		["Deathlash Scorpid"] = "Todespeitschenskorpid",
		["Bloodaxe Worg"] = "Worg der Blutäxte ",
		["Giant Ember Worg"] = "Riesiger Glutworg",
		["Firetail Scorpid"] = "Feuerschwanzskorpid",
		["Spire Scorpid"] = "Spitzenskorpid",
		["Yorba Screwspigot"] = "Yorba Schraubstutz",
		["Burning Imp"] = "Brennender Wichtel",
		["Bloodaxe Warmonger"] = "Kriegstreiber der Blutäxte ",
		["Bloodaxe Summoner"] = "Beschwörer der Blutäxte ",
		["Ghok Bashguud"] = "Ghok Haudrauf ",
		["Quartermaster Zigris"] = "Rüstmeister Zigris ",
		["Flamekin Spitter"] = "Flammkinspeier",
		["Flamekin Sprite"] = "Flammkingrimmling",
		["Flamekin Torcher"] = "Flammkinfackler",
		["Flamekin Rager"] = "Flammkinwüterich",
		["Pyroguard Emberseer"] = "Feuerwache Glutseher",
		["Blackhand Dreadweaver"] = "Schreckenswirker der Schwarzfaustlegion ",
		["Blackhand Summoner"] = "Beschwörer der Schwarzfaustlegion ",
		["Blackhand Veteran"] = "Veteran der Schwarzfaustlegion ",
		["Mathredis Firestar"] = "Mathredis Feuerstern",
		["Auctioneer Grimful"] = "Auktionator Ingrimm",
		["Auctioneer Grizzlin"] = "Auktionatorin Grizzlin",
		["Auctioneer Kresky"] = "Auktionator Kresky",
		["Auctioneer Lympkin"] = "Auktionatorin Lympkin",
		["Salia"] = "Salia",
		["Moora"] = "Moora",
		["Jaedenar Legionnaire"] = "Legionär von Jaedenar",
		["Locheed"] = "Locheed",
		["Prince Xavalis"] = "Prinz Xavalis",
		["Entropic Beast"] = "Entropische Bestie",
		["Entropic Horror"] = "Entropischer Schrecken",
		["Jarquia"] = "Jarquia",
		["Magmus"] = "Magmus",
		["Shadowforge Flame Keeper"] = "Flammenbewahrer der Schattenschmiede",
		["Xon'cha"] = "Xon'cha ",
		["Winna Hazzard"] = "Winna Hazzard",
		["Ringo"] = "Ringo",
		["Brackenwall Enforcer"] = "Vollstrecker von Brackenwall",
		["Lakeshire Guard"] = "Wache von Seenhain",
		["Gorishi Hive Guard"] = "Schwarmwache der Gorishi",
		["Gorishi Hive Queen"] = "Schwarmkönigin der Gorishi",
		["Ribbly's Crony"] = "Ribblys Spießgeselle",
		["Gereck"] = "Gereck ",
		["Grimestack"] = "Rußhuf ",
		["High Priestess of Thaurissan"] = "Hohepriesterin von Thaurissan",
		["Deathmaw"] = "Totenreißer",
		["Terrorspark"] = "Terrorstifter",
		["Rage Talon Flamescale"] = "Flammenschuppe der Zornkrallen",
		["High Justice Grimstone"] = "Oberrichter Grimmstein",
		["Volchan"] = "Volchan",
		["Rookery Whelp"] = "Krähenhorstwelpe",
		["Lord Victor Nefarius"] = "Lord Victor Nefarius ",
		["Spire Scarab"] = "Spitzenskarabäus",
		["Lady Sylvanas Windrunner"] = "Fürstin Sylvanas Windläufer ",
		["Rexxar"] = "Rexxar ",
		["Moonflare Totem"] = "Totem des Mondleuchtfeuers",
		["Onyxia"] = "Onyxia",
		["General Colbatann"] = "General Colbatann",
		["Mezzir the Howler"] = "Mezzir der Heuler",
		["Kashoch the Reaver"] = "Kashoch der Häscher",
		["Grizzle Snowpaw"] = "Grizzel Schneepfote",
		["Rak'shiri"] = "Rak'shiri",
		["Lady Hederine"] = "Lady Hederine",
		["Azurous"] = "Azurous",
		["Misha"] = "Misha",
		["Flame Buffet Totem"] = "Totem des Flammenpuffers",
		["Superior Healing Ward"] = "Überragender Heilungszauberschutz",
		["Halycon"] = "Halycon",
		["Bloodaxe Worg Pup"] = "Worgwelpe der Blutäxte",
		["Bijou"] = "Bijou",
		["Rookery Guardian"] = "Krähenhorstwächter",
		["Kibler"] = "Kibler",
		["Burning Felhound"] = "Brennender Teufelshund",
		["Burning Felguard"] = "Brennende Teufelswache",
		["Solakar Flamewreath"] = "Solakar Feuerkrone",
		["Tinkee Steamboil"] = "Tinkee Kesseldampf",
		["Gizrul the Slavener"] = "Gizrul der Geifernde",
		["Dulciea Frostmoon"] = "Dulciea Eismond ",
		["Vaelan"] = "Vaelan",
		["Scarshield Infiltrator"] = "Spitzel der Schmetterschilde ",
		["Ranshalla"] = "Ranshalla",
		["Jaron Stoneshaper"] = "Jaron Steinschleifer ",
		["Krakle"] = "Krakle",
		["Storm Shadowhoof"] = "Storm Schattenhuf",
		["Aurora Skycaller"] = "Aurora Himmelsrufer",
		["Umi Rumplesnicker"] = "Umi Rumpelkicher",
		["Trull Failbane"] = "Trull Scheiterbann",
		["Witch Doctor Mau'ari"] = "Hexendoktorin Mau'ari",
		["Blackhand Incarcerator"] = "Einkerkerer der Schwarzfaustlegion ",
		["Blackhand Elite"] = "Elitesoldat der Schwarzfaustlegion ",
		["Blackhand Assassin"] = "Auftragsmörder der Schwarzfaustlegion ",
		["Blackhand Iron Guard"] = "Eisenwache der Schwarzfaustlegion ",
		["Emberstrife"] = "Aschenschwinge",
		["Gyth"] = "Gyth ",
		["Vaelastrasz the Red"] = "Vaelastrasz der Rote",
		["Kergul Bloodaxe"] = "Kergul Blutaxt ",
		["Gruul Darkblade"] = "Gruul Dunkelklinge ",
		["General Drakkisath"] = "General Drakkisath",
		["Yaelika Farclaw"] = "Yaelika Klauengift ",
		["Rage Talon Dragon Guard"] = "Drachenwache der Zornkrallen",
		["Shrye Ragefist"] = "Shrye Zornfaust ",
		["Rage Talon Captain"] = "Hauptmann der Zornkrallen",
		["Rage Talon Fire Tongue"] = "Feuerzunge der Zornkrallen",
		["Spire Spider"] = "Spitzenspinne",
		["Spire Spiderling"] = "Spitzenspinnling",
		["Crystal Fang"] = "Kristallfangzahn",
		["Omusa Thunderhorn"] = "Omusa Donnerhorn ",
		["Ravaged Cadaver"] = "Verheerter Kadaver",
		["Mangled Cadaver"] = "Geschundener Kadaver",
		["Broken Cadaver"] = "Zerbrochener Kadaver",
		["Spectral Citizen"] = "Spektraler Bürger",
		["Ghostly Citizen"] = "Geisterhafter Bürger",
		["Vengeful Phantom"] = "Rachsüchtiges Phantom",
		["Spiteful Phantom"] = "Boshaftes Phantom",
		["Wrath Phantom"] = "Zorniges Phantom",
		["Skeletal Guardian"] = "Skelettwächter",
		["Skeletal Berserker"] = "Skelettberserker",
		["Skul"] = "Skul",
		["Black Guard Sentry"] = "Wachposten der schwarzen Wache",
		["Thuzadin Shadowcaster"] = "Thuzadinschattenzauberer",
		["Thuzadin Acolyte"] = "Thuzadinakolyt",
		["Thuzadin Necromancer"] = "Thuzadintotenbeschwörer",
		["Plague Ghoul"] = "Seuchenghul",
		["Ghoul Ravener"] = "Tobsüchtiger Ghul",
		["Fleshflayer Ghoul"] = "Fleischschinderghul",
		["Rockwing Gargoyle"] = "Steinschwingengargoyle",
		["Rockwing Screecher"] = "Steinschwingenkreischer",
		["Eye of Naxxramas"] = "Auge von Naxxramas",
		["Crypt Crawler"] = "Gruftkriecher",
		["Crypt Beast"] = "Gruftbestie",
		["Patchwork Horror"] = "Flickwerkschrecken",
		["Ash'ari Crystal"] = "Kristall von Ash'ari",
		["Bile Spewer"] = "Gallspucker",
		["Venom Belcher"] = "Giftrülpser",
		["Crimson Guardsman"] = "Purpurroter Gardist",
		["Crimson Conjuror"] = "Purpurroter Herbeizauberer",
		["Crimson Initiate"] = "Purpurroter Initiand",
		["Crimson Defender"] = "Purpurroter Verteidiger",
		["Crimson Sorcerer"] = "Purpurroter Zauberhexer",
		["Crimson Priest"] = "Purpurroter Priester",
		["Crimson Gallant"] = "Purpurroter Kavalier",
		["Crimson Battle Mage"] = "Purpurroter Kampfmagier",
		["Crimson Inquisitor"] = "Purpurroter Inquisitor",
		["Warchief Rend Blackhand"] = "Kriegshäuptling Rend Schwarzfaust",
		["The Beast"] = "Die Bestie",
		["Gregor Greystone"] = "Gregor Graustein ",
		["Vectus"] = "Vectus",
		["Marduk Blackpool"] = "Marduk Blackpool",
		["Magistrate Barthilas"] = "Magistrat Barthilas",
		["Baroness Anastari"] = "Baroness Anastari",
		["Nerub'enkan"] = "Nerub'enkan",
		["Maleki the Pallid"] = "Maleki der Leichenblasse",
		["Ramstein the Gorger"] = "Ramstein der Verschlinger",
		["Baron Rivendare"] = "Baron Totenschwur",
		["Plagued Rat"] = "Verseuchte Ratte",
		["Chromatic Whelp"] = "Chromatischer Welpe",
		["Chromatic Dragonspawn"] = "Chromatische Drachenbrut",
		["Prospector Ironboot"] = "Ausgrabungsleiter Eisenschuh",
		["Plagued Insect"] = "Verseuchtes Insekt",
		["Shrieking Banshee"] = "Kreischende Banshee",
		["Wailing Banshee"] = "Heulende Banshee",
		["Felnok Steelspring"] = "Felnok Stahlfeder",
		["Scholomance Adept"] = "Adept aus Scholomance",
		["Scholomance Neophyte"] = "Neophyt aus Scholomance",
		["Scholomance Acolyte"] = "Akolyt aus Scholomance",
		["Scholomance Occultist"] = "Okkultist aus Scholomance",
		["Scholomance Student"] = "Student aus Scholomance",
		["Scholomance Necrolyte"] = "Nekrolyt aus Scholomance",
		["Scholomance Necromancer"] = "Totenbeschwörer aus Scholomance",
		["Splintered Skeleton"] = "Abgesplittertes Skelett",
		["Skulking Corpse"] = "Schleichender Kadaver ",
		["Unstable Corpse"] = "Faulender Leichnam",
		["Reanimated Corpse"] = "Reanimierter Kadaver",
		["Risen Lackey"] = "Auferstandener Lakai",
		["Risen Aberration"] = "Auferstandene Entartung",
		["Risen Warrior"] = "Auferstandener Krieger",
		["Risen Protector"] = "Auferstandener Beschützer",
		["Risen Construct"] = "Auferstandenes Konstrukt",
		["Risen Guard"] = "Auferstandene Wache",
		["Risen Bonewarder"] = "Auferstandener Knochenwärter",
		["Diseased Ghoul"] = "Kranker Ghul",
		["Ragged Ghoul"] = "Struppiger Ghul ",
		["Spectral Tutor"] = "Spektraler Tutor",
		["Spectral Researcher"] = "Spektraler Forscher",
		["Spectral Teacher"] = "Spektraler Lehrmeister",
		["Lady Illucia Barov"] = "Lady Illucia Barov",
		["Jandice Barov"] = "Jandice Barov",
		["Lord Alexei Barov"] = "Lord Alexei Barov",
		["Instructor Malicia"] = "Instrukteurin Malicia",
		["Kirtonos the Herald"] = "Kirtonos der Herold",
		["The Ravenian"] = "Der Ravenier",
		["Ras Frostwhisper"] = "Ras Frostraunen",
		["Jed Runewatcher"] = "Jed Runenblick ",
		["The Unforgiven"] = "Der Unverziehene",
		["Plagued Maggot"] = "Verseuchte Made",
		["Cliffwatcher Longhorn"] = "Klippenbehüter Langhorn",
		["Vaelastrasz"] = "Vaelastrasz",
		["Vol'jin"] = "Vol'jin",
		["Flametongue Totem IV"] = "Totem der Flammenzunge IV",
		["Hearthsinger Forresten"] = "Herdsinger Forresten",
		["Fetid Zombie"] = "Stinkender Zombie",
		["Gryfe"] = "Gryfe ",
		["Urok Doomhowl"] = "Urok Schreckensbote",
		["Mother Smolderweb"] = "Mutter Glimmernetz",
		["Urok Enforcer"] = "Uroks Vollstrecker",
		["Urok Ogre Magus"] = "Uroks Ogermagus",
		["Scarlet Medic"] = "Scharlachroter Sanitäter",
		["Scarlet Priest"] = "Scharlachroter Priester",
		["Rivern Frostwind"] = "Rivern Frostwind ",
		["Glacier"] = "Gletscher ",
		["Malyfous Darkhammer"] = "Malyfous Düsterhammer ",
		["Xavaric"] = "Xavaric",
		["Cobalt Whelp"] = "Kobaltblauer Welpe",
		["Cobalt Broodling"] = "Kobaltblauer Brutling",
		["Spell Eater"] = "Zauberfresser",
		["Spellmaw"] = "Zaubermaul",
		["Manaclaw"] = "Manaklaue",
		["Scryer"] = "Scryer",
		["Chromie"] = "Chromie",
		["Plagued Hatchling"] = "Verseuchtes Jungtier",
		["Summoned Blackhand Dreadweaver"] = "Beschworener Schreckenswirker der Schwar",
		["Summoned Blackhand Veteran"] = "Beschworener Veteran der Schwarzfaustleg",
		["Rookery Hatcher"] = "Krähenhorstbrutwächter",
		["Remorseful Highborne"] = "Reuevolle Hochgeborene",
		["Bile Slime"] = "Gallbrühschleimer",
		["Summoned Zombie"] = "Beschworener Zombie",
		["Carrion Scarab"] = "Aasskarabäus",
		["Temporal Parasite"] = "Temporaler Parasit",
		["Herald of Thrall"] = "Herold von Thrall",
		["Shy-Rotam"] = "Shy-Rotam",
		["High Chief Winterfall"] = "Oberhäuptling der Winterfelle",
		["Mulgris Deepriver"] = "Mulgris Tiefenfurt",
		["Awbee"] = "Awbee",
		["Sian-Rotam"] = "Sian-Rotam",
		["Blackhand Dragon Handler"] = "Großdrachenführer der Schwarzfaustlegion",
		["Blackhand Thug"] = "Rohling der Schwarzfaustlegion",
		["Finkle Einhorn"] = "Finkle Einhorn",
		["Warosh"] = "Warosh ",
		["Warosh the Redeemed"] = "Warosh der Erlöste",
		["Jabbering Ghoul"] = "Plappernder Ghul",
		["Hitah'ya the Keeper"] = "Hitah'ya die Bewahrerin",
		["Ursius"] = "Ursius",
		["Brumeran"] = "Brumeran",
		["Timmy the Cruel"] = "Timmy der Grausame",
		["Stonespine"] = "Steinbuckel",
		["Archivist Galford"] = "Archivar Galford",
		["Grand Crusader Dathrohan"] = "Oberster Kreuzzügler Dathrohan",
		["Balnazzar"] = "Balnazzar",
		["Chromatic Elite Guard"] = "Chromatische Elitewache",
		["Wandering Skeleton"] = "Wanderndes Skelett",
		["Duggan Wildhammer"] = "Duggan Wildhammer",
		["Hed'mush the Rotting"] = "Hed'mush der Faulende",
		["Warlord Thresh'jin"] = "Kriegsherr Thresh'jin",
		["Zul'Brin Warpbranch"] = "Zul'Brin Wirbelstab",
		["Ranger Lord Hawkspear"] = "Waldläuferlord Falkenspeer",
		["Gish the Unmoving"] = "Gish der Unbewegliche",
		["Lord Darkscythe"] = "Lord Finstersense",
		["Deathspeaker Selendre"] = "Todessprecher Selendre ",
		["High General Abbendis"] = "Hochgeneral Abbendis",
		["Farmer Dalson"] = "Bauer Dalson",
		["High Executor Derrington"] = "Hochexekutor Derrington",
		["Commander Ashlam Valorfist"] = "Kommandant Ashlam Ehrenschlag",
		["Argent Officer Garush"] = "Argentumoffizier Garush ",
		["Argent Officer Pureheart"] = "Argentumoffizier Reinherz ",
		["Argent Quartermaster Hasana"] = "Argentumrüstmeister Hasana ",
		["Argent Quartermaster Lightspark"] = "Argentumrüstmeister Funkellicht ",
		["Undead Scarab"] = "Untoter Skarabäus",
		["Sindrayl"] = "Sindrayl ",
		["Goraluk Anvilcrack"] = "Goraluk Hammerbruch ",
		["Lorekeeper Polkelt"] = "Hüter des Wissens Polkelt",
		["Winterfall Runner"] = "Läufer der Winterfelle",
		["Aurius"] = "Aurius",
		["Lorax"] = "Lorax",
		["Shatterspear Troll"] = "Troll der Splitterspeere",
		["Kelek Skykeeper"] = "Kelek Himmelshüter ",
		["Taronn Redfeather"] = "Taronn Rotfeder ",
		["Greta Mosshoof"] = "Greta Mooshuf ",
		["Tenell Leafrunner"] = "Tenell Laubschleicher ",
		["Ivy Leafrunner"] = "Ivy Laubschleicher ",
		["Rotting Worm"] = "Faulender Wurm",
		["Haleh"] = "Haleh",
		["Joseph Redpath"] = "Joseph Rotpfad",
		["Captain Redpath"] = "Hauptmann Rotpfad",
		["Redpath the Corrupted"] = "Rotpfad der Verderbte",
		["Marduk the Black"] = "Marduk der Schwarze",
		["Ghost of the Past"] = "Geist der Vergangenheit",
		["Nessy"] = "Nessy",
		["Decrepit Guardian"] = "Altersschwacher Wächter",
		["Davil Lightfire"] = "Davil Lichterschein",
		["Davil Crokford"] = "Davil Crokford",
		["Horgus the Ravager"] = "Horgus der Verheerer",
		["Darrowshire Betrayer"] = "Verräter von Darroheim",
		["Darrowshire Defender"] = "Verteidiger von Darroheim",
		["Silver Hand Disciple"] = "Silberhandjünger",
		["Redpath Militia"] = "Rotpfadmiliz",
		["Marauding Corpse"] = "Plündernder Kadaver",
		["Marauding Skeleton"] = "Plünderndes Skelett",
		["Servant of Horgus"] = "Diener von Horgus",
		["Bloodletter"] = "Blutvergießer",
		["Summoned Water Elemental"] = "Beschworener Wasserelementar",
		["Naga Siren"] = "Sirene der Naga",
		["Jeziba"] = "Jeziba",
		["Quixxil"] = "Quixxil",
		["Legacki"] = "Legacki",
		["Scarlet Hound"] = "Scharlachroter Hund",
		["Frostwolf"] = "Frostwolf",
		["Whitewhisker Vermin"] = "Gezücht der Weißstoppel",
		["Winterax Troll"] = "Winterax Troll",
		["Winterax Berserker"] = "Winterax-Berserker",
		["Snowblind Harpy"] = "Schneeblinde Harpyie",
		["Irondeep Trogg"] = "Trogg der Eisenschachtmine",
		["Kodo Spirit"] = "Kodogeist",
		["Alterac Ram"] = "Alteracwidder",
		["Wildpaw Gnoll"] = "Gnoll der Wildpfoten",
		["Fallen Hero"] = "Gefallener Held",
		["Cannon Master Willey"] = "Kanonenmeister Willey",
		["Captured Arko'narin"] = "Gefangener Arko'narin",
		["Roxxik"] = "Roxxik ",
		["Arko'narin"] = "Arko'narin",
		["Jessir Moonbow"] = "Jessir Mondsichel",
		["Remains of Trey Lightforge"] = "Überreste von Trey Lichtschmied",
		["Alexi Barov"] = "Alexi Barov ",
		["Weldon Barov"] = "Weldon Barov ",
		["Della"] = "Della ",
		["Illusory Wraith"] = "Illusionäres Gespenst",
		["Mindless Undead"] = "Hirnloser Untoter",
		["Malor the Zealous"] = "Malor der Eifrige",
		["Smokey LaRue"] = "Smokey LaRue",
		["Lord Maxwell Tyrosus"] = "Lord Maxwell Tyrosus ",
		["Betina Bigglezink"] = "Betina Moppelzink ",
		["Leonid Barthalomew the Revered"] = "Leonid Barthalomew der Geachtete ",
		["Caretaker Alen"] = "Verwalter Alen ",
		["Duke Nicholas Zverenhoff"] = "Fürst Nicholas Zverenhoff ",
		["Crimson Monk"] = "Purpurroter Mönch",
		["Timothy Worthington"] = "Timothy Wertinger ",
		["High Priestess MacDonnell"] = "Hohepriesterin MacDonnell",
		["Crimson Rifleman"] = "Scharlachroter Scharfschütze",
		["Shadow Priestess Vandis"] = "Schattenpriesterin Vandis",
		["Alchemist Arbington"] = "Alchimist Arbington",
		["Apothecary Dithers"] = "Apotheker Zitters",
		["Fras Siabi"] = "Fras Siabi",
		["Carlin Redpath"] = "Carlin Rotpfad ",
		["Darrowshire Spirit"] = "Geist von Darroheim",
		["Jenova Stoneshield"] = "Jenova Steinschild ",
		["Annora"] = "Annora ",
		["Hgarth"] = "Hgarth ",
		["Cauldron Lord Bilemaw"] = "Kessellord Gallenschlund",
		["Cauldron Lord Razarch"] = "Kessellord Razarch",
		["Cauldron Lord Malvinious"] = "Kessellord Malvinius",
		["Cauldron Lord Soulwrath"] = "Kessellord Rachseele",
		["Wynd Nightchaser"] = "Wynd Nachtjagd",
		["Stratholme Courier"] = "Kurier von Stratholme",
		["Drakk Stonehand"] = "Drakk Steinhand ",
		["Hahrana Ironhide"] = "Hahrana Eisenhaut ",
		["Argent Guard"] = "Argent Guard ",
		["Mana Tide Totem II"] = "Totem der Manaflut II",
		["Mana Tide Totem III"] = "Totem der Manaflut III",
		["Argent Rider"] = "Argentumreiter ",
		["Azzleby"] = "Azzelby ",
		["Crimson Hammersmith"] = "Purpurroter Hammerschmied",
		["Black Guard Swordsmith"] = "Schwertschmied der schwarzen Wache",
		["Maethrya"] = "Maethrya ",
		["Yugrek"] = "Yugrek ",
		["Egan"] = "Egan",
		["Spirit of Trey Lightforge"] = "Geist von Trey Lichtschmied",
		["Undead Postman"] = "Untoter Postbote",
		["Postmaster Malown"] = "Postmeister Malown",
		["Myolor Sunderfury"] = "Myolor Zornesbeben",
		["Ironus Coldsteel"] = "Ironus Froststahl ",
		["Krathok Moltenfist"] = "Krathok Glutfaust",
		["Okothos Ironrager"] = "Okothos Eisenwüter ",
		["Borgosh Corebender"] = "Borgosh Glutformer ",
		["Bloodvenom Post Brave"] = "Kriegerheld des Blutgiftpostens",
		["Shi'alune"] = "Shi'alune ",
		["Nixxrak"] = "Nixxrak ",
		["Blixxrak"] = "Blixxrak ",
		["Wixxrak"] = "Wixxrak ",
		["Xizzer Fizzbolt"] = "Xizzer Zischbolz ",
		["Lunnix Sprocketslip"] = "Lunnix Schmierspross ",
		["Himmik"] = "Himmik ",
		["Evie Whirlbrew"] = "Evie Wirbelbräu ",
		["Qia"] = "Qia ",
		["Everlook Bruiser"] = "Haudrauf der Ewigen Warte",
		["Lilith the Lithe"] = "Lilith die Liebliche",
		["Kilram"] = "Kilram",
		["Seril Scourgebane"] = "Seril Geißelbann",
		["Argent Defender"] = "Argentumverteidiger ",
		["Shatterspear Drummer"] = "Trommler der Splitterspeere",
		["Mindless Skeleton"] = "Hirnloses Skelett",
		["Crimson Cannon"] = "Purpurrote Kanone",
		["Summoned Skeleton"] = "Beschworenes Skelett",
		["Eva Sarkhoff"] = "Eva Sarkhoff",
		["Lucien Sarkhoff"] = "Lucien Sarkhoff",
		["Manifestation of Water"] = "Wassermanifestation",
		["Scholomance Handler"] = "Tierführer aus Scholomance",
		["Doctor Theolen Krastinov"] = "Doktor Theolen Krastinov ",
		["Onyxian Whelp"] = "Welpe von Onyxia",
		["Spectral Projection"] = "Spektrale Projektion",
		["Caer Darrow Citizen"] = "Bürger von Darrowehr",
		["Magnus Frostwake"] = "Magnus Frostrufer",
		["Caer Darrow Guardsman"] = "Gardist von Darrowehr",
		["Caer Darrow Cannoneer"] = "Kanonier von Darrowehr",
		["Caer Darrow Horseman"] = "Reiter von Darrowehr",
		["Dark Shade"] = "Dunkler Schatten",
		["Magistrate Marduke"] = "Magistrat Marduk",
		["Spectral Betrayer"] = "Spektraler Verräter",
		["Spectral Defender"] = "Spektraler Verteidiger",
		["Mossflayer Zombie"] = "Zombie der Moosschinder",
		["Unliving Mossflayer"] = "Untoter der Moosschinder",
		["Hakkari Shadowcaster"] = "Schattenzauberer der Hakkari",
		["Hakkari Shadow Hunter"] = "Schattenjäger der Hakkari",
		["Hakkari Blood Priest"] = "Blutpriester der Hakkari",
		["Hakkari Oracle"] = "Orakel der Hakkari",
		["Zealot Lor'Khan"] = "Zelot Lor'Khan",
		["Zealot Zath"] = "Zelot Zath",
		["Gurubashi Axe Thrower"] = "Axtwerfer der Gurubashi",
		["Gurubashi Headhunter"] = "Kopfjäger der Gurubashi",
		["Gurubashi Berserker"] = "Berserker der Gurubashi",
		["Gurubashi Blood Drinker"] = "Bluttrinker der Gurubashi",
		["Gurubashi Warrior"] = "Krieger der Gurubashi",
		["Gurubashi Champion"] = "Champion der Gurubashi",
		["Son of Hakkar"] = "Sohn von Hakkar",
		["Soulflayer"] = "Seelenschinder",
		["Zulian Cub"] = "Zulianisches Junges",
		["Zulian Tiger"] = "Zulianischer Tiger",
		["Zulian Panther"] = "Zulianischer Panther",
		["Bloodseeker Bat"] = "Blutsucherfledermaus",
		["Razzashi Broodwidow"] = "Razzashibrutwitwe",
		["Razzashi Serpent"] = "Razzashischlange",
		["Razzashi Adder"] = "Razzashiotter",
		["Razzashi Cobra"] = "Razzashikobra",
		["Hooktooth Frenzy"] = "Fangzahnfrenzy",
		["Jin'do the Hexxer"] = "Jin'do der Verhexer",
		["Bloodlord Mandokir"] = "Blutfürst Mandokir",
		["High Priestess Hai'watna"] = "Hohepriesterin Hai'watna",
		["Sandfury Speaker"] = "Sprecher der Sandwüter ",
		["Witherbark Speaker"] = "Sprecher der Bleichborken ",
		["Bloodscalp Speaker"] = "Sprecher der Blutskalpe ",
		["Skullsplitter Speaker"] = "Sprecher der Schädelspalter ",
		["Vilebranch Speaker"] = "Sprecher der Blutfratzen ",
		["Priestess Alathea"] = "Priesterin Alathea ",
		["High Priest Rohan"] = "Hohepriester Rohan ",
		["Illusion of Jandice Barov"] = "Illusion von Jandice Barov",
		["Gordok Enforcer"] = "Vollstrecker der Gordok",
		["Gordok Brute"] = "SchlÃ¤ger der Gordok",
		["Gordok Mauler"] = "Raufer der Gordok",
		["Gordok Ogre-Mage"] = "Ogermagier der Gordok",
		["Gordok Mage-Lord"] = "Magierlord der Gordok",
		["Gordok Captain"] = "Hauptmann der Gordok",
		["Gordok Spirit"] = "Geist der Gordok",
		["Mushgog"] = "Mushgog",
		["Gordok Warlock"] = "Hexenmeister der Gordok",
		["Gordok Reaver"] = "Häscher der Gordok",
		["Wildspawn Satyr"] = "Satyr der Wildhufe",
		["Wildspawn Rogue"] = "Schurke der Wildhufe",
		["Wildspawn Trickster"] = "Schwindler der Wildhufe",
		["Wildspawn Betrayer"] = "Verräter der Wildhufe",
		["Wildspawn Felsworn"] = "Teufelsanbeter der Wildhufe",
		["Wildspawn Shadowstalker"] = "Schattenpirscher der Wildhufe",
		["Wildspawn Hellcaller"] = "Höllenrufer der Wildhufe",
		["Petrified Treant"] = "Versteinerter Treant",
		["Ironbark Protector"] = "Beschützer von Eisenborke",
		["Alzzin's Minion"] = "Alzzins Diener",
		["Warpwood Guardian"] = "Wucherborkenwächter",
		["Warpwood Treant"] = "Wucherborkentreant",
		["Warpwood Tangler"] = "Wucherborkengreifer",
		["Warpwood Stomper"] = "Wucherborkenstampfer",
		["Highborne Summoner"] = "Hochgeborenenbeschwörer",
		["Tsu'zee"] = "Tsu'zee",
		["Eldreth Seether"] = "Schnauber der Eldreth",
		["Eldreth Sorcerer"] = "Zauberhexer der Eldreth",
		["Eldreth Apparition"] = "Erscheinung der Eldreth",
		["Eldreth Spirit"] = "Geist der Eldreth",
		["Eldreth Spectre"] = "Schreckgespenst der Eldreth",
		["Eldreth Phantasm"] = "Trugbild der Eldreth",
		["Skeletal Highborne"] = "Skeletthochgeborener",
		["Rotting Highborne"] = "Faulender Hochgeborener",
		["Arcane Aberration"] = "Arkane Entartung",
		["Mana Remnant"] = "Manarest",
		["Residual Monstrosity"] = "Verbleibender Monsterrest",
		["Prince Tortheldrin"] = "Prinz Tortheldrin",
		["Magister Kalendris"] = "Magister Kalendris",
		["Illyanna Ravenoak"] = "Illyanna Rabeneiche",
		["Tendris Warpwood"] = "Tendris Wucherborke",
		["Zevrim Thornhoof"] = "Zevrim Dornhuf",
		["Old Ironbark"] = "Eisenborke der GroÃ?e",
		["Alzzin the Wildshaper"] = "Alzzin der Wildformer",
		["Immol'thar"] = "Immol'thar",
		["The Razza"] = "Der Razza",
		["Skarr the Unbreakable"] = "Skarr der Unbezwingbare",
		["King Gordok"] = "König Gordok",
		["Ragnaros"] = "Ragnaros",
		["Timbermaw Warder"] = "Wärter der Holzschlundfeste",
		["Quartermaster Miranda Breechlock"] = "Rüstmeisterin Miranda Knackschloss ",
		["Jack Sterling"] = "Jack Silberling",
		["Necrofiend"] = "Nekrobestie",
		["Timbermaw Mystic"] = "Mystiker der Holzschlundfeste",
		["Timbermaw Woodbender"] = "Holzbieger der Holzschlundfeste",
		["Grazle"] = "Grazle",
		["Gorn One Eye"] = "Gorn Einauge",
		["Salfa"] = "Salfa",
		["Meilosh"] = "Meilosh",
		["Kernda"] = "Kernda",
		["Scholomance Dark Summoner"] = "Dunkler Beschwörer aus Scholomance",
		["Nefarian"] = "Nefarian ",
		["Risen Guardian"] = "Auferstandener Wächter",
		["Irondeep Shaman"] = "Schamane der Eisenschachtmine",
		["Irondeep Skullthumper"] = "Schädelhauer der Eisenschachtmine",
		["Whitewhisker Digger"] = "Buddler der Weißstoppel",
		["Whitewhisker Geomancer"] = "Geomant der Weißstoppel",
		["Whitewhisker Overseer"] = "Vorarbeiter der Weißstoppel",
		["Bardu Sharpeye"] = "Bardu Scharfblick",
		["Alexia Ironknife"] = "Alexia Eisendolch",
		["Kirsta Deepshadow"] = "Kirsta Grubenschatten",
		["Cavalier Durgen"] = "Chevalier Durgen",
		["Huntsman Radley"] = "Jägerin Radley",
		["Bloodshot"] = "Blutläufer",
		["Mickey Levine"] = "Mickey Levine",
		["Nathaniel Dumah"] = "Nathaniel Dumah",
		["Spectral Marauder"] = "Spektraler Marodeur",
		["Spectral Corpse"] = "Spektraler Kadaver",
		["Rattlegore"] = "Blutrippe",
		["Scourge Summoning Crystal"] = "Scourge Summoning Crystal",
		["Taiga Wisemane"] = "Taiga Klugmähne",
		["Jessica Redpath"] = "Jessica Rotpfad",
		["Servant of Weldon Barov"] = "Diener von Weldon Barov",
		["Servant of Alexi Barov"] = "Diener von Alexi Barov",
		["Morloch"] = "Morloch",
		["Molten Giant"] = "Geschmolzener Riese",
		["Molten Destroyer"] = "Geschmolzener Zerstörer",
		["Flamewaker"] = "Feuerschuppe",
		["Flamewaker Priest"] = "Feuerschuppenpriester",
		["Flamewaker Healer"] = "Feuerschuppenheiler",
		["Flamewaker Elite"] = "Feuerschuppenelite",
		["Lava Annihilator"] = "Lavavernichter",
		["Firewalker"] = "Feuergänger",
		["Flameguard"] = "Flammenwächter",
		["Firelord"] = "Feuerlord",
		["Flame Imp"] = "Flammenwichtel",
		["Core Hound"] = "Kernhund",
		["Core Rager"] = "Kernwüterich",
		["Ancient Core Hound"] = "Uralter Kernhund",
		["Snowblind Windcaller"] = "Schneeblinde Windruferin",
		["Taskmaster Snivvle"] = "Zuchtmeister Schnuffel",
		["Snowblind Ambusher"] = "Schneeblinde Wegelagerin",
		["Winterax Witch Doctor"] = "Winterax-Hexendoktor",
		["Chal Fairwind"] = "Chal Sanftwind ",
		["Hive'Ashi Stinger"] = "Stecher des Ashischwarms",
		["Varian Wrynn"] = "Varian Wrynn ",
		["Sarin Starlight"] = "Sarin Sternenfeuer",
		["Mor'vek"] = "Mor'vek ",
		["Arin'sor"] = "Arin'sor ",
		["Graw Cornerstone"] = "Graw Eckstein ",
		["Kriss Goldenlight"] = "Kriss Goldenlicht",
		["Rayan Dawnrisen"] = "Rayan Morgenschein",
		["Adon"] = "Adon",
		["Joy Ar'nareth"] = "Joy Ar'nareth",
		["Coral Moongale"] = "Coral Mondsturm",
		["Mirador"] = "Mirador",
		["Celes Earthborne"] = "Celes Irdenschoß",
		["Sar Browneye"] = "Sar Nussauge",
		["Hive'Ashi Worker"] = "Arbeiter des Ashischwarms",
		["Hive'Ashi Defender"] = "Verteidiger des Ashischwarms",
		["Hive'Ashi Sandstalker"] = "Sandpirscher des Ashischwarms",
		["Hive'Ashi Swarmer"] = "Schwärmer des Ashischwarms",
		["Hive'Zora Waywatcher"] = "Wegbehüter des Zoraschwarms",
		["Hive'Zora Tunneler"] = "Tunnelgräber des Zoraschwarms",
		["Hive'Zora Wasp"] = "Wespe des Zoraschwarms",
		["Hive'Zora Reaver"] = "Häscher des Zoraschwarms",
		["Hive'Zora Hive Sister"] = "Schwester des Zoraschwarms",
		["Hive'Regal Ambusher"] = "Wegelagerer des Regalschwarms",
		["Hive'Regal Burrower"] = "Einbuddler des Regalschwarms",
		["Hive'Regal Spitfire"] = "Feuerspucker des Regalschwarms",
		["Hive'Regal Slavemaker"] = "Sklavenjäger des Regalschwarms",
		["Hive'Regal Hive Lord"] = "Schwarmlord des Regalschwarms",
		["Stonelash Scorpid"] = "Steinpanzerskorpid",
		["Stonelash Pincer"] = "Steinpanzerzange",
		["Stonelash Flayer"] = "Steinpanzerschinder",
		["Sand Skitterer"] = "Sandhuscher",
		["Rock Stalker"] = "Felspirscher",
		["Dredge Striker"] = "Baggerschläger",
		["Dredge Crusher"] = "Baggerbrecher",
		["Dust Stormer"] = "Staubstürmer",
		["Cyclone Warrior"] = "Wirbelsturmkrieger",
		["Desert Rumbler"] = "Wüstenrumpler",
		["Desert Rager"] = "Wüstenwüterich",
		["Feran Strongwind"] = "Feran Starkwind",
		["Blaise Montgomery"] = "Blaise Montgomery",
		["Gogo"] = "Gogo",
		["Meggi Peppinrocker"] = "Meggi Frickelprotz",
		["Harlo Wigglesworth"] = "Harlo Wackelwert",
		["Andi Lynn"] = "Andi Lynn",
		["Theradrim Shardling"] = "Theradrimsteinling",
		["Theradrim Guardian"] = "Theradrimwächter",
		["Deep Borer"] = "Tiefbohrer",
		["Mylentha Riverbend"] = "Mylentha Flusslauf",
		["Bessany Plainswind"] = "Bessany Steppenwind",
		["Moren Riverbend"] = "Moren Flusslauf",
		["Bunthen Plainswind"] = "Bunthen Steppenwind ",
		["Tajarri"] = "Tajarri",
		["Silva Fil'naveth"] = "Silva Fil'naveth ",
		["Rabine Saturna"] = "Rabine Saturna",
		["Dendrite Starblaze"] = "Dendrite Sternenschauer",
		["Twilight Keeper Exeter"] = "Bewahrer des Schattenhammers Exeter ",
		["Twilight Keeper Havunth"] = "Bewahrer des Schattenhammers Havunth ",
		["Jarund Stoutstrider"] = "Jarund Firmschreiter ",
		["Grum Redbeard"] = "Grum Rotbart",
		["Voriya"] = "Voriya",
		["Moonglade Warden"] = "Aufseher der Mondlichtung",
		["Paige Felixe"] = "Paige Felixe",
		["Hakkari Priest"] = "Priester der Hakkari",
		["Hakkari Witch Doctor"] = "Hexendoktor der Hakkari",
		["Keeper Remulos"] = "Bewahrer Remulos",
		["Theodore Griffs"] = "Theodore Griffs",
		["Wildpaw Shaman"] = "Schamane der Wildpfoten",
		["Wildpaw Mystic"] = "Mystiker der Wildpfoten",
		["Wildpaw Brute"] = "Schläger der Wildpfoten",
		["Wildpaw Alpha"] = "Alpha der Wildpfoten",
		["Doomguard"] = "Verdammniswache",
		["Azore Aldamort"] = "Azore Aldamort ",
		["Buliwyf Stonehand"] = "Buliwyf Steinhand ",
		["Ilyenia Moonfire"] = "Ilyenia Mondfeuer ",
		["Woo Ping"] = "Woo Ping ",
		["Sayoc"] = "Sayoc ",
		["Ansekhwa"] = "Ansekhwa ",
		["Archibald"] = "Archibald ",
		["Grinning Dog"] = "Grinsender Hund",
		["Myranda the Hag"] = "Myranda die Vettel",
		["Spectral Attendant"] = "Spektraler Knecht",
		["Masat T'andr"] = "Masat T'andr ",
		["Nathanos Blightcaller"] = "Nathanos Pestrufer ",
		["Twilight Avenger"] = "Rächer des Schattenhammers ",
		["Twilight Geolord"] = "Geolord des Schattenhammers ",
		["Twilight Stonecaller"] = "Steinrufer des Schattenhammers ",
		["Twilight Master"] = "Meister des Schattenhammers ",
		["Blighthound"] = "Pesthund",
		["Mercutio Filthgorger"] = "Mercutio Dreckschipper",
		["Crypt Robber"] = "Grufträuber",
		["Borelgore"] = "Blutschlinger",
		["Duskwing"] = "Dämmerschwinge",
		["Crusader Lord Valdelmar"] = "Kreuzzüglerlord Valdelmar",
		["Shardi"] = "Shardi ",
		["Brakkar"] = "Brakkar ",
		["Andruk"] = "Andruk ",
		["Umber"] = "Umber",
		["Drek'Thar"] = "Drek'Thar ",
		["Captain Galvangar"] = "Hauptmann Galvangar ",
		["Vanndar Stormpike"] = "Vanndar Sturmlanze ",
		["Captain Balinda Stonehearth"] = "Hauptmann Balinda Steinbruch ",
		["Great Bear Spirit"] = "Großer Bärengeist",
		["Great Cat Spirit"] = "Großer Katzengeist",
		["Flamegor"] = "Flammenmaul",
		["Magmadar"] = "Magmadar",
		["Firemaw"] = "Feuerschwinge",
		["Golemagg the Incinerator"] = "Golemagg der Verbrenner",
		["Stormpike Herald"] = "Herold der Sturmlanzen",
		["Frostwolf Herald"] = "Herold der Frostwölfe",
		["Broodlord Lashlayer"] = "Brutwächter Dreschbringer",
		["Majordomo Executus"] = "Majordomus Exekutus",
		["Dargon"] = "Dargon ",
		["Daeolyn Summerleaf"] = "Daeolyn Sommerblatt ",
		["Lorelae Wintersong"] = "Lorelae Winterklang ",
		["Kharedon"] = "Kharedon ",
		["Meliri"] = "Meliri ",
		["Malvor"] = "Malvor ",
		["My'lanna"] = "My'lanna ",
		["Narianna"] = "Narianna ",
		["Koiter"] = "Koiter",
		["Loganaar"] = "Loganaar ",
		["Gor'marok the Ravager"] = "Gor'marok der Verheerer",
		["Stormpike Mountaineer"] = "Gebirgsjäger der Sturmlanzen",
		["Alliance Sentinel"] = "Alliance Sentinel",
		["Stormpike Defender"] = "Verteidiger der Sturmlanzen",
		["Frostwolf Legionnaire"] = "Legionär der Frostwölfe",
		["Frostwolf Warrior"] = "Frostwolf Warrior",
		["Frostwolf Guardian"] = "Gardist der Frostwölfe",
		["Baron Geddon"] = "Baron Geddon",
		["Garr"] = "Garr",
		["Lava Elemental"] = "Lavaelementar",
		["Stormpike Quartermaster"] = "Rüstmeister der Sturmlanzen",
		["Frostwolf Quartermaster"] = "Rüstmeister der Frostwölfe",
		["Sulfuron Harbinger"] = "Sulfuronherold",
		["Firesworn"] = "Feueranbeter",
		["Lava Reaver"] = "Lavahäscher",
		["Lava Surger"] = "Lavawoger",
		["Lucifron"] = "Lucifron",
		["Flamewaker Protector"] = "Beschützer der Feuerschuppe",
		["Drakan"] = "Drakan",
		["Duros"] = "Duros",
		["Great Shark"] = "Großer Hai",
		["Mammoth Shark"] = "Mammuthai",
		["Lord Tirion Fordring"] = "Lord Tirion Fordring ",
		["Stormpike Guardsman"] = "Gardist der Sturmlanzen",
		["Crimson Elite"] = "Purpurrote Elite",
		["Onyxian Warder"] = "Onyxias Wärter",
		["Guardian of Elune"] = "Wächter von Elune",
		["Ice Totem"] = "Eistotem",
		["Son of Flame"] = "Sohn der Flamme",
		["Winterax Axe Thrower"] = "Winterax-Axtwerfer",
		["Winterax Shadow Hunter"] = "Winterax-Schattenjäger",
		["Winterax Hunter"] = "Winterax-Jäger",
		["Korrak the Bloodrager"] = "Korrak der Blutrünstige",
		["Shadowglen Sentinel"] = "Schildwache des Laubschattentals",
		["Tortured Druid"] = "Gequälter Druide",
		["Tortured Sentinel"] = "Gequälte Schildwache",
		["Glordrum Steelbeard"] = "Glordrum Stahlbart ",
		["Martin Lindsey"] = "Martin Lindsey ",
		["Shade of Ambermoon"] = "Schemen von Mondglanz",
		["Princess Theradras"] = "Prinzessin Theradras",
		["Landslide"] = "Erdrutsch",
		["Spitelash Raider"] = "Räuber der Grollflossen",
		["Spitelash Witch"] = "Hexe der Grollflossen",
		["Primordial Behemoth"] = "Urzeitliches Ungetüm",
		["Thessala Hydra"] = "Thessalahydra",
		["Conquered Soul of the Blightcaller"] = "Eroberte Seele des Pestrufers",
		["Noxious Slime"] = "Giftiger Brühschleimer",
		["Cavern Shambler"] = "Höhlenschlurfer",
		["Celebras the Cursed"] = "Celebras der Verfluchte",
		["Lord Vyletongue"] = "Lord Schlangenzunge",
		["Meshlok the Harvester"] = "Meshlok der Ernter",
		["Zaetar's Spirit"] = "Zaetars Geist",
		["Spirit of Maraudos"] = "Geist von Maraudos ",
		["Spirit of Veng"] = "Geist von Veng ",
		["Infiltrator Hameya"] = "Spitzel Hameya",
		["Zaeldarr the Outcast"] = "Zaeldarr der Ausgestoßene",
		["Mechanical Yeti"] = "Mechanischer Yeti",
		["Razorlash"] = "Schlingwurzler",
		["Gehennas"] = "Gehennas",
		["Infected Mossflayer"] = "Infizierter Moosschinder",
		["Ziggurat Protector"] = "Zigguratbeschützer",
		["Slaughterhouse Protector"] = "Schlachthausbeschützer",
		["Shazzrah"] = "Shazzrah",
		["Lava Spawn"] = "Lavabrut",
		["Quel'Lithien Protector"] = "Beschützer von Quel'Lithien",
		["Crimson Courier"] = "Purpurroter Kurier",
		["Shadowprey Guardian"] = "Wächter von Schattenflucht",
		["Demetria"] = "Demetria ",
		["Scarlet Trooper"] = "Scharlachroter Reitersoldat",
		["Wailing Spectre"] = "Klagendes Schreckgespenst",
		["Damned Soul"] = "Verdammte Seele",
		["Unliving Caretaker"] = "Untoter Verwalter",
		["Unliving Resident"] = "Untoter Einwohner",
		["Ley Sprite"] = "Leygrimmling",
		["Mana Sprite"] = "Managrimmling",
		["Augustus the Touched"] = "Augustus der Meschuggene",
		["Large Vile Slime"] = "Großer böser Brühschleimer",
		["Doomguard Commander"] = "Kommandant der Verdammniswache",
		["Lord Kazzak"] = "Lord Kazzak",
		["Blackwing Legionnaire"] = "Legionär der Pechschwingen",
		["Gordok Hyena"] = "Hyäne der Gordok",
		["Blackwing Mage"] = "Magier der Pechschwingen",
		["Death Talon Dragonspawn"] = "Drachenbrut der Todeskrallen",
		["Flint Shadowmore"] = "Flint Schattenmoor ",
		["Razorgore the Untamed"] = "Feuerkralle der Ungezähmte",
		["Blackwing Spellbinder"] = "Zauberbinder der Pechschwingen",
		["Blackwing Taskmaster"] = "Zuchtmeister der Pechschwingen",
		["Blackwing Warlock"] = "Hexenmeister der Pechschwingen",
		["Death Talon Wyrmguard"] = "Wyrmwache der Todeskrallen",
		["Death Talon Overseer"] = "Aufseher der Todeskrallen",
		["Death Talon Flamescale"] = "Flammenschuppe der Todeskrallen",
		["Death Talon Seether"] = "Schnauber der Todeskrallen",
		["Death Talon Wyrmkin"] = "Wyrmkin der Todeskrallen",
		["Death Talon Captain"] = "Hauptmann der Todeskrallen",
		["Death Talon Hatcher"] = "Brutwächter der Todeskrallen",
		["Arcanite Dragonling"] = "Arkanitdrachling",
		["Emeraldon Boughguard"] = "Smaragdonastwache",
		["Emeraldon Tree Warder"] = "Smaragdonbaumwärter",
		["Emeraldon Oracle"] = "Smaragdonorakel",
		["Verdantine Boughguard"] = "Tiefgrüne Astwache",
		["Verdantine Oracle"] = "Tiefgrünes Orakel",
		["Verdantine Tree Warder"] = "Tiefgrüner Baumwärter",
		["Melris Malagan"] = "Melris Malagan ",
		["Justine Demalier"] = "Justine Demalier ",
		["Dreamtracker"] = "Traumsucher",
		["Dreamroarer"] = "Traumbrüller",
		["Dreamstalker"] = "Traumpirscher",
		["Grethok the Controller"] = "Grethok der Aufseher",
		["Jarrodenus"] = "Jarrodenus ",
		["Mishellena"] = "Mishellena ",
		["Reginald Windsor"] = "Reginald Windsor",
		["Bibilfaz Featherwhistle"] = "Bibilfaz Federpfiff ",
		["Vhulgra"] = "Vhulgra ",
		["Khaelyn Steelwing"] = "Khaelyn Stahlschwing ",
		["Georgia"] = "Georgia ",
		["Onyxia's Elite Guard"] = "Elitewache von Onyxia",
		["Faustron"] = "Faustron ",
		["Lady Onyxia"] = "Lady Onyxia",
		["Captain Dirgehammer"] = "Hauptmann Klagehammer ",
		["Lieutenant Rachel Vaccar"] = "Leutnant Rachel Vaccar",
		["Archmage Gaiman"] = "Erzmagier Gaiman",
		["Sergeant Major Skyshadow"] = "Stabsfeldwebel Himmelsschatten",
		["Master Sergeant Biggins"] = "Hauptfeldwebel Biggins ",
		["Captain O'Neal"] = "Hauptmann O'Neal ",
		["Lieutenant Karter"] = "Leutnant Karter ",
		["Lieutenant Jackspring"] = "Leutnant Federwinde ",
		["Sergeant Major Clate"] = "Stabsfeldwebel Clate ",
		["Guard Quine"] = "Wache Quinn",
		["Guard Hammon"] = "Wache Hammond",
		["Legionnaire Teena"] = "Legionär Teena",
		["Blood Guard Hini'wana"] = "Blutwache Hini'wana",
		["Advisor Willington"] = "Berater Willington",
		["Chieftain Earthbind"] = "Häuptling Erdenbund",
		["Lady Palanseer"] = "Lady Palasseher ",
		["Brave Stonehide"] = "Kriegerheld Steinfell ",
		["Stone Guard Zarg"] = "Steinwache Zarg ",
		["First Sergeant Hola'mahi"] = "Stabsfeldwebel Hola'mahi ",
		["Raider Bork"] = "Räuber Bork ",
		["Grunt Korf"] = "Grunzer Korf",
		["Grunt Bek'rah"] = "Grunzer Bek'rah",
		["Sergeant Ba'sha"] = "Sergeant Ba'sha ",
		["Chimaerok"] = "Chimaerok",
		["Arcane Chimaerok"] = "Arkaner Chimaerok",
		["Chimaerok Devourer"] = "Chimaerokverschlinger",
		["Lord Lakmaeran"] = "Lord Lakmaeran",
		["Officer Areyn"] = "Offizier Areyn ",
		["Greshka"] = "Greshka ",
		["Axtroz"] = "Axtroz",
		["Somnus"] = "Somnus",
		["Spirit Of Redemption"] = "Geist der Erlösung",
		["Doctor Gregory Victor"] = "Doktor Gregory Victor ",
		["Imp Minion"] = "Wichteldiener",
		["Critically Injured Soldier"] = "Schwerst verletzter Soldat",
		["Critically Injured Alliance Soldier"] = "Schwerst verletzter Soldat der Allianz",
		["Doctor Gustaf VanHowzen"] = "Doktor Gustaf Van Howzen ",
		["Jase Farlane"] = "Jase Langweg ",
		["Leonard Porter"] = "Leonard Porter ",
		["Werg Thickblade"] = "Werg Breitklinge ",
		["Lokhtos Darkbargainer"] = "Lokhtos Düsterfeilsch ",
		["Zannok Hidepiercer"] = "Zannok Ledernaht ",
		["Blimo Gadgetspring"] = "Blimo Federdings ",
		["Gigget Zipcoil"] = "Gigget Reißspule ",
		["Nergal"] = "Nergal ",
		["Vaelastrasz the Corrupt"] = "Vaelastrasz der Verdorbene",
		["Warpwood Crusher"] = "Wucherborkenzermalmer",
		["Whip Lasher"] = "Dornenpeitscher",
		["Gordok Mastiff"] = "Mastiff der Gordok",
		["Umi Thorson"] = "Umi Thorson",
		["Keetar"] = "Keetar",
		["Irondeep Guard"] = "Wache der Eisenschachtmine",
		["Irondeep Raider"] = "Räuber der Eisenschachtmine",
		["Bixi Wobblebonk"] = "Bixi Wobbelbonk ",
		["Myrokos Silentform"] = "Myrokos Leisetritt",
		["Aggi Rumblestomp"] = "Aggi Polterbein",
		["Coldmine Invader"] = "Eindringling der Eisbeißermine",
		["Masha Swiftcut"] = "Masha Schnellstreich",
		["Coldmine Guard"] = "Wache der Eisbeißermine",
		["Coldmine Explorer"] = "Forscher der Eisbeißermine",
		["Coldmine Surveyor"] = "Feldmesser der Eisbeißermine",
		["Irondeep Surveyor"] = "Feldmesser der Eisenschachtmine",
		["Irondeep Explorer"] = "Forscher der Eisenschachtmine",
		["Alliance Spirit Guide"] = "Geisterführer der Allianz",
		["Horde Spirit Guide"] = "Geisterführer der Horde",
		["Crimson Bodyguard"] = "Purpurrote Leibwache",
		["Hive'Ashi Drone"] = "Drohne des Ashischwarms",
		["Lieutenant Rugba"] = "Leutnant Rugba",
		["Lieutenant Spencer"] = "Leutnant Spencer",
		["Commander Randolph"] = "Kommandant Randolph",
		["Commander Dardosh"] = "Kommandant Dardosh",
		["Lieutenant Stronghoof"] = "Leutnant Starkhuf",
		["Lieutenant Vol'talar"] = "Leutnant Vol'talar",
		["Lieutenant Grummus"] = "Leutnant Grummus",
		["Lieutenant Murp"] = "Leutnant Murp",
		["Lieutenant Lewis"] = "Leutnant Lewis",
		["Syndicate Brigand"] = "Syndikats Brigant",
		["Syndicate Agent"] = "Syndikats Agent",
		["Syndicate Master Ryson"] = "Syndikats Meister Ryson",
		["Commander Malgor"] = "Kommandant Malgor",
		["Commander Mulfort"] = "Kommandant Mulfort",
		["Commander Louis Philips"] = "Kommandant Louis Philips",
		["Deathstalker Agent"] = "Todespirscher Agent",
		["Carrion Swarmer"] = "Aasschwärmer",
		["Aerie Gryphon"] = "Nistgipfelgreif",
		["Smith Regzar"] = "Schmied Regzar",
		["Vahgruk"] = "Vahgruk ",
		["War Rider"] = "Kriegsreiter",
		["Wing Commander Guse"] = "Schwadronskommandant Guse",
		["Wing Commander Jeztor"] = "Schwadronskommandant Jeztor",
		["Wing Commander Mulverick"] = "Schwadronskommandant Mulverick",
		["Phase Lasher"] = "Phasenpeitscher",
		["Fel Lash"] = "Teufelspeitscher",
		["Gaelden Hammersmith"] = "Gaelden Hammerschmied ",
		["Thanthaldis Snowgleam"] = "Thanthaldis Schneeglanz ",
		["Grunnda Wolfheart"] = "Grunnda Wolfsherz ",
		["Jekyll Flandring"] = "Jekyll Flandring ",
		["Layo Starstrike"] = "Layo Sternenschlag ",
		["Ryson's Eye in the Sky"] = "Rysons Auge im Himmel",
		["Primalist Thurloga"] = "Primalist Thurloga",
		["Lokholar the Ice Lord"] = "Lokholar der Eislord",
		["Murgot Deepforge"] = "Murgot Tiefenschmied",
		["Wildspawn Imp"] = "Wichtel der Wildhufe",
		["Duke Hydraxis"] = "Fürst Hydraxis",
		["Discordant Surge"] = "Disharmonische Woge",
		["Hydrospawn"] = "Hydrobrut",
		["Noxxion"] = "Noxxion",
		["Lord Tony Romano"] = "Lord Tony Romano ",
		["Frostwolf Shaman"] = "Schamane der Frostwölfe",
		["Death Lash"] = "Todespeitsche",
		["Lieutenant Largent"] = "Leutnant Largent",
		["Lieutenant Stouthandle"] = "Leutnant Starkhand",
		["Lieutenant Greywand"] = "Leutnant Graustab",
		["Lieutenant Lonadin"] = "Leutnant Lonadin",
		["Lieutenant Mancuso"] = "Leutnant Mancuso",
		["Hive'Ashi Ambusher"] = "Wegelagerer des Ashischwarms",
		["Coldmine Peon"] = "Peon der Eisbeißermine",
		["Coldmine Miner"] = "Minenarbeiter der Eisbeißermine",
		["Commander Mortimer"] = "Kommandant Mortimer",
		["Commander Duffy"] = "Kommandant Duffy",
		["Commander Karl Philips"] = "Kommandant Karl Philips",
		["Hydraxian Honor Guard"] = "Hydraxianische Ehrenwache",
		["Subterranean Diemetradon"] = "Unterirdisches Dimetrodon",
		["Seasoned Guardsman"] = "Erfahrener Gardist",
		["Seasoned Mountaineer"] = "Seasoned Mountaineer",
		["Seasoned Defender"] = "Erfahrener Verteidiger",
		["Seasoned Sentinel"] = "Seasoned Sentinel",
		["Seasoned Guardian"] = "Erfahrener Wächter",
		["Seasoned Legionnaire"] = "Erfahrener Legionär",
		["Seasoned Warrior"] = "Erfahrener Krieger",
		["Veteran Defender"] = "Verteidigerveteran",
		["Veteran Guardian"] = "Wächterveteran",
		["Veteran Guardsman"] = "Gardistenveteran",
		["Veteran Legionnaire"] = "Legionärveteran",
		["Veteran Mountaineer"] = "Veteran Mountaineer",
		["Veteran Sentinel"] = "Veteran Sentinel",
		["Veteran Warrior"] = "Kriegerveteran",
		["Stormpike Mine Layer"] = "Meuchler der Sturmlanzenklanmine",
		["Frostwolf Mine Layer"] = "Meuchler der Frostwolfmine",
		["Stormpike Bowman"] = "Bogenschütze der Sturmlanzen",
		["Frostwolf Bowman"] = "Bogenschütze der Frostwölfe",
		["Master Engineer Zinfizzlex"] = "Master Engineer Zinfizzlex",
		["Frostwolf Shredder Unit"] = "Schreddereinheit der Frostwölfe",
		["Irondeep Miner"] = "Minenarbeiter der Eisenschachtmine",
		["Irondeep Peon"] = "Peon der Eisenschachtmine",
		["Stormpike Shredder Unit"] = "Schreddereinheit der Sturmlanzen",
		["Sagorne Creststrider"] = "Sagorne Gratläufer ",
		["Ivus the Forest Lord"] = "Ivus der Waldfürst",
		["Champion Guardian"] = "Wächterchampion",
		["Champion Defender"] = "Verteidigerchampion",
		["Champion Guardsman"] = "Gardistenchampion",
		["Champion Legionnaire"] = "Legionärchampion",
		["Champion Mountaineer"] = "Champion Mountaineer",
		["Champion Sentinel"] = "Schildwachenchampion",
		["Champion Warrior"] = "Champion Warrior",
		["Wing Commander Ichman"] = "Schwadronskommandant Ichman",
		["Wing Commander Slidore"] = "Schwadronskommandant Erzrutsch",
		["Wing Commander Vipore"] = "Schwadronskommandant Vipore",
		["Frostwolf Wolf Rider"] = "Wolfsreiter der Frostwölfe",
		["Frostwolf Wolf Rider Commander"] = "Wolfsreiterkommandant der Frostwölfe",
		["Arch Druid Renferal"] = "Erzdruide Renferal",
		["Druid of the Grove"] = "Druide des Hains",
		["Field Marshal Teravaine"] = "Feldmarschall Teravaine",
		["Corporal Noreg Stormpike"] = "Korporal Noreg Sturmlanze",
		["Sergeant Yazra Bloodsnarl"] = "Unteroffizier Yazra Murrblut",
		["Warmaster Garrick"] = "Kriegsmeister Garrick",
		["Noxxion's Spawn"] = "Noxxions Brut",
		["Stormpike Commando"] = "Kommandosoldat der Sturmlanzen",
		["Seasoned Commando"] = "Erfahrener Kommandosoldat",
		["Veteran Commando"] = "Kommandosoldatenveteran",
		["Champion Commando"] = "Kommandosoldatenchampion",
		["Frostwolf Reaver"] = "Häscher der Frostwölfe",
		["Seasoned Reaver"] = "Erfahrener Häscher",
		["Veteran Reaver"] = "Häscherveteran",
		["Champion Reaver"] = "Häscherchampion",
		["Seasoned Coldmine Guard"] = "Erfahrene Wache der Eisbeißermine",
		["Veteran Coldmine Guard"] = "Wachenveteran der Eisbeißermine",
		["Champion Coldmine Guard"] = "Wachenchampion der Eisbeißermine",
		["Seasoned Coldmine Surveyor"] = "Erfahrener Feldmesser der Eisbeißermine",
		["Veteran Coldmine Surveyor"] = "Feldmesserveteran der Eisbeißermine",
		["Champion Coldmine Surveyor"] = "Feldmesserchampion der Eisbeißermine",
		["Seasoned Irondeep Explorer"] = "Erfahrener Forscher der Eisenschachtmine",
		["Veteran Irondeep Explorer"] = "Forscherveteran der Eisenschachtmine",
		["Champion Irondeep Explorer"] = "Forscherchampion der Eisenschachtmine",
		["Seasoned Irondeep Raider"] = "Erfahrener Räuber der Eisenschachtmine",
		["Veteran Irondeep Raider"] = "Räuberveteran der Eisenschachtmine",
		["Champion Irondeep Raider"] = "Räuberchampion der Eisenschachtmine",
		["Seasoned Coldmine Explorer"] = "Erfahrener Forscher der Eisbeißermine",
		["Veteran Coldmine Explorer"] = "Forscherveteran der Eisbeißermine",
		["Champion Coldmine Explorer"] = "Forscherchampion der Eisbeißermine",
		["Seasoned Coldmine Invader"] = "Erfahrener Eindringling der Eisbeißermin",
		["Veteran Coldmine Invader"] = "Eindringlingveteran der Eisbeißermine",
		["Champion Coldmine Invader"] = "Eindringlingchampion der Eisbeißermine",
		["Seasoned Irondeep Guard"] = "Erfahrene Wache der Eisenschachtmine",
		["Veteran Irondeep Guard"] = "Wachenveteran der Eisenschachtmine",
		["Champion Irondeep Guard"] = "Wachenchampion der Eisenschachtmine",
		["Seasoned Irondeep Surveyor"] = "Erfahrener Feldmesser der Eisenschachtmi",
		["Veteran Irondeep Surveyor"] = "Feldmesserveteran der Eisenschachtmine",
		["Champion Irondeep Surveyor"] = "Feldmesserchampion der Eisenschachtmine",
		["Stormpike Ram Rider"] = "Widderreiter der Sturmlanzen",
		["Stormpike Ram Rider Commander"] = "Kommandant der Sturmlanzenwidderreiter",
		["Rotgrip"] = "Faulschnapper",
		["Frostwolf Explosives Expert"] = "Sprengmeister der Frostwölfe",
		["Stormpike Explosives Expert"] = "Sprengmeister der Sturmlanzen",
		["Stolid Snapjaw"] = "Phlegmatischer Schnappkiefer",
		["Tinkerer Gizlock"] = "Tüftler Gizlock",
		["Frostwolf Stable Master"] = "Stallmeisterin der Frostwölfe ",
		["Stormpike Stable Master"] = "Stallmeister der Sturmlanzen ",
		["Stabled Frostwolf"] = "Gezähmter Frostwolf",
		["Stabled Alterac Ram"] = "Gezähmter Alteracwidder",
		["Selendra"] = "Selendra",
		["Celebras the Redeemed"] = "Celebras der Erlöste",
		["Veng"] = "Veng ",
		["Maraudos"] = "Maraudos ",
		["Corporal Teeka Bloodsnarl"] = "Korporal Teeka Murrblut",
		["Sergeant Durgen Stormpike"] = "Unteroffizier Durgen Sturmlanze",
		["Mountaineer Boombellow"] = "Gebirgsjäger Donnerbrüll",
		["Jotek"] = "Jotek",
		["Prospector Stonehewer"] = "Ausgrabungsleiter Steinhauer",
		["Voggah Deathgrip"] = "Voggah Todesgriff",
		["Royal Dreadguard"] = "Königliche Schreckenswache",
		["Warmaster Laggrond"] = "Kriegsmeister Laggrond",
		["Lieutenant Haggerdin"] = "Leutnant Haggerdin",
		["Frostwolf Ambassador Rokhstrom"] = "Botschafterin Rokhstrom der Frostwölfe",
		["Lieutenant Rotimer"] = "Leutnant Rotimer ",
		["Scalebeard"] = "Schuppenbart",
		["Dire Maul Crystal Totem"] = "Düsterbruch Kristalltotem",
		["Izzy Coppergrab"] = "Izzy Kupferzock ",
		["Winterax Mystic"] = "Winterax des Mystikers",
		["Winterax Warrior"] = "Winterax des Krieges",
		["Winterax Seer"] = "Winterax des Sehers",
		["Alterac Yeti"] = "Alterac Yeti",
		["Blackwing Technician"] = "Techniker der Pechschwingen",
		["Chromaggus"] = "Chromaggus",
		["Corrupted Red Whelp"] = "Verderbter roter Welpe",
		["Corrupted Green Whelp"] = "Verderbter grüner Welpe",
		["Corrupted Blue Whelp"] = "Verderbter blauer Welpe",
		["Corrupted Bronze Whelp"] = "Verderbter bronzener Welpe",
		["Enraged Felguard"] = "Wütende Teufelswache",
		["Massive Geyser"] = "Massiver Geysir",
		["Bounty Hunter Kolark"] = "Kopfgeldjäger Kolark",
		["Artilleryman Sheldonore"] = "Artillerist Sheldonor",
		["Najak Hexxen"] = "Najak Hexxen ",
		["Ravak Grimtotem"] = "Ravak Grimmtotem ",
		["Athramanis"] = "Athramanis ",
		["Dirk Swindle"] = "Dirk Schwindel ",
		["Lord Angler"] = "Lord Angler",
		["Ironbark the Redeemed"] = "Eisenborke der Erlöste",
		["Blue Drakonid"] = "Blauer Drakonid",
		["Green Drakonid"] = "Grüner Drakonid",
		["Bronze Drakonid"] = "Bronzener Drakonid",
		["Red Drakonid"] = "Roter Drakonid",
		["Black Drakonid"] = "Schwarzer Drakonid",
		["Frostwolf Bloodhound"] = "Bluthund der Frostwölfe",
		["Stormpike Owl"] = "Eule der Sturmlanzen",
		["Stormpike Battleguard"] = "Schlachtwache der Sturmlanzen",
		["Frostwolf Battleguard"] = "Schildwache der Frostwölfe",
		["Chromatic Drakonid"] = "Chromatischer Drakonid",
		["Petrified Guardian"] = "Versteinerter Wächter",
		["Kor'kron Elite"] = "Elitesoldat der Kor'kron",
		["Ferra"] = "Ferra",
		["Guard Fengus"] = "Wache Fengus",
		["Stomper Kreeg"] = "Stampfer Kreeg ",
		["Guard Slip'kik"] = "Wache Slip'kik",
		["Cho'Rush the Observer"] = "Cho'Rush der Beobachter",
		["Captain Kromcrush"] = "Hauptmann Krombruch",
		["Guard Mol'dar"] = "Wache Mol'dar",
		["Lethtendris"] = "Lethtendris",
		["Field Repair Bot 74A"] = "Feldreparaturbot-74A",
		["Knot Thimblejack"] = "Knot Zwingschraub",
		["Death Howl"] = "Todesheuler",
		["Alshirr Banebreath"] = "Alshirr Teufelsodem",
		["Ragepaw"] = "Wutpranke",
		["Olm the Wise"] = "Olm der Weise",
		["Mongress"] = "Mongress",
		["The Ongar"] = "Der Ongar",
		["Highlord Demitrian"] = "Hochlord Demitrian",
		["Earthcaller Franzahl"] = "Erdenrufer Franzahl",
		["Pimgib"] = "Pimgib",
		["Hydroling"] = "Hydroling",
		["Gordok Bushwacker"] = "Einzelkämpfer der Gordok",
		["Mizzle the Crafty"] = "Mizzle der Gewiefte",
		["Pusillin"] = "Pusillin",
		["Azj'Tordin"] = "Azj'Tordin",
		["Shen'dralar Ancient"] = "Uralte Shen'dralar",
		["Thornling"] = "Dornling",
		["Thief Catcher Shadowdelve"] = "Diebesjäger Schattenlauer",
		["Shen'dralar Spirit"] = "Geist der Shen'dralar",
		["Thief Catcher Farmountain"] = "Diebesjäger Bergweh",
		["Thief Catcher Thunderbrew"] = "Diebesjäger Donnerbräu",
		["Lorekeeper Lydros"] = "Hüter des Wissens Lydros",
		["Shen'dralar Zealot"] = "Zelot der Shen'dralar",
		["Cadaverous Worm"] = "Kadaverwurm",
		["Shen'dralar Provisioner"] = "Versorger der Shen'dralar",
		["Winterfall Ambusher"] = "Weglagerer der Winterfelle",
		["Sage Korolusk"] = "Weiser Korolusk",
		["Scholar Runethorn"] = "Gelehrter Runendorn",
		["Scout Stronghand"] = "Späher Starkpranke",
		["Scout Manslayer"] = "Späher Menschentod",
		["Scout Tharr"] = "Späher Tharr",
		["Huntress Skymane"] = "Jägerin Himmelshaar",
		["Huntress Ravenoak"] = "Jägerin Rabeneiche",
		["Huntress Leafrunner"] = "Jägerin Laubschleicher",
		["Lorekeeper Javon"] = "Hüter des Wissens Javon",
		["Lorekeeper Mykos"] = "Hüterin des Wissens Mykos",
		["Lorekeeper Kildrath"] = "Hüter des Wissens Kildrath",
		["Doomguard Minion"] = "Diener der Verdammniswache",
		["Wandering Eye of Kilrogg"] = "Wanderndes Auge von Kilrogg",
		["Lothos Riftwaker"] = "Lothos Felsspalter",
		["Rogue Black Drake"] = "Abtrünniger Schwarzdrache",
		["Netherwalker"] = "Netherwandler",
		["Overlord Runthak"] = "Oberanführer Runthak",
		["Major Mattingly"] = "Major Mattingly",
		["Griniblix the Spectator"] = "Griniblix der Beobachter",
		["Eye of Immol'thar"] = "Auge von Immol'thar",
		["Mana Burst"] = "Manaeruption",
		["Eldreth Darter"] = "Grimmlingflitzer der Eldreth",
		["Arcane Torrent"] = "Arkaner Strom",
		["Arcane Feedback"] = "Arkane Rückkopplung",
		["Master Elemental Shaper Krixix"] = "Meisterelementarformer Krixix",
		["Seeker Cromwell"] = "Sucher Kromwell",
		["Seeker Nahr"] = "Sucher Nahr",
		["Seeker Thompson"] = "Sucher Thompson",
		["Officer Jaxon"] = "Offizier Jaxon",
		["Prince Thunderaan"] = "Prinz Donneraan ",
		["Mor'zul Bloodbringer"] = "Mor'zul Blutbringer",
		["Gorzeeki Wildeyes"] = "Gorzeeki Wildaug",
		["Officer Pomeroy"] = "Offizier Pomeroy",
		["Officer Brady"] = "Offizier Brady",
		["Hunter Sagewind"] = "Jäger Weisenwind",
		["Hunter Ragetotem"] = "Jäger Rachtotem",
		["Hunter Thunderhorn"] = "Jäger Donnerhorn",
		["Enslaved Doomguard Commander"] = "Versklavter Kommandant der Verdammniswac",
		["The Windreaver"] = "Der Windhäscher",
		["Whirling Invader"] = "Wirbelnder Eindringling",
		["Blackwing Guardsman"] = "Gardist der Pechschwingen",
		["Princess Tempestria"] = "Prinzessin Tempestria",
		["Watery Invader"] = "Wässriger Eindringling",
		["Blazing Invader"] = "Lodernder Eindringling",
		["Baron Charr"] = "Baron Glutarr",
		["Thundering Invader"] = "Donnernder Eindringling",
		["Daio the Decrepit"] = "Daio der Klapprige",
		["Avalanchion"] = "Lavinius",
		["Kroshius"] = "Kroshius",
		["Niby the Almighty"] = "Niby der Allmächtige",
		["Impsy"] = "Impsy ",
		["Setis"] = "Setis",
		["Gretheer"] = "Gretheer",
		["Lapress"] = "Lapress",
		["Zora"] = "Zora",
		["Rex Ashil"] = "Rex Ashil",
		["Krellack"] = "Krellack",
		["Grubthor"] = "Grubthor",
		["Huricanian"] = "Hurrikanus",
		["Twilight Lord Everun"] = "Lord des Schattenhammers Everun ",
		["Xorothian Imp"] = "Xorothianischer Wichtel",
		["Dread Guard"] = "Schreckenswache",
		["Injured Peasant"] = "Verletzter Arbeiter",
		["Plagued Peasant"] = "Verseuchter Arbeiter",
		["Scourge Footsoldier"] = "Fußsoldat der Geißel ",
		["Scourge Archer"] = "Bogenschütze der Geißel",
		["Eris Havenfire"] = "Eris Himmelsfeuer",
		["J'eevee"] = "J'eevee",
		["Xorothian Dreadsteed"] = "Xorothianisches Schreckensross",
		["The Cleaner"] = "Der Läuterer",
		["Dreadsteed Spirit"] = "Schreckensrossgeist",
		["Lord Hel'nurath"] = "Lord Hel'nurath",
		["High Priest Venoxis"] = "Hohepriester Venoxis",
		["Short John Mithril"] = "Störtebrecher",
		["High Priest Thekal"] = "Hohepriester Thekal",
		["High Priestess Mar'li"] = "Hohepriesterin Mar'li",
		["Shadowed Spirit"] = "Schattenhafter Geist",
		["Corrupted Spirit"] = "Verdorbener Geist",
		["Malicious Spirit"] = "Bösartiger Geist",
		["Banal Spirit"] = "Banaler Geist",
		["High Priestess Arlokk"] = "Hohepriesterin Arlokk",
		["Death Knight Darkreaver"] = "Todesritter Schattensichel",
		["High Priestess Jeklik"] = "Hohepriesterin Jeklik",
		["Aspect of Banality"] = "Aspekt der Banalität",
		["Aspect of Corruption"] = "Aspekt der Verdorbenheit",
		["Aspect of Malice"] = "Aspekt der Bosheit",
		["Aspect of Shadow"] = "Aspekt des Schattens",
		["Ur'dan"] = "Ur'dan",
		["Ulathek"] = "Ulathek",
		["Vartrus the Ancient"] = "Vartrus der Uralte",
		["Stoma the Ancient"] = "Stoma der Uralte",
		["Hastat the Ancient"] = "Hastat der Uralte",
		["Simone the Inconspicuous"] = "Simone die Unauffällige",
		["Precious"] = "Pummelchen ",
		["Franklin the Friendly"] = "Franklin der Freundliche",
		["Solenor the Slayer"] = "Solenor der Schlächter",
		["Artorius the Amiable"] = "Artorius der Liebenswürdige",
		["Razzashi Venombrood"] = "Razzashigiftbrut",
		["Simone the Seductress"] = "Simone die Verführerin",
		["Klinfran the Crazed"] = "Klinfran der Wahnsinnige",
		["Artorius the Doombringer"] = "Artorius der Verhängnisvolle",
		["Nelson the Nice"] = "Nelson der Nette",
		["Precious the Devourer"] = "Pummelchen der Fresser ",
		["Terrordale Spirit"] = "Schreckenstalgeist",
		["Ancient Equine Spirit"] = "Uralter Pferdegeist",
		["Derotain Mudsipper"] = "Derotain Matschnipper ",
		["Darkreaver's Fallen Charger"] = "Schattensichels gefallenes Streitross",
		["Sergeant Thunderhorn"] = "Unteroffizier Donnerhorn ",
		["Ebonroc"] = "Schattenschwinge",
		["Zapped Shore Strider"] = "Geschrumpfter Uferschreiter",
		["Zapped Land Walker"] = "Geschrumpfter Landgänger",
		["Bone Construct"] = "Knochenkonstrukt",
		["Overseer Maltorius"] = "Aufseher Maltorius",
		["Thorium Brotherhood Lookout"] = "Ausguck der Thoriumbruderschaft ",
		["Master Smith Burninate"] = "Meisterschmied Burninate ",
		["Overseer Oilfist"] = "Aufseher Schlickfaust ",
		["Taskmaster Scrange"] = "Zuchtmeister Skrange ",
		["Hansel Heavyhands"] = "Hansel Bleihand ",
		["Evonice Sootsmoker"] = "Evonice Rußraucher ",
		["Lookout Captain Lolo Longstriker"] = "Hauptmann des Ausgucks Lolo Langhieb ",
		["Zapped Wave Strider"] = "Geschrumpfter Wellenschreiter",
		["Zapped Deep Strider"] = "Geschrumpfter Tiefenschreiter",
		["Zapped Cliff Giant"] = "Geschrumpfter Klippenriese",
		["Stinglasher"] = "Zornstachel",
		["Corrupted Fire Nova Totem V"] = "Verdorbenes Totem der Feuernova V",
		["Corrupted Stoneskin Totem VI"] = "Verdorbenes Totem der Steinhaut VI",
		["Corrupted Healing Stream Totem V"] = "Verdorbenes Totem des heilendes Flusses ",
		["Corrupted Windfury Totem III"] = "Verdorbenes Totem des Windzorns III",
		["Corrupted Infernal"] = "Verdorbene Höllenbestie",
		["Balzaphon"] = "Balzaphon",
		["Revanchion"] = "Revanchion",
		["Scorn"] = "Der Verächter",
		["Lord Blackwood"] = "Lord Blackwood",
		["Silverwing Elite"] = "Elitesoldat der Silberschwingen",
		["Horde Elite"] = "Elitesoldat der Horde",
		["High Overlord Saurfang"] = "Hochfürst Saurfang",
		["Field Marshal Afrasiabi"] = "Feldmarschall Afrasiabi",
		["Revantusk Watcher"] = "Behüter der Bruchhauer",
		["Lard"] = "Lard ",
		["PvP CTF Credit Marker"] = "PvP CTF Anrechnungsmarkierung",
		["Sentinel Farsong"] = "Schildwache Weltensang",
		["Revantusk Drummer"] = "Trommler der Bruchhauer",
		["Primal Torntusk"] = "Ura Stumpfhauer",
		["Mystic Yayo'jin"] = "Mystikerin Yayo'jin ",
		["Zap Farflinger"] = "Zap Weitschleuder ",
		["Jhordy Lapforge"] = "Jhordy Lapforge ",
		["Vilebranch Kidnapper"] = "Kidnapper der Blutfratzen",
		["Gurubashi Bat Rider"] = "Blutsucherreiter",
		["Illiyana Moonblaze"] = "Illiyana Mondflamm ",
		["Kelm Hargunth"] = "Kelm Hargunth ",
		["Elder Torntusk"] = "Ältester Stumpfhauer",
		["Creeping Doom"] = "Schleichende Verdammnis",
		["Dun Baldar North Marshal"] = "Marschall der Nordtruppen von Dun Baldar",
		["Dun Baldar South Marshal"] = "Marschall der Südtruppen von Dun Baldar",
		["Icewing Marshal"] = "Marschall der Eisschwingentruppen",
		["Stonehearth Marshal"] = "Marschall der Steinbruchtruppen",
		["Iceblood Marshal"] = "Marschall der Eisbluttruppen",
		["Tower Point Marshal"] = "Marschall der Turmstellung",
		["East Frostwolf Marshal"] = "Ostmarschall der Frostwölfe",
		["West Frostwolf Marshal"] = "Westmarschall der Frostwölfe",
		["Dun Baldar North Warmaster"] = "Kriegsmeister der Nordtruppen von Dun Ba",
		["Dun Baldar South Warmaster"] = "Kriegsmeister der Südtruppen von Dun Bal",
		["East Frostwolf Warmaster"] = "Ostkriegsmeister der Frostwölfe",
		["Iceblood Warmaster"] = "Kriegsmeister der Eisbluttruppen",
		["Icewing Warmaster"] = "Kriegsmeister der Eissschwingentruppen",
		["Stonehearth Warmaster"] = "Kriegsmeister der Steinbruchtruppen",
		["Tower Point Warmaster"] = "Kriegsmeister der Turmstellung",
		["West Frostwolf Warmaster"] = "Westkriegsmeister der Frostwölfe",
		["Captain Shatterskull"] = "Hauptmann Schädelbrecher",
		["Razzashi Raptor"] = "Razzashiraptor",
		["Sayge"] = "Sayge ",
		["Silas Darkmoon"] = "Silas Darkmoon ",
		["Withered Mistress"] = "Herrin des Siechens",
		["Sacrificed Troll"] = "Geopferter Troll",
		["Burth"] = "Burth",
		["Yebb Neblegear"] = "Yebb Nabetand",
		["Kerri Hicks"] = "Kerri Hicks ",
		["Chronos"] = "Chronos ",
		["Hakkar"] = "Hakkar",
		["Rinling"] = "Rinling",
		["Blood Steward of Kirtonos"] = "Blutdiener von Kirtonos",
		["Emissary Roman'khan"] = "Abgesandter Roman'khan",
		["Molthor"] = "Molthor ",
		["Zandalar Headshrinker"] = "Zandalarianischer Kopfschrumpfer",
		["Razzashi Skitterer"] = "Razzashihuscher",
		["Atal'ai Mistress"] = "Herrin der Atal'ai",
		["Voodoo Slave"] = "Voodoosklave",
		["Parasitic Serpent"] = "Schmarotzerschlange",
		["Ysondre"] = "Ysondre",
		["Lethon"] = "Lethon",
		["Emeriss"] = "Smariss",
		["Taerar"] = "Taerar",
		["Jin'rokh the Breaker"] = "Jin'rokh der Zerstörer",
		["Al'tabim the All-Seeing"] = "Al'tabim der Allsehende",
		["Maywiki of Zuldazar"] = "Maywiki von Zuldazar",
		["Falthir the Sightless"] = "Falthir der Blinde",
		["Exzhal"] = "Exzhal ",
		["Zandalar Enforcer"] = "Zandalarianischer Vollstrecker",
		["Captured Hakkari Zealot"] = "Gefangener Zelot der Hakkari",
		["Rin'wosho the Trader"] = "Rin'wosho der Händler ",
		["Kartra Bloodsnarl"] = "Kartra Murrblut ",
		["Guse's War Rider"] = "Guses Kriegsreiter",
		["Jeztor's War Rider"] = "Jeztors Kriegsreiter",
		["Mulverick's War Rider"] = "Mulvericks Kriegsreiter",
		["Slidore's Gryphon"] = "Erzrutschs Greif",
		["Ichman's Gryphon"] = "Ichmans Greif",
		["Vipore's Gryphon"] = "Vipores Greif",
		["Frenzied Bloodseeker Bat"] = "Rasende Blutsucherfledermaus",
		["Elfarran"] = "Elfarran ",
		["Lylandris"] = "Lylandris ",
		["Field Marshal Oslight"] = "Feldmarschall Lichtmark",
		["Sergeant Maclear"] = "Unteroffizier Maclear",
		["Shade of Jin'do"] = "Schemengestalt von Jin'do",
		["Powerful Healing Ward"] = "Mächtiger Heilungszauberschutz",
		["Ohgan"] = "Ohgan",
		["Deze Snowbane"] = "Deze Schneefluch ",
		["Sir Malory Wheeler"] = "Sire Malory Wagner ",
		["Lady Hoteshem"] = "Lady Hoteshem ",
		["Wagner Hammerstrike"] = "Wagner Hammerstrike",
		["Javnir Nashak"] = "Javnir Nashak",
		["Deathmaster Dwire"] = "Todesmeister Dunkels",
		["Deathstalker Mortis"] = "Todespirscher Mortis",
		["Spawn of Mar'li"] = "Brut von Mar'li",
		["Zanza the Restless"] = "Zanza der Ruhelose",
		["Zulian Crocolisk"] = "Zulianischer Krokilisk",
		["Zulian Stalker"] = "Zulianischer Pirscher",
		["Zulian Guardian"] = "Zulianischer Wächter",
		["Vinchaxa"] = "Vinchaxa ",
		["Zandalarian Emissary"] = "Abgesandter der Zandalarianer",
		["Riggle Bassbait"] = "Riggle Barschfang ",
		["Jang"] = "Jang ",
		["Fishbot 5000"] = "Fischbot 5000",
		["Servant of the Hand"] = "Diener der Hand ",
		["Gri'lek"] = "Gri'lek",
		["Hazza'rah the Dreamweaver"] = "Hazza'rah",
		["Renataki"] = "Renataki",
		["Wushoolay"] = "Wushoolay",
		["Booty Bay Elite"] = "Elitesoldat von Beutebucht",
		["Zulian Prowler"] = "Zulianischer Streuner",
		["Stormpike Emissary"] = "Abgesandter der Sturmlanzen",
		["Frostwolf Emissary"] = "Abgesandter der Frostwölfe",
		["Gurubashi Prisoner"] = "Gurubashi Prisoner",
		["Mad Servant"] = "Verrückter Diener",
		["Brain Wash Totem"] = "Totem der Gehirnwäsche",
		["Honored Hero"] = "Geehrter Held",
		["Gahz'ranka"] = "Gahz'ranka",
		["Honored Ancestor"] = "Verehrter Vorfahr",
		["Grinkle"] = "Grinkel ",
		["Chained Spirit"] = "Angeketteter Geist",
		["Barrus"] = "Barrus ",
		["Rutherford Twing"] = "Rutherford Twing ",
		["Samuel Hawke"] = "Samuel Falk ",
		["Defiler Elite"] = "Elitesoldat der Entweihten",
		["Hammerfall Elite"] = "Elitesoldat von Hammerfall",
		["Menethil Elite"] = "Elitesoldat von Menethil",
		["Silverpine Elite"] = "Elitesoldat des Silberwalds",
		["Mad Voidwalker"] = "Wahnsinniger Leerwandler",
		["Scarlet Inquisitor"] = "Scharlachroter Inquisitor",
		["Nightmare Illusion"] = "Alptraumillusion",
		["Haughty Modiste"] = "Eingebildete Modeschöpferin ",
		["Vile Scarab"] = "Übler Skarabäus",
		["Ralo'shan the Eternal Watcher"] = "Ralo'shan die ewige Wächterin",
		["Rutgar Glyphshaper"] = "Rutgar Glyphenschnitzer",
		["Frankal Stonebridge"] = "Frankal Steinsteg",
		["Glibb"] = "Glibb ",
		["Calandrath"] = "Calandrath ",
		["Khur Hornstriker"] = "Khur Hornstürmer ",
		["Vargus"] = "Vargus ",
		["Cloud Skydancer"] = "Celest Himmelstänzer ",
		["Runk Windtamer"] = "Runk Windlenker ",
		["Mishta"] = "Mishta ",
		["Baristolth of the Shifting Sands"] = "Baristolth der Sandstürme",
		["Commander Mar'alith"] = "Kommandant Mar'alith",
		["Vish Kozus"] = "Vish Kozus ",
		["Geologist Larksbane"] = "Geologin Lerchenbann",
		["Cenarion Hold Infantry"] = "Soldat der Burg Cenarius",
		["Brood of Nozdormu"] = "Brut Nozdormus",
		["Cenarion Emissary Jademoon"] = "Abgesandte Jademond ",
		["Cenarion Emissary Blackhoof"] = "Abgesandter Schwarzhuf ",
		["Beetix Ficklespragg"] = "Beetix Wankelhupf",
		["Noggle Ficklespragg"] = "Noggle Wankelhupf",
		["Windcaller Proudhorn"] = "Windruferin Prachthorn",
		["Anachronos"] = "Anachronos",
		["Hermit Ortell"] = "Ortell der Einsiedler",
		["Wickerman Guardian"] = "Weidenmannwache",
		["Deathclasp"] = "Totenstachel",
		["Darkcaller Yanka"] = "Ruferin der Dunkelheit Yanka",
		["Sergeant Hartman"] = "Sergeant Hartman",
		["Twilight Keeper Mayna"] = "Bewahrer des Schattenhammers Mayna ",
		["Twilight Flamereaver"] = "Flammenhäscher des Schattenhammerklans",
		["Vyral the Vile"] = "Vyral der Üble ",
		["Prince Skaldrenox"] = "Prinz Skaldrenox ",
		["High Marshal Whirlaxis"] = "Hochmarschall Wirbelgrat ",
		["Baron Kazum"] = "Baron Kazum ",
		["The Duke of Cynders"] = "Der Fürst der Asche ",
		["The Duke of Fathoms"] = "Der Fürst der Tiefen ",
		["The Duke of Shards"] = "Der Fürst der Splitter ",
		["Crimson Templar"] = "Purpurroter Templer ",
		["Azure Templar"] = "Azurblauer Templer ",
		["Hoary Templar"] = "Weißgrauer Templer ",
		["Twilight Overlord"] = "Oberanführer des Schattenhammers ",
		["Mistress Natalia Mar'alith"] = "Herrin Natalia Mar'alith ",
		["The Duke of Zephyrs"] = "Der Fürst der Stürme ",
		["Vekniss Soldier"] = "Soldat der Vekniss",
		["Vekniss Warrior"] = "Krieger der Vekniss",
		["Vekniss Guardian"] = "Wächter der Vekniss",
		["Vekniss Stinger"] = "Peitscher der Vekniss",
		["Vekniss Wasp"] = "Wespe der Vekniss",
		["Vekniss Hive Crawler"] = "Schwarmkriecher der Vekniss",
		["Qiraji Mindslayer"] = "Willensbrecher der Qiraji",
		["Qiraji Brainwasher"] = "Gehirnwäscher der Qiraji",
		["Qiraji Lasher"] = "Wutstachel der Qiraji",
		["Qiraji Slayer"] = "Vollstrecker der Qiraji",
		["Qiraji Champion"] = "Champion der Qiraji",
		["Demented Druid Spirit"] = "Verrückter Druidengeist",
		["Spirit Shade"] = "Geisterschatten",
		["Obsidian Eradicator"] = "Obsidianvernichter",
		["The Prophet Skeram"] = "Der Prophet Skeram",
		["Anubisath Sentinel"] = "Wächter des Anubisath",
		["Huum Wildmane"] = "Huum Wildmähne",
		["Emperor Vek'nilash"] = "Imperator Vek'nilash",
		["Emperor Vek'lor"] = "Imperator Vek'lor",
		["Anubisath Defender"] = "Verteidiger des Anubisath",
		["Aurel Goldleaf"] = "Aurel Goldblatt",
		["Xil'xix"] = "Xil'xix ",
		["Aluntir"] = "Aluntir ",
		["Arakis"] = "Arakis ",
		["Aendel Windspear"] = "Aendel Windspeer",
		["Viscidus"] = "Viscidus",
		["Vekniss Drone"] = "Drohne der Vekniss",
		["Shade of Taerar"] = "Taerars Schemen",
		["Lord Skwol"] = "Lord Skwol ",
		["Bor Wildmane"] = "Bor Wildmähne",
		["Earthen Templar"] = "Irdener Templer ",
		["Twilight Prophet"] = "Prophet des Schattenhammers ",
		["Anubisath Warder"] = "Aufseher des Anubisath",
		["Obsidian Nullifier"] = "Obsidianauslöscher",
		["Mylini Frostmoon"] = "Mylini Eismond ",
		["Qiraji Scarab"] = "Skarabäus der Qiraji",
		["Qiraji Scorpion"] = "Skorpion der Qiraji",
		["Hive'Zara Drone"] = "Drohne des Zaraschwarms",
		["Hive'Zara Collector"] = "Sammler des Zaraschwarms",
		["Hive'Zara Soldier"] = "Soldat des Zaraschwarms",
		["Hive'Zara Sandstalker"] = "Sandpirscher des Zaraschwarms",
		["Qiraji Gladiator"] = "Gladiator der Qiraji",
		["Hive'Zara Wasp"] = "Wespe des Zaraschwarms",
		["Hive'Zara Stinger"] = "Stecher des Zaraschwarms",
		["Silicate Feeder"] = "Silikatfresser",
		["Giant Eye Tentacle"] = "Riesiges Augententakel",
		["Flesh Hunter"] = "Fleischjäger",
		["Hive'Zara Tail Lasher"] = "Schwanzpeitscher des Zaraschwarms",
		["Obsidian Destroyer"] = "Obsidianzerstörer",
		["Ossirian the Unscarred"] = "Ossirian der Narbenlose",
		["Moam"] = "Moam",
		["General Rajaxx"] = "General Rajaxx",
		["Qiraji Swarmguard"] = "Schwarmwache der Qiraji",
		["Swarmguard Needler"] = "Nadelspeiende Schwarmwache",
		["Kurinnaxx"] = "Kurinnaxx",
		["Horde Warbringer"] = "Kriegshetzer der Horde",
		["Alliance Brigadier General"] = "Brigadegeneral der Allianz",
		["Anubisath Guardian"] = "Beschützer des Anubisath",
		["Tonk Mine"] = "Panzermine",
		["Ayamiss the Hunter"] = "Ayamiss der Jäger",
		["Buru the Gorger"] = "Buru der Verschlinger",
		["Merithra of the Dream"] = "Merithra des Traums",
		["Caelestrasz"] = "Caelestrasz",
		["Arygos"] = "Arygos",
		["Anachronos the Ancient"] = "Anachronos der Uralte",
		["Fandral Staghelm"] = "Fandral Hirschhaupt",
		["Sergeant Stonebrow"] = "Unteroffizier Steinbraue ",
		["OLDWorld Trigger"] = "ALTWorld Trigger",
		["Colonel Zerran"] = "Oberst Zerran",
		["Major Yeggeth"] = "Major Yeggeth",
		["Qiraji Warrior"] = "Krieger der Qiraji",
		["Major Pakkon"] = "Major Pakkon",
		["Captain Drenn"] = "Hauptmann Drenn",
		["Captain Xurrem"] = "Hauptmann Xurrem",
		["Captain Qeez"] = "Hauptmann Qeez",
		["Captain Tuubid"] = "Hauptmann Tuubid",
		["Nafien"] = "Nafien",
		["Qiraji Wasp"] = "Wespe der Qiraji",
		["Kania"] = "Kania ",
		["Qiraji Drone"] = "Drohne der Qiraji",
		["Qiraji Tank"] = "Qirajipanzer",
		["Kaldorei Infantry"] = "Infanterie der Kaldorei",
		["Anubisath Conqueror"] = "Eroberer des Anubisath",
		["Ahn'Qiraj Trigger"] = "Ahn'Qiraj Trigger",
		["Sand Vortex"] = "Sandvortex",
		["Corporal Carnes"] = "Korporal Schwarz ",
		["Dame Twinbraid"] = "Dame Doppelzopf ",
		["Private Draxlegauge"] = "Gefreiter Drechselknacks ",
		["Master Nightsong"] = "Meister Nachtweise ",
		["Captain Blackanvil"] = "Hauptmann Rußschmied ",
		["Ironforge Brigade Rifleman"] = "Scharfschütze der Eisenschmiedebrigade",
		["Ironforge Brigade Footman"] = "Fußsoldat der Eisenschmiedebrigade",
		["Janela Stouthammer"] = "Janela Starkhammer",
		["Arcanist Nozzlespring"] = "Arkanist Sprinkelstaub",
		["Sergeant Major Germaine"] = "Stabsfeldwebel Germaine ",
		["Bonnie Stoneflayer"] = "Bonnie Steinschinder ",
		["Private Porter"] = "Gefreiter Porter ",
		["Hive'Zora Abomination"] = "Monstrosität des Zoraschwarms",
		["Marta Finespindle"] = "Marta Feinspindel ",
		["Sentinel Silversky"] = "Schildwache Silberwolke ",
		["Nurse Stonefield"] = "Schwester Steinfeld ",
		["Keeper Moonshade"] = "Bewahrer Mondschatten ",
		["Slicky Gastronome"] = "Gastronom Glitschko ",
		["Sarah Sadwhistle"] = "Sarah Sorgenpfeifer ",
		["Huntress Swiftriver"] = "Jägerin Flinkbach ",
		["Commander Stronghammer"] = "Kommandant Hammerkraft ",
		["Miner Cromwell"] = "Minenarbeiter Kromwell ",
		["Grunt Maug"] = "Grunzer Maug ",
		["Shrieker Scarab"] = "Kreischerskarabäus",
		["Spitting Scarab"] = "Speiender Skarabäus",
		["Grace of Air Totem III"] = "Totem der luftgleichen Anmut III",
		["Strength of Earth Totem V"] = "Totem der Erdstärke V",
		["Minion of Omen"] = "Diener von Omen",
		["Omen"] = "Omen",
		["Senior Sergeant T'kelah"] = "Schlachtrufer T'kelah ",
		["Lieutenant General Andorov"] = "Generalleutnant Andorov",
		["Kaldorei Elite"] = "Elitesoldat der Kaldorei",
		["Herbalist Proudfeather"] = "Kräuterkundige Stolzfeder ",
		["Spirit of Azuregos"] = "Geist von Azuregos",
		["Eranikus, Tyrant of the Dream"] = "Eranikus der Tyrann des Traums",
		["Nighthaven Defender"] = "Verteidiger von Nachthafen",
		["Windcaller Yessendra"] = "Windruferin Yessendra",
		["Warden Haro"] = "Aufseher Haro",
		["Keyl Swiftclaw"] = "Keyl Flinkklaue",
		["Andorgos"] = "Andorgos ",
		["Kandrostrasz"] = "Kandrostrasz ",
		["Vethsera"] = "Vethsera ",
		["Canal Frenzy"] = "Kanalfrenzy",
		["Batrider Pele'keiki"] = "Fledermausreiter Pele'keiki ",
		["Princess Huhuran"] = "Prinzessin Huhuran",
		["Fankriss the Unyielding"] = "Fankriss der Unnachgiebige",
		["Lord Kri"] = "Lord Kri",
		["Buru Egg"] = "Ei von Buru",
		["Skinner Jamani"] = "Kürschner Jamani ",
		["Battleguard Sartura"] = "Schlachtwache Sartura",
		["Ouro"] = "Ouro",
		["Hive'Zara Hatchling"] = "Jungtier des Zaraschwarms",
		["Meridith the Mermaiden"] = "Meridith die Meerjungfrau",
		["Mana Fiend"] = "Manageist",
		["Healer Longrunner"] = "Heiler Fernläufer ",
		["Bloodguard Rawtar"] = "Blutwache Rawtar ",
		["Fisherman Lin'do"] = "Angler Lin'do ",
		["Chief Sharpclaw"] = "Häuptling Scharfklaue ",
		["Anubisath Warrior"] = "Krieger des Anubisath",
		["Anubisath Swarmguard"] = "Schwarmwache des Anubisath",
		["General Zog"] = "General Zog ",
		["Windcaller Kaldon"] = "Windrufer Kaldon",
		["Twilight Marauder Morna"] = "Marodeur des Schattenhammers Morna ",
		["Twilight Marauder"] = "Marodeur des Schattenhammers ",
		["Princess Yauj"] = "Prinzessin Yauj",
		["Vem"] = "Vem",
		["Cenarion Outrider"] = "Cenarische Vorhut",
		["Hive'Zara Swarmer"] = "Schwärmer des Zaraschwarms",
		["Elder Morndeep"] = "Urahne Schwermut",
		["Doctor Weavil"] = "Doktor Weavil",
		["Doctor Weavil's Flying Machine"] = "Doktor Weavils Flugmaschine",
		["Number Two"] = "Nummer Zwei",
		["Hive'Zara Larva"] = "Larve des Zaraschwarms",
		["Elder Splitrock"] = "Urahne Splitterfels",
		["Elder Rumblerock"] = "Urahne Rumpelfels",
		["Elder Silvervein"] = "Urahne Silberader",
		["Elder Highpeak"] = "Urahne Höhengipfel",
		["Elder Stonefort"] = "Urahne Steinwehr",
		["Elder Obsidian"] = "Urahne Obsidian",
		["Elder Hammershout"] = "Urahne Hammerklang",
		["Elder Bellowrage"] = "Urahne Heulwut",
		["Elder Darkcore"] = "Urahne Dunkelherz",
		["Elder Stormbrow"] = "Urahne Sturmbraue",
		["Elder Snowcrown"] = "Urahne Schneekrone",
		["Elder Ironband"] = "Urahne Eisenband",
		["Elder Graveborn"] = "Urahne Grufthauch",
		["Elder Goldwell"] = "Urahne Goldbrunn",
		["Elder Primestone"] = "Urahne Urstein",
		["Maws"] = "Der weiße Riese",
		["Elder Runetotem"] = "Urahnin Runentotem",
		["Elder Ragetotem"] = "Urahnin Rachtotem",
		["Elder Stonespire"] = "Urahne Steinkeil",
		["Elder Bloodhoof"] = "Urahne Bluthuf",
		["Elder Winterhoof"] = "Urahne Winterhuf",
		["Elder Skychaser"] = "Urahnin Himmelsjäger",
		["Elder Wildmane"] = "Urahnin Wildmähne",
		["Elder Darkhorn"] = "Urahnin Dunkelhorn",
		["Elder Proudhorn"] = "Urahne Prachthorn",
		["Elder Grimtotem"] = "Urahne Grimmtotem",
		["Elder Windtotem"] = "Urahnin Windtotem",
		["Elder Thunderhorn"] = "Urahne Donnerhorn",
		["Elder Skyseer"] = "Urahne Himmelsdeuter",
		["Elder Dawnstrider"] = "Urahnin Morgenwandler",
		["Elder Dreamseer"] = "Urahne Traumdeuter",
		["Elder Mistwalker"] = "Urahnin Nebelgänger",
		["Elder High Mountain"] = "Urahne Hohenberg",
		["Eye of C'Thun"] = "Auge von C'Thun",
		["Ossirian Crystal Trigger"] = "Ossiriankristall Trigger",
		["Minion of Weavil"] = "Diener von Weavil",
		["Elder Windrun"] = "Urahne Windflucht",
		["Elder Starsong"] = "Urahnin Sternensang",
		["Elder Moonstrike"] = "Urahnin Mondkuss",
		["Elder Moonstrike"] = "Urahne Messerblatt",
		["Elder Starglade"] = "Urahne Sternenau",
		["Elder Moonwarden"] = "Urahne Mondwacht",
		["Elder Bladeswift"] = "Urahne Klingentanz",
		["Elder Bladesing"] = "Urahnin Klingensang",
		["Elder Skygleam"] = "Urahne Himmelsglanz",
		["Elder Starweave"] = "Urahnin Sternweber",
		["Elder Meadowrun"] = "Urahnin Auenflucht",
		["Elder Nightwind"] = "Urahne Nachtwind",
		["Elder Morningdew"] = "Urahnin Morgentau",
		["Elder Riversong"] = "Urahnin Flussweise",
		["Elder Brightspear"] = "Urahnin Prunkspeer",
		["Elder Farwhisper"] = "Urahne Fernwisper",
		["Cenarion Scout Landion"] = "Cenarischer Späher Landion",
		["Cenarion Scout Azenel"] = "Cenarische Späherin Azenel",
		["Cenarion Scout Jalia"] = "Cenarische Späherin Jalia",
		["Krug Skullsplit"] = "Krug Schädelspalt ",
		["Merok Longstride"] = "Merok Langschreiter",
		["J.D. Shadesong"] = "J.D. Schattenklang",
		["Shadow Priestess Shai"] = "Schattenpriesterin Shai",
		["Orgrimmar Legion Grunt"] = "Grunzer der Orgrimmarlegion",
		["Orgrimmar Legion Axe Thrower"] = "Axtwerfer der Orgrimmarlegion",
		["Hive'Regal Hunter-Killer"] = "Jägerbestie des Regalschwarms",
		["Yauj Brood"] = "Yaujbrut",
		["Vekniss Borer"] = "Bohrer der Vekniss",
		["Xandivious"] = "Xandivious",
		["Twilight Corrupter"] = "Twilight Corrupter",
		["Eranikus the Redeemed"] = "Eranikus der Erlöste",
		["Nightmare Phantasm"] = "Alptraumphantom",
		["Spawn of Fankriss"] = "Brut von Fankriss",
		["Tyrande"] = "Tyrande ",
		["Priestess of the Moon"] = "Mondpriesterin",
		["Auctioneer Jaxon"] = "Auktionatorin Jaxon",
		["Glob of Viscidus"] = "Klumpen von Viscidus",
		["Auctioneer Stockton"] = "Auktionator Stockton",
		["Auctioneer Yarly"] = "Auktionator Yarly",
		["Auctioneer Graves"] = "Auktionator Greves",
		["Auctioneer Silva'las"] = "Auktionator Silva'las",
		["Auctioneer Cazarez"] = "Auktionator Cazarez",
		["Auctioneer O'reely"] = "Auktionator O'reely",
		["Auctioneer Cain"] = "Auktionator Cain",
		["Auctioneer Naxxremis"] = "Auktionatorin Naxxremis",
		["Auctioneer Tricket"] = "Auktionatorin Tricket",
		["Auctioneer Rhyker"] = "Auktionatorin Rhyker",
		["Jonathan the Revelator"] = "Jonathan der Weise",
		["Vek Twins Trigger"] = "Vek Twins Trigger",
		["Warlord Gorchuk"] = "Kriegsherr Gorchuk ",
		["Field Marshal Snowfall"] = "Feldmarschall Schneefall ",
		["Senior Sergeant Taiga"] = "Schlachtruferin Taiga ",
		["Senior Sergeant Grimsford"] = "Schlachtrufer Grimmwald ",
		["Senior Sergeant Kai'jin"] = "Schlachtruferin Kai'jin ",
		["Master Sergeant Fizzlebolt"] = "Hauptfeldwebel Zischelbolz ",
		["Master Sergeant Maclure"] = "Hauptfeldwebel Maclure ",
		["Master Sergeant Moonshadow"] = "Hauptfeldwebel Mondnacht ",
		["Ouro Scarab"] = "Skarabäus von Ouro",
		["Timbermaw Ancestor"] = "Vorfahre der Holzschlundfeste",
		["Squire Leoren Mal'derath"] = "Knappe Leoren Mal'derath ",
		["Drunken Bruiser"] = "Betrunkener Haudrauf",
		["Claw Tentacle"] = "Klauententakel",
		["Eye Tentacle"] = "Augententakel",
		["C'Thun"] = "C'Thun",
		["Giant Claw Tentacle"] = "Riesiges Klauententakel",
		["Darnassus Commendation Officer"] = "Belobigungsoffizier von Darnassus",
		["Gnomeregan Commendation Officer"] = "Belobigungsoffizier von Gnomeregan",
		["Ironforge Commendation Officer"] = "Belobigungsoffizier von Eisenschmiede",
		["Stormwind Commendation Officer"] = "Belobigungsoffizier von Sturmwind",
		["Orgrimmar Commendation Officer"] = "Belobigungsoffizier von Orgrimmar",
		["Darkspear Commendation Officer"] = "Belobigungsoffizier der Dunkelspeere",
		["Undercity Commendation Officer"] = "Belobigungsoffizier von Unterstadt",
		["Thunder Bluff Commendation Officer"] = "Belobigungsoffizier von Donnerfels",
		["Colossus of Zora"] = "Koloss des Zoraschwarms",
		["Colossus of Regal"] = "Koloss des Regalschwarms",
		["Colossus of Ashi"] = "Koloss des Ashischwarms",
		["Colossal Anubisath Warbringer"] = "Kolossaler Kriegshetzer des Anubisath",
		["Imperial Qiraji Destroyer"] = "Imperialer Qirajizerstörer",
		["Qiraji Brigadier General"] = "Brigadegeneral der Qiraji",
		["Greater Anubisath Warbringer"] = "Starker Kriegshetzer des Anubisath",
		["Greater Silithid Flayer"] = "Starker Silithidschinder",
		["Qiraji Lieutenant General"] = "Generallieutenant der Qiraji",
		["Supreme Anubisath Warbringer"] = "Mächtiger Kriegshetzer des Anubisath",
		["Supreme Silithid Flayer"] = "Mächtiger Silithidschinder",
		["Officer Vu'Shalay"] = "Offizier Vu'Shalay ",
		["Officer Lunalight"] = "Offizier Mondlicht ",
		["Officer Porterhouse"] = "Offizier Packhäuser ",
		["Officer Ironbeard"] = "Offizier Eisenbart ",
		["Officer Redblade"] = "Offizier Rotklinge ",
		["Officer Maloof"] = "Offizier Maloof ",
		["Officer Thunderstrider"] = "Offizier Donnerschreiter ",
		["Officer Gothena"] = "Offizier Gothena ",
		["Christmas Emperor Dagran Thaurissan"] = "Christmas Emperor Dagran Thaurissan",
		["Colossus Researcher Sophia"] = "Kolossforscherin Sophia",
		["Colossus Researcher Nestor"] = "Kolossforscher Nestor",
		["Colossus Researcher Eazel"] = "Kolossforscher Eazel",
		["Flesh Tentacle"] = "Fleischtentakel",
		["Tranquil Air Totem"] = "Totem der beruhigenden Winde",
		["Qiraji Major He'al-ie"] = "Major der Qiraji He'al-ie",
		["Qiraji Brigadier General Pax-lish"] = "Brigadegeneral der Qiraji Pax-lish",
		["Lieutenant General Nokhor"] = "Generallieutenant Nokhor",
		["Orgrimmar Elite Shieldguard"] = "Eliteschildwache von Orgrimmar",
		["Orgrimmar Elite Infantryman"] = "Eliteinfanterist von Orgrimmar",
		["Orgrimmar Elite Cavalryman"] = "Elitekavallerist von Orgrimmar",
		["Tauren Rifleman"] = "Scharfschütze der Tauren",
		["Tauren Primalist"] = "Primalist der Tauren",
		["Stormwind Cavalryman"] = "Kavallerist von Sturmwind",
		["Stormwind Infantryman"] = "Infanterist von Sturmwind",
		["Stormwind Archmage"] = "Erzmagier von Sturmwind",
		["Kaldorei Marksman"] = "Schütze der Kaldorei",
		["Ironforge Infantryman"] = "Infanterist von Eisenschmiede",
		["Ironforge Cavalryman"] = "Kavallerist von Eisenschmiede",
		["Darkspear Shaman"] = "Schamane der Dunkelspeere",
		["Valadar Starsong"] = "Valadar Sternensang ",
		["High Commander Lynore Windstryke"] = "Oberkommandantin Lynore Windstryke",
		["Highlord Leoric Von Zeldig"] = "Hochlord Leoric Von Zeldig",
		["Malagav the Tactician"] = "Malagav der Taktiker",
		["Duke August Foehammer"] = "Fürst August Foehammer",
		["Elder Bronzebeard"] = "Urahne Bronzebart",
		["Pat's Firework Guy - RED"] = "Pats Feuerwerkstyp - ROT",
		["Pat's Firework Guy - YELLOW"] = "Pats Feuerwerkstyp - GELB",
		["Lunar Festival Herald"] = "Herold des Mondfests",
		["Lunar Festival Emissary"] = "Abgesandter des Mondfests",
		["Lunar Firework Credit Marker"] = "Mondfeuerwerk Credit Marker",
		["Lunar Cluster Credit Marker"] = "Mondraketenbündel Credit Marker",
		["Lunar Festival Harbinger"] = "Botin des Mondfests",
		["Lunar Festival Vendor"] = "Verkäufer des Mondfests",
		["Giant Spotlight"] = "Riesiges Spotlight",
		["Sergeant Carnes"] = "Unteroffizier Schwarz",
		["Fariel Starsong"] = "Fariel Sternensang ",
		["Pat's Firework Cluster Guy (ELUNE)"] = "Pats Feuerwerkstyp (ELUNE)",
		["Thaddius"] = "Thaddius",
		["Stalagg"] = "Stalagg",
		["Feugen"] = "Feugen",
		["Grobbulus"] = "Grobbulus",
		["Gluth"] = "Gluth",
		["Poison Cloud"] = "Giftwolke",
		["Hive'Zara Hornet"] = "Hornisse des Zaraschwarms",
		["Heigan the Unclean"] = "Heigan der Unreine",
		["Maexxna"] = "Maexxna",
		["Grand Widow Faerlina"] = "Großwitwe Faerlina",
		["Noth the Plaguebringer"] = "Noth der Seuchenfürst",
		["Anub'Rekhan"] = "Anub'Rekhan",
		["Ouro Spawner"] = "Ouro Spawner",
		["Lunar Festival Sentinel"] = "Schildwache des Mondfests",
		["Vekniss Hatchling"] = "Jungtier der Vekniss",
		["The Master's Eye"] = "Auge des Meisters",
		["Buru Egg Trigger"] = "Ei von Buru Auslöser",
		["Dread Creeper"] = "Schreckenskrabbler",
		["Carrion Spinner"] = "Aasspinner",
		["Venom Stalker"] = "Giftpirscher",
		["Poisonous Skitterer"] = "Ansteckender Huscher",
		["Crypt Reaver"] = "Grufthäscher",
		["Tomb Horror"] = "Grabschrecken",
		["Naxxramas Cultist"] = "Kultist von Naxxramas",
		["Naxxramas Acolyte"] = "Akolyt von Naxxramas",
		["Sartura's Royal Guard"] = "Sarturas Königswache",
		["Sapphiron"] = "Saphiron",
		["Kel'Thuzad"] = "Kel'Thuzad",
		["InCombat Trigger"] = "ImKampfauslöser",
		["Loatheb"] = "Loatheb",
		["Mokvar"] = "Mokvar",
		["Deliana"] = "Deliana",
		["Mux Manascrambler"] = "Mux Manamix",
		["Vi'el"] = "Vi'el ",
		["Anthion Harmon"] = "Anthion Harmon",
		["Patchwork Golem"] = "Zusammengeflickter Golem",
		["Bile Retcher"] = "Gallwürger",
		["Mad Scientist"] = "Verrückter Wissenschaftler",
		["Living Monstrosity"] = "Lebende Monströsität",
		["Surgical Assistant"] = "Chirurgenassistent",
		["Embalming Slime"] = "Einhüllender Brühschleimer",
		["Stitched Spewer"] = "Narbenspeier",
		["Living Poison"] = "Lebende Giftlache",
		["Patchwerk"] = "Flickwerk",
		["Sludge Belcher"] = "Schlammspucker",
		["Ysida Harmon"] = "Ysida Harmon",
		["Falrin Treeshaper"] = "Falrin Rankenweber",
		["Bodley"] = "Bodley",
		["Plague Beast"] = "Seuchenbestie",
		["Frenzied Bat"] = "Rasende Fledermaus",
		["Plagued Bat"] = "Verseuchte Fledermaus",
		["Lord Valthalak"] = "Lord Valthalak",
		["Magma Lord Bokk"] = "Magmafürst Bokk",
		["Lefty"] = "Lefty",
		["Rotfang"] = "Totenbiss",
		["Snokh Blackspine"] = "Snokh Schwarzrücken",
		["Malgen Longspear"] = "Malgen Langspeer",
		["Korv"] = "Korv",
		["Rezznik"] = "Rezznik",
		["Va'jashni"] = "Va'jashni",
		["Diseased Maggot"] = "Kranke Made",
		["Rotting Maggot"] = "Faulende Made",
		["Volida"] = "Volida",
		["Theldren"] = "Theldren",
		["Gothik the Harvester"] = "Gothik der Seelenjäger",
		["Instructor Razuvious"] = "Instrukteur Razuvious",
		["Highlord Mograine"] = "Hochlord Mograine ",
		["Sir Zeliek"] = "Sire Zeliek",
		["Thane Korth'azz"] = "Thane Korth'azz",
		["Lady Blaumeux"] = "Lady Blaumeux",
		["Spectral Assassin"] = "Spektraler Assassine",
		["Deathcharger Steed"] = "Skelettross",
		["Tidelord Rrurgaz"] = "Gezeitenlord Rrurgaz",
		["Spirit of Lord Valthalak"] = "Geist von Lord Valthalak",
		["Mor Grayhoof"] = "Mor Grauhuf",
		["Naxxramas Trigger"] = "Auslöser von Naxxramas",
		["Rousch"] = "Rousch",
		["Dirk Thunderwood"] = "Dirk Donnerholz",
		["Spectral Stalker"] = "Spektraler Pirscher",
		["Gnashjaw"] = "Knirschkiefer ",
		["Steamwheedle Bruiser"] = "Haudrauf des Dampfdruckkartells",
		["Isalien"] = "Isalien",
		["Empyrean"] = "Lichtschwinge",
		["Jarien"] = "Jarien",
		["Sothos"] = "Sothos",
		["Spirit of Jarien"] = "Geist von Jarien",
		["Spirit of Sothos"] = "Geist von Sothos",
		["Korfax, Champion of the Light"] = "Korfax der Held des Lichts ",
		["Father Inigo Montoy"] = "Vater Inigo Montoy ",
		["Scarlet Commander Marjhan"] = "Scharlachroter Kommandant Marjhan ",
		["Commander Eligor Dawnbringer"] = "Kommandant Eligor Morgenbringer ",
		["Archmage Angela Dosantos"] = "Erzmagierin Angela Dosantos ",
		["Plagued Swine"] = "Verseuchtes Schwein",
		["Kormok"] = "Kormok",
		["Bone Minion"] = "Knochendiener",
		["Bone Mage"] = "Knochenmagier",
		["Unrelenting Trainee"] = "Unerbittlicher Lehrling",
		["Unrelenting Deathknight"] = "Unerbittlicher Todesritter",
		["Unrelenting Rider"] = "Unerbittlicher Reiter",
		["Spectral Trainee"] = "Spektraler Lehrling",
		["Shadow Fissure"] = "Schattenspalt",
		["Rohan the Assassin"] = "Rohan der Assassine ",
		["Huntsman Leopold"] = "Jäger Leopold ",
		["Mataus the Wrathcaster"] = "Mataus der Zornwirker ",
		["Rimblat Earthshatter"] = "Rimblat Erdspalter ",
		["Rayne"] = "Rayne ",
		["Naxxramas Military Sub-Boss Trigger"] = "Auslöser für Subbosse von Naxxramas",
		["Cenarion Hold Reservist"] = "Reservist der Burg Cenarius",
		["Ghoul Berserker"] = "Ghulberserker",
		["Bile Sludge"] = "Gallschlamm",
		["Deathknight Captain"] = "Hauptmann der Todesritter",
		["Deathknight"] = "Todesritter",
		["Spectral Deathknight"] = "Spektraler Todesritter",
		["Spectral Horse"] = "Spektrales Pferd",
		["Spectral Rider"] = "Spektraler Reiter",
		["Risen Deathknight"] = "Auferstandener Todesritter",
		["Dark Touched Warrior"] = "Dunkelgeplagter Krieger ",
		["Doom Touched Warrior"] = "Unheilgeplagter Krieger",
		["Death Touched Warrior"] = "Todesgeplagter Krieger",
		["Deathknight Cavalier"] = "Chevalier der Todesritter",
		["Shade of Naxxramas"] = "Schemen von Naxxramas",
		["Necro Knight"] = "Nekroritter",
		["Bony Construct"] = "Knochiges Konstrukt",
		["Stoneskin Gargoyle"] = "Steinhautgargoyle",
		["Nerubian Overseer"] = "Nerubischer Aufseher",
		["Skeletal Smith"] = "Skelettschmied",
		["Unholy Axe"] = "Unheilige Axt",
		["Dispatch Commander Metz"] = "Einsatzkommandant Metz ",
		["Unholy Staff"] = "Unheiliger Stab",
		["Unholy Swords"] = "Unheilige Schwerter",
		["Tesla Coil"] = "Teslaspul",
		["Guard Didier"] = "Wache Didier ",
		["Bragok"] = "Bragok ",
		["Argent Dawn Infantry"] = "Argentuminfanterist ",
		["Injured Argent Dawn Infantry"] = "Verletzter Argentuminfanterist ",
		["Caravan Mule"] = "Karawanentier",
		["Eye Stalk"] = "Augenstrunk",
		["Plague Slime"] = "Seuchenbrühschleimer",
		["Infectious Ghoul"] = "Ansteckender Ghul",
		["Field Marshal Chambers"] = "Feldmarschall Kammerer ",
		["Jessica Chambers"] = "Jessica Kammerer ",
		["Packmaster Stonebruiser"] = "Rottenkommandant Steinberster ",
		["Argent Medic"] = "Argentumsanitäter ",
		["Spore"] = "Spore",
		["Fallout Slime"] = "Verstrahlter Brühschleimer",
		["Mutated Grub"] = "Mutierte Larve",
		["Skeletal Shocktrooper"] = "Skelettstoßtruppsoldat",
		["Argent Messenger"] = "Argentumbote ",
		["Zombie Chow"] = "Zombiefraß",
		["Commander Thomas Helleran"] = "Kommandant Thomas Helleran ",
		["Master Craftsman Omarion"] = "Meisterhandwerker Omarion ",
		["Necropolis Acolyte"] = "Akolyt der Nekropole",
		["Sewage Slime"] = "Abwasserbrühschleimer",
		["Craftsman Wilhelm"] = "Handwerker Wilhelm ",
		["Argent Sentry"] = "Argentumwachposten ",
		["Spirit of the Damned"] = "Geist der Verdammten",
		["Bone Witch"] = "Knochenhexe",
		["Archmage Tarsis Kir-Moldir"] = "Erzmagier Tarsis Kir-Moldir",
		["Flameshocker"] = "Flammenschocker",
		["Lightning Totem"] = "Blitzschlagtotem",
		["Atiesh"] = "Atiesh >",
		["Deathchill Servant"] = "Todesfrostdiener",
		["Captain Armando Ossex"] = "Hauptmann Armando Ossex",
		["Toxic Tunnel"] = "Toxischer Tunnel",
		["Necropolis"] = "Nekropole",
		["Bronn Fitzwrench"] = "Bronn Fitzzang",
		["Ghost of Naxxramas"] = "Phantom von Naxxramas",
		["Soldier of the Frozen Wastes"] = "Soldat der eisigen Weiten",
		["Unstoppable Abomination"] = "Unaufhaltsame Monströsität",
		["Soul Weaver"] = "Seelenwirker",
		["Cracked Necrotic Crystal"] = "Gesprungener nekrotischer Kristall",
		["Guardian of Icecrown"] = "Wächter von Eiskrone",
		["Plagued Gargoyle"] = "Verseuchter Gargoyle",
		["Plagued Ghoul"] = "Verseuchter Ghul",
		["Plagued Deathhound"] = "Verseuchter Todeshund",
		["Spirit of Naxxramas"] = "Geist von Naxxramas",
		["Deathknight Vindicator"] = "Vollstrecker der Todesritter",
		["Necro Knight Guardian"] = "Nekrowächter",
		["Necro Stalker"] = "Nekropirscher",
		["Web Wrap"] = "Fangnetz",
		["Naxxramas Follower"] = "Anhänger von Naxxramas",
		["Naxxramas Worshipper"] = "Jünger von Naxxramas",
		["Argent Warhorse"] = "Argent Warhorse",
		["Faint Necrotic Crystal"] = "Schwacher nekrotischer Kristall",
		["Garon Hutchins"] = "Garon Hutchins",
		["Crypt Guard"] = "Gruftwache",
		["Corpse Scarab"] = "Leichenskarabäus",
		["Spirit of Mograine"] = "Geist von Mograine ",
		["Spirit of Blaumeux"] = "Geist von Blaumeux",
		["Spirit of Zeliek"] = "Geist von Zeliek",
		["Spirit of Korth'azz"] = "Geist von Korth'azz",
		["Festival Flamekeeper"] = "Flammenbewahrer",
		["Deathknight Understudy"] = "Reservist der Todesritter",
		["Festival Loremaster"] = "Meister der Lehren ",
		["Festival Talespinner"] = "Geschichtenerzähler",
		["Death Lord"] = "Todesfürst",
		["The Lich King"] = "Der Lichkönig",
		["Plagued Guardian"] = "Verseuchter Wächter",
		["Plagued Construct"] = "Verseuchtes Konstrukt",
		["Plagued Champion"] = "Verseuchter Champion",
		["Plagued Warrior"] = "Verseuchter Krieger",
		["Festival Flamekeeper Costume: Human"] = "Prinz Kael'Thas Sonnenwanderer",
		["Orgrimmar Fireeater"] = "Feuerschlucker von Orgrimmar",
		["Maexxna Spiderling"] = "Maexxnaspinnling",
		["Chief Expeditionary Requisitioner Enkles"] = "Chefexpeditionslieferant Enkles",
		["Emissary Whitebeard"] = "Abgesandter Weißbart",
		["Apothecary Quinard"] = "Apotheker Quinard",
		["Emissary Gormok"] = "Abgesandter Gormok",
		["General Kirika"] = "General Kirika",
		["Marshal Bluewall"] = "Marschall Himmelswand",
		["Scout Bloodfist"] = "Späher Blutfaust",
		["Rifleman Torrig"] = "Scharfschütze Torrig",
		["William Kielar"] = "William Kielar ",
		["Landro Longshot"] = "Landro Fernblick ",
		["Felguard"] = "Teufelswache",
		["Renn'az"] = "Renn'az ",
		["Lordaeron Commander"] = "Kommandant von Lordaeron",
		["Lordaeron Soldier"] = "Soldat von Lordaeron",
		["Alliance Silithyst Sentinel"] = "Silithystschildwache der Allianz",
		["Horde Silithyst Sentinel"] = "Silithystschildwache der Horde",
		["Squire Rowe"] = "Knappe Rowe",
		["Lordaeron Veteran"] = "Veteran von Lordaeron",
		["Lordaeron Fighter"] = "Kämpfer von Lordaeron",
		["The Bug Family"] = "The Bug Family",
		["The Twin Emperors"] = "The Twin Emperors",
		["The Four Horsemen"] = "The Four Horsemen",
		["Environment"] = "Umgebung",
		["Unknown"] = "Unbekannt",
	}
end)

NPCDB:RegisterTranslations("frFR", function()
	return {
		["Waypoint"] = "Point de SPAWN (Seuls les MJ peuvent le ",
		["Spawnpoint"] = "Spawn Point (Only GM can see it)",
		["Stormwind City Guard"] = "Guet de Stormwind",
		["Infernal"] = "Infernal",
		["Blue Dragonspawn"] = "Draconide bleu",
		["Earth Elemental"] = "Elémentaire de terre",
		["Maginor Dumas"] = "Maginor Dumas",
		["Master Mathias Shaw"] = "Maître Mathias Shaw",
		["Kendor Kabonka"] = "Kendor Kabonka",
		["Grizzle Halfmane"] = "Grison Demi-crin",
		["Dungar Longdrink"] = "Dungar Grangodet",
		["High Priestess Laurena"] = "Grande prêtresse Laurena",
		["Felhunter"] = "Chasseur corrompu",
		["Demisette Cloyce"] = "Demisette Cloyce",
		["General Marcus Jonathan"] = "Général Marcus Jonathan",
		["Lieutenant Doren"] = "Lieutenant Doren",
		["Water Elemental"] = "Elémentaire d'eau",
		["Thor"] = "Thor",
		["Grimtooth"] = "Mâchinistre",
		["El Pollo Grande"] = "Poulet de Nouvel An",
		["Mai'Zoth"] = "Mai'Zoth",
		["Coldridge Mountaineer"] = "Montagnard des Frigères",
		["Donal Osgood"] = "Donal Bongoulot",
		["Stonard Hunter"] = "Chasseur de Pierrêche",
		["Stonard Orc"] = "Orc de Pierrêche",
		["Stonard Wayfinder"] = "Pisteur de Pierrêche",
		["Stonard Grunt"] = "Grunt de Pierrêche",
		["Stonard Cartographer"] = "Cartographe de Pierrêche",
		["Stonard Shaman"] = "Chaman de Pierrêche",
		["Keras Wolfheart"] = "Keras C?ur-de-loup",
		["Ander Germaine"] = "Ander Germaine",
		["Osborne the Night Man"] = "Osborne l'homme de la nuit",
		["Lord Grayson Shadowbreaker"] = "Seigneur Grayson Brisombre",
		["Ariena Stormfeather"] = "Ariena Plumorage",
		["Swamp Talker"] = "Parleur des marécages",
		["Brother Neals"] = "Frère Neals",
		["Malosh"] = "Malosh",
		["Haromm"] = "Haromm",
		["Ogromm"] = "Ogromm",
		["Kartosh"] = "Kartosh",
		["Banalash"] = "Banalash",
		["Red Dragonspawn"] = "Draconide rouge",
		["Red Wyrmkin"] = "Wyrmide rouge",
		["Red Scalebane"] = "Plaiedécaille rouge",
		["Scalebane Lieutenant"] = "Lieutenant Plaiedécaille",
		["Wyrmkin Firebrand"] = "Wyrmide de braise",
		["Scalebane Royal Guard"] = "Garde royal Plaiedécaille",
		["Jade"] = "Jade",
		["Grom'gol Grunt"] = "Grunt de Grom'gol",
		["Watcher Merant"] = "Gardienne Merant",
		["Angus Stern"] = "Angus Stern",
		["Senator Barin Redstone"] = "Sénateur Barin Rougepierre",
		["Archbishop Benedictus"] = "Archevêque Benedictus",
		["Felder Stover"] = "Felder Stover",
		["Charys Yserian"] = "Charys Yserian",
		["Adair Gilroy"] = "Adair Gilroy",
		["Elly Langston"] = "Elly Langston",
		["Mayda Thane"] = "Mayda Thane",
		["Georgio Bolero"] = "Georgio Bolero",
		["Rogvar"] = "Rogvar",
		["Thysta"] = "Thysta",
		["Ian Strom"] = "Ian Strom",
		["Grimand Elmore"] = "Grimand Elmore",
		["Zardeth of the Black Claw"] = "Zardeth de la Griffe noire",
		["Helgrum the Swift"] = "Helgrum l'Agile",
		["Fel'zerul"] = "Fel'zerul",
		["Jannos Ironwill"] = "Jannos Volonté-de-fer",
		["Morgg Stormshot"] = "Morgg Trait-tempête",
		["Menethil Guard"] = "Garde de Menethil",
		["Mok'rash"] = "Mok'rash",
		["Negolash"] = "Negolash",
		["Gunther Arcanus"] = "Gunther Arcanus",
		["Bethor Iceshard"] = "Bethor Glacéclat",
		["King Mukla"] = "Roi Mukla",
		["Shellei Brondir"] = "Shellei Brondir",
		["Thorgrum Borrelson"] = "Thorgrum Borrelson",
		["Gryth Thurden"] = "Gryth Thurden",
		["Northshire Guard"] = "Garde de Comté-du-nord",
		["Deathguard Randolph"] = "Nécrogarde Randolph",
		["Deathguard Oliver"] = "Nécrogarde Oliver",
		["Deathguard Phillip"] = "Nécrogarde Phillip",
		["Deathguard Bartrand"] = "Nécrogarde Bartrand",
		["Highlord Bolvar Fordragon"] = "Généralissime Bolvar Fordragon",
		["Lady Katrana Prestor"] = "Dame Katrana Prestor",
		["Grand Admiral Jes-Tereth"] = "Grand amiral Jes-Tereth",
		["Mithras Ironhill"] = "Mithras Colline-de-fer",
		["Caledra Dawnbreeze"] = "Caledra Bisedelaube",
		["Stormwind Royal Guard"] = "Garde royal de Stormwind",
		["Zun'dartha"] = "Zun'dartha",
		["Skeletal Flayer"] = "Ecorcheur Squelette",
		["Skeletal Sorcerer"] = "Ensorceleur squelette",
		["Skeletal Terror"] = "Terreur Squelette",
		["Skeletal Executioner"] = "Bourreau Squelette",
		["Skeletal Warlord"] = "Chef de guerre Squelette",
		["Skeletal Acolyte"] = "Acolyte Squelette",
		["Slavering Ghoul"] = "Goule écumante",
		["Rotting Ghoul"] = "Goule purulente",
		["Soulless Ghoul"] = "Goule dévastée",
		["Searing Ghoul"] = "Goule incendiaire",
		["Freezing Ghoul"] = "Goule glaciale",
		["Cold Wraith"] = "Ame en peine froide",
		["Blood Wraith"] = "Ame en peine sanglante",
		["Hungering Wraith"] = "Ame en peine affamée",
		["Wailing Death"] = "Morte gémissante",
		["Flesh Golem"] = "Golem de chair",
		["Vile Slime"] = "Gelée maléfique",
		["Devouring Ooze"] = "Limon dévorant",
		["Carrion Vulture"] = "Vautour putride",
		["Rotting Behemoth"] = "Béhémoth pourrissant",
		["Decaying Horror"] = "Horreur décomposée",
		["Diseased Black Bear"] = "Ours noir malade",
		["Diseased Grizzly"] = "Grizzly malade",
		["Diseased Wolf"] = "Loup malade",
		["Carrion Lurker"] = "Rôdeuse putride",
		["Venom Mist Lurker"] = "Rôdeuse venimeuse des brumes",
		["Plague Lurker"] = "Rôdeuse pestilentielle",
		["Scarlet Mage"] = "Mage écarlate",
		["Scarlet Sentinel"] = "Sentinelle écarlate",
		["Scarlet Hunter"] = "Chasseur écarlate",
		["Scarlet Magus"] = "Magus écarlate",
		["Scarlet Knight"] = "Chevalier écarlate",
		["Scarlet Paladin"] = "Paladin écarlate",
		["Scarlet Invoker"] = "Invocateur écarlate",
		["Scarlet Cavalier"] = "Cavalier écarlate",
		["Scarlet Judge"] = "Juge écarlate",
		["Scarlet Interrogator"] = "Inquisiteur écarlate",
		["Scarlet High Clerist"] = "Grand prêtre écarlate",
		["Grand Inquisitor Isillien"] = "Grand Inquisiteur Isillien",
		["Scarlet Executioner"] = "Bourreau écarlate",
		["Highlord Taelan Fordring"] = "Généralissime Taelan Fordring",
		["Foreman Jerris"] = "Contremaître Jerris",
		["Foreman Marcrid"] = "Contremaître Marcrid",
		["High Protector Tarsen"] = "Haut Protecteur Tarsen",
		["High Protector Lorik"] = "Haut Protecteur Lorik",
		["Foulmane"] = "Vilcrin",
		["Lord Maldazzar"] = "Seigneur Maldazzar",
		["Putridius"] = "Putridius",
		["The Husk"] = "La Bogue",
		["Araj the Summoner"] = "Araj l'Invocateur",
		["Darkmaster Gandling"] = "Sombre Maître Gandling",
		["High Priest Thel'danis"] = "Grand prêtre Thel'danis",
		["Tirion Fordring"] = "Tirion Fordring",
		["Scarlet Worker"] = "Ouvrier écarlate",
		["Scarlet Lumberjack"] = "Bûcheron écarlate",
		["Scarlet Smith"] = "Forgeron écarlate",
		["Treant"] = "Tréant",
		["Stormwind City Patroller"] = "Patrouilleur de Stormwind",
		["Senator Mehr Stonehallow"] = "Sénateur Mehr Sacrepierre",
		["Ancient Protector"] = "Ancien protecteur",
		["Master Apothecary Faranell"] = "Maître apothicaire Faranell",
		["Natheril Raincaller"] = "Natheril Mande-pluie",
		["High Executor Darthalia"] = "Grande exécutrice Darthalia",
		["Zora Guthrek"] = "Zora Guthrek",
		["Karos Razok"] = "Karos Razok",
		["Borgus Stoutarm"] = "Borgus Rudebras",
		["Aethalas"] = "Aethalas",
		["Lyranne Feathersong"] = "Lyranne Chanteplume",
		["Southshore Guard"] = "Garde d?Austrivage",
		["Zarise"] = "Zarise",
		["Daryl Stack"] = "Daryl Stack",
		["Tarren Mill Deathguard"] = "Nécrogarde de Moulin-de-Tarren",
		["Felicia Maline"] = "Felicia Maline",
		["Varimathras"] = "Varimathras",
		["Darla Harris"] = "Darla Harris",
		["Major Samuelson"] = "Major Samuelson",
		["Commander Aggro'gosh"] = "Commandant Aggro'gosh",
		["Bro'kin"] = "Fra'kas",
		["Larimaine Purdue"] = "Larimaine Purdue",
		["Fleet Master Seahorn"] = "Amiral Corne-de-mer",
		["Baron Revilgaz"] = "Baron Revilgaz",
		["Saltwater Snapjaw"] = "Gueule d'acier des mers",
		["Skymane Gorilla"] = "Gorille Crins-de-ciel",
		["Jaguero Stalker"] = "Traqueur jaguero",
		["Zanzil the Outcast"] = "Zanzil le Paria",
		["Catelyn the Blade"] = "Catelyn la Lame",
		["Fleet Master Firallon"] = "Amiral Firallon",
		["Captain Keelhaul"] = "Capitaine Soulaquille",
		["Hammerfall Guardian"] = "Gardien de Trépas-d'Orgrim",
		["Sly Garrett"] = "Garrett le Matois",
		["Viznik Goldgrubber"] = "Viznik Fouinelor",
		["Vilebranch Witch Doctor"] = "Sorcier-docteur Vilebranche",
		["Vilebranch Headhunter"] = "Chasseur de têtes Vilebranche",
		["Vilebranch Shadowcaster"] = "Exhalombre Vilebranche",
		["Vilebranch Berserker"] = "Berserker Vilebranche",
		["Vilebranch Hideskinner"] = "Ecorcheur Vilebranche",
		["Vilebranch Shadow Hunter"] = "Chasseur des ombres Vilebranche",
		["Vilebranch Blood Drinker"] = "Sanguinaire Vilebranche",
		["Vilebranch Soul Eater"] = "Mangeur d?âmes Vilebranche",
		["Vilebranch Aman'zasi Guard"] = "Garde Aman'zasi Vilebranche",
		["Green Sludge"] = "Limace verte",
		["Jade Ooze"] = "Limon de jade",
		["Razorbeak Skylord"] = "Seigneur du ciel Bec-rasoir",
		["Mechanical Dragonling"] = "Petit dragon mécanique",
		["Vilebranch Wolf Pup"] = "Louveteau Vilebranche",
		["Vilebranch Raiding Wolf"] = "Loup d'attaque Vilebranche",
		["Ruppo Zipcoil"] = "Ruppo Filebobine",
		["Highvale Ranger"] = "Forestier du Haut-val",
		["Hanashi"] = "Hanashi",
		["Shadra"] = "Shadra",
		["Archaedas"] = "Archaedas",
		["Blacklash"] = "Cinglenoir",
		["Hematus"] = "Hematus",
		["King Magni Bronzebeard"] = "Roi Magni Barbe-de-bronze",
		["Zaruk"] = "Zaruk",
		["Susan Tillinghast"] = "Susan Tillinghast",
		["Malygen"] = "Malygen",
		["Kurden Bloodclaw"] = "Kurden Griffe-de-sang",
		["Bale"] = "Bale",
		["Cedrik Prose"] = "Cedrik Prose",
		["Brikk Keencraft"] = "Brikk Belouvrage",
		["Jaxin Chong"] = "Jaxin Chong",
		["Crazk Sparks"] = "Crazk Escarbilles",
		["Hurklor"] = "Hurklor",
		["Jansen Underwood"] = "Jansen Sousbois",
		["Urda"] = "Urda",
		["Gringer"] = "Gringer",
		["Gyll"] = "Gyll",
		["Gorrik"] = "Gorrik",
		["Grunenstur Balindom"] = "Grunenstur Balindom",
		["Hurom Juggendolf"] = "Hurom Juggendolf",
		["Durdek Karrin"] = "Durdek Karrin",
		["Garek"] = "Garek",
		["Silvermane Stalker"] = "Traqueur Crins-d'argent",
		["Savage Owlbeast"] = "Chouettard sauvage",
		["Zaricotl"] = "Zaricotl",
		["Lanie Reed"] = "Lanie Roseau",
		["Tal"] = "Tal",
		["Una"] = "Una",
		["Siln Skychaser"] = "Siln Chasse-le-ciel",
		["Beram Skychaser"] = "Beram Chasse-le-ciel",
		["Turak Runetotem"] = "Turak Totem-runique",
		["Sheal Runetotem"] = "Sheal Totem-runique",
		["Kary Thunderhorn"] = "Kary Corne-tonnerre",
		["Holt Thunderhorn"] = "Holt Corne-tonnerre",
		["Torm Ragetotem"] = "Torm Totem-de-rage",
		["Sark Ragetotem"] = "Sark Totem-de-rage",
		["Malakai Cross"] = "Malakai Lacroix",
		["Father Cobb"] = "Père Cobb",
		["Archmage Shymm"] = "Archimage Shymm",
		["Thurston Xane"] = "Thurston Xane",
		["Cairne Bloodhoof"] = "Cairne Sabot-de-sang",
		["Honor Guard"] = "Garde d?honneur",
		["Bluffwatcher"] = "Gardien des Hauts",
		["Unseen"] = "Invisible",
		["Brave Proudsnout"] = "Brave Mufle-fier",
		["Brave Lightninghorn"] = "Brave Corne-de-foudre",
		["Brave Running Wolf"] = "Brave Cours-comme-le-loup",
		["Brave Greathoof"] = "Brave Grand-sabot",
		["Nazgrel"] = "Nazgrel",
		["Orgrimmar Grunt"] = "Grunt d?Orgrimmar",
		["Gabrielle Chase"] = "Gabrielle Chase",
		["Grisha"] = "Grisha",
		["Doras"] = "Doras",
		["Olvia"] = "Olvia",
		["Grol'dar"] = "Grol'dar",
		["Mirket"] = "Mirket",
		["Zevrost"] = "Zevrost",
		["Ormok"] = "Ormok",
		["Sergra Darkthorn"] = "Sergra Sombreronce",
		["Grelkor"] = "Grelkor",
		["Kardris Dreamseeker"] = "Kardris Cherche-rêve",
		["Ormak Grimshot"] = "Ormak Trait-sinistre",
		["Grezz Ragefist"] = "Grezz Ragepoing",
		["Sorek"] = "Sorek",
		["Gazlowe"] = "Gazleu",
		["Shenthul"] = "Shenthul",
		["Xor'juul"] = "Xor'juul",
		["Melor Stonehoof"] = "Melor Sabot-de-pierre",
		["Ancient of Lore"] = "Ancien du savoir",
		["Ancient of War"] = "Ancien de la guerre",
		["Tarhus"] = "Tarhus",
		["Ratchet Bruiser"] = "Cogneur de Ratchet",
		["Arch Druid Fandral Staghelm"] = "Archidruide Fandral Forteramure",
		["Teldrassil Sentinel"] = "Sentinelle de Teldrassil",
		["Sewer Beast"] = "Bête des égouts",
		["Aman"] = "Aman",
		["Devrak"] = "Devrak",
		["Onu"] = "Onu",
		["Kurll"] = "Kurll",
		["Rarck"] = "Rarck",
		["Vrang Wildgore"] = "Vrang Fauvétripe",
		["Raene Wolfrunner"] = "Raene Mène-loup",
		["Jadenvis Seawatcher"] = "Jadenvis Guettemer",
		["Gruna"] = "Gruna",
		["Syurana"] = "Syurana",
		["Vesprystus"] = "Vesprystus",
		["Caylais Moonfeather"] = "Caylais Plumelune",
		["Brakgul Deathbringer"] = "Brakgul Porte-mort",
		["Healing Stream Totem IV"] = "Totem guérisseur IV",
		["Healing Stream Totem V"] = "Totem guérisseur V",
		["Shandris Feathermoon"] = "Shandris Pennelune",
		["Magatha Grimtotem"] = "Magatha Totem-sinistre",
		["Zor Lonetree"] = "Zor Arbre-solitaire",
		["Rat"] = "Rat",
		["Arias'ta Bladesinger"] = "Arias'ta Chantelame",
		["Elanaria"] = "Elanaria",
		["Astarii Starseeker"] = "Astarii Cherchétoile",
		["Jandria"] = "Jandria",
		["Jeen'ra Nightrunner"] = "Jeen'ra Coursenuit",
		["Ainethil"] = "Ainethil",
		["Lysheana"] = "Lysheana",
		["Ealyshia Dewwhisper"] = "Ealyshia Murmerosée",
		["Geenia Sunshadow"] = "Geenia Ombresoleil",
		["Dorion"] = "Dorion",
		["Telonis"] = "Telonis",
		["Erion Shadewhisper"] = "Erion Murmombre",
		["Anishar"] = "Anishar",
		["Mathrengyl Bearwalker"] = "Mathrengyl Pasdelours",
		["Denatharion"] = "Denatharion",
		["Frostsaber Companion"] = "Compagnon sabre-de-givre",
		["Nightshade"] = "Belladone",
		["Brogus Thunderbrew"] = "Brogus Tonnebière",
		["Lana Thunderbrew"] = "Lana Tonnebière",
		["Darnassus Sentinel"] = "Sentinelle de Darnassus",
		["Daelyshia"] = "Daelyshia",
		["Gorm Grimtotem"] = "Gorm Totem-sinistre",
		["Cor Grimtotem"] = "Cor Totem-sinistre",
		["Tharm"] = "Tharm",
		["Gorkas"] = "Gorkas",
		["Nyse"] = "Nyse",
		["Thyssiana"] = "Thyssiana",
		["Baldruc"] = "Baldruc",
		["Strashaz Warrior"] = "Guerrier Strashaz",
		["Strashaz Serpent Guard"] = "Garde-serpent Strashaz",
		["Strashaz Myrmidon"] = "Myrmidon Strashaz",
		["Strashaz Sorceress"] = "Sorcière de Strashaz",
		["Strashaz Siren"] = "Sirène Strashaz",
		["Strashaz Hydra"] = "Hydre Strashaz",
		["Teloren"] = "Teloren",
		["Darnassian Protector"] = "Protecteur de Darnassus",
		["Vilebranch Scalper"] = "Scalpeur Vilebranche",
		["Vilebranch Soothsayer"] = "Devin Vilebranche",
		["Jade Sludge"] = "Limace de jade",
		["Emerald Ooze"] = "Limon émeraude",
		["Haunting Vision"] = "Vision cauchemardesque",
		["Rotting Cadaver"] = "Cadavre pourrissant",
		["Blighted Zombie"] = "Zombie corrompu",
		["Scarlet Avenger"] = "Vengeur écarlate",
		["Scarlet Spellbinder"] = "Lieur de sort écarlate",
		["Michael Garrett"] = "Michael Garrett",
		["Kaal Soulreaper"] = "Kaal Fauche-l'âme",
		["Luther Pickman"] = "Luther Pickman",
		["Kaelystia Hatebringer"] = "Kaelystia Porte-haine",
		["Pierce Shackleton"] = "Pierce Shackleton",
		["Miles Dexter"] = "Miles Dexter",
		["Gregory Charles"] = "Gregory Charles",
		["Christoph Walker"] = "Christoph Walker",
		["Angela Curthas"] = "Angela Curthas",
		["Aelthalyste"] = "Aelthalyste",
		["Father Lazarus"] = "Père Lazarus",
		["Doctor Herbert Halsey"] = "Docteur Herbert Halsey",
		["Christopher Drakul"] = "Christopher Drakul",
		["Booty Bay Bruiser"] = "Cogneur de Baie-du-Butin",
		["Kildar"] = "Kildar",
		["Truk Wildbeard"] = "Truk Barbe-hirsute",
		["Nazeer Bloodpike"] = "Nazeer Pique-sanglante",
		["Stone Keeper"] = "Gardien des pierres",
		["Guard Edward"] = "Garde Edward",
		["Guard Jarad"] = "Garde Jarad",
		["Combat Master Criton"] = "Maître de guerre Criton",
		["Captain Garran Vimes"] = "Capitaine Garran Vimaire",
		["Theramore Lieutenant"] = "Lieutenant de Theramore",
		["Thrall"] = "Thrall",
		["Theramore Practicing Guard"] = "Garde de Theramore à l'entraînement",
		["Lady Jaina Proudmoore"] = "Dame Jaina Portvaillant",
		["Guard Lasiter"] = "Garde Lasiter",
		["Aldwin Laughlin"] = "Aldwin Laughlin",
		["Theramore Guard"] = "Garde de Theramore",
		["Krumn"] = "Krumn",
		["Combat Master Szigeti"] = "Maître de Combat Szigeti",
		["Guard Kahil"] = "Garde Kahil",
		["Guard Lana"] = "Garde Lana",
		["Guard Narrisha"] = "Garde Narrisha",
		["Guard Tark"] = "Garde Tark",
		["Captain Andrews"] = "Capitaine Andrews",
		["Captain Thomas"] = "Capitaine Thomas",
		["Daera Brightspear"] = "Daera Brillelance",
		["Olmin Burningbeard"] = "Olmin Barbe-en-feu",
		["Brogun Stoneshield"] = "Brogun Targe-pierre",
		["Jondor Steelbrow"] = "Jondor Front-d'acier",
		["Jonivera Farmountain"] = "Jonivera Mont-lointain",
		["Svalbrad Farmountain"] = "Svalbrad Mont-lointain",
		["Kurdrum Barleybeard"] = "Kurdrum Barbe-d'orge",
		["Theodrus Frostbeard"] = "Theodrus Barbe-de-givre",
		["Braenna Flintcrag"] = "Braenna Combesilex",
		["Juli Stormkettle"] = "Juli Cyclobouilloire",
		["Nittlebur Sparkfizzle"] = "Nittegousse Pfutincelle",
		["Valgar Highforge"] = "Valgar Forge-haute",
		["Beldruk Doombrow"] = "Beldruk Ruinefront",
		["Brandur Ironhammer"] = "Brandur Martel-de-fer",
		["Grumnus Steelshaper"] = "Grumnus Sculptacier",
		["Hulfdan Blackbeard"] = "Hulfdan Barbe-noire",
		["Ormyr Flinteye"] = "Ormyr Oeil-de-quartz",
		["Briarthorn"] = "Eglantin",
		["Alexander Calder"] = "Alexander Calder",
		["Medic Tamberlyn"] = "Docteur Tamberlyn",
		["Medic Helaina"] = "Docteur Helaina",
		["Murk Spitter"] = "Cracheur des boues",
		["Murk Worm"] = "Ver des boues",
		["Saturated Ooze"] = "Limon saturé",
		["Gordunni Warlord"] = "Seigneur de guerre Gordunni",
		["Atal'ai Warrior"] = "Guerrier atal'ai",
		["Atal'ai Witch Doctor"] = "Sorcier-docteur atal'ai",
		["Groddoc Thunderer"] = "Foudroyeur Groddoc",
		["Mummified Atal'ai"] = "Atal'ai momifié",
		["Unliving Atal'ai"] = "Atal'ai non-vivant",
		["Atal'ai Priest"] = "Prêtre atal'ai",
		["Atal'ai Corpse Eater"] = "Charognard atal'ai",
		["Atal'ai Deathwalker"] = "Marche-mort atal'ai",
		["Atal'ai High Priest"] = "Grand prêtre atal'ai",
		["Ironfur Patriarch"] = "Patriarche Ferpoil",
		["Sprite Dragon"] = "Petit dragon féerique",
		["Nightmare Scalebane"] = "Cavale de cauchemar Plaiedécaille",
		["Nightmare Wyrmkin"] = "Cavale de cauchemar wyrmide",
		["Nightmare Wanderer"] = "Cavale de cauchemar errante",
		["Rabid Longtooth"] = "Longues-dents enragé",
		["Hakkari Frostwing"] = "Aile-de-givre Hakkari",
		["Rage Scar Yeti"] = "Yéti Griffe féroce",
		["Elder Rage Scar"] = "Ancien Griffe féroce",
		["Ferocious Rage Scar"] = "Griffe féroce enragé",
		["Frayfeather Patriarch"] = "Patriarche Aigreplume",
		["Lethlas"] = "Lethlas",
		["Phantim"] = "Phantim",
		["Jademir Oracle"] = "Oracle Jademir",
		["Jademir Tree Warder"] = "Gardien sylvestre Jademir",
		["Jademir Boughguard"] = "Gardien des marais Jademir",
		["Lady Szallah"] = "Dame Szallah",
		["Diamond Head"] = "Tête-de-diamant",
		["Bloodroar the Stalker"] = "Rugissang le Traqueur",
		["Antilus the Soarer"] = "Antilus le Planeur",
		["Arash-ethis"] = "Arash-ethis",
		["Qirot"] = "Qirot",
		["Land Walker"] = "Marcheur des terres",
		["Cliff Giant"] = "Géant des falaises",
		["Shore Strider"] = "Marcheur des rivages",
		["Deep Strider"] = "Marcheur des profondeurs",
		["Wave Strider"] = "Marcheur des vagues",
		["Northspring Harpy"] = "Harpie Ruissenord",
		["Northspring Roguefeather"] = "Volplume Ruissenord",
		["Northspring Slayer"] = "Pourfendeuse Ruissenord",
		["Northspring Windcaller"] = "Imploratrice céleste Ruissenord",
		["High Explorer Magellas"] = "Grand Explorateur Magellas",
		["Sage Palerunner"] = "Sage Coureur-pâle",
		["Quartermaster Lungertz"] = "Intendant Lungertz",
		["Veyzhak the Cannibal"] = "Veyzhak le Cannibale",
		["Zekkis"] = "Zekkis",
		["Kazkaz the Unholy"] = "Kazkaz l'Impie",
		["Glasshide Petrifier"] = "Pétrificateur Peau-de-verre",
		["Scorpid Dunestalker"] = "Scorpide des sables",
		["Rabid Blisterpaw"] = "Brûlepatte enragée",
		["Searing Roc"] = "Roc incendiaire",
		["Surf Glider"] = "Glisseur",
		["Giant Surf Glider"] = "Glisseur géant",
		["Coral Shark"] = "Requin du corail",
		["Hazzali Wasp"] = "Guêpe Hazzali",
		["Hazzali Stinger"] = "Dard Hazzali",
		["Hazzali Swarmer"] = "Traînard Hazzali",
		["Hazzali Worker"] = "Ouvrier Hazzali",
		["Hazzali Tunneler"] = "Tunnelière Hazzali",
		["Hazzali Sandreaver"] = "Saccageur des sables Hazzali",
		["Centipaar Wasp"] = "Guêpe Centipaar",
		["Centipaar Stinger"] = "Dard Centipaar",
		["Centipaar Swarmer"] = "Traînard Centipaar",
		["Centipaar Worker"] = "Ouvrier Centipaar",
		["Centipaar Tunneler"] = "Tunnelière centipaar",
		["Centipaar Sandreaver"] = "Saccageur des sables Centipaar",
		["Sea Elemental"] = "Elémentaire marin",
		["Sea Spray"] = "Embruns",
		["Coast Strider"] = "Marcheur des côtes",
		["Deep Dweller"] = "Etre des profondeurs",
		["Dune Smasher"] = "Compresseur",
		["Raging Dune Smasher"] = "Compresseur déchaîné",
		["Dunemaul Enforcer"] = "Massacreur Cognedune",
		["Dunemaul Ogre Mage"] = "Ogre-mage Cognedune",
		["Dunemaul Brute"] = "Brute Cognedune",
		["Dunemaul Warlock"] = "Démoniste Cognedune",
		["Wu Shen"] = "Wu Shen",
		["Thistleshrub Dew Collector"] = "Chardonnier collecteur de rosée",
		["Thistleshrub Rootshaper"] = "Chardonnier taille-racine",
		["Brother Joshua"] = "Frère Joshua",
		["Gnarled Thistleshrub"] = "Chardonnier noueux",
		["Katherine the Pure"] = "Katherine la Pure",
		["Sandahl"] = "Sandahl",
		["Elsharin"] = "Elsharin",
		["Kaerbrus"] = "Kaerbrus",
		["Sheldras Moontree"] = "Sheldras Arbrelune",
		["Theridran"] = "Theridran",
		["Einris Brightspear"] = "Einris Brillelance",
		["Ulfir Ironbeard"] = "Ulfir Barbe-de-fer",
		["Grunt Zuul"] = "Grunt Zuul",
		["Grunt Tharlak"] = "Grunt Tharlak",
		["Bruuk Barleybeard"] = "Bruuk Barbe-d'orge",
		["Ironforge Guard"] = "Garde d'Ironforge",
		["Undercity Guardian"] = "Garde d'Undercity",
		["Falstad Wildhammer"] = "Falstad Marteau-hardi",
		["Gunther's Visage"] = "Visage de Gunther",
		["Carendin Halgar"] = "Carendin Halgar",
		["High Sorcerer Andromath"] = "Grand sorcier Andromath",
		["Spawn of Hakkar"] = "Rejeton d'Hakkar",
		["Shade of Eranikus"] = "Ombre d'Eranikus",
		["Jammal'an the Prophet"] = "Jammal'an le prophète",
		["Ogom the Wretched"] = "Ogom le Misérable",
		["Zolo"] = "Zolo",
		["Gasher"] = "Balafreur",
		["Loro"] = "Loro",
		["Hukku"] = "Hukku",
		["Zul'Lor"] = "Zul'Lor",
		["Mijan"] = "Mijan",
		["Rothos"] = "Rothos",
		["Morphaz"] = "Morphaz",
		["Weaver"] = "Tisserand",
		["Dreamscythe"] = "Fauche-rêve",
		["Hazzas"] = "Hazzas",
		["Guardian of Blizzard"] = "Garde du Blizzard",
		["Arch Druid Hamuul Runetotem"] = "Archidruide Hamuul Totem-runique",
		["Nara Wildmane"] = "Nara Crin-sauvage",
		["Crildor"] = "Crildor",
		["Margol the Rager"] = "Margol l'Enragée",
		["Dark Iron Steamsmith"] = "Forge-vapeur Sombrefer",
		["Dark Iron Taskmaster"] = "Sous-chef Sombrefer",
		["Inferno Elemental"] = "Elémentaire infernal",
		["Heavy War Golem"] = "Golem de guerre lourd",
		["Magma Elemental"] = "Elémentaire de magma",
		["Greater Lava Spider"] = "Grande araignée de lave",
		["Twilight Dark Shaman"] = "Chaman noir du crépuscule",
		["Twilight Fire Guard"] = "Gardefeu du crépuscule",
		["Twilight Geomancer"] = "Géomancien du crépuscule",
		["Pephredo"] = "Pephredo",
		["Enyo"] = "Enyo",
		["Tremor Totem"] = "Totem de séisme",
		["Den Grunt"] = "Grunt de la tanière",
		["Dreadmaul Ogre Mage"] = "Ogre-mage Cognepeur",
		["Dreadmaul Brute"] = "Brute Cognepeur",
		["Dreadmaul Mauler"] = "Marteleur Cognepeur",
		["Dreadmaul Warlock"] = "Démoniste Cognepeur",
		["Portal Seeker"] = "Chercheur du portail",
		["Black Slayer"] = "Pourfendeur noir",
		["Bonepicker"] = "Racleur-d'os",
		["Snickerfang Hyena"] = "Hyène Raillecroc",
		["Scorpok Stinger"] = "Aiguillonneur scorpok",
		["Redstone Basilisk"] = "Basilic Rougepierre",
		["Redstone Crystalhide"] = "Peau de cristal Rougepierre",
		["Ashmane Boar"] = "Sanglier cendré",
		["Helboar"] = "Sanglier infernal",
		["Zayus"] = "Zayus",
		["Nethergarde Miner"] = "Mineur de Rempart-du-Néant",
		["Nethergarde Engineer"] = "Ingénieur de Rempart-du-Néant",
		["Nethergarde Foreman"] = "Contremaître de Rempart-du-Néant",
		["Nethergarde Soldier"] = "Soldat de Rempart-du-Néant",
		["Nethergarde Cleric"] = "Clerc de Rempart-du-Néant",
		["Nethergarde Analyst"] = "Analyste de Rempart-du-Néant",
		["Nethergarde Riftwatcher"] = "Guette-faille de Rempart-du-Néant",
		["Nethergarde Officer"] = "Officier de Rempart-du-Néant",
		["Shadowsworn Cultist"] = "Sectateur lige d'ombre",
		["Shadowsworn Thug"] = "Nervi lige d'ombre",
		["Shadowsworn Adept"] = "Adepte lige d'ombre",
		["Shadowsworn Enforcer"] = "Massacreur lige d'ombre",
		["Shadowsworn Warlock"] = "Démoniste lige d'ombre",
		["Shadowsworn Dreadweaver"] = "Lige d'ombre tisseur d?effroi",
		["Felhound"] = "Gangrechien",
		["Felguard Sentry"] = "Gangregarde sentinelle",
		["Flametongue Totem II"] = "Totem Langue de feu II",
		["X'yera"] = "X'yera",
		["Elemental Protection Totem"] = "Totem de protection élémentaire",
		["Breyk"] = "Breyk",
		["Earthgrab Totem"] = "Totem de poigne de terre",
		["Harry Burlguard"] = "Harry Burlguard",
		["Azuregos"] = "Azuregos",
		["Muren Stormpike"] = "Muren Foudrepique",
		["Highborne Lichling"] = "Jeune liche bien-née",
		["Varo'then's Ghost"] = "Fantôme de Varo'then",
		["Gakin the Darkbinder"] = "Gakin le Tisse-noir",
		["Haldarr Felsworn"] = "Gangrelige Haldarr",
		["Draconic Magelord"] = "Seigneur-mage draconique",
		["Blue Scalebane"] = "Plaiedécaille bleu",
		["Draconic Mageweaver"] = "Tisse-magie draconique",
		["Lord Arkkoroc"] = "Seigneur Arkkoroc",
		["Arkkoran Clacker"] = "Claqueur Arkkoran",
		["Arkkoran Muckdweller"] = "Habitant du bourbier Arkkoran",
		["Arkkoran Pincer"] = "Pinceur Arkkoran",
		["Arkkoran Oracle"] = "Oracle Arkkoran",
		["Hetaera"] = "Hetaera",
		["Servant of Arkkoroc"] = "Servant d?Arkkoroc",
		["Son of Arkkoroc"] = "Fils d?Arkkoroc",
		["Cliff Breaker"] = "Brise-falaises",
		["Cliff Thunderer"] = "Foudroyeur des falaises",
		["Cliff Walker"] = "Marcheur des falaises",
		["Duthorian Rall"] = "Duthorian Rall",
		["Tiza Battleforge"] = "Tiza Forgeguerre",
		["Timbermaw Pathfinder"] = "Guide Grumegueule",
		["Timbermaw Warrior"] = "Guerrier Grumegueule",
		["Timbermaw Totemic"] = "Totémique Grumegueule",
		["Timbermaw Den Watcher"] = "Protecteur Grumegueule",
		["Timbermaw Shaman"] = "Chaman Grumegueule",
		["Timbermaw Ursa"] = "Ursa Grumegueule",
		["Spitelash Warrior"] = "Guerrier Fouette-bile",
		["Spitelash Screamer"] = "Hurleuse Fouette-bile",
		["Spitelash Serpent Guard"] = "Garde-serpent Fouette-bile",
		["Spitelash Siren"] = "Sirène Fouette-bile",
		["Spitelash Myrmidon"] = "Myrmidon Fouette-bile",
		["Blood Elf Surveyor"] = "Géomètre elfe de sang",
		["Blood Elf Reclaimer"] = "Vengeur elfe de sang",
		["Legashi Satyr"] = "Satyre Legashi",
		["Legashi Rogue"] = "Voleur Legashi",
		["Legashi Hellcaller"] = "Prêtre noir Legashi",
		["Klannoc Macleod"] = "Klannoc Macleod",
		["Strahad Farsan"] = "Strahad Farsan",
		["Menara Voidrender"] = "Menara Taillevide",
		["Young Wavethrasher"] = "Jeune Vandale des flots",
		["Wavethrasher"] = "Vandale des flots",
		["Great Wavethrasher"] = "Grand Vandale des flots",
		["Makrinni Razorclaw"] = "Pince-tranchante Makrinni",
		["Storm Bay Oracle"] = "Oracle de la baie des tempêtes",
		["Coralshell Lurker"] = "Rôdeuse corail",
		["Coralshell Tortoise"] = "Tortue corail",
		["Makrinni Scrabbler"] = "Fouilleur Makrinni",
		["Storm Bay Warrior"] = "Guerrier de la baie des tempêtes",
		["Makrinni Snapclaw"] = "Claque-pinces Makrinni",
		["Thunderhead Hippogryph"] = "Hippogriffe Tête-tonnerre",
		["Thunderhead Stagwing"] = "Cerf ailé Tête-tonnerre",
		["Thunderhead Skystormer"] = "Tempétueux Tête-tonnerre",
		["Thunderhead Patriarch"] = "Patriarche Tête-tonnerre",
		["Thunderhead Consort"] = "Concubine Tête-tonnerre",
		["Jubahl Corpseseeker"] = "Jubahl Cherchecadavre",
		["Ward of Zanzil"] = "Garde de Zanzil",
		["Henen Ragetotem"] = "Henen Totem-de-rage",
		["Sergeant Rutger"] = "Sergent Rutger",
		["Spirit Healer"] = "Gardien des âmes",
		["Devilsaur"] = "Diablosaure",
		["Ironhide Devilsaur"] = "Diablosaure Cuiracier",
		["Tyrant Devilsaur"] = "Diablosaure Rex",
		["Stegodon"] = "Stégodon",
		["Plated Stegodon"] = "Stégodon cuirassé",
		["Spiked Stegodon"] = "Stégodon à pointes",
		["Thunderstomp Stegodon"] = "Stégodon Grondeterre",
		["Ravasaur"] = "Ravasaure",
		["Ravasaur Runner"] = "Ravasaure coureur",
		["Ravasaur Hunter"] = "Ravasaure chasseur",
		["Venomhide Ravasaur"] = "Ravasaure peau-de-venin",
		["Bloodpetal Lasher"] = "Flagellant Pétale-de-sang",
		["Bloodpetal Flayer"] = "Ecorcheur Pétale-de-sang",
		["Bloodpetal Thresher"] = "Batteur Pétale-de-sang",
		["Bloodpetal Trapper"] = "Trappeur Pétale-de-sang",
		["Un'Goro Stomper"] = "Marteleur d'Un'Goro",
		["Un'Goro Gorilla"] = "Gorille d'Un'Goro",
		["Un'Goro Thunderer"] = "Foudroyeur d'Un'Goro",
		["Tar Beast"] = "Bête du goudron",
		["Tar Lurker"] = "Rôdeur du goudron",
		["Tar Lord"] = "Seigneur du goudron",
		["Scorching Elemental"] = "Elémentaire de brûlure",
		["Living Blaze"] = "Brasier vivant",
		["Tar Creeper"] = "Rampeur du goudron",
		["Tabetha"] = "Tabetha",
		["Magus Tirth"] = "Magus Tirth",
		["Gorishi Wasp"] = "Guêpe Gorishi",
		["Gorishi Worker"] = "Ouvrier Gorishi",
		["Gorishi Reaver"] = "Saccageur Gorishi",
		["Gorishi Stinger"] = "Dard Gorishi",
		["Gorishi Tunneler"] = "Tunnelière Gorishi",
		["Muculent Ooze"] = "Limon de mucus",
		["Primal Ooze"] = "Limon primitif",
		["Glutinous Ooze"] = "Limon gluant",
		["Stone Guardian"] = "Garde de pierre",
		["Vizzklick"] = "Vizzklick",
		["Ravasaur Matriarch"] = "Matriarche ravasaure",
		["Clutchmother Zavas"] = "Matriarche Zavas",
		["Gruff"] = "Gruff",
		["King Mosh"] = "Roi Mosh",
		["Uhk'loc"] = "Uhk'loc",
		["Monnos the Elder"] = "Monnos l?Ancien",
		["Magister Hawkhelm"] = "Magistère Falcoiffe",
		["Antilos"] = "Antilos",
		["Lady Sesspira"] = "Dame Sesspira",
		["General Fangferror"] = "Général Crocdangoiffe",
		["Gatekeeper Rageroar"] = "Portier Hurlerage",
		["Master Feardred"] = "Maître Trouilleffroi",
		["Baritanas Skyriver"] = "Baritanas Ruisseciel",
		["Fahrad"] = "Fahrad",
		["Thalon"] = "Thalon",
		["Ravenholdt Guard"] = "Garde de Ravenholdt",
		["Lord Jorach Ravenholdt"] = "Seigneur Jorach Ravenholdt",
		["Ravenholdt Assassin"] = "Assassin de Ravenholdt",
		["Zan Shivsproket"] = "Zan Tympeustache",
		["Innkeeper Skindle"] = "Aubergiste Tôlardier",
		["Talvash del Kissel"] = "Talvash del Kissel",
		["Yalda"] = "Yalda",
		["Blackrock Soldier"] = "Soldat Rochenoire",
		["Blackrock Sorcerer"] = "Ensorceleur Rochenoire",
		["Blackrock Slayer"] = "Pourfendeur Rochenoire",
		["Blackrock Warlock"] = "Démoniste Rochenoire",
		["Blackrock Battlemaster"] = "Maître de guerre Rochenoire",
		["Obsidian Elemental"] = "Elémentaire d'obsidienne",
		["Greater Obsidian Elemental"] = "Grand élémentaire d'obsidienne",
		["Firegut Ogre"] = "Ogre Brûlentrailles",
		["Firegut Ogre Mage"] = "Ogre-mage Brûlentrailles",
		["Firegut Brute"] = "Brute Brûlentrailles",
		["Thaurissan Spy"] = "Espion de Thaurissan",
		["Thaurissan Firewalker"] = "Marchefeu de Thaurissan",
		["Thaurissan Agent"] = "Agent de Thaurissan",
		["War Reaver"] = "Saccageur de guerre",
		["Black Dragonspawn"] = "Draconide noir",
		["Black Wyrmkin"] = "Wyrmide noir",
		["Flamescale Dragonspawn"] = "Draconide Flammécaille",
		["Flamescale Wyrmkin"] = "Wyrmide Flammécaille",
		["Black Drake"] = "Drake noir",
		["Scalding Drake"] = "Drake brûlant",
		["Searscale Drake"] = "Drake torride",
		["Black Broodling"] = "Rejeton noir",
		["Scalding Broodling"] = "Rejeton brûlant",
		["Flamescale Broodling"] = "Rejeton Flammécaille",
		["Blackrock Worg"] = "Worg Rochenoire",
		["Condemned Acolyte"] = "Acolyte condamné",
		["Condemned Monk"] = "Moine condamné",
		["Condemned Cleric"] = "Clerc condamné",
		["Cursed Paladin"] = "Paladin maudit",
		["Cursed Justicar"] = "Justicier maudit",
		["Writhing Mage"] = "Mage frémissant",
		["Cursed Ooze"] = "Limon maudit",
		["Tainted Ooze"] = "Limon corrompu",
		["Vile Ooze"] = "Vil limon",
		["Ironbeak Owl"] = "Chouette Bec-de-fer",
		["Ironbeak Screecher"] = "Hurleur Bec-de-fer",
		["Ironbeak Hunter"] = "Chasseur Bec-de-fer",
		["Warpwood Moss Flayer"] = "Ecorcheur Crochebois",
		["Warpwood Shredder"] = "Déchireur Crochebois",
		["Dessecus"] = "Dessecus",
		["Jadefire Satyr"] = "Satyre Jadefeu",
		["Jadefire Rogue"] = "Voleur Jadefeu",
		["Jadefire Trickster"] = "Entourloupeur Jadefeu",
		["Jadefire Betrayer"] = "Traître Jadefeu",
		["Jadefire Felsworn"] = "Gangrelige Jadefeu",
		["Jadefire Shadowstalker"] = "Traqueur des ombres Jadefeu",
		["Jadefire Hellcaller"] = "Implorateur de l'enfer Jadefeu",
		["Jaedenar Cultist"] = "Sectateur de Jaedenar",
		["Jaedenar Guardian"] = "Gardien de Jaedenar",
		["Jaedenar Enforcer"] = "Massacreur de Jaedenar",
		["Jaedenar Adept"] = "Adepte de Jaedenar",
		["Jaedenar Darkweaver"] = "Tisseur d'ombre de Jaedenar",
		["Jaedenar Warlock"] = "Démoniste de Jaedenar",
		["Jaedenar Hound"] = "Molosse de Jaedenar",
		["Jaedenar Hunter"] = "Chasseur de Jaedenar",
		["Toxic Horror"] = "Horreur toxique",
		["Infernal Bodyguard"] = "Garde du corps infernal",
		["Infernal Sentry"] = "Infernal Sentinelle",
		["Immolatus"] = "Immolatus",
		["Irontree Wanderer"] = "Arbrefer errant",
		["Irontree Stomper"] = "Arbrefer marteleur",
		["Withered Protector"] = "Protecteur flétri",
		["Deadwood Warrior"] = "Guerrier Mort-bois",
		["Deadwood Gardener"] = "Jardinier Mort-bois",
		["Deadwood Pathfinder"] = "Guide Mort-bois",
		["Deadwood Den Watcher"] = "Protecteur Mort-bois",
		["Deadwood Avenger"] = "Vengeur Mort-bois",
		["Deadwood Shaman"] = "Chaman Mort-bois",
		["Lore Keeper of Norgannon"] = "Gardien du savoir de Norgannon",
		["Shayis Steelfury"] = "Shayis Furie-d'acier",
		["Kelgruk Bloodaxe"] = "Kelgruk Hache-sanglante",
		["Borgus Steelhand"] = "Borgus Main-d'acier",
		["Ember"] = "Braise",
		["Chief Ukorz Sandscalp"] = "Chef Ukorz Scalpessable",
		["Witch Doctor Zum'rah"] = "Sorcier-docteur Zum'rah",
		["Gahz'rilla"] = "Gahz'rilla",
		["Sandfury Executioner"] = "Bourreau Furie-des-sables",
		["Shadowpriest Sezz'ziz"] = "Prêtre des ombres Sezz'ziz",
		["Darnath Bladesinger"] = "Darnath Chantelame",
		["Winstone Wolfe"] = "Winstone Wolfe",
		["Master Kang"] = "Maître Kang",
		["Kum'isha the Collector"] = "Kum'isha le collecteur",
		["Flawless Draenethyst Sphere"] = "Sphère de draenéthyste sans défaut",
		["Stoneskin Totem VI"] = "Totem de peau de pierre VI",
		["Deadwind Brute"] = "Brute de Deuillevent",
		["Restless Shade"] = "Ame errante",
		["Deadwind Mauler"] = "Marteleur de Deuillevent",
		["Deadwind Warlock"] = "Démoniste de Deuillevent",
		["Sky Shadow"] = "Ombre céleste",
		["Deadwind Ogre Mage"] = "Ogre-mage de Deuillevent",
		["Stoneclaw Totem V"] = "Totem de griffes de pierre V",
		["Stoneclaw Totem VI"] = "Totem de griffes de pierre VI",
		["Searing Totem V"] = "Totem incendiaire V",
		["Searing Totem VI"] = "Totem incendiaire VI",
		["Strength of Earth Totem IV"] = "Totem de force de la Terre IV",
		["Oglethorpe Obnoticus"] = "Orguignard Odipnoticus",
		["Chief Engineer Bilgewhizzle"] = "Ingénieur en chef Vizisanie",
		["Thelman Slatefist"] = "Thelman Poing-d'ardoise",
		["Frost Resistance Totem II"] = "Totem de résistance au Givre II",
		["Frost Resistance Totem III"] = "Totem de résistance au Givre III",
		["Mana Spring Totem III"] = "Totem Fontaine de mana III",
		["Mana Spring Totem IV"] = "Totem Fontaine de mana IV",
		["Flametongue Totem III"] = "Totem Langue de feu III",
		["Fire Resistance Totem II"] = "Totem de résistance au Feu II",
		["Fire Resistance Totem III"] = "Totem de résistance au Feu III",
		["Taim Ragetotem"] = "Taim Totem-de-rage",
		["Frostmaul Giant"] = "Géant Cognegivre",
		["Frostmaul Preserver"] = "Protecteur Cognegivre",
		["Frostsaber Cub"] = "Jeune sabre-de-givre",
		["Frostsaber"] = "Sabre-de-givre",
		["Frostsaber Stalker"] = "Traqueur sabre-de-givre",
		["Frostsaber Huntress"] = "Chasseresse sabre-de-givre",
		["Frostsaber Pride Watcher"] = "Guetteur sabre-de-givre",
		["Cobalt Wyrmkin"] = "Wyrmide cobalt",
		["Cobalt Scalebane"] = "Plaiedécaille cobalt",
		["Cobalt Mageweaver"] = "Tisse-magie cobalt",
		["Winterfall Ursa"] = "Ursa Tombe-hiver",
		["Winterfall Shaman"] = "Chaman Tombe-hiver",
		["Winterfall Den Watcher"] = "Protecteur Tombe-hiver",
		["Winterfall Totemic"] = "Totémique Tombe-hiver",
		["Winterfall Pathfinder"] = "Guide Tombe-hiver",
		["Shardtooth Mauler"] = "Marteleur Crocs acérés",
		["Shardtooth Bear"] = "Ours Crocs acérés",
		["Elder Shardtooth"] = "Ancien Croc acéré",
		["Rabid Shardtooth"] = "Croc acéré enragé",
		["Fledgling Chillwind"] = "Petite Noroît",
		["Chillwind Chimaera"] = "Chimère Noroît",
		["Chillwind Ravager"] = "Ravageur Noroît",
		["Ragged Owlbeast"] = "Chouettard éreinté",
		["Raging Owlbeast"] = "Chouettard déchaîné",
		["Crazed Owlbeast"] = "Chouettard affolé",
		["Moontouched Owlbeast"] = "Chouettard lunaire",
		["Berserk Owlbeast"] = "Chouettard berserk",
		["Winterspring Owl"] = "Chouette du Berceau-de-l'Hiver",
		["Winterspring Screecher"] = "Hurleur du Berceau-de-l'Hiver",
		["Rogue Ice Thistle"] = "Chardon de glace insoumis",
		["Ice Thistle Yeti"] = "Yéti Chardon de glace",
		["Ice Thistle Matriarch"] = "Matriarche Chardon de glace",
		["Ice Thistle Patriarch"] = "Patriarche Chardon de glace",
		["Hederine Initiate"] = "Initiée Hederine",
		["Hederine Manastalker"] = "Traque-mana Hederine",
		["Hederine Slayer"] = "Pourfendeur Hederine",
		["Magma Totem III"] = "Totem de magma III",
		["Magma Totem IV"] = "Totem de magma IV",
		["Nature Resistance Totem II"] = "Totem de résistance à la Nature II",
		["Nature Resistance Totem III"] = "Totem de résistance à la Nature III",
		["Windfury Totem II"] = "Totem Furie-des-vents II",
		["Windfury Totem III"] = "Totem Furie-des-vents III",
		["Grace of Air Totem II"] = "Totem de Grâce aérienne II",
		["Bloodmage Drazial"] = "Mage de sang Drazial",
		["Bloodmage Lynnore"] = "Mage de sang Lynnore",
		["Suffering Highborne"] = "Bien-né souffrant",
		["Anguished Highborne"] = "Bien-née angoissée",
		["Fallen Hero of the Horde"] = "Héros de la Horde tombé au combat",
		["Dispatch Commander Ruag"] = "Commandant d'ordonnance Ruag",
		["Razelikh the Defiler"] = "Razelikh le Souilleur",
		["Grol the Destroyer"] = "Grol le Destructeur",
		["Archmage Allistarj"] = "Archimage Allistarj",
		["Lady Sevine"] = "Dame Sevine",
		["Servant of Razelikh"] = "Serviteur de Razelikh",
		["Servant of Grol"] = "Serviteur de Grol",
		["Servant of Allistarj"] = "Serviteur d'Allistarj",
		["Servant of Sevine"] = "Serviteur de Sevine",
		["Kirith the Damned"] = "Kirith le Damné",
		["Spirit of Kirith"] = "Esprit de Kirith",
		["Ilifar"] = "Ilifar",
		["Felcular"] = "Gangrénaire",
		["Gracina Spiritmight"] = "Gracina Forcesprit",
		["Corporal Thund Splithoof"] = "Caporal Thund Sabot-fendu",
		["Curgle Cranklehop"] = "Gruzouille Sautelevier",
		["Troyas Moonbreeze"] = "Troyas Lunebise",
		["Kalin Windflight"] = "Kalin Volevent",
		["Marli Wishrunner"] = "Marli Coursevise",
		["Shay Leafrunner"] = "Shay Coursefeuille",
		["Talo Thornhoof"] = "Talo Sabot-de-ronce",
		["Rin'ji"] = "Rin'ji",
		["Loramus Thalipedes"] = "Loramus Thalipedes",
		["Ward of Zum'rah"] = "Garde de Zum'rah",
		["Aturk the Anvil"] = "Aturk l'Enclume",
		["Ox"] = "Boeuf",
		["McGavan"] = "McGavan",
		["Hydromancer Velratha"] = "Hydromancienne Velratha",
		["Ruuzlu"] = "Ruuzlu",
		["Hank the Hammer"] = "Hank le Marteau",
		["Galvan the Ancient"] = "Galvan l'Ancien",
		["Scorpid Duneburrower"] = "Scorpide fouisseur de dune",
		["Trenton Lighthammer"] = "Trenton Martelume",
		["Marauding Owlbeast"] = "Chouettard maraudeur",
		["Vilebranch Ambusher"] = "Embusqué Vilebranche",
		["Bera Stonehammer"] = "Bera Martel-de-pierre",
		["Bulkrek Ragefist"] = "Bulkrek Ragepoing",
		["Oran Snakewrithe"] = "Oran Serpenroule",
		["Ambassador Ardalan"] = "Ambassadeur Ardalan",
		["Fire Nova Totem IV"] = "Totem Nova de feu IV",
		["Fire Nova Totem V"] = "Totem Nova de feu V",
		["Teremus the Devourer"] = "Teremus le Dévoreur",
		["Caliph Scorpidsting"] = "Caliph Dard-de-scorpide",
		["Lurking Feral Scar"] = "Griffe farouche patibulaire",
		["Nethergarde Elite"] = "Elite de Rempart-du-Néant",
		["Pratt McGrubben"] = "Pratt McGrubben",
		["Jangdor Swiftstrider"] = "Jangdor Rôdeur-agile",
		["Lingering Highborne"] = "Bien-né survivant",
		["Wildhammer Sentry"] = "Sentinelle marteau-hardi",
		["Peter Galen"] = "Peter Galen",
		["Thorkaf Dragoneye"] = "Thorkaf Oeil-de-dragon",
		["Sarah Tanner"] = "Sarah Tanner",
		["Brumn Winterhoof"] = "Brumn Sabot-d'hiver",
		["Caryssia Moonhunter"] = "Caryssia Chasselune",
		["Se'Jib"] = "Se'Jib",
		["Latronicus Moonspear"] = "Latronicus Lancelune",
		["Vestia Moonspear"] = "Vestia Lancelune",
		["Quintis Jonespyre"] = "Quintis Gentecime",
		["Ginro Hearthkindle"] = "Ginro Attisâtre",
		["Security Chief Bilgewhizzle"] = "Chef de la sécurité Vizisanie",
		["Fraggar Thundermantle"] = "Fraggar Tonnemante",
		["Spitelash Battlemaster"] = "Maître de guerre Fouette-bile",
		["Spitelash Enchantress"] = "Enchanteresse Fouette-bile",
		["Angelas Moonbreeze"] = "Angelas Lunebise",
		["Brother Sarno"] = "Frère Sarno",
		["Stone Watcher of Norgannon"] = "Gardien des pierres de Norgannon",
		["High Tinker Mekkatorque"] = "Grand Bricoleur Mekkanivelle",
		["Feathermoon Sentinel"] = "Sentinelle de Pennelune",
		["Darnall"] = "Darnall",
		["Mardrack Greenwell"] = "Mardrack Puisevert",
		["Faralorn"] = "Faralorn",
		["Harklane"] = "Harklane",
		["Tinkmaster Overspark"] = "Maître-bricoleur Suprétincelle",
		["Savanne"] = "Savanne",
		["Brannock"] = "Brannock",
		["Vivianna"] = "Vivianna",
		["Kylanna Windwhisper"] = "Kylanna Murmure-du-vent",
		["Xylinnia Starshine"] = "Xylinnia Luisétoile",
		["Master Mechanic Castpipe"] = "Maître mécanicien Fontuyau",
		["Xar'Ti"] = "Xar'Ti",
		["Binjy Featherwhistle"] = "Binjy Siffleplume",
		["Milli Featherwhistle"] = "Milli Siffleplume",
		["Mulgore Protector"] = "Protecteur de Mulgore",
		["Gammerita"] = "Gammerita",
		["Deathguard Elite"] = "Nécrogarde d'élite",
		["Vile Priestess Hexx"] = "Vile prêtresse Maléficia",
		["Qiaga the Keeper"] = "Qiaga la Gardienne",
		["Tyrande Whisperwind"] = "Tyrande Murmevent",
		["Sen'jin Guardian"] = "Gardien de Sen'jin",
		["Guthrum Thunderfist"] = "Guthrum Tonnepoing",
		["Fyldren Moonfeather"] = "Fyldren Plumelune",
		["Shyn"] = "Shyn",
		["Orwin Gizzmick"] = "Orwin Azztuche",
		["Thadius Grimshade"] = "Thadius Sinissombre",
		["Sharpbeak's Father"] = "Père de Bec-tranchant",
		["Sharpbeak's Mother"] = "Mère de Bec-tranchant",
		["Thyn'tel Bladeweaver"] = "Thyn'tel Tisselame",
		["Edana Hatetalon"] = "Edana Serrehaine",
		["Witch Doctor Uzer'i"] = "Sorcier-docteur Uzer'i",
		["Ziggle Sparks"] = "Ziggle Escarbilles",
		["Sul'lithuz Abomination"] = "Abomination Sul'lithuz",
		["Rickle Goldgrubber"] = "Rickle Fouinelor",
		["Nixx Sprocketspring"] = "Nixx Elastirouage",
		["Antu'sul"] = "Antu'sul",
		["Lord Shalzaru"] = "Seigneur Shalzaru",
		["Jannos Lighthoof"] = "Jannos Sabot-léger",
		["Loorana"] = "Loorana",
		["Kulleg Stonehorn"] = "Kulleg Corne-de-pierre",
		["Sheendra Tallgrass"] = "Sheendra Hautes-herbes",
		["Ruw"] = "Ruw",
		["Camp Mojache Brave"] = "Brave du Camp Mojache",
		["Nijel's Point Guard"] = "Garde de la Combe de Nijel",
		["Ghost Walker Brave"] = "Brave de Rôdeur-fantôme",
		["Kargath Grunt"] = "Grunt de Kargath",
		["Servant of Antu'sul"] = "Serviteur d'Antu'sul",
		["Logannas"] = "Logannas",
		["Bronk"] = "Bronk",
		["Worb Strongstitch"] = "Worb Point-solide",
		["Nioma"] = "Nioma",
		["Harggan"] = "Harggan",
		["Nina Lightbrew"] = "Nina Bouillelume",
		["Occulus"] = "Occulus",
		["Chronalis"] = "Chronalis",
		["Tick"] = "Tique",
		["Jin'Zallah the Sandbringer"] = "Jin'Zallah Porte-sable",
		["Omgorn the Lost"] = "Omgorn l'Egaré",
		["Cyclok the Mad"] = "Cyclok le Fol",
		["Kregg Keelhaul"] = "Kregg Soulaquille",
		["Soriid the Devourer"] = "Soriid le Dévoreur",
		["Haarka the Ravenous"] = "Haarka le Féroce",
		["Greater Firebird"] = "Grand oiseau de feu",
		["The Reak"] = "Le Jonc",
		["Ironback"] = "Dos-de-fer",
		["Jalinde Summerdrake"] = "Jalinde Drake-d'été",
		["Grimungous"] = "Grimungous",
		["Retherokk the Berserker"] = "Retherokk le Berserker",
		["Mith'rethis the Enchanter"] = "Mith'rethis l'Enchanteur",
		["Rekk'tilac"] = "Rekk'tilac",
		["Smoldar"] = "Fumar",
		["Faulty War Golem"] = "Golem de guerre défaillant",
		["Shleipnarr"] = "Shleipnarr",
		["Scald"] = "Brûlar",
		["Highlord Mastrogonde"] = "Généralissime Mastrogonde",
		["Slave Master Blackheart"] = "Maître des esclaves Coeur-noir",
		["Mojo the Twisted"] = "Mojo le Tordu",
		["Magronos the Unyielding"] = "Magronos l'Inflexible",
		["Akubar the Seer"] = "Akubar le Prophète",
		["Spiteflayer"] = "Ecorchebile",
		["Ravage"] = "Ravage",
		["Clack the Reaver"] = "Clack le Saccageur",
		["Deatheye"] = "Oeil-de-mort",
		["Grunter"] = "Grunter",
		["Dreadscorn"] = "Dériseffroi",
		["Carlo Aurelius"] = "Carlo Aurelius",
		["Atal'ai Deathwalker's Spirit"] = "Esprit de marche-mort atal'ai",
		["Atal'ai Slave"] = "Esclave atal'ai",
		["Nightmare Whelp"] = "Dragonnet de cauchemar",
		["Atal'ai Skeleton"] = "Squelette atal'ai",
		["Hakkari Sapper"] = "Sapeur Hakkari",
		["Dark Iron Marksman"] = "Tireur d'élite Sombrefer",
		["Archmage Xylem"] = "Archimage Xylem",
		["Captain Vanessa Beltis"] = "Capitaine Vanessa Beltis",
		["Lindros"] = "Lindros",
		["Patrick Mills"] = "Patrick Mills",
		["Master Wood"] = "Maître Wood",
		["Deep Lurker"] = "Rôdeur des Abysses",
		["Lathoric the Black"] = "Lathoric le Noir",
		["Sanath Lim-yo"] = "Sanath Lim-yo",
		["Obsidion"] = "Obsidion",
		["Enslaved Archaeologist"] = "Archéologue asservi",
		["Warlord Krellian"] = "Seigneur de guerre Krellian",
		["Caravan Master Tset"] = "Maître-caravanier Tset",
		["Dying Archaeologist"] = "Archéologue mourant",
		["Twilight Idolater"] = "Idolâtre du crépuscule",
		["Kim'jael"] = "Kim'jael",
		["Dorius"] = "Dorius",
		["Zamael Lunthistle"] = "Zamael Chardelune",
		["Hakkari Bloodkeeper"] = "Garde de sang Hakkar'i",
		["Nilith Lokrav"] = "Nilith Lokrav",
		["Raze"] = "Raze",
		["Shadowsilk Poacher"] = "Braconnier Ombresoie",
		["Avatar of Hakkar"] = "Avatar d'Hakkar",
		["Trade Master Kovic"] = "Maître d'artisanat Kovic",
		["Clunk"] = "Clonk",
		["Skeletal Servant"] = "Serviteur squelettique",
		["Kalaran Windblade"] = "Kalaran Lamevent",
		["Kalaran the Deceiver"] = "Kalaran le Menteur",
		["Nightmare Suppressor"] = "Confineur de cauchemar",
		["Dark Iron Sentry"] = "Sentinelle Sombrefer",
		["Squire Maltrake"] = "Ecuyer Maltrake",
		["Atal'ai Totem"] = "Totem atal'ai",
		["Blighted Surge"] = "Vague chancreuse",
		["Plague Ravager"] = "Ravageur de la peste",
		["Blighted Horror"] = "Horreur corrompue",
		["Plague Monstrosity"] = "Monstruosité de la peste",
		["Scourge Soldier"] = "Soldat du Fléau",
		["Cursed Mage"] = "Mage maudit",
		["Scourge Warder"] = "Gardien du Fléau",
		["Dark Caster"] = "Magicien noir",
		["Scourge Guard"] = "Garde du Fléau",
		["Dread Weaver"] = "Tisseur d'effroi",
		["Scourge Champion"] = "Champion du Fléau",
		["Cannibal Ghoul"] = "Goule cannibale",
		["Gibbering Ghoul"] = "Goule bredouillante",
		["Diseased Flayer"] = "Ecorcheur maladif",
		["Putrid Gargoyle"] = "Gargouille putride",
		["Putrid Shrieker"] = "Hurleuse putride",
		["Unseen Servant"] = "Serviteur invisible",
		["Eyeless Watcher"] = "Gardien aveugle",
		["Torn Screamer"] = "Hurleuse déchirée",
		["Hate Shrieker"] = "Hurleuse haineuse",
		["Death Singer"] = "Chante-mort",
		["Stitched Horror"] = "Horreur raccommodée",
		["Gangled Golem"] = "Golem dégingandé",
		["Stitched Golem"] = "Golem raccommodé",
		["Dark Adept"] = "Adepte noir",
		["Death Cultist"] = "Sectateur de la mort",
		["Vile Tutor"] = "Vil tuteur",
		["Shadowmage"] = "Ombremage",
		["Dark Summoner"] = "Invocateur des ténèbres",
		["Necromancer"] = "Nécromancien",
		["Crypt Stalker"] = "Traqueur de crypte",
		["Crypt Walker"] = "Marcheur des cryptes",
		["Crypt Horror"] = "Horreur des cryptes",
		["Crypt Slayer"] = "Pourfendeur des cryptes",
		["Mossflayer Scout"] = "Eclaireur Ecorchemousse",
		["Mossflayer Shadowhunter"] = "Chasseur des ténèbres Ecorchemousse",
		["Mossflayer Cannibal"] = "Cannibale Ecorchemousse",
		["Woodsman"] = "Bûcheron",
		["Ranger"] = "Forestier",
		["Pathstrider"] = "Eclaireur",
		["Dark Iron Lookout"] = "Guetteur Sombrefer",
		["Magus Rimtori"] = "Magus Rimtori",
		["Atal'alarion"] = "Atal'alarion",
		["Blood Elf Defender"] = "Défenseur elfe de sang",
		["Jediga"] = "Jediga",
		["Umbranse the Spiritspeaker"] = "Umbranse l'Implorateur d'espri",
		["Plaguehound Runt"] = "Chien pestiféré chétif",
		["Plaguehound"] = "Chien pestiféré",
		["Frenzied Plaguehound"] = "Chien pestiféré frénétique",
		["Plaguebat"] = "Chauve-souris pestiférée",
		["Noxious Plaguebat"] = "Chauve-souris pestiférée nocive",
		["Monstrous Plaguebat"] = "Chauve-souris pestiférée monstrueuse",
		["Carrion Grub"] = "Larve putride",
		["Carrion Devourer"] = "Dévoreur putride",
		["Living Decay"] = "Pourriture vivante",
		["Rotting Sludge"] = "Vase pourrissante",
		["Angered Infernal"] = "Infernal colérique",
		["Alexandra Constantine"] = "Alexandra Constantine",
		["Kroum"] = "Kroum",
		["Mithril Dragonling"] = "Petit dragon mécanique en mithril",
		["Infernal Servant"] = "Serviteur Infernal",
		["Morta'gya the Keeper"] = "Morta'gya la Gardienne",
		["Hukku's Voidwalker"] = "Marcheur du Vide d'Hukku",
		["Hukku's Succubus"] = "Succube d'Hukku",
		["Hukku's Imp"] = "Diablotin d'Hukku",
		["Jes'rimon"] = "Jes'rimon",
		["The Evalcharr"] = "L'Evalcharr",
		["Auctioneer Beardo"] = "Commissaire-priseur Beardo",
		["Saern Priderunner"] = "Saern Cours-avec-fierté",
		["Auctioneer Tolon"] = "Commissaire-priseur Tolon",
		["Auctioneer Chilton"] = "Commissaire-priseur Chilton",
		["Auctioneer Buckler"] = "Commissaire-priseur Latarge",
		["Auctioneer Leeka"] = "Commissaire-priseur Leeka",
		["Auctioneer Thathung"] = "Commissaire-priseur Thathung",
		["Auctioneer Stampi"] = "Commissaire-priseur Stampi",
		["Felbeast"] = "Gangrebête",
		["Jubie Gadgetspring"] = "Jubie Gadgettaressort",
		["Massive Infernal"] = "Infernal massif",
		["Dreadlord"] = "Seigneur de l'effroi",
		["Felguard Elite"] = "Gangregarde d'élite",
		["Manahound"] = "Molosse de mana",
		["Auctioneer Fitch"] = "Commissaire-priseur Fitch",
		["Auctioneer Redmuse"] = "Commissaire-priseur Songerouge",
		["Auctioneer Epitwee"] = "Commissaire-priseur Epitwee",
		["Auctioneer Gullem"] = "Commissaire-priseur Gullem",
		["Auctioneer Golothas"] = "Commissaire-priseur Golothas",
		["Auctioneer Wabang"] = "Commissaire-priseur Wabang",
		["Buzzek Bracketswing"] = "Buzzek Braquéquerre",
		["Raytaf"] = "Raytaf",
		["Shahiar"] = "Shahiar",
		["Zaman"] = "Zaman",
		["Mosshoof Stag"] = "Cerf Sabot-de-mousse",
		["Mosshoof Courser"] = "Coursier Sabot-de-mousse",
		["Timberweb Recluse"] = "Grumetoile solitaire",
		["Mistwing Rogue"] = "Brumaile insoumis",
		["Mistwing Ravager"] = "Ravageur Brumaile",
		["Forest Ooze"] = "Limon forestier",
		["Deathly Usher"] = "Mort silencieuse",
		["Muck Splash"] = "Eclabousseur des marais",
		["Royal Historian Archesonus"] = "Historienne royale Archesonus",
		["A tormented voice"] = "Une voix tourmentée",
		["Franclorn Forgewright"] = "Franclorn Le Forgebusier",
		["Anvilrage Overseer"] = "Surveillant Ragenclume",
		["Anvilrage Warden"] = "Gardien Ragenclume",
		["Anvilrage Guardsman"] = "Garde Ragenclume",
		["Anvilrage Footman"] = "Fantassin Ragenclume",
		["Anvilrage Soldier"] = "Soldat Ragenclume",
		["Anvilrage Medic"] = "Médecin Ragenclume",
		["Anvilrage Officer"] = "Officier Ragenclume",
		["Shadowforge Peasant"] = "Paysan Ombreforge",
		["Doomforge Craftsman"] = "Artisan Forge-ruine",
		["Anvilrage Marshal"] = "Maréchal Ragenclume",
		["Doomforge Dragoon"] = "Dragon Forge-ruine",
		["Doomforge Arcanasmith"] = "Forgeron des arcanes Forge-ruine",
		["Anvilrage Reservist"] = "Réserviste Ragenclume",
		["Shadowforge Citizen"] = "Citoyen Ombreforge",
		["Anvilrage Captain"] = "Capitaine Ragenclume",
		["Shadowforge Senator"] = "Sénateur Ombreforge",
		["Warbringer Construct"] = "Assemblage porteguerre",
		["Ragereaver Golem"] = "Golem ravarage",
		["Wrath Hammer Construct"] = "Assemblage du marteau de colère",
		["Molten War Golem"] = "Golem de lave de guerre",
		["Fireguard"] = "Garde du feu",
		["Blazing Fireguard"] = "Garde du feu flamboyant",
		["Fireguard Destroyer"] = "Destructeur Garde du feu",
		["Twilight's Hammer Torturer"] = "Tortionnaire du Marteau du crépuscule",
		["Twilight Emissary"] = "Emissaire du crépuscule",
		["Twilight Bodyguard"] = "Garde du corps du crépuscule",
		["Twilight's Hammer Ambassador"] = "Ambassadeur du Marteau du crépuscule",
		["Arena Spectator"] = "Spectateur de l'arène",
		["Quarry Slave"] = "Esclave de la carrière",
		["Weapon Technician"] = "Ingénieur de siège",
		["Bloodhound"] = "Limier",
		["Bloodhound Mastiff"] = "Limier mastiff",
		["Panzor the Invincible"] = "Panzor l'Invincible",
		["The Behemoth"] = "Le Béhémoth",
		["Dredge Worm"] = "Ver térébrant",
		["Deep Stinger"] = "Dard des Abysses",
		["Dark Screecher"] = "Hurleur noir",
		["Burrowing Thundersnout"] = "Groin-tonnerre fouisseur",
		["Princess Moira Bronzebeard"] = "Princesse Moira Barbe-de-bronze",
		["Borer Beetle"] = "Hanneton perceur",
		["Cave Creeper"] = "Rampant des cavernes",
		["Angerclaw Bear"] = "Ours de la griffe colérique",
		["Angerclaw Grizzly"] = "Grizzly de la griffe colérique",
		["Angerclaw Mauler"] = "Marteleur de la griffe colérique",
		["Felpaw Wolf"] = "Loup Gangrepatte",
		["Felpaw Scavenger"] = "Charognard Gangrepatte",
		["Felpaw Ravager"] = "Ravageur Gangrepatte",
		["Blackrock Drake"] = "Drake Rochenoire",
		["Hematos"] = "Hematos",
		["Krom'Grul"] = "Krom'Grul",
		["Thauris Balgarr"] = "Thauris Balgarr",
		["Gruklash"] = "Gruklash",
		["Firegut Captain"] = "Capitaine Brûlentrailles",
		["Malfunctioning Reaver"] = "Saccageur défectueux",
		["Ironhand Guardian"] = "Gardien Main-de-fer",
		["Golem Lord Argelmach"] = "Seigneur golem Argelmach",
		["Voidwalker Minion"] = "Sbire marcheur du Vide",
		["Bael'Gar"] = "Bael'Gar",
		["Lord Incendius"] = "Seigneur Incendius",
		["High Interrogator Gerstahn"] = "Grande Interrogatrice Gerstahn",
		["Emperor Dagran Thaurissan"] = "Empereur Dagran Thaurissan",
		["Commander Gor'shak"] = "Commandant Gor'shak",
		["Kharan Mighthammer"] = "Kharan Force-martel",
		["Dughal Stormwing"] = "Dughal Aile-tempête",
		["Marshal Windsor"] = "Maréchal Windsor",
		["Pyromancer Loregrain"] = "Pyromancien Blé-du-savoir",
		["Lord Roccor"] = "Seigneur Roccor",
		["Overmaster Pyron"] = "Grand seigneur Pyron",
		["Gorosh the Dervish"] = "Gorosh le Derviche",
		["Grizzle"] = "Grison",
		["Eviscerator"] = "Eviscérateur",
		["Ok'thor the Breaker"] = "Ok'thor le Briseur",
		["Anub'shiah"] = "Anub'shiah",
		["Hedrum the Creeper"] = "Hedrum le Rampant",
		["General Angerforge"] = "Général Forgehargne",
		["Hate'rel"] = "Haine'rel",
		["Anger'rel"] = "Colé'rel",
		["Vile'rel"] = "Ignobl'rel",
		["Gloom'rel"] = "Funéb'rel",
		["Seeth'rel"] = "Fulmi'rel",
		["Doom'rel"] = "Tragi'rel",
		["Dope'rel"] = "Demeu'rel",
		["Warder Stilgiss"] = "Gardien Stilgiss",
		["Verek"] = "Verek",
		["Scarshield Grunt"] = "Grunt du Bouclier balafré",
		["Scarshield Sentry"] = "Sentinelle du Bouclier balafré",
		["Scarshield Acolyte"] = "Acolyte du Bouclier balafré",
		["Scarshield Quartermaster"] = "Intendant du Bouclier balafré",
		["Fineous Darkvire"] = "Fineous Sombrevire",
		["Warlord Goretooth"] = "Seigneur de guerre Sangredent",
		["Shadowmage Vivian Lagrave"] = "Ombremage Vivian Lagrave",
		["Hierophant Theodora Mulvadania"] = "Hiérophante Theodora Mulvadania",
		["Lexlort"] = "Lexlort",
		["Galamav the Marksman"] = "Galamav le Tireur d'élite",
		["Thal'trak Proudtusk"] = "Thal'trak Fière-défense",
		["Razal'blade"] = "Razal'lame",
		["Thunderheart"] = "C?ur-de-tonnerre",
		["Initiate Amakkar"] = "Initié Amakkar",
		["Grunt Gargal"] = "Grunt Gargal",
		["Rage Talon Dragonspawn"] = "Draconide de la Griffe enragée",
		["Scarshield Legionnaire"] = "Légionnaire du Bouclier balafré",
		["Scarshield Spellbinder"] = "Lieur de sort du Bouclier balafré",
		["Eridan Bluewind"] = "Eridan Ventebleu",
		["J.D. Collie"] = "J.D. Collie",
		["Larion"] = "Larion",
		["Muigin"] = "Muigin",
		["Sha'ni Proudtusk"] = "Sha'ni Fière-défense",
		["Ambassador Flamelash"] = "Ambassadeur Cinglefouet",
		["Young Diemetradon"] = "Jeune dimeurtrodon",
		["Diemetradon"] = "Dimeurtrodon",
		["Elder Diemetradon"] = "Ancien dimeurtrodon",
		["Fledgling Pterrordax"] = "Petit pterreurdactyle",
		["Pterrordax"] = "Pterreurdactyle",
		["Frenzied Pterrordax"] = "Pterreurdactyle frénétique",
		["Gor'tesh"] = "Gor'tesh",
		["Oralius"] = "Oralius",
		["Highlord Omokk"] = "Généralissime Omokk",
		["Spirestone Battle Mage"] = "Mage de bataille Pierre-du-pic",
		["Spirestone Mystic"] = "Mystique Pierre-du-pic",
		["Spirestone Enforcer"] = "Massacreur Pierre-du-pic",
		["Spirestone Reaver"] = "Saccageur Pierre-du-pic",
		["Spirestone Ogre Magus"] = "Ogre-magus Pierre-du-pic",
		["Spirestone Warlord"] = "Chef de guerre Pierre-du-pic",
		["Spirestone Lord Magus"] = "Seigneur magus Pierre-du-pic",
		["Spirestone Battle Lord"] = "Seigneur de bataille Pierre-du-pic",
		["Spirestone Butcher"] = "Boucher Pierre-du-pic",
		["Shadow Hunter Vosh'gajin"] = "Chasseresse des ombres Vosh'gajin",
		["War Master Voone"] = "Maître de guerre Voone",
		["Smolderthorn Mystic"] = "Mystique Brûleronce",
		["Smolderthorn Shadow Priest"] = "Prêtre des ombres Brûleronce",
		["Smolderthorn Headhunter"] = "Chasseur de têtes Brûleronce",
		["Scarshield Warlock"] = "Démoniste du Bouclier balafré",
		["Scarshield Raider"] = "Ecumeur du Bouclier balafré",
		["Firebrand Grunt"] = "Grunt de Brandefeu",
		["Firebrand Legionnaire"] = "Légionnaire de Brandefeu",
		["Firebrand Darkweaver"] = "Tisseur d?ombre de Brandefeu",
		["Firebrand Invoker"] = "Invocateur de Brandefeu",
		["Firebrand Dreadweaver"] = "Tisseur d?effroi de Brandefeu",
		["Firebrand Pyromancer"] = "Pyromancien de Brandefeu",
		["Smolderthorn Shadow Hunter"] = "Chasseur d'ombre Brûleronce",
		["Smolderthorn Witch Doctor"] = "Sorcier-docteur Brûleronce",
		["Smolderthorn Axe Thrower"] = "Lanceur de haches Brûleronce",
		["Smolderthorn Berserker"] = "Berserker Brûleronce",
		["Smolderthorn Seer"] = "Prophète Brûleronce",
		["Williden Marshal"] = "Williden Marshal",
		["Petra Grossen"] = "Petra Grossen",
		["Enraged Wyvern"] = "Wyverne enragée",
		["Donova Snowden"] = "Donova Antreneige",
		["Gaeriyan"] = "Gaeriyan",
		["Incendosaur"] = "Incendosaure",
		["Houndmaster Grebmar"] = "Maître-chien Grebmar",
		["Blazerunner"] = "Coursembrase",
		["Living Storm"] = "Tempête vivante",
		["Twilight's Hammer Executioner"] = "Bourreau du Marteau du crépuscule",
		["Scarshield Worg"] = "Worg du Bouclier balafré",
		["Spawn of Bael'Gar"] = "Rejeton de Bael'Gar",
		["Dark Keeper Vorfalk"] = "Gardien noir Vorfalk",
		["Dark Keeper Bethek"] = "Gardien noir Bethek",
		["Dark Keeper Uggel"] = "Gardien noir Uggel",
		["Dark Keeper Zimrel"] = "Gardien noir Zimrel",
		["Dark Keeper Ofgut"] = "Gardien noir Ofgut",
		["Dark Keeper Pelver"] = "Gardien noir Pelver",
		["Dark Guard"] = "Sombre garde",
		["Scarlet Warder"] = "Gardien écarlate",
		["Scarlet Praetorian"] = "Prétorien écarlate",
		["Scarlet Cleric"] = "Clerc écarlate",
		["Scarlet Curate"] = "Vicaire écarlate",
		["Scarlet Archmage"] = "Archimage écarlate",
		["Scarlet Enchanter"] = "Enchanteur écarlate",
		["Aquementas"] = "Aquementas",
		["Xavathras"] = "Xavathras",
		["Cyrus Therepentous"] = "Cyrus Lerepenti",
		["Gadgetzan Bruiser"] = "Cogneur de Gadgetzan",
		["Frenzied Black Drake"] = "Drake noir frénétique",
		["Chieftain Bloodmaw"] = "Chef de clan Saignegueule",
		["Overlord Ror"] = "Seigneur Ror",
		["Golhine the Hooded"] = "Golhine l'Encapuchonné",
		["Miblon Snarltooth"] = "Miblon Grondequenotte",
		["Watchman Doomgrip"] = "Sentinelle Ruinepoigne",
		["Cloned Ooze"] = "Limon cloné",
		["Gorishi Egg"] = "Oeuf Gorishi",
		["Gorishi Grub"] = "Larve Gorishi",
		["Plugger Spazzring"] = "Lanfiche Brouillecircuit",
		["Mistress Nagmara"] = "Gouvernante Nagmara",
		["Phalanx"] = "Phalange",
		["Private Rocknot"] = "Soldat Rochenoeud",
		["Lord Banehollow"] = "Seigneur Creuseplaie",
		["Shadow Lord Fel'dan"] = "Seigneur des ténèbres Gangr'dan",
		["Rakaiah"] = "Rakaiah",
		["Grark Lorkrub"] = "Grark Lorkrub",
		["Enraged Felbat"] = "Chauve-souris déchue enragée",
		["Blackrock Ambusher"] = "Embusqué Rochenoire",
		["Enraged Gryphon"] = "Griffon enragé",
		["Enraged Hippogryph"] = "Hippogriffe enragé",
		["Arathandris Silversky"] = "Arathandris Argeciel",
		["Maybess Riverbreeze"] = "Maybess Brise-des-rivières",
		["Hurley Blackbreath"] = "Hurley Soufflenoir",
		["High Executioner Nuzrak"] = "Bourreau suprême Nuzrak",
		["Shadow of Lexlort"] = "Ombre de Lexlort",
		["Enohar Thunderbrew"] = "Enohar Tonnebière",
		["Blackbreath Crony"] = "Compère de Soufflenoir",
		["Ribbly Screwspigot"] = "Ribbly Fermevanne",
		["Yuka Screwspigot"] = "Yuka Fermevanne",
		["Grim Patron"] = "Client sinistre",
		["Guzzling Patron"] = "Client assoiffé",
		["Cawind Trueaim"] = "Cawind Trait-précis",
		["Hammered Patron"] = "Client fin saoul",
		["Marshal Maxwell"] = "Maréchal Maxwell",
		["Jalinda Sprig"] = "Jalinda Brindille",
		["Helendis Riverhorn"] = "Helendis Ruissecorne",
		["Ragged John"] = "John le Loqueteux",
		["Mayara Brightwing"] = "Mayara Luisaile",
		["Overlord Wyrmthalak"] = "Seigneur Wyrmthalak",
		["Bloodaxe Veteran"] = "Vétéran Hache-sanglante",
		["Bannok Grimaxe"] = "Bannok Hache-sinistre",
		["Arei"] = "Arei",
		["Treant Spirit"] = "Esprit de tréant",
		["Hahk'Zor"] = "Hahk'Zor",
		["Gorgon'och"] = "Gorgon'och",
		["Blackrock Raider"] = "Ecumeur Rochenoire",
		["Laris Geardawdle"] = "Laris Bringuefourbi",
		["Karna Remtravel"] = "Karna Songerrance",
		["Torwa Pathfinder"] = "Torwa le Guide",
		["Dreka'Sur"] = "Dreka'Sur",
		["Gargantuan Ooze"] = "Limon gargantuesque",
		["U'cha"] = "U'cha",
		["A-Me 01"] = "A-mi 01",
		["Scorching Totem"] = "Totem de brûlure",
		["Agnar Beastamer"] = "Agnar Domptebêtes",
		["Ograbisi"] = "Ograbisi",
		["Shill Dinger"] = "Shill Dinger",
		["Tobias Seecher"] = "Tobias Seecher",
		["Crest Killer"] = "Tueur des sommets",
		["Jaz"] = "Jaz",
		["Marshal Reginald Windsor"] = "Maréchal Reginald Windsor",
		["Lar'korwi Mate"] = "Compagne de Lar'korwi",
		["Lar'korwi"] = "Lar'korwi",
		["Windwall Totem III"] = "Totem de Mur des vents III",
		["Ember Worg"] = "Worg de braise",
		["Venomtip Scorpid"] = "Scorpide dard-venin",
		["Bloodaxe Raider"] = "Ecumeur Hache-sanglante",
		["Bloodaxe Evoker"] = "Evocateur Hache-sanglante",
		["Slavering Ember Worg"] = "Worg de braise écumant",
		["Deathlash Scorpid"] = "Scorpide fouet-mortel",
		["Bloodaxe Worg"] = "Worg Hache-sanglante",
		["Giant Ember Worg"] = "Worg de braise géant",
		["Firetail Scorpid"] = "Scorpide queue-de-feu",
		["Spire Scorpid"] = "Scorpide du pic",
		["Yorba Screwspigot"] = "Yorba Fermevanne",
		["Burning Imp"] = "Diablotin ardent",
		["Bloodaxe Warmonger"] = "Combattant Hache-sanglante",
		["Bloodaxe Summoner"] = "Invocateur Hache-sanglante",
		["Ghok Bashguud"] = "Ghok Bounnebaffe",
		["Quartermaster Zigris"] = "Intendant Zigris",
		["Flamekin Spitter"] = "Flammide cracheur",
		["Flamekin Sprite"] = "Fée Flammide",
		["Flamekin Torcher"] = "Flammide incendiaire",
		["Flamekin Rager"] = "Flammide enragé",
		["Pyroguard Emberseer"] = "Pyrogarde Prophète ardent",
		["Blackhand Dreadweaver"] = "Tisseur d?effroi Main-noire",
		["Blackhand Summoner"] = "Invocateur Main-noire",
		["Blackhand Veteran"] = "Vétéran Main-noire",
		["Mathredis Firestar"] = "Mathredis Brasétoile",
		["Auctioneer Grimful"] = "Commissaire-priseur Grimful",
		["Auctioneer Grizzlin"] = "Commissaire-priseur Grizzlin",
		["Auctioneer Kresky"] = "Commissaire-priseur Kresky",
		["Auctioneer Lympkin"] = "Commissaire-priseur Lympkin",
		["Salia"] = "Salia",
		["Moora"] = "Moora",
		["Jaedenar Legionnaire"] = "Légionnaire de Jaedenar",
		["Locheed"] = "Locheed",
		["Prince Xavalis"] = "Prince Xavalis",
		["Entropic Beast"] = "Bête d'entropie",
		["Entropic Horror"] = "Horreur d'entropie",
		["Jarquia"] = "Jarquia",
		["Magmus"] = "Magmus",
		["Shadowforge Flame Keeper"] = "Garde-flammes Ombreforge",
		["Xon'cha"] = "Xon'cha",
		["Winna Hazzard"] = "Winna Hazzard",
		["Ringo"] = "Ringo",
		["Brackenwall Enforcer"] = "Massacreur de Mur-de-Fougères",
		["Lakeshire Guard"] = "Garde de Comté-du-lac",
		["Gorishi Hive Guard"] = "Garde Gorishi",
		["Gorishi Hive Queen"] = "Reine Gorishi",
		["Ribbly's Crony"] = "Compère de Ribbly",
		["Gereck"] = "Gereck",
		["Grimestack"] = "Galimatache",
		["High Priestess of Thaurissan"] = "Grande prêtresse de Thaurissan",
		["Deathmaw"] = "Gueule-du-trépas",
		["Terrorspark"] = "Lueur terrifiante",
		["Rage Talon Flamescale"] = "Flammécaille de la Griffe enragée",
		["High Justice Grimstone"] = "Juge Supérieur Mornepierre",
		["Volchan"] = "Volchan",
		["Rookery Whelp"] = "Dragonnet de la colonie",
		["Lord Victor Nefarius"] = "Seigneur Victor Nefarius",
		["Spire Scarab"] = "Scarabée du pic",
		["Lady Sylvanas Windrunner"] = "Dame Sylvanas Coursevent",
		["Rexxar"] = "Rexxar",
		["Moonflare Totem"] = "Totem de Feu lunaire",
		["Onyxia"] = "Onyxia",
		["General Colbatann"] = "Général Colbatann",
		["Mezzir the Howler"] = "Mezzir le hurleur",
		["Kashoch the Reaver"] = "Kashoch le saccageur",
		["Grizzle Snowpaw"] = "Grison Neigepatte",
		["Rak'shiri"] = "Rak'shiri",
		["Lady Hederine"] = "Dame Hederine",
		["Azurous"] = "Azurous",
		["Misha"] = "Misha",
		["Flame Buffet Totem"] = "Totem de rafale de flammes",
		["Superior Healing Ward"] = "Gardien guérisseur supérieur",
		["Halycon"] = "Halycon",
		["Bloodaxe Worg Pup"] = "Jeune worgen Hache-sanglante",
		["Bijou"] = "Bijou",
		["Rookery Guardian"] = "Gardien de la colonie",
		["Kibler"] = "Kibler",
		["Burning Felhound"] = "Gangrechien ardent",
		["Burning Felguard"] = "Gangregarde ardent",
		["Solakar Flamewreath"] = "Solakar Voluteflamme",
		["Tinkee Steamboil"] = "Brikolette Toutevapeur",
		["Gizrul the Slavener"] = "Gizrul l'esclavagiste",
		["Dulciea Frostmoon"] = "Dulciea Givrelune",
		["Vaelan"] = "Vaelan",
		["Scarshield Infiltrator"] = "Infiltrateur du Bouclier balafré",
		["Ranshalla"] = "Ranshalla",
		["Jaron Stoneshaper"] = "Jaron Sculptepierre",
		["Krakle"] = "Krakle",
		["Storm Shadowhoof"] = "Storm Sabot-d'ombre",
		["Aurora Skycaller"] = "Aurora Mandeciel",
		["Umi Rumplesnicker"] = "Umi Sarcafroisse",
		["Trull Failbane"] = "Trull Faillemort",
		["Witch Doctor Mau'ari"] = "Sorcier-docteur Mau'ari",
		["Blackhand Incarcerator"] = "Geôlier Main-noire",
		["Blackhand Elite"] = "Elite Main-noire",
		["Blackhand Assassin"] = "Assassin Main-noire",
		["Blackhand Iron Guard"] = "Garde de fer Main-noire",
		["Emberstrife"] = "Brandeguerre",
		["Gyth"] = "Gyth",
		["Vaelastrasz the Red"] = "Vaelastrasz le rouge",
		["Kergul Bloodaxe"] = "Kergul Hache-sanglante",
		["Gruul Darkblade"] = "Gruul Sombrelame",
		["General Drakkisath"] = "Général Drakkisath",
		["Yaelika Farclaw"] = "Yaelika Griffe-loin",
		["Rage Talon Dragon Guard"] = "Garde de la Griffe enragée",
		["Shrye Ragefist"] = "Shrye Ragepoing",
		["Rage Talon Captain"] = "Capitaine de la Griffe enragée",
		["Rage Talon Fire Tongue"] = "Langue de feu de la Griffe enragée",
		["Spire Spider"] = "Araignée du pic",
		["Spire Spiderling"] = "Jeune araignée du pic",
		["Crystal Fang"] = "Croc cristallin",
		["Omusa Thunderhorn"] = "Omusa Corne-tonnerre",
		["Ravaged Cadaver"] = "Cadavre en décomposition",
		["Mangled Cadaver"] = "Cadavre mutilé",
		["Broken Cadaver"] = "Cadavre démembré",
		["Spectral Citizen"] = "Citoyen spectral",
		["Ghostly Citizen"] = "Citoyen fantomatique",
		["Vengeful Phantom"] = "Fantôme vengeur",
		["Spiteful Phantom"] = "Fantôme malveillant",
		["Wrath Phantom"] = "Fantôme de colère",
		["Skeletal Guardian"] = "Gardien squelettique",
		["Skeletal Berserker"] = "Berserker squelettique",
		["Skul"] = "Krân",
		["Black Guard Sentry"] = "Sentinelle de la Garde noire",
		["Thuzadin Shadowcaster"] = "Exhalombre Thuzadin",
		["Thuzadin Acolyte"] = "Acolyte Thuzadin",
		["Thuzadin Necromancer"] = "Nécromancien Thuzadin",
		["Plague Ghoul"] = "Goule pestilentielle",
		["Ghoul Ravener"] = "Goule vorace",
		["Fleshflayer Ghoul"] = "Goule écorcheuse",
		["Rockwing Gargoyle"] = "Gargouille Aile-de-roc",
		["Rockwing Screecher"] = "Hurleuse Aile-de-roc",
		["Eye of Naxxramas"] = "Oeil de Naxxramas",
		["Crypt Crawler"] = "Rampant des cryptes",
		["Crypt Beast"] = "Bête des cryptes",
		["Patchwork Horror"] = "Horreur régénérée",
		["Ash'ari Crystal"] = "Cristal Ash'ari",
		["Bile Spewer"] = "Cracheur de bile",
		["Venom Belcher"] = "Crache venin",
		["Crimson Guardsman"] = "Gardien cramoisi",
		["Crimson Conjuror"] = "Conjurateur cramoisi",
		["Crimson Initiate"] = "Initié cramoisi",
		["Crimson Defender"] = "Défenseur cramoisi",
		["Crimson Sorcerer"] = "Ensorceleur cramoisi",
		["Crimson Priest"] = "Prêtre cramoisi",
		["Crimson Gallant"] = "Vaillant cramoisi",
		["Crimson Battle Mage"] = "Mage de bataille cramoisi",
		["Crimson Inquisitor"] = "Inquisiteur cramoisi",
		["Warchief Rend Blackhand"] = "Chef de guerre Rend Main-noire",
		["The Beast"] = "La Bête",
		["Gregor Greystone"] = "Gregor Grispierre",
		["Vectus"] = "Vectus",
		["Marduk Blackpool"] = "Marduk Noirétang",
		["Magistrate Barthilas"] = "Magistrat Barthilas",
		["Baroness Anastari"] = "Baronne Anastari",
		["Nerub'enkan"] = "Nerub'enkan",
		["Maleki the Pallid"] = "Maleki le Blafard",
		["Ramstein the Gorger"] = "Ramstein Grandgosier",
		["Baron Rivendare"] = "Baron Vaillefendre",
		["Plagued Rat"] = "Rat pestiféré",
		["Chromatic Whelp"] = "Dragonnet chromatique",
		["Chromatic Dragonspawn"] = "Draconide chromatique",
		["Prospector Ironboot"] = "Prospecteur Botte-de-fer",
		["Plagued Insect"] = "Insecte pestiféré",
		["Shrieking Banshee"] = "Banshee hurlante",
		["Wailing Banshee"] = "Banshee gémissante",
		["Felnok Steelspring"] = "Felnok Ressordacier",
		["Scholomance Adept"] = "Adepte de la Scholomance",
		["Scholomance Neophyte"] = "Néophyte de la Scholomance",
		["Scholomance Acolyte"] = "Acolyte de la Scholomance",
		["Scholomance Occultist"] = "Occultiste de la Scholomance",
		["Scholomance Student"] = "Etudiant de la Scholomance",
		["Scholomance Necrolyte"] = "Nécrolyte de la Scholomance",
		["Scholomance Necromancer"] = "Nécromancien de la Scholomance",
		["Splintered Skeleton"] = "Squelette brisé",
		["Skulking Corpse"] = "Cadavre rôdeur",
		["Unstable Corpse"] = "Corps instable",
		["Reanimated Corpse"] = "Cadavre réanimé",
		["Risen Lackey"] = "Laquais ressuscité",
		["Risen Aberration"] = "Aberration ressuscitée",
		["Risen Warrior"] = "Guerrier ressuscité",
		["Risen Protector"] = "Protecteur ressuscité",
		["Risen Construct"] = "Assemblage ressuscité",
		["Risen Guard"] = "Garde ressuscité",
		["Risen Bonewarder"] = "Gardien d'os ressuscité",
		["Diseased Ghoul"] = "Goule infectée",
		["Ragged Ghoul"] = "Goule décharnée",
		["Spectral Tutor"] = "Educateur spectral",
		["Spectral Researcher"] = "Chercheur spectral",
		["Spectral Teacher"] = "Professeur spectral",
		["Lady Illucia Barov"] = "Dame Illucia Barov",
		["Jandice Barov"] = "Jandice Barov",
		["Lord Alexei Barov"] = "Seigneur Alexei Barov",
		["Instructor Malicia"] = "Instructeur Malicia",
		["Kirtonos the Herald"] = "Kirtonos le Héraut",
		["The Ravenian"] = "Le Voracien",
		["Ras Frostwhisper"] = "Ras Murmegivre",
		["Jed Runewatcher"] = "Jed Guette-runes",
		["The Unforgiven"] = "Le Condamné",
		["Plagued Maggot"] = "Asticot pestiféré",
		["Cliffwatcher Longhorn"] = "Garde-falaise Longue-corne",
		["Vaelastrasz"] = "Vaelastrasz",
		["Vol'jin"] = "Vol'jin",
		["Flametongue Totem IV"] = "Totem Langue de feu IV",
		["Hearthsinger Forresten"] = "Chanteloge Forrestin",
		["Fetid Zombie"] = "Zombie fétide",
		["Gryfe"] = "Gryfe",
		["Urok Doomhowl"] = "Urok Hurleruine",
		["Mother Smolderweb"] = "Matriarche Couveuse",
		["Urok Enforcer"] = "Massacreur d'Urok",
		["Urok Ogre Magus"] = "Ogre-magus d'Urok",
		["Scarlet Medic"] = "Médecin écarlate",
		["Scarlet Priest"] = "Prêtre écarlate",
		["Rivern Frostwind"] = "Rivern Givrevent",
		["Glacier"] = "Glacier",
		["Malyfous Darkhammer"] = "Malyfous Sombremartel",
		["Xavaric"] = "Xavaric",
		["Cobalt Whelp"] = "Dragonnet cobalt",
		["Cobalt Broodling"] = "Rejeton de cobalt",
		["Spell Eater"] = "Dévoreur de sorts",
		["Spellmaw"] = "Gueulesort",
		["Manaclaw"] = "Griffemana",
		["Scryer"] = "Clairvoyant",
		["Chromie"] = "Chromie",
		["Plagued Hatchling"] = "Jeune pestiféré",
		["Summoned Blackhand Dreadweaver"] = "Tisseur d?effroi Main-noire invoqué",
		["Summoned Blackhand Veteran"] = "Vétéran Main-noire invoqué",
		["Rookery Hatcher"] = "Démolisseur de la colonie",
		["Remorseful Highborne"] = "Bien-née pénitente",
		["Bile Slime"] = "Gelée bilieuse",
		["Summoned Zombie"] = "Zombie invoqué",
		["Carrion Scarab"] = "Scarabée putride",
		["Temporal Parasite"] = "Parasite temporel",
		["Herald of Thrall"] = "Héraut de Thrall",
		["Shy-Rotam"] = "Shy-Rotam",
		["High Chief Winterfall"] = "Grand chef Tombe-hiver",
		["Mulgris Deepriver"] = "Mulgris Rivière-profonde",
		["Awbee"] = "Awbee",
		["Sian-Rotam"] = "Sian-Rotam",
		["Blackhand Dragon Handler"] = "Dresseur de dragons Main-noire",
		["Blackhand Thug"] = "Nervi Main-noire",
		["Finkle Einhorn"] = "Finkle Einhorn",
		["Warosh"] = "Warosh",
		["Warosh the Redeemed"] = "Warosh le Racheté",
		["Jabbering Ghoul"] = "Goule jacassante",
		["Hitah'ya the Keeper"] = "Hitah'ya la Gardienne",
		["Ursius"] = "Ursius",
		["Brumeran"] = "Brumeran",
		["Timmy the Cruel"] = "Timmy le Cruel",
		["Stonespine"] = "Echine-de-pierre",
		["Archivist Galford"] = "Archiviste Galford",
		["Grand Crusader Dathrohan"] = "Grand croisé Dathrohan",
		["Balnazzar"] = "Balnazzar",
		["Chromatic Elite Guard"] = "Garde d'élite chromatique",
		["Wandering Skeleton"] = "Squelette errant",
		["Duggan Wildhammer"] = "Duggan Marteau-hardi",
		["Hed'mush the Rotting"] = "Hed'mush le Pourrissant",
		["Warlord Thresh'jin"] = "Seigneur de guerre Thresh'jin",
		["Zul'Brin Warpbranch"] = "Zul'Brin Voilebranche",
		["Ranger Lord Hawkspear"] = "Seigneur forestier Eperlance",
		["Gish the Unmoving"] = "Gish l'Immobile",
		["Lord Darkscythe"] = "Seigneur Sombrefaux",
		["Deathspeaker Selendre"] = "Nécrorateur Selendre",
		["High General Abbendis"] = "Grand général Abbendis",
		["Farmer Dalson"] = "Fermier Dalson",
		["High Executor Derrington"] = "Grand exécuteur Derrington",
		["Commander Ashlam Valorfist"] = "Commandant Ashlam Vaillepoing",
		["Argent Officer Garush"] = "Officier de l'Aube d'argent Ga",
		["Argent Officer Pureheart"] = "Officier de l'Aube d'argent Pu",
		["Argent Quartermaster Hasana"] = "Intendante de l'Aube d'argent ",
		["Argent Quartermaster Lightspark"] = "Intendant de l'Aube d'argent F",
		["Undead Scarab"] = "Scarabée mort-vivant",
		["Sindrayl"] = "Sindrayl",
		["Goraluk Anvilcrack"] = "Goraluk Brisenclume",
		["Lorekeeper Polkelt"] = "Gardien du savoir Polkelt",
		["Winterfall Runner"] = "Messager Tombe-hiver",
		["Aurius"] = "Aurius",
		["Lorax"] = "Lorax",
		["Shatterspear Troll"] = "Troll Briselance",
		["Kelek Skykeeper"] = "Kelek Gardeciel",
		["Taronn Redfeather"] = "Taronn Plume-rouge",
		["Greta Mosshoof"] = "Greta Sabot-de-mousse",
		["Tenell Leafrunner"] = "Tenell Coursefeuille",
		["Ivy Leafrunner"] = "Ivy Coursefeuille",
		["Rotting Worm"] = "Ver pourrissant",
		["Haleh"] = "Haleh",
		["Joseph Redpath"] = "Joseph Senterouge",
		["Captain Redpath"] = "Capitaine Senterouge",
		["Redpath the Corrupted"] = "Senterouge le Corrompu",
		["Marduk the Black"] = "Marduk le Noir",
		["Ghost of the Past"] = "Fantôme du passé",
		["Nessy"] = "Nessy",
		["Decrepit Guardian"] = "Gardien décrépit",
		["Davil Lightfire"] = "Davil Lumefeu",
		["Davil Crokford"] = "Davil Crokford",
		["Horgus the Ravager"] = "Horgus le Ravageur",
		["Darrowshire Betrayer"] = "Traître de Comté-de-Darrow",
		["Darrowshire Defender"] = "Défenseur de Comté-de-Darrow",
		["Silver Hand Disciple"] = "Disciple de la Main d'argent",
		["Redpath Militia"] = "Milice de Senterouge",
		["Marauding Corpse"] = "Cadavre maraudeur",
		["Marauding Skeleton"] = "Squelette maraudeur",
		["Servant of Horgus"] = "Serviteur d'Horgus",
		["Bloodletter"] = "Saigneur",
		["Summoned Water Elemental"] = "Elémentaire d'eau invoqué",
		["Naga Siren"] = "Sirène naga",
		["Jeziba"] = "Jeziba",
		["Quixxil"] = "Quixxil",
		["Legacki"] = "Legacki",
		["Scarlet Hound"] = "Chien écarlate",
		["Frostwolf"] = "Loup de givre",
		["Whitewhisker Vermin"] = "Vermine Blanche-moustache",
		["Winterax Troll"] = "Troll Hache-d?hiver",
		["Winterax Berserker"] = "Berserker Hache-d?hiver",
		["Snowblind Harpy"] = "Harpie Brûleglace",
		["Irondeep Trogg"] = "Trogg de Gouffrefer",
		["Kodo Spirit"] = "Esprit de kodo",
		["Alterac Ram"] = "Bélier d'Alterac",
		["Wildpaw Gnoll"] = "Gnoll Follepatte",
		["Fallen Hero"] = "Héros déchu",
		["Cannon Master Willey"] = "Maître canonnier Willey",
		["Captured Arko'narin"] = "Arko'narin captive",
		["Roxxik"] = "Roxxik",
		["Arko'narin"] = "Arko'narin",
		["Jessir Moonbow"] = "Jessir Arquelune",
		["Remains of Trey Lightforge"] = "Cadavre de Trey Sancteforge",
		["Alexi Barov"] = "Alexi Barov",
		["Weldon Barov"] = "Weldon Barov",
		["Della"] = "Della",
		["Illusory Wraith"] = "Ame en peine illusoire",
		["Mindless Undead"] = "Mort-vivant décérébré",
		["Malor the Zealous"] = "Malor le Zélé",
		["Smokey LaRue"] = "Smokey LaRue",
		["Lord Maxwell Tyrosus"] = "Seigneur Maxwell Tyrosus",
		["Betina Bigglezink"] = "Betina Bigglezink",
		["Leonid Barthalomew the Revered"] = "Leonid Barthalomew le Révéré",
		["Caretaker Alen"] = "Conservateur Alen",
		["Duke Nicholas Zverenhoff"] = "Duc Nicholas Zverenhoff",
		["Crimson Monk"] = "Moine cramoisi",
		["Timothy Worthington"] = "Timothy Worthington",
		["High Priestess MacDonnell"] = "Grande prêtresse MacDonnell",
		["Crimson Rifleman"] = "Fusilier cramoisi",
		["Shadow Priestess Vandis"] = "Prêtresse des ombres Vandis",
		["Alchemist Arbington"] = "Alchimiste Arbington",
		["Apothecary Dithers"] = "Apothicaire Dithers",
		["Fras Siabi"] = "Fras Siabi",
		["Carlin Redpath"] = "Carlin Senterouge",
		["Darrowshire Spirit"] = "Esprit de Comté-de-Darrow",
		["Jenova Stoneshield"] = "Jenova Targe-pierre",
		["Annora"] = "Annora",
		["Hgarth"] = "Hgarth",
		["Cauldron Lord Bilemaw"] = "Seigneur du chaudron Bilegueule",
		["Cauldron Lord Razarch"] = "Seigneur du chaudron Razarch",
		["Cauldron Lord Malvinious"] = "Seigneur du chaudron Malvinious",
		["Cauldron Lord Soulwrath"] = "Seigneur du chaudron Irâme",
		["Wynd Nightchaser"] = "Wynd Pistenuit",
		["Stratholme Courier"] = "Messager de Stratholme",
		["Drakk Stonehand"] = "Drakk Main-de-pierre",
		["Hahrana Ironhide"] = "Hahrana Cuiracier",
		["Argent Guard"] = "Garde de l'Aube d'argent",
		["Mana Tide Totem II"] = "Totem de Vague de mana II",
		["Mana Tide Totem III"] = "Totem de Vague de mana III",
		["Argent Rider"] = "Cavalier de l'Aube d'argent",
		["Azzleby"] = "Azzleby",
		["Crimson Hammersmith"] = "Forgeur de marteaux cramoisi",
		["Black Guard Swordsmith"] = "Fabricant d'épées de la Garde noire",
		["Maethrya"] = "Maethrya",
		["Yugrek"] = "Yugrek",
		["Egan"] = "Egan",
		["Spirit of Trey Lightforge"] = "Esprit de Trey Sancteforge",
		["Undead Postman"] = "Postier mort-vivant",
		["Postmaster Malown"] = "Postier Malown",
		["Myolor Sunderfury"] = "Myolor Fractefurie",
		["Ironus Coldsteel"] = "Ferronus Acier-froid",
		["Krathok Moltenfist"] = "Krathok Fontepoing",
		["Okothos Ironrager"] = "Okothos Ferrenrage",
		["Borgosh Corebender"] = "Borgosh Tord-noyau",
		["Bloodvenom Post Brave"] = "Brave du Poste de la Vénéneuse",
		["Shi'alune"] = "Shi'alune",
		["Nixxrak"] = "Nixxrak",
		["Blixxrak"] = "Blixxrak",
		["Wixxrak"] = "Wixxrak",
		["Xizzer Fizzbolt"] = "Xizzer Klaqueboulon",
		["Lunnix Sprocketslip"] = "Lunnix Glisserouage",
		["Himmik"] = "Himmik",
		["Evie Whirlbrew"] = "Evie Tourbouillon",
		["Qia"] = "Qia",
		["Everlook Bruiser"] = "Cogneur de Long-guet",
		["Lilith the Lithe"] = "Lilith l'Agile",
		["Kilram"] = "Kilram",
		["Seril Scourgebane"] = "Seril Plaie-du-Fléau",
		["Argent Defender"] = "Défenseur de l'Aube d'argent",
		["Shatterspear Drummer"] = "Tambour Briselance",
		["Mindless Skeleton"] = "Squelette décérébré",
		["Crimson Cannon"] = "Canon cramoisi",
		["Summoned Skeleton"] = "Squelette invoqué",
		["Eva Sarkhoff"] = "Eva Sarkhoff",
		["Lucien Sarkhoff"] = "Lucien Sarkhoff",
		["Manifestation of Water"] = "Manifestation d'eau",
		["Scholomance Handler"] = "Eleveur de la Scholomance",
		["Doctor Theolen Krastinov"] = "Docteur Theolen Krastinov",
		["Onyxian Whelp"] = "Dragonnet onyxien",
		["Spectral Projection"] = "Projection spectrale",
		["Caer Darrow Citizen"] = "Citoyen de Caer Darrow",
		["Magnus Frostwake"] = "Magnus Givréveil",
		["Caer Darrow Guardsman"] = "Garde de Caer Darrow",
		["Caer Darrow Cannoneer"] = "Canonnier de Caer Darrow",
		["Caer Darrow Horseman"] = "Cavalier de Caer Darrow",
		["Dark Shade"] = "Ombre noire",
		["Magistrate Marduke"] = "Magistrat Marduke",
		["Spectral Betrayer"] = "Traître spectral",
		["Spectral Defender"] = "Défenseur spectral",
		["Mossflayer Zombie"] = "Zombie Ecorchemousse",
		["Unliving Mossflayer"] = "Non-vivant Ecorchemousse",
		["Hakkari Shadowcaster"] = "Exhalombre Hakkari",
		["Hakkari Shadow Hunter"] = "Chasseur des ombres Hakkari",
		["Hakkari Blood Priest"] = "Grand prêtre Hakkari",
		["Hakkari Oracle"] = "Oracle Hakkari",
		["Zealot Lor'Khan"] = "Zélote Lor'Khan",
		["Zealot Zath"] = "Zélote Zath",
		["Gurubashi Axe Thrower"] = "Lanceur de haches Gurubashi",
		["Gurubashi Headhunter"] = "Chasseur de têtes Gurubashi",
		["Gurubashi Berserker"] = "Berserker gurubashi",
		["Gurubashi Blood Drinker"] = "Buveur de sang Gurubashi",
		["Gurubashi Warrior"] = "Guerrier Gurubashi",
		["Gurubashi Champion"] = "Champion Gurubashi",
		["Son of Hakkar"] = "Fils d'Hakkar",
		["Soulflayer"] = "Ecorcheur d'âmes",
		["Zulian Cub"] = "Jeune zulien",
		["Zulian Tiger"] = "Tigre zulien",
		["Zulian Panther"] = "Panthère zulienne",
		["Bloodseeker Bat"] = "Chauve-souris sanguinaire",
		["Razzashi Broodwidow"] = "Sombre-veuve Razzashi",
		["Razzashi Serpent"] = "Serpent Razzashi",
		["Razzashi Adder"] = "Aspic Razzashi",
		["Razzashi Cobra"] = "Cobra Razzashi",
		["Hooktooth Frenzy"] = "Crochedents frénétique",
		["Jin'do the Hexxer"] = "Jin'do le Maléficieur",
		["Bloodlord Mandokir"] = "Seigneur sanglant Mandokir",
		["High Priestess Hai'watna"] = "Grande prêtresse Hai'watna",
		["Sandfury Speaker"] = "Porte-parole Furie-des-sables",
		["Witherbark Speaker"] = "Porte-parole Fânécorce",
		["Bloodscalp Speaker"] = "Porte-parole Scalp-rouge",
		["Skullsplitter Speaker"] = "Porte-parole Casse-crâne",
		["Vilebranch Speaker"] = "Porte-parole Vilebranche",
		["Priestess Alathea"] = "Prêtresse Alathea",
		["High Priest Rohan"] = "Grand prêtre Rohan",
		["Illusion of Jandice Barov"] = "Illusion de Jandice Barov",
		["Gordok Enforcer"] = "Massacreur gordok",
		["Gordok Brute"] = "Brute gordok",
		["Gordok Mauler"] = "Marteleur gordok",
		["Gordok Ogre-Mage"] = "Ogre-mage gordok",
		["Gordok Mage-Lord"] = "Seigneur-mage gordok",
		["Gordok Captain"] = "Capitaine gordok",
		["Gordok Spirit"] = "Esprit gordok",
		["Mushgog"] = "Mushgog",
		["Gordok Warlock"] = "Démoniste gordok",
		["Gordok Reaver"] = "Saccageur gordok",
		["Wildspawn Satyr"] = "Satyre Follengeance",
		["Wildspawn Rogue"] = "Voleur Follengeance",
		["Wildspawn Trickster"] = "Entourloupeur Follengeance",
		["Wildspawn Betrayer"] = "Traître Follengeance",
		["Wildspawn Felsworn"] = "Gangrelige Follengeance",
		["Wildspawn Shadowstalker"] = "Traqueur des ombres Follengeance",
		["Wildspawn Hellcaller"] = "Invocateur infernal Follengeance",
		["Petrified Treant"] = "Tréant pétrifié",
		["Ironbark Protector"] = "Protecteur Ecorcefer",
		["Alzzin's Minion"] = "Serviteur d'Alzzin",
		["Warpwood Guardian"] = "Gardien Crochebois",
		["Warpwood Treant"] = "Tréant Crochebois",
		["Warpwood Tangler"] = "Noueur Crochebois",
		["Warpwood Stomper"] = "Ecraseur Crochebois",
		["Highborne Summoner"] = "Invocateur bien-né",
		["Tsu'zee"] = "Tsu'zee",
		["Eldreth Seether"] = "Rongé Eldreth",
		["Eldreth Sorcerer"] = "Ensorceleur Eldreth",
		["Eldreth Apparition"] = "Fantôme Eldreth",
		["Eldreth Spirit"] = "Esprit Eldreth",
		["Eldreth Spectre"] = "Spectre Eldreth",
		["Eldreth Phantasm"] = "Fantasme Eldreth",
		["Skeletal Highborne"] = "Bien-né squelette",
		["Rotting Highborne"] = "Bien-né pourrissant",
		["Arcane Aberration"] = "Aberration des arcanes",
		["Mana Remnant"] = "Restes de mana",
		["Residual Monstrosity"] = "Monstruosité résiduelle",
		["Prince Tortheldrin"] = "Prince Tortheldrin",
		["Magister Kalendris"] = "Magistère Kalendris",
		["Illyanna Ravenoak"] = "Illyanna Corvichêne",
		["Tendris Warpwood"] = "Tendris Crochebois",
		["Zevrim Thornhoof"] = "Zevrim Sabot-de-ronce",
		["Old Ironbark"] = "Vieil Ecorcefer",
		["Alzzin the Wildshaper"] = "Alzzin le Modeleur",
		["Immol'thar"] = "Immol'thar",
		["The Razza"] = "La Razza",
		["Skarr the Unbreakable"] = "Bâlhafr l'Invaincu",
		["King Gordok"] = "Roi Gordok",
		["Ragnaros"] = "Ragnaros",
		["Timbermaw Warder"] = "Garde Grumegueule",
		["Quartermaster Miranda Breechlock"] = "Intendante Miranda Coinceculasse",
		["Jack Sterling"] = "Jack Sterling",
		["Necrofiend"] = "Nécrofiel",
		["Timbermaw Mystic"] = "Mystique Grumegueule",
		["Timbermaw Woodbender"] = "Pliebois Grumegueule",
		["Grazle"] = "Grifleur",
		["Gorn One Eye"] = "Gorn le Borgne",
		["Salfa"] = "Salfa",
		["Meilosh"] = "Meilosh",
		["Kernda"] = "Kernda",
		["Scholomance Dark Summoner"] = "Invocateur des ténèbres de la Scholomanc",
		["Nefarian"] = "Nefarian",
		["Risen Guardian"] = "Gardien ressuscité",
		["Irondeep Shaman"] = "Chaman de Gouffrefer",
		["Irondeep Skullthumper"] = "Brise-crâne de Gouffrefer",
		["Whitewhisker Digger"] = "Terrassier Blanche-moustache",
		["Whitewhisker Geomancer"] = "Géomancien Blanche-moustache",
		["Whitewhisker Overseer"] = "Surveillant Blanche-moustache",
		["Bardu Sharpeye"] = "Bardu Oeil-vif",
		["Alexia Ironknife"] = "Alexia Coutel-de-fer",
		["Kirsta Deepshadow"] = "Kirsta Fondombre",
		["Cavalier Durgen"] = "Cavalier Durgen",
		["Huntsman Radley"] = "Veneur Radley",
		["Bloodshot"] = "Sanguin",
		["Mickey Levine"] = "Mickey Levine",
		["Nathaniel Dumah"] = "Nathaniel Dumah",
		["Spectral Marauder"] = "Maraudeur spectral",
		["Spectral Corpse"] = "Cadavre spectral",
		["Rattlegore"] = "Cliquettripes",
		["Scourge Summoning Crystal"] = "Cristal d'invocation du Fléau",
		["Taiga Wisemane"] = "Taiga Crin-sage",
		["Jessica Redpath"] = "Jessica Senterouge",
		["Servant of Weldon Barov"] = "Serviteur de Weldon Barov",
		["Servant of Alexi Barov"] = "Serviteur d'Alexi Barov",
		["Morloch"] = "Morloch",
		["Molten Giant"] = "Géant de lave",
		["Molten Destroyer"] = "Destructeur de lave",
		["Flamewaker"] = "Attise-flammes",
		["Flamewaker Priest"] = "Prêtre Attise-flammes",
		["Flamewaker Healer"] = "Soigneur Attise-flammes",
		["Flamewaker Elite"] = "Elite Attise-flammes",
		["Lava Annihilator"] = "Annihilateur de lave",
		["Firewalker"] = "Marchefeu",
		["Flameguard"] = "Garde des flammes",
		["Firelord"] = "Seigneur du feu",
		["Flame Imp"] = "Diablotin des flammes",
		["Core Hound"] = "Chien du Magma",
		["Core Rager"] = "Rageur du Magma",
		["Ancient Core Hound"] = "Ancien chien du Magma",
		["Snowblind Windcaller"] = "Imploratrice céleste Brûleglace",
		["Taskmaster Snivvle"] = "Sous-chef Snivvle",
		["Snowblind Ambusher"] = "Embusquée Brûleglace",
		["Winterax Witch Doctor"] = "Sorcier-docteur Hache-d?hiver",
		["Chal Fairwind"] = "Chal Doucebrise",
		["Hive'Ashi Stinger"] = "Aiguillonneur de la Ruche'Ashi",
		["Varian Wrynn"] = "Varian Wrynn",
		["Sarin Starlight"] = "Sarin Lumétoile",
		["Mor'vek"] = "Mor'vek",
		["Arin'sor"] = "Arin'sor",
		["Graw Cornerstone"] = "Graw Pierrangulaire",
		["Kriss Goldenlight"] = "Kriss Lumedor",
		["Rayan Dawnrisen"] = "Rayan Montelaube",
		["Adon"] = "Adon",
		["Joy Ar'nareth"] = "Joy Ar'nareth",
		["Coral Moongale"] = "Coral Trombelune",
		["Mirador"] = "Mirador",
		["Celes Earthborne"] = "Celes Terreporte",
		["Sar Browneye"] = "Sar Oeil-de-biche",
		["Hive'Ashi Worker"] = "Ouvrière de la Ruche'Ashi",
		["Hive'Ashi Defender"] = "Défenseur de la Ruche'Ashi",
		["Hive'Ashi Sandstalker"] = "Traqueuse des sables de la Ruche'As",
		["Hive'Ashi Swarmer"] = "Grouillante de la Ruche'Ashi",
		["Hive'Zora Waywatcher"] = "Patrouilleuse de la Ruche'Zora",
		["Hive'Zora Tunneler"] = "Tunnelière de la Ruche'Zora",
		["Hive'Zora Wasp"] = "Guêpe de la Ruche'Zora",
		["Hive'Zora Reaver"] = "Saccageuse de la Ruche'Zora",
		["Hive'Zora Hive Sister"] = "S?ur de la Ruche'Zora",
		["Hive'Regal Ambusher"] = "Embusquée de la Ruche'Regal",
		["Hive'Regal Burrower"] = "Fouisseuse de la Ruche'Regal",
		["Hive'Regal Spitfire"] = "Crache-feu de la Ruche'Regal",
		["Hive'Regal Slavemaker"] = "Esclavagiste de la Ruche'Regal",
		["Hive'Regal Hive Lord"] = "Seigneur de la Ruche'Regal",
		["Stonelash Scorpid"] = "Scorpide Cinglepierre",
		["Stonelash Pincer"] = "Pinceur Cinglepierre",
		["Stonelash Flayer"] = "Ecorcheur Cinglepierre",
		["Sand Skitterer"] = "Glisseuse du sable",
		["Rock Stalker"] = "Traqueuse des rochers",
		["Dredge Striker"] = "Frappeur térébrant",
		["Dredge Crusher"] = "Ecraseur térébrant",
		["Dust Stormer"] = "Tempétueux de poussière",
		["Cyclone Warrior"] = "Guerrier cyclone",
		["Desert Rumbler"] = "Grondeur du désert",
		["Desert Rager"] = "Enragé du désert",
		["Feran Strongwind"] = "Feran Vent-violent",
		["Blaise Montgomery"] = "Blaise Montgomery",
		["Gogo"] = "Gogo",
		["Meggi Peppinrocker"] = "Meggi Bringueballot",
		["Harlo Wigglesworth"] = "Harlo Valtrémousse",
		["Andi Lynn"] = "Andi Lynn",
		["Theradrim Shardling"] = "Eclaté Theradrim",
		["Theradrim Guardian"] = "Gardien Theradrim",
		["Deep Borer"] = "Fouisseur des profondeurs",
		["Mylentha Riverbend"] = "Mylentha Méandre-de-rivière",
		["Bessany Plainswind"] = "Bessany Vent-des-plaines",
		["Moren Riverbend"] = "Moren Méandre-de-rivière",
		["Bunthen Plainswind"] = "Bunthen Vent-des-plaines",
		["Tajarri"] = "Tajarri",
		["Silva Fil'naveth"] = "Silva Fil'naveth",
		["Rabine Saturna"] = "Rabine Saturna",
		["Dendrite Starblaze"] = "Dendrite Brasétoile",
		["Twilight Keeper Exeter"] = "Gardien du crépuscule Exeter",
		["Twilight Keeper Havunth"] = "Gardien du crépuscule Havunth",
		["Jarund Stoutstrider"] = "Jarund Rôdeur-vaillant",
		["Grum Redbeard"] = "Grum Barbe-rouge",
		["Voriya"] = "Voriya",
		["Moonglade Warden"] = "Gardien de Reflet-de-Lune",
		["Paige Felixe"] = "Paige Felixe",
		["Hakkari Priest"] = "Prêtre Hakkari",
		["Hakkari Witch Doctor"] = "Sorcier-docteur Hakkari",
		["Keeper Remulos"] = "Gardien Remulos",
		["Theodore Griffs"] = "Theodore Griffs",
		["Wildpaw Shaman"] = "Chaman Follepatte",
		["Wildpaw Mystic"] = "Mystique Follepatte",
		["Wildpaw Brute"] = "Brute Follepatte",
		["Wildpaw Alpha"] = "Alpha Follepatte",
		["Doomguard"] = "Garde funeste",
		["Azore Aldamort"] = "Azore Aldamort",
		["Buliwyf Stonehand"] = "Buliwyf Main-de-pierre",
		["Ilyenia Moonfire"] = "Ilyenia Lunéclat",
		["Woo Ping"] = "Woo Ping",
		["Sayoc"] = "Sayoc",
		["Ansekhwa"] = "Ansekhwa",
		["Archibald"] = "Archibald",
		["Grinning Dog"] = "Rictus",
		["Myranda the Hag"] = "Myranda la Mégère",
		["Spectral Attendant"] = "Domestique spectral",
		["Masat T'andr"] = "Masat T'andr",
		["Nathanos Blightcaller"] = "Nathanos le Flétrisseur",
		["Twilight Avenger"] = "Vengeur du crépuscule",
		["Twilight Geolord"] = "Géoseigneur du crépuscule",
		["Twilight Stonecaller"] = "Invocateur des pierres du crépuscule",
		["Twilight Master"] = "Maître du crépuscule",
		["Blighthound"] = "Chien chancreux",
		["Mercutio Filthgorger"] = "Mercutio Goinfrecrasse",
		["Crypt Robber"] = "Pilleur de crypte",
		["Borelgore"] = "Creusechair",
		["Duskwing"] = "Crépuscaile",
		["Crusader Lord Valdelmar"] = "Seigneur croisé Valdelmar",
		["Shardi"] = "Shardi",
		["Brakkar"] = "Brakkar",
		["Andruk"] = "Andruk",
		["Umber"] = "Umber",
		["Drek'Thar"] = "Drek'Thar",
		["Captain Galvangar"] = "Capitaine Galvangar",
		["Vanndar Stormpike"] = "Vanndar Foudrepique",
		["Captain Balinda Stonehearth"] = "Capitaine Balinda Gîtepierre",
		["Great Bear Spirit"] = "Esprit du Grand ours",
		["Great Cat Spirit"] = "Esprit du Grand félin",
		["Flamegor"] = "Flamegor",
		["Magmadar"] = "Magmadar",
		["Firemaw"] = "Gueule-de-feu",
		["Golemagg the Incinerator"] = "Golemagg l'Incinérateur",
		["Stormpike Herald"] = "Héraut foudrepique",
		["Frostwolf Herald"] = "Héraut Loup-de-givre",
		["Broodlord Lashlayer"] = "Seigneur des couvées Lanistaire",
		["Majordomo Executus"] = "Chambellan Executus",
		["Dargon"] = "Dargon",
		["Daeolyn Summerleaf"] = "Daeolyn Feuille-d'été",
		["Lorelae Wintersong"] = "Lorelae Chante-hiver",
		["Kharedon"] = "Kharedon",
		["Meliri"] = "Meliri",
		["Malvor"] = "Malvor",
		["My'lanna"] = "My'lanna",
		["Narianna"] = "Narianna",
		["Koiter"] = "Koiter",
		["Loganaar"] = "Loganaar",
		["Gor'marok the Ravager"] = "Gor'marok le Ravageur",
		["Stormpike Mountaineer"] = "Montagnard foudrepique",
		["Alliance Sentinel"] = "Sentinelle de l'Alliance",
		["Stormpike Defender"] = "Défenseur foudrepique",
		["Frostwolf Legionnaire"] = "Légionnaire loup-de-givre",
		["Frostwolf Warrior"] = "Guerrier Loup-de-givre",
		["Frostwolf Guardian"] = "Gardien loup-de-givre",
		["Baron Geddon"] = "Baron Geddon",
		["Garr"] = "Garr",
		["Lava Elemental"] = "Elémentaire de lave",
		["Stormpike Quartermaster"] = "Intendant foudrepique",
		["Frostwolf Quartermaster"] = "Intendant loup-de-givre",
		["Sulfuron Harbinger"] = "Messager de Sulfuron",
		["Firesworn"] = "Lige du feu",
		["Lava Reaver"] = "Saccageur de lave",
		["Lava Surger"] = "Surgisseur de lave",
		["Lucifron"] = "Lucifron",
		["Flamewaker Protector"] = "Protecteur Attise-flammes",
		["Drakan"] = "Drakan",
		["Duros"] = "Duros",
		["Great Shark"] = "Grand requin",
		["Mammoth Shark"] = "Requin mammouth",
		["Lord Tirion Fordring"] = "Seigneur Tirion Fordring",
		["Stormpike Guardsman"] = "Garde foudrepique",
		["Crimson Elite"] = "Elite cramoisie",
		["Onyxian Warder"] = "Gardien onyxien",
		["Guardian of Elune"] = "Gardien d'Elune",
		["Ice Totem"] = "Totem de glace",
		["Son of Flame"] = "Fils des flammes",
		["Winterax Axe Thrower"] = "Lanceur de haches Hache-d?hiver",
		["Winterax Shadow Hunter"] = "Chasseur des ombres Hache-d?hiver",
		["Winterax Hunter"] = "Chasseur Hache-d?hiver",
		["Korrak the Bloodrager"] = "Korrak le Ragesang",
		["Shadowglen Sentinel"] = "Sentinelle de Sombrevallon",
		["Tortured Druid"] = "Druide torturé",
		["Tortured Sentinel"] = "Sentinelle torturée",
		["Glordrum Steelbeard"] = "Glordrum Barbe-d'acier",
		["Martin Lindsey"] = "Martin Lindsey",
		["Shade of Ambermoon"] = "Ombre d'Ambrelune",
		["Princess Theradras"] = "Princesse Theradras",
		["Landslide"] = "Glissement de terrain",
		["Spitelash Raider"] = "Ecumeur Fouette-bile",
		["Spitelash Witch"] = "Sorcière Fouette-bile",
		["Primordial Behemoth"] = "Béhémoth primordial",
		["Thessala Hydra"] = "Hydre de Thessala",
		["Conquered Soul of the Blightcaller"] = "Ame conquise du Flétrisseur",
		["Noxious Slime"] = "Gelée nocive",
		["Cavern Shambler"] = "Traînard des grottes",
		["Celebras the Cursed"] = "Celebras le Maudit",
		["Lord Vyletongue"] = "Seigneur Vylelangue",
		["Meshlok the Harvester"] = "Meshlok le Moissonneur",
		["Zaetar's Spirit"] = "Esprit de Zaetar",
		["Spirit of Maraudos"] = "Esprit de Maraudos",
		["Spirit of Veng"] = "Esprit de Veng",
		["Infiltrator Hameya"] = "Infiltrateur Hameya",
		["Zaeldarr the Outcast"] = "Zaeldarr le Banni",
		["Mechanical Yeti"] = "Yéti mécanique",
		["Razorlash"] = "Tranchefouet",
		["Gehennas"] = "Gehennas",
		["Infected Mossflayer"] = "Ecorchemousse infecté",
		["Ziggurat Protector"] = "Protecteur de la ziggourat",
		["Slaughterhouse Protector"] = "Protecteur de l'abattoir",
		["Shazzrah"] = "Shazzrah",
		["Lava Spawn"] = "Engeance de lave",
		["Quel'Lithien Protector"] = "Protecteur Quel'Lithien",
		["Crimson Courier"] = "Messager cramoisi",
		["Shadowprey Guardian"] = "Gardien de Proie-de-l'Ombre",
		["Demetria"] = "Demetria",
		["Scarlet Trooper"] = "Soldat écarlate",
		["Wailing Spectre"] = "Spectre gémissant",
		["Damned Soul"] = "Ame damnée",
		["Unliving Caretaker"] = "Fossoyeur non-vivant",
		["Unliving Resident"] = "Résident non vivant",
		["Ley Sprite"] = "Fée Ley",
		["Mana Sprite"] = "FÃ©e de mana",
		["Augustus the Touched"] = "Augustus le Touché",
		["Large Vile Slime"] = "Grande gelée maléfique",
		["Doomguard Commander"] = "Commandant de la Garde funeste",
		["Lord Kazzak"] = "Seigneur Kazzak",
		["Blackwing Legionnaire"] = "Légionnaire de l'Aile noire",
		["Gordok Hyena"] = "Hyène gordok",
		["Blackwing Mage"] = "Mage de l'Aile noire",
		["Death Talon Dragonspawn"] = "Draconide Griffemort",
		["Flint Shadowmore"] = "Flint Ombrelongue",
		["Razorgore the Untamed"] = "Tranchetripe l'Indompté",
		["Blackwing Spellbinder"] = "Lieur de sort de l'Aile noire",
		["Blackwing Taskmaster"] = "Sous-chef de l'Aile noire",
		["Blackwing Warlock"] = "Démoniste de l'Aile noire",
		["Death Talon Wyrmguard"] = "Garde wyrm Griffemort",
		["Death Talon Overseer"] = "Surveillant Griffemort",
		["Death Talon Flamescale"] = "Flammécaille Griffemort",
		["Death Talon Seether"] = "Rongé Griffemort",
		["Death Talon Wyrmkin"] = "Wyrmide Griffemort",
		["Death Talon Captain"] = "Capitaine Griffemort",
		["Death Talon Hatcher"] = "Eveilleur Griffemort",
		["Arcanite Dragonling"] = "Petit dragon en arcanite",
		["Emeraldon Boughguard"] = "Gardien Emeraldon",
		["Emeraldon Tree Warder"] = "Gardien sylvestre Emeraldon",
		["Emeraldon Oracle"] = "Oracle Emeraldon",
		["Verdantine Boughguard"] = "Gardien de Verdantis",
		["Verdantine Oracle"] = "Oracle de Verdantis",
		["Verdantine Tree Warder"] = "Gardien sylvestre de Verdantis",
		["Melris Malagan"] = "Melris Malagan",
		["Justine Demalier"] = "Justine Demalier",
		["Dreamtracker"] = "Cherche-rêve",
		["Dreamroarer"] = "Hurle-rêve",
		["Dreamstalker"] = "Traquerêve",
		["Grethok the Controller"] = "Grethok le Contrôleur",
		["Jarrodenus"] = "Jarrodenus",
		["Mishellena"] = "Mishellena",
		["Reginald Windsor"] = "Reginald Windsor",
		["Bibilfaz Featherwhistle"] = "Bibilfaz Siffleplume",
		["Vhulgra"] = "Vhulgra",
		["Khaelyn Steelwing"] = "Khaelyn Aile-d'acier",
		["Georgia"] = "Georgia",
		["Onyxia's Elite Guard"] = "Garde d'élite d'Onyxia",
		["Faustron"] = "Faustron",
		["Lady Onyxia"] = "Dame Onyxia",
		["Captain Dirgehammer"] = "Capitaine Mornemartel",
		["Lieutenant Rachel Vaccar"] = "Lieutenant Rachel Vaccar",
		["Archmage Gaiman"] = "Archimage Gaiman",
		["Sergeant Major Skyshadow"] = "Sergent-major Ombreciel",
		["Master Sergeant Biggins"] = "Sergent-chef Socquet",
		["Captain O'Neal"] = "Capitaine O'Neal",
		["Lieutenant Karter"] = "Lieutenant Karter",
		["Lieutenant Jackspring"] = "Lieutenant Elascric",
		["Sergeant Major Clate"] = "Sergent-major Clate",
		["Guard Quine"] = "Garde Quine",
		["Guard Hammon"] = "Garde Hammond",
		["Legionnaire Teena"] = "Légionnaire Teena",
		["Blood Guard Hini'wana"] = "Garde de sang Hini'Wana",
		["Advisor Willington"] = "Conseiller Willington",
		["Chieftain Earthbind"] = "Chef Lien-terrestre",
		["Lady Palanseer"] = "Dame Palanseer",
		["Brave Stonehide"] = "Brave Cuir-de-pierre",
		["Stone Guard Zarg"] = "Garde de pierre Zarg",
		["First Sergeant Hola'mahi"] = "Sergent Hola'mahi",
		["Raider Bork"] = "Ecumeur Bork",
		["Grunt Korf"] = "Grunt Korf",
		["Grunt Bek'rah"] = "Grunt Bek'rah",
		["Sergeant Ba'sha"] = "Sergent Ba'sha",
		["Chimaerok"] = "Chimaerok",
		["Arcane Chimaerok"] = "Chimaerok des arcanes",
		["Chimaerok Devourer"] = "Dévoreur Chimaerok",
		["Lord Lakmaeran"] = "Seigneur Lakmaeran",
		["Officer Areyn"] = "Officier Areyn",
		["Greshka"] = "Greshka",
		["Axtroz"] = "Axtroz",
		["Somnus"] = "Somnus",
		["Spirit Of Redemption"] = "Esprit de rédemption",
		["Doctor Gregory Victor"] = "Docteur Gregory Victor",
		["Imp Minion"] = "Diablotin serviteur",
		["Critically Injured Soldier"] = "Soldat agonisant",
		["Critically Injured Alliance Soldier"] = "Soldat de l'Alliance agonisant",
		["Doctor Gustaf VanHowzen"] = "Docteur Gustaf VanHowzen",
		["Jase Farlane"] = "Jase Farlane",
		["Leonard Porter"] = "Leonard Porter",
		["Werg Thickblade"] = "Werg Largelame",
		["Lokhtos Darkbargainer"] = "Lokhtos Sombrescompte",
		["Zannok Hidepiercer"] = "Zannok Percecuir",
		["Blimo Gadgetspring"] = "Blimo Gadgettaressort",
		["Gigget Zipcoil"] = "Giguette Filebobine",
		["Nergal"] = "Nergal",
		["Vaelastrasz the Corrupt"] = "Vaelastrasz le Corrompu",
		["Warpwood Crusher"] = "Cogneur Crochebois",
		["Whip Lasher"] = "Claque-fouet",
		["Gordok Mastiff"] = "Mastiff gordok",
		["Umi Thorson"] = "Umi Thorson",
		["Keetar"] = "Keetar",
		["Irondeep Guard"] = "Garde de Gouffrefer",
		["Irondeep Raider"] = "Ecumeur de Gouffrefer",
		["Bixi Wobblebonk"] = "Bixi Oscillognon",
		["Myrokos Silentform"] = "Myrokos Sourdeforme",
		["Aggi Rumblestomp"] = "Aggi Grondécrase",
		["Coldmine Invader"] = "Envahisseur de Froidemine",
		["Masha Swiftcut"] = "Masha Vivecoupe",
		["Coldmine Guard"] = "Garde de Froidemine",
		["Coldmine Explorer"] = "Explorateur de Froidemine",
		["Coldmine Surveyor"] = "Géomètre de Froidemine",
		["Irondeep Surveyor"] = "Géomètre de Gouffrefer",
		["Irondeep Explorer"] = "Explorateur de Gouffrefer",
		["Alliance Spirit Guide"] = "Guide spirituel de l'Alliance",
		["Horde Spirit Guide"] = "Guide spirituel de la Horde",
		["Crimson Bodyguard"] = "Garde du corps cramoisi",
		["Hive'Ashi Drone"] = "Bourdon de la Ruche'Ashi",
		["Lieutenant Rugba"] = "Lieutenant Rugba",
		["Lieutenant Spencer"] = "Lieutenant Spencer",
		["Commander Randolph"] = "Commandant Randolph",
		["Commander Dardosh"] = "Commandant Dardosh",
		["Lieutenant Stronghoof"] = "Lieutenant Sabot-puissant",
		["Lieutenant Vol'talar"] = "Lieutenant Vol'talar",
		["Lieutenant Grummus"] = "Lieutenant Grummus",
		["Lieutenant Murp"] = "Lieutenant Murp",
		["Lieutenant Lewis"] = "Lieutenant Lewis",
		["Syndicate Brigand"] = "Brigand du Syndicat",
		["Syndicate Agent"] = "Agent du Syndicat",
		["Syndicate Master Ryson"] = "Maître du Syndicat Ryson",
		["Commander Malgor"] = "Commandant Malgor",
		["Commander Mulfort"] = "Commandant Mulfort",
		["Commander Louis Philips"] = "Commandant Louis Philips",
		["Deathstalker Agent"] = "Agent Nécrotraqueur",
		["Carrion Swarmer"] = "Grouillant putride",
		["Aerie Gryphon"] = "Griffon du Nid-de-l'Aigle",
		["Smith Regzar"] = "Forgeron Regzar",
		["Vahgruk"] = "Vahgruk",
		["War Rider"] = "Chevaucheur de guerre",
		["Wing Commander Guse"] = "Chef d'escadrille Guse",
		["Wing Commander Jeztor"] = "Chef d'escadrille Jeztor",
		["Wing Commander Mulverick"] = "Chef d'escadrille Mulverick",
		["Phase Lasher"] = "Flagellant phasique",
		["Fel Lash"] = "Gangrefouet",
		["Gaelden Hammersmith"] = "Gaelden Forgemartel",
		["Thanthaldis Snowgleam"] = "Thanthaldis Luireneige",
		["Grunnda Wolfheart"] = "Grunnda Coeur-de-loup",
		["Jekyll Flandring"] = "Jekyll Flandring",
		["Layo Starstrike"] = "Layo Heurtétoile",
		["Ryson's Eye in the Sky"] = "Oeil céleste de Ryson",
		["Primalist Thurloga"] = "Primaliste Thurloga",
		["Lokholar the Ice Lord"] = "Lokholar le Seigneur de glace",
		["Murgot Deepforge"] = "Murgot Forge-profonde",
		["Wildspawn Imp"] = "Diablotin Follengeance",
		["Duke Hydraxis"] = "Duc Hydraxis",
		["Discordant Surge"] = "Vague discordante",
		["Hydrospawn"] = "Hydrogénos",
		["Noxxion"] = "Noxcion",
		["Lord Tony Romano"] = "Seigneur Tony Romano",
		["Frostwolf Shaman"] = "Chaman loup-de-givre",
		["Death Lash"] = "Fouet-mortel",
		["Lieutenant Largent"] = "Lieutenant Largent",
		["Lieutenant Stouthandle"] = "Lieutenant Rudemanche",
		["Lieutenant Greywand"] = "Lieutenant Grisebaguette",
		["Lieutenant Lonadin"] = "Lieutenant Lonadin",
		["Lieutenant Mancuso"] = "Lieutenant Mancuso",
		["Hive'Ashi Ambusher"] = "Embusquée de la Ruche'Ashi",
		["Coldmine Peon"] = "Péon de Froidemine",
		["Coldmine Miner"] = "Mineur de Froidemine",
		["Commander Mortimer"] = "Commandant Mortimer",
		["Commander Duffy"] = "Commandant Duffy",
		["Commander Karl Philips"] = "Commandant Karl Philips",
		["Hydraxian Honor Guard"] = "Garde d?honneur hydraxien",
		["Subterranean Diemetradon"] = "Dimeurtrodon souterrain",
		["Seasoned Guardsman"] = "Garde aguerri",
		["Seasoned Mountaineer"] = "Montagnard aguerri",
		["Seasoned Defender"] = "Défenseur aguerri",
		["Seasoned Sentinel"] = "Sentinelle aguerrie",
		["Seasoned Guardian"] = "Gardien aguerri",
		["Seasoned Legionnaire"] = "Légionnaire aguerri",
		["Seasoned Warrior"] = "Guerrier aguerri",
		["Veteran Defender"] = "Défenseur vétéran",
		["Veteran Guardian"] = "Gardien vétéran",
		["Veteran Guardsman"] = "Garde vétéran",
		["Veteran Legionnaire"] = "Légionnaire vétéran",
		["Veteran Mountaineer"] = "Montagnard vétéran",
		["Veteran Sentinel"] = "Sentinelle vétéran",
		["Veteran Warrior"] = "Guerrier vétéran",
		["Stormpike Mine Layer"] = "Poseur de mines Foudrepique",
		["Frostwolf Mine Layer"] = "Poseur de mines Loup-de-givre",
		["Stormpike Bowman"] = "Franc-archer foudrepique",
		["Frostwolf Bowman"] = "Franc-archer loup-de-givre",
		["Master Engineer Zinfizzlex"] = "Maître ingénieur Zinfizzlex",
		["Frostwolf Shredder Unit"] = "Déchiqueteur Loup-de-givre",
		["Irondeep Miner"] = "Mineur de Gouffrefer",
		["Irondeep Peon"] = "Péon de Gouffrefer",
		["Stormpike Shredder Unit"] = "Déchiqueteur Foudrepique",
		["Sagorne Creststrider"] = "Sagorne Rôdeur-des-crêtes",
		["Ivus the Forest Lord"] = "Ivus le Seigneur de la forêt",
		["Champion Guardian"] = "Gardien champion",
		["Champion Defender"] = "Défenseur champion",
		["Champion Guardsman"] = "Garde champion",
		["Champion Legionnaire"] = "Légionnaire champion",
		["Champion Mountaineer"] = "Montagnard champion",
		["Champion Sentinel"] = "Sentinelle champion",
		["Champion Warrior"] = "Guerrier champion",
		["Wing Commander Ichman"] = "Chef d'escadrille Ichman",
		["Wing Commander Slidore"] = "Chef d'escadrille Slidore",
		["Wing Commander Vipore"] = "Chef d'escadrille Vipore",
		["Frostwolf Wolf Rider"] = "Chevaucheur de loup Loup-de-givre",
		["Frostwolf Wolf Rider Commander"] = "Commandant chevaucheur de loup loup-de-g",
		["Arch Druid Renferal"] = "Archidruide Ranfarouche",
		["Druid of the Grove"] = "Druide du Bosquet",
		["Field Marshal Teravaine"] = "Grand maréchal Teravaine",
		["Corporal Noreg Stormpike"] = "Caporal Noreg Foudrepique",
		["Sergeant Yazra Bloodsnarl"] = "Sergent Yazra Gronde-sang",
		["Warmaster Garrick"] = "Maître de guerre Garrick",
		["Noxxion's Spawn"] = "Rejeton de Noxcion",
		["Stormpike Commando"] = "Commando Foudrepique",
		["Seasoned Commando"] = "Commando aguerri",
		["Veteran Commando"] = "Commando vétéran",
		["Champion Commando"] = "Commando champion",
		["Frostwolf Reaver"] = "Ravageur Loup-de-givre",
		["Seasoned Reaver"] = "Ravageur aguerri",
		["Veteran Reaver"] = "Ravageur vétéran",
		["Champion Reaver"] = "Ravageur champion",
		["Seasoned Coldmine Guard"] = "Garde de Froidemine aguerri",
		["Veteran Coldmine Guard"] = "Garde de Froidemine vétéran",
		["Champion Coldmine Guard"] = "Garde de Froidemine champion",
		["Seasoned Coldmine Surveyor"] = "Géomètre de Froidemine aguerri",
		["Veteran Coldmine Surveyor"] = "Géomètre de Froidemine vétéran",
		["Champion Coldmine Surveyor"] = "Géomètre de Froidemine champion",
		["Seasoned Irondeep Explorer"] = "Explorateur aguerri de Gouffrefer",
		["Veteran Irondeep Explorer"] = "Explorateur de Gouffrefer vétéran",
		["Champion Irondeep Explorer"] = "Explorateur de Gouffrefer champion",
		["Seasoned Irondeep Raider"] = "Ecumeur de Gouffrefer aguerri",
		["Veteran Irondeep Raider"] = "Ecumeur de Gouffrefer vétéran",
		["Champion Irondeep Raider"] = "Ecumeur de Gouffrefer champion",
		["Seasoned Coldmine Explorer"] = "Explorateur de Froidemine aguerri",
		["Veteran Coldmine Explorer"] = "Explorateur de Froidemine vétéran",
		["Champion Coldmine Explorer"] = "Explorateur de Froidemine champion",
		["Seasoned Coldmine Invader"] = "Envahisseur de Froidemine aguerri",
		["Veteran Coldmine Invader"] = "Envahisseur de Froidemine vétéran",
		["Champion Coldmine Invader"] = "Envahisseur de Froidemine champion",
		["Seasoned Irondeep Guard"] = "Garde de Gouffrefer aguerri",
		["Veteran Irondeep Guard"] = "Garde de Gouffrefer vétéran",
		["Champion Irondeep Guard"] = "Garde de Gouffrefer champion",
		["Seasoned Irondeep Surveyor"] = "Géomètre de Gouffrefer aguerri",
		["Veteran Irondeep Surveyor"] = "Géomètre de Gouffrefer vétéran",
		["Champion Irondeep Surveyor"] = "Géomètre de Gouffrefer champion",
		["Stormpike Ram Rider"] = "Chevaucheur de bélier Foudrepique",
		["Stormpike Ram Rider Commander"] = "Commandant chevaucheur de bélier foudrep",
		["Rotgrip"] = "Grippe-charogne",
		["Frostwolf Explosives Expert"] = "Expert en explosifs Loup-de-givre",
		["Stormpike Explosives Expert"] = "Expert en explosifs Foudrepique",
		["Stolid Snapjaw"] = "Gueule d'acier impassible",
		["Tinkerer Gizlock"] = "Bricoleur Kadenaz",
		["Frostwolf Stable Master"] = "Maître des écuries loup-de-givre",
		["Stormpike Stable Master"] = "Maître des écuries foudrepique",
		["Stabled Frostwolf"] = "Loup-de-givre à l'écurie",
		["Stabled Alterac Ram"] = "Bélier d'Alterac à l'écurie",
		["Selendra"] = "Selendra",
		["Celebras the Redeemed"] = "Celebras le Racheté",
		["Veng"] = "Veng",
		["Maraudos"] = "Maraudos",
		["Corporal Teeka Bloodsnarl"] = "Caporal Teeka Gronde-sang",
		["Sergeant Durgen Stormpike"] = "Sergent Durgen Foudrepique",
		["Mountaineer Boombellow"] = "Montagnard Souffleboum",
		["Jotek"] = "Jotek",
		["Prospector Stonehewer"] = "Prospectrice Taillepierre",
		["Voggah Deathgrip"] = "Voggah Mortepoigne",
		["Royal Dreadguard"] = "Gardeffroi royal",
		["Warmaster Laggrond"] = "Maître de guerre Laggrond",
		["Lieutenant Haggerdin"] = "Lieutenant Haggerdin",
		["Frostwolf Ambassador Rokhstrom"] = "Ambassadrice Rokhstrom des Loups-de-givr",
		["Lieutenant Rotimer"] = "Lieutenant Rotimer",
		["Scalebeard"] = "Barbe-d'écailles",
		["Dire Maul Crystal Totem"] = "Totem de cristal de Hache-tripes",
		["Izzy Coppergrab"] = "Izzy Fauchsou",
		["Winterax Mystic"] = "Mystique Hache-d?hiver",
		["Winterax Warrior"] = "Guerrier Hache-d?hiver",
		["Winterax Seer"] = "Prophète Hache-d?hiver",
		["Alterac Yeti"] = "Yéti d'Alterac",
		["Blackwing Technician"] = "Technicien de l'Aile noire",
		["Chromaggus"] = "Chromaggus",
		["Corrupted Red Whelp"] = "Dragonnet rouge corrompu",
		["Corrupted Green Whelp"] = "Dragonnet vert corrompu",
		["Corrupted Blue Whelp"] = "Dragonnet bleu corrompu",
		["Corrupted Bronze Whelp"] = "Dragonnet bronze corrompu",
		["Enraged Felguard"] = "Gangregarde enragé",
		["Massive Geyser"] = "Geyser massif",
		["Bounty Hunter Kolark"] = "Chasseur de primes Kolark",
		["Artilleryman Sheldonore"] = "Artilleur Mitraillemine",
		["Najak Hexxen"] = "Najak Maléficas",
		["Ravak Grimtotem"] = "Ravak Totem-sinistre",
		["Athramanis"] = "Athramanis",
		["Dirk Swindle"] = "Dirk Lembrouille",
		["Lord Angler"] = "Seigneur Baudroie",
		["Ironbark the Redeemed"] = "Ecorcefer le Racheté",
		["Blue Drakonid"] = "Drakônide bleu",
		["Green Drakonid"] = "Drakônide vert",
		["Bronze Drakonid"] = "Drakônide bronze",
		["Red Drakonid"] = "Drakônide rouge",
		["Black Drakonid"] = "Drakônide noir",
		["Frostwolf Bloodhound"] = "Limier loup-de-givre",
		["Stormpike Owl"] = "Hibou foudrepique",
		["Stormpike Battleguard"] = "Garde de guerre foudrepique",
		["Frostwolf Battleguard"] = "Garde de guerre loup-de-givre",
		["Chromatic Drakonid"] = "Drakônide chromatique",
		["Petrified Guardian"] = "Gardien pétrifié",
		["Kor'kron Elite"] = "Kor'kron d'élite",
		["Ferra"] = "Ferra",
		["Guard Fengus"] = "Garde Fengus",
		["Stomper Kreeg"] = "Kreeg le Marteleur",
		["Guard Slip'kik"] = "Garde Slip'kik",
		["Cho'Rush the Observer"] = "Cho'Rush l'Observateur",
		["Captain Kromcrush"] = "Capitaine Kromcrush",
		["Guard Mol'dar"] = "Garde Mol'dar",
		["Lethtendris"] = "Lethtendris",
		["Field Repair Bot 74A"] = "Robot réparateur 74A",
		["Knot Thimblejack"] = "Noué Dédodevie",
		["Death Howl"] = "Hurlemort",
		["Alshirr Banebreath"] = "Alshirr Souffléau",
		["Ragepaw"] = "Ragepatte",
		["Olm the Wise"] = "Olm la Sage",
		["Mongress"] = "Mongress",
		["The Ongar"] = "L'Ongar",
		["Highlord Demitrian"] = "Généralissime Demitrian",
		["Earthcaller Franzahl"] = "Implorateur de la terre Franzahl",
		["Pimgib"] = "Pimgib",
		["Hydroling"] = "Hydrogénide",
		["Gordok Bushwacker"] = "Gordok Guet-apens",
		["Mizzle the Crafty"] = "Mizzle l'Ingénieux",
		["Pusillin"] = "Pusillin",
		["Azj'Tordin"] = "Azj'Tordin",
		["Shen'dralar Ancient"] = "Ancienne de Shen'Dralar",
		["Thornling"] = "Epineux",
		["Thief Catcher Shadowdelve"] = "Grippe-larrons Fouillelombre",
		["Shen'dralar Spirit"] = "Esprit Shen'dralar",
		["Thief Catcher Farmountain"] = "Grippe-larrons Mont-lointain",
		["Thief Catcher Thunderbrew"] = "Grippe-larrons Tonnebière",
		["Lorekeeper Lydros"] = "Gardien du savoir Lydros",
		["Shen'dralar Zealot"] = "Zélote Shen'dralar",
		["Cadaverous Worm"] = "Ver cadavérique",
		["Shen'dralar Provisioner"] = "Approvisionneur Shen'dralar",
		["Winterfall Ambusher"] = "Embusqué Tombe-hiver",
		["Sage Korolusk"] = "Sage Korolusk",
		["Scholar Runethorn"] = "Erudite Roncerune",
		["Scout Stronghand"] = "Eclaireur Fortemain",
		["Scout Manslayer"] = "Eclaireur Tueuse-d'homme",
		["Scout Tharr"] = "Eclaireur Tharr",
		["Huntress Skymane"] = "Chasseresse Crins-de-ciel",
		["Huntress Ravenoak"] = "Chasseresse Corvichêne",
		["Huntress Leafrunner"] = "Chasseresse Coursefeuille",
		["Lorekeeper Javon"] = "Gardien du savoir Javon",
		["Lorekeeper Mykos"] = "Gardienne du savoir Mykos",
		["Lorekeeper Kildrath"] = "Gardien du savoir Kildrath",
		["Doomguard Minion"] = "Serviteur de la Garde funeste",
		["Wandering Eye of Kilrogg"] = "?il errant de Kilrogg",
		["Lothos Riftwaker"] = "Lothos Ouvrefaille",
		["Rogue Black Drake"] = "Drake noir insoumis",
		["Netherwalker"] = "Marcheur du Néant",
		["Overlord Runthak"] = "Seigneur Runthak",
		["Major Mattingly"] = "Major Mattingly",
		["Griniblix the Spectator"] = "Griniblix le Spectateur",
		["Eye of Immol'thar"] = "?il d'Immol'thar",
		["Mana Burst"] = "Explosion de mana",
		["Eldreth Darter"] = "Fléchetteuse Eldreth",
		["Arcane Torrent"] = "Torrent arcanique",
		["Arcane Feedback"] = "Réaction arcanique",
		["Master Elemental Shaper Krixix"] = "Maître élémentaire Krixix le Sculpteur",
		["Seeker Cromwell"] = "Chercheur Cromwell",
		["Seeker Nahr"] = "Chercheur Nahr",
		["Seeker Thompson"] = "Chercheuse Thompson",
		["Officer Jaxon"] = "Officier Jaxon",
		["Prince Thunderaan"] = "Prince Tonneraan",
		["Mor'zul Bloodbringer"] = "Mor'zul Porte-sang",
		["Gorzeeki Wildeyes"] = "Gorzeeki Fouregard",
		["Officer Pomeroy"] = "Officier Pomeroy",
		["Officer Brady"] = "Officier Brady",
		["Hunter Sagewind"] = "Chasseur Vent-de-sagesse",
		["Hunter Ragetotem"] = "Chasseur Totem-de-rage",
		["Hunter Thunderhorn"] = "Chasseur Corne-tonnerre",
		["Enslaved Doomguard Commander"] = "Commandant de la Garde funeste asservi",
		["The Windreaver"] = "Le Déchirevent",
		["Whirling Invader"] = "Envahisseur tourbillonnant",
		["Blackwing Guardsman"] = "Garde de l'Aile noire",
		["Princess Tempestria"] = "Princesse Tempestria",
		["Watery Invader"] = "Envahisseur aqueux",
		["Blazing Invader"] = "Envahisseur flamboyant",
		["Baron Charr"] = "Baron Charr",
		["Thundering Invader"] = "Envahisseur grondant",
		["Daio the Decrepit"] = "Daio le Décrépit",
		["Avalanchion"] = "Avalanchion",
		["Kroshius"] = "Kroshius",
		["Niby the Almighty"] = "Niby le Tout-puissant",
		["Impsy"] = "Démonet",
		["Setis"] = "Setis",
		["Gretheer"] = "Gretheer",
		["Lapress"] = "Lapress",
		["Zora"] = "Zora",
		["Rex Ashil"] = "Rex Ashil",
		["Krellack"] = "Krellack",
		["Grubthor"] = "Grubthor",
		["Huricanian"] = "Ouraganien",
		["Twilight Lord Everun"] = "Seigneur du crépuscule Everun",
		["Xorothian Imp"] = "Diablotin xorothien",
		["Dread Guard"] = "Garde de l'effroi",
		["Injured Peasant"] = "Paysan blessé",
		["Plagued Peasant"] = "Paysan contaminé",
		["Scourge Footsoldier"] = "Fantassin du Fléau",
		["Scourge Archer"] = "Archer du Fléau",
		["Eris Havenfire"] = "Eris Havrefeu",
		["J'eevee"] = "J'eevee",
		["Xorothian Dreadsteed"] = "Destrier de l'effroi xorothien",
		["The Cleaner"] = "Le Nettoyeur",
		["Dreadsteed Spirit"] = "Esprit de Destrier de l'effroi",
		["Lord Hel'nurath"] = "Seigneur Hel'nurath",
		["High Priest Venoxis"] = "Grand prêtre Venoxis",
		["Short John Mithril"] = "Court John Mithril",
		["High Priest Thekal"] = "Grand prêtre Thekal",
		["High Priestess Mar'li"] = "Grande prêtresse Mar'li",
		["Shadowed Spirit"] = "Esprit de l'ombre",
		["Corrupted Spirit"] = "Esprit corrompu",
		["Malicious Spirit"] = "Esprit malveillant",
		["Banal Spirit"] = "Esprit banal",
		["High Priestess Arlokk"] = "Grande prêtresse Arlokk",
		["Death Knight Darkreaver"] = "Chevalier de la mort Ravassombre",
		["High Priestess Jeklik"] = "Grande prêtresse Jeklik",
		["Aspect of Banality"] = "Aspect de banalité",
		["Aspect of Corruption"] = "Aspect de corruption",
		["Aspect of Malice"] = "Aspect de malveillance",
		["Aspect of Shadow"] = "Aspect d'ombre",
		["Ur'dan"] = "Ur'dan",
		["Ulathek"] = "Ulathek",
		["Vartrus the Ancient"] = "L'Ancien Vartrus",
		["Stoma the Ancient"] = "L'Ancien Stoma",
		["Hastat the Ancient"] = "L'Ancien Hastat",
		["Simone the Inconspicuous"] = "Simona la Discrète",
		["Precious"] = "Bijou",
		["Franklin the Friendly"] = "Franklin l'Amical",
		["Solenor the Slayer"] = "Solenor le Tueur",
		["Artorius the Amiable"] = "Artorius l'Aimable",
		["Razzashi Venombrood"] = "Fils du venin Razzashi",
		["Simone the Seductress"] = "Simona la Séductrice",
		["Klinfran the Crazed"] = "Klinfran l'Enragé",
		["Artorius the Doombringer"] = "Artorius le Néfaste",
		["Nelson the Nice"] = "Nelson le Gentil",
		["Precious the Devourer"] = "Bijou le Dévoreur",
		["Terrordale Spirit"] = "Esprit de Val-Terreur",
		["Ancient Equine Spirit"] = "Ancien esprit équin",
		["Derotain Mudsipper"] = "Derotain Siroteboue",
		["Darkreaver's Fallen Charger"] = "Destrier déchu de Ravassombre",
		["Sergeant Thunderhorn"] = "Sergent Corne-tonnerre",
		["Ebonroc"] = "Rochébène",
		["Zapped Shore Strider"] = "Marcheur des rivages zappé",
		["Zapped Land Walker"] = "Marcheur des terres zappé",
		["Bone Construct"] = "Assemblage d'os",
		["Overseer Maltorius"] = "Surveillant Maltorius",
		["Thorium Brotherhood Lookout"] = "Guetteur de la Confrérie du thorium",
		["Master Smith Burninate"] = "Maître forgeron Brûlotin",
		["Overseer Oilfist"] = "Surveillant Huilepoing",
		["Taskmaster Scrange"] = "Sous-chef Scrange",
		["Hansel Heavyhands"] = "Hansel Lourdemains",
		["Evonice Sootsmoker"] = "Evonice Fumesuie",
		["Lookout Captain Lolo Longstriker"] = "Capitaine de guet Lolo Heurteloin",
		["Zapped Wave Strider"] = "Marcheur des vagues zappé",
		["Zapped Deep Strider"] = "Marcheur des profondeurs zappé",
		["Zapped Cliff Giant"] = "Géant des falaises zappé",
		["Stinglasher"] = "Fouet-cuisant",
		["Corrupted Fire Nova Totem V"] = "Totem Nova de feu V corrompu",
		["Corrupted Stoneskin Totem VI"] = "Totem de Peau de pierre VI corrompu",
		["Corrupted Healing Stream Totem V"] = "Totem guérisseur V corrompu",
		["Corrupted Windfury Totem III"] = "Totem Furie-des-vents III corrompu",
		["Corrupted Infernal"] = "Infernal corrompu",
		["Balzaphon"] = "Balzaphon",
		["Revanchion"] = "Revanchion",
		["Scorn"] = "Dédain",
		["Lord Blackwood"] = "Seigneur Noirbois",
		["Silverwing Elite"] = "Soldat d'élite d'Aile-argent",
		["Horde Elite"] = "Soldat d'élite de la Horde",
		["High Overlord Saurfang"] = "Haut seigneur Saurcroc",
		["Field Marshal Afrasiabi"] = "Grand maréchal Afrasiabi",
		["Revantusk Watcher"] = "Gardien Vengebroche",
		["Lard"] = "Lard",
		["PvP CTF Credit Marker"] = "Marqueur JcJ JdD",
		["Sentinel Farsong"] = "Sentinelle Chanteloin",
		["Revantusk Drummer"] = "Tambour Vengebroche",
		["Primal Torntusk"] = "L'Aînée Brocherompue",
		["Mystic Yayo'jin"] = "Yayo'jin la Mystique",
		["Zap Farflinger"] = "Zap Flanqueloin",
		["Jhordy Lapforge"] = "Jhordy Lanforge",
		["Vilebranch Kidnapper"] = "Kidnappeur Vilebranche",
		["Gurubashi Bat Rider"] = "Chevaucheur de sanguinaire",
		["Illiyana Moonblaze"] = "Illiyana Braselune",
		["Kelm Hargunth"] = "Kelm Hargunth",
		["Elder Torntusk"] = "L'Ancien Brocherompue",
		["Creeping Doom"] = "Mort rampante",
		["Dun Baldar North Marshal"] = "Maréchal de Dun Baldar nord",
		["Dun Baldar South Marshal"] = "Maréchal de Dun Baldar sud",
		["Icewing Marshal"] = "Maréchal de l'Aile de glace",
		["Stonehearth Marshal"] = "Maréchal de Gîtepierre",
		["Iceblood Marshal"] = "Maréchal de Glace-sang",
		["Tower Point Marshal"] = "Maréchal de la Tour de la halte",
		["East Frostwolf Marshal"] = "Maréchal Loup-de-givre est",
		["West Frostwolf Marshal"] = "Maréchal Loup-de-givre ouest",
		["Dun Baldar North Warmaster"] = "Maître de guerre de Dun Baldar nord",
		["Dun Baldar South Warmaster"] = "Maître de guerre de Dun Baldar sud",
		["East Frostwolf Warmaster"] = "Maître de guerre loup-de-givre est",
		["Iceblood Warmaster"] = "Maître de guerre de Glace-sang",
		["Icewing Warmaster"] = "Maître de guerre de l'Aile de glace",
		["Stonehearth Warmaster"] = "Maître de guerre de Gîtepierre",
		["Tower Point Warmaster"] = "Maître de guerre de la Tour de la halte",
		["West Frostwolf Warmaster"] = "Maître de guerre loup-de-givre ouest",
		["Captain Shatterskull"] = "Capitaine Fracasse-crâne",
		["Razzashi Raptor"] = "Raptor Razzashi",
		["Sayge"] = "Sayge",
		["Silas Darkmoon"] = "Silas Sombrelune",
		["Withered Mistress"] = "Maîtresse desséchée",
		["Sacrificed Troll"] = "Troll sacrifié",
		["Burth"] = "Burth",
		["Yebb Neblegear"] = "Yebb Becquabrach",
		["Kerri Hicks"] = "Kerri Hicks",
		["Chronos"] = "Chronos",
		["Hakkar"] = "Hakkar",
		["Rinling"] = "Rinling",
		["Blood Steward of Kirtonos"] = "Régisseuse sanglante de Kirtonos",
		["Emissary Roman'khan"] = "Emissaire Roman'khan",
		["Molthor"] = "Molthor",
		["Zandalar Headshrinker"] = "Réducteur de têtes Zandalar",
		["Razzashi Skitterer"] = "Glisseuse Razzashi",
		["Atal'ai Mistress"] = "Maîtresse atal'ai",
		["Voodoo Slave"] = "Esclave vaudou",
		["Parasitic Serpent"] = "Serpent parasite",
		["Ysondre"] = "Ysondre",
		["Lethon"] = "Léthon",
		["Emeriss"] = "Emeriss",
		["Taerar"] = "Taerar",
		["Jin'rokh the Breaker"] = "Jin'rokh le Briseur",
		["Al'tabim the All-Seeing"] = "Al'tabim Qui-voit-tout",
		["Maywiki of Zuldazar"] = "Maywiki de Zuldazar",
		["Falthir the Sightless"] = "Falthir le Sans-vue",
		["Exzhal"] = "Exzhal",
		["Zandalar Enforcer"] = "Massacreur Zandalar",
		["Captured Hakkari Zealot"] = "Zélote Hakkari capturé",
		["Rin'wosho the Trader"] = "Rin'wosho le marchand",
		["Kartra Bloodsnarl"] = "Kartra Gronde-sang",
		["Guse's War Rider"] = "Chevaucheur de guerre de Guse",
		["Jeztor's War Rider"] = "Chevaucheur de guerre de Jeztor",
		["Mulverick's War Rider"] = "Chevaucheur de guerre de Mulverick",
		["Slidore's Gryphon"] = "Griffon de Slidore",
		["Ichman's Gryphon"] = "Griffon d'Ichman",
		["Vipore's Gryphon"] = "Griffon de Vipore",
		["Frenzied Bloodseeker Bat"] = "Chauve-souris sanguinaire frénétique",
		["Elfarran"] = "Elfarran",
		["Lylandris"] = "Lylandris",
		["Field Marshal Oslight"] = "Grand maréchal Oslight",
		["Sergeant Maclear"] = "Sergent Maclear",
		["Shade of Jin'do"] = "Ombre de Jin'do",
		["Powerful Healing Ward"] = "Gardien guérisseur puissant",
		["Ohgan"] = "Ohgan",
		["Deze Snowbane"] = "Deze Plaie-des-neiges",
		["Sir Malory Wheeler"] = "Sire Malory Charron",
		["Lady Hoteshem"] = "Dame Hoteshem",
		["Wagner Hammerstrike"] = "Wagner Frappemartel",
		["Javnir Nashak"] = "Javnir Nashak",
		["Deathmaster Dwire"] = "Nécromaître Dwire",
		["Deathstalker Mortis"] = "Nécrotraqueur Mortis",
		["Spawn of Mar'li"] = "Rejetons de Mar'li",
		["Zanza the Restless"] = "Zanza le Sans-Repos",
		["Zulian Crocolisk"] = "Crocilisque zulien",
		["Zulian Stalker"] = "Traqueur zulien",
		["Zulian Guardian"] = "Gardien zulien",
		["Vinchaxa"] = "Vinchaxa",
		["Zandalarian Emissary"] = "Emissaire zandalarien",
		["Riggle Bassbait"] = "Riggle Perchappât",
		["Jang"] = "Jang",
		["Fishbot 5000"] = "Pêchebot 5000",
		["Servant of the Hand"] = "Serviteur de la Main",
		["Gri'lek"] = "Gri'lek",
		["Hazza'rah the Dreamweaver"] = "Hazza'rah",
		["Renataki"] = "Renataki",
		["Wushoolay"] = "Wushoolay",
		["Booty Bay Elite"] = "Elite de Baie-du-Butin",
		["Zulian Prowler"] = "Rôdeur zulien",
		["Stormpike Emissary"] = "Emissaire foudrepique",
		["Frostwolf Emissary"] = "Emissaire loup-de-givre",
		["Gurubashi Prisoner"] = "Prisonnier gurubashi",
		["Mad Servant"] = "Serviteur fou",
		["Brain Wash Totem"] = "Totem de lavage de cerveau",
		["Honored Hero"] = "Héros honoré",
		["Gahz'ranka"] = "Gahz'ranka",
		["Honored Ancestor"] = "Ancêtre honoré",
		["Grinkle"] = "Grinkle",
		["Chained Spirit"] = "Esprit enchaîné",
		["Barrus"] = "Barrus",
		["Rutherford Twing"] = "Rutherford Twing",
		["Samuel Hawke"] = "Samuel Hawke",
		["Defiler Elite"] = "Profanateur d'élite",
		["Hammerfall Elite"] = "Elite de Trépas-d'Orgrim",
		["Menethil Elite"] = "Elite de Menethil",
		["Silverpine Elite"] = "Elite des Pins argentés",
		["Mad Voidwalker"] = "Marcheur du Vide fou",
		["Scarlet Inquisitor"] = "Inquisiteur écarlate",
		["Nightmare Illusion"] = "Illusion de cauchemar",
		["Haughty Modiste"] = "Hautaine Lamodiste",
		["Vile Scarab"] = "Scarabée vil",
		["Ralo'shan the Eternal Watcher"] = "Ralo'shan, la Gardienne éternelle",
		["Rutgar Glyphshaper"] = "Rutgar Formeglyphe",
		["Frankal Stonebridge"] = "Frankal Pont-de-pierre",
		["Glibb"] = "Glibb",
		["Calandrath"] = "Calandrath",
		["Khur Hornstriker"] = "Khur Sonneur-de-corne",
		["Vargus"] = "Vargus",
		["Cloud Skydancer"] = "Nébul Danseciel",
		["Runk Windtamer"] = "Runk Dompte-le-vent",
		["Mishta"] = "Mishta",
		["Baristolth of the Shifting Sands"] = "Baristolth des Sables changeants",
		["Commander Mar'alith"] = "Commandant Mar'alith",
		["Vish Kozus"] = "Vish Kozus",
		["Geologist Larksbane"] = "Géologue Fablemort",
		["Cenarion Hold Infantry"] = "Soldat d'infanterie du Fort cénarie",
		["Brood of Nozdormu"] = "Progéniture de Nozdormu",
		["Cenarion Emissary Jademoon"] = "Emissaire cénarienne Jadelune",
		["Cenarion Emissary Blackhoof"] = "Emissaire cénarien Sabot-noir",
		["Beetix Ficklespragg"] = "Beetix Kalefragile",
		["Noggle Ficklespragg"] = "Noggle Kalefragile",
		["Windcaller Proudhorn"] = "Imploratrice céleste Corne-fière",
		["Anachronos"] = "Anachronos",
		["Hermit Ortell"] = "Ortell l'ermite",
		["Wickerman Guardian"] = "Gardien de l'homme d'osier",
		["Deathclasp"] = "Poigne-de-mort",
		["Darkcaller Yanka"] = "Sombre imploratrice Yanka",
		["Sergeant Hartman"] = "Sergent Hartman",
		["Twilight Keeper Mayna"] = "Gardienne du crépuscule Mayna",
		["Twilight Flamereaver"] = "Flamme ravageuse du crépuscule",
		["Vyral the Vile"] = "Vyral le Vil",
		["Prince Skaldrenox"] = "Prince Skaldrenox",
		["High Marshal Whirlaxis"] = "Haut maréchal Trombe",
		["Baron Kazum"] = "Baron Kazum",
		["The Duke of Cynders"] = "Le duc des Cendres",
		["The Duke of Fathoms"] = "Le duc des Profondeurs",
		["The Duke of Shards"] = "Le duc des Eclats",
		["Crimson Templar"] = "Templier cramoisi",
		["Azure Templar"] = "Templier d'azur",
		["Hoary Templar"] = "Templier chenu",
		["Twilight Overlord"] = "Suzerain du crépuscule",
		["Mistress Natalia Mar'alith"] = "Dame Natalia Mar'alith",
		["The Duke of Zephyrs"] = "Le duc des Zéphyrs",
		["Vekniss Soldier"] = "Soldat vekniss",
		["Vekniss Warrior"] = "Patrouilleur vekniss",
		["Vekniss Guardian"] = "Gardien vekniss",
		["Vekniss Stinger"] = "Flagellant vekniss",
		["Vekniss Wasp"] = "Guêpe vekniss",
		["Vekniss Hive Crawler"] = "Rampant de la ruche vekniss",
		["Qiraji Mindslayer"] = "Tueur d'esprit qiraji",
		["Qiraji Brainwasher"] = "Lave-cerveaux qiraji",
		["Qiraji Lasher"] = "Aiguillonneur courroucé qiraji",
		["Qiraji Slayer"] = "Massacreur qiraji",
		["Qiraji Champion"] = "Champion qiraji",
		["Demented Druid Spirit"] = "Esprit de druide dément",
		["Spirit Shade"] = "Ombre spirituelle",
		["Obsidian Eradicator"] = "Eradicateur d'obsidienne",
		["The Prophet Skeram"] = "Le Prophète Skeram",
		["Anubisath Sentinel"] = "Sentinelle Anubisath",
		["Huum Wildmane"] = "Huum Crin-sauvage",
		["Emperor Vek'nilash"] = "Empereur Vek'nilash",
		["Emperor Vek'lor"] = "Empereur Vek'lor",
		["Anubisath Defender"] = "Défenseur Anubisath",
		["Aurel Goldleaf"] = "Aurel Feuilledor",
		["Xil'xix"] = "Xil'xix",
		["Aluntir"] = "Aluntir",
		["Arakis"] = "Arakis",
		["Aendel Windspear"] = "Aendel Lancevent",
		["Viscidus"] = "Viscidus",
		["Vekniss Drone"] = "Bourdon vekniss",
		["Shade of Taerar"] = "Ombre de Taerar",
		["Lord Skwol"] = "Seigneur Skwol",
		["Bor Wildmane"] = "Bor Crin-sauvage",
		["Earthen Templar"] = "Templier terrestre",
		["Twilight Prophet"] = "Prophète du crépuscule",
		["Anubisath Warder"] = "Garde Anubisath",
		["Obsidian Nullifier"] = "Annulateur d'obsidienne",
		["Mylini Frostmoon"] = "Mylini Givrelune",
		["Qiraji Scarab"] = "Scarabée qiraji",
		["Qiraji Scorpion"] = "Scorpion qiraji",
		["Hive'Zara Drone"] = "Bourdon de la Ruche'Zara",
		["Hive'Zara Collector"] = "Collecteur de la Ruche'Zara",
		["Hive'Zara Soldier"] = "Soldat de la Ruche'Zara",
		["Hive'Zara Sandstalker"] = "Traqueur des sables de la Ruche'Zar",
		["Qiraji Gladiator"] = "Gladiateur qiraji",
		["Hive'Zara Wasp"] = "Guêpe de la Ruche'Zara",
		["Hive'Zara Stinger"] = "Aiguillonneur de la Ruche'Zara",
		["Silicate Feeder"] = "Nourrisseur silicieux",
		["Giant Eye Tentacle"] = "Tentacule oculaire géant",
		["Flesh Hunter"] = "Chasse-chair",
		["Hive'Zara Tail Lasher"] = "Fouette-queue de la Ruche'Zara",
		["Obsidian Destroyer"] = "Destructeur d'obsidienne",
		["Ossirian the Unscarred"] = "Ossirian l'Intouché",
		["Moam"] = "Moam",
		["General Rajaxx"] = "Général Rajaxx",
		["Qiraji Swarmguard"] = "Garde-essaim qiraji",
		["Swarmguard Needler"] = "Piqueur garde-essaim",
		["Kurinnaxx"] = "Kurinnaxx",
		["Horde Warbringer"] = "Porteguerre de la Horde",
		["Alliance Brigadier General"] = "Général de brigade de l'Alliance",
		["Anubisath Guardian"] = "Gardien Anubisath",
		["Tonk Mine"] = "Télé-mine",
		["Ayamiss the Hunter"] = "Ayamiss le Chasseur",
		["Buru the Gorger"] = "Buru Grandgosier",
		["Merithra of the Dream"] = "Merithra du Rêve",
		["Caelestrasz"] = "Caelestrasz",
		["Arygos"] = "Arygos",
		["Anachronos the Ancient"] = "Anachronos l'Ancien",
		["Fandral Staghelm"] = "Fandral Forteramure",
		["Sergeant Stonebrow"] = "Sergent Front-de-pierre",
		["OLDWorld Trigger"] = "Déclencheur universel",
		["Colonel Zerran"] = "Colonel Zerran",
		["Major Yeggeth"] = "Major Yeggeth",
		["Qiraji Warrior"] = "Guerrier qiraji",
		["Major Pakkon"] = "Major Parron",
		["Captain Drenn"] = "Capitaine Drenn",
		["Captain Xurrem"] = "Capitaine Xurrem",
		["Captain Qeez"] = "Capitaine Qeez",
		["Captain Tuubid"] = "Capitaine Tuubid",
		["Nafien"] = "Nafien",
		["Qiraji Wasp"] = "Guêpe qiraji",
		["Kania"] = "Kania",
		["Qiraji Drone"] = "Bourdon qiraji",
		["Qiraji Tank"] = "Blindé qiraji",
		["Kaldorei Infantry"] = "Soldat d'infanterie kaldorei",
		["Anubisath Conqueror"] = "Conquérant Anubisath",
		["Ahn'Qiraj Trigger"] = "Déclencheur Ahn'Qiraj",
		["Sand Vortex"] = "Vortex de sable",
		["Corporal Carnes"] = "Caporal Carnes",
		["Dame Twinbraid"] = "Dame Doublenattes",
		["Private Draxlegauge"] = "Soldat Draunessieu",
		["Master Nightsong"] = "Maître Chantenuit",
		["Captain Blackanvil"] = "Capitaine Noirenclume",
		["Ironforge Brigade Rifleman"] = "Fusilier de la brigade d'Ironforge",
		["Ironforge Brigade Footman"] = "Fantassin de la brigade d'Ironforge",
		["Janela Stouthammer"] = "Janela Rudemartel",
		["Arcanist Nozzlespring"] = "Arcaniste Tuyèraressort",
		["Sergeant Major Germaine"] = "Sergent-major Germaine",
		["Bonnie Stoneflayer"] = "Bonnie Ecorchepierre",
		["Private Porter"] = "Soldat Porter",
		["Hive'Zora Abomination"] = "Abomination de la Ruche'Zora",
		["Marta Finespindle"] = "Marta Finfuseau",
		["Sentinel Silversky"] = "Sentinelle Argeciel",
		["Nurse Stonefield"] = "Infirmière Champierreux",
		["Keeper Moonshade"] = "Gardien Lunombrage",
		["Slicky Gastronome"] = "Khirry le Gastrognome",
		["Sarah Sadwhistle"] = "Sarah Tristesifflet",
		["Huntress Swiftriver"] = "Chasseresse Ondevive",
		["Commander Stronghammer"] = "Commandant Fortmartel",
		["Miner Cromwell"] = "Mineur Cromwell",
		["Grunt Maug"] = "Grunt Maug",
		["Shrieker Scarab"] = "Scarabée hurleur",
		["Spitting Scarab"] = "Scarabée cracheur",
		["Grace of Air Totem III"] = "Totem de Grâce aérienne III",
		["Strength of Earth Totem V"] = "Totem de force de la Terre V",
		["Minion of Omen"] = "Serviteur de Présage",
		["Omen"] = "Présage",
		["Senior Sergeant T'kelah"] = "Sergent-chef T'kelah",
		["Lieutenant General Andorov"] = "Général de division Andorov",
		["Kaldorei Elite"] = "Elite kaldorei",
		["Herbalist Proudfeather"] = "Herboriste Plume-fière",
		["Spirit of Azuregos"] = "Esprit d'Azuregos",
		["Eranikus, Tyrant of the Dream"] = "Eranikus, le tyran du Rêve",
		["Nighthaven Defender"] = "Défenseur de Havrenuit",
		["Windcaller Yessendra"] = "Imploratrice céleste Yessendra",
		["Warden Haro"] = "Gardien Haro",
		["Keyl Swiftclaw"] = "Keyl Vivegriffe",
		["Andorgos"] = "Andorgos",
		["Kandrostrasz"] = "Kandrostrasz",
		["Vethsera"] = "Vethsera",
		["Canal Frenzy"] = "Furie des canaux",
		["Batrider Pele'keiki"] = "Chevaucheur de chauve-souris Pele'k",
		["Princess Huhuran"] = "Princesse Huhuran",
		["Fankriss the Unyielding"] = "Fankriss l'Inflexible",
		["Lord Kri"] = "Seigneur Kri",
		["Buru Egg"] = "?uf de Buru",
		["Skinner Jamani"] = "Dépeceuse Jamani",
		["Battleguard Sartura"] = "Garde de guerre Sartura",
		["Ouro"] = "Ouro",
		["Hive'Zara Hatchling"] = "Jeune de la Ruche'Zara",
		["Meridith the Mermaiden"] = "Meridith la Vierge de mer",
		["Mana Fiend"] = "Démon de mana",
		["Healer Longrunner"] = "Guérisseur Longue-course",
		["Bloodguard Rawtar"] = "Garde de sang Brutepoix",
		["Fisherman Lin'do"] = "Pêcheur Lin'do",
		["Chief Sharpclaw"] = "Chef Griffe-tranchante",
		["Anubisath Warrior"] = "Guerrier Anubisath",
		["Anubisath Swarmguard"] = "Garde-essaim Anubisath",
		["General Zog"] = "Général Zog",
		["Windcaller Kaldon"] = "Implorateur céleste Kaldon",
		["Twilight Marauder Morna"] = "Maraudeuse du crépuscule Morna",
		["Twilight Marauder"] = "Maraudeur du crépuscule",
		["Princess Yauj"] = "Princesse Yauj",
		["Vem"] = "Vem",
		["Cenarion Outrider"] = "Voltigeur cénarien",
		["Hive'Zara Swarmer"] = "Grouillant de la Ruche'Zara",
		["Elder Morndeep"] = "Ancien Gouffre-du-matin",
		["Doctor Weavil"] = "Docteur Dwenfer",
		["Doctor Weavil's Flying Machine"] = "Machine volante du docteur Dwenfer",
		["Number Two"] = "Numéro Deux",
		["Hive'Zara Larva"] = "Larve de la Ruche'Zara",
		["Elder Splitrock"] = "Ancien Pierre-fendue",
		["Elder Rumblerock"] = "Ancien Pierre-grondante",
		["Elder Silvervein"] = "Ancien Veine-argent",
		["Elder Highpeak"] = "Ancien Haut-pic",
		["Elder Stonefort"] = "Ancien Fort-de-pierre",
		["Elder Obsidian"] = "Ancien Obsidien",
		["Elder Hammershout"] = "Ancien Cri-du-marteau",
		["Elder Bellowrage"] = "Ancien Soufflet-de-rage",
		["Elder Darkcore"] = "Ancien Sombre-noyau",
		["Elder Stormbrow"] = "Ancien Front-de-tempête",
		["Elder Snowcrown"] = "Ancien Couronne-de-neige",
		["Elder Ironband"] = "Ancien Baguefer",
		["Elder Graveborn"] = "Ancien Né-au-tombeau",
		["Elder Goldwell"] = "Ancien Puits-doré",
		["Elder Primestone"] = "Ancien Prime-pierre",
		["Maws"] = "Crocs-de-la-mer",
		["Elder Runetotem"] = "Ancienne Totem-runique",
		["Elder Ragetotem"] = "Ancienne Totem-de-rage",
		["Elder Stonespire"] = "Ancien Cime-de-pierre",
		["Elder Bloodhoof"] = "Ancien Sabot-de-sang",
		["Elder Winterhoof"] = "Ancien Sabot-d'hiver",
		["Elder Skychaser"] = "Ancienne Chasse-le-ciel",
		["Elder Wildmane"] = "Ancienne Crin-sauvage",
		["Elder Darkhorn"] = "Ancienne Sombre-corne",
		["Elder Proudhorn"] = "Ancien Ezra Sabot-de-blé",
		["Elder Grimtotem"] = "Ancien Totem-sinistre",
		["Elder Windtotem"] = "Ancienne Totem-de-vent",
		["Elder Thunderhorn"] = "Ancien Corne-tonnerre",
		["Elder Skyseer"] = "Ancien Oeil-des-cieux",
		["Elder Dawnstrider"] = "Ancienne Aube-glorieuse",
		["Elder Dreamseer"] = "Ancien Oeil-des-rêves",
		["Elder Mistwalker"] = "Ancienne Marche-brume",
		["Elder High Mountain"] = "Ancien Haute-montagne",
		["Eye of C'Thun"] = "?il de C'Thun",
		["Ossirian Crystal Trigger"] = "Déclencheur du cristal d'Ossiran",
		["Minion of Weavil"] = "Séide de Dwenfer",
		["Elder Windrun"] = "Ancien Tracevent",
		["Elder Starsong"] = "Ancienne Chantétoile",
		["Elder Moonstrike"] = "Ancienne Heurtelune",
		["Elder Moonstrike"] = "Ancien Feuillelame",
		["Elder Starglade"] = "Ancien Reflet-d'étoile",
		["Elder Moonwarden"] = "Ancien Gardelune",
		["Elder Bladeswift"] = "Ancien Lamevive",
		["Elder Bladesing"] = "Ancienne Lamechant",
		["Elder Skygleam"] = "Ancien Luireciel",
		["Elder Starweave"] = "Ancienne Tissétoile",
		["Elder Meadowrun"] = "Ancienne Courresteppe",
		["Elder Nightwind"] = "Ancien Ventenuit",
		["Elder Morningdew"] = "Ancienne Rosée-du-matin",
		["Elder Riversong"] = "Ancienne Chantelonde",
		["Elder Brightspear"] = "Ancienne Brillelance",
		["Elder Farwhisper"] = "Ancien Murmeloin",
		["Cenarion Scout Landion"] = "Eclaireur cénarien Landion",
		["Cenarion Scout Azenel"] = "Eclaireur cénarien Azenel",
		["Cenarion Scout Jalia"] = "Eclaireur cénarien Jalia",
		["Krug Skullsplit"] = "Krug Caskrâne",
		["Merok Longstride"] = "Merok Long-pas",
		["J.D. Shadesong"] = "J.D. Chantombrage",
		["Shadow Priestess Shai"] = "Prêtresse des ombres Shai",
		["Orgrimmar Legion Grunt"] = "Grunt de la Légion d'Orgrimmar",
		["Orgrimmar Legion Axe Thrower"] = "Lanceur de haches de la Légion d'Or",
		["Hive'Regal Hunter-Killer"] = "Chasseur-tueur de Ruche'Regal",
		["Yauj Brood"] = "Progéniture de Yauj",
		["Vekniss Borer"] = "Fouisseuse vekniss",
		["Xandivious"] = "Xandivious",
		["Twilight Corrupter"] = "Corrupteur du crépuscule",
		["Eranikus the Redeemed"] = "Eranikus le Racheté",
		["Nightmare Phantasm"] = "Illusion cauchemardesque",
		["Spawn of Fankriss"] = "Rejeton de Fankriss",
		["Tyrande"] = "Tyrande",
		["Priestess of the Moon"] = "Prêtresse de la lune",
		["Auctioneer Jaxon"] = "Commissaire-priseur Jaxon",
		["Glob of Viscidus"] = "Globule de Viscidus",
		["Auctioneer Stockton"] = "Commissaire-priseur Stockton",
		["Auctioneer Yarly"] = "Commissaire-priseur Yarly",
		["Auctioneer Graves"] = "Commissaire-priseur Graves",
		["Auctioneer Silva'las"] = "Commissaire-priseur Silva'las",
		["Auctioneer Cazarez"] = "Commissaire-priseur Cazarez",
		["Auctioneer O'reely"] = "Commissaire-priseur O'reely",
		["Auctioneer Cain"] = "Commissaire-priseur Cain",
		["Auctioneer Naxxremis"] = "Commissaire-priseur Naxxremis",
		["Auctioneer Tricket"] = "Commissaire-priseur Tricket",
		["Auctioneer Rhyker"] = "Commissaire-priseur Rhyker",
		["Jonathan the Revelator"] = "Jonathan le Révélateur",
		["Vek Twins Trigger"] = "Déclencheur des jumeaux vek",
		["Warlord Gorchuk"] = "Seigneur de guerre Gorchuk",
		["Field Marshal Snowfall"] = "Grand maréchal Tombeneige",
		["Senior Sergeant Taiga"] = "Sergent-chef Taiga",
		["Senior Sergeant Grimsford"] = "Sergent-chef Grimsford",
		["Senior Sergeant Kai'jin"] = "Sergent-chef Kai'jin",
		["Master Sergeant Fizzlebolt"] = "Sergent-chef Perrunboulon",
		["Master Sergeant Maclure"] = "Sergent-chef Maclure",
		["Master Sergeant Moonshadow"] = "Sergent-chef Lunombre",
		["Ouro Scarab"] = "Scarabée d'Ouro",
		["Timbermaw Ancestor"] = "Ancêtre grumegueule",
		["Squire Leoren Mal'derath"] = "Ecuyer Leoren Mal'derath",
		["Drunken Bruiser"] = "Cogneur ivre",
		["Claw Tentacle"] = "Tentacule griffu",
		["Eye Tentacle"] = "Tentacule oculaire",
		["C'Thun"] = "C'Thun",
		["Giant Claw Tentacle"] = "Tentacule griffu géant",
		["Darnassus Commendation Officer"] = "Officier du mérite de Darnassus",
		["Gnomeregan Commendation Officer"] = "Officier du mérite de Gnomeregan",
		["Ironforge Commendation Officer"] = "Officier du mérite d'Ironforge",
		["Stormwind Commendation Officer"] = "Officier du mérite de Stormwind",
		["Orgrimmar Commendation Officer"] = "Officier du mérite d'Orgrimmar",
		["Darkspear Commendation Officer"] = "Officier du mérite des Sombrelances",
		["Undercity Commendation Officer"] = "Officier du mérite d'Undercity",
		["Thunder Bluff Commendation Officer"] = "Officier du mérite de Thunder Bluff",
		["Colossus of Zora"] = "Colosse de Zora",
		["Colossus of Regal"] = "Colosse de Regal",
		["Colossus of Ashi"] = "Colosse d'Ashi",
		["Colossal Anubisath Warbringer"] = "Porteguerre anubisath colossal",
		["Imperial Qiraji Destroyer"] = "Destructeur impérial qiraji",
		["Qiraji Brigadier General"] = "Général de brigade qiraji",
		["Greater Anubisath Warbringer"] = "Porteguerre anubisath supérieur",
		["Greater Silithid Flayer"] = "Ecorcheuse silithide supérieure",
		["Qiraji Lieutenant General"] = "Général de division qiraji",
		["Supreme Anubisath Warbringer"] = "Porteguerre anubisath suprême",
		["Supreme Silithid Flayer"] = "Ecorcheur silithide suprême",
		["Officer Vu'Shalay"] = "Officier Vu'Shalay",
		["Officer Lunalight"] = "Officier Lunelume",
		["Officer Porterhouse"] = "Officier Tournedos",
		["Officer Ironbeard"] = "Officier Barbe-de-fer",
		["Officer Redblade"] = "Officier Rougelame",
		["Officer Maloof"] = "Officier Maloof",
		["Officer Thunderstrider"] = "Officier Marche-sous-le-tonnerre",
		["Officer Gothena"] = "Officier Gothena",
		["Christmas Emperor Dagran Thaurissan"] = "Empereur Dagran Thaurissan de Noël",
		["Colossus Researcher Sophia"] = "Chercheuse du colosse Sophia",
		["Colossus Researcher Nestor"] = "Chercheur du colosse Nestor",
		["Colossus Researcher Eazel"] = "Chercheur du colosse Eazel",
		["Flesh Tentacle"] = "Tentacule de chair",
		["Tranquil Air Totem"] = "Totem de Tranquillité de l'air",
		["Qiraji Major He'al-ie"] = "Major qiraji He'al-ie",
		["Qiraji Brigadier General Pax-lish"] = "Général de brigade qiraji Pax-lish",
		["Lieutenant General Nokhor"] = "Général de division qiraji Nokhor",
		["Orgrimmar Elite Shieldguard"] = "Garde-bouclier d'élite d'Orgri",
		["Orgrimmar Elite Infantryman"] = "Infanterie d'élite d'Orgrimmar",
		["Orgrimmar Elite Cavalryman"] = "Cavalier d'élite d'Orgrimmar",
		["Tauren Rifleman"] = "Fusillier tauren",
		["Tauren Primalist"] = "Primaliste tauren",
		["Stormwind Cavalryman"] = "Cavalier de Stormwind",
		["Stormwind Infantryman"] = "Fantassin de Stormwind",
		["Stormwind Archmage"] = "Archimage de Stormwind",
		["Kaldorei Marksman"] = "Tireur d'élite kaldorei",
		["Ironforge Infantryman"] = "Fantassin d'Ironforge",
		["Ironforge Cavalryman"] = "Cavalier d'Ironforge",
		["Darkspear Shaman"] = "Chaman sombrelance",
		["Valadar Starsong"] = "Valadar Chantétoile",
		["High Commander Lynore Windstryke"] = "Haut commandant Lynore Windstryke",
		["Highlord Leoric Von Zeldig"] = "Généralissime Leoric von Zeldig",
		["Malagav the Tactician"] = "Malagav le Tacticien",
		["Duke August Foehammer"] = "Duc August Foehammer",
		["Elder Bronzebeard"] = "Ancien Barbe-de-bronze",
		["Pat's Firework Guy - RED"] = "Type des feux d'artifice de Pat - R",
		["Pat's Firework Guy - YELLOW"] = "Type des feux d'artifice de Pat - J",
		["Lunar Festival Herald"] = "Héraut de la fête lunaire",
		["Lunar Festival Emissary"] = "Emissaire de la fête lunaire",
		["Lunar Firework Credit Marker"] = "Marqueur de feux d'artifice lunaire",
		["Lunar Cluster Credit Marker"] = "Marqueur de chapelet lunaire",
		["Lunar Festival Harbinger"] = "Messagère de la fête lunaire",
		["Lunar Festival Vendor"] = "Marchand de la fête lunaire",
		["Giant Spotlight"] = "Projecteur géant",
		["Sergeant Carnes"] = "Sergent Carnes",
		["Fariel Starsong"] = "Fariel Chantétoile",
		["Pat's Firework Cluster Guy (ELUNE)"] = "Type des chapelets de feux d'artifi",
		["Thaddius"] = "Thaddius",
		["Stalagg"] = "Stalagg",
		["Feugen"] = "Feugen",
		["Grobbulus"] = "Grobbulus",
		["Gluth"] = "Gluth",
		["Poison Cloud"] = "Nuage empoisonné",
		["Hive'Zara Hornet"] = "Frelon de la Ruche'Zara",
		["Heigan the Unclean"] = "Heigan l'Impur",
		["Maexxna"] = "Maexxna",
		["Grand Widow Faerlina"] = "Grande veuve Faerlina",
		["Noth the Plaguebringer"] = "Noth le Porte-peste",
		["Anub'Rekhan"] = "Anub'Rekhan",
		["Ouro Spawner"] = "Générateur d'Ouro",
		["Lunar Festival Sentinel"] = "Sentinelle de la fête lunaire",
		["Vekniss Hatchling"] = "Jeune vekniss",
		["The Master's Eye"] = "L'?il du Maître",
		["Buru Egg Trigger"] = "Déclencheur de l'?uf de Buru",
		["Dread Creeper"] = "Rampant de l'effroi",
		["Carrion Spinner"] = "Tisse-charogne",
		["Venom Stalker"] = "Traque-venin",
		["Poisonous Skitterer"] = "Glisseuse venimeuse",
		["Crypt Reaver"] = "Saccageur des cryptes",
		["Tomb Horror"] = "Horreur des tombes",
		["Naxxramas Cultist"] = "Sectateur de Naxxramas",
		["Naxxramas Acolyte"] = "Acolyte de Naxxramas",
		["Sartura's Royal Guard"] = "Garde royal de Sartura",
		["Sapphiron"] = "Saphiron",
		["Kel'Thuzad"] = "Kel'Thuzad",
		["InCombat Trigger"] = "Déclencheur enCombat",
		["Loatheb"] = "Horreb",
		["Mokvar"] = "Mokvar",
		["Deliana"] = "Deliana",
		["Mux Manascrambler"] = "Mux Brouillemana",
		["Vi'el"] = "Vi'el",
		["Anthion Harmon"] = "Anthion Harmon",
		["Patchwork Golem"] = "Golem recousu",
		["Bile Retcher"] = "Vomisseur de bile",
		["Mad Scientist"] = "Savant fou",
		["Living Monstrosity"] = "Monstruosité vivante",
		["Surgical Assistant"] = "Assistant en chirurgie",
		["Embalming Slime"] = "Gelée d'embaumement",
		["Stitched Spewer"] = "Géant raccommodé",
		["Living Poison"] = "Poison vivant",
		["Patchwerk"] = "Le Recousu",
		["Sludge Belcher"] = "Crache-vase",
		["Ysida Harmon"] = "Ysida Harmon",
		["Falrin Treeshaper"] = "Falrin Sculpteflore",
		["Bodley"] = "Bodley",
		["Plague Beast"] = "Bête de peste",
		["Frenzied Bat"] = "Chauve-souris frénétique",
		["Plagued Bat"] = "Chauve-souris pesteuse",
		["Lord Valthalak"] = "Seigneur Valthalak",
		["Magma Lord Bokk"] = "Seigneur-magma Bokk",
		["Lefty"] = "Le Gaucher",
		["Rotfang"] = "Crocs-pourris",
		["Snokh Blackspine"] = "Snokh Noire-échine",
		["Malgen Longspear"] = "Malgen Long-épieu",
		["Korv"] = "Korv",
		["Rezznik"] = "Rezznik",
		["Va'jashni"] = "Va'jashni",
		["Diseased Maggot"] = "Asticot malade",
		["Rotting Maggot"] = "Asticot pourrissant",
		["Volida"] = "Volida",
		["Theldren"] = "Theldren",
		["Gothik the Harvester"] = "Gothik le Moissonneur",
		["Instructor Razuvious"] = "Instructeur Razuvious",
		["Highlord Mograine"] = "Généralissime Mograine",
		["Sir Zeliek"] = "Sire Zeliek",
		["Thane Korth'azz"] = "Thane Korth'azz",
		["Lady Blaumeux"] = "Dame Blaumeux",
		["Spectral Assassin"] = "Assassin spectral",
		["Deathcharger Steed"] = "Palefroi de la mort",
		["Tidelord Rrurgaz"] = "Seigneur-marées Rrurgaz",
		["Spirit of Lord Valthalak"] = "Esprit du seigneur Valthalak",
		["Mor Grayhoof"] = "Mor Sabot-gris",
		["Naxxramas Trigger"] = "Déclencheur de Naxxramas",
		["Rousch"] = "Rousch",
		["Dirk Thunderwood"] = "Dirk Tonnebois",
		["Spectral Stalker"] = "Traqueur spectral",
		["Gnashjaw"] = "Grince-mâchoires",
		["Steamwheedle Bruiser"] = "Cogneur de Gentepression",
		["Isalien"] = "Isalien",
		["Empyrean"] = "Empyréen",
		["Jarien"] = "Jarien",
		["Sothos"] = "Sothos",
		["Spirit of Jarien"] = "Esprit de Jarien",
		["Spirit of Sothos"] = "Esprit de Sothos",
		["Korfax, Champion of the Light"] = "Korfax, Champion de la Lumière",
		["Father Inigo Montoy"] = "Père Inigo Montoy",
		["Scarlet Commander Marjhan"] = "Commandant écarlate Marjhan",
		["Commander Eligor Dawnbringer"] = "Commandant Eligor Portelaube",
		["Archmage Angela Dosantos"] = "Archimage Angela Dosantos",
		["Plagued Swine"] = "Pourceau pestiféré",
		["Kormok"] = "Kormok",
		["Bone Minion"] = "Séide des os",
		["Bone Mage"] = "Mage des os",
		["Unrelenting Trainee"] = "Jeune recrue tenace",
		["Unrelenting Deathknight"] = "Chevalier de la mort tenace",
		["Unrelenting Rider"] = "Cavalier tenace",
		["Spectral Trainee"] = "Jeune recrue spectrale",
		["Shadow Fissure"] = "Fissure d'ombre",
		["Rohan the Assassin"] = "Rohan l'Assassin",
		["Huntsman Leopold"] = "Veneur Leopold",
		["Mataus the Wrathcaster"] = "Mataus la Voix du courroux",
		["Rimblat Earthshatter"] = "Rimblat Brise-terre",
		["Rayne"] = "Rayne",
		["Naxxramas Military Sub-Boss Trigger"] = "Déclencheur du sous-boss militaire de Na",
		["Cenarion Hold Reservist"] = "Réserviste du Fort cénarien",
		["Ghoul Berserker"] = "Goule berserker",
		["Bile Sludge"] = "Vase bilieuse",
		["Deathknight Captain"] = "Capitaine des chevaliers de la mort",
		["Deathknight"] = "Chevalier de la mort",
		["Spectral Deathknight"] = "Chevalier de la mort spectral",
		["Spectral Horse"] = "Cheval spectral",
		["Spectral Rider"] = "Cavalier spectral",
		["Risen Deathknight"] = "Chevalier de la mort ressuscité",
		["Dark Touched Warrior"] = "Guerrier touché par l'obscurité",
		["Doom Touched Warrior"] = "Guerrier touché par le destin",
		["Death Touched Warrior"] = "Guerrier touché par la mort",
		["Deathknight Cavalier"] = "Cavalier de la mort",
		["Shade of Naxxramas"] = "Ombre de Naxxramas",
		["Necro Knight"] = "Nécrochevalier",
		["Bony Construct"] = "Assemblage osseux",
		["Stoneskin Gargoyle"] = "Gargouille peau de pierre",
		["Nerubian Overseer"] = "Surveillant nérubien",
		["Skeletal Smith"] = "Forgeron squelettique",
		["Unholy Axe"] = "Hache impie",
		["Dispatch Commander Metz"] = "Commandant d'ordonnance Metz",
		["Unholy Staff"] = "Bâton impie",
		["Unholy Swords"] = "Epées impies",
		["Tesla Coil"] = "Bobine de Tesla",
		["Guard Didier"] = "Garde Didier",
		["Bragok"] = "Bragok",
		["Argent Dawn Infantry"] = "Soldat d'infanterie de l'Aube ",
		["Injured Argent Dawn Infantry"] = "Soldat d'infanterie de l'Aube ",
		["Caravan Mule"] = "Mule de la caravane",
		["Eye Stalk"] = "Oeil pédonculé",
		["Plague Slime"] = "Gelée de la peste",
		["Infectious Ghoul"] = "Goule infectieuse",
		["Field Marshal Chambers"] = "Grand maréchal Chambers",
		["Jessica Chambers"] = "Jessica Chambers",
		["Packmaster Stonebruiser"] = "Maître-fourrier Navrepierre",
		["Argent Medic"] = "Infirmier de l'Aube d'argent",
		["Spore"] = "Spore",
		["Fallout Slime"] = "Gelée polluée",
		["Mutated Grub"] = "Larve mutante",
		["Skeletal Shocktrooper"] = "Troupier squelettique de choc",
		["Argent Messenger"] = "Messager de l'Aube d'argent",
		["Zombie Chow"] = "Croq'zombie",
		["Commander Thomas Helleran"] = "Commandant Thomas Helleran",
		["Master Craftsman Omarion"] = "Maître-artisan Omarion",
		["Necropolis Acolyte"] = "Acolyte de la nécropole",
		["Sewage Slime"] = "Gelée d'égout",
		["Craftsman Wilhelm"] = "Artisan Wilhelm",
		["Argent Sentry"] = "Factionnaire de l'Aube d'argen",
		["Spirit of the Damned"] = "Esprit de damné",
		["Bone Witch"] = "Sorcière des ossements",
		["Archmage Tarsis Kir-Moldir"] = "Archimage Tarsis Kir-Moldir",
		["Flameshocker"] = "Bouteflammes",
		["Lightning Totem"] = "Totem de foudre",
		["Atiesh"] = "Atiesh",
		["Deathchill Servant"] = "Serviteur de Froidemort",
		["Captain Armando Ossex"] = "Capitaine Armando Ossex",
		["Toxic Tunnel"] = "Tunnel toxique",
		["Necropolis"] = "Nécropole",
		["Bronn Fitzwrench"] = "Bronn Fisseclef",
		["Ghost of Naxxramas"] = "Fantôme de Naxxramas",
		["Soldier of the Frozen Wastes"] = "Soldat des terres gelées",
		["Unstoppable Abomination"] = "Abomination irrésistible",
		["Soul Weaver"] = "Tisseur d'âme",
		["Cracked Necrotic Crystal"] = "Cristal nécrotique lézardé",
		["Guardian of Icecrown"] = "Gardien de la Couronne de glace",
		["Plagued Gargoyle"] = "Gargouille pestiférée",
		["Plagued Ghoul"] = "Goule pestiférée",
		["Plagued Deathhound"] = "Doguemort pestiféré",
		["Spirit of Naxxramas"] = "Esprit de Naxxramas",
		["Deathknight Vindicator"] = "Chevalier de la mort justicier",
		["Necro Knight Guardian"] = "Nécro-chevalier garde",
		["Necro Stalker"] = "Nécro-traqueuse",
		["Web Wrap"] = "Entoilage",
		["Naxxramas Follower"] = "Suivant de Naxxramas",
		["Naxxramas Worshipper"] = "Adorateur de Naxxramas",
		["Argent Warhorse"] = "Cheval de guerre de l'Aube d'a",
		["Faint Necrotic Crystal"] = "Cristal nécrotique voilé",
		["Garon Hutchins"] = "Garon Hutchins",
		["Crypt Guard"] = "Gardien des cryptes",
		["Corpse Scarab"] = "Scarabée mange-cadavres",
		["Spirit of Mograine"] = "Esprit de Mograine",
		["Spirit of Blaumeux"] = "Esprit de Blaumeux",
		["Spirit of Zeliek"] = "Esprit de Zeliek",
		["Spirit of Korth'azz"] = "Esprit de Korth'azz",
		["Festival Flamekeeper"] = "Gardien des flammes de la fête",
		["Deathknight Understudy"] = "Doublure de chevalier de la mort",
		["Festival Loremaster"] = "Maître des traditions de la fête",
		["Festival Talespinner"] = "Conteur de la fête",
		["Death Lord"] = "Seigneur de la mort",
		["The Lich King"] = "Le roi-liche",
		["Plagued Guardian"] = "Gardien pestiféré",
		["Plagued Construct"] = "Assemblage pestiféré",
		["Plagued Champion"] = "Champion pestiféré",
		["Plagued Warrior"] = "Guerrier pestiféré",
		["Festival Flamekeeper Costume: Human"] = "Costume de gardien des flammes : humain",
		["Orgrimmar Fireeater"] = "Avaleur de feu d'Orgrimmar",
		["Maexxna Spiderling"] = "Jeune araignée de Maexxna",
		["Chief Expeditionary Requisitioner Enkles"] = "Responsable-chef des réquisitions Enkles",
		["Emissary Whitebeard"] = "Emissaire Blanchebarbe",
		["Apothecary Quinard"] = "Apothicaire Quinard",
		["Emissary Gormok"] = "Emissaire Gormok",
		["General Kirika"] = "Général Kirika",
		["Marshal Bluewall"] = "Maréchal Murbleu",
		["Scout Bloodfist"] = "Eclaireur Poing-de-sang",
		["Rifleman Torrig"] = "Fusilier Torrig",
		["William Kielar"] = "William Kielar",
		["Landro Longshot"] = "Landro Tirauloin",
		["Felguard"] = "Gangregarde",
		["Renn'az"] = "Renn'az",
		["Lordaeron Commander"] = "Commandant de Lordaeron",
		["Lordaeron Soldier"] = "Soldat de Lordaeron",
		["Alliance Silithyst Sentinel"] = "Sentinelle de silithyste, Alliance",
		["Horde Silithyst Sentinel"] = "Sentinelle de silithyste, Horde",
		["Squire Rowe"] = "Ecuyer Rowe",
		["Lordaeron Veteran"] = "Vétéran de Lordaeron",
		["Lordaeron Fighter"] = "Guerrier de Lordaeron",
		["The Bug Family"] = "The Bug Family",
		["The Twin Emperors"] = "The Twin Emperors",
		["The Four Horsemen"] = "The Four Horsemen",
		["Environment"] = "Environment",
		["Unknown"] = "Unknown",
	}
end)

NPCDB:Debug()
NPCDB:SetStrictness(true)

function NPCDB:Contains(name)
	if npcs[name] or npcs[self:HasReverseTranslation(name) and self:GetReverseTranslation(name) or false] then
		return true
	end
	return false
end

AceLibrary:Register(NPCDB, MAJOR_VERSION, MINOR_VERSION)
NPCDB = nil